C**********************************************************************
C***
C***  THIS ARE THE FUNCTIONS FOR THE SQUARK-ANTISQUARK PROCESS
C***
C**********************************************************************

      REAL*8 FUNCTION DSBGG1(ALPHAS,S,T1,MS,MG,SCA)
C***  GIVES THE SCALE DEPENDENCE OF VIRTUAL
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,U1,MS,MG,SCA,PI,BETAL,PGG,DSBGGB,NS
      PI = 4.D0*ATAN(1.D0)
      NS = 6.D0
      U1 = -S -T1
      BETAL = -11.D0 + 2.D0/3.D0*(NS -1.D0)
      PGG = 3.D0*(-2.D0*LOG(-T1/MS**2) -2*LOG(-U1/MS**2) +11.D0/3.D0)
     +     -2.D0/3.D0*(NS -1.D0)
      DSBGG1 = DSBGGB(ALPHAS,S,T1,MS,MG)*
     +     ALPHAS/2.D0/PI*LOG(1.D0/SCA)*( BETAL +PGG)
      RETURN
      END


      REAL*8 FUNCTION DSBGG2(ALPHAS,S,T1,S4,MS,MG,DEL,S4MAX,SCA)
C***  GIVES THE SCALE DEPENDENCE OF LOG(DEL)
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,S4,S4MAX,MS,MG,SCA,PI,DSBGGB,DLDEL1,DEL
      PI = 4.D0*ATAN(1.D0)
      DLDEL1 = LOG(S4MAX/MS**2) -(S4MAX -DEL)/S4
      DSBGG2 = 3.D0*4.D0*DLDEL1*DSBGGB(ALPHAS,S,T1,MS,MG)
     +     * ALPHAS/2.D0/PI*LOG(1.D0/SCA)
      RETURN
      END


      REAL*8 FUNCTION DSBGG3(ALPHAS,S,T1,S4,MS,MG,SCA)
C***  GIVES THE SCALE DEPENDENCE OF HARD
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,U1,S4,MS,MG,PI
      REAL*8 DSBGGB,PGG1,PGG2,SCA,X1,X2
      PI = 4.D0*ATAN(1.D0)
      U1 = S4 -S -T1
      X1 = -U1/(S+T1)
      X2 = -T1/(S+U1)
      PGG1 = 3.D0*( 2.D0/(1.D0 -X1) +2.D0/X1 -4.D0 +2.D0*X1 -2.D0*X1**2)
      PGG2 = 3.D0*( 2.D0/(1.D0 -X2) +2.D0/X2 -4.D0 +2.D0*X2 -2.D0*X2**2)
      DSBGG3 = ALPHAS/2.D0/PI*LOG(1.D0/SCA) *
     +     (-1/U1*DSBGGB(ALPHAS,X1*S,X1*T1,MS,MG)*PGG1*X1**2
     +      -1/T1*DSBGGB(ALPHAS,X2*S,T1,MS,MG)*PGG2*X2**2 )
      RETURN
      END

      REAL*8 FUNCTION DSBGGB(ALPHAS,S,T1,MS,MG)
C***  BORN CROSS SECTIONS FOR G + G -> SQ + SQB
C***  SUMMED OVER LL +RR 
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,U1,MS,MG,MS2
      REAL*8 NS,CONV,PI,N,CO,CK,AVG
      NS = 6.D0
      CONV = 389379660.D0
      PI = 4.D0*ATAN(1.D0)
      N = 3.D0
      CO = N*(N**2 -1.D0)
      CK = (N**2 -1.D0)/N
      AVG = (1.D0/2.D0)**2 * (1.D0/(N**2 -1.D0))**2
      U1 = -S -T1
      MS2 = MS**2
      DSBGGB = ALPHAS**2 * 2.D0*(NS -1.D0) * CONV *AVG *PI/S**2 *
     +     ( CO*(1.D0 -2.D0*T1*U1/S**2) -CK ) * 
     +     (2.D0 - 4.D0*S*MS2/T1/U1*(1.D0 -S*MS2/T1/U1) )
      RETURN
      END

      REAL*8 FUNCTION DSBGGB_NG(ALPHAS,S,T1,MS,MG)
C***  BORN CROSS SECTIONS FOR G + G -> SQ + SQB
C***  SUMMED OVER LL +RR 
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,U1,MS,MG,MS2
      REAL*8 NS,CONV,PI,N,CO,CK,AVG
      NS = 6.D0
      CONV = 389379660.D0
      PI = 4.D0*ATAN(1.D0)
      N = 3.D0
      CO = N*(N**2 -1.D0)
      CK = (N**2 -1.D0)/N
      AVG = (1.D0/2.D0)**2 * (1.D0/(N**2 -1.D0))**2
      U1 = -S -T1
      MS2 = MS**2
      DSBGGB_NG = ALPHAS**2 * CONV *AVG *PI/S**2 *
     +     ( CO*(1.D0 -2.D0*T1*U1/S**2) -CK ) * 
     +     (2.D0 - 4.D0*S*MS2/T1/U1*(1.D0 -S*MS2/T1/U1) )
      RETURN
      END


      REAL*8 FUNCTION DSBGGV(ALPHAS,S,T1,MS,MG,MT)
C***  CROSS SECTIONS FOR G + G -> SQ + SQB
C***  SUMMED OVER LL +RR 
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T,T1,U,U1,SB,S1,MS,MG,MT,MS2,MG2,MT2,M2
      REAL*8 NS,CONV,PI,ZETA2,N,CO,CK,CQED,AVG,BETA,XS,SPENCE,THREE
      REAL*8 SK1B0A(1:3), SK1B0B(1:4), SK1B0C(1:2), SK1B0D(1:3,1:2)
      REAL*8 SK1BP(1:3), SK1C0A(1:6), SK1C0B(1:4)
      REAL*8 SK1C0C(1:6,1:2), SK1D0(1:9,1:2)
      REAL*8 SL1B0A, SL1B0B, SL1B0C, SL1B0D
      REAL*8 SL1BP, SL1C0A, SL1C0B
      REAL*8 SL1C0C, SL1D0
      REAL*8 SOF1(1:8)
      REAL*8 M2GGSV,DSBGGB
      COMPLEX*16 KBETAG,KXG,KBETAT,KXT            
      PI = 4.D0*ATAN(1.D0)
      ZETA2 = PI**2/6.D0
      CONV = 389379660.D0

      NS = 6
      N = 3.D0
      CO = N*(N**2 -1.D0)
      CK = (N**2-1.D0)/N
      CQED = (N**4 -1.D0)/N**2
      AVG = (1.D0/2.D0)**2 * (1.D0/(N**2 -1.D0))**2
      THREE = 3.D0

      MS2 = MS**2
      MG2 = MG**2
      MT2 = MT**2
      M2 = MG2 - MS2
      S1 = 4.D0*MS2 - S
      U1 = -S -T1
      T = T1 +MS2
      U = U1 +MS2

      BETA = SQRT(1.D0 - 4.D0*MS2/S)
      XS = (1.D0 -BETA)*(1.D0+BETA)**(-1)
      SB = S*BETA

      KBETAG = SQRT( DCMPLX(1.D0 - 4.D0*MG2/S)) 
      KXG = (DCMPLX(1.D0) -KBETAG)*(DCMPLX(1.D0)+KBETAG)**(-1)
      KBETAT = SQRT( DCMPLX(1.D0 - 4.D0*MT2/S)) 
      KXT = (DCMPLX(1.D0) -KBETAT)*(DCMPLX(1.D0)+KBETAT)**(-1)
      

      
      SK1B0A(1) = SL1B0A(MS2,MG2,MT2,1)
      SK1B0A(2) = SL1B0A(MS2,MG2,MT2,2)
C$$$      SK1B0A(3) = SL1B0A(MS2,MG2,MT2,3)

      SK1B0B(1) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,1)
      SK1B0B(2) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,2)
      SK1B0B(3) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,3)
      SK1B0B(4) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,4)


      SK1B0C(1) = SL1B0C(MS2,MG2,MT2,1)
      SK1B0C(2) = SL1B0C(MS2,MG2,MT2,2)

      SK1B0D(1,1) = SL1B0D(MS2,MG2,MT2,T,1)
      SK1B0D(2,1) = SL1B0D(MS2,MG2,MT2,T,2)
C$$$      SK1B0D(3,1) = SL1B0D(MS2,MG2,MT2,T,3)
      SK1B0D(1,2) = SL1B0D(MS2,MG2,MT2,U,1)
      SK1B0D(2,2) = SL1B0D(MS2,MG2,MT2,U,2)
C$$$      SK1B0D(3,2) = SL1B0D(MS2,MG2,MT2,U,3)

C$$$      SK1B0E(1) = SL1B0E(MS2,MG2,MT2,1)
C$$$      SK1B0E(2) = SL1B0E(MS2,MG2,MT2,2)
C$$$      SK1B0E(3) = SL1B0E(MS2,MG2,MT2,3)

      SK1BP(1) = SL1BP(MS2,MG2,MT2,1)
      SK1BP(2) = SL1BP(MS2,MG2,MT2,2)
C$$$      SK1BP(3) = SL1BP(MS2,MG2,MT2,3)


      SK1C0A(1) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,1)
      SK1C0A(2) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,2)
      SK1C0A(3) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,3)
      SK1C0A(4) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,4)
C$$$      SK1C0A(5) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,5)
C$$$      SK1C0A(6) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,6)

      SK1C0B(1) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,1)
      SK1C0B(2) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,2)
      SK1C0B(3) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,3)
      SK1C0B(4) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,4)

      SK1C0C(1,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,1)
      SK1C0C(2,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,2)
      SK1C0C(3,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,3)
      SK1C0C(4,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,4)
C$$$      SK1C0C(5,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,5)
C$$$      SK1C0C(6,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,6)

      SK1C0C(1,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,1)
      SK1C0C(2,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,2)
      SK1C0C(3,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,3)
      SK1C0C(4,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,4)
C$$$      SK1C0C(5,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,5)
C$$$      SK1C0C(6,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,6)


      SK1D0(1,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,1)
      SK1D0(2,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,2)
      SK1D0(3,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,3)
      SK1D0(4,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,4)
      SK1D0(5,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,5)
      SK1D0(6,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,6)
C$$$      SK1D0(7,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,7)
C$$$      SK1D0(8,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,8)
C$$$      SK1D0(9,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,9)

      SK1D0(1,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,1)
      SK1D0(2,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,2)
C$$$      SK1D0(3,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,3)
      SK1D0(4,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,4)
      SK1D0(5,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,5)
C$$$      SK1D0(6,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,6)
C$$$      SK1D0(7,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,7)
C$$$      SK1D0(8,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,8)
C$$$      SK1D0(9,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,9)

      SOF1(1) = 0.125D0/SB * ( +4*SPENCE(XS) +4*SPENCE(-XS) 
     +     +4*LOG(XS)*LOG(1-XS**2)-2*LOG(XS)**2 -2*ZETA2 )
      SOF1(2) = 0.125D0/MS2* (+2*(S -2*MS2)/SB*LOG(XS))
      SOF1(3) = 0.125D0/MS2*(-2)
      SOF1(4) = 0.125D0/U1*(-1.5D0*ZETA2)
      SOF1(5) = 0.125D0/T1*(-1.5D0*ZETA2)
      SOF1(6) = 0.125D0/T1*(-2*LOG(XS)**2 +LOG(XS*U1/T1)**2 
     +     +2*SPENCE(1 -T1/XS/U1) -2*SPENCE(1 -U1/XS/T1) -1.5D0*ZETA2 )
      SOF1(7) = 0.125D0/U1*(-2*LOG(XS)**2 +LOG(XS*T1/U1)**2 
     +     +2*SPENCE(1 -U1/XS/T1) -2*SPENCE(1 -T1/XS/U1) -1.5D0*ZETA2)
      SOF1(8) = 0.125D0/S*(LOG(T1*U1/MS2/S)**2
     +     +2*SPENCE(1 -MS2*S/T1/U1) -3*ZETA2 )

      
      IF (MS.EQ.MG) THEN
      M2GGSV = 0.D0
      M2GGSV = M2GGSV + SK1B0A(2)*CO*THREE**(-1) * (  - 8 + 16*S**(-2)*
     +    T1*U1 - 96*S**(-2)*MS2*MT2 - 32*S**(-1)*MS2 + 16*S*T1**(-1)*
     +    U1**(-1)*MS2 - 16*S**2*T1**(-2)*U1**(-2)*MS2**2 + 24*T1**(-1)
     +    *U1**(-1)*MS2*MT2 + 32*T1**(-1)*U1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0A(2)*CK*THREE**(-1) * ( 8 - 16*S*T1**(-1)*
     +    U1**(-1)*MS2 + 16*S**2*T1**(-2)*U1**(-2)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0B(1)*N*CO * (  - 4 + 8*S**(-2)*T1*U1 + 8*
     +    S**(-1)*T1**2*S1**(-1) - 8*S**(-1)*MS2 + 4*S*T1**(-1)*
     +    U1**(-1)*MS2 - 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 + 8*
     +    T1**(-1)*U1**(-1)*MS2**2 + 8*T1*S1**(-1) + 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(1)*N*CK * (  - 2 + 8*S**(-1)*T1**2*
     +    S1**(-1) + 2*S*T1**(-1)*U1**(-1)*MS2 - 32*T1**(-1)*U1**(-1)*
     +    S1**(-1)*MS2**3 + 8*T1**(-1)*U1**(-1)*MS2**2 + 8*T1*S1**(-1)
     +     + 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(2)*N*CO * ( 2 + 32*S**(-2)*MS2**2 - 8*
     +    S**(-1)*T1**2*S1**(-1) - 2*S*T1**(-1)*U1**(-1)*MS2 + 32*
     +    T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 - 16*T1**(-1)*U1**(-1)*
     +    MS2**2 - 8*T1*S1**(-1) - 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(2)*N*CK * ( 4 - 8*S**(-2)*T1*U1 - 8*
     +    S**(-1)*T1**2*S1**(-1) + 8*S**(-1)*MS2 - 4*S*T1**(-1)*
     +    U1**(-1)*MS2 + 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 - 8*
     +    T1**(-1)*U1**(-1)*MS2**2 - 8*T1*S1**(-1) - 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(2)*CO * (  - 32*S**(-2)*NS*MS2**2 + 8*
     +    T1**(-1)*U1**(-1)*NS*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0B(4)*CO * ( 32*S**(-2)*MS2*MT2 - 8*
     +    T1**(-1)*U1**(-1)*MS2*MT2 )
     +
      M2GGSV = M2GGSV + SK1B0C(1)*N*CO * ( 4 - 8*S**(-2)*T1*U1 + 24*
     +    S**(-1)*MS2 - 12*S*T1**(-1)*U1**(-1)*MS2 + 24*S**2*T1**(-2)*
     +    U1**(-2)*MS2**2 + 16*T1**(-3)*MS2**3 - 56*T1**(-1)*U1**(-1)*
     +    MS2**2 + 16*U1**(-3)*MS2**3 )
     +
      M2GGSV = M2GGSV + SK1B0C(1)*N*CK * (  - 12 + 8*S**(-2)*T1*U1 - 24
     +    *S**(-1)*MS2 - 64*S*T1**(-2)*U1**(-2)*MS2**3 + 36*S*T1**(-1)*
     +    U1**(-1)*MS2 - 72*S**2*T1**(-2)*U1**(-2)*MS2**2 + 32*S**3*
     +    T1**(-3)*U1**(-3)*MS2**3 - 16*T1**(-3)*MS2**3 + 88*T1**(-1)*
     +    U1**(-1)*MS2**2 - 16*U1**(-3)*MS2**3 )
     +
      M2GGSV = M2GGSV + SK1B0C(1)*CQED * ( 4 + 32*S*T1**(-2)*U1**(-2)*
     +    MS2**3 - 12*S*T1**(-1)*U1**(-1)*MS2 + 24*S**2*T1**(-2)*
     +    U1**(-2)*MS2**2 - 16*S**3*T1**(-3)*U1**(-3)*MS2**3 - 16*
     +    T1**(-1)*U1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,1)*N*CO * (  - 8*S**(-1)*T1**(-1)*
     +    MS2**2 + 2*S**(-1)*U1 - 8*S**(-1)*MS2 + 24*S*T1**(-2)*
     +    U1**(-1)*MS2**2 - 16*T1**(-3)*MS2**3 + 24*T1**(-1)*U1**(-1)*
     +    MS2**2 - 10*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,1)*N*CK * ( 4 + 8*S**(-1)*T1**(-1)*
     +    MS2**2 - 2*S**(-1)*U1 + 8*S**(-1)*MS2 + 32*S*T1**(-2)*
     +    U1**(-2)*MS2**3 - 72*S*T1**(-2)*U1**(-1)*MS2**2 - 4*S*
     +    T1**(-1)*U1**(-1)*MS2 + 32*S**2*T1**(-3)*U1**(-2)*MS2**3 + 16
     +    *T1**(-3)*MS2**3 - 40*T1**(-1)*U1**(-1)*MS2**2 + 26*T1**(-1)*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,1)*CQED * (  - 2 - 16*S*T1**(-2)*
     +    U1**(-2)*MS2**3 + 24*S*T1**(-2)*U1**(-1)*MS2**2 + 2*S*
     +    T1**(-1)*U1**(-1)*MS2 - 16*S**2*T1**(-3)*U1**(-2)*MS2**3 + 8*
     +    T1**(-1)*U1**(-1)*MS2**2 - 8*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,2)*N*CO * (  - 2 + 8*S**(-1)*T1**(-1)*
     +    MS2**2 - 2*S**(-1)*U1 - 8*S**(-1)*MS2 - 24*S*T1**(-2)*
     +    U1**(-1)*MS2**2 + 10*S*T1**(-1)*U1**(-1)*MS2 - 24*S**2*
     +    T1**(-2)*U1**(-2)*MS2**2 + 32*T1**(-1)*U1**(-1)*MS2**2 + 10*
     +    T1**(-1)*MS2 - 16*U1**(-3)*MS2**3 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,2)*N*CK * ( 6 - 8*S**(-1)*T1**(-1)*
     +    MS2**2 + 2*S**(-1)*U1 + 8*S**(-1)*MS2 + 32*S*T1**(-2)*
     +    U1**(-2)*MS2**3 + 72*S*T1**(-2)*U1**(-1)*MS2**2 - 30*S*
     +    T1**(-1)*U1**(-1)*MS2 - 32*S**2*T1**(-3)*U1**(-2)*MS2**3 + 72
     +    *S**2*T1**(-2)*U1**(-2)*MS2**2 - 32*S**3*T1**(-3)*U1**(-3)*
     +    MS2**3 - 48*T1**(-1)*U1**(-1)*MS2**2 - 26*T1**(-1)*MS2 + 16*
     +    U1**(-3)*MS2**3 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,2)*CQED * (  - 2 - 16*S*T1**(-2)*
     +    U1**(-2)*MS2**3 - 24*S*T1**(-2)*U1**(-1)*MS2**2 + 10*S*
     +    T1**(-1)*U1**(-1)*MS2 + 16*S**2*T1**(-3)*U1**(-2)*MS2**3 - 24
     +    *S**2*T1**(-2)*U1**(-2)*MS2**2 + 16*S**3*T1**(-3)*U1**(-3)*
     +    MS2**3 + 8*T1**(-1)*U1**(-1)*MS2**2 + 8*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(1)*N*CO * (  - 16*S**(-2)*T1*U1*MS2 + 32*
     +    S**(-1)*MS2**2 - 16*S*T1**(-1)*U1**(-1)*MS2**2 + 16*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 - 32*T1**(-1)*U1**(-1)*MS2**3 + 8*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(1)*N*CK * ( 16*S**(-2)*T1*U1*MS2 - 32*
     +    S**(-1)*MS2**2 + 48*S*T1**(-1)*U1**(-1)*MS2**2 - 48*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 + 32*T1**(-1)*U1**(-1)*MS2**3 - 24*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(1)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 + 16*S**2*T1**(-2)*U1**(-2)*MS2**3 + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(1)*N*CO * ( 4*S**2*T1**(-1)*U1**(-1)*MS2
     +     - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(1)*N*CK * ( S + S**2*T1**(-1)*U1**(-1)*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(1)*CQED * ( 2*S - 2*S**2*T1**(-1)*
     +    U1**(-1)*MS2 - S**3*T1**(-1)*U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*N*CO * ( 16*S**(-1)*MS2**2 - 8*S*
     +    T1**(-1)*U1**(-1)*MS2**2 - S + S**2*T1**(-1)*U1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*N*CK * (  - 12*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 4*S + 4*S**2*T1**(-1)*U1**(-1)*MS2 + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*CO * (  - 16*S**(-1)*NS*MS2**2 + 4*S*
     +    T1**(-1)*U1**(-1)*NS*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*CQED * ( 24*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 12*S**2*T1**(-1)*U1**(-1)*MS2 + 2*S**3*T1**(-1)*
     +    U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1C0A(4)*CO * ( 16*S**(-1)*MS2*MT2 - 4*S*
     +    T1**(-1)*U1**(-1)*MS2*MT2 )
     +
      M2GGSV = M2GGSV + SK1C0B(1)*N*CO * (  - S - S**2*T1**(-1)*
     +    U1**(-1)*MS2 + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(1)*N*CK * ( 16*S**(-2)*T1*U1*MS2 - 8*
     +    S**(-1)*T1*U1 - 48*S**(-1)*MS2**2 + 8*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 4*S**2*T1**(-1)*U1**(-1)*MS2 + 24*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(1)*CQED * ( 8*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 4*S - 8*S**2*T1**(-1)*U1**(-1)*MS2 + 2*S**3*T1**(-1)*
     +    U1**(-1) + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(3)*N*CO * (  - 12*S**(-1)*T1*U1 + 8*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + 4*S - 4*S**2*T1**(-1)*U1**(-1)*MS2
     +     - 64*T1**(-1)*U1**(-1)*S1**(-1)*MS2**4 + 16*T1**(-1)*
     +    U1**(-1)*MS2**3 + 16*T1*S1**(-1)*MS2 - 4*T1 + 4*T1**2*
     +    S1**(-1) + 32*S1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1C0B(3)*N*CK * (  - 4*S**(-1)*T1*U1 + 4*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + S - S**2*T1**(-1)*U1**(-1)*MS2 - 
     +    64*T1**(-1)*U1**(-1)*S1**(-1)*MS2**4 + 16*T1**(-1)*U1**(-1)*
     +    MS2**3 + 16*T1*S1**(-1)*MS2 - 4*T1 + 4*T1**2*S1**(-1) + 32*
     +    S1**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(3)*CQED * ( 2*S + 2*S**2*T1**(-1)*
     +    U1**(-1)*MS2 - S**3*T1**(-1)*U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1C0C(1,1)*N*CO * (  - 2*S**(-1)*T1*U1 + 2*S
     +     + 2*U1 - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,1)*N*CK * (  - 8*S**(-1)*U1*MS2 + 16*
     +    S**(-1)*MS2**2 - 6*S*T1**(-1)*MS2 + 8*S - 6*S**2*T1**(-1)*
     +    U1**(-1)*MS2 + 8*U1 - 16*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,1)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 + 16*S*T1**(-1)*MS2 + 4*S + 16*S**2*T1**(-1)*U1**(-1)*
     +    MS2 - 4*S**2*T1**(-1) - 4*S**3*T1**(-1)*U1**(-1) - 16*
     +    T1**(-1)*MS2**2 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,2)*N*CO * (  - 2*S**(-1)*T1*U1 - 2*U1
     +     - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,2)*N*CK * ( 8*S**(-1)*U1*MS2 + 16*
     +    S**(-1)*MS2**2 + 6*S*T1**(-1)*MS2 - 8*U1 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,2)*CQED * (  - 16*S*T1**(-1)*MS2 + 4*S
     +     + 4*S**2*T1**(-1) + 16*T1**(-1)*MS2**2 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,1)*N*CO * ( 8*S**(-1)*T1*U1 + 8*
     +    S**(-1)*U1*MS2 - 4*S - 4*U1 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,1)*N*CK * ( 2*S**(-1)*T1*U1 - 16*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + 6*S*T1**(-1)*MS2 - 6*S + 6*S**2*
     +    T1**(-1)*U1**(-1)*MS2 - 16*T1**(-1)*MS2**2 - 6*U1 - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,1)*CQED * ( 2*S*T1**(-1)*MS2 - 4*S + 2
     +    *S**2*T1**(-1)*U1**(-1)*MS2 + 2*S**2*T1**(-1) + 2*S**3*
     +    T1**(-1)*U1**(-1) - 2*U1 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,2)*N*CO * ( 8*S**(-1)*T1*U1 - 8*
     +    S**(-1)*U1*MS2 + 4*U1 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,2)*N*CK * ( 2*S**(-1)*T1*U1 - 6*S*
     +    T1**(-1)*MS2 + 16*T1**(-1)*MS2**2 + 6*U1 - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,2)*CQED * (  - 2*S*T1**(-1)*MS2 - 2*S
     +     - 2*S**2*T1**(-1) + 2*U1 )
     +
      M2GGSV = M2GGSV + SK1D0(1,1)*N*CO * (  - 4*S**(-1)*T1*U1**2 - 8*S
     +    *T1**(-1)*MS2**2 + 2*S*U1 + 2*S**2 - 4*T1*U1 + 4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(1,1)*N*CK * ( S*U1 + S*MS2 + S**2 - T1*U1
     +     )
     +
      M2GGSV = M2GGSV + SK1D0(1,1)*CQED * ( S*U1 - S**2*T1**(-1)*MS2 + 
     +    2*S**2 - S**3*T1**(-1)*U1**(-1)*MS2 - S**3*T1**(-1) - S**4*
     +    T1**(-1)*U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1D0(1,2)*N*CO * ( 4*S**(-1)*T1*U1**2 + 8*S*
     +    T1**(-1)*MS2**2 - 2*S*U1 - 4*S*MS2 + 8*S**2*T1**(-1)*U1**(-1)
     +    *MS2**2 - 4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(1,2)*N*CK * (  - S*U1 + S*MS2 - T1*U1 )
     +
      M2GGSV = M2GGSV + SK1D0(1,2)*CQED * (  - S*U1 + S**2*T1**(-1)*MS2
     +     + S**2 + S**3*T1**(-1) )
     +
      M2GGSV = M2GGSV + SK1D0(2,1)*N*CO * (  - S*U1 + 3*S*MS2 - S**2 + 
     +    T1*U1 + 4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,1)*N*CK * (  - 8*S**(-1)*U1*MS2**2 + 8*
     +    S*T1**(-1)*MS2**2 - 2*S*U1 + 12*S*MS2 - 2*S**2 - 16*T1**(-1)*
     +    MS2**3 + 8*U1*MS2 - 16*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,1)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**3 + 16*S*T1**(-1)*MS2**2 + 8*S*MS2 + 24*S**2*T1**(-1)*
     +    U1**(-1)*MS2**2 - 12*S**2*T1**(-1)*MS2 - 2*S**2 - 12*S**3*
     +    T1**(-1)*U1**(-1)*MS2 + 2*S**3*T1**(-1) + 2*S**4*T1**(-1)*
     +    U1**(-1) - 8*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,2)*N*CO * ( S*U1 - S*MS2 + T1*U1 - 4*U1
     +    *MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,2)*N*CK * ( 8*S**(-1)*U1*MS2**2 + 16*S*
     +    T1**(-1)*U1**(-1)*MS2**3 - 8*S*T1**(-1)*MS2**2 + 2*S*U1 + 4*S
     +    *MS2 - 8*S**2*T1**(-1)*U1**(-1)*MS2**2 + 16*T1**(-1)*MS2**3
     +     - 8*U1*MS2 - 8*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,2)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**3 - 16*S*T1**(-1)*MS2**2 + 8*S*MS2 + 8*S**2*T1**(-1)*
     +    U1**(-1)*MS2**2 + 12*S**2*T1**(-1)*MS2 - 2*S**2 - 2*S**3*
     +    T1**(-1) - 8*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(3,1)*N*CK * (  - 10*S*MS2 + 16*S**2*
     +    T1**(-1)*U1**(-1)*MS2**2 + 4*T1*U1 - 16*MS2**2 )
     +
      M2GGSV = M2GGSV + SOF1(1)*N*CK * (  - 64*S**(-2)*T1*U1*MS2 + 32*
     +    S**(-1)*T1*U1 + 128*S**(-1)*MS2**2 - 16*S + 64*S**2*T1**(-2)*
     +    U1**(-2)*MS2**3 + 32*S**2*T1**(-1)*U1**(-1)*MS2 - 32*S**3*
     +    T1**(-2)*U1**(-2)*MS2**2 - 128*T1**(-1)*U1**(-1)*MS2**3 - 32*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(1)*CQED * ( 64*S*T1**(-1)*U1**(-1)*MS2**2
     +     + 16*S - 64*S**2*T1**(-2)*U1**(-2)*MS2**3 - 32*S**2*T1**(-1)
     +    *U1**(-1)*MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 - 32*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(2)*N*CO * ( 32*S**(-2)*T1*U1*MS2 - 64*
     +    S**(-1)*MS2**2 + 32*S*T1**(-1)*U1**(-1)*MS2**2 - 32*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 + 64*T1**(-1)*U1**(-1)*MS2**3 - 16*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(2)*N*CK * (  - 32*S**(-2)*T1*U1*MS2 + 64*
     +    S**(-1)*MS2**2 - 96*S*T1**(-1)*U1**(-1)*MS2**2 + 96*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 - 64*T1**(-1)*U1**(-1)*MS2**3 + 48*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(2)*CQED * ( 32*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 32*S**2*T1**(-2)*U1**(-2)*MS2**3 - 16*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(3)*N*CO * ( 32*S**(-2)*T1*U1*MS2 - 64*
     +    S**(-1)*MS2**2 + 32*S*T1**(-1)*U1**(-1)*MS2**2 - 32*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 + 64*T1**(-1)*U1**(-1)*MS2**3 - 16*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(3)*N*CK * (  - 32*S**(-2)*T1*U1*MS2 + 64*
     +    S**(-1)*MS2**2 - 96*S*T1**(-1)*U1**(-1)*MS2**2 + 96*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 - 64*T1**(-1)*U1**(-1)*MS2**3 + 48*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(3)*CQED * ( 32*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 32*S**2*T1**(-2)*U1**(-2)*MS2**3 - 16*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(4)*N*CO * (  - 16*S**(-2)*T1*U1**2 - 16*
     +    S**(-1)*T1*U1 + 32*S**(-1)*U1*MS2 - 32*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 32*T1**(-1)*MS2**2 + 32*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(4)*N*CK * ( 32*S*T1**(-1)*MS2 - 32*S**2*
     +    T1**(-2)*U1**(-1)*MS2**2 - 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(5)*N*CO * ( 16*S**(-2)*T1*U1**2 - 32*
     +    S**(-1)*U1*MS2 + 32*T1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SOF1(5)*N*CK * (  - 32*S*T1**(-1)*MS2 + 16*S + 
     +    32*S**2*T1**(-2)*U1**(-1)*MS2**2 - 32*S**2*T1**(-1)*U1**(-1)*
     +    MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 + 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(6)*N*CO * ( 16*S**(-2)*T1*U1**2 - 32*
     +    S**(-1)*U1*MS2 + 32*T1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SOF1(6)*N*CK * (  - 32*S*T1**(-1)*MS2 + 16*S + 
     +    32*S**2*T1**(-2)*U1**(-1)*MS2**2 - 32*S**2*T1**(-1)*U1**(-1)*
     +    MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 + 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(7)*N*CO * (  - 16*S**(-2)*T1*U1**2 - 16*
     +    S**(-1)*T1*U1 + 32*S**(-1)*U1*MS2 - 32*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 32*T1**(-1)*MS2**2 + 32*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(7)*N*CK * ( 32*S*T1**(-1)*MS2 - 32*S**2*
     +    T1**(-2)*U1**(-1)*MS2**2 - 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(8)*N*CO * (  - 32*S**(-1)*T1*U1 - 64*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + 16*S - 32*S**2*T1**(-1)*U1**(-1)*
     +    MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 + 64*MS2 )


      ELSE

      M2GGSV = 0.D0
      M2GGSV = M2GGSV + SK1B0A(1)*N*CO*THREE**(-1) * (  - 8 - 96*M2*
     +    S**(-2)*MS2 + 24*M2*T1**(-1)*U1**(-1)*MS2 + 16*S**(-2)*T1*U1
     +     - 96*S**(-2)*MS2**2 - 32*S**(-1)*MS2 + 16*S*T1**(-1)*
     +    U1**(-1)*MS2 - 16*S**2*T1**(-2)*U1**(-2)*MS2**2 + 56*T1**(-1)
     +    *U1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0A(1)*N*CK*THREE**(-1) * ( 8 - 16*S*
     +    T1**(-1)*U1**(-1)*MS2 + 16*S**2*T1**(-2)*U1**(-2)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0A(2)*CO*THREE**(-1) * (  - 8 + 16*S**(-2)*
     +    T1*U1 - 96*S**(-2)*MS2*MT2 - 32*S**(-1)*MS2 + 16*S*T1**(-1)*
     +    U1**(-1)*MS2 - 16*S**2*T1**(-2)*U1**(-2)*MS2**2 + 24*T1**(-1)
     +    *U1**(-1)*MS2*MT2 + 32*T1**(-1)*U1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0A(2)*CK*THREE**(-1) * ( 8 - 16*S*T1**(-1)*
     +    U1**(-1)*MS2 + 16*S**2*T1**(-2)*U1**(-2)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0B(1)*N*CO * (  - 4 + 8*S**(-2)*T1*U1 + 8*
     +    S**(-1)*T1**2*S1**(-1) - 8*S**(-1)*MS2 + 4*S*T1**(-1)*
     +    U1**(-1)*MS2 - 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 + 8*
     +    T1**(-1)*U1**(-1)*MS2**2 + 8*T1*S1**(-1) + 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(1)*N*CK * (  - 2 + 16*M2*S**(-2)*T1**2*
     +    S1**(-1) + 16*M2*S**(-1)*T1*S1**(-1) + 4*M2*S**(-1) - 16*M2*
     +    T1**(-1)*U1**(-1)*S1**(-1)*MS2**2 + 4*M2*T1**(-1)*U1**(-1)*
     +    MS2 + 8*M2*S1**(-1) + 8*S**(-1)*T1**2*S1**(-1) + 2*S*T1**(-1)
     +    *U1**(-1)*MS2 - 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 + 8*
     +    T1**(-1)*U1**(-1)*MS2**2 + 8*T1*S1**(-1) + 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(2)*N*CK * ( 4 - 8*S**(-2)*T1*U1 - 8*
     +    S**(-1)*T1**2*S1**(-1) + 8*S**(-1)*MS2 - 4*S*T1**(-1)*
     +    U1**(-1)*MS2 + 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 - 8*
     +    T1**(-1)*U1**(-1)*MS2**2 - 8*T1*S1**(-1) - 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(2)*CO * (  - 32*S**(-2)*NS*MS2**2 + 8*
     +    T1**(-1)*U1**(-1)*NS*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0B(3)*N*CO * ( 2 - 16*M2*S**(-2)*T1**2*
     +    S1**(-1) + 32*M2*S**(-2)*MS2 - 16*M2*S**(-1)*T1*S1**(-1) - 4*
     +    M2*S**(-1) + 16*M2*T1**(-1)*U1**(-1)*S1**(-1)*MS2**2 - 12*M2*
     +    T1**(-1)*U1**(-1)*MS2 - 8*M2*S1**(-1) + 32*S**(-2)*MS2**2 - 8
     +    *S**(-1)*T1**2*S1**(-1) - 2*S*T1**(-1)*U1**(-1)*MS2 + 32*
     +    T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 - 16*T1**(-1)*U1**(-1)*
     +    MS2**2 - 8*T1*S1**(-1) - 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0B(4)*CO * ( 32*S**(-2)*MS2*MT2 - 8*
     +    T1**(-1)*U1**(-1)*MS2*MT2 )
     +
      M2GGSV = M2GGSV + SK1B0C(1)*N*CO * ( 8 - 8*S**(-2)*T1*U1 - 8*
     +    S**(-1)*T1**2*S1**(-1) + 24*S**(-1)*MS2 - 16*S*T1**(-1)*
     +    U1**(-1)*MS2 + 24*S**2*T1**(-2)*U1**(-2)*MS2**2 + 16*T1**(-3)
     +    *MS2**3 + 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 - 64*T1**(-1)*
     +    U1**(-1)*MS2**2 - 8*T1*S1**(-1) + 16*U1**(-3)*MS2**3 - 16*
     +    S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0C(1)*N*CK * (  - 24 + 8*S**(-2)*T1*U1 + 8*
     +    S**(-1)*T1**2*S1**(-1) - 24*S**(-1)*MS2 - 64*S*T1**(-2)*
     +    U1**(-2)*MS2**3 + 48*S*T1**(-1)*U1**(-1)*MS2 - 72*S**2*
     +    T1**(-2)*U1**(-2)*MS2**2 + 32*S**3*T1**(-3)*U1**(-3)*MS2**3
     +     - 16*T1**(-3)*MS2**3 - 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3
     +     + 96*T1**(-1)*U1**(-1)*MS2**2 + 8*T1*S1**(-1) - 16*U1**(-3)*
     +    MS2**3 + 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0C(1)*CQED * ( 8 + 32*S*T1**(-2)*U1**(-2)*
     +    MS2**3 - 16*S*T1**(-1)*U1**(-1)*MS2 + 24*S**2*T1**(-2)*
     +    U1**(-2)*MS2**2 - 16*S**3*T1**(-3)*U1**(-3)*MS2**3 - 16*
     +    T1**(-1)*U1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1B0C(2)*N*CO * (  - 4 + 16*M2*S**(-2)*T1**2*
     +    S1**(-1) + 16*M2*S**(-1)*T1*S1**(-1) + 12*M2*S**(-1) - 4*M2*S
     +    *T1**(-1)*U1**(-1) + 12*M2*S**2*T1**(-2)*U1**(-2)*MS2 + 8*M2*
     +    T1**(-3)*MS2**2 - 16*M2*T1**(-1)*U1**(-1)*S1**(-1)*MS2**2 - 
     +    24*M2*T1**(-1)*U1**(-1)*MS2 + 8*M2*U1**(-3)*MS2**2 + 8*M2*
     +    S1**(-1) + 8*S**(-1)*T1**2*S1**(-1) + 4*S*T1**(-1)*U1**(-1)*
     +    MS2 - 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 + 8*T1**(-1)*
     +    U1**(-1)*MS2**2 + 8*T1*S1**(-1) + 16*S1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0C(2)*N*CK * ( 12 - 16*M2*S**(-2)*T1**2*
     +    S1**(-1) - 16*M2*S**(-1)*T1*S1**(-1) - 12*M2*S**(-1) - 32*M2*
     +    S*T1**(-2)*U1**(-2)*MS2**2 + 12*M2*S*T1**(-1)*U1**(-1) - 36*
     +    M2*S**2*T1**(-2)*U1**(-2)*MS2 + 16*M2*S**3*T1**(-3)*U1**(-3)*
     +    MS2**2 - 8*M2*T1**(-3)*MS2**2 + 16*M2*T1**(-1)*U1**(-1)*
     +    S1**(-1)*MS2**2 + 40*M2*T1**(-1)*U1**(-1)*MS2 - 8*M2*U1**(-3)
     +    *MS2**2 - 8*M2*S1**(-1) - 8*S**(-1)*T1**2*S1**(-1) - 12*S*
     +    T1**(-1)*U1**(-1)*MS2 + 32*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3
     +     - 8*T1**(-1)*U1**(-1)*MS2**2 - 8*T1*S1**(-1) - 16*S1**(-1)*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1B0C(2)*CQED * (  - 4 + 16*M2*S*T1**(-2)*
     +    U1**(-2)*MS2**2 - 4*M2*S*T1**(-1)*U1**(-1) + 12*M2*S**2*
     +    T1**(-2)*U1**(-2)*MS2 - 8*M2*S**3*T1**(-3)*U1**(-3)*MS2**2 - 
     +    8*M2*T1**(-1)*U1**(-1)*MS2 + 4*S*T1**(-1)*U1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,1)*N*CO * (  - 8*S**(-1)*T1**(-1)*
     +    MS2**2 + 4*S**(-1)*U1 - 8*S**(-1)*MS2 + 24*S*T1**(-2)*
     +    U1**(-1)*MS2**2 - 16*T1**(-3)*MS2**3 + 24*T1**(-1)*U1**(-1)*
     +    MS2**2 - 12*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,1)*N*CK * ( 8 + 8*S**(-1)*T1**(-1)*
     +    MS2**2 - 4*S**(-1)*U1 + 8*S**(-1)*MS2 + 32*S*T1**(-2)*
     +    U1**(-2)*MS2**3 - 72*S*T1**(-2)*U1**(-1)*MS2**2 - 8*S*
     +    T1**(-1)*U1**(-1)*MS2 + 32*S**2*T1**(-3)*U1**(-2)*MS2**3 + 16
     +    *T1**(-3)*MS2**3 - 40*T1**(-1)*U1**(-1)*MS2**2 + 28*T1**(-1)*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,1)*CQED * (  - 4 - 16*S*T1**(-2)*
     +    U1**(-2)*MS2**3 + 24*S*T1**(-2)*U1**(-1)*MS2**2 + 4*S*
     +    T1**(-1)*U1**(-1)*MS2 - 16*S**2*T1**(-3)*U1**(-2)*MS2**3 + 8*
     +    T1**(-1)*U1**(-1)*MS2**2 - 8*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,2)*N*CO * (  - 4 + 8*S**(-1)*T1**(-1)*
     +    MS2**2 - 4*S**(-1)*U1 - 8*S**(-1)*MS2 - 24*S*T1**(-2)*
     +    U1**(-1)*MS2**2 + 12*S*T1**(-1)*U1**(-1)*MS2 - 24*S**2*
     +    T1**(-2)*U1**(-2)*MS2**2 + 32*T1**(-1)*U1**(-1)*MS2**2 + 12*
     +    T1**(-1)*MS2 - 16*U1**(-3)*MS2**3 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,2)*N*CK * ( 12 - 8*S**(-1)*T1**(-1)*
     +    MS2**2 + 4*S**(-1)*U1 + 8*S**(-1)*MS2 + 32*S*T1**(-2)*
     +    U1**(-2)*MS2**3 + 72*S*T1**(-2)*U1**(-1)*MS2**2 - 36*S*
     +    T1**(-1)*U1**(-1)*MS2 - 32*S**2*T1**(-3)*U1**(-2)*MS2**3 + 72
     +    *S**2*T1**(-2)*U1**(-2)*MS2**2 - 32*S**3*T1**(-3)*U1**(-3)*
     +    MS2**3 - 48*T1**(-1)*U1**(-1)*MS2**2 - 28*T1**(-1)*MS2 + 16*
     +    U1**(-3)*MS2**3 )
     +
      M2GGSV = M2GGSV + SK1B0D(1,2)*CQED * (  - 4 - 16*S*T1**(-2)*
     +    U1**(-2)*MS2**3 - 24*S*T1**(-2)*U1**(-1)*MS2**2 + 12*S*
     +    T1**(-1)*U1**(-1)*MS2 + 16*S**2*T1**(-3)*U1**(-2)*MS2**3 - 24
     +    *S**2*T1**(-2)*U1**(-2)*MS2**2 + 16*S**3*T1**(-3)*U1**(-3)*
     +    MS2**3 + 8*T1**(-1)*U1**(-1)*MS2**2 + 8*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(2,1)*N*CO * (  - 4*M2*S**(-1)*T1**(-1)*
     +    MS2 - 4*M2*S**(-1) + 12*M2*S*T1**(-2)*U1**(-1)*MS2 - 8*M2*
     +    T1**(-3)*MS2**2 + 12*M2*T1**(-1)*U1**(-1)*MS2 - 4*M2*T1**(-1)
     +     - 2*S**(-1)*U1 + 2*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(2,1)*N*CK * (  - 4 + 4*M2*S**(-1)*
     +    T1**(-1)*MS2 + 4*M2*S**(-1) + 16*M2*S*T1**(-2)*U1**(-2)*
     +    MS2**2 - 36*M2*S*T1**(-2)*U1**(-1)*MS2 + 16*M2*S**2*T1**(-3)*
     +    U1**(-2)*MS2**2 + 8*M2*T1**(-3)*MS2**2 - 20*M2*T1**(-1)*
     +    U1**(-1)*MS2 + 12*M2*T1**(-1) + 2*S**(-1)*U1 + 4*S*T1**(-1)*
     +    U1**(-1)*MS2 - 2*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(2,1)*CQED * ( 2 - 8*M2*S*T1**(-2)*
     +    U1**(-2)*MS2**2 + 12*M2*S*T1**(-2)*U1**(-1)*MS2 - 8*M2*S**2*
     +    T1**(-3)*U1**(-2)*MS2**2 + 4*M2*T1**(-1)*U1**(-1)*MS2 - 4*M2*
     +    T1**(-1) - 2*S*T1**(-1)*U1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(2,2)*N*CO * ( 2 + 4*M2*S**(-1)*T1**(-1)*
     +    MS2 - 4*M2*S**(-1) - 12*M2*S*T1**(-2)*U1**(-1)*MS2 + 4*M2*S*
     +    T1**(-1)*U1**(-1) - 12*M2*S**2*T1**(-2)*U1**(-2)*MS2 + 16*M2*
     +    T1**(-1)*U1**(-1)*MS2 + 4*M2*T1**(-1) - 8*M2*U1**(-3)*MS2**2
     +     + 2*S**(-1)*U1 - 2*S*T1**(-1)*U1**(-1)*MS2 - 2*T1**(-1)*MS2
     +     )
     +
      M2GGSV = M2GGSV + SK1B0D(2,2)*N*CK * (  - 6 - 4*M2*S**(-1)*
     +    T1**(-1)*MS2 + 4*M2*S**(-1) + 16*M2*S*T1**(-2)*U1**(-2)*
     +    MS2**2 + 36*M2*S*T1**(-2)*U1**(-1)*MS2 - 12*M2*S*T1**(-1)*
     +    U1**(-1) - 16*M2*S**2*T1**(-3)*U1**(-2)*MS2**2 + 36*M2*S**2*
     +    T1**(-2)*U1**(-2)*MS2 - 16*M2*S**3*T1**(-3)*U1**(-3)*MS2**2
     +     - 24*M2*T1**(-1)*U1**(-1)*MS2 - 12*M2*T1**(-1) + 8*M2*
     +    U1**(-3)*MS2**2 - 2*S**(-1)*U1 + 6*S*T1**(-1)*U1**(-1)*MS2 + 
     +    2*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1B0D(2,2)*CQED * ( 2 - 8*M2*S*T1**(-2)*
     +    U1**(-2)*MS2**2 - 12*M2*S*T1**(-2)*U1**(-1)*MS2 + 4*M2*S*
     +    T1**(-1)*U1**(-1) + 8*M2*S**2*T1**(-3)*U1**(-2)*MS2**2 - 12*
     +    M2*S**2*T1**(-2)*U1**(-2)*MS2 + 8*M2*S**3*T1**(-3)*U1**(-3)*
     +    MS2**2 + 4*M2*T1**(-1)*U1**(-1)*MS2 + 4*M2*T1**(-1) - 2*S*
     +    T1**(-1)*U1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(1)*N*CO * (  - 16*S**(-2)*T1*U1*MS2 + 32*
     +    S**(-1)*MS2**2 - 16*S*T1**(-1)*U1**(-1)*MS2**2 + 16*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 - 32*T1**(-1)*U1**(-1)*MS2**3 + 8*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(1)*N*CK * ( 16*S**(-2)*T1*U1*MS2 - 32*
     +    S**(-1)*MS2**2 + 48*S*T1**(-1)*U1**(-1)*MS2**2 - 48*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 + 32*T1**(-1)*U1**(-1)*MS2**3 - 24*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(1)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 + 16*S**2*T1**(-2)*U1**(-2)*MS2**3 + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1BP(2)*N*CO * (  - 8*M2*S**(-2)*T1*U1 + 16*M2
     +    *S**(-1)*MS2 - 8*M2*S*T1**(-1)*U1**(-1)*MS2 + 8*M2*S**2*
     +    T1**(-2)*U1**(-2)*MS2**2 - 16*M2*T1**(-1)*U1**(-1)*MS2**2 + 4
     +    *M2 )
     +
      M2GGSV = M2GGSV + SK1BP(2)*N*CK * ( 8*M2*S**(-2)*T1*U1 - 16*M2*
     +    S**(-1)*MS2 + 24*M2*S*T1**(-1)*U1**(-1)*MS2 - 24*M2*S**2*
     +    T1**(-2)*U1**(-2)*MS2**2 + 16*M2*T1**(-1)*U1**(-1)*MS2**2 - 
     +    12*M2 )
     +
      M2GGSV = M2GGSV + SK1BP(2)*CQED * (  - 8*M2*S*T1**(-1)*U1**(-1)*
     +    MS2 + 8*M2*S**2*T1**(-2)*U1**(-2)*MS2**2 + 4*M2 )
     +
      M2GGSV = M2GGSV + SK1C0A(1)*N*CO * ( 4*S**2*T1**(-1)*U1**(-1)*MS2
     +     - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(1)*N*CK * ( M2*S**2*T1**(-1)*U1**(-1) + 
     +    4*M2 + 2*M2**2*S*T1**(-1)*U1**(-1) + S + S**2*T1**(-1)*
     +    U1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(1)*CQED * (  - 4*M2*S**2*T1**(-1)*
     +    U1**(-1) - 4*M2**2*S*T1**(-1)*U1**(-1) + 2*S - 2*S**2*
     +    T1**(-1)*U1**(-1)*MS2 - S**3*T1**(-1)*U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*N*CK * (  - 12*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 4*S + 4*S**2*T1**(-1)*U1**(-1)*MS2 + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*CO * (  - 16*S**(-1)*NS*MS2**2 + 4*S*
     +    T1**(-1)*U1**(-1)*NS*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1C0A(2)*CQED * ( 24*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 12*S**2*T1**(-1)*U1**(-1)*MS2 + 2*S**3*T1**(-1)*
     +    U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1C0A(3)*N*CO * ( 16*M2*S**(-1)*MS2 - 4*M2*S*
     +    T1**(-1)*U1**(-1)*MS2 + M2*S**2*T1**(-1)*U1**(-1) - 4*M2 + 2*
     +    M2**2*S*T1**(-1)*U1**(-1) + 16*S**(-1)*MS2**2 - 8*S*T1**(-1)*
     +    U1**(-1)*MS2**2 - S + S**2*T1**(-1)*U1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0A(4)*CO * ( 16*S**(-1)*MS2*MT2 - 4*S*
     +    T1**(-1)*U1**(-1)*MS2*MT2 )
     +
      M2GGSV = M2GGSV + SK1C0B(1)*N*CK * ( 16*S**(-2)*T1*U1*MS2 - 8*
     +    S**(-1)*T1*U1 - 48*S**(-1)*MS2**2 + 8*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 4*S**2*T1**(-1)*U1**(-1)*MS2 + 24*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(1)*CQED * ( 8*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 4*S - 8*S**2*T1**(-1)*U1**(-1)*MS2 + 2*S**3*T1**(-1)*
     +    U1**(-1) + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(2)*N*CO * (  - 8*M2*S**(-2)*T1*U1 + 8*M2
     +    *S**(-1)*T1**2*S1**(-1) + 24*M2*S**(-1)*MS2 - 2*M2*S*T1**(-1)
     +    *U1**(-1)*MS2 - M2*S**2*T1**(-1)*U1**(-1) - 32*M2*T1**(-1)*
     +    U1**(-1)*S1**(-1)*MS2**3 + 8*M2*T1**(-1)*U1**(-1)*MS2**2 + 8*
     +    M2*T1*S1**(-1) + 16*M2*S1**(-1)*MS2 + 2*M2 + 16*M2**2*S**(-2)
     +    *T1**2*S1**(-1) + 16*M2**2*S**(-1)*T1*S1**(-1) + 12*M2**2*
     +    S**(-1) - 2*M2**2*S*T1**(-1)*U1**(-1) - 16*M2**2*T1**(-1)*
     +    U1**(-1)*S1**(-1)*MS2**2 + 4*M2**2*T1**(-1)*U1**(-1)*MS2 + 8*
     +    M2**2*S1**(-1) - S - S**2*T1**(-1)*U1**(-1)*MS2 + 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(3)*N*CO * (  - 12*S**(-1)*T1*U1 + 8*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + 4*S - 4*S**2*T1**(-1)*U1**(-1)*MS2
     +     - 64*T1**(-1)*U1**(-1)*S1**(-1)*MS2**4 + 16*T1**(-1)*
     +    U1**(-1)*MS2**3 + 16*T1*S1**(-1)*MS2 - 4*T1 + 4*T1**2*
     +    S1**(-1) + 32*S1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1C0B(4)*N*CK * ( 16*M2*S**(-1)*T1**2*S1**(-1)
     +     + 2*M2*S*T1**(-1)*U1**(-1)*MS2 - M2*S**2*T1**(-1)*U1**(-1)
     +     - 64*M2*T1**(-1)*U1**(-1)*S1**(-1)*MS2**3 + 16*M2*T1**(-1)*
     +    U1**(-1)*MS2**2 + 16*M2*T1*S1**(-1) + 32*M2*S1**(-1)*MS2 + 2*
     +    M2 + 16*M2**2*S**(-2)*T1**2*S1**(-1) + 16*M2**2*S**(-1)*T1*
     +    S1**(-1) + 12*M2**2*S**(-1) - 2*M2**2*S*T1**(-1)*U1**(-1) - 
     +    16*M2**2*T1**(-1)*U1**(-1)*S1**(-1)*MS2**2 + 4*M2**2*T1**(-1)
     +    *U1**(-1)*MS2 + 8*M2**2*S1**(-1) - 4*S**(-1)*T1*U1 + 4*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + S - S**2*T1**(-1)*U1**(-1)*MS2 - 
     +    64*T1**(-1)*U1**(-1)*S1**(-1)*MS2**4 + 16*T1**(-1)*U1**(-1)*
     +    MS2**3 + 16*T1*S1**(-1)*MS2 - 4*T1 + 4*T1**2*S1**(-1) + 32*
     +    S1**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0B(4)*CQED * ( 4*M2*S*T1**(-1)*U1**(-1)*MS2
     +     - 2*M2*S**2*T1**(-1)*U1**(-1) + 4*M2 + 2*S + 2*S**2*T1**(-1)
     +    *U1**(-1)*MS2 - S**3*T1**(-1)*U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1C0C(1,1)*N*CK * (  - 8*S**(-1)*U1*MS2 + 16*
     +    S**(-1)*MS2**2 - 8*S*T1**(-1)*MS2 + 8*S - 8*S**2*T1**(-1)*
     +    U1**(-1)*MS2 + 8*U1 - 16*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,1)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 + 16*S*T1**(-1)*MS2 + 4*S + 16*S**2*T1**(-1)*U1**(-1)*
     +    MS2 - 4*S**2*T1**(-1) - 4*S**3*T1**(-1)*U1**(-1) - 16*
     +    T1**(-1)*MS2**2 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,2)*N*CK * ( 8*S**(-1)*U1*MS2 + 16*
     +    S**(-1)*MS2**2 + 8*S*T1**(-1)*MS2 - 8*U1 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(1,2)*CQED * (  - 16*S*T1**(-1)*MS2 + 4*S
     +     + 4*S**2*T1**(-1) + 16*T1**(-1)*MS2**2 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(2,1)*N*CO * ( 4*M2*S**(-1)*U1 - 8*M2*
     +    S**(-1)*MS2 + 2*M2 - 4*M2**2*S**(-1) - 2*S**(-1)*T1*U1 + 2*S
     +     + 2*U1 - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(2,1)*N*CK * ( 2*M2*S*T1**(-1) + 2*M2*
     +    S**2*T1**(-1)*U1**(-1) + 4*M2**2*S*T1**(-1)*U1**(-1) + 4*
     +    M2**2*T1**(-1) + 2*S*T1**(-1)*MS2 + 2*S**2*T1**(-1)*U1**(-1)*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(2,2)*N*CO * (  - 4*M2*S**(-1)*U1 - 8*M2*
     +    S**(-1)*MS2 - 2*M2 - 4*M2**2*S**(-1) - 2*S**(-1)*T1*U1 - 2*U1
     +     - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(2,2)*N*CK * (  - 2*M2*S*T1**(-1) - 4*
     +    M2**2*T1**(-1) - 2*S*T1**(-1)*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,1)*N*CO * ( 8*S**(-1)*T1*U1 + 8*
     +    S**(-1)*U1*MS2 - 4*S - 4*U1 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,1)*N*CK * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 + 8*S*T1**(-1)*MS2 - 4*S + 8*S**2*T1**(-1)*U1**(-1)*
     +    MS2 - 16*T1**(-1)*MS2**2 - 4*U1 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,2)*N*CO * ( 8*S**(-1)*T1*U1 - 8*
     +    S**(-1)*U1*MS2 + 4*U1 - 8*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(3,2)*N*CK * (  - 8*S*T1**(-1)*MS2 + 16*
     +    T1**(-1)*MS2**2 + 4*U1 )
     +
      M2GGSV = M2GGSV + SK1C0C(4,1)*N*CK * (  - 4*M2*S**(-1)*U1 - 8*M2*
     +    S*T1**(-1)*U1**(-1)*MS2 - 2*M2*S*T1**(-1) - 2*M2*S**2*
     +    T1**(-1)*U1**(-1) - 8*M2*T1**(-1)*MS2 - 6*M2 - 4*M2**2*
     +    S**(-1) - 4*M2**2*S*T1**(-1)*U1**(-1) - 4*M2**2*T1**(-1) + 2*
     +    S**(-1)*T1*U1 - 2*S*T1**(-1)*MS2 - 2*S - 2*S**2*T1**(-1)*
     +    U1**(-1)*MS2 - 2*U1 - 2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(4,1)*CQED * ( 6*M2*S*T1**(-1) + 6*M2*
     +    S**2*T1**(-1)*U1**(-1) - 4*M2 + 4*M2**2*S*T1**(-1)*U1**(-1)
     +     + 4*M2**2*T1**(-1) + 2*S*T1**(-1)*MS2 - 4*S + 2*S**2*
     +    T1**(-1)*U1**(-1)*MS2 + 2*S**2*T1**(-1) + 2*S**3*T1**(-1)*
     +    U1**(-1) - 2*U1 )
     +
      M2GGSV = M2GGSV + SK1C0C(4,2)*N*CK * ( 4*M2*S**(-1)*U1 + 2*M2*S*
     +    T1**(-1) + 8*M2*T1**(-1)*MS2 - 2*M2 - 4*M2**2*S**(-1) + 4*
     +    M2**2*T1**(-1) + 2*S**(-1)*T1*U1 + 2*S*T1**(-1)*MS2 + 2*U1 - 
     +    2*MS2 )
     +
      M2GGSV = M2GGSV + SK1C0C(4,2)*CQED * (  - 6*M2*S*T1**(-1) - 4*M2
     +     - 4*M2**2*T1**(-1) - 2*S*T1**(-1)*MS2 - 2*S - 2*S**2*
     +    T1**(-1) + 2*U1 )
     +
      M2GGSV = M2GGSV + SK1D0(1,1)*N*CO * (  - 4*S**(-1)*T1*U1**2 - 8*S
     +    *T1**(-1)*MS2**2 + 2*S*U1 + 2*S**2 - 4*T1*U1 + 4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(1,2)*N*CO * ( 4*S**(-1)*T1*U1**2 + 8*S*
     +    T1**(-1)*MS2**2 - 2*S*U1 - 4*S*MS2 + 8*S**2*T1**(-1)*U1**(-1)
     +    *MS2**2 - 4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,1)*N*CK * (  - 8*S**(-1)*U1*MS2**2 + 8*
     +    S*T1**(-1)*MS2**2 - 2*S*U1 + 12*S*MS2 - 2*S**2 - 16*T1**(-1)*
     +    MS2**3 + 8*U1*MS2 - 16*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,1)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**3 + 16*S*T1**(-1)*MS2**2 + 8*S*MS2 + 24*S**2*T1**(-1)*
     +    U1**(-1)*MS2**2 - 12*S**2*T1**(-1)*MS2 - 2*S**2 - 12*S**3*
     +    T1**(-1)*U1**(-1)*MS2 + 2*S**3*T1**(-1) + 2*S**4*T1**(-1)*
     +    U1**(-1) - 8*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,2)*N*CK * ( 8*S**(-1)*U1*MS2**2 + 16*S*
     +    T1**(-1)*U1**(-1)*MS2**3 - 8*S*T1**(-1)*MS2**2 + 2*S*U1 + 4*S
     +    *MS2 - 8*S**2*T1**(-1)*U1**(-1)*MS2**2 + 16*T1**(-1)*MS2**3
     +     - 8*U1*MS2 - 8*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(2,2)*CQED * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**3 - 16*S*T1**(-1)*MS2**2 + 8*S*MS2 + 8*S**2*T1**(-1)*
     +    U1**(-1)*MS2**2 + 12*S**2*T1**(-1)*MS2 - 2*S**2 - 2*S**3*
     +    T1**(-1) - 8*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(3,1)*N*CK * (  - 8*S*MS2 + 16*S**2*
     +    T1**(-1)*U1**(-1)*MS2**2 + 4*T1*U1 - 16*MS2**2 )
     +
      M2GGSV = M2GGSV + SK1D0(4,1)*N*CK * (  - M2*S*T1**(-1)*MS2 + 4*M2
     +    *S + 3*M2*U1 - M2**2*S*T1**(-1) + 4*M2**2 - 2*M2**3*T1**(-1)
     +     + S*U1 + S*MS2 + S**2 - T1*U1 )
     +
      M2GGSV = M2GGSV + SK1D0(4,1)*CQED * ( 3*M2*S - M2*S**2*T1**(-1)*
     +    U1**(-1)*MS2 - 4*M2*S**2*T1**(-1) - 4*M2*S**3*T1**(-1)*
     +    U1**(-1) - 4*M2**2*S*T1**(-1) - 5*M2**2*S**2*T1**(-1)*
     +    U1**(-1) - 2*M2**3*S*T1**(-1)*U1**(-1) + S*U1 - S**2*T1**(-1)
     +    *MS2 + 2*S**2 - S**3*T1**(-1)*U1**(-1)*MS2 - S**3*T1**(-1) - 
     +    S**4*T1**(-1)*U1**(-1) )
     +
      M2GGSV = M2GGSV + SK1D0(4,2)*N*CK * ( M2*S*T1**(-1)*MS2 + M2*S + 
     +    M2*S**2*T1**(-1)*U1**(-1)*MS2 - 3*M2*U1 + M2**2*S*T1**(-1) + 
     +    M2**2*S**2*T1**(-1)*U1**(-1) + 4*M2**2 + 2*M2**3*S*T1**(-1)*
     +    U1**(-1) + 2*M2**3*T1**(-1) - S*U1 + S*MS2 - T1*U1 )
     +
      M2GGSV = M2GGSV + SK1D0(4,2)*CQED * ( 3*M2*S - M2*S**2*T1**(-1)*
     +    U1**(-1)*MS2 + 4*M2*S**2*T1**(-1) + 4*M2**2*S*T1**(-1) - 
     +    M2**2*S**2*T1**(-1)*U1**(-1) - 2*M2**3*S*T1**(-1)*U1**(-1) - 
     +    S*U1 + S**2*T1**(-1)*MS2 + S**2 + S**3*T1**(-1) )
     +
      M2GGSV = M2GGSV + SK1D0(5,1)*N*CO * ( 4*M2*S**(-1)*U1*MS2 + M2*S*
     +    T1**(-1)*MS2 + 8*M2*T1**(-1)*MS2**2 + M2*U1 + 8*M2*MS2 + 4*
     +    M2**2*S**(-1)*U1 + M2**2*S*T1**(-1) + 8*M2**2*T1**(-1)*MS2 + 
     +    4*M2**2 + 2*M2**3*T1**(-1) - S*U1 + 3*S*MS2 - S**2 + T1*U1 + 
     +    4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(5,2)*N*CO * (  - 4*M2*S**(-1)*U1*MS2 - 8*
     +    M2*S*T1**(-1)*U1**(-1)*MS2**2 - M2*S*T1**(-1)*MS2 - M2*S - M2
     +    *S**2*T1**(-1)*U1**(-1)*MS2 - 8*M2*T1**(-1)*MS2**2 - M2*U1 + 
     +    4*M2*MS2 - 4*M2**2*S**(-1)*U1 - 8*M2**2*S*T1**(-1)*U1**(-1)*
     +    MS2 - M2**2*S*T1**(-1) - M2**2*S**2*T1**(-1)*U1**(-1) - 8*
     +    M2**2*T1**(-1)*MS2 - 2*M2**3*S*T1**(-1)*U1**(-1) - 2*M2**3*
     +    T1**(-1) + S*U1 - S*MS2 + T1*U1 - 4*U1*MS2 )
     +
      M2GGSV = M2GGSV + SK1D0(6,1)*N*CK * (  - 2*M2*S - 2*M2*S**2*
     +    T1**(-1)*U1**(-1)*MS2 - 8*M2*MS2 - 8*M2**2*S*T1**(-1)*
     +    U1**(-1)*MS2 - 2*M2**2*S**2*T1**(-1)*U1**(-1) - 4*M2**2 - 4*
     +    M2**3*S*T1**(-1)*U1**(-1) - 2*S*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(1)*N*CK * (  - 64*S**(-2)*T1*U1*MS2 + 32*
     +    S**(-1)*T1*U1 + 128*S**(-1)*MS2**2 - 16*S + 64*S**2*T1**(-2)*
     +    U1**(-2)*MS2**3 + 32*S**2*T1**(-1)*U1**(-1)*MS2 - 32*S**3*
     +    T1**(-2)*U1**(-2)*MS2**2 - 128*T1**(-1)*U1**(-1)*MS2**3 - 32*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(1)*CQED * ( 64*S*T1**(-1)*U1**(-1)*MS2**2
     +     + 16*S - 64*S**2*T1**(-2)*U1**(-2)*MS2**3 - 32*S**2*T1**(-1)
     +    *U1**(-1)*MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 - 32*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(2)*N*CO * ( 32*S**(-2)*T1*U1*MS2 - 64*
     +    S**(-1)*MS2**2 + 32*S*T1**(-1)*U1**(-1)*MS2**2 - 32*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 + 64*T1**(-1)*U1**(-1)*MS2**3 - 16*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(2)*N*CK * (  - 32*S**(-2)*T1*U1*MS2 + 64*
     +    S**(-1)*MS2**2 - 96*S*T1**(-1)*U1**(-1)*MS2**2 + 96*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 - 64*T1**(-1)*U1**(-1)*MS2**3 + 48*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(2)*CQED * ( 32*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 32*S**2*T1**(-2)*U1**(-2)*MS2**3 - 16*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(3)*N*CO * ( 32*S**(-2)*T1*U1*MS2 - 64*
     +    S**(-1)*MS2**2 + 32*S*T1**(-1)*U1**(-1)*MS2**2 - 32*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 + 64*T1**(-1)*U1**(-1)*MS2**3 - 16*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(3)*N*CK * (  - 32*S**(-2)*T1*U1*MS2 + 64*
     +    S**(-1)*MS2**2 - 96*S*T1**(-1)*U1**(-1)*MS2**2 + 96*S**2*
     +    T1**(-2)*U1**(-2)*MS2**3 - 64*T1**(-1)*U1**(-1)*MS2**3 + 48*
     +    MS2 )
     +
      M2GGSV = M2GGSV + SOF1(3)*CQED * ( 32*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 32*S**2*T1**(-2)*U1**(-2)*MS2**3 - 16*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(4)*N*CO * (  - 16*S**(-2)*T1*U1**2 - 16*
     +    S**(-1)*T1*U1 + 32*S**(-1)*U1*MS2 - 32*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 32*T1**(-1)*MS2**2 + 32*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(4)*N*CK * ( 32*S*T1**(-1)*MS2 - 32*S**2*
     +    T1**(-2)*U1**(-1)*MS2**2 - 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(5)*N*CO * ( 16*S**(-2)*T1*U1**2 - 32*
     +    S**(-1)*U1*MS2 + 32*T1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SOF1(5)*N*CK * (  - 32*S*T1**(-1)*MS2 + 16*S + 
     +    32*S**2*T1**(-2)*U1**(-1)*MS2**2 - 32*S**2*T1**(-1)*U1**(-1)*
     +    MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 + 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(6)*N*CO * ( 16*S**(-2)*T1*U1**2 - 32*
     +    S**(-1)*U1*MS2 + 32*T1**(-1)*MS2**2 )
     +
      M2GGSV = M2GGSV + SOF1(6)*N*CK * (  - 32*S*T1**(-1)*MS2 + 16*S + 
     +    32*S**2*T1**(-2)*U1**(-1)*MS2**2 - 32*S**2*T1**(-1)*U1**(-1)*
     +    MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 + 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(7)*N*CO * (  - 16*S**(-2)*T1*U1**2 - 16*
     +    S**(-1)*T1*U1 + 32*S**(-1)*U1*MS2 - 32*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 32*T1**(-1)*MS2**2 + 32*MS2 )
     +
      M2GGSV = M2GGSV + SOF1(7)*N*CK * ( 32*S*T1**(-1)*MS2 - 32*S**2*
     +    T1**(-2)*U1**(-1)*MS2**2 - 16*U1 )
     +
      M2GGSV = M2GGSV + SOF1(8)*N*CO * (  - 32*S**(-1)*T1*U1 - 64*S*
     +    T1**(-1)*U1**(-1)*MS2**2 + 16*S - 32*S**2*T1**(-1)*U1**(-1)*
     +    MS2 + 32*S**3*T1**(-2)*U1**(-2)*MS2**2 + 64*MS2 )

      END IF

      DSBGGV = 
     +     ALPHAS**3 * AVG *2.D0*(NS-1.D0)* M2GGSV /4.D0 /S**2 *CONV
     +    + ALPHAS/PI*DSBGGB(ALPHAS,S,T1,MS,MG) 
     +    * (LOG(MG2/MS2)*(-1.D0) + LOG(MT2/MS2)*(-1.D0/3.D0))
C***  CHANGES TO SUBTRACTED MSBAR 
      RETURN
      END



      REAL*8 FUNCTION DSBGGH(ALPHAS,S,T1,S4,MS,MG)
C***  CROSS SECTIONS FOR G + G -> SQ + SQB +G
C***  SUMMED OVER LL +RR 
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,U1,MS,MG,MS2,MG2,M2
      REAL*8 M2GGH
      REAL*8 NS,CONV,N,CO,CK,CQED,AVG
      REAL*8 ANGDEF(1:11), ANA(1:3,1:9), ANB(1:3,1:9), ANC(1:3,1:9)
      REAL*8 AHP1P1, ABP1P1, ABP1M1, ABP1P2, A4P2P2, A4P1P2, A4P1P1
      REAL*8 A4M1P2, A4M1P1, A4P0P2, A4P1P0,A4P2M2, A4M2P1,ABP2P2
      REAL*8 ABP2P1, ABP2P0
      REAL*8 ANG4(1:71), COLO1(1:9)
      
      CONV = 389379660.D0
      NS = 6.D0

      N = 3.D0
      CO = N*(N**2 -1.D0)
      CK = (N**2 -1.D0)/N
      CQED = (N**4 -1.D0)/N**2

      AVG = (1.D0/2.D0)**2 * (1.D0/(N**2 -1.D0))**2

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 - MS2
      U1 = S4 -S -T1


      ANGDEF(1) = 2.D0*SQRT(S4 +MS2)
      ANGDEF(2) = (S +U1)/ANGDEF(1)
      ANGDEF(3) = (S +T1)/ANGDEF(1)
      ANGDEF(4) = S4/ANGDEF(1)
      ANGDEF(5) = (S4 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(6) = -(T1 +U1 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(7) = SQRT((T1 +U1)**2 -4.D0*MS2*S)/ANGDEF(1)
      ANGDEF(8) = 
     +  (T1*S4 -S*(U1+2.D0*MS2))/(S+T1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(9) = SQRT(1 -ANGDEF(8)**2)
      ANGDEF(10) =
     +  (U1*S4 -S*(T1+2.D0*MS2))/(S+U1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(11) = SQRT(1 -ANGDEF(10)**2)

      ANA(1,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)
      ANC(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(1,2) = -ANB(1,1)
      ANC(1,2) = -ANC(1,1)
      ANA(1,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(1,3) = -ANA(1,3)
      ANC(1,3) =  0.D0
      ANA(1,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(1,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)-2.D0*ANGDEF(3)*ANGDEF(4)
      ANC(1,4) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(1,5) = -ANB(1,3)
      ANC(1,5) = -ANC(1,3)
      ANA(1,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(1,6) = -ANB(1,4)
      ANC(1,6) = -ANC(1,4)
      ANA(1,7) = +ANA(1,1) -M2
      ANB(1,7) = +ANB(1,1)
      ANC(1,7) = +ANC(1,1)
      ANA(1,8) = +ANA(1,5) -M2
      ANB(1,8) = +ANB(1,5)
      ANC(1,8) = +ANC(1,5)
      ANA(1,9) = +ANA(1,6) -M2
      ANB(1,9) = +ANB(1,6)
      ANC(1,9) = +ANC(1,6)


      ANA(2,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(2,1) = -2.D0*ANGDEF(4)*ANGDEF(7)
      ANC(2,1) =  0.D0
      ANA(2,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(2,2) = -ANB(2,1)
      ANC(2,2) = -ANC(2,1)
      ANA(2,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(2,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7) -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,4) = -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(2,5) = -ANB(2,3)
      ANC(2,5) = -ANC(2,3)
      ANA(2,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(2,6) = -ANB(2,4)
      ANC(2,6) = -ANC(2,4)
      ANA(2,7) = +ANA(2,1) -M2
      ANB(2,7) = +ANB(2,1)
      ANC(2,7) = +ANC(2,1)
      ANA(2,8) = +ANA(2,5) -M2
      ANB(2,8) = +ANB(2,5)
      ANC(2,8) = +ANC(2,5)
      ANA(2,9) = +ANA(2,6) -M2
      ANB(2,9) = +ANB(2,6)
      ANC(2,9) = +ANC(2,6)


      ANA(3,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10)
      ANC(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(3,2) = -ANB(3,1)
      ANC(3,2) = -ANC(3,1)
      ANA(3,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(3,3) = 
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10) -2.D0*ANGDEF(2)*ANGDEF(4)
      ANC(3,3) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(3,4) = -ANA(3,4)
      ANC(3,4) =  0.D0
      ANA(3,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(3,5) = -ANB(3,3)
      ANC(3,5) = -ANC(3,3)
      ANA(3,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(3,6) = -ANB(3,4)
      ANC(3,6) = -ANC(3,4)
      ANA(3,7) = +ANA(3,1) -M2
      ANB(3,7) = +ANB(3,1)
      ANC(3,7) = +ANC(3,1)
      ANA(3,8) = +ANA(3,5) -M2
      ANB(3,8) = +ANB(3,5)
      ANC(3,8) = +ANC(3,5)
      ANA(3,9) = +ANA(3,6) -M2
      ANB(3,9) = +ANB(3,6)
      ANC(3,9) = +ANC(3,6)


      ANG4(1) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(2) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(3) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(4) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(5) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(6) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(7) =A4P1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(8) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(9) =A4P2M2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(10) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(11) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(12) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(13) =A4P2M2(ANA(2,2),ANB(2,2),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(14) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(15) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(16) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(17) =A4P1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(18) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(19) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
      ANG4(20) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,5),ANB(3,5),ANC(3,5))
      ANG4(21) =A4M1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))

      ANG4(24) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(25) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(26) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,7),ANB(1,7),ANC(1,7))
      ANG4(27) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(28) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
      ANG4(29) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
      ANG4(30) =A4P1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))
      ANG4(31) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
      ANG4(32) =A4M1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))
      ANG4(33) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
      ANG4(34) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
      ANG4(35) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))

      ANG4(36) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(37) =A4P1P0(ANA(2,1),ANB(2,1),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(38) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(39) =A4P1P0(ANA(2,2),ANB(2,2),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(40) =A4P0P2(ANA(1,1),ANB(1,1),ANA(2,5),ANB(2,5),ANC(2,5))
      ANG4(41) =A4P1P0(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))

      ANG4(43) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,6),ANB(1,6),ANC(1,6))
      ANG4(44) =A4P1P0(ANA(3,6),ANB(3,6),ANA(1,1),ANB(1,1),ANC(1,1))

      ANG4(48) =AHP1P1(ANA(1,3),ANB(1,3),ANA(1,4),ANB(1,4),ANC(1,4))
      ANG4(49) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(50) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,1),ANB(3,1),ANC(3,1))

      ANG4(51) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(52) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(53) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(54) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))

      ANG4(55) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(56) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(57) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(58) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))


      ANG4(59) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(60) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(61) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(62) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(63) =ABP1M1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 

      ANG4(64) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 
      ANG4(65) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 
      ANG4(66) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 
      ANG4(67) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 
      ANG4(68) =ABP1M1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 

      ANG4(70) =ABP2P0(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(71) =ABP2P0(ANA(3,4),ANB(3,4),ANA(1,1),ANB(1,1),ANC(1,1))

      COLO1(9) = LOG(S4**2/MS2/(S4+MS2))


      M2GGH = 0.D0
      M2GGH = M2GGH + N*(S4+MS2)*CO * ( 32*S**(-2)*T1**(-2)*U1**3*
     +    S4**(-1) + 80*S**(-2)*T1**(-1)*U1**2*S4**(-1) - 40*S**(-2)*
     +    T1**(-1)*U1**3*S4**(-2) - 8*S**(-2)*T1**(-1)*U1**3*S4**(-1)*
     +    (T1+U1)**(-1) - 96*S**(-2)*T1*U1*S4**(-2) + 128*S**(-2)*T1*
     +    S4**(-1) + 64*S**(-2)*T1**2*U1**(-1)*S4**(-1) - 64*S**(-2)*
     +    T1**2*S4**(-2) + 32*S**(-2)*T1**3*U1**(-2)*S4**(-1) - 32*
     +    S**(-2)*T1**3*U1**(-1)*S4**(-2) + 96*S**(-2)*U1*S4**(-1) - 56
     +    *S**(-2)*U1**2*S4**(-2) + 8*S**(-2)*U1**2*S4**(-1)*
     +    (T1+U1)**(-1) - 128*S**(-1)*T1**(-3)*U1*MS2 + 64*S**(-1)*
     +    T1**(-3)*U1**2*S4**(-1)*MS2 + 64*S**(-1)*T1**(-2)*U1*S4**(-1)
     +    *MS2 + 56*S**(-1)*T1**(-2)*U1**2*S4**(-1) + 72*S**(-1)*
     +    T1**(-1)*U1*S4**(-1) - 48*S**(-1)*T1**(-1)*U1**2*S4**(-2) - 8
     +    *S**(-1)*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1) - 8*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-1)*(S+U1)**(-1) - 8*S**(-1)*T1**(-1)*
     +    U1**2*S4**(-1)*(T1+U1)**(-1) + 16*S**(-1)*T1**(-1)*U1**3*
     +    S4**(-2)*(S+T1)**(-1) )
     +
      M2GGH = M2GGH + N*(S4+MS2)*CO * ( 8*S**(-1)*T1**(-1)*U1**3*
     +    S4**(-2)*(T1+U1)**(-1) - 128*S**(-1)*T1*U1**(-3)*MS2 + 64*
     +    S**(-1)*T1*U1**(-2)*S4**(-1)*MS2 + 64*S**(-1)*T1*U1**(-1)*
     +    S4**(-1) - 8*S**(-1)*T1*U1*S4**(-1)*(S+T1)**(-2) - 8*S**(-1)*
     +    T1*U1*S4**(-1)*(S+U1)**(-2) - 112*S**(-1)*T1*S4**(-2) - 16*
     +    S**(-1)*T1*S4**(-1)*(S+T1)**(-1) - 16*S**(-1)*T1*S4**(-1)*
     +    (S+U1)**(-1) + 64*S**(-1)*T1**2*U1**(-3)*S4**(-1)*MS2 + 56*
     +    S**(-1)*T1**2*U1**(-2)*S4**(-1) - 32*S**(-1)*T1**2*U1**(-1)*
     +    S4**(-2) - 8*S**(-1)*T1**2*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 8
     +    *S**(-1)*T1**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 16*S**(-1)*
     +    T1**2*S4**(-2)*(S+U1)**(-1) + 16*S**(-1)*T1**3*U1**(-1)*
     +    S4**(-2)*(S+U1)**(-1) - 80*S**(-1)*U1*S4**(-2) - 8*S**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1) - 8*S**(-1)*U1*S4**(-1)*(S+U1)**(-1)
     +     - 8*S**(-1)*U1*S4**(-1)*(T1+U1)**(-1) - 24*S**(-1)*U1**2*
     +    S4**(-2)*(S+T1)**(-1) - 8*S**(-1)*U1**2*S4**(-2)*(S+U1)**(-1)
     +     - 8*S**(-1)*U1**2*S4**(-2)*(T1+U1)**(-1) )
     +
      M2GGH = M2GGH + N*(S4+MS2)*CO * ( 72*S**(-1)*S4**(-1) + 64*S*
     +    T1**(-4)*S4**(-1)*MS2**2 + 64*S*T1**(-3)*U1**(-1)*S4**(-1)*
     +    MS2**2 - 64*S*T1**(-3)*U1**(-1)*MS2 - 32*S*T1**(-3)*S4**(-1)*
     +    MS2 + 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2 + 16*S*T1**(-2)*
     +    S4**(-1)*(S+U1)**(-1)*MS2 + 16*S*T1**(-2)*S4**(-1) + 64*S*
     +    T1**(-1)*U1**(-3)*S4**(-1)*MS2**2 - 64*S*T1**(-1)*U1**(-3)*
     +    MS2 + 32*S*T1**(-1)*U1**(-2)*S4**(-1)*MS2 + 64*S*U1**(-4)*
     +    S4**(-1)*MS2**2 - 32*S*U1**(-3)*S4**(-1)*MS2 + 16*S*U1**(-2)*
     +    S4**(-1)*(S+T1)**(-1)*MS2 + 16*S*U1**(-2)*S4**(-1) + 96*S**2*
     +    T1**(-4)*U1**(-2)*MS2**2 - 64*S**2*T1**(-3)*U1**(-2)*S4**(-1)
     +    *MS2**2 + 32*S**2*T1**(-3)*U1**(-1)*S4**(-1)*MS2 + 96*S**2*
     +    T1**(-2)*U1**(-4)*MS2**2 - 64*S**2*T1**(-2)*U1**(-3)*S4**(-1)
     +    *MS2**2 + 32*S**2*T1**(-1)*U1**(-3)*S4**(-1)*MS2 - 64*S**3*
     +    T1**(-4)*U1**(-2)*S4**(-1)*MS2**2 - 64*S**3*T1**(-2)*U1**(-4)
     +    *S4**(-1)*MS2**2 + 64*T1**(-4)*MS2**2 + 16*T1**(-2)*U1*
     +    S4**(-1)*(S+U1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + N*(S4+MS2)*CO * ( 48*T1**(-2)*U1*S4**(-1) + 8*
     +    T1**(-2)*U1**2*S4**(-1)*(S+T1)**(-1) - 80*T1**(-2)*S4**(-1)*
     +    MS2 - 8*T1**(-1)*U1*S4**(-2) - 8*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1) - 8*T1**(-1)*U1*S4**(-1)*(S+U1)**(-1) - 8*
     +    T1**(-1)*U1**2*S4**(-2)*(T1+U1)**(-1) + 16*T1**(-1)*S4**(-1)
     +     + 16*T1*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MS2 + 48*T1*U1**(-2)*
     +    S4**(-1) - 16*T1*U1**(-1)*S4**(-2) - 8*T1*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) - 8*T1*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 8*T1*
     +    S4**(-1)*(S+U1)**(-2) + 8*T1**2*U1**(-2)*S4**(-1)*
     +    (S+U1)**(-1) + 64*U1**(-4)*MS2**2 - 80*U1**(-2)*S4**(-1)*MS2
     +     + 16*U1**(-1)*S4**(-1) - 8*U1*S4**(-2)*(S+T1)**(-1) - 8*U1*
     +    S4**(-2)*(S+U1)**(-1) - 8*U1*S4**(-2)*(T1+U1)**(-1) + 8*U1*
     +    S4**(-1)*(S+T1)**(-2) - 24*S4**(-2) + 8*S4**(-1)*(S+T1)**(-1)
     +     + 8*S4**(-1)*(S+U1)**(-1) )
     +
      M2GGH = M2GGH + N*(S4+MS2)*CK * ( 64*S*T1**(-3)*U1**(-1)*MS2 - 32
     +    *S*T1**(-3)*S4**(-1)*MS2 - 32*S*T1**(-2)*U1**(-1)*S4**(-1)*
     +    MS2 - 16*S*T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2 - 16*S*T1**(-2)
     +    *S4**(-1) + 64*S*T1**(-1)*U1**(-3)*MS2 - 32*S*T1**(-1)*
     +    U1**(-2)*S4**(-1)*MS2 - 8*S*T1**(-1)*S4**(-1)*(S+U1)**(-1) - 
     +    32*S*U1**(-3)*S4**(-1)*MS2 - 16*S*U1**(-2)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 - 16*S*U1**(-2)*S4**(-1) - 8*S*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) - 96*S**2*T1**(-4)*U1**(-2)*MS2**2 + 64
     +    *S**2*T1**(-4)*U1**(-1)*S4**(-1)*MS2**2 + 64*S**2*T1**(-3)*
     +    U1**(-2)*S4**(-1)*MS2**2 - 32*S**2*T1**(-3)*U1**(-1)*S4**(-1)
     +    *MS2 - 96*S**2*T1**(-2)*U1**(-4)*MS2**2 + 64*S**2*T1**(-2)*
     +    U1**(-3)*S4**(-1)*MS2**2 + 64*S**2*T1**(-1)*U1**(-4)*S4**(-1)
     +    *MS2**2 - 32*S**2*T1**(-1)*U1**(-3)*S4**(-1)*MS2 + 64*S**3*
     +    T1**(-4)*U1**(-2)*S4**(-1)*MS2**2 + 64*S**3*T1**(-2)*U1**(-4)
     +    *S4**(-1)*MS2**2 - 16*T1**(-2)*U1*S4**(-1)*(S+U1)**(-1)*MS2
     +     - 16*T1**(-2)*U1*S4**(-1) )
     +
      M2GGH = M2GGH + N*(S4+MS2)*CK * ( 16*T1**(-2)*S4**(-1)*MS2 - 8*
     +    T1**(-1)*U1*S4**(-1)*(S+U1)**(-1) - 8*T1**(-1)*S4**(-1) - 16*
     +    T1*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MS2 - 16*T1*U1**(-2)*
     +    S4**(-1) - 8*T1*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 8*T1*
     +    S4**(-2)*(S+U1)**(-1) - 16*T1*S4**(-1)*(S+U1)**(-2) + 16*
     +    U1**(-2)*S4**(-1)*MS2 - 8*U1**(-1)*S4**(-1) + 8*U1*S4**(-2)*
     +    (S+T1)**(-1) - 16*U1*S4**(-1)*(S+T1)**(-2) - 16*S4**(-2) - 24
     +    *S4**(-1)*(S+T1)**(-1) - 24*S4**(-1)*(S+U1)**(-1) )
     +
      M2GGH = M2GGH + N*CO * (  - 32*S**(-2)*T1**(-2)*U1**3*S4**(-1)*
     +    MS2 - 32*S**(-2)*T1**(-2)*U1**3 - 64*S**(-2)*T1**(-1)*U1**2*
     +    S4**(-1)*MS2 - 64*S**(-2)*T1**(-1)*U1**2 + 32*S**(-2)*
     +    T1**(-1)*U1**3*S4**(-2)*MS2 + 32*S**(-2)*T1**(-1)*U1**3*
     +    S4**(-1) - 32*S**(-2)*T1*U1**2*S4**(-1)*(S+U1)**(-2)*MS2 - 32
     +    *S**(-2)*T1*U1**2*(S+U1)**(-2) - 64*S**(-2)*T1**2*U1**(-1)*
     +    S4**(-1)*MS2 - 64*S**(-2)*T1**2*U1**(-1) - 32*S**(-2)*T1**2*
     +    U1*S4**(-1)*(S+T1)**(-2)*MS2 - 32*S**(-2)*T1**2*U1*
     +    (S+T1)**(-2) - 32*S**(-2)*T1**2*S4**(-1)*(S+T1)**(-1)*MS2 - 
     +    32*S**(-2)*T1**2*(S+T1)**(-1) - 32*S**(-2)*T1**3*U1**(-2)*
     +    S4**(-1)*MS2 - 32*S**(-2)*T1**3*U1**(-2) + 32*S**(-2)*T1**3*
     +    U1**(-1)*S4**(-2)*MS2 + 32*S**(-2)*T1**3*U1**(-1)*S4**(-1) - 
     +    16*S**(-2)*U1**2*S4**(-2)*MS2 - 32*S**(-2)*U1**2*S4**(-1)*
     +    (S+U1)**(-1)*MS2 - 32*S**(-2)*U1**2*(S+U1)**(-1) - 64*S**(-1)
     +    *T1**(-2)*U1**2*S4**(-1)*MS2 - 64*S**(-1)*T1**(-2)*U1**2 - 64
     +    *S**(-1)*T1**(-1)*U1*S4**(-1)*MS2 )
     +
      M2GGH = M2GGH + N*CO * (  - 56*S**(-1)*T1**(-1)*U1 + 64*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-2)*MS2 + 60*S**(-1)*T1**(-1)*U1**2*
     +    S4**(-1) + 8*S**(-1)*T1**(-1)*MS2 - 64*S**(-1)*T1*U1**(-1)*
     +    S4**(-1)*MS2 - 56*S**(-1)*T1*U1**(-1) - 32*S**(-1)*T1*U1*
     +    S4**(-1)*(S+T1)**(-2)*MS2 - 32*S**(-1)*T1*U1*S4**(-1)*
     +    (S+U1)**(-2)*MS2 - 32*S**(-1)*T1*U1*(S+T1)**(-2) - 32*S**(-1)
     +    *T1*U1*(S+U1)**(-2) - 32*S**(-1)*T1*S4**(-1)*(S+T1)**(-1)*MS2
     +     - 32*S**(-1)*T1*(S+T1)**(-1) - 64*S**(-1)*T1**2*U1**(-2)*
     +    S4**(-1)*MS2 - 64*S**(-1)*T1**2*U1**(-2) + 64*S**(-1)*T1**2*
     +    U1**(-1)*S4**(-2)*MS2 + 64*S**(-1)*T1**2*U1**(-1)*S4**(-1) + 
     +    16*S**(-1)*U1**(-1)*S4**(-1)*MS2**2 + 16*S**(-1)*U1**(-1)*MS2
     +     - 16*S**(-1)*U1*S4**(-2)*MS2 - 32*S**(-1)*U1*S4**(-1)*
     +    (S+U1)**(-1)*MS2 + 4*S**(-1)*U1*S4**(-1) - 32*S**(-1)*U1*
     +    (S+U1)**(-1) - 32*S**(-1)*S4**(-2)*MS2**2 - 32*S**(-1)*
     +    S4**(-1)*MS2 - 16*S*T1**(-2)*S4**(-1)*MS2 - 16*S*T1**(-2) + 
     +    16*S*T1**(-1)*S4**(-2)*MS2 )
     +
      M2GGH = M2GGH + N*CO * ( 16*S*T1**(-1)*S4**(-1) - 16*S*U1**(-2)*
     +    S4**(-1)*MS2 - 16*S*U1**(-2) + 16*S*U1**(-1)*S4**(-2)*MS2 + 
     +    16*S*U1**(-1)*S4**(-1) - 48*T1**(-2)*U1*S4**(-1)*MS2 - 48*
     +    T1**(-2)*U1 - 16*T1**(-2)*S4**(-2)*MS2**3 - 16*T1**(-2)*
     +    S4**(-1)*MS2**2 - 8*T1**(-2)*MS2 + 48*T1**(-1)*U1*S4**(-2)*
     +    MS2 + 44*T1**(-1)*U1*S4**(-1) - 16*T1**(-1)*S4**(-2)*MS2**2
     +     - 32*T1**(-1)*S4**(-1)*MS2 - 28*T1**(-1) - 48*T1*U1**(-2)*
     +    S4**(-1)*MS2 - 48*T1*U1**(-2) + 48*T1*U1**(-1)*S4**(-2)*MS2
     +     + 44*T1*U1**(-1)*S4**(-1) - 16*T1*S4**(-1)*(S+U1)**(-2)*MS2
     +     - 16*T1*(S+U1)**(-2) - 16*U1**(-2)*S4**(-2)*MS2**3 - 16*
     +    U1**(-2)*S4**(-1)*MS2**2 - 8*U1**(-2)*MS2 - 16*U1**(-1)*
     +    S4**(-2)*MS2**2 - 48*U1**(-1)*S4**(-1)*MS2 - 28*U1**(-1) - 16
     +    *U1*S4**(-1)*(S+T1)**(-2)*MS2 - 16*U1*(S+T1)**(-2) - 8*
     +    S4**(-2)*MS2 - 16*S4**(-1)*(S+T1)**(-1)*MS2 - 16*S4**(-1)*
     +    (S+U1)**(-1)*MS2 + 8*S4**(-1) - 16*(S+T1)**(-1) - 16*
     +    (S+U1)**(-1) )
     +
      M2GGH = M2GGH + N*CK * ( 64*S**(-2)*T1**(-1)*U1**(-1)*MS2**3 + 8*
     +    S**(-2)*T1**(-1)*U1*MS2 - 8*S**(-2)*T1**(-1)*U1**2*S4**(-1)*
     +    MS2 + 4*S**(-2)*T1**(-1)*U1**2 - 4*S**(-2)*T1**(-1)*U1**3*
     +    S4**(-1) - 64*S**(-2)*T1**(-1)*S4**(-1)*MS2**3 - 64*S**(-2)*
     +    U1**(-1)*S4**(-1)*MS2**3 - 8*S**(-2)*U1*S4**(-1)*MS2 + 16*
     +    S**(-2)*U1**2*S4**(-2)*MS2 - 4*S**(-2)*U1**2*S4**(-1) - 8*
     +    S**(-1)*T1**(-2)*U1*MS2 + 8*S**(-1)*T1**(-2)*U1**2*S4**(-1)*
     +    MS2 + 64*S**(-1)*T1**(-1)*U1**(-1)*MS2**2 - 16*S**(-1)*
     +    T1**(-1)*U1*S4**(-2)*MS2**2 + 8*S**(-1)*T1**(-1)*U1*S4**(-1)*
     +    MS2 - 4*S**(-1)*T1**(-1)*U1 - 4*S**(-1)*T1**(-1)*U1**2*
     +    S4**(-1) - 64*S**(-1)*T1**(-1)*S4**(-2)*MS2**3 - 48*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2**2 - 16*S**(-1)*T1**(-1)*MS2 - 8*
     +    S**(-1)*T1*U1**(-1) - 64*S**(-1)*U1**(-1)*S4**(-2)*MS2**3 - 
     +    80*S**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 16*S**(-1)*U1**(-1)*MS2
     +     + 16*S**(-1)*U1*S4**(-2)*MS2 - 8*S**(-1)*U1*S4**(-1) + 16*
     +    S**(-1)*S4**(-2)*MS2**2 )
     +
      M2GGH = M2GGH + N*CK * ( 40*S**(-1)*S4**(-1)*MS2 + 16*S*T1**(-2)*
     +    S4**(-1)*MS2 + 16*S*T1**(-2) + 16*S*T1**(-1)*U1**(-1) - 16*S*
     +    T1**(-1)*S4**(-2)*MS2 - 16*S*T1**(-1)*S4**(-1) + 16*S*
     +    U1**(-2)*S4**(-1)*MS2 + 16*S*U1**(-2) - 16*S*U1**(-1)*
     +    S4**(-2)*MS2 - 16*S*U1**(-1)*S4**(-1) + 24*T1**(-2)*U1*
     +    S4**(-1)*MS2 + 16*T1**(-2)*U1 + 48*T1**(-2)*S4**(-2)*MS2**3
     +     + 48*T1**(-2)*S4**(-1)*MS2**2 + 24*T1**(-2)*MS2 + 48*
     +    T1**(-1)*U1**(-1)*MS2 - 16*T1**(-1)*U1*S4**(-2)*MS2 - 16*
     +    T1**(-1)*U1*S4**(-1) + 32*T1**(-1)*S4**(-2)*MS2**2 + 40*
     +    T1**(-1)*S4**(-1)*MS2 + 44*T1**(-1) + 16*T1*U1**(-2)*S4**(-1)
     +    *MS2 + 16*T1*U1**(-2) - 16*T1*U1**(-1)*S4**(-2)*MS2 - 12*T1*
     +    U1**(-1)*S4**(-1) + 16*T1*S4**(-1)*(S+U1)**(-2)*MS2 + 16*T1*
     +    (S+U1)**(-2) + 48*U1**(-2)*S4**(-2)*MS2**3 + 48*U1**(-2)*
     +    S4**(-1)*MS2**2 + 24*U1**(-2)*MS2 + 48*U1**(-1)*S4**(-2)*
     +    MS2**2 + 48*U1**(-1)*S4**(-1)*MS2 + 44*U1**(-1) + 16*U1*
     +    S4**(-1)*(S+T1)**(-2)*MS2 )
     +
      M2GGH = M2GGH + N*CK * ( 16*U1*(S+T1)**(-2) + 24*S4**(-2)*MS2 + 
     +    16*S4**(-1)*(S+T1)**(-1)*MS2 + 16*S4**(-1)*(S+U1)**(-1)*MS2
     +     - 8*S4**(-1) + 16*(S+T1)**(-1) + 16*(S+U1)**(-1) )
     +
      M2GGH = M2GGH + CQED * (  - 32*S**(-2)*T1**(-1)*U1**(-1)*MS2**3
     +     + 32*S**(-2)*T1**(-1)*S4**(-1)*MS2**3 + 32*S**(-2)*U1**(-1)*
     +    S4**(-1)*MS2**3 - 32*S**(-1)*T1**(-1)*U1**(-1)*MS2**2 + 32*
     +    S**(-1)*T1**(-1)*S4**(-2)*MS2**3 + 32*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**2 + 32*S**(-1)*U1**(-1)*S4**(-2)*MS2**3 + 32*
     +    S**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 8*S*T1**(-1)*U1**(-1) - 16
     +    *T1**(-2)*S4**(-2)*MS2**3 - 16*T1**(-2)*S4**(-1)*MS2**2 - 8*
     +    T1**(-2)*MS2 - 24*T1**(-1)*U1**(-1)*MS2 - 16*T1**(-1)*
     +    S4**(-2)*MS2**2 - 8*T1**(-1) - 16*U1**(-2)*S4**(-2)*MS2**3 - 
     +    16*U1**(-2)*S4**(-1)*MS2**2 - 8*U1**(-2)*MS2 - 16*U1**(-1)*
     +    S4**(-2)*MS2**2 - 8*U1**(-1) - 8*S4**(-2)*MS2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * ( 32*S**(-2)*T1**(-2)*U1**3*
     +    S4**(-1)*MS2 + 32*S**(-2)*T1**(-2)*U1**3 + 64*S**(-2)*
     +    T1**(-1)*U1**2*S4**(-1)*MS2 + 64*S**(-2)*T1**(-1)*U1**2 - 32*
     +    S**(-2)*T1**(-1)*U1**3*S4**(-2)*MS2 - 32*S**(-2)*T1**(-1)*
     +    U1**3*S4**(-1) + 32*S**(-2)*T1*U1**2*S4**(-1)*(S+U1)**(-2)*
     +    MS2 + 32*S**(-2)*T1*U1**2*(S+U1)**(-2) + 64*S**(-2)*T1**2*
     +    U1**(-1)*S4**(-1)*MS2 + 64*S**(-2)*T1**2*U1**(-1) + 32*
     +    S**(-2)*T1**2*U1*S4**(-1)*(S+T1)**(-2)*MS2 + 32*S**(-2)*T1**2
     +    *U1*(S+T1)**(-2) + 32*S**(-2)*T1**2*S4**(-1)*(S+T1)**(-1)*MS2
     +     + 32*S**(-2)*T1**2*(S+T1)**(-1) + 32*S**(-2)*T1**3*U1**(-2)*
     +    S4**(-1)*MS2 + 32*S**(-2)*T1**3*U1**(-2) - 32*S**(-2)*T1**3*
     +    U1**(-1)*S4**(-2)*MS2 - 32*S**(-2)*T1**3*U1**(-1)*S4**(-1) + 
     +    32*S**(-2)*U1**2*S4**(-1)*(S+U1)**(-1)*MS2 + 32*S**(-2)*U1**2
     +    *(S+U1)**(-1) - 64*S**(-1)*T1**(-3)*U1**2*S4**(-1)*MS2**2 - 
     +    64*S**(-1)*T1**(-3)*U1**2*MS2 - 128*S**(-1)*T1**(-2)*U1*
     +    S4**(-1)*MS2**2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * (  - 128*S**(-1)*T1**(-2)*U1*MS2
     +     + 64*S**(-1)*T1**(-2)*U1**2*S4**(-2)*MS2**2 + 128*S**(-1)*
     +    T1**(-2)*U1**2*S4**(-1)*MS2 + 64*S**(-1)*T1**(-2)*U1**2 - 64*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*(S+U1)**(-1)*MS2**2 + 64*S**(-1)
     +    *T1**(-1)*U1*S4**(-1)*MS2 - 64*S**(-1)*T1**(-1)*U1*
     +    (S+U1)**(-1)*MS2 + 64*S**(-1)*T1**(-1)*U1 - 64*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-2)*MS2 - 64*S**(-1)*T1**(-1)*U1**2*
     +    S4**(-1) - 128*S**(-1)*T1*U1**(-2)*S4**(-1)*MS2**2 - 128*
     +    S**(-1)*T1*U1**(-2)*MS2 - 64*S**(-1)*T1*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2 + 64*S**(-1)*T1*U1**(-1)*S4**(-1)*MS2 - 
     +    64*S**(-1)*T1*U1**(-1)*(S+T1)**(-1)*MS2 + 64*S**(-1)*T1*
     +    U1**(-1) + 32*S**(-1)*T1*U1*S4**(-1)*(S+T1)**(-2)*MS2 + 32*
     +    S**(-1)*T1*U1*S4**(-1)*(S+U1)**(-2)*MS2 + 32*S**(-1)*T1*U1*
     +    (S+T1)**(-2) + 32*S**(-1)*T1*U1*(S+U1)**(-2) - 64*S**(-1)*T1*
     +    S4**(-1)*(S+T1)**(-2)*MS2**2 + 32*S**(-1)*T1*S4**(-1)*
     +    (S+T1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * (  - 64*S**(-1)*T1*(S+T1)**(-2)*
     +    MS2 + 32*S**(-1)*T1*(S+T1)**(-1) - 64*S**(-1)*T1**2*U1**(-3)*
     +    S4**(-1)*MS2**2 - 64*S**(-1)*T1**2*U1**(-3)*MS2 + 64*S**(-1)*
     +    T1**2*U1**(-2)*S4**(-2)*MS2**2 + 128*S**(-1)*T1**2*U1**(-2)*
     +    S4**(-1)*MS2 + 64*S**(-1)*T1**2*U1**(-2) - 64*S**(-1)*T1**2*
     +    U1**(-1)*S4**(-2)*MS2 - 64*S**(-1)*T1**2*U1**(-1)*S4**(-1) - 
     +    64*S**(-1)*U1*S4**(-1)*(S+U1)**(-2)*MS2**2 + 32*S**(-1)*U1*
     +    S4**(-1)*(S+U1)**(-1)*MS2 - 64*S**(-1)*U1*(S+U1)**(-2)*MS2 + 
     +    32*S**(-1)*U1*(S+U1)**(-1) + 128*S*T1**(-4)*S4**(-1)*MS2**3
     +     + 128*S*T1**(-4)*MS2**2 + 128*S*T1**(-3)*U1**(-1)*S4**(-1)*
     +    MS2**3 + 128*S*T1**(-3)*U1**(-1)*MS2**2 - 128*S*T1**(-3)*
     +    S4**(-2)*MS2**3 - 224*S*T1**(-3)*S4**(-1)*MS2**2 - 96*S*
     +    T1**(-3)*MS2 + 64*S*T1**(-2)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*
     +    MS2**3 - 64*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 + 64*S*
     +    T1**(-2)*U1**(-1)*(S+U1)**(-1)*MS2**2 - 64*S*T1**(-2)*
     +    U1**(-1)*MS2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * ( 96*S*T1**(-2)*S4**(-2)*MS2**2
     +     + 112*S*T1**(-2)*S4**(-1)*MS2 + 16*S*T1**(-2) + 128*S*
     +    T1**(-1)*U1**(-3)*S4**(-1)*MS2**3 + 128*S*T1**(-1)*U1**(-3)*
     +    MS2**2 + 64*S*T1**(-1)*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MS2**3
     +     - 64*S*T1**(-1)*U1**(-2)*S4**(-1)*MS2**2 + 64*S*T1**(-1)*
     +    U1**(-2)*(S+T1)**(-1)*MS2**2 - 64*S*T1**(-1)*U1**(-2)*MS2 + 
     +    64*S*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-2)*MS2**3 - 32*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2 + 64*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-2)*MS2**3 - 32*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2**2 + 64*S*
     +    T1**(-1)*U1**(-1)*(S+T1)**(-2)*MS2**2 - 32*S*T1**(-1)*
     +    U1**(-1)*(S+T1)**(-1)*MS2 + 64*S*T1**(-1)*U1**(-1)*
     +    (S+U1)**(-2)*MS2**2 - 32*S*T1**(-1)*U1**(-1)*(S+U1)**(-1)*MS2
     +     - 16*S*T1**(-1)*S4**(-2)*MS2 - 32*S*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-2)*MS2**2 - 16*S*T1**(-1)*S4**(-1) - 32*S*T1**(-1)*
     +    (S+T1)**(-2)*MS2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * ( 128*S*U1**(-4)*S4**(-1)*MS2**3
     +     + 128*S*U1**(-4)*MS2**2 - 128*S*U1**(-3)*S4**(-2)*MS2**3 - 
     +    224*S*U1**(-3)*S4**(-1)*MS2**2 - 96*S*U1**(-3)*MS2 + 96*S*
     +    U1**(-2)*S4**(-2)*MS2**2 + 112*S*U1**(-2)*S4**(-1)*MS2 + 16*S
     +    *U1**(-2) - 16*S*U1**(-1)*S4**(-2)*MS2 - 32*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-2)*MS2**2 - 16*S*U1**(-1)*S4**(-1) - 32*S*
     +    U1**(-1)*(S+U1)**(-2)*MS2 + 96*S**2*T1**(-4)*U1**(-1)*
     +    S4**(-1)*MS2**3 + 96*S**2*T1**(-4)*U1**(-1)*MS2**2 + 64*S**2*
     +    T1**(-3)*U1**(-2)*S4**(-1)*MS2**3 + 64*S**2*T1**(-3)*U1**(-2)
     +    *MS2**2 - 96*S**2*T1**(-3)*U1**(-1)*S4**(-2)*MS2**3 - 128*
     +    S**2*T1**(-3)*U1**(-1)*S4**(-1)*MS2**2 - 32*S**2*T1**(-3)*
     +    U1**(-1)*MS2 + 64*S**2*T1**(-2)*U1**(-3)*S4**(-1)*MS2**3 + 64
     +    *S**2*T1**(-2)*U1**(-3)*MS2**2 + 32*S**2*T1**(-2)*U1**(-2)*
     +    S4**(-1)*(S+T1)**(-1)*MS2**3 + 32*S**2*T1**(-2)*U1**(-2)*
     +    S4**(-1)*(S+U1)**(-1)*MS2**3 + 32*S**2*T1**(-2)*U1**(-2)*
     +    (S+T1)**(-1)*MS2**2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * ( 32*S**2*T1**(-2)*U1**(-2)*
     +    (S+U1)**(-1)*MS2**2 + 32*S**2*T1**(-2)*U1**(-1)*S4**(-2)*
     +    MS2**2 + 32*S**2*T1**(-2)*U1**(-1)*S4**(-1)*(S+T1)**(-2)*
     +    MS2**3 + 32*S**2*T1**(-2)*U1**(-1)*S4**(-1)*MS2 + 32*S**2*
     +    T1**(-2)*U1**(-1)*(S+T1)**(-2)*MS2**2 + 96*S**2*T1**(-1)*
     +    U1**(-4)*S4**(-1)*MS2**3 + 96*S**2*T1**(-1)*U1**(-4)*MS2**2
     +     - 96*S**2*T1**(-1)*U1**(-3)*S4**(-2)*MS2**3 - 128*S**2*
     +    T1**(-1)*U1**(-3)*S4**(-1)*MS2**2 - 32*S**2*T1**(-1)*U1**(-3)
     +    *MS2 + 32*S**2*T1**(-1)*U1**(-2)*S4**(-2)*MS2**2 + 32*S**2*
     +    T1**(-1)*U1**(-2)*S4**(-1)*(S+U1)**(-2)*MS2**3 + 32*S**2*
     +    T1**(-1)*U1**(-2)*S4**(-1)*MS2 + 32*S**2*T1**(-1)*U1**(-2)*
     +    (S+U1)**(-2)*MS2**2 + 32*S**3*T1**(-4)*U1**(-2)*S4**(-1)*
     +    MS2**3 + 32*S**3*T1**(-4)*U1**(-2)*MS2**2 - 32*S**3*T1**(-3)*
     +    U1**(-2)*S4**(-2)*MS2**3 - 32*S**3*T1**(-3)*U1**(-2)*S4**(-1)
     +    *MS2**2 + 32*S**3*T1**(-2)*U1**(-4)*S4**(-1)*MS2**3 + 32*S**3
     +    *T1**(-2)*U1**(-4)*MS2**2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * (  - 32*S**3*T1**(-2)*U1**(-3)*
     +    S4**(-2)*MS2**3 - 32*S**3*T1**(-2)*U1**(-3)*S4**(-1)*MS2**2
     +     + 64*T1**(-4)*U1*S4**(-1)*MS2**3 + 64*T1**(-4)*U1*MS2**2 - 
     +    64*T1**(-3)*U1*S4**(-2)*MS2**3 - 192*T1**(-3)*U1*S4**(-1)*
     +    MS2**2 - 128*T1**(-3)*U1*MS2 + 128*T1**(-3)*S4**(-1)*MS2**3
     +     + 128*T1**(-3)*MS2**2 + 128*T1**(-2)*U1*S4**(-2)*MS2**2 + 
     +    176*T1**(-2)*U1*S4**(-1)*MS2 + 48*T1**(-2)*U1 + 64*T1**(-2)*
     +    S4**(-1)*(S+U1)**(-1)*MS2**3 - 128*T1**(-2)*S4**(-1)*MS2**2
     +     + 64*T1**(-2)*(S+U1)**(-1)*MS2**2 - 128*T1**(-2)*MS2 - 48*
     +    T1**(-1)*U1*S4**(-2)*MS2 - 48*T1**(-1)*U1*S4**(-1) + 64*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-2)*MS2**3 - 64*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MS2**2 + 32*T1**(-1)*S4**(-1)*MS2 + 64*T1**(-1)*
     +    (S+U1)**(-2)*MS2**2 - 64*T1**(-1)*(S+U1)**(-1)*MS2 + 32*
     +    T1**(-1) + 64*T1*U1**(-4)*S4**(-1)*MS2**3 + 64*T1*U1**(-4)*
     +    MS2**2 - 64*T1*U1**(-3)*S4**(-2)*MS2**3 - 192*T1*U1**(-3)*
     +    S4**(-1)*MS2**2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CO * (  - 128*T1*U1**(-3)*MS2 + 128*T1
     +    *U1**(-2)*S4**(-2)*MS2**2 + 176*T1*U1**(-2)*S4**(-1)*MS2 + 48
     +    *T1*U1**(-2) - 48*T1*U1**(-1)*S4**(-2)*MS2 - 48*T1*U1**(-1)*
     +    S4**(-1) + 16*T1*S4**(-1)*(S+U1)**(-2)*MS2 + 16*T1*
     +    (S+U1)**(-2) + 128*U1**(-3)*S4**(-1)*MS2**3 + 128*U1**(-3)*
     +    MS2**2 + 64*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MS2**3 - 128*
     +    U1**(-2)*S4**(-1)*MS2**2 + 64*U1**(-2)*(S+T1)**(-1)*MS2**2 - 
     +    128*U1**(-2)*MS2 + 64*U1**(-1)*S4**(-1)*(S+T1)**(-2)*MS2**3
     +     - 64*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2 + 32*U1**(-1)*
     +    S4**(-1)*MS2 + 64*U1**(-1)*(S+T1)**(-2)*MS2**2 - 64*U1**(-1)*
     +    (S+T1)**(-1)*MS2 + 32*U1**(-1) + 16*U1*S4**(-1)*(S+T1)**(-2)*
     +    MS2 + 16*U1*(S+T1)**(-2) - 64*S4**(-1)*(S+T1)**(-2)*MS2**2 + 
     +    16*S4**(-1)*(S+T1)**(-1)*MS2 - 64*S4**(-1)*(S+U1)**(-2)*
     +    MS2**2 + 16*S4**(-1)*(S+U1)**(-1)*MS2 - 64*(S+T1)**(-2)*MS2
     +     + 16*(S+T1)**(-1) - 64*(S+U1)**(-2)*MS2 + 16*(S+U1)**(-1) )
     +
      M2GGH = M2GGH + COLO1(9)*N*CK * ( 32*S*T1**(-3)*S4**(-1)*MS2**2
     +     + 32*S*T1**(-3)*MS2 + 64*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2
     +     + 64*S*T1**(-2)*U1**(-1)*MS2 - 32*S*T1**(-2)*S4**(-2)*MS2**2
     +     - 48*S*T1**(-2)*S4**(-1)*MS2 - 16*S*T1**(-2) + 64*S*T1**(-1)
     +    *U1**(-2)*S4**(-1)*MS2**2 + 64*S*T1**(-1)*U1**(-2)*MS2 + 32*S
     +    *T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2 + 32*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2**2 + 32*S*
     +    T1**(-1)*U1**(-1)*(S+T1)**(-1)*MS2 + 32*S*T1**(-1)*U1**(-1)*
     +    (S+U1)**(-1)*MS2 + 16*S*T1**(-1)*S4**(-2)*MS2 + 32*S*T1**(-1)
     +    *S4**(-1)*(S+T1)**(-2)*MS2**2 + 16*S*T1**(-1)*S4**(-1) + 32*S
     +    *T1**(-1)*(S+T1)**(-2)*MS2 + 32*S*U1**(-3)*S4**(-1)*MS2**2 + 
     +    32*S*U1**(-3)*MS2 - 32*S*U1**(-2)*S4**(-2)*MS2**2 - 48*S*
     +    U1**(-2)*S4**(-1)*MS2 - 16*S*U1**(-2) + 16*S*U1**(-1)*
     +    S4**(-2)*MS2 + 32*S*U1**(-1)*S4**(-1)*(S+U1)**(-2)*MS2**2 + 
     +    16*S*U1**(-1)*S4**(-1) + 32*S*U1**(-1)*(S+U1)**(-2)*MS2 - 32*
     +    S**2*T1**(-4)*U1**(-1)*S4**(-1)*MS2**3 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CK * (  - 32*S**2*T1**(-4)*U1**(-1)*
     +    MS2**2 - 64*S**2*T1**(-3)*U1**(-2)*S4**(-1)*MS2**3 - 64*S**2*
     +    T1**(-3)*U1**(-2)*MS2**2 + 32*S**2*T1**(-3)*U1**(-1)*S4**(-2)
     +    *MS2**3 + 64*S**2*T1**(-3)*U1**(-1)*S4**(-1)*MS2**2 + 32*S**2
     +    *T1**(-3)*U1**(-1)*MS2 - 64*S**2*T1**(-2)*U1**(-3)*S4**(-1)*
     +    MS2**3 - 64*S**2*T1**(-2)*U1**(-3)*MS2**2 - 32*S**2*T1**(-2)*
     +    U1**(-2)*S4**(-1)*(S+T1)**(-1)*MS2**3 - 32*S**2*T1**(-2)*
     +    U1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2**3 - 32*S**2*T1**(-2)*
     +    U1**(-2)*(S+T1)**(-1)*MS2**2 - 32*S**2*T1**(-2)*U1**(-2)*
     +    (S+U1)**(-1)*MS2**2 - 32*S**2*T1**(-2)*U1**(-1)*S4**(-2)*
     +    MS2**2 - 32*S**2*T1**(-2)*U1**(-1)*S4**(-1)*(S+T1)**(-2)*
     +    MS2**3 - 32*S**2*T1**(-2)*U1**(-1)*S4**(-1)*MS2 - 32*S**2*
     +    T1**(-2)*U1**(-1)*(S+T1)**(-2)*MS2**2 - 32*S**2*T1**(-1)*
     +    U1**(-4)*S4**(-1)*MS2**3 - 32*S**2*T1**(-1)*U1**(-4)*MS2**2
     +     + 32*S**2*T1**(-1)*U1**(-3)*S4**(-2)*MS2**3 + 64*S**2*
     +    T1**(-1)*U1**(-3)*S4**(-1)*MS2**2 )
     +
      M2GGH = M2GGH + COLO1(9)*N*CK * ( 32*S**2*T1**(-1)*U1**(-3)*MS2
     +     - 32*S**2*T1**(-1)*U1**(-2)*S4**(-2)*MS2**2 - 32*S**2*
     +    T1**(-1)*U1**(-2)*S4**(-1)*(S+U1)**(-2)*MS2**3 - 32*S**2*
     +    T1**(-1)*U1**(-2)*S4**(-1)*MS2 - 32*S**2*T1**(-1)*U1**(-2)*
     +    (S+U1)**(-2)*MS2**2 - 32*S**3*T1**(-4)*U1**(-2)*S4**(-1)*
     +    MS2**3 - 32*S**3*T1**(-4)*U1**(-2)*MS2**2 + 32*S**3*T1**(-3)*
     +    U1**(-2)*S4**(-2)*MS2**3 + 32*S**3*T1**(-3)*U1**(-2)*S4**(-1)
     +    *MS2**2 - 32*S**3*T1**(-2)*U1**(-4)*S4**(-1)*MS2**3 - 32*S**3
     +    *T1**(-2)*U1**(-4)*MS2**2 + 32*S**3*T1**(-2)*U1**(-3)*
     +    S4**(-2)*MS2**3 + 32*S**3*T1**(-2)*U1**(-3)*S4**(-1)*MS2**2
     +     - 16*T1**(-2)*U1*S4**(-1)*MS2 - 16*T1**(-2)*U1 + 16*T1**(-1)
     +    *U1*S4**(-2)*MS2 + 16*T1**(-1)*U1*S4**(-1) - 32*T1**(-1)*
     +    S4**(-1)*MS2 - 32*T1**(-1) - 16*T1*U1**(-2)*S4**(-1)*MS2 - 16
     +    *T1*U1**(-2) + 16*T1*U1**(-1)*S4**(-2)*MS2 + 16*T1*U1**(-1)*
     +    S4**(-1) - 16*T1*S4**(-1)*(S+U1)**(-2)*MS2 - 16*T1*
     +    (S+U1)**(-2) )
     +
      M2GGH = M2GGH + COLO1(9)*N*CK * (  - 32*U1**(-1)*S4**(-1)*MS2 - 
     +    32*U1**(-1) - 16*U1*S4**(-1)*(S+T1)**(-2)*MS2 - 16*U1*
     +    (S+T1)**(-2) - 16*S4**(-1)*(S+T1)**(-1)*MS2 - 16*S4**(-1)*
     +    (S+U1)**(-1)*MS2 - 16*(S+T1)**(-1) - 16*(S+U1)**(-1) )
     +
      M2GGH = M2GGH + ANG4(1)*N*CO * (  - 8*MS2**3 )
     +
      M2GGH = M2GGH + ANG4(1)*N*CK * ( 24*MS2**3 )
     +
      M2GGH = M2GGH + ANG4(1)*CQED * (  - 8*MS2**3 )
     +
      M2GGH = M2GGH + ANG4(2)*N*CO * (  - 8*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(2)*N*CK * ( 32*U1**(-1)*MS2**3 + 24*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(2)*CQED * (  - 16*U1**(-1)*MS2**3 - 8*MS2**2
     +     )
     +
      M2GGH = M2GGH + ANG4(3)*N*CO * (  - 8*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(3)*N*CK * (  - 32*S**(-1)*MS2**3 + 24*MS2**2
     +     )
     +
      M2GGH = M2GGH + ANG4(3)*CQED * ( 16*S**(-1)*MS2**3 - 8*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(4)*N*CO * ( 8*S**(-1)*T1*U1**(-1)*MS2**2 + 8
     +    *S**(-1)*U1*(T1+U1)**(-1)*MS2**2 + 8*S*U1**(-1)*(T1+U1)**(-1)
     +    *MS2**2 + 2*S*U1*(T1+U1)**(-1) + 8*S*(T1+U1)**(-1)*MS2 + 8*
     +    U1**(-1)*MS2**2 - 8*U1*(T1+U1)**(-1)*MS2 - 8*(T1+U1)**(-1)*
     +    MS2**2 )
     +
      M2GGH = M2GGH + ANG4(4)*N*CK * (  - 32*S**(-2)*MS2**3 - 16*
     +    S**(-1)*T1**(-1)*U1*MS2**2 - 16*S**(-1)*T1**(-1)*MS2**3 - 24*
     +    S**(-1)*T1*U1**(-1)*MS2**2 - 16*S**(-1)*U1**(-1)*MS2**3 - 8*
     +    S**(-1)*U1*S4**(-1)*MS2**2 - 32*S**(-1)*MS2**2 - 8*S*T1**(-1)
     +    *U1**(-1)*MS2**2 - 2*S*T1**(-1)*U1 - 8*S*T1**(-1)*MS2 - 8*S*
     +    U1**(-1)*(S+T1)**(-1)*MS2**2 - 2*S*U1*(S+T1)**(-1) - 4*S*
     +    (S+T1)**(-1)*MS2 + 8*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 + 4*
     +    T1**(-1)*U1*MS2 + 8*T1**(-1)*MS2**2 + 8*T1*U1**(-1)*S4**(-1)*
     +    MS2**2 + 2*T1*U1*S4**(-1) + 4*T1*S4**(-1)*MS2 + 16*U1**(-1)*
     +    S4**(-1)*MS2**3 + 12*U1*S4**(-1)*MS2 + 4*U1*(S+T1)**(-1)*MS2
     +     - 2*U1 + 2*U1**2*S4**(-1) + 16*S4**(-1)*MS2**2 + 8*
     +    (S+T1)**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(4)*CQED * ( 16*S**(-2)*MS2**3 + 8*S**(-1)*
     +    T1**(-1)*U1*MS2**2 + 16*S**(-1)*T1**(-1)*MS2**3 + 8*S**(-1)*
     +    T1*U1**(-1)*MS2**2 + 16*S**(-1)*U1**(-1)*MS2**3 + 16*S**(-1)*
     +    MS2**2 - 24*T1**(-1)*U1*S4**(-1)*MS2**2 + 8*T1**(-1)*U1*MS2
     +     - 12*T1**(-1)*U1**2*S4**(-1)*MS2 + 2*T1**(-1)*U1**2 - 2*
     +    T1**(-1)*U1**3*S4**(-1) - 16*T1**(-1)*S4**(-1)*MS2**3 + 8*
     +    T1**(-1)*MS2**2 - 8*T1*U1**(-1)*S4**(-1)*MS2**2 - 2*T1*U1*
     +    S4**(-1) - 4*T1*S4**(-1)*MS2 - 16*U1**(-1)*S4**(-1)*MS2**3 - 
     +    16*U1*S4**(-1)*MS2 + 2*U1 - 4*U1**2*S4**(-1) - 24*S4**(-1)*
     +    MS2**2 + 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(5)*N*CO * (  - 8*MS2**3 )
     +
      M2GGH = M2GGH + ANG4(5)*N*CK * ( 24*MS2**3 )
     +
      M2GGH = M2GGH + ANG4(5)*CQED * (  - 8*MS2**3 )
     +
      M2GGH = M2GGH + ANG4(6)*N*CO * (  - 8*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(6)*N*CK * ( 32*T1**(-1)*MS2**3 + 24*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(6)*CQED * (  - 16*T1**(-1)*MS2**3 - 8*MS2**2
     +     )
     +
      M2GGH = M2GGH + ANG4(7)*N*CO * (  - 8*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(7)*N*CK * (  - 32*S**(-1)*MS2**3 + 24*MS2**2
     +     )
     +
      M2GGH = M2GGH + ANG4(7)*CQED * ( 16*S**(-1)*MS2**3 - 8*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(8)*N*CO * ( 8*S**(-1)*T1**(-1)*U1*MS2**2 - 8
     +    *S**(-1)*U1*(T1+U1)**(-1)*MS2**2 + 8*S**(-1)*MS2**2 + 8*S*
     +    T1**(-1)*U1**(-1)*MS2**2 - 8*S*U1**(-1)*(T1+U1)**(-1)*MS2**2
     +     - 2*S*U1*(T1+U1)**(-1) + 8*S*(T1+U1)**(-1)*MS2 + 2*S + 8*
     +    T1**(-1)*MS2**2 + 8*U1*(T1+U1)**(-1)*MS2 - 8*(T1+U1)**(-1)*
     +    MS2**2 - 8*MS2 )
     +
      M2GGH = M2GGH + ANG4(8)*N*CK * (  - 32*S**(-2)*MS2**3 - 24*
     +    S**(-1)*T1**(-1)*U1*MS2**2 - 16*S**(-1)*T1**(-1)*MS2**3 - 16*
     +    S**(-1)*T1*U1**(-1)*MS2**2 - 16*S**(-1)*U1**(-1)*MS2**3 + 8*
     +    S**(-1)*U1*S4**(-1)*MS2**2 - 40*S**(-1)*MS2**2 - 8*S*T1**(-1)
     +    *U1**(-1)*MS2**2 - 8*S*T1**(-1)*(S+U1)**(-1)*MS2**2 - 2*S*T1*
     +    U1**(-1) - 2*S*T1*(S+U1)**(-1) - 8*S*U1**(-1)*MS2 - 4*S*
     +    (S+U1)**(-1)*MS2 + 8*T1**(-1)*U1*S4**(-1)*MS2**2 + 16*
     +    T1**(-1)*S4**(-1)*MS2**3 + 8*T1*U1**(-1)*(S+U1)**(-1)*MS2**2
     +     + 4*T1*U1**(-1)*MS2 + 2*T1*U1*S4**(-1) + 12*T1*S4**(-1)*MS2
     +     + 4*T1*(S+U1)**(-1)*MS2 - 2*T1 + 2*T1**2*S4**(-1) + 8*
     +    U1**(-1)*MS2**2 + 4*U1*S4**(-1)*MS2 + 24*S4**(-1)*MS2**2 + 8*
     +    (S+U1)**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(8)*CQED * ( 16*S**(-2)*MS2**3 + 8*S**(-1)*
     +    T1**(-1)*U1*MS2**2 + 16*S**(-1)*T1**(-1)*MS2**3 + 8*S**(-1)*
     +    T1*U1**(-1)*MS2**2 + 16*S**(-1)*U1**(-1)*MS2**3 + 16*S**(-1)*
     +    MS2**2 - 8*T1**(-1)*U1*S4**(-1)*MS2**2 - 16*T1**(-1)*S4**(-1)
     +    *MS2**3 - 24*T1*U1**(-1)*S4**(-1)*MS2**2 + 8*T1*U1**(-1)*MS2
     +     - 2*T1*U1*S4**(-1) - 16*T1*S4**(-1)*MS2 + 2*T1 - 12*T1**2*
     +    U1**(-1)*S4**(-1)*MS2 + 2*T1**2*U1**(-1) - 4*T1**2*S4**(-1)
     +     - 2*T1**3*U1**(-1)*S4**(-1) - 16*U1**(-1)*S4**(-1)*MS2**3 + 
     +    8*U1**(-1)*MS2**2 - 4*U1*S4**(-1)*MS2 - 24*S4**(-1)*MS2**2 + 
     +    4*MS2 )
     +
      M2GGH = M2GGH + ANG4(9)*N*CO * (  - 8*S**(-2)*MS2 )
     +
      M2GGH = M2GGH + ANG4(9)*N*CK * ( 8*S**(-2)*MS2 )
     +
      M2GGH = M2GGH + ANG4(10)*N*CO * (  - 8*S**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(10)*N*CK * ( 8*S**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(11)*N*CO * ( 16*S**(-2)*(T1+U1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(12)*N*CO * ( 16*S**(-2)*U1*(T1+U1)**(-1)*MS2
     +     + 16*S**(-1)*(T1+U1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(12)*N*CK * (  - 4*S**(-2)*T1**(-1)*U1*MS2 + 
     +    4*S**(-2)*T1**(-1)*U1**2*S4**(-1)*MS2 - 2*S**(-2)*T1**(-1)*
     +    U1**2 + 2*S**(-2)*T1**(-1)*U1**3*S4**(-1) - 12*S**(-2)*U1*
     +    S4**(-1)*MS2 + 2*S**(-2)*U1**2*S4**(-1) + 4*S**(-1)*T1**(-2)*
     +    U1*MS2 - 4*S**(-1)*T1**(-2)*U1**2*S4**(-1)*MS2 - 8*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2 - 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2
     +     + 4*S**(-1)*T1**(-1)*MS2 + 2*S**(-1)*T1*U1**(-1) + 8*S**(-1)
     +    *U1**(-1)*S4**(-1)*MS2**2 + 4*S**(-1)*U1**(-1)*MS2 + 6*
     +    S**(-1)*U1*S4**(-1) - 16*S**(-1)*S4**(-1)*MS2 - 4*S**(-1) - 4
     +    *T1**(-2)*U1*S4**(-1)*MS2 - 2*T1*U1**(-1)*S4**(-1) - 8*
     +    U1**(-1)*S4**(-1)*MS2 + 4*S4**(-1) )
     +
      M2GGH = M2GGH + ANG4(13)*N*CO * (  - 16*S**(-2)*MS2 )
     +
      M2GGH = M2GGH + ANG4(14)*N*CO * (  - 32*S**(-2)*U1*MS2 - 16*
     +    S**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(15)*N*CO * ( 16*S**(-2)*(T1+U1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(16)*N*CO * (  - 16*S**(-2)*U1*S4**(-1)*MS2
     +     + 16*S**(-2)*U1*(T1+U1)**(-1)*MS2 - 8*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2 - 2*S**(-1)*T1**(-1)*U1**2*S4**(-1) - 8*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2**2 + 4*S**(-1)*T1**(-1)*MS2 + 2*S**(-1)
     +    *T1*U1**(-1) + 8*S**(-1)*U1**(-1)*S4**(-1)*MS2**2 + 4*S**(-1)
     +    *U1**(-1)*MS2 + 6*S**(-1)*U1*S4**(-1) - 16*S**(-1)*S4**(-1)*
     +    MS2 + 16*S**(-1)*(T1+U1)**(-1)*MS2 - 4*S**(-1) - 2*T1*
     +    U1**(-1)*S4**(-1) - 8*U1**(-1)*S4**(-1)*MS2 + 4*S4**(-1) )
     +
      M2GGH = M2GGH + ANG4(17)*N*CO * (  - 6*S**(-1)*T1*U1 - 4*S**(-1)*
     +    T1*MS2 - 8*S**(-1)*T1**2 - 4*S**(-1)*T1**3*U1**(-1) - 8*
     +    S**(-1)*U1*(T1+U1)**(-1)*MS2**2 - 2*S**(-1)*U1**2 - 8*S*
     +    T1**(-1)*U1**(-1)*MS2**2 - 2*S*T1**(-1)*U1 + 8*S*T1**(-1)*
     +    (S+U1)**(-1)*MS2**2 + 8*S*T1**(-1)*MS2 + 16*S*T1*U1**(-2)*MS2
     +     - 8*S*T1*U1**(-1) + 2*S*T1*(S+U1)**(-1) - 8*S*U1**(-1)*
     +    (T1+U1)**(-1)*MS2**2 + 8*S*U1**(-1)*MS2 - 2*S*U1*
     +    (T1+U1)**(-1) + 4*S*(S+U1)**(-1)*MS2 + 8*S*(T1+U1)**(-1)*MS2
     +     - 4*S - 2*T1**(-1)*U1**2 - 8*T1**(-1)*MS2**2 + 8*T1*U1**(-1)
     +    *MS2 - 4*T1*(S+U1)**(-1)*MS2 - 22*T1 - 16*T1**2*U1**(-1) - 4*
     +    T1**2*(S+U1)**(-1) - 4*T1**3*U1**(-1)*(S+U1)**(-1) + 8*U1*
     +    (T1+U1)**(-1)*MS2 - 12*U1 - 8*(T1+U1)**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(18)*N*CO * (  - 4*S**(-1)*T1**(-1)*U1**3 - 6
     +    *S**(-1)*T1*U1 - 2*S**(-1)*T1**2 + 8*S**(-1)*U1*(T1+U1)**(-1)
     +    *MS2**2 - 4*S**(-1)*U1*MS2 - 8*S**(-1)*U1**2 - 8*S**(-1)*
     +    MS2**2 + 16*S*T1**(-2)*U1*MS2 - 16*S*T1**(-1)*U1**(-1)*MS2**2
     +     - 8*S*T1**(-1)*U1 + 8*S*T1**(-1)*MS2 - 2*S*T1*U1**(-1) + 8*S
     +    *U1**(-1)*(S+T1)**(-1)*MS2**2 + 8*S*U1**(-1)*(T1+U1)**(-1)*
     +    MS2**2 + 8*S*U1**(-1)*MS2 + 2*S*U1*(S+T1)**(-1) + 2*S*U1*
     +    (T1+U1)**(-1) + 4*S*(S+T1)**(-1)*MS2 + 8*S*(T1+U1)**(-1)*MS2
     +     - 6*S + 8*T1**(-1)*U1*MS2 - 16*T1**(-1)*U1**2 - 4*T1**(-1)*
     +    U1**3*(S+T1)**(-1) - 12*T1 - 2*T1**2*U1**(-1) - 8*U1**(-1)*
     +    MS2**2 - 4*U1*(S+T1)**(-1)*MS2 - 8*U1*(T1+U1)**(-1)*MS2 - 22*
     +    U1 - 4*U1**2*(S+T1)**(-1) - 8*(T1+U1)**(-1)*MS2**2 + 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(19)*N*CK * ( 32*S**(-2)*MS2**3 + 16*S**(-1)*
     +    T1**(-1)*U1*MS2**2 + 16*S**(-1)*T1**(-1)*MS2**3 + 16*S**(-1)*
     +    T1*U1**(-1)*MS2**2 + 16*S**(-1)*U1**(-1)*MS2**3 + 32*S**(-1)*
     +    MS2**2 + 2*S*T1**(-1)*U1 + 8*S*T1**(-1)*(S+U1)**(-1)*MS2**2
     +     + 12*S*T1**(-1)*MS2 + 2*S*T1*U1**(-1) + 2*S*T1*(S+U1)**(-1)
     +     + 8*S*U1**(-1)*(S+T1)**(-1)*MS2**2 + 12*S*U1**(-1)*MS2 + 2*S
     +    *U1*(S+T1)**(-1) + 4*S*(S+T1)**(-1)*MS2 + 4*S*(S+U1)**(-1)*
     +    MS2 + 8*S + 2*S**2*T1**(-1) + 2*S**2*U1**(-1) - 8*T1**(-1)*U1
     +    *(S+T1)**(-1)*MS2**2 + 4*T1**(-1)*U1*MS2 + 16*T1**(-1)*MS2**2
     +     - 8*T1*U1**(-1)*(S+U1)**(-1)*MS2**2 + 4*T1*U1**(-1)*MS2 + 4*
     +    T1*(S+U1)**(-1)*MS2 + 16*U1**(-1)*MS2**2 + 4*U1*(S+T1)**(-1)*
     +    MS2 + 8*(S+T1)**(-1)*MS2**2 + 8*(S+U1)**(-1)*MS2**2 + 24*MS2
     +     )
     +
      M2GGH = M2GGH + ANG4(19)*CQED * (  - 16*S**(-2)*MS2**3 - 8*
     +    S**(-1)*T1**(-1)*U1*MS2**2 - 16*S**(-1)*T1**(-1)*MS2**3 - 8*
     +    S**(-1)*T1*U1**(-1)*MS2**2 - 16*S**(-1)*U1**(-1)*MS2**3 - 16*
     +    S**(-1)*MS2**2 - 24*S*T1**(-1)*U1**(-1)*MS2**2 - 2*S*T1**(-1)
     +    *U1 - 16*S*T1**(-1)*MS2 - 2*S*T1*U1**(-1) - 16*S*U1**(-1)*MS2
     +     - 4*S - 12*S**2*T1**(-1)*U1**(-1)*MS2 - 4*S**2*T1**(-1) - 4*
     +    S**2*U1**(-1) - 2*S**3*T1**(-1)*U1**(-1) - 16*T1**(-1)*
     +    U1**(-1)*MS2**3 - 4*T1**(-1)*U1*MS2 - 24*T1**(-1)*MS2**2 - 4*
     +    T1*U1**(-1)*MS2 - 24*U1**(-1)*MS2**2 - 12*MS2 )
     +
      M2GGH = M2GGH + ANG4(20)*N*CK * ( 8*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**2 - 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2 - 2*S**(-1)*T1*
     +    U1**(-1) - 16*S**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 4*S**(-1)*
     +    U1**(-1)*MS2 - 2*S**(-1)*U1*S4**(-1) - 4*S**(-1)*S4**(-1)*MS2
     +     - 2*S**(-1) + 2*S*T1**(-1)*U1**(-1) + 4*T1**(-1)*U1**(-1)*
     +    MS2 + 2*T1**(-1)*U1*S4**(-1) + 4*T1**(-1)*S4**(-1)*MS2 + 2*
     +    T1**(-1) )
     +
      M2GGH = M2GGH + ANG4(20)*CQED * (  - 8*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**2 + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2 + 8*S**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2 - 2*S*T1**(-1)*U1**(-1) - 4*T1**(-1)
     +    *U1**(-1)*MS2 - 2*T1**(-1)*U1*S4**(-1) - 4*T1**(-1)*S4**(-1)*
     +    MS2 - 2*T1**(-1) - 2*U1**(-1) )
     +
      M2GGH = M2GGH + ANG4(21)*N*CK * ( 8*S**(-1)*T1**(-2)*U1*S4**(-1)*
     +    MS2**2 - 8*S**(-1)*T1**(-2)*MS2**2 + 8*S**(-1)*T1**(-1)*
     +    U1**(-1)*MS2**2 + 4*S**(-1)*T1**(-1)*U1*S4**(-1)*MS2 - 2*
     +    S**(-1)*T1**(-1)*U1 - 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2 - 8*
     +    S**(-1)*T1**(-1)*MS2 - 8*S**(-1)*U1**(-1)*S4**(-1)*MS2**2 + 2
     +    *S**(-1)*U1*S4**(-1) - 4*S**(-1) + 2*S*T1**(-1)*U1**(-1) + 8*
     +    T1**(-2)*S4**(-1)*MS2**2 + 4*T1**(-1)*U1**(-1)*MS2 + 4*
     +    T1**(-1)*S4**(-1)*MS2 + 2*T1*U1**(-1)*S4**(-1) + 4*U1**(-1)*
     +    S4**(-1)*MS2 + 2*U1**(-1) + 2*S4**(-1) )
     +
      M2GGH = M2GGH + ANG4(21)*CQED * (  - 8*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**2 + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2 + 8*S**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2 - 2*S*T1**(-1)*U1**(-1) - 4*T1**(-1)
     +    *U1**(-1)*MS2 - 2*T1**(-1) - 2*T1*U1**(-1)*S4**(-1) - 4*
     +    U1**(-1)*S4**(-1)*MS2 - 2*U1**(-1) )
     +
      M2GGH = M2GGH + ANG4(36)*N*CO * (  - 16*S**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(36)*N*CK * ( 16*S**(-1)*MS2**2 + 12*MS2 )
     +
      M2GGH = M2GGH + ANG4(36)*CQED * (  - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(37)*N*CO * (  - 4*S**(-1)*T1 + 8*S**(-1)*U1*
     +    (T1+U1)**(-1)*MS2 + 4*S**(-1)*U1 - 8*S**(-1)*U1**2*
     +    (T1+U1)**(-1) - 16*S**(-1)*(T1+U1)**(-1)*MS2**2 - 16*S**(-1)*
     +    MS2 + 16*(T1+U1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(37)*N*CK * (  - 4*S**(-2)*T1**(-1)*U1**2*MS2
     +     + 4*S**(-2)*T1**(-1)*U1**3*S4**(-1)*MS2 - 2*S**(-2)*T1**(-1)
     +    *U1**3 + 2*S**(-2)*T1**(-1)*U1**4*S4**(-1) + 4*S**(-2)*U1**2*
     +    S4**(-1)*MS2 + 2*S**(-2)*U1**3*S4**(-1) + 4*S**(-1)*T1**(-2)*
     +    U1**2*MS2 - 4*S**(-1)*T1**(-2)*U1**3*S4**(-1)*MS2 - 16*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MS2**2 + 4*S**(-1)*T1**(-1)*U1*
     +    MS2 - 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MS2 + 2*S**(-1)*
     +    T1**(-1)*U1**3*S4**(-1) + 8*S**(-1)*T1**(-1)*MS2**2 - 4*
     +    S**(-1)*T1*U1**(-1)*MS2 - 8*S**(-1)*U1**(-1)*MS2**2 - 16*
     +    S**(-1)*U1*S4**(-1)*MS2 + 48*S**(-1)*S4**(-1)*MS2**2 + 16*
     +    S**(-1)*MS2 - 4*T1**(-2)*U1**2*S4**(-1)*MS2 - 4*T1**(-1)*U1*
     +    S4**(-1)*MS2 - 8*T1**(-1)*S4**(-1)*MS2**2 + 4*T1**(-1)*MS2 + 
     +    12*T1*U1**(-1)*S4**(-1)*MS2 - 4*T1*S4**(-1) + 2*T1**2*
     +    U1**(-1)*S4**(-1) + 16*U1**(-1)*S4**(-1)*MS2**2 - 2*U1*
     +    S4**(-1) - 32*S4**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(37)*CQED * ( 8*T1**(-1)*U1*S4**(-1)*MS2 + 2*
     +    T1**(-1)*U1**2*S4**(-1) + 8*T1**(-1)*S4**(-1)*MS2**2 - 4*
     +    T1**(-1)*MS2 + 8*T1*U1**(-1)*S4**(-1)*MS2 + 2*T1*S4**(-1) + 2
     +    *T1**2*U1**(-1)*S4**(-1) + 8*U1**(-1)*S4**(-1)*MS2**2 - 4*
     +    U1**(-1)*MS2 + 2*U1*S4**(-1) + 8*S4**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(38)*N*CO * (  - 16*S**(-2)*U1**2*MS2 - 16*
     +    S**(-1)*U1*MS2 - 48*MS2 )
     +
      M2GGH = M2GGH + ANG4(39)*N*CO * (  - 16*S**(-2)*U1**2*S4**(-1)*
     +    MS2 - 8*S**(-1)*T1**(-1)*U1*S4**(-1)*MS2**2 + 4*S**(-1)*
     +    T1**(-1)*U1*MS2 - 4*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MS2 + 4*
     +    S**(-1)*T1**(-1)*U1**2 - 2*S**(-1)*T1**(-1)*U1**3*S4**(-1) + 
     +    4*S**(-1)*T1*U1**(-1)*MS2 - 10*S**(-1)*T1 + 4*S**(-1)*T1**2*
     +    U1**(-1) - 20*S**(-1)*U1*S4**(-1)*MS2 + 8*S**(-1)*U1*
     +    (T1+U1)**(-1)*MS2 + 8*S**(-1)*U1 - 10*S**(-1)*U1**2*S4**(-1)
     +     - 8*S**(-1)*U1**2*(T1+U1)**(-1) + 8*S**(-1)*S4**(-1)*MS2**2
     +     - 16*S**(-1)*(T1+U1)**(-1)*MS2**2 - 28*S**(-1)*MS2 + 4*
     +    T1**(-1)*U1*S4**(-1)*MS2 - 2*T1**(-1)*U1**2*S4**(-1) - 8*
     +    T1**(-1)*MS2 + 8*T1*U1**(-1)*S4**(-1)*MS2 + 4*T1*S4**(-1) + 8
     +    *U1**(-1)*S4**(-1)*MS2**2 - 12*U1**(-1)*MS2 - 4*U1*S4**(-1)
     +     - 8*S4**(-1)*MS2 + 16*(T1+U1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(40)*N*CO * (  - 8*U1**(-2)*MS2**3 - 8*
     +    U1**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(40)*N*CK * ( 24*U1**(-2)*MS2**3 + 24*
     +    U1**(-1)*MS2**2 + 12*MS2 )
     +
      M2GGH = M2GGH + ANG4(40)*CQED * (  - 8*U1**(-2)*MS2**3 - 8*
     +    U1**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(41)*N*CO * (  - 4 - 2*S*U1**(-1) - 2*T1*
     +    U1**(-1) - 8*U1**(-2)*MS2**2 - 8*U1**(-1)*(S+T1)**(-1)*MS2**2
     +     - 8*U1**(-1)*MS2 - 4*U1*(S+T1)**(-1) - 8*(S+T1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(41)*N*CK * ( 16*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**3 - 16*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 - 4*S**(-1)*T1*
     +    U1**(-1)*MS2 - 16*S**(-1)*U1**(-1)*S4**(-1)*MS2**3 - 8*
     +    S**(-1)*U1**(-1)*MS2**2 - 4*S**(-1)*U1*S4**(-1)*MS2 - 8*
     +    S**(-1)*S4**(-1)*MS2**2 - 4*S**(-1)*MS2 + 12*S*T1**(-1)*
     +    U1**(-1)*MS2 + 2*S*U1**(-1) + 2*S**2*T1**(-1)*U1**(-1) + 16*
     +    T1**(-1)*U1**(-1)*MS2**2 + 8*T1**(-1)*U1*S4**(-1)*MS2 - 4*
     +    T1**(-1)*U1 + 8*T1**(-1)*S4**(-1)*MS2**2 + 32*U1**(-2)*
     +    S4**(-1)*MS2**3 + 24*U1**(-2)*MS2**2 + 16*U1**(-1)*S4**(-1)*
     +    MS2**2 + 8*U1**(-1)*(S+T1)**(-1)*MS2**2 + 8*U1**(-1)*MS2 + 4*
     +    U1*S4**(-1) + 4*U1*(S+T1)**(-1) + 16*S4**(-1)*MS2 + 8*
     +    (S+T1)**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(41)*CQED * (  - 4 - 16*S**(-1)*T1**(-1)*
     +    U1**(-1)*MS2**3 + 16*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 + 16*
     +    S**(-1)*U1**(-1)*S4**(-1)*MS2**3 - 16*S*T1**(-1)*U1**(-1)*MS2
     +     - 6*S*T1**(-1) - 6*S*U1**(-1) - 4*S**2*T1**(-1)*U1**(-1) - 
     +    24*T1**(-1)*U1**(-1)*MS2**2 - 12*T1**(-1)*U1*S4**(-1)*MS2 - 2
     +    *T1**(-1)*U1 - 2*T1**(-1)*U1**2*S4**(-1) - 16*T1**(-1)*
     +    S4**(-1)*MS2**2 - 12*T1**(-1)*MS2 - 2*T1*U1**(-1) - 16*
     +    U1**(-2)*S4**(-1)*MS2**3 - 8*U1**(-2)*MS2**2 - 8*U1**(-1)*
     +    S4**(-1)*MS2**2 - 12*U1**(-1)*MS2 - 2*U1*S4**(-1) - 8*
     +    S4**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(43)*N*CO * (  - 8*T1**(-2)*MS2**3 - 8*
     +    T1**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(43)*N*CK * ( 24*T1**(-2)*MS2**3 + 24*
     +    T1**(-1)*MS2**2 + 12*MS2 )
     +
      M2GGH = M2GGH + ANG4(43)*CQED * (  - 8*T1**(-2)*MS2**3 - 8*
     +    T1**(-1)*MS2**2 - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(44)*N*CO * (  - 4 - 2*S*T1**(-1) - 8*
     +    T1**(-2)*MS2**2 - 2*T1**(-1)*U1 - 8*T1**(-1)*(S+U1)**(-1)*
     +    MS2**2 - 8*T1**(-1)*MS2 - 4*T1*(S+U1)**(-1) - 8*(S+U1)**(-1)*
     +    MS2 )
     +
      M2GGH = M2GGH + ANG4(44)*N*CK * ( 16*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**3 + 8*S**(-1)*T1**(-1)*U1*S4**(-1)*MS2**2 - 4*S**(-1)*
     +    T1**(-1)*U1*MS2 - 16*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 - 16*
     +    S**(-1)*T1**(-1)*MS2**2 - 16*S**(-1)*U1**(-1)*S4**(-1)*MS2**3
     +     + 4*S**(-1)*U1*S4**(-1)*MS2 - 8*S**(-1)*MS2 + 12*S*T1**(-1)*
     +    U1**(-1)*MS2 + 2*S*T1**(-1) + 2*S**2*T1**(-1)*U1**(-1) + 32*
     +    T1**(-2)*S4**(-1)*MS2**3 + 24*T1**(-2)*MS2**2 + 16*T1**(-1)*
     +    U1**(-1)*MS2**2 + 24*T1**(-1)*S4**(-1)*MS2**2 + 8*T1**(-1)*
     +    (S+U1)**(-1)*MS2**2 + 8*T1**(-1)*MS2 + 8*T1*U1**(-1)*S4**(-1)
     +    *MS2 - 4*T1*U1**(-1) + 4*T1*S4**(-1) + 4*T1*(S+U1)**(-1) + 8*
     +    U1**(-1)*S4**(-1)*MS2**2 + 20*S4**(-1)*MS2 + 8*(S+U1)**(-1)*
     +    MS2 )
     +
      M2GGH = M2GGH + ANG4(44)*CQED * (  - 4 - 16*S**(-1)*T1**(-1)*
     +    U1**(-1)*MS2**3 + 16*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 + 16*
     +    S**(-1)*U1**(-1)*S4**(-1)*MS2**3 - 16*S*T1**(-1)*U1**(-1)*MS2
     +     - 6*S*T1**(-1) - 6*S*U1**(-1) - 4*S**2*T1**(-1)*U1**(-1) - 
     +    16*T1**(-2)*S4**(-1)*MS2**3 - 8*T1**(-2)*MS2**2 - 24*T1**(-1)
     +    *U1**(-1)*MS2**2 - 2*T1**(-1)*U1 - 8*T1**(-1)*S4**(-1)*MS2**2
     +     - 12*T1**(-1)*MS2 - 12*T1*U1**(-1)*S4**(-1)*MS2 - 2*T1*
     +    U1**(-1) - 2*T1*S4**(-1) - 2*T1**2*U1**(-1)*S4**(-1) - 16*
     +    U1**(-1)*S4**(-1)*MS2**2 - 12*U1**(-1)*MS2 - 8*S4**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(48)*N*CO * (  - 8*S**(-1)*T1**(-1)*U1**3 - 
     +    24*S**(-1)*T1*U1 - 16*S**(-1)*T1**2 - 8*S**(-1)*T1**3*
     +    U1**(-1) - 16*S**(-1)*U1**2 - 6*S*T1**(-1)*U1 - 8*S*T1**(-1)*
     +    (S+U1)**(-1)*MS2**2 - 6*S*T1*U1**(-1) - 2*S*T1*(S+U1)**(-1)
     +     - 8*S*U1**(-1)*(S+T1)**(-1)*MS2**2 - 2*S*U1*(S+T1)**(-1) - 4
     +    *S*(S+T1)**(-1)*MS2 - 4*S*(S+U1)**(-1)*MS2 - 16*S - 2*S**2*
     +    T1**(-1) - 2*S**2*U1**(-1) - 4*T1**(-1)*U1*MS2 - 8*T1**(-1)*
     +    U1**2 + 4*T1**(-1)*U1**3*(S+T1)**(-1) - 4*T1*U1**(-1)*MS2 - 4
     +    *T1*(S+U1)**(-1)*MS2 - 28*T1 - 8*T1**2*U1**(-1) - 4*T1**2*
     +    (S+U1)**(-1) + 4*T1**3*U1**(-1)*(S+U1)**(-1) - 4*U1*
     +    (S+T1)**(-1)*MS2 - 28*U1 - 4*U1**2*(S+T1)**(-1) )
     +
      M2GGH = M2GGH + ANG4(49)*N*CO * (  - 8*S**(-1)*T1**(-1)*U1*MS2**2
     +     + 2*S**(-1)*T1*U1 - 2*S**(-1)*T1**2 - 4*S**(-1)*U1*MS2 - 4*
     +    S**(-1)*U1**2 + 4*S**(-1)*U1**3*(T1+U1)**(-1) - 8*S**(-1)*
     +    MS2**2 - 8*S*T1**(-1)*U1**(-1)*MS2**2 + 8*S*U1**(-1)*
     +    (T1+U1)**(-1)*MS2**2 + 2*S*U1*(T1+U1)**(-1) - 4*S*
     +    (T1+U1)**(-1)*MS2 - 2*S - 8*T1**(-1)*MS2**2 - 4*T1 + 4*U1*
     +    (T1+U1)**(-1)*MS2 + 4*U1 - 4*U1**2*(T1+U1)**(-1) - 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(49)*N*CK * ( 8*S*T1**(-1)*U1**(-1)*MS2**2 + 
     +    2*S*T1*U1**(-1) - 8*S*U1**(-1)*(S+T1)**(-1)*MS2**2 + 4*S*
     +    U1**(-1)*MS2 - 2*S*U1*(S+T1)**(-1) - 4*S*(S+T1)**(-1)*MS2 + 8
     +    *T1**(-1)*U1*(S+T1)**(-1)*MS2**2 + 8*T1**(-1)*MS2**2 + 2*
     +    T1**2*U1**(-1) + 8*U1**(-1)*MS2**2 + 4*U1*(S+T1)**(-1)*MS2 + 
     +    2*U1 + 8*(S+T1)**(-1)*MS2**2 + 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(50)*N*CO * (  - 8*S**(-1)*T1*U1**(-1)*MS2**2
     +     - 2*S**(-1)*T1*U1 - 4*S**(-1)*T1*MS2 + 2*S**(-1)*U1**2 - 4*
     +    S**(-1)*U1**3*(T1+U1)**(-1) - 8*S**(-1)*MS2**2 - 8*S*U1**(-1)
     +    *(T1+U1)**(-1)*MS2**2 - 2*S*U1*(T1+U1)**(-1) - 4*S*
     +    (T1+U1)**(-1)*MS2 - 8*U1**(-1)*MS2**2 - 4*U1*(T1+U1)**(-1)*
     +    MS2 - 4*U1**2*(T1+U1)**(-1) )
     +
      M2GGH = M2GGH + ANG4(50)*N*CK * ( 8*S*T1**(-1)*U1**(-1)*MS2**2 + 
     +    2*S*T1**(-1)*U1 - 8*S*T1**(-1)*(S+U1)**(-1)*MS2**2 + 4*S*
     +    T1**(-1)*MS2 - 2*S*T1*(S+U1)**(-1) - 4*S*(S+U1)**(-1)*MS2 + 2
     +    *T1**(-1)*U1**2 + 8*T1**(-1)*MS2**2 + 8*T1*U1**(-1)*
     +    (S+U1)**(-1)*MS2**2 + 4*T1*(S+U1)**(-1)*MS2 + 2*T1 + 8*
     +    U1**(-1)*MS2**2 + 8*(S+U1)**(-1)*MS2**2 + 4*MS2 )
     +
      M2GGH = M2GGH + ANG4(51)*N*CO * (  - 16*S**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(52)*N*CO * (  - 16*S*MS2 )
     +
      M2GGH = M2GGH + ANG4(53)*N*CO * (  - 32*S*T1*U1**(-1)*MS2 - 16*
     +    S**2*U1**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(54)*N*CO * ( 8*S**(-1)*T1**(-1)*U1**3 + 40*
     +    S**(-1)*T1*U1 + 24*S**(-1)*T1**2 + 16*S**(-1)*T1**3*U1**(-1)
     +     + 16*S**(-1)*U1**2 + 4*S**(-1)*U1**3*S4**(-1) + 4*S**(-1)*
     +    U1**3*(T1+U1)**(-1) + 8*S*T1**(-1)*U1**(-1)*MS2**2 + 2*S*
     +    T1**(-1)*U1 - 4*S*T1**(-1)*MS2 - 16*S*T1*U1**(-2)*MS2 + 8*S*
     +    T1*U1**(-1) + 8*S*U1**(-1)*(S+T1)**(-1)*MS2**2 + 8*S*U1**(-1)
     +    *(T1+U1)**(-1)*MS2**2 - 24*S*U1**(-1)*MS2 + 2*S*U1*
     +    (S+T1)**(-1) + 2*S*U1*(T1+U1)**(-1) + 4*S*(S+T1)**(-1)*MS2 - 
     +    4*S*(T1+U1)**(-1)*MS2 + 4*S - 4*T1**(-1)*U1*MS2 + 4*T1**(-1)*
     +    U1**2 - 4*T1**(-1)*U1**3*(S+T1)**(-1) - 8*T1*U1**(-1)*
     +    S4**(-1)*MS2**2 - 16*T1*U1**(-1)*MS2 - 2*T1*U1*S4**(-1) - 4*
     +    T1*S4**(-1)*MS2 + 24*T1 + 24*T1**2*U1**(-1) + 8*U1**(-1)*
     +    MS2**2 - 4*U1*(S+T1)**(-1)*MS2 + 4*U1*(T1+U1)**(-1)*MS2 + 22*
     +    U1 + 2*U1**2*S4**(-1) - 4*U1**2*(S+T1)**(-1) - 4*U1**2*
     +    (T1+U1)**(-1) - 8*S4**(-1)*MS2**2 - 44*MS2 )
     +
      M2GGH = M2GGH + ANG4(55)*N*CO * (  - 16*S**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(56)*N*CO * (  - 16*S*MS2 )
     +
      M2GGH = M2GGH + ANG4(57)*N*CO * (  - 32*S*T1**(-1)*U1*MS2 - 16*
     +    S**2*T1**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(58)*N*CO * ( 16*S**(-1)*T1**(-1)*U1**3 + 32*
     +    S**(-1)*T1*U1 + 24*S**(-1)*T1**2 + 8*S**(-1)*T1**3*U1**(-1)
     +     + 32*S**(-1)*U1**2 - 4*S**(-1)*U1**3*S4**(-1) - 4*S**(-1)*
     +    U1**3*(T1+U1)**(-1) - 16*S*T1**(-2)*U1*MS2 + 16*S*T1**(-1)*
     +    U1**(-1)*MS2**2 + 8*S*T1**(-1)*U1 + 8*S*T1**(-1)*(S+U1)**(-1)
     +    *MS2**2 - 24*S*T1**(-1)*MS2 + 2*S*T1*U1**(-1) + 2*S*T1*
     +    (S+U1)**(-1) - 8*S*U1**(-1)*(T1+U1)**(-1)*MS2**2 - 4*S*
     +    U1**(-1)*MS2 - 2*S*U1*(T1+U1)**(-1) + 4*S*(S+U1)**(-1)*MS2 - 
     +    4*S*(T1+U1)**(-1)*MS2 + 6*S - 8*T1**(-1)*U1*S4**(-1)*MS2**2
     +     - 16*T1**(-1)*U1*MS2 + 24*T1**(-1)*U1**2 + 8*T1**(-1)*MS2**2
     +     - 4*T1*U1**(-1)*MS2 + 2*T1*U1*S4**(-1) - 4*T1*(S+U1)**(-1)*
     +    MS2 + 18*T1 + 4*T1**2*U1**(-1) - 2*T1**2*S4**(-1) - 4*T1**2*
     +    (S+U1)**(-1) - 4*T1**3*U1**(-1)*(S+U1)**(-1) - 4*U1*S4**(-1)*
     +    MS2 - 4*U1*(T1+U1)**(-1)*MS2 + 28*U1 - 4*U1**2*S4**(-1) - 4*
     +    U1**2*(T1+U1)**(-1) - 8*S4**(-1)*MS2**2 - 40*MS2 )
     +
      M2GGH = M2GGH + ANG4(59)*N*CO * (  - 8*T1**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(59)*N*CK * ( 8*T1**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(60)*N*CO * (  - 8*T1*MS2 - 16*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(60)*N*CK * ( 8*T1*MS2 + 16*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(61)*N*CO * (  - 16*S*T1*U1**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(61)*N*CK * ( 16*S*T1*U1**(-1)*MS2 + 16*T1**2
     +    *U1**(-1)*MS2 )
     +
      M2GGH = M2GGH + ANG4(62)*N*CO * ( 8*S**(-1)*T1**(-1)*U1*MS2**2 + 
     +    2*S**(-1)*T1*U1 - 4*S**(-1)*T1*MS2 + 4*S**(-1)*T1**2 + 4*
     +    S**(-1)*T1**3*U1**(-1) - 4*S**(-1)*U1*MS2 + 16*S*T1**(-1)*
     +    U1**(-1)*MS2**2 - 8*S*T1**(-1)*(S+U1)**(-1)*MS2**2 - 16*S*T1*
     +    U1**(-2)*MS2 + 8*S*T1*U1**(-1) - 2*S*T1*(S+U1)**(-1) - 8*S*
     +    U1**(-1)*MS2 - 4*S*(S+U1)**(-1)*MS2 + 16*T1**(-1)*MS2**2 - 8*
     +    T1*U1**(-1)*MS2 - 4*T1*(S+U1)**(-1)*MS2 + 6*T1 + 16*T1**2*
     +    U1**(-1) - 4*T1**2*(S+U1)**(-1) + 4*T1**3*U1**(-1)*
     +    (S+U1)**(-1) - 20*MS2 )
     +
      M2GGH = M2GGH + ANG4(62)*N*CK * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 8*S*T1*U1**(-1) + 8*S*U1**(-1)*(S+T1)**(-1)*MS2**2
     +     + 8*S*U1**(-1)*MS2 + 2*S*U1*(S+T1)**(-1) + 4*S*(S+T1)**(-1)*
     +    MS2 - 8*T1**(-1)*U1*S4**(-1)*MS2**2 - 8*T1**(-1)*U1*
     +    (S+T1)**(-1)*MS2**2 - 8*T1**(-1)*MS2**2 - 8*T1*U1**(-1)*
     +    S4**(-1)*MS2**2 + 16*T1*U1**(-1)*MS2 - 2*T1*U1*S4**(-1) - 4*
     +    T1*S4**(-1)*MS2 - 4*T1 - 8*T1**2*U1**(-1) + 40*U1**(-1)*
     +    MS2**2 - 4*U1*S4**(-1)*MS2 + 4*U1*(S+T1)**(-1)*MS2 - 2*U1 - 8
     +    *S4**(-1)*MS2**2 + 8*(S+T1)**(-1)*MS2**2 + 20*MS2 )
     +
      M2GGH = M2GGH + ANG4(63)*N*CO * (  - 2*S**(-1)*T1*U1**(-1) - 8*
     +    S**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 4*S**(-1)*U1**(-1)*MS2 - 2
     +    *S**(-1)*U1*S4**(-1) - 4*S**(-1)*S4**(-1)*MS2 - 2*S**(-1) - 2
     +    *U1**(-1) )
     +
      M2GGH = M2GGH + ANG4(63)*N*CK * (  - 8*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**2 + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2 + 8*S**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2 - 2*S*T1**(-1)*U1**(-1) - 4*T1**(-1)
     +    *U1**(-1)*MS2 - 2*T1**(-1)*U1*S4**(-1) - 4*T1**(-1)*S4**(-1)*
     +    MS2 - 2*T1**(-1) - 2*U1**(-1) )
     +
      M2GGH = M2GGH + ANG4(64)*N*CO * (  - 8*U1**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(64)*N*CK * ( 8*U1**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(65)*N*CO * (  - 8*U1*MS2 - 16*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(65)*N*CK * ( 8*U1*MS2 + 16*MS2**2 )
     +
      M2GGH = M2GGH + ANG4(66)*N*CO * (  - 16*S*T1**(-1)*U1*MS2 )
     +
      M2GGH = M2GGH + ANG4(66)*N*CK * ( 16*S*T1**(-1)*U1*MS2 + 16*
     +    T1**(-1)*U1**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(67)*N*CO * ( 4*S**(-1)*T1**(-1)*U1**3 + 8*
     +    S**(-1)*T1*U1**(-1)*MS2**2 + 2*S**(-1)*T1*U1 - 4*S**(-1)*T1*
     +    MS2 - 4*S**(-1)*U1*MS2 + 4*S**(-1)*U1**2 - 16*S*T1**(-2)*U1*
     +    MS2 + 16*S*T1**(-1)*U1**(-1)*MS2**2 + 8*S*T1**(-1)*U1 - 8*S*
     +    T1**(-1)*MS2 - 8*S*U1**(-1)*(S+T1)**(-1)*MS2**2 - 2*S*U1*
     +    (S+T1)**(-1) - 4*S*(S+T1)**(-1)*MS2 - 8*T1**(-1)*U1*MS2 + 16*
     +    T1**(-1)*U1**2 + 4*T1**(-1)*U1**3*(S+T1)**(-1) + 16*U1**(-1)*
     +    MS2**2 - 4*U1*(S+T1)**(-1)*MS2 + 6*U1 - 4*U1**2*(S+T1)**(-1)
     +     - 20*MS2 )
     +
      M2GGH = M2GGH + ANG4(67)*N*CK * (  - 16*S*T1**(-1)*U1**(-1)*
     +    MS2**2 - 8*S*T1**(-1)*U1 + 8*S*T1**(-1)*(S+U1)**(-1)*MS2**2
     +     + 8*S*T1**(-1)*MS2 + 2*S*T1*(S+U1)**(-1) + 4*S*(S+U1)**(-1)*
     +    MS2 - 8*T1**(-1)*U1*S4**(-1)*MS2**2 + 16*T1**(-1)*U1*MS2 - 8*
     +    T1**(-1)*U1**2 + 40*T1**(-1)*MS2**2 - 8*T1*U1**(-1)*S4**(-1)*
     +    MS2**2 - 8*T1*U1**(-1)*(S+U1)**(-1)*MS2**2 - 2*T1*U1*S4**(-1)
     +     - 4*T1*S4**(-1)*MS2 + 4*T1*(S+U1)**(-1)*MS2 - 2*T1 - 8*
     +    U1**(-1)*MS2**2 - 4*U1*S4**(-1)*MS2 - 4*U1 - 8*S4**(-1)*
     +    MS2**2 + 8*(S+U1)**(-1)*MS2**2 + 20*MS2 )
     +
      M2GGH = M2GGH + ANG4(68)*N*CO * (  - 4*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2 - 2*S**(-1)*T1**(-1)*U1 - 8*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**2 + 2*S**(-1)*U1*S4**(-1) - 8*S**(-1)*S4**(-1)*
     +    MS2 - 4*S**(-1) - 4*T1**(-1)*S4**(-1)*MS2 - 2*T1**(-1) + 2*
     +    S4**(-1) )
     +
      M2GGH = M2GGH + ANG4(68)*N*CK * (  - 8*S**(-1)*T1**(-1)*U1**(-1)*
     +    MS2**2 + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2 + 8*S**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2 - 2*S*T1**(-1)*U1**(-1) - 4*T1**(-1)
     +    *U1**(-1)*MS2 - 2*T1**(-1) - 2*T1*U1**(-1)*S4**(-1) - 4*
     +    U1**(-1)*S4**(-1)*MS2 - 2*U1**(-1) )
     +
      M2GGH = M2GGH + ANG4(70)*N*CO * (  - 16*S*T1*U1**(-2)*MS2 - 8*
     +    S**2*U1**(-2)*MS2 - 8*T1**2*U1**(-2)*MS2 )
     +
      M2GGH = M2GGH + ANG4(70)*N*CK * ( 16*S*T1*U1**(-2)*MS2 + 8*S**2*
     +    U1**(-2)*MS2 + 8*T1**2*U1**(-2)*MS2 )
     +
      M2GGH = M2GGH + ANG4(71)*N*CO * (  - 16*S*T1**(-2)*U1*MS2 - 8*
     +    S**2*T1**(-2)*MS2 - 8*T1**(-2)*U1**2*MS2 )
     +
      M2GGH = M2GGH + ANG4(71)*N*CK * ( 16*S*T1**(-2)*U1*MS2 + 8*S**2*
     +    T1**(-2)*MS2 + 8*T1**(-2)*U1**2*MS2 )

      DSBGGH = S4/(S4+MS2)/2.D0 *
     +     ALPHAS**3 * AVG *2.D0*(NS-1.D0)* M2GGH /4.D0 /S**2 *CONV
      RETURN
      END


      REAL*8 FUNCTION DSBGGD(ALPHAS,S,T1,S4,MS,MG,DEL,S4MAX)
C***  LOG(DEL) PART OF CROSS SECTIONS FOR G +G -> SQ + SQB +G 
C***  SUMMED OVER LL +RR 
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,SB,MS,MG,MS2
      REAL*8 M2GGD,CO,CK,CQED
      REAL*8 NS,CONV,N,AVG,BETA,XS,DEL,S4MAX,DLDEL1,DLDEL2
      REAL*8 COLO1(1:9)

      NS = 6.D0
      CONV = 389379660.D0
      N = 3.D0
      CO = N*(N**2 -1.D0)
      CK = (N**2-1.D0)/N
      CQED = (N**4 -1.D0)/N**2
      AVG = (1.D0/2.D0)**2 * (1.D0/(N**2 -1.D0))**2

      MS2 = MS**2
      SB = S*SQRT(1 -4.D0*MS2/S)
      BETA = SQRT(1 -4*MS2/S)
      XS = (1.D0 -BETA)/(1.D0 +BETA)

      DLDEL1 = LOG(S4MAX/MS2) -(S4MAX-DEL)/S4
      DLDEL2 = LOG(S4MAX/MS2)**2 -2.D0*(S4MAX -DEL)/S4*LOG(S4/MS2)

      COLO1(1) = LOG(XS)
      COLO1(3) = LOG(S/MS2)
      COLO1(4) = LOG(-T1/MS2)
      COLO1(8) = LOG((S+T1)/MS2)


      M2GGD = 0.D0
      M2GGD = M2GGD + N*CO*DLDEL1 * (  - 8 - 16*S**(-2)*T1**2 - 16*
     +    S**(-1)*T1 - 32*S**(-1)*MS2 - 16*S*T1**(-1)*(S+T1)**(-1)*MS2
     +     - 16*S**2*T1**(-2)*(S+T1)**(-2)*MS2**2 - 32*T1**(-1)*
     +    (S+T1)**(-1)*MS2**2 )
     +
      M2GGD = M2GGD + N*CO*DLDEL2 * ( 16 + 32*S**(-2)*T1**2 + 32*
     +    S**(-1)*T1 + 64*S**(-1)*MS2 + 32*S*T1**(-1)*(S+T1)**(-1)*MS2
     +     + 32*S**2*T1**(-2)*(S+T1)**(-2)*MS2**2 + 64*T1**(-1)*
     +    (S+T1)**(-1)*MS2**2 )
     +
      M2GGD = M2GGD + N*CK*DLDEL1 * ( 24 + 16*S**(-2)*T1**2 + 16*
     +    S**(-1)*T1 + 32*S**(-1)*MS2 + 48*S*T1**(-1)*(S+T1)**(-1)*MS2
     +     + 48*S**2*T1**(-2)*(S+T1)**(-2)*MS2**2 + 32*T1**(-1)*
     +    (S+T1)**(-1)*MS2**2 )
     +
      M2GGD = M2GGD + N*CK*DLDEL2 * (  - 16 - 32*S*T1**(-1)*
     +    (S+T1)**(-1)*MS2 - 32*S**2*T1**(-2)*(S+T1)**(-2)*MS2**2 )
     +
      M2GGD = M2GGD + CQED*DLDEL1 * (  - 8 - 16*S*T1**(-1)*(S+T1)**(-1)
     +    *MS2 - 16*S**2*T1**(-2)*(S+T1)**(-2)*MS2**2 )
     +
      M2GGD = M2GGD + COLO1(1)*N*CK*DLDEL1 * (  - 32*S**(-2)*T1**2*
     +    SB**(-1)*MS2 - 32*S**(-1)*T1*SB**(-1)*MS2 + 16*S**(-1)*T1**2*
     +    SB**(-1) - 64*S**(-1)*SB**(-1)*MS2**2 + 8*S*SB**(-1) - 32*
     +    S**2*T1**(-2)*(S+T1)**(-2)*SB**(-1)*MS2**3 + 16*S**2*T1**(-1)
     +    *(S+T1)**(-1)*SB**(-1)*MS2 + 16*S**3*T1**(-2)*(S+T1)**(-2)*
     +    SB**(-1)*MS2**2 - 64*T1**(-1)*(S+T1)**(-1)*SB**(-1)*MS2**3 + 
     +    16*T1*SB**(-1) + 16*SB**(-1)*MS2 )
     +
      M2GGD = M2GGD + COLO1(1)*CQED*DLDEL1 * ( 32*S*T1**(-1)*
     +    (S+T1)**(-1)*SB**(-1)*MS2**2 - 8*S*SB**(-1) + 32*S**2*
     +    T1**(-2)*(S+T1)**(-2)*SB**(-1)*MS2**3 - 16*S**2*T1**(-1)*
     +    (S+T1)**(-1)*SB**(-1)*MS2 - 16*S**3*T1**(-2)*(S+T1)**(-2)*
     +    SB**(-1)*MS2**2 + 16*SB**(-1)*MS2 )
     +
      M2GGD = M2GGD + COLO1(3)*N*CO*DLDEL1 * ( 8 + 16*S**(-2)*T1**2 + 
     +    16*S**(-1)*T1 + 32*S**(-1)*MS2 + 16*S*T1**(-1)*(S+T1)**(-1)*
     +    MS2 + 16*S**2*T1**(-2)*(S+T1)**(-2)*MS2**2 + 32*T1**(-1)*
     +    (S+T1)**(-1)*MS2**2 )
     +
      M2GGD = M2GGD + COLO1(4)*N*CO*DLDEL1 * (  - 16*S**(-2)*T1**2 - 32
     +    *S**(-1)*MS2 + 32*S*T1**(-2)*(S+T1)**(-1)*MS2**2 - 32*S*
     +    T1**(-1)*(S+T1)**(-1)*MS2 - 32*S**2*T1**(-2)*(S+T1)**(-2)*
     +    MS2**2 - 32*T1**(-1)*(S+T1)**(-1)*MS2**2 + 32*T1**(-1)*MS2 )
     +
      M2GGD = M2GGD + COLO1(8)*N*CO*DLDEL1 * (  - 16 - 16*S**(-2)*T1**2
     +     - 32*S**(-1)*T1 - 32*S**(-1)*MS2 - 32*S*T1**(-2)*
     +    (S+T1)**(-1)*MS2**2 - 32*T1**(-1)*(S+T1)**(-1)*MS2**2 - 32*
     +    T1**(-1)*MS2 )

      DSBGGD = ALPHAS**3 * AVG *2.D0*(NS-1.D0)* M2GGD /4.D0 /S**2 *CONV
      RETURN
      END




      REAL*8 FUNCTION DSBQB1(ALPHAS,S,T1,MS,MG,SCA,IFL)
C***  GIVES THE SCALE DEPENDENCE OF VIRTUAL
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,U1,MS,MG,SCA,PI,BETAL,PQQ,DSBQBB,NS
      INTEGER IFL
      PI = 4.D0*ATAN(1.D0)
      NS = 6.D0
      U1 = -S -T1
      BETAL = -11.D0 + 2.D0/3.D0*(NS -1.D0)
      PQQ = 4.D0/3.D0*(-2.D0*LOG(-T1/MS**2) -2*LOG(-U1/MS**2) +3.D0)
      DSBQB1 = DSBQBB(ALPHAS,S,T1,MS,MG,IFL)*
     +     ALPHAS/2.D0/PI*LOG(1.D0/SCA)*( BETAL +PQQ)
      RETURN
      END


      REAL*8 FUNCTION DSBQB2(ALPHAS,S,T1,S4,MS,MG,DEL,S4MAX,SCA,IFL)
C***  GIVES THE SCALE DEPENDENCE OF LOG(DEL)
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,S4,S4MAX,MS,MG,SCA,PI,DSBQBB,DLDEL1,DEL
      INTEGER IFL
      PI = 4.D0*ATAN(1.D0)
      DLDEL1 = LOG(S4MAX/MS**2) -(S4MAX -DEL)/S4
      DSBQB2 = 16.D0/3.D0*DLDEL1*DSBQBB(ALPHAS,S,T1,MS,MG,IFL)
     +     * ALPHAS/2.D0/PI*LOG(1.D0/SCA)
      RETURN
      END


      REAL*8 FUNCTION DSBQB3(ALPHAS,S,T1,S4,MS,MG,SCA,IFL)
C***  GIVES THE SCALE DEPENDENCE OF HARD
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,U1,S4,MS,MG,PI
      REAL*8 DSBQBB,PQQ1,PQQ2,SCA,X1,X2
      INTEGER IFL
      PI = 4.D0*ATAN(1.D0)
      U1 = S4 -S -T1
      X1 = -U1/(S+T1)
      X2 = -T1/(S+U1)
      PQQ1 = 4.D0/3.D0*(1.D0 +X1**2)/(1.D0 -X1)
      PQQ2 = 4.D0/3.D0*(1.D0 +X2**2)/(1.D0 -X2)
      DSBQB3 = ALPHAS/2.D0/PI*LOG(1.D0/SCA) *
     +     (-1/U1*DSBQBB(ALPHAS,X1*S,X1*T1,MS,MG,IFL)*PQQ1*X1**2
     +      -1/T1*DSBQBB(ALPHAS,X2*S,T1,MS,MG,IFL)*PQQ2*X2**2 )
      RETURN
      END

      REAL*8 FUNCTION DSBQBR(ALPHAS,S,T1,MS,MG,IFL)
C***  GIVES THE CHANGE FROM MSBAR TO DRBAR FOR THE YUKAWA COUPLINGS
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,TG,MS,MG,MS2,MG2
      REAL*8 M2QBB1,M2QBB2,M2QBB3,M2QBB4,M2QBB
      REAL*8 DRMS0, DRMS2, DRMS4
      REAL*8 NS,CONV,PI,N,CF,AVG
      INTEGER IFL
      NS = 6.D0
      CONV = 389379660.D0
      PI = 4.D0*ATAN(1.D0)
      N = 3.D0
      CF = (N**2 -1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N**2
      MS2 = MS**2
      MG2 = MG**2
      TG = T1 -MG2 +MS2 
      M2QBB1 = N*CF*4.D0/S**2*(T1*(-S-T1) -MS2*S)
      M2QBB2 = N*CF*2.D0/TG**2*(T1*(-S-T1) -MS2*S)
      M2QBB3 = N*CF*2.D0/TG**2*S*MG2
      M2QBB4 = -CF*4.D0/S/TG*(T1*(-S-T1) -MS2*S)

      DRMS0 = 0.D0
      DRMS2 = ALPHAS * 2.D0 /3.D0 /PI
      DRMS4 = ALPHAS * 4.D0 /3.D0 /PI

      IF (IFL.EQ.0) M2QBB = (2.D0*M2QBB2 +2.D0*M2QBB3)*DRMS4
      IF (IFL.EQ.1) M2QBB = 
     +     (2.D0*M2QBB2 +2.D0*M2QBB3)*DRMS4 + 2.D0*M2QBB4 *DRMS2
      DSBQBR = ALPHAS**2*AVG*M2QBB *PI/S**2 *CONV
      RETURN
      END


      REAL*8 FUNCTION DSBQBB(ALPHAS,S,T1,MS,MG,IFL)
C***  BORN CROSS SECTIONS FOR Q + QB -> SQ + SQB
C***  IFL = 0 <--> Q NOT QB
C***  IFL = 1 <--> Q  =  QB
C***  SUMMED OVER LL +RR +LR +RL
C***  M2QB?1: S-CHANNEL                ( Q QB  --> SQP SQBP )
C***  M2QB?2: T-CHANNEL, SAME HELICITY ( Q QBP --> SQL  SQBPL )
C***  M2QB?3: T-CHANNEL, DIFF HELICITY ( Q QBP --> SQL  SQBPR )
C***  M2QB?4: INTERFERENCE OF S- AND T-CHANNEL

      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,TG,MS,MS2,MG,MG2
      REAL*8 M2QBB1,M2QBB2,M2QBB3,M2QBB4,M2QBB
      REAL*8 NS,CONV,PI,N,CF,AVG
      INTEGER IFL
      NS = 6.D0
      CONV = 389379660.D0
      PI = 4.D0*ATAN(1.D0)
      N = 3.D0
      CF = (N**2 -1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N**2
      MS2 = MS**2
      MG2 = MG**2
      TG = T1 -MG2 +MS2 
      M2QBB1 = N*CF*4.D0/S**2*(T1*(-S-T1) -MS2*S)
      M2QBB2 = N*CF*2.D0/TG**2*(T1*(-S-T1) -MS2*S)
      M2QBB3 = N*CF*2.D0/TG**2*S*MG2
      M2QBB4 = -CF*4.D0/S/TG*(T1*(-S-T1) -MS2*S)
      IF (IFL.EQ.0) M2QBB = 2.D0*M2QBB2 +2.D0*M2QBB3 
      IF (IFL.EQ.1) M2QBB = 
     + 2.D0*(NS-1.D0)*M2QBB1 +2.D0*M2QBB2 +2.D0*M2QBB3 +2.D0*M2QBB4 
ctp      IF (IFL.EQ.1) M2QBB = 
ctp     + 2.D0*(NS-4.D0)*M2QBB1 +0*2.D0*M2QBB2 +0*2.D0*M2QBB3 +2.D0*M2QBB4 
      DSBQBB = ALPHAS**2*AVG*M2QBB *PI/S**2 *CONV
      RETURN
      END


      REAL*8 FUNCTION DSBQBB_NG(ALPHAS,S,T1,MS1,MS2,MG,ISQ1,ISQ2,PDF)

      IMPLICIT NONE
      REAL*8 PDF(1:6,1:6),PDFs,PDFt
      REAL*8 ALPHAS,S,T1,U2,TG,MG,MS1,MS2,T,U
      REAL*8 M2QBB1,M2QBB2,M2QBB3,M2QBB4,M2QBB
      REAL*8 MMsRR,MMtRR,MMtLR,MMst,MMs,MMt
      REAL*8 NS,CONV,PI,N,CF,AVG
      INTEGER ISQ1,ISQ2,I1

      NS = 6.D0
      CONV = 389379660.D0
      PI = 4.D0*ATAN(1.D0)
      N = 3.D0
      CF = (N**2 -1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N**2

ctp n.b. ISQ fixes final state, not initial state, so add for s channel
      PDFt = PDF(ABS(ISQ1),ABS(ISQ2))
      PDFs = 0.D0
      DO I1 = 1,5,1
         PDFs = PDFs + PDF(I1,I1)
      END DO

      U2 = -S -T1
      TG = T1 + MS1**2 - MG**2
      T = T1 +MS1**2
      U = U2 +MS2**2

      MMsRR = N*Cf * ( 4*s**(-2)*t*u - 4*ms1**2*ms2**2*s**(-2) )
      MMtRR = N*Cf*tg**(-2) * ( 2*t*u - 2*ms1**2*ms2**2 )
      MMtLR = N*Cf*tg**(-2) * ( 2*mg**2*s )
      MMst  = Cf*tg**(-1) * ( -4*s**(-1)*t*u + 4*ms1**2*ms2**2*s**(-1) )

ctp same flavor+helicity in final state? -> means s channel exists
      IF ( ISQ1 .EQ. ISQ2 ) THEN
         MMs = PDFs * MMsRR + PDFt * MMst
      ELSE  
         MMs = 0.D0
      END IF 

ctp same helicity in final state?
      IF ( (ISQ1/ABS(ISQ1)) .EQ. ISQ2/ABS(ISQ2) ) THEN
         MMt = PDFt * MMtRR
      ELSE 
         MMt = PDFt * MMtLR
      END IF 

      M2QBB = MMs + MMt

      DSBQBB_NG = ALPHAS**2*AVG*M2QBB *PI/S**2 *CONV

      RETURN
      END

      REAL*8 FUNCTION DSBQBV(ALPHAS,S,T1,MS,MG,MT,IFL)
C***  CROSS SECTIONS FOR Q + QB -> SQ + SQB
C***  IFL = 0 <--> Q NOT QB
C***  IFL = 1 <--> Q  =  QB
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T,T1,TG,U,U1,UG,SB,S1,MS,MG,MT,MS2,MG2,MT2,M2
      REAL*8 NS,CONV,PI,ZETA2,N,CF,AVG,BETA,XS,SPENCE,THREE
      REAL*8 SK1B0A(1:3), SK1B0B(1:4), SK1B0C(1:2), SK1B0D(1:3,1:2)
      REAL*8 SK1BP(1:4), SK1C0A(1:6), SK1C0B(1:4)
      REAL*8 SK1C0C(1:6,1:2), SK1D0(1:9,1:2), SK1B0E(1:3)
      REAL*8 SL1B0A, SL1B0B, SL1B0C, SL1B0D, SL1B0E
      REAL*8 SL1BP, SL1C0A, SL1C0B
      REAL*8 SL1C0C, SL1D0
      REAL*8 SOF1(1:8)
      REAL*8 M2QBV1,M2QBV2,M2QBV3,M2QBV4,M2QBV,DSBQBB, DSBQBR
      COMPLEX*16 KBETAG,KXG,KBETAT,KXT            
      INTEGER IFL
      PI = 4.D0*ATAN(1.D0)
      ZETA2 = PI**2/6.D0
      CONV = 389379660.D0

      NS = 6
      N = 3.D0
      CF = (N**2-1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N**2
      THREE = 3.D0

      MS2 = MS**2
      MG2 = MG**2
      MT2 = MT**2
      M2 = MG2 - MS2
      S1 = 4.D0*MS2 - S
      U1 = -S -T1
      T = T1 +MS2
      U = U1 +MS2
      TG = T1 -M2
      UG = U1 -M2

      BETA = SQRT(1.D0 - 4.D0*MS2/S)
      XS = (1.D0 -BETA)*(1.D0+BETA)**(-1)
      SB = S*BETA

      KBETAG = SQRT( DCMPLX(1.D0 - 4.D0*MG2/S)) 
      KXG = (DCMPLX(1.D0) -KBETAG)*(DCMPLX(1.D0)+KBETAG)**(-1)
      KBETAT = SQRT( DCMPLX(1.D0 - 4.D0*MT2/S)) 
      KXT = (DCMPLX(1.D0) -KBETAT)*(DCMPLX(1.D0)+KBETAT)**(-1)
      

      
      SK1B0A(1) = SL1B0A(MS2,MG2,MT2,1)
      SK1B0A(2) = SL1B0A(MS2,MG2,MT2,2)
      SK1B0A(3) = SL1B0A(MS2,MG2,MT2,3)

      SK1B0B(1) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,1)
      SK1B0B(2) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,2)
      SK1B0B(3) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,3)
      SK1B0B(4) = SL1B0B(MS2,MG2,MT2,S,BETA,XS,KBETAG,KXG,KBETAT,KXT,4)

      SK1B0C(1) = SL1B0C(MS2,MG2,MT2,1)
      SK1B0C(2) = SL1B0C(MS2,MG2,MT2,2)

      SK1B0D(1,1) = SL1B0D(MS2,MG2,MT2,T,1)
      SK1B0D(2,1) = SL1B0D(MS2,MG2,MT2,T,2)
      SK1B0D(3,1) = SL1B0D(MS2,MG2,MT2,T,3)
      SK1B0D(1,2) = SL1B0D(MS2,MG2,MT2,U,1)
      SK1B0D(2,2) = SL1B0D(MS2,MG2,MT2,U,2)
      SK1B0D(3,2) = SL1B0D(MS2,MG2,MT2,U,3)

      SK1B0E(1) = SL1B0E(MS2,MG2,MT2,1)
      SK1B0E(2) = SL1B0E(MS2,MG2,MT2,2)
      SK1B0E(3) = SL1B0E(MS2,MG2,MT2,3)

      SK1BP(1) = SL1BP(MS2,MG2,MT2,1)
      SK1BP(2) = SL1BP(MS2,MG2,MT2,2)
      SK1BP(3) = SL1BP(MS2,MG2,MT2,3)
      SK1BP(4) = SL1BP(MS2,MG2,MT2,4)


      IF (MS.EQ.MG) THEN

         SK1C0A(1) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,1)
         SK1C0A(2) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,2)
C$$$         SK1C0A(3) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,3)
         SK1C0A(4) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,4)
C$$$         SK1C0A(5) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,5)
C$$$         SK1C0A(6) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,6)
         
         SK1C0B(1) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,1)
C$$$         SK1C0B(2) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,2)
         SK1C0B(3) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,3)
C$$$         SK1C0B(4) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,4)
         
         SK1C0C(1,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,1)
C$$$         SK1C0C(2,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,2)
         SK1C0C(3,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,3)
C$$$         SK1C0C(4,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,4)
C$$$         SK1C0C(5,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,5)
C$$$         SK1C0C(6,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,6)
         
         SK1C0C(1,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,1)
C$$$         SK1C0C(2,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,2)
         SK1C0C(3,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,3)
C$$$         SK1C0C(4,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,4)
C$$$         SK1C0C(5,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,5)
C$$$         SK1C0C(6,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,6)
         
         SK1D0(1,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,1)
         SK1D0(2,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,2)
         SK1D0(3,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,3)
C$$$         SK1D0(4,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,4)
C$$$         SK1D0(5,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,5)
C$$$         SK1D0(6,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,6)
C$$$         SK1D0(7,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,7)
C$$$         SK1D0(8,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,8)
C$$$         SK1D0(9,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,9)
         
         SK1D0(1,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,1)
         SK1D0(2,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,2)
         SK1D0(3,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,3)
C$$$         SK1D0(4,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,4)
C$$$         SK1D0(5,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,5)
C$$$         SK1D0(6,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,6)
C$$$         SK1D0(7,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,7)
C$$$         SK1D0(8,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,8)
C$$$         SK1D0(9,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,9)

      ELSE

         SK1C0A(1) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,1)
C$$$         SK1C0A(2) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,2)
C$$$         SK1C0A(3) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,3)
C$$$         SK1C0A(4) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,4)
         SK1C0A(5) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,5)
         SK1C0A(6) = SL1C0A(MS2,MG2,MT2,S,XS,ZETA2,KXG,KXT,6)
         
         SK1C0B(1) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,1)
         SK1C0B(2) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,2)
         SK1C0B(3) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,3)
         SK1C0B(4) = SL1C0B(MS2,MG2,MT2,S,XS,ZETA2,SB,4)
         
C$$$         SK1C0C(1,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,1)
C$$$         SK1C0C(2,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,2)
         SK1C0C(3,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,3)
         SK1C0C(4,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,4)
         SK1C0C(5,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,5)
         SK1C0C(6,1) = SL1C0C(MS2,MG2,MT2,T,ZETA2,6)
         
C$$$         SK1C0C(1,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,1)
C$$$         SK1C0C(2,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,2)
         SK1C0C(3,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,3)
C$$$         SK1C0C(4,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,4)
C$$$         SK1C0C(5,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,5)
         SK1C0C(6,2) = SL1C0C(MS2,MG2,MT2,U,ZETA2,6)
         
         SK1D0(1,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,1)
C$$$         SK1D0(2,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,2)
C$$$         SK1D0(3,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,3)
         SK1D0(4,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,4)
C$$$         SK1D0(5,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,5)
C$$$         SK1D0(6,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,6)
         SK1D0(7,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,7)
         SK1D0(8,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,8)
         SK1D0(9,1) = SL1D0(MS2,MG2,MT2,S,T,U,SB,XS,ZETA2,9)
         
         SK1D0(1,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,1)
C$$$         SK1D0(2,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,2)
C$$$         SK1D0(3,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,3)
C$$$         SK1D0(4,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,4)
C$$$         SK1D0(5,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,5)
C$$$         SK1D0(6,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,6)
C$$$         SK1D0(7,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,7)
C$$$         SK1D0(8,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,8)
         SK1D0(9,2) = SL1D0(MS2,MG2,MT2,S,U,T,SB,XS,ZETA2,9)


      END IF

      SOF1(1) = 0.125D0/SB * ( +4*SPENCE(XS) +4*SPENCE(-XS) 
     +     +4*LOG(XS)*LOG(1-XS**2)-2*LOG(XS)**2 -2*ZETA2 )
      SOF1(2) = 0.125D0/MS2* (+2*(S -2*MS2)/SB*LOG(XS))
      SOF1(3) = 0.125D0/MS2*(-2)
      SOF1(4) = 0.125D0/U1*(-1.5D0*ZETA2)
      SOF1(5) = 0.125D0/T1*(-1.5D0*ZETA2)
      SOF1(6) = 0.125D0/T1*(-2*LOG(XS)**2 +LOG(XS*U1/T1)**2 
     +     +2*SPENCE(1 -T1/XS/U1) -2*SPENCE(1 -U1/XS/T1) -1.5D0*ZETA2 )
      SOF1(7) = 0.125D0/U1*(-2*LOG(XS)**2 +LOG(XS*T1/U1)**2 
     +     +2*SPENCE(1 -U1/XS/T1) -2*SPENCE(1 -T1/XS/U1) -1.5D0*ZETA2)
      SOF1(8) = 0.125D0/S*(LOG(T1*U1/MS2/S)**2
     +     +2*SPENCE(1 -MS2*S/T1/U1) -3*ZETA2 )

      IF (MS.EQ.MG) THEN

         IF (IFL.EQ.1) THEN

      M2QBV1 = 0.D0
      M2QBV1 = M2QBV1 + N*CF**2*THREE**(-1) * ( 24*S**(-2)*T1**2 + 24*
     +    S**(-1)*T1 + 24*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + N**2*CF*THREE**(-1) * (  - 8*S**(-2)*T1**2 - 8*
     +    S**(-1)*T1 - 8*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0A(2)*N*CF*THREE**(-1) * (  - 32*S**(-3)*
     +    T1**2*MT2 - 32*S**(-2)*T1*MT2 - 32*S**(-2)*MS2*MT2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(1)*N*CF*THREE**(-1) * ( 16*S**(-2)*T1**2
     +    *NS - 16*S**(-2)*T1**2 + 16*S**(-1)*T1*NS - 16*S**(-1)*T1 + 
     +    16*S**(-1)*NS*MS2 - 16*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(1)*N*CF**2*THREE**(-1) * ( 72*S**(-2)*
     +    T1**2 + 72*S**(-1)*T1 + 48*S**(-1)*T1**2*S1**(-1) + 72*
     +    S**(-1)*MS2 + 48*T1*S1**(-1) + 48*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(1)*N**2*CF*THREE**(-1) * (  - 64*S**(-2)
     +    *T1**2 - 64*S**(-1)*T1 - 48*S**(-1)*T1**2*S1**(-1) - 64*
     +    S**(-1)*MS2 - 48*T1*S1**(-1) - 48*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(2)*N*CF*THREE**(-1) * (  - 32*S**(-3)*
     +    T1**2*NS*MS2 - 32*S**(-2)*T1*NS*MS2 + 8*S**(-2)*T1**2*NS - 32
     +    *S**(-2)*NS*MS2**2 + 8*S**(-1)*T1*NS + 8*S**(-1)*NS*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(2)*N*CF**2*THREE**(-1) * ( 24*S**(-2)*
     +    T1**2 + 24*S**(-1)*T1 - 48*S**(-1)*T1**2*S1**(-1) + 24*
     +    S**(-1)*MS2 - 48*T1*S1**(-1) - 48*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(2)*N**2*CF*THREE**(-1) * ( 32*S**(-3)*
     +    T1**2*MS2 + 32*S**(-2)*T1*MS2 - 8*S**(-2)*T1**2 + 32*S**(-2)*
     +    MS2**2 - 8*S**(-1)*T1 + 48*S**(-1)*T1**2*S1**(-1) - 8*S**(-1)
     +    *MS2 + 48*T1*S1**(-1) + 48*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(4)*N*CF*THREE**(-1) * ( 32*S**(-3)*T1**2
     +    *MT2 + 32*S**(-2)*T1*MT2 + 16*S**(-2)*T1**2 + 32*S**(-2)*MS2*
     +    MT2 + 16*S**(-1)*T1 + 16*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0C(1)*N*CF**2 * (  - 16*S**(-2)*T1**2 - 16*
     +    S**(-1)*T1 - 16*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1BP(1)*N*CF**2 * (  - 32*S**(-2)*T1**2*MS2 - 
     +    32*S**(-1)*T1*MS2 - 32*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(1)*N*CF**2 * ( 16*S**(-1)*T1**2 + 16*S
     +     + 48*T1 + 16*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(1)*N**2*CF * (  - 16*S**(-1)*T1**2 - 8*S
     +     - 28*T1 - 12*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(2)*N*CF**2 * (  - 16*S**(-2)*T1**2*MS2
     +     - 16*S**(-1)*T1*MS2 - 16*S**(-1)*MS2**2 - 8*S - 16*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(2)*N**2*CF * ( 16*S**(-2)*T1**2*MS2 + 16
     +    *S**(-1)*T1*MS2 + 16*S**(-1)*MS2**2 + 4*S + 6*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(1)*N*CF**2 * (  - 32*S**(-2)*T1**2*MS2
     +     - 32*S**(-1)*T1*MS2 + 16*S**(-1)*T1**2 - 32*S**(-1)*MS2**2
     +     - 8*S + 16*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(1)*N**2*CF * ( 16*S**(-2)*T1**2*MS2 + 16
     +    *S**(-1)*T1*MS2 - 8*S**(-1)*T1**2 + 16*S**(-1)*MS2**2 + 4*S
     +     - 2*T1 - 12*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(3)*N*CF**2 * (  - 64*S**(-1)*T1*MS2 + 8*
     +    S**(-1)*T1**2 + 16*S + 32*T1*S1**(-1)*MS2 + 32*T1 + 8*T1**2*
     +    S1**(-1) + 32*S1**(-1)*MS2**2 - 32*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(3)*N**2*CF * ( 24*S**(-1)*T1*MS2 - 16*
     +    S**(-1)*T1**2 - 8*S - 32*T1*S1**(-1)*MS2 - 20*T1 - 8*T1**2*
     +    S1**(-1) - 32*S1**(-1)*MS2**2 + 12*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(1,1)*N*CF**2 * (  - 16*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(1,1)*N**2*CF * ( 4*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(1,2)*N*CF**2 * ( 16*S**(-1)*T1**2 + 16*S
     +     + 32*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(1,2)*N**2*CF * (  - 8*S**(-1)*T1**2 - 8*
     +    S - 16*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,1)*N*CF**2 * ( 32*S**(-1)*T1*MS2 + 32*
     +    S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,1)*N**2*CF * (  - 8*S**(-1)*T1*MS2 - 8
     +    *S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,2)*N*CF**2 * ( 32*S**(-1)*T1*MS2 - 32*
     +    S**(-1)*T1**2 - 32*S - 64*T1 + 32*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,2)*N**2*CF * (  - 16*S**(-1)*T1*MS2 + 
     +    16*S**(-1)*T1**2 + 16*S + 32*T1 - 16*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,1)*N*CF**2 * ( 32*S**(-1)*T1**3 + 16*T1
     +    *MS2 + 16*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,1)*N**2*CF * (  - 8*S**(-1)*T1**3 - 4*
     +    T1*MS2 - 4*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,2)*N*CF**2 * ( 32*S**(-1)*T1**3 + 64*S*
     +    T1 + 16*S*MS2 + 16*S**2 + 16*T1*MS2 + 80*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,2)*N**2*CF * (  - 16*S**(-1)*T1**3 - 32
     +    *S*T1 - 8*S*MS2 - 8*S**2 - 8*T1*MS2 - 40*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(2,1)*N*CF**2 * ( 16*T1*MS2 + 8*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(2,1)*N**2*CF * (  - 4*T1*MS2 - 2*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(2,2)*N*CF**2 * (  - 16*S*T1 + 16*S*MS2 - 
     +    8*S**2 + 16*T1*MS2 - 8*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(2,2)*N**2*CF * ( 8*S*T1 - 8*S*MS2 + 4*
     +    S**2 - 8*T1*MS2 + 4*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(1)*N*CF**2 * ( 128*S**(-2)*T1**2*MS2 + 128
     +    *S**(-1)*T1*MS2 - 64*S**(-1)*T1**2 + 128*S**(-1)*MS2**2 - 64*
     +    T1 - 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(1)*N**2*CF * (  - 64*S**(-2)*T1**2*MS2 - 
     +    64*S**(-1)*T1*MS2 + 32*S**(-1)*T1**2 - 64*S**(-1)*MS2**2 + 32
     +    *T1 + 32*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(2)*N*CF**2 * ( 64*S**(-2)*T1**2*MS2 + 64*
     +    S**(-1)*T1*MS2 + 64*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(3)*N*CF**2 * ( 64*S**(-2)*T1**2*MS2 + 64*
     +    S**(-1)*T1*MS2 + 64*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(4)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 256*S**(-1)*T1**2 - 128*T1 - 128*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(4)*N**2*CF * ( 64*S**(-2)*T1**3 + 64*
     +    S**(-1)*T1*MS2 + 128*S**(-1)*T1**2 + 64*T1 + 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(5)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 128*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(5)*N**2*CF * ( 32*S**(-2)*T1**3 + 32*
     +    S**(-1)*T1*MS2 + 32*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(6)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 128*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(6)*N**2*CF * ( 32*S**(-2)*T1**3 + 32*
     +    S**(-1)*T1*MS2 + 32*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(7)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 256*S**(-1)*T1**2 - 128*T1 - 128*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(7)*N**2*CF * ( 64*S**(-2)*T1**3 + 64*
     +    S**(-1)*T1*MS2 + 128*S**(-1)*T1**2 + 64*T1 + 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(8)*N*CF**2 * (  - 64*S**(-1)*T1**2 - 64*T1
     +     - 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(8)*N**2*CF * ( 32*S**(-1)*T1**2 + 32*T1 + 
     +    32*MS2 )


      M2QBV4 = 0.D0
      M2QBV4 = M2QBV4 + N*CF*THREE**(-1) * (  - 8 + 12*S**(-1)*T**(-1)*
     +    T1*MS2 + 24*S**(-1)*T**(-1)*MS2**2 - 8*S**(-1)*T1 - 24*
     +    S**(-1)*MS2 + 24*T**(-1)*T1**(-2)*MS2**3 + 36*T**(-1)*
     +    T1**(-1)*MS2**2 + 12*T**(-1)*MS2 - 24*T1**(-2)*MS2**2 - 32*
     +    T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + CF*THREE**(-1) * (  - 12*S**(-1)*T**(-1)*T1*NS*
     +    MS2 + 12*S**(-1)*T**(-1)*T1*MT2 - 24*S**(-1)*T**(-1)*NS*
     +    MS2**2 + 24*S**(-1)*T**(-1)*MS2*MT2 + 24*S**(-1)*NS*MS2 - 24*
     +    S**(-1)*MT2 - 24*T**(-1)*T1**(-2)*NS*MS2**3 + 24*T**(-1)*
     +    T1**(-2)*MS2**2*MT2 - 36*T**(-1)*T1**(-1)*NS*MS2**2 + 36*
     +    T**(-1)*T1**(-1)*MS2*MT2 - 12*T**(-1)*NS*MS2 + 12*T**(-1)*MT2
     +     + 24*T1**(-2)*NS*MS2**2 - 24*T1**(-2)*MS2*MT2 + 24*T1**(-1)*
     +    NS*MS2 - 24*T1**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + CF**2*THREE**(-1) * (  - 12 - 12*S**(-1)*T1 - 
     +    12*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0A(2)*CF*THREE**(-1) * ( 16*S**(-2)*T1*MT2
     +     + 12*S**(-1)*T**(-1)*T1*MT2 + 24*S**(-1)*T**(-1)*MS2*MT2 + 
     +    16*S**(-1)*T1**(-1)*MS2*MT2 - 8*S**(-1)*MT2 + 24*T**(-1)*
     +    T1**(-2)*MS2**2*MT2 + 36*T**(-1)*T1**(-1)*MS2*MT2 + 12*
     +    T**(-1)*MT2 - 24*T1**(-2)*MS2*MT2 - 24*T1**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(1)*N*CF*THREE**(-1) * ( 8 + 32*S**(-1)*
     +    T1 + 96*T1**(-1)*S1**(-1)*MS2**2 + 8*T1**(-1)*MS2 + 24*T1*
     +    S1**(-1) + 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(1)*CF*THREE**(-1) * ( 8 - 8*S**(-1)*T1*
     +    NS + 8*S**(-1)*T1 - 8*T1**(-1)*NS*MS2 + 8*T1**(-1)*MS2 - 8*NS
     +     )
     +
      M2QBV4 = M2QBV4 + SK1B0B(1)*CF**2*THREE**(-1) * (  - 12 - 36*
     +    S**(-1)*T1 - 96*T1**(-1)*S1**(-1)*MS2**2 - 12*T1**(-1)*MS2 - 
     +    24*T1*S1**(-1) - 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(2)*N*CF*THREE**(-1) * ( 28 - 16*S**(-2)*
     +    T1*MS2 - 16*S**(-1)*T1**(-1)*MS2**2 + 4*S**(-1)*T1 - 16*
     +    S**(-1)*MS2 - 96*T1**(-1)*S1**(-1)*MS2**2 + 28*T1**(-1)*MS2
     +     - 24*T1*S1**(-1) - 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(2)*CF*THREE**(-1) * ( 16*S**(-2)*T1*NS*
     +    MS2 + 16*S**(-1)*T1**(-1)*NS*MS2**2 - 4*S**(-1)*T1*NS + 16*
     +    S**(-1)*NS*MS2 - 4*T1**(-1)*NS*MS2 - 4*NS )
     +
      M2QBV4 = M2QBV4 + SK1B0B(2)*CF**2*THREE**(-1) * (  - 36 - 12*
     +    S**(-1)*T1 + 96*T1**(-1)*S1**(-1)*MS2**2 - 36*T1**(-1)*MS2 + 
     +    24*T1*S1**(-1) + 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(4)*CF*THREE**(-1) * (  - 8 - 16*S**(-2)*
     +    T1*MT2 - 16*S**(-1)*T1**(-1)*MS2*MT2 - 8*S**(-1)*T1 - 16*
     +    S**(-1)*MT2 - 8*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0C(1)*CF**2 * ( 16 + 16*S**(-1)*T1 + 32*
     +    S**(-1)*MS2 + 32*T1**(-2)*MS2**2 + 48*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*N*CF * ( 12 - 4*S**(-1)*T**(-1)*T1*
     +    MS2 - 8*S**(-1)*T**(-1)*MS2**2 + 12*S**(-1)*T1 + 24*S**(-1)*
     +    MS2 - 8*T**(-1)*T1**(-2)*MS2**3 - 12*T**(-1)*T1**(-1)*MS2**2
     +     - 4*T**(-1)*MS2 + 24*T1**(-2)*MS2**2 + 36*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*CF * ( 4 + 4*S**(-1)*T**(-1)*T1*NS*
     +    MS2 - 4*S**(-1)*T**(-1)*T1*MS2 + 8*S**(-1)*T**(-1)*NS*MS2**2
     +     - 8*S**(-1)*T**(-1)*MS2**2 - 4*S**(-1)*T1*NS + 4*S**(-1)*T1
     +     - 8*S**(-1)*NS*MS2 + 8*S**(-1)*MS2 + 8*T**(-1)*T1**(-2)*NS*
     +    MS2**3 - 8*T**(-1)*T1**(-2)*MS2**3 + 12*T**(-1)*T1**(-1)*NS*
     +    MS2**2 - 12*T**(-1)*T1**(-1)*MS2**2 + 4*T**(-1)*NS*MS2 - 4*
     +    T**(-1)*MS2 - 8*T1**(-2)*NS*MS2**2 + 8*T1**(-2)*MS2**2 - 12*
     +    T1**(-1)*NS*MS2 + 12*T1**(-1)*MS2 - 4*NS )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*CF**2 * (  - 16 - 16*S**(-1)*T1 - 
     +    32*S**(-1)*MS2 - 32*T1**(-2)*MS2**2 - 48*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(3,1)*CF * (  - 4 + 4*S**(-1)*T**(-1)*T1*
     +    MS2 - 4*S**(-1)*T**(-1)*T1*MT2 - 8*S**(-1)*T**(-1)*MS2*MT2 + 
     +    8*S**(-1)*T**(-1)*MS2**2 - 4*S**(-1)*T1 - 8*S**(-1)*MS2 - 8*
     +    T**(-1)*T1**(-2)*MS2**2*MT2 + 8*T**(-1)*T1**(-2)*MS2**3 - 12*
     +    T**(-1)*T1**(-1)*MS2*MT2 + 12*T**(-1)*T1**(-1)*MS2**2 + 4*
     +    T**(-1)*MS2 - 4*T**(-1)*MT2 - 8*T1**(-2)*MS2**2 - 12*T1**(-1)
     +    *MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0C(1)*N*CF * (  - 16*S**(-1)*MS2 - 16*
     +    T1**(-2)*MS2**2 - 16*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0E(3)*CF * ( 8*S**(-1)*MT2 + 8*T1**(-2)*MS2
     +    *MT2 + 8*T1**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + SK1BP(1)*CF**2 * ( 32*S**(-1)*T1*MS2 + 32*
     +    T1**(-1)*MS2**2 + 32*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(1)*N*CF * ( 6*S*T1**(-1)*MS2 + 16*S + 4*
     +    S**2*T1**(-1) + 10*T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(1)*CF**2 * (  - 8*S*T1**(-1)*MS2 - 20*S
     +     - 4*S**2*T1**(-1) - 12*T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(2)*N*CF * (  - 8*S**(-1)*T1*MS2 + 6*S*
     +    T1**(-1)*MS2 - 6*S - 4*S**2*T1**(-1) - 8*T1**(-1)*MS2**2 - 4*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(2)*CF**2 * ( 8*S**(-1)*T1*MS2 - 8*S*
     +    T1**(-1)*MS2 + 8*S + 4*S**2*T1**(-1) + 8*T1**(-1)*MS2**2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(1)*N*CF * (  - 16*S**(-1)*T1*MS2 + 14*S*
     +    T1**(-1)*MS2 + 2*S - 4*S**2*T1**(-1) - 8*T1**(-1)*MS2**2 + 8*
     +    T1 - 8*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(1)*CF**2 * ( 32*S**(-1)*T1*MS2 - 16*S*
     +    T1**(-1)*MS2 - 8*S + 4*S**2*T1**(-1) + 16*T1**(-1)*MS2**2 - 
     +    16*T1 + 24*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(3)*N*CF * (  - 6*S*T1**(-1)*MS2 + 8*S + 
     +    4*S**2*T1**(-1) + 64*T1**(-1)*S1**(-1)*MS2**3 - 16*T1**(-1)*
     +    MS2**2 + 16*T1*S1**(-1)*MS2 + 2*T1 + 64*S1**(-1)*MS2**2 - 24*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(3)*CF**2 * ( 8*S*T1**(-1)*MS2 - 4*S - 4*
     +    S**2*T1**(-1) - 64*T1**(-1)*S1**(-1)*MS2**3 + 16*T1**(-1)*
     +    MS2**2 - 16*T1*S1**(-1)*MS2 + 4*T1 - 64*S1**(-1)*MS2**2 + 24*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(1,1)*N*CF * ( 16*S**(-1)*T1*MS2 + 8*
     +    S**(-1)*T1**2 + 16*T1**(-1)*MS2**2 + 4*T1 + 20*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(1,1)*CF**2 * (  - 24*S**(-1)*T1*MS2 - 8*
     +    S**(-1)*T1**2 - 16*T1**(-1)*MS2**2 - 24*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(1,2)*N*CF * ( 8*S**(-1)*T1*MS2 - 8*
     +    S**(-1)*T1**2 - 8*S*T1**(-1)*MS2 + 8*S + 8*S**2*T1**(-1) - 8*
     +    T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(1,2)*CF**2 * (  - 8*S**(-1)*T1*MS2 + 8*
     +    S**(-1)*T1**2 + 8*S*T1**(-1)*MS2 - 8*S - 8*S**2*T1**(-1) + 8*
     +    T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,1)*N*CF * ( 8*S**(-1)*T1*MS2 + 12*
     +    S**(-1)*T1**2 + 16*T1 + 20*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,1)*CF**2 * (  - 8*S**(-1)*T1*MS2 - 16*
     +    S**(-1)*T1**2 - 24*T1 - 24*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,2)*N*CF * (  - 8*S**(-1)*T1*MS2 + 8*
     +    S**(-1)*T1**2 + 8*S*T1**(-1)*MS2 - 8*S - 8*S**2*T1**(-1) + 8*
     +    T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,2)*CF**2 * ( 8*S**(-1)*T1*MS2 - 8*
     +    S**(-1)*T1**2 - 8*S*T1**(-1)*MS2 + 8*S + 8*S**2*T1**(-1) - 8*
     +    T1 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,1)*N*CF * ( 4*S*T1 + 6*S*MS2 + 6*T1**2
     +     )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,1)*CF**2 * (  - 8*S*T1 - 12*S*MS2 - 12*
     +    T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,2)*N*CF * ( 20*S*T1 + 4*S*MS2 + 4*S**2*
     +    T1**(-1)*MS2 + 16*S**2 + 4*S**3*T1**(-1) + 8*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,2)*CF**2 * (  - 20*S*T1 - 4*S*MS2 - 4*
     +    S**2*T1**(-1)*MS2 - 16*S**2 - 4*S**3*T1**(-1) - 8*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(2,1)*N*CF * ( 2*S*T1 + 6*S*MS2 - 8*MS2**2
     +     )
     +
      M2QBV4 = M2QBV4 + SK1D0(2,1)*CF**2 * (  - 4*S*T1 - 12*S*MS2 + 16*
     +    MS2**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(2,2)*N*CF * (  - 4*S*T1 + 20*S*MS2 + 12*
     +    S**2*T1**(-1)*MS2 - 8*S**2 - 4*S**3*T1**(-1) + 8*T1*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(2,2)*CF**2 * ( 4*S*T1 - 20*S*MS2 - 12*
     +    S**2*T1**(-1)*MS2 + 8*S**2 + 4*S**3*T1**(-1) - 8*T1*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(3,1)*N*CF * ( 8*S**(-1)*T1**2*MS2 + 8*
     +    S**(-1)*T1**3 + 8*S*T1 + 16*S*MS2 + 24*T1*MS2 + 16*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(3,1)*CF**2 * (  - 8*S**(-1)*T1**2*MS2 - 8
     +    *S**(-1)*T1**3 - 8*S*T1 - 16*S*MS2 - 24*T1*MS2 - 16*T1**2 )
     +
      M2QBV4 = M2QBV4 + SOF1(1)*N*CF * ( 64*S**(-1)*T1*MS2 - 32*S*
     +    T1**(-1)*MS2 - 32*S + 64*T1**(-1)*MS2**2 - 32*T1 + 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(1)*CF**2 * (  - 128*S**(-1)*T1*MS2 + 64*S*
     +    T1**(-1)*MS2 + 64*S - 128*T1**(-1)*MS2**2 + 64*T1 - 128*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(2)*CF**2 * (  - 64*S**(-1)*T1*MS2 - 64*
     +    T1**(-1)*MS2**2 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(3)*CF**2 * (  - 64*S**(-1)*T1*MS2 - 64*
     +    T1**(-1)*MS2**2 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(4)*N*CF * (  - 64*S**(-1)*T1**2 - 64*S*
     +    T1**(-1)*MS2 - 64*S - 128*T1 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(4)*CF**2 * ( 64*S**(-1)*T1**2 + 64*S*
     +    T1**(-1)*MS2 + 64*S + 128*T1 + 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(5)*N*CF * (  - 32*S**(-1)*T1**2 - 32*T1 - 
     +    32*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(5)*CF**2 * ( 64*S**(-1)*T1**2 + 64*T1 + 64
     +    *MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(6)*N*CF * (  - 32*S**(-1)*T1**2 - 32*T1 - 
     +    32*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(6)*CF**2 * ( 64*S**(-1)*T1**2 + 64*T1 + 64
     +    *MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(7)*N*CF * (  - 64*S**(-1)*T1**2 - 64*S*
     +    T1**(-1)*MS2 - 64*S - 128*T1 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(7)*CF**2 * ( 64*S**(-1)*T1**2 + 64*S*
     +    T1**(-1)*MS2 + 64*S + 128*T1 + 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(8)*N*CF * (  - 32*S*T1**(-1)*MS2 - 32*S - 
     +    32*T1 )
     +
      M2QBV4 = M2QBV4 + SOF1(8)*CF**2 * ( 64*S*T1**(-1)*MS2 + 64*S + 64
     +    *T1 )

         END IF

      M2QBV2 = 0.D0
      M2QBV2 = M2QBV2 + N*CF * ( 8*S*T**(-1)*T1**(-3)*NS*MS2**3 - 8*S*
     +    T**(-1)*T1**(-3)*MS2**2*MT2 + 12*S*T**(-1)*T1**(-2)*NS*MS2**2
     +     - 12*S*T**(-1)*T1**(-2)*MS2*MT2 + 4*S*T**(-1)*T1**(-1)*NS*
     +    MS2 - 4*S*T**(-1)*T1**(-1)*MT2 - 8*S*T1**(-3)*NS*MS2**2 + 8*S
     +    *T1**(-3)*MS2*MT2 - 8*S*T1**(-2)*NS*MS2 + 8*S*T1**(-2)*MT2 + 
     +    8*T**(-1)*T1**(-1)*NS*MS2**2 - 8*T**(-1)*T1**(-1)*MS2*MT2 + 4
     +    *T**(-1)*NS*MS2 - 4*T**(-1)*MT2 - 8*T1**(-1)*NS*MS2 + 8*
     +    T1**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + N**2*CF * ( 4 - 8*S*T**(-1)*T1**(-3)*MS2**3 - 
     +    12*S*T**(-1)*T1**(-2)*MS2**2 - 4*S*T**(-1)*T1**(-1)*MS2 + 8*S
     +    *T1**(-3)*MS2**2 + 12*S*T1**(-2)*MS2 + 4*S*T1**(-1) - 8*
     +    T**(-1)*T1**(-1)*MS2**2 - 4*T**(-1)*MS2 + 8*T1**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0A(2)*N*CF * (  - 8*S*T**(-1)*T1**(-3)*
     +    MS2**2*MT2 - 12*S*T**(-1)*T1**(-2)*MS2*MT2 - 4*S*T**(-1)*
     +    T1**(-1)*MT2 + 8*S*T1**(-3)*MS2*MT2 + 8*S*T1**(-2)*MT2 - 8*
     +    T**(-1)*T1**(-1)*MS2*MT2 - 4*T**(-1)*MT2 + 8*T1**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + SK1B0C(1)*N*CF**2 * (  - 8 - 32*S*T1**(-3)*
     +    MS2**2 - 40*S*T1**(-2)*MS2 - 8*S*T1**(-1) - 32*T1**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N*CF * (  - 4 - 8*S*T**(-1)*
     +    T1**(-3)*NS*MS2**3 + 8*S*T**(-1)*T1**(-3)*MS2**3 - 12*S*
     +    T**(-1)*T1**(-2)*NS*MS2**2 + 12*S*T**(-1)*T1**(-2)*MS2**2 - 4
     +    *S*T**(-1)*T1**(-1)*NS*MS2 + 4*S*T**(-1)*T1**(-1)*MS2 + 8*S*
     +    T1**(-3)*NS*MS2**2 - 8*S*T1**(-3)*MS2**2 + 12*S*T1**(-2)*NS*
     +    MS2 - 12*S*T1**(-2)*MS2 + 4*S*T1**(-1)*NS - 4*S*T1**(-1) - 8*
     +    T**(-1)*T1**(-1)*NS*MS2**2 + 8*T**(-1)*T1**(-1)*MS2**2 - 4*
     +    T**(-1)*NS*MS2 + 4*T**(-1)*MS2 + 8*T1**(-1)*NS*MS2 - 8*
     +    T1**(-1)*MS2 + 4*NS )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N*CF**2 * ( 16 + 32*S*T1**(-3)*
     +    MS2**2 + 48*S*T1**(-2)*MS2 + 16*S*T1**(-1) + 32*T1**(-1)*MS2
     +     )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N**2*CF * (  - 12 + 8*S*T**(-1)*
     +    T1**(-3)*MS2**3 + 12*S*T**(-1)*T1**(-2)*MS2**2 + 4*S*T**(-1)*
     +    T1**(-1)*MS2 - 24*S*T1**(-3)*MS2**2 - 36*S*T1**(-2)*MS2 - 12*
     +    S*T1**(-1) + 8*T**(-1)*T1**(-1)*MS2**2 + 4*T**(-1)*MS2 - 24*
     +    T1**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(3,1)*N*CF * ( 4 + 8*S*T**(-1)*T1**(-3)*
     +    MS2**2*MT2 - 8*S*T**(-1)*T1**(-3)*MS2**3 + 12*S*T**(-1)*
     +    T1**(-2)*MS2*MT2 - 12*S*T**(-1)*T1**(-2)*MS2**2 - 4*S*T**(-1)
     +    *T1**(-1)*MS2 + 4*S*T**(-1)*T1**(-1)*MT2 + 8*S*T1**(-3)*
     +    MS2**2 + 12*S*T1**(-2)*MS2 + 4*S*T1**(-1) + 8*T**(-1)*
     +    T1**(-1)*MS2*MT2 - 8*T**(-1)*T1**(-1)*MS2**2 - 4*T**(-1)*MS2
     +     + 4*T**(-1)*MT2 + 8*T1**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0C(1)*N**2*CF * ( 16*S*T1**(-3)*MS2**2 + 16
     +    *S*T1**(-2)*MS2 + 16*T1**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0E(3)*N*CF * (  - 8*S*T1**(-3)*MS2*MT2 - 8*
     +    S*T1**(-2)*MT2 - 8*T1**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + SK1BP(1)*N*CF**2 * (  - 16*S*T1**(-2)*MS2**2 - 
     +    16*S*T1**(-1)*MS2 - 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0A(1)*N*CF**2 * ( 8*S + 8*S**2*T1**(-1) )
     +
      M2QBV2 = M2QBV2 + SK1C0A(1)*N**2*CF * (  - 2*S - 2*S**2*T1**(-1)
     +     )
     +
      M2QBV2 = M2QBV2 + SK1C0A(2)*N*CF**2 * ( 16*S*T1**(-1)*MS2 - 8*
     +    S**2*T1**(-1) )
     +
      M2QBV2 = M2QBV2 + SK1C0A(2)*N**2*CF * (  - 4*S*T1**(-1)*MS2 + 2*
     +    S**2*T1**(-1) )
     +
      M2QBV2 = M2QBV2 + SK1C0B(1)*N*CF**2 * (  - 16*S*T1**(-1)*MS2 + 16
     +    *S + 8*S**2*T1**(-1) - 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0B(1)*N**2*CF * ( 4*S*T1**(-1)*MS2 - 4*S - 
     +    2*S**2*T1**(-1) + 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0B(3)*N*CF**2 * ( 16*S*T1**(-1)*MS2 - 8*S
     +     - 8*S**2*T1**(-1) )
     +
      M2QBV2 = M2QBV2 + SK1C0B(3)*N**2*CF * (  - 4*S*T1**(-1)*MS2 + 2*S
     +     + 2*S**2*T1**(-1) )
     +
      M2QBV2 = M2QBV2 + SK1C0C(1,1)*N*CF**2 * ( 16*S*T1**(-2)*MS2**2 + 
     +    16*S*T1**(-1)*MS2 + 16*T1 + 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(1,1)*N**2*CF * (  - 16*S*T1**(-2)*MS2**2
     +     - 16*S*T1**(-1)*MS2 - 4*S - 8*T1 - 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(1,2)*N*CF**2 * ( 16*S*T1**(-1)*MS2 - 32*
     +    S - 16*S**2*T1**(-1) - 16*T1 + 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(1,2)*N**2*CF * (  - 8*S*T1**(-1)*MS2 + 
     +    16*S + 8*S**2*T1**(-1) + 8*T1 - 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,1)*N*CF**2 * ( 16*S*T1**(-1)*MS2 + 16*
     +    S + 16*T1 + 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,1)*N**2*CF * (  - 16*S*T1**(-1)*MS2 - 
     +    12*S - 12*T1 - 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,2)*N*CF**2 * (  - 16*S*T1**(-1)*MS2 + 
     +    32*S + 16*S**2*T1**(-1) + 16*T1 - 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,2)*N**2*CF * ( 8*S*T1**(-1)*MS2 - 16*S
     +     - 8*S**2*T1**(-1) - 8*T1 + 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(1,1)*N*CF**2 * ( 8*S*T1 + 16*S**2*
     +    T1**(-1)*MS2 + 8*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(1,1)*N**2*CF * (  - 2*S*T1 - 4*S**2*
     +    T1**(-1)*MS2 - 2*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(2,1)*N*CF**2 * (  - 32*S*T1**(-1)*MS2**2
     +     - 16*S*MS2 + 16*S**2*T1**(-1)*MS2 + 8*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(2,1)*N**2*CF * ( 8*S*T1**(-1)*MS2**2 + 4*
     +    S*MS2 - 4*S**2*T1**(-1)*MS2 - 2*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(3,1)*N*CF**2 * ( 32*S*T1 + 48*S*MS2 + 32*
     +    S**2*T1**(-1)*MS2 + 16*S**2 + 16*T1*MS2 + 16*T1**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(3,1)*N**2*CF * (  - 16*S*T1 - 24*S*MS2 - 
     +    16*S**2*T1**(-1)*MS2 - 8*S**2 - 8*T1*MS2 - 8*T1**2 )
     +
      M2QBV2 = M2QBV2 + SOF1(1)*N*CF**2 * ( 128*S*T1**(-2)*MS2**2 + 128
     +    *S*T1**(-1)*MS2 - 64*S - 64*S**2*T1**(-2)*MS2 - 64*S**2*
     +    T1**(-1) + 128*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(1)*N**2*CF * (  - 32*S*T1**(-2)*MS2**2 - 
     +    32*S*T1**(-1)*MS2 + 16*S + 16*S**2*T1**(-2)*MS2 + 16*S**2*
     +    T1**(-1) - 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(2)*N*CF**2 * ( 32*S*T1**(-2)*MS2**2 + 32*S
     +    *T1**(-1)*MS2 + 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(3)*N*CF**2 * ( 32*S*T1**(-2)*MS2**2 + 32*S
     +    *T1**(-1)*MS2 + 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(4)*N*CF**2 * (  - 64*S*T1**(-1)*MS2 - 128*
     +    S - 64*S**2*T1**(-2)*MS2 - 64*S**2*T1**(-1) - 64*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(4)*N**2*CF * ( 32*S*T1**(-1)*MS2 + 64*S + 
     +    32*S**2*T1**(-2)*MS2 + 32*S**2*T1**(-1) + 32*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(5)*N*CF**2 * (  - 32*S*T1**(-1)*MS2 - 32*S
     +     - 32*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(5)*N**2*CF * ( 16*S*T1**(-1)*MS2 + 16*S + 
     +    16*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(6)*N*CF**2 * (  - 32*S*T1**(-1)*MS2 - 32*S
     +     - 32*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(6)*N**2*CF * ( 16*S*T1**(-1)*MS2 + 16*S + 
     +    16*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(7)*N*CF**2 * (  - 64*S*T1**(-1)*MS2 - 128*
     +    S - 64*S**2*T1**(-2)*MS2 - 64*S**2*T1**(-1) - 64*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(7)*N**2*CF * ( 32*S*T1**(-1)*MS2 + 64*S + 
     +    32*S**2*T1**(-2)*MS2 + 32*S**2*T1**(-1) + 32*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(8)*N*CF**2 * (  - 64*S - 64*S**2*T1**(-2)*
     +    MS2 - 64*S**2*T1**(-1) )
     +
      M2QBV2 = M2QBV2 + SOF1(8)*N**2*CF * ( 16*S + 16*S**2*T1**(-2)*MS2
     +     + 16*S**2*T1**(-1) )


      M2QBV3 = 0.D0
      M2QBV3 = M2QBV3 + N*CF * (  - 8*S*T**(-1)*T1**(-3)*NS*MS2**3 + 8*
     +    S*T**(-1)*T1**(-3)*MS2**2*MT2 - 8*S*T**(-1)*T1**(-2)*NS*
     +    MS2**2 + 8*S*T**(-1)*T1**(-2)*MS2*MT2 + 8*S*T1**(-3)*NS*
     +    MS2**2 - 8*S*T1**(-3)*MS2*MT2 + 4*S*T1**(-2)*NS*MS2 - 4*S*
     +    T1**(-2)*MT2 )
     +
      M2QBV3 = M2QBV3 + N**2*CF * ( 8*S*T**(-1)*T1**(-3)*MS2**3 + 8*S*
     +    T**(-1)*T1**(-2)*MS2**2 - 8*S*T1**(-3)*MS2**2 - 8*S*T1**(-2)*
     +    MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0A(2)*N*CF * ( 8*S*T**(-1)*T1**(-3)*MS2**2*
     +    MT2 + 8*S*T**(-1)*T1**(-2)*MS2*MT2 - 8*S*T1**(-3)*MS2*MT2 - 4
     +    *S*T1**(-2)*MT2 )
     +
      M2QBV3 = M2QBV3 + SK1B0C(1)*N*CF**2 * ( 32*S*T1**(-3)*MS2**2 + 24
     +    *S*T1**(-2)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(1,1)*N*CF * ( 8*S*T**(-1)*T1**(-3)*NS*
     +    MS2**3 - 8*S*T**(-1)*T1**(-3)*MS2**3 + 8*S*T**(-1)*T1**(-2)*
     +    NS*MS2**2 - 8*S*T**(-1)*T1**(-2)*MS2**2 - 8*S*T1**(-3)*NS*
     +    MS2**2 + 8*S*T1**(-3)*MS2**2 - 8*S*T1**(-2)*NS*MS2 + 8*S*
     +    T1**(-2)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(1,1)*N*CF**2 * (  - 32*S*T1**(-3)*MS2**2
     +     - 32*S*T1**(-2)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(1,1)*N**2*CF * (  - 8*S*T**(-1)*T1**(-3)
     +    *MS2**3 - 8*S*T**(-1)*T1**(-2)*MS2**2 + 24*S*T1**(-3)*MS2**2
     +     + 24*S*T1**(-2)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(3,1)*N*CF * (  - 8*S*T**(-1)*T1**(-3)*
     +    MS2**2*MT2 + 8*S*T**(-1)*T1**(-3)*MS2**3 - 8*S*T**(-1)*
     +    T1**(-2)*MS2*MT2 + 8*S*T**(-1)*T1**(-2)*MS2**2 - 8*S*T1**(-3)
     +    *MS2**2 - 8*S*T1**(-2)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0C(1)*N**2*CF * (  - 16*S*T1**(-3)*MS2**2
     +     - 8*S*T1**(-2)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0E(3)*N*CF * ( 8*S*T1**(-3)*MS2*MT2 + 4*S*
     +    T1**(-2)*MT2 )
     +
      M2QBV3 = M2QBV3 + SK1BP(1)*N*CF**2 * ( 16*S*T1**(-2)*MS2**2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(1,1)*N*CF**2 * (  - 16*S*T1**(-2)*MS2**2
     +     - 16*S*T1**(-1)*MS2 - 16*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(1,1)*N**2*CF * ( 16*S*T1**(-2)*MS2**2 + 
     +    16*S*T1**(-1)*MS2 + 8*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(1,2)*N*CF**2 * ( 16*S*T1**(-1)*MS2 + 16*
     +    MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(1,2)*N**2*CF * (  - 8*S*T1**(-1)*MS2 - 8
     +    *MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,1)*N*CF**2 * (  - 16*S*T1**(-1)*MS2 - 
     +    16*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,1)*N**2*CF * ( 16*S*T1**(-1)*MS2 + 8*
     +    MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,2)*N*CF**2 * ( 16*S*T1**(-1)*MS2 + 16*
     +    MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,2)*N**2*CF * (  - 8*S*T1**(-1)*MS2 - 8
     +    *MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(1,1)*N*CF**2 * ( 16*S*T1**(-1)*U1*MS2 + 
     +    16*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(1,1)*N**2*CF * (  - 4*S*T1**(-1)*U1*MS2
     +     - 4*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(2,1)*N*CF**2 * ( 16*S*T1**(-1)*U1*MS2 + 
     +    32*S*T1**(-1)*MS2**2 + 16*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(2,1)*N**2*CF * (  - 4*S*T1**(-1)*U1*MS2
     +     - 8*S*T1**(-1)*MS2**2 - 4*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(3,1)*N*CF**2 * ( 32*S*T1**(-1)*U1*MS2 - 
     +    16*S*MS2 - 16*T1*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(3,1)*N**2*CF * (  - 16*S*T1**(-1)*U1*MS2
     +     + 8*S*MS2 + 8*T1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(1)*N*CF**2 * (  - 64*S*T1**(-2)*U1*MS2 - 
     +    128*S*T1**(-2)*MS2**2 - 64*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(1)*N**2*CF * ( 16*S*T1**(-2)*U1*MS2 + 32*S
     +    *T1**(-2)*MS2**2 + 16*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(2)*N*CF**2 * (  - 32*S*T1**(-2)*MS2**2 )
     +
      M2QBV3 = M2QBV3 + SOF1(3)*N*CF**2 * (  - 32*S*T1**(-2)*MS2**2 )
     +
      M2QBV3 = M2QBV3 + SOF1(4)*N*CF**2 * (  - 64*S*T1**(-2)*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(4)*N**2*CF * ( 32*S*T1**(-2)*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(5)*N*CF**2 * ( 32*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(5)*N**2*CF * (  - 16*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(6)*N*CF**2 * ( 32*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(6)*N**2*CF * (  - 16*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(7)*N*CF**2 * (  - 64*S*T1**(-2)*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(7)*N**2*CF * ( 32*S*T1**(-2)*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(8)*N*CF**2 * (  - 64*S*T1**(-2)*U1*MS2 - 
     +    64*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(8)*N**2*CF * ( 16*S*T1**(-2)*U1*MS2 + 16*S
     +    *T1**(-1)*MS2 )

      ELSE

         IF (IFL.EQ.1) THEN

      M2QBV1 = 0.D0
      M2QBV1 = M2QBV1 + N*CF**2*THREE**(-1) * ( 48*M2*S**(-3)*T1**2 + 
     +    48*M2*S**(-2)*T1 + 48*M2*S**(-2)*MS2 + 24*S**(-2)*T1**2 + 24*
     +    S**(-1)*T1 + 24*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + N**2*CF*THREE**(-1) * (  - 8*S**(-2)*T1**2 - 8*
     +    S**(-1)*T1 - 8*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0A(1)*N*CF**2*THREE**(-1) * ( 48*M2*S**(-3)
     +    *T1**2 + 48*M2*S**(-2)*T1 + 48*M2*S**(-2)*MS2 + 48*S**(-3)*
     +    T1**2*MS2 + 48*S**(-2)*T1*MS2 + 48*S**(-2)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1B0A(1)*N**2*CF*THREE**(-1) * (  - 32*M2*
     +    S**(-3)*T1**2 - 32*M2*S**(-2)*T1 - 32*M2*S**(-2)*MS2 - 32*
     +    S**(-3)*T1**2*MS2 - 32*S**(-2)*T1*MS2 - 32*S**(-2)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1B0A(2)*N*CF*THREE**(-1) * (  - 32*S**(-3)*
     +    T1**2*MT2 - 32*S**(-2)*T1*MT2 - 32*S**(-2)*MS2*MT2 )
     +
      M2QBV1 = M2QBV1 + SK1B0A(3)*N*CF**2 * ( 8*S**(-2)*T1**2 + 8*
     +    S**(-1)*T1 + 8*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(1)*N*CF*THREE**(-1) * ( 16*S**(-2)*T1**2
     +    *NS - 16*S**(-2)*T1**2 + 16*S**(-1)*T1*NS - 16*S**(-1)*T1 + 
     +    16*S**(-1)*NS*MS2 - 16*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(1)*N*CF**2*THREE**(-1) * ( 96*M2*S**(-2)
     +    *T1**2*S1**(-1) + 96*M2*S**(-1)*T1*S1**(-1) + 24*M2*S**(-1)
     +     + 24*M2*S1**(-1) + 72*S**(-2)*T1**2 + 72*S**(-1)*T1 + 48*
     +    S**(-1)*T1**2*S1**(-1) + 72*S**(-1)*MS2 + 48*T1*S1**(-1) + 48
     +    *S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(1)*N**2*CF*THREE**(-1) * (  - 48*M2*
     +    S**(-2)*T1**2*S1**(-1) - 48*M2*S**(-1)*T1*S1**(-1) - 12*M2*
     +    S**(-1) - 12*M2*S1**(-1) - 64*S**(-2)*T1**2 - 64*S**(-1)*T1
     +     - 48*S**(-1)*T1**2*S1**(-1) - 64*S**(-1)*MS2 - 48*T1*
     +    S1**(-1) - 48*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(2)*N*CF*THREE**(-1) * (  - 32*S**(-3)*
     +    T1**2*NS*MS2 - 32*S**(-2)*T1*NS*MS2 + 8*S**(-2)*T1**2*NS - 32
     +    *S**(-2)*NS*MS2**2 + 8*S**(-1)*T1*NS + 8*S**(-1)*NS*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(2)*N*CF**2*THREE**(-1) * (  - 48*M2*
     +    S**(-3)*T1**2 - 48*M2*S**(-2)*T1 - 48*M2*S**(-2)*MS2 + 24*
     +    S**(-2)*T1**2 + 24*S**(-1)*T1 - 48*S**(-1)*T1**2*S1**(-1) + 
     +    24*S**(-1)*MS2 - 48*T1*S1**(-1) - 48*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(2)*N**2*CF*THREE**(-1) * ( 24*M2*S**(-3)
     +    *T1**2 + 24*M2*S**(-2)*T1 + 24*M2*S**(-2)*MS2 - 12*S**(-2)*
     +    T1**2 - 12*S**(-1)*T1 + 24*S**(-1)*T1**2*S1**(-1) - 12*
     +    S**(-1)*MS2 + 24*T1*S1**(-1) + 24*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(3)*N**2*CF*THREE**(-1) * ( 8*M2*S**(-3)*
     +    T1**2 + 8*M2*S**(-2)*T1 + 48*M2*S**(-2)*T1**2*S1**(-1) + 8*M2
     +    *S**(-2)*MS2 + 48*M2*S**(-1)*T1*S1**(-1) + 12*M2*S**(-1) + 12
     +    *M2*S1**(-1) + 32*S**(-3)*T1**2*MS2 + 32*S**(-2)*T1*MS2 + 4*
     +    S**(-2)*T1**2 + 32*S**(-2)*MS2**2 + 4*S**(-1)*T1 + 24*S**(-1)
     +    *T1**2*S1**(-1) + 4*S**(-1)*MS2 + 24*T1*S1**(-1) + 24*
     +    S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0B(4)*N*CF*THREE**(-1) * ( 32*S**(-3)*T1**2
     +    *MT2 + 32*S**(-2)*T1*MT2 + 16*S**(-2)*T1**2 + 32*S**(-2)*MS2*
     +    MT2 + 16*S**(-1)*T1 + 16*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0C(1)*N*CF**2 * (  - 16*S**(-2)*T1**2 - 16*
     +    S**(-1)*T1 + 16*S**(-1)*T1**2*S1**(-1) - 16*S**(-1)*MS2 + 16*
     +    T1*S1**(-1) + 16*S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1B0C(2)*N*CF**2 * (  - 32*M2*S**(-2)*T1**2*
     +    S1**(-1) - 32*M2*S**(-1)*T1*S1**(-1) - 8*M2*S**(-1) - 8*M2*
     +    S1**(-1) - 16*S**(-1)*T1**2*S1**(-1) - 16*T1*S1**(-1) - 16*
     +    S1**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1BP(1)*N*CF**2 * (  - 32*S**(-2)*T1**2*MS2 - 
     +    32*S**(-1)*T1*MS2 - 32*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1BP(2)*N*CF**2 * (  - 16*M2*S**(-2)*T1**2 - 
     +    16*M2*S**(-1)*T1 - 16*M2*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1BP(3)*N*CF**2 * ( 8*M2*S**(-2)*T1**2 + 8*M2*
     +    S**(-1)*T1 + 8*M2*S**(-1)*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(1)*N*CF**2 * ( 16*S**(-1)*T1**2 + 16*S
     +     + 48*T1 + 16*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(1)*N**2*CF * (  - 16*S**(-1)*T1**2 - 8*S
     +     - 28*T1 - 12*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(5)*N*CF**2 * (  - 16*M2*S**(-2)*T1**2 - 
     +    16*M2*S**(-1)*T1 - 16*M2*S**(-1)*MS2 - 16*M2**2*S**(-3)*T1**2
     +     - 16*M2**2*S**(-2)*T1 - 16*M2**2*S**(-2)*MS2 - 16*S**(-2)*
     +    T1**2*MS2 - 16*S**(-1)*T1*MS2 - 16*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(5)*N**2*CF * ( 8*M2*S**(-2)*T1**2 + 8*M2
     +    *S**(-1)*T1 + 8*M2*S**(-1)*MS2 + 8*M2**2*S**(-3)*T1**2 + 8*
     +    M2**2*S**(-2)*T1 + 8*M2**2*S**(-2)*MS2 + 8*S**(-2)*T1**2*MS2
     +     + 8*S**(-1)*T1*MS2 + 8*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(6)*N*CF**2 * (  - 8*S - 16*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0A(6)*N**2*CF * ( 8*M2*S**(-2)*T1**2 + 8*M2
     +    *S**(-1)*T1 + 8*M2*S**(-1)*MS2 + 2*M2 + 8*M2**2*S**(-3)*T1**2
     +     + 8*M2**2*S**(-2)*T1 + 8*M2**2*S**(-2)*MS2 + 8*S**(-2)*T1**2
     +    *MS2 + 8*S**(-1)*T1*MS2 + 8*S**(-1)*MS2**2 + 4*S + 6*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(1)*N*CF**2 * (  - 32*S**(-2)*T1**2*MS2
     +     - 32*S**(-1)*T1*MS2 + 16*S**(-1)*T1**2 - 32*S**(-1)*MS2**2
     +     + 16*T1 + 16*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(1)*N**2*CF * ( 16*S**(-2)*T1**2*MS2 + 16
     +    *S**(-1)*T1*MS2 - 8*S**(-1)*T1**2 + 16*S**(-1)*MS2**2 - 8*T1
     +     - 8*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(2)*N*CF**2 * (  - 32*M2*S**(-1)*T1 - 16*
     +    M2 - 8*S - 16*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(2)*N**2*CF * (  - 8*M2*S**(-2)*T1**2 + 4
     +    *M2*S**(-1)*T1 - 8*M2*S**(-1)*T1**2*S1**(-1) - 8*M2*S**(-1)*
     +    MS2 - 8*M2*T1*S1**(-1) - 8*M2*S1**(-1)*MS2 + 6*M2 - 16*M2**2*
     +    S**(-2)*T1**2*S1**(-1) - 16*M2**2*S**(-1)*T1*S1**(-1) - 4*
     +    M2**2*S**(-1) - 4*M2**2*S1**(-1) + 4*S + 6*T1 - 4*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(3)*N*CF**2 * (  - 64*S**(-1)*T1*MS2 + 16
     +    *S + 32*T1 - 32*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(3)*N**2*CF * ( 24*S**(-1)*T1*MS2 - 12*
     +    S**(-1)*T1**2 - 8*S - 16*T1*S1**(-1)*MS2 - 20*T1 - 4*T1**2*
     +    S1**(-1) - 16*S1**(-1)*MS2**2 + 12*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(4)*N*CF**2 * ( 32*M2*S**(-1)*T1**2*
     +    S1**(-1) + 32*M2*T1*S1**(-1) + 32*M2*S1**(-1)*MS2 + 32*M2**2*
     +    S**(-2)*T1**2*S1**(-1) + 32*M2**2*S**(-1)*T1*S1**(-1) + 8*
     +    M2**2*S**(-1) + 8*M2**2*S1**(-1) + 8*S**(-1)*T1**2 + 32*T1*
     +    S1**(-1)*MS2 + 8*T1**2*S1**(-1) + 32*S1**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0B(4)*N**2*CF * (  - 16*M2*S**(-1)*T1**2*
     +    S1**(-1) - 16*M2*T1*S1**(-1) - 16*M2*S1**(-1)*MS2 - 16*M2**2*
     +    S**(-2)*T1**2*S1**(-1) - 16*M2**2*S**(-1)*T1*S1**(-1) - 4*
     +    M2**2*S**(-1) - 4*M2**2*S1**(-1) - 4*S**(-1)*T1**2 - 16*T1*
     +    S1**(-1)*MS2 - 4*T1**2*S1**(-1) - 16*S1**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,1)*N*CF**2 * ( 32*S**(-1)*T1*MS2 + 32*
     +    S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,1)*N**2*CF * (  - 8*S**(-1)*T1*MS2 - 8
     +    *S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,2)*N*CF**2 * ( 32*S**(-1)*T1*MS2 - 32*
     +    S**(-1)*T1**2 - 32*S - 64*T1 + 32*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(3,2)*N**2*CF * (  - 16*S**(-1)*T1*MS2 + 
     +    16*S**(-1)*T1**2 + 16*S + 32*T1 - 16*MS2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(6,1)*N*CF**2 * ( 16*M2*S**(-1)*T1 - 16*
     +    S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(6,1)*N**2*CF * (  - 4*M2*S**(-1)*T1 + 4*
     +    S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(6,2)*N*CF**2 * ( 16*M2*S**(-1)*T1 + 16*
     +    M2 + 16*S**(-1)*T1**2 + 16*S + 32*T1 )
     +
      M2QBV1 = M2QBV1 + SK1C0C(6,2)*N**2*CF * (  - 8*M2*S**(-1)*T1 - 8*
     +    M2 - 8*S**(-1)*T1**2 - 8*S - 16*T1 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,1)*N*CF**2 * ( 32*S**(-1)*T1**3 + 16*T1
     +    *MS2 + 16*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,1)*N**2*CF * (  - 8*S**(-1)*T1**3 - 4*
     +    T1*MS2 - 4*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,2)*N*CF**2 * ( 32*S**(-1)*T1**3 + 64*S*
     +    T1 + 16*S*MS2 + 16*S**2 + 16*T1*MS2 + 80*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(1,2)*N**2*CF * (  - 16*S**(-1)*T1**3 - 32
     +    *S*T1 - 8*S*MS2 - 8*S**2 - 8*T1*MS2 - 40*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(9,1)*N*CF**2 * ( 8*M2*T1 + 16*M2**2*
     +    S**(-1)*T1 + 16*T1*MS2 + 8*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(9,1)*N**2*CF * (  - 2*M2*T1 - 4*M2**2*
     +    S**(-1)*T1 - 4*T1*MS2 - 2*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(9,2)*N*CF**2 * ( 8*M2*S + 8*M2*T1 + 16*
     +    M2**2*S**(-1)*T1 + 16*M2**2 - 16*S*T1 + 16*S*MS2 - 8*S**2 + 
     +    16*T1*MS2 - 8*T1**2 )
     +
      M2QBV1 = M2QBV1 + SK1D0(9,2)*N**2*CF * (  - 4*M2*S - 4*M2*T1 - 8*
     +    M2**2*S**(-1)*T1 - 8*M2**2 + 8*S*T1 - 8*S*MS2 + 4*S**2 - 8*T1
     +    *MS2 + 4*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(1)*N*CF**2 * ( 128*S**(-2)*T1**2*MS2 + 128
     +    *S**(-1)*T1*MS2 - 64*S**(-1)*T1**2 + 128*S**(-1)*MS2**2 - 64*
     +    T1 - 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(1)*N**2*CF * (  - 64*S**(-2)*T1**2*MS2 - 
     +    64*S**(-1)*T1*MS2 + 32*S**(-1)*T1**2 - 64*S**(-1)*MS2**2 + 32
     +    *T1 + 32*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(2)*N*CF**2 * ( 64*S**(-2)*T1**2*MS2 + 64*
     +    S**(-1)*T1*MS2 + 64*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(3)*N*CF**2 * ( 64*S**(-2)*T1**2*MS2 + 64*
     +    S**(-1)*T1*MS2 + 64*S**(-1)*MS2**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(4)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 256*S**(-1)*T1**2 - 128*T1 - 128*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(4)*N**2*CF * ( 64*S**(-2)*T1**3 + 64*
     +    S**(-1)*T1*MS2 + 128*S**(-1)*T1**2 + 64*T1 + 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(5)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 128*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(5)*N**2*CF * ( 32*S**(-2)*T1**3 + 32*
     +    S**(-1)*T1*MS2 + 32*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(6)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 128*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(6)*N**2*CF * ( 32*S**(-2)*T1**3 + 32*
     +    S**(-1)*T1*MS2 + 32*S**(-1)*T1**2 )
     +
      M2QBV1 = M2QBV1 + SOF1(7)*N*CF**2 * (  - 128*S**(-2)*T1**3 - 128*
     +    S**(-1)*T1*MS2 - 256*S**(-1)*T1**2 - 128*T1 - 128*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(7)*N**2*CF * ( 64*S**(-2)*T1**3 + 64*
     +    S**(-1)*T1*MS2 + 128*S**(-1)*T1**2 + 64*T1 + 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(8)*N*CF**2 * (  - 64*S**(-1)*T1**2 - 64*T1
     +     - 64*MS2 )
     +
      M2QBV1 = M2QBV1 + SOF1(8)*N**2*CF * ( 32*S**(-1)*T1**2 + 32*T1 + 
     +    32*MS2 )


      M2QBV4 = 0.D0
      M2QBV4 = M2QBV4 + N*CF*THREE**(-1) * (  - 8 + 72*M2*TG**(-2)*
     +    T**(-1)*MS2**2 - 48*M2*TG**(-2)*MS2 + 48*M2*TG**(-1)*S**(-1)*
     +    T**(-1)*MS2**2 - 48*M2*TG**(-1)*S**(-1)*MS2 + 72*M2*TG**(-1)*
     +    T**(-1)*MS2 - 32*M2*TG**(-1) + 12*M2*S**(-1)*T**(-1)*T1 + 60*
     +    M2*S**(-1)*T**(-1)*MS2 - 32*M2*S**(-1) + 12*M2*T**(-1) + 24*
     +    M2**2*TG**(-2)*S**(-1)*T**(-1)*MS2**2 - 24*M2**2*TG**(-2)*
     +    S**(-1)*MS2 + 72*M2**2*TG**(-2)*T**(-1)*MS2 - 24*M2**2*
     +    TG**(-2) + 108*M2**2*TG**(-1)*S**(-1)*T**(-1)*MS2 - 56*M2**2*
     +    TG**(-1)*S**(-1) + 36*M2**2*TG**(-1)*T**(-1) + 36*M2**2*
     +    S**(-1)*T**(-1) + 48*M2**3*TG**(-2)*S**(-1)*T**(-1)*MS2 - 24*
     +    M2**3*TG**(-2)*S**(-1) + 24*M2**3*TG**(-2)*T**(-1) + 60*M2**3
     +    *TG**(-1)*S**(-1)*T**(-1) + 24*M2**4*TG**(-2)*S**(-1)*T**(-1)
     +     + 24*TG**(-2)*T**(-1)*MS2**3 - 24*TG**(-2)*MS2**2 + 36*
     +    TG**(-1)*T**(-1)*MS2**2 - 32*TG**(-1)*MS2 + 12*S**(-1)*
     +    T**(-1)*T1*MS2 + 24*S**(-1)*T**(-1)*MS2**2 - 8*S**(-1)*T1 - 
     +    24*S**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + N*CF*THREE**(-1) * ( 12*T**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + CF*THREE**(-1) * (  - 48*M2*TG**(-2)*T**(-1)*NS
     +    *MS2**2 + 48*M2*TG**(-2)*T**(-1)*MS2*MT2 + 24*M2*TG**(-2)*NS*
     +    MS2 - 24*M2*TG**(-2)*MT2 - 48*M2*TG**(-1)*S**(-1)*T**(-1)*NS*
     +    MS2**2 + 48*M2*TG**(-1)*S**(-1)*T**(-1)*MS2*MT2 + 48*M2*
     +    TG**(-1)*S**(-1)*NS*MS2 - 48*M2*TG**(-1)*S**(-1)*MT2 - 36*M2*
     +    TG**(-1)*T**(-1)*NS*MS2 + 36*M2*TG**(-1)*T**(-1)*MT2 - 36*M2*
     +    S**(-1)*T**(-1)*NS*MS2 + 36*M2*S**(-1)*T**(-1)*MT2 - 24*M2**2
     +    *TG**(-2)*S**(-1)*T**(-1)*NS*MS2**2 + 24*M2**2*TG**(-2)*
     +    S**(-1)*T**(-1)*MS2*MT2 + 24*M2**2*TG**(-2)*S**(-1)*NS*MS2 - 
     +    24*M2**2*TG**(-2)*S**(-1)*MT2 - 24*M2**2*TG**(-2)*T**(-1)*NS*
     +    MS2 + 24*M2**2*TG**(-2)*T**(-1)*MT2 - 60*M2**2*TG**(-1)*
     +    S**(-1)*T**(-1)*NS*MS2 + 60*M2**2*TG**(-1)*S**(-1)*T**(-1)*
     +    MT2 - 24*M2**3*TG**(-2)*S**(-1)*T**(-1)*NS*MS2 + 24*M2**3*
     +    TG**(-2)*S**(-1)*T**(-1)*MT2 - 24*TG**(-2)*T**(-1)*NS*MS2**3
     +     + 24*TG**(-2)*T**(-1)*MS2**2*MT2 + 24*TG**(-2)*NS*MS2**2 - 
     +    24*TG**(-2)*MS2*MT2 )
     +
      M2QBV4 = M2QBV4 + CF*THREE**(-1) * (  - 36*TG**(-1)*T**(-1)*NS*
     +    MS2**2 + 36*TG**(-1)*T**(-1)*MS2*MT2 + 24*TG**(-1)*NS*MS2 - 
     +    24*TG**(-1)*MT2 - 12*S**(-1)*T**(-1)*T1*NS*MS2 + 12*S**(-1)*
     +    T**(-1)*T1*MT2 - 24*S**(-1)*T**(-1)*NS*MS2**2 + 24*S**(-1)*
     +    T**(-1)*MS2*MT2 + 24*S**(-1)*NS*MS2 - 24*S**(-1)*MT2 - 12*
     +    T**(-1)*NS*MS2 + 12*T**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + CF**2*THREE**(-1) * (  - 12 - 24*M2*TG**(-1)*
     +    S**(-1)*MS2 - 12*M2*TG**(-1) - 24*M2*S**(-2)*T1 - 36*M2*
     +    S**(-1) - 36*M2**2*TG**(-1)*S**(-1) - 24*M2**2*S**(-2) - 24*
     +    M2**3*TG**(-1)*S**(-2) - 12*TG**(-1)*MS2 - 12*S**(-1)*T1 )
     +
      M2QBV4 = M2QBV4 + SK1B0A(1)*N*CF*THREE**(-1) * ( 72*M2*TG**(-2)*
     +    T**(-1)*MS2**2 - 48*M2*TG**(-2)*MS2 + 48*M2*TG**(-1)*S**(-1)*
     +    T**(-1)*MS2**2 - 16*M2*TG**(-1)*S**(-1)*MS2 + 72*M2*TG**(-1)*
     +    T**(-1)*MS2 - 24*M2*TG**(-1) + 16*M2*S**(-2)*T1 + 16*M2*
     +    S**(-2)*MS2 + 12*M2*S**(-1)*T**(-1)*T1 + 60*M2*S**(-1)*
     +    T**(-1)*MS2 - 8*M2*S**(-1) + 12*M2*T**(-1) + 24*M2**2*
     +    TG**(-2)*S**(-1)*T**(-1)*MS2**2 - 24*M2**2*TG**(-2)*S**(-1)*
     +    MS2 + 72*M2**2*TG**(-2)*T**(-1)*MS2 - 24*M2**2*TG**(-2) + 16*
     +    M2**2*TG**(-1)*S**(-2)*MS2 + 108*M2**2*TG**(-1)*S**(-1)*
     +    T**(-1)*MS2 - 32*M2**2*TG**(-1)*S**(-1) + 36*M2**2*TG**(-1)*
     +    T**(-1) + 16*M2**2*S**(-2) + 36*M2**2*S**(-1)*T**(-1) + 48*
     +    M2**3*TG**(-2)*S**(-1)*T**(-1)*MS2 - 24*M2**3*TG**(-2)*
     +    S**(-1) + 24*M2**3*TG**(-2)*T**(-1) + 16*M2**3*TG**(-1)*
     +    S**(-2) + 60*M2**3*TG**(-1)*S**(-1)*T**(-1) + 24*M2**4*
     +    TG**(-2)*S**(-1)*T**(-1) + 24*TG**(-2)*T**(-1)*MS2**3 - 24*
     +    TG**(-2)*MS2**2 )
     +
      M2QBV4 = M2QBV4 + SK1B0A(1)*N*CF*THREE**(-1) * ( 16*TG**(-1)*
     +    S**(-1)*MS2**2 + 36*TG**(-1)*T**(-1)*MS2**2 - 24*TG**(-1)*MS2
     +     + 16*S**(-2)*T1*MS2 + 12*S**(-1)*T**(-1)*T1*MS2 + 24*S**(-1)
     +    *T**(-1)*MS2**2 - 8*S**(-1)*MS2 + 12*T**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0A(1)*CF**2*THREE**(-1) * (  - 48*M2*
     +    TG**(-1)*S**(-1)*MS2 - 24*M2*S**(-2)*T1 - 24*M2*S**(-2)*MS2
     +     - 24*M2*S**(-1) - 24*M2**2*TG**(-1)*S**(-2)*MS2 - 24*M2**2*
     +    TG**(-1)*S**(-1) - 24*M2**2*S**(-2) - 24*M2**3*TG**(-1)*
     +    S**(-2) - 24*TG**(-1)*S**(-1)*MS2**2 - 24*S**(-2)*T1*MS2 - 24
     +    *S**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0A(2)*CF*THREE**(-1) * ( 48*M2*TG**(-2)*
     +    T**(-1)*MS2*MT2 - 24*M2*TG**(-2)*MT2 + 48*M2*TG**(-1)*S**(-1)
     +    *T**(-1)*MS2*MT2 - 32*M2*TG**(-1)*S**(-1)*MT2 + 36*M2*
     +    TG**(-1)*T**(-1)*MT2 + 16*M2*S**(-2)*MT2 + 36*M2*S**(-1)*
     +    T**(-1)*MT2 + 24*M2**2*TG**(-2)*S**(-1)*T**(-1)*MS2*MT2 - 24*
     +    M2**2*TG**(-2)*S**(-1)*MT2 + 24*M2**2*TG**(-2)*T**(-1)*MT2 + 
     +    16*M2**2*TG**(-1)*S**(-2)*MT2 + 60*M2**2*TG**(-1)*S**(-1)*
     +    T**(-1)*MT2 + 24*M2**3*TG**(-2)*S**(-1)*T**(-1)*MT2 + 24*
     +    TG**(-2)*T**(-1)*MS2**2*MT2 - 24*TG**(-2)*MS2*MT2 + 16*
     +    TG**(-1)*S**(-1)*MS2*MT2 + 36*TG**(-1)*T**(-1)*MS2*MT2 - 24*
     +    TG**(-1)*MT2 + 16*S**(-2)*T1*MT2 + 12*S**(-1)*T**(-1)*T1*MT2
     +     + 24*S**(-1)*T**(-1)*MS2*MT2 - 8*S**(-1)*MT2 + 12*T**(-1)*
     +    MT2 )
     +
      M2QBV4 = M2QBV4 + SK1B0A(3)*CF**2 * (  - 8 - 8*M2*TG**(-1) - 8*M2
     +    *S**(-1) - 8*M2**2*TG**(-1)*S**(-1) - 8*TG**(-1)*MS2 - 8*
     +    S**(-1)*T1 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(1)*N*CF*THREE**(-1) * ( 8 + 120*M2*
     +    TG**(-1)*S1**(-1)*MS2 + 8*M2*TG**(-1) + 24*M2*S**(-1)*T1*
     +    S1**(-1) + 32*M2*S**(-1) + 48*M2*S1**(-1) + 32*M2**2*TG**(-1)
     +    *S**(-1) + 48*M2**2*TG**(-1)*S1**(-1) + 24*M2**2*S**(-1)*
     +    S1**(-1) + 24*M2**3*TG**(-1)*S**(-1)*S1**(-1) + 96*TG**(-1)*
     +    S1**(-1)*MS2**2 + 8*TG**(-1)*MS2 + 32*S**(-1)*T1 + 24*T1*
     +    S1**(-1) + 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(1)*CF*THREE**(-1) * ( 8 - 8*M2*TG**(-1)*
     +    NS + 8*M2*TG**(-1) - 8*M2*S**(-1)*NS + 8*M2*S**(-1) - 8*M2**2
     +    *TG**(-1)*S**(-1)*NS + 8*M2**2*TG**(-1)*S**(-1) - 8*TG**(-1)*
     +    NS*MS2 + 8*TG**(-1)*MS2 - 8*S**(-1)*T1*NS + 8*S**(-1)*T1 - 8*
     +    NS )
     +
      M2QBV4 = M2QBV4 + SK1B0B(1)*CF**2*THREE**(-1) * (  - 12 - 144*M2*
     +    TG**(-1)*S1**(-1)*MS2 - 12*M2*TG**(-1) - 48*M2*S**(-1)*T1*
     +    S1**(-1) - 36*M2*S**(-1) - 72*M2*S1**(-1) - 36*M2**2*TG**(-1)
     +    *S**(-1) - 72*M2**2*TG**(-1)*S1**(-1) - 48*M2**2*S**(-1)*
     +    S1**(-1) - 48*M2**3*TG**(-1)*S**(-1)*S1**(-1) - 96*TG**(-1)*
     +    S1**(-1)*MS2**2 - 12*TG**(-1)*MS2 - 36*S**(-1)*T1 - 24*T1*
     +    S1**(-1) - 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(2)*N*CF*THREE**(-1) * ( 18 - 12*M2*
     +    TG**(-1)*S**(-1)*MS2 - 48*M2*TG**(-1)*S1**(-1)*MS2 + 18*M2*
     +    TG**(-1) - 12*M2*S**(-2)*T1 - 6*M2*S**(-1) - 12*M2*S1**(-1)
     +     - 6*M2**2*TG**(-1)*S**(-1) - 12*M2**2*TG**(-1)*S1**(-1) - 12
     +    *M2**2*S**(-2) - 12*M2**3*TG**(-1)*S**(-2) - 48*TG**(-1)*
     +    S1**(-1)*MS2**2 + 18*TG**(-1)*MS2 + 6*S**(-1)*T1 - 12*T1*
     +    S1**(-1) - 48*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(2)*CF*THREE**(-1) * ( 16*M2*TG**(-1)*
     +    S**(-1)*NS*MS2 - 4*M2*TG**(-1)*NS + 16*M2*S**(-2)*NS*MS2 - 4*
     +    M2*S**(-1)*NS + 16*M2**2*TG**(-1)*S**(-2)*NS*MS2 - 4*M2**2*
     +    TG**(-1)*S**(-1)*NS + 16*TG**(-1)*S**(-1)*NS*MS2**2 - 4*
     +    TG**(-1)*NS*MS2 + 16*S**(-2)*T1*NS*MS2 - 4*S**(-1)*T1*NS + 16
     +    *S**(-1)*NS*MS2 - 4*NS )
     +
      M2QBV4 = M2QBV4 + SK1B0B(2)*CF**2*THREE**(-1) * (  - 36 + 24*M2*
     +    TG**(-1)*S**(-1)*MS2 + 96*M2*TG**(-1)*S1**(-1)*MS2 - 36*M2*
     +    TG**(-1) + 24*M2*S**(-2)*T1 + 12*M2*S**(-1) + 24*M2*S1**(-1)
     +     + 12*M2**2*TG**(-1)*S**(-1) + 24*M2**2*TG**(-1)*S1**(-1) + 
     +    24*M2**2*S**(-2) + 24*M2**3*TG**(-1)*S**(-2) + 96*TG**(-1)*
     +    S1**(-1)*MS2**2 - 36*TG**(-1)*MS2 - 12*S**(-1)*T1 + 24*T1*
     +    S1**(-1) + 96*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(3)*N*CF*THREE**(-1) * ( 10 - 20*M2*
     +    TG**(-1)*S**(-1)*MS2 - 72*M2*TG**(-1)*S1**(-1)*MS2 + 10*M2*
     +    TG**(-1) - 4*M2*S**(-2)*T1 - 16*M2*S**(-2)*MS2 - 24*M2*
     +    S**(-1)*T1*S1**(-1) - 6*M2*S**(-1) - 36*M2*S1**(-1) - 16*
     +    M2**2*TG**(-1)*S**(-2)*MS2 - 6*M2**2*TG**(-1)*S**(-1) - 36*
     +    M2**2*TG**(-1)*S1**(-1) - 4*M2**2*S**(-2) - 24*M2**2*S**(-1)*
     +    S1**(-1) - 4*M2**3*TG**(-1)*S**(-2) - 24*M2**3*TG**(-1)*
     +    S**(-1)*S1**(-1) - 16*TG**(-1)*S**(-1)*MS2**2 - 48*TG**(-1)*
     +    S1**(-1)*MS2**2 + 10*TG**(-1)*MS2 - 16*S**(-2)*T1*MS2 - 2*
     +    S**(-1)*T1 - 16*S**(-1)*MS2 - 12*T1*S1**(-1) - 48*S1**(-1)*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0B(4)*CF*THREE**(-1) * (  - 8 - 16*M2*
     +    TG**(-1)*S**(-1)*MT2 - 8*M2*TG**(-1) - 16*M2*S**(-2)*MT2 - 8*
     +    M2*S**(-1) - 16*M2**2*TG**(-1)*S**(-2)*MT2 - 8*M2**2*TG**(-1)
     +    *S**(-1) - 16*TG**(-1)*S**(-1)*MS2*MT2 - 8*TG**(-1)*MS2 - 16*
     +    S**(-2)*T1*MT2 - 8*S**(-1)*T1 - 16*S**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + SK1B0C(1)*CF**2 * ( 32 + 16*M2*TG**(-1)*S**(-1)
     +    *MS2 - 32*M2*TG**(-1)*S1**(-1)*MS2 + 32*M2*TG**(-1) + 24*M2*
     +    S**(-1) - 8*M2*S1**(-1) + 24*M2**2*TG**(-1)*S**(-1) - 8*M2**2
     +    *TG**(-1)*S1**(-1) + 16*TG**(-1)*T1**(-1)*MS2**2 - 32*
     +    TG**(-1)*S1**(-1)*MS2**2 + 48*TG**(-1)*MS2 + 24*S**(-1)*T1 + 
     +    16*S**(-1)*MS2 - 8*T1*S1**(-1) - 32*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0C(2)*CF**2 * (  - 16 + 16*M2*TG**(-1)*
     +    S**(-1)*MS2 + 48*M2*TG**(-1)*S1**(-1)*MS2 + 16*M2*S**(-1)*T1*
     +    S1**(-1) + 8*M2*S**(-1) + 24*M2*S1**(-1) + 8*M2**2*TG**(-1)*
     +    S**(-1) + 24*M2**2*TG**(-1)*S1**(-1) + 16*M2**2*S**(-1)*
     +    S1**(-1) + 16*M2**3*TG**(-1)*S**(-1)*S1**(-1) + 16*TG**(-1)*
     +    T1**(-1)*MS2**2 + 32*TG**(-1)*S1**(-1)*MS2**2 + 16*TG**(-1)*
     +    MS2 - 8*S**(-1)*T1 + 16*S**(-1)*MS2 - 16*T1**(-1)*MS2 + 8*T1*
     +    S1**(-1) + 32*S1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*N*CF * ( 8 + 16*M2*TG**(-1)*S**(-1)
     +    *MS2 + 16*M2*TG**(-1) + 16*M2*S**(-1) + 16*M2**2*TG**(-1)*
     +    S**(-1) + 16*TG**(-1)*T1**(-1)*MS2**2 + 32*TG**(-1)*MS2 + 8*
     +    S**(-1)*T1 + 16*S**(-1)*MS2 - 8*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*CF * ( 4 + 16*M2*TG**(-2)*T**(-1)*
     +    NS*MS2**2 - 16*M2*TG**(-2)*T**(-1)*MS2**2 - 16*M2*TG**(-2)*NS
     +    *MS2 + 16*M2*TG**(-2)*MS2 + 16*M2*TG**(-1)*S**(-1)*T**(-1)*NS
     +    *MS2**2 - 16*M2*TG**(-1)*S**(-1)*T**(-1)*MS2**2 - 16*M2*
     +    TG**(-1)*S**(-1)*NS*MS2 + 16*M2*TG**(-1)*S**(-1)*MS2 + 12*M2*
     +    TG**(-1)*T**(-1)*NS*MS2 - 12*M2*TG**(-1)*T**(-1)*MS2 - 12*M2*
     +    TG**(-1)*NS + 12*M2*TG**(-1) + 12*M2*S**(-1)*T**(-1)*NS*MS2
     +     - 12*M2*S**(-1)*T**(-1)*MS2 - 12*M2*S**(-1)*NS + 12*M2*
     +    S**(-1) + 8*M2**2*TG**(-2)*S**(-1)*T**(-1)*NS*MS2**2 - 8*
     +    M2**2*TG**(-2)*S**(-1)*T**(-1)*MS2**2 - 8*M2**2*TG**(-2)*
     +    S**(-1)*NS*MS2 + 8*M2**2*TG**(-2)*S**(-1)*MS2 + 8*M2**2*
     +    TG**(-2)*T**(-1)*NS*MS2 - 8*M2**2*TG**(-2)*T**(-1)*MS2 - 8*
     +    M2**2*TG**(-2)*NS + 8*M2**2*TG**(-2) + 20*M2**2*TG**(-1)*
     +    S**(-1)*T**(-1)*NS*MS2 - 20*M2**2*TG**(-1)*S**(-1)*T**(-1)*
     +    MS2 - 20*M2**2*TG**(-1)*S**(-1)*NS + 20*M2**2*TG**(-1)*
     +    S**(-1) )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*CF * ( 8*M2**3*TG**(-2)*S**(-1)*
     +    T**(-1)*NS*MS2 - 8*M2**3*TG**(-2)*S**(-1)*T**(-1)*MS2 - 8*
     +    M2**3*TG**(-2)*S**(-1)*NS + 8*M2**3*TG**(-2)*S**(-1) + 8*
     +    TG**(-2)*T**(-1)*NS*MS2**3 - 8*TG**(-2)*T**(-1)*MS2**3 - 8*
     +    TG**(-2)*NS*MS2**2 + 8*TG**(-2)*MS2**2 + 12*TG**(-1)*T**(-1)*
     +    NS*MS2**2 - 12*TG**(-1)*T**(-1)*MS2**2 - 12*TG**(-1)*NS*MS2
     +     + 12*TG**(-1)*MS2 + 4*S**(-1)*T**(-1)*T1*NS*MS2 - 4*S**(-1)*
     +    T**(-1)*T1*MS2 + 8*S**(-1)*T**(-1)*NS*MS2**2 - 8*S**(-1)*
     +    T**(-1)*MS2**2 - 4*S**(-1)*T1*NS + 4*S**(-1)*T1 - 8*S**(-1)*
     +    NS*MS2 + 8*S**(-1)*MS2 + 4*T**(-1)*NS*MS2 - 4*T**(-1)*MS2 - 4
     +    *NS )
     +
      M2QBV4 = M2QBV4 + SK1B0D(1,1)*CF**2 * (  - 16 - 32*M2*TG**(-1)*
     +    S**(-1)*MS2 - 32*M2*TG**(-1) - 32*M2*S**(-1) - 32*M2**2*
     +    TG**(-1)*S**(-1) - 32*TG**(-1)*T1**(-1)*MS2**2 - 64*TG**(-1)*
     +    MS2 - 16*S**(-1)*T1 - 32*S**(-1)*MS2 + 16*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(2,1)*N*CF * ( 4 - 24*M2*TG**(-2)*T**(-1)
     +    *MS2**2 + 48*M2*TG**(-2)*MS2 - 16*M2*TG**(-1)*S**(-1)*T**(-1)
     +    *MS2**2 + 32*M2*TG**(-1)*S**(-1)*MS2 - 24*M2*TG**(-1)*T**(-1)
     +    *MS2 + 20*M2*TG**(-1) - 4*M2*S**(-1)*T**(-1)*T1 - 20*M2*
     +    S**(-1)*T**(-1)*MS2 + 20*M2*S**(-1) - 4*M2*T**(-1) - 8*M2**2*
     +    TG**(-2)*S**(-1)*T**(-1)*MS2**2 + 24*M2**2*TG**(-2)*S**(-1)*
     +    MS2 - 24*M2**2*TG**(-2)*T**(-1)*MS2 + 24*M2**2*TG**(-2) - 36*
     +    M2**2*TG**(-1)*S**(-1)*T**(-1)*MS2 + 44*M2**2*TG**(-1)*
     +    S**(-1) - 12*M2**2*TG**(-1)*T**(-1) - 12*M2**2*S**(-1)*
     +    T**(-1) - 16*M2**3*TG**(-2)*S**(-1)*T**(-1)*MS2 + 24*M2**3*
     +    TG**(-2)*S**(-1) - 8*M2**3*TG**(-2)*T**(-1) - 20*M2**3*
     +    TG**(-1)*S**(-1)*T**(-1) - 8*M2**4*TG**(-2)*S**(-1)*T**(-1)
     +     - 8*TG**(-2)*T**(-1)*MS2**3 + 24*TG**(-2)*MS2**2 - 12*
     +    TG**(-1)*T**(-1)*MS2**2 - 16*TG**(-1)*T1**(-1)*MS2**2 + 4*
     +    TG**(-1)*MS2 - 4*S**(-1)*T**(-1)*T1*MS2 - 8*S**(-1)*T**(-1)*
     +    MS2**2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(2,1)*N*CF * ( 4*S**(-1)*T1 + 8*S**(-1)*
     +    MS2 - 4*T**(-1)*MS2 + 8*T1**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(3,1)*CF * (  - 4 - 16*M2*TG**(-2)*
     +    T**(-1)*MS2*MT2 + 16*M2*TG**(-2)*T**(-1)*MS2**2 - 16*M2*
     +    TG**(-2)*MS2 - 16*M2*TG**(-1)*S**(-1)*T**(-1)*MS2*MT2 + 16*M2
     +    *TG**(-1)*S**(-1)*T**(-1)*MS2**2 - 16*M2*TG**(-1)*S**(-1)*MS2
     +     + 12*M2*TG**(-1)*T**(-1)*MS2 - 12*M2*TG**(-1)*T**(-1)*MT2 - 
     +    12*M2*TG**(-1) + 12*M2*S**(-1)*T**(-1)*MS2 - 12*M2*S**(-1)*
     +    T**(-1)*MT2 - 12*M2*S**(-1) - 8*M2**2*TG**(-2)*S**(-1)*
     +    T**(-1)*MS2*MT2 + 8*M2**2*TG**(-2)*S**(-1)*T**(-1)*MS2**2 - 8
     +    *M2**2*TG**(-2)*S**(-1)*MS2 + 8*M2**2*TG**(-2)*T**(-1)*MS2 - 
     +    8*M2**2*TG**(-2)*T**(-1)*MT2 - 8*M2**2*TG**(-2) + 20*M2**2*
     +    TG**(-1)*S**(-1)*T**(-1)*MS2 - 20*M2**2*TG**(-1)*S**(-1)*
     +    T**(-1)*MT2 - 20*M2**2*TG**(-1)*S**(-1) + 8*M2**3*TG**(-2)*
     +    S**(-1)*T**(-1)*MS2 - 8*M2**3*TG**(-2)*S**(-1)*T**(-1)*MT2 - 
     +    8*M2**3*TG**(-2)*S**(-1) - 8*TG**(-2)*T**(-1)*MS2**2*MT2 + 8*
     +    TG**(-2)*T**(-1)*MS2**3 - 8*TG**(-2)*MS2**2 - 12*TG**(-1)*
     +    T**(-1)*MS2*MT2 )
     +
      M2QBV4 = M2QBV4 + SK1B0D(3,1)*CF * ( 12*TG**(-1)*T**(-1)*MS2**2
     +     - 12*TG**(-1)*MS2 + 4*S**(-1)*T**(-1)*T1*MS2 - 4*S**(-1)*
     +    T**(-1)*T1*MT2 - 8*S**(-1)*T**(-1)*MS2*MT2 + 8*S**(-1)*
     +    T**(-1)*MS2**2 - 4*S**(-1)*T1 - 8*S**(-1)*MS2 + 4*T**(-1)*MS2
     +     - 4*T**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + SK1B0E(1)*N*CF * (  - 32*M2*TG**(-2)*MS2 - 32*
     +    M2*TG**(-1)*S**(-1)*MS2 - 16*M2*TG**(-1) - 16*M2*S**(-1) - 16
     +    *M2**2*TG**(-2)*S**(-1)*MS2 - 16*M2**2*TG**(-2) - 32*M2**2*
     +    TG**(-1)*S**(-1) - 16*M2**3*TG**(-2)*S**(-1) - 16*TG**(-2)*
     +    MS2**2 - 16*TG**(-1)*MS2 - 16*S**(-1)*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1B0E(2)*CF * ( 8*M2*TG**(-2)*NS*MS2 - 8*M2*
     +    TG**(-2)*MS2 + 8*M2*TG**(-1)*NS - 8*M2*TG**(-1) + 8*M2*
     +    S**(-1)*NS - 8*M2*S**(-1) + 8*M2**2*TG**(-2)*NS - 8*M2**2*
     +    TG**(-2) + 16*M2**2*TG**(-1)*S**(-1)*NS - 16*M2**2*TG**(-1)*
     +    S**(-1) + 8*M2**3*TG**(-2)*S**(-1)*NS - 8*M2**3*TG**(-2)*
     +    S**(-1) )
     +
      M2QBV4 = M2QBV4 + SK1B0E(3)*CF * ( 8*M2*TG**(-2)*MS2 + 8*M2*
     +    TG**(-2)*MT2 + 16*M2*TG**(-1)*S**(-1)*MT2 + 8*M2*TG**(-1) + 8
     +    *M2*S**(-1) + 8*M2**2*TG**(-2)*S**(-1)*MT2 + 8*M2**2*TG**(-2)
     +     + 16*M2**2*TG**(-1)*S**(-1) + 8*M2**3*TG**(-2)*S**(-1) + 8*
     +    TG**(-2)*MS2*MT2 + 8*TG**(-1)*MT2 + 8*S**(-1)*MT2 )
     +
      M2QBV4 = M2QBV4 + SK1BP(1)*CF**2 * ( 32*M2*TG**(-1)*MS2 + 32*M2*
     +    S**(-1)*MS2 + 32*M2**2*TG**(-1)*S**(-1)*MS2 + 32*TG**(-1)*
     +    MS2**2 + 32*S**(-1)*T1*MS2 + 32*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1BP(2)*CF**2 * ( 16*M2*TG**(-1)*MS2 + 16*M2*
     +    S**(-1)*T1 + 16*M2 + 16*M2**2*TG**(-1) + 16*M2**2*S**(-1) + 
     +    16*M2**3*TG**(-1)*S**(-1) )
     +
      M2QBV4 = M2QBV4 + SK1BP(3)*CF**2 * (  - 8*M2*TG**(-1)*MS2 - 8*M2*
     +    S**(-1)*T1 - 8*M2 - 8*M2**2*TG**(-1) - 8*M2**2*S**(-1) - 8*
     +    M2**3*TG**(-1)*S**(-1) )
     +
      M2QBV4 = M2QBV4 + SK1C0A(1)*N*CF * ( 14*M2*TG**(-1)*S + 10*M2 + 8
     +    *M2**2*TG**(-1) + 6*TG**(-1)*S*MS2 + 4*TG**(-1)*S**2 + 16*S
     +     + 10*T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(1)*CF**2 * (  - 16*M2*TG**(-1)*S - 12*M2
     +     - 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2 - 4*TG**(-1)*S**2 - 20
     +    *S - 12*T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(5)*N*CF * (  - 8*M2*TG**(-1)*MS2 - 4*M2*
     +    S**(-1)*T1 - 4*M2*S**(-1)*MS2 - 4*M2 - 8*M2**2*TG**(-1)*
     +    S**(-1)*MS2 - 4*M2**2*TG**(-1) - 4*M2**2*S**(-2)*T1 - 8*M2**2
     +    *S**(-1) - 8*M2**3*TG**(-1)*S**(-1) - 4*M2**3*S**(-2) - 4*
     +    M2**4*TG**(-1)*S**(-2) - 4*TG**(-1)*MS2**2 - 4*S**(-1)*T1*MS2
     +     - 2*S )
     +
      M2QBV4 = M2QBV4 + SK1C0A(5)*CF**2 * ( 16*M2*TG**(-1)*MS2 + 8*M2*
     +    S**(-1)*T1 + 8*M2*S**(-1)*MS2 + 8*M2 + 16*M2**2*TG**(-1)*
     +    S**(-1)*MS2 + 8*M2**2*TG**(-1) + 8*M2**2*S**(-2)*T1 + 16*
     +    M2**2*S**(-1) + 16*M2**3*TG**(-1)*S**(-1) + 8*M2**3*S**(-2)
     +     + 8*M2**4*TG**(-1)*S**(-2) + 8*TG**(-1)*MS2**2 + 8*S**(-1)*
     +    T1*MS2 + 4*S )
     +
      M2QBV4 = M2QBV4 + SK1C0A(6)*N*CF * (  - 2*M2*TG**(-1)*S - 8*M2*
     +    TG**(-1)*MS2 - 4*M2*S**(-1)*T1 - 4*M2*S**(-1)*MS2 - 4*M2 - 8*
     +    M2**2*TG**(-1)*S**(-1)*MS2 - 4*M2**2*TG**(-1) - 4*M2**2*
     +    S**(-2)*T1 - 8*M2**2*S**(-1) - 8*M2**3*TG**(-1)*S**(-1) - 4*
     +    M2**3*S**(-2) - 4*M2**4*TG**(-1)*S**(-2) + 6*TG**(-1)*S*MS2
     +     - 4*TG**(-1)*S**2 - 4*TG**(-1)*MS2**2 - 4*S**(-1)*T1*MS2 - 4
     +    *S - 4*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0A(6)*CF**2 * (  - 8*TG**(-1)*S*MS2 + 4*
     +    TG**(-1)*S**2 + 4*S )
     +
      M2QBV4 = M2QBV4 + SK1C0B(1)*N*CF * ( 4*M2*TG**(-1)*S - 8*M2*
     +    TG**(-1)*MS2 - 8*M2*S**(-1)*MS2 + 4*M2 - 8*M2**2*TG**(-1)*
     +    S**(-1)*MS2 + 4*M2**2*TG**(-1) + 4*TG**(-1)*S*MS2 - 8*
     +    TG**(-1)*MS2**2 - 16*S**(-1)*T1*MS2 + 6*S + 8*T1 - 12*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(1)*CF**2 * (  - 8*M2*TG**(-1)*S + 16*M2*
     +    TG**(-1)*MS2 + 16*M2*S**(-1)*MS2 - 8*M2 + 16*M2**2*TG**(-1)*
     +    S**(-1)*MS2 - 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2 + 16*
     +    TG**(-1)*MS2**2 + 32*S**(-1)*T1*MS2 - 12*S - 16*T1 + 24*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(2)*N*CF * (  - 6*M2*TG**(-1)*S + 16*M2*
     +    TG**(-1)*S1**(-1)*MS2**2 + 4*M2*TG**(-1)*MS2 + 4*M2*S**(-1)*
     +    T1 + 4*M2*T1*S1**(-1) + 16*M2*S1**(-1)*MS2 - 4*M2 + 24*M2**2*
     +    TG**(-1)*S1**(-1)*MS2 - 4*M2**2*TG**(-1) + 8*M2**2*S**(-1)*T1
     +    *S1**(-1) + 4*M2**2*S**(-1) + 12*M2**2*S1**(-1) + 4*M2**3*
     +    TG**(-1)*S**(-1) + 12*M2**3*TG**(-1)*S1**(-1) + 8*M2**3*
     +    S**(-1)*S1**(-1) + 8*M2**4*TG**(-1)*S**(-1)*S1**(-1) + 10*
     +    TG**(-1)*S*MS2 - 4*TG**(-1)*S**2 - 4*S + 4*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(2)*CF**2 * ( 8*M2*TG**(-1)*S + 8*M2 + 8*
     +    M2**2*TG**(-1) - 8*TG**(-1)*S*MS2 + 4*TG**(-1)*S**2 + 4*S )
     +
      M2QBV4 = M2QBV4 + SK1C0B(3)*N*CF * ( 10*M2*TG**(-1)*S + 32*M2*
     +    TG**(-1)*S1**(-1)*MS2**2 - 20*M2*TG**(-1)*MS2 + 8*M2*S1**(-1)
     +    *MS2 + 4*M2 + 8*M2**2*TG**(-1)*S1**(-1)*MS2 + 4*M2**2*
     +    TG**(-1) - 6*TG**(-1)*S*MS2 + 4*TG**(-1)*S**2 + 32*TG**(-1)*
     +    S1**(-1)*MS2**3 - 8*TG**(-1)*MS2**2 + 10*S + 8*T1*S1**(-1)*
     +    MS2 + 4*T1 + 32*S1**(-1)*MS2**2 - 20*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(3)*CF**2 * (  - 8*M2*TG**(-1)*S + 16*M2*
     +    TG**(-1)*MS2 + 8*TG**(-1)*S*MS2 - 4*TG**(-1)*S**2 - 8*S + 16*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(4)*N*CF * ( 64*M2*TG**(-1)*S1**(-1)*
     +    MS2**2 - 16*M2*TG**(-1)*MS2 - 4*M2*S**(-1)*T1 + 8*M2*T1*
     +    S1**(-1) + 40*M2*S1**(-1)*MS2 - 10*M2 + 48*M2**2*TG**(-1)*
     +    S1**(-1)*MS2 - 8*M2**2*TG**(-1) + 8*M2**2*S**(-1)*T1*S1**(-1)
     +     + 16*M2**2*S1**(-1) + 16*M2**3*TG**(-1)*S1**(-1) + 8*M2**3*
     +    S**(-1)*S1**(-1) + 8*M2**4*TG**(-1)*S**(-1)*S1**(-1) + 32*
     +    TG**(-1)*S1**(-1)*MS2**3 - 8*TG**(-1)*MS2**2 - 2*S + 8*T1*
     +    S1**(-1)*MS2 - 2*T1 + 32*S1**(-1)*MS2**2 - 4*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0B(4)*CF**2 * (  - 128*M2*TG**(-1)*S1**(-1)
     +    *MS2**2 + 32*M2*TG**(-1)*MS2 + 8*M2*S**(-1)*T1 - 16*M2*T1*
     +    S1**(-1) - 80*M2*S1**(-1)*MS2 + 20*M2 - 96*M2**2*TG**(-1)*
     +    S1**(-1)*MS2 + 16*M2**2*TG**(-1) - 16*M2**2*S**(-1)*T1*
     +    S1**(-1) - 32*M2**2*S1**(-1) - 32*M2**3*TG**(-1)*S1**(-1) - 
     +    16*M2**3*S**(-1)*S1**(-1) - 16*M2**4*TG**(-1)*S**(-1)*
     +    S1**(-1) - 64*TG**(-1)*S1**(-1)*MS2**3 + 16*TG**(-1)*MS2**2
     +     + 4*S - 16*T1*S1**(-1)*MS2 + 4*T1 - 64*S1**(-1)*MS2**2 + 8*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,1)*N*CF * ( 12*M2*TG**(-1)*MS2 + 8*M2*
     +    S**(-1)*T1 + 12*M2 + 12*M2**2*TG**(-1) + 8*M2**2*S**(-1) + 8*
     +    M2**3*TG**(-1)*S**(-1) + 8*S**(-1)*T1**2 + 12*T1 + 12*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,1)*CF**2 * (  - 24*M2*TG**(-1)*MS2 - 
     +    16*M2*S**(-1)*T1 - 24*M2 - 24*M2**2*TG**(-1) - 16*M2**2*
     +    S**(-1) - 16*M2**3*TG**(-1)*S**(-1) - 16*S**(-1)*T1**2 - 24*
     +    T1 - 24*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,2)*N*CF * (  - 16*M2*TG**(-1)*S + 8*M2
     +    *TG**(-1)*MS2 - 8*M2 - 8*M2**2*TG**(-1) + 8*TG**(-1)*S*MS2 - 
     +    8*TG**(-1)*S**2 - 8*S**(-1)*T1*MS2 + 8*S**(-1)*T1**2 - 8*S + 
     +    8*T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(3,2)*CF**2 * ( 16*M2*TG**(-1)*S - 8*M2*
     +    TG**(-1)*MS2 + 8*M2 + 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2 + 8
     +    *TG**(-1)*S**2 + 8*S**(-1)*T1*MS2 - 8*S**(-1)*T1**2 + 8*S - 8
     +    *T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(4,1)*N*CF * ( 8*M2*TG**(-1)*MS2 + 12*M2*
     +    S**(-1)*T1 + 8*M2 + 8*M2**2*TG**(-1) + 8*M2**2*S**(-1) + 8*
     +    M2**3*TG**(-1)*S**(-1) + 8*S**(-1)*T1*MS2 + 4*S**(-1)*T1**2
     +     + 4*T1 + 8*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(4,1)*CF**2 * (  - 8*M2*S**(-1)*T1 - 8*
     +    S**(-1)*T1*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(5,1)*N*CF * ( 24*M2*TG**(-1)*MS2 + 8*M2*
     +    S**(-1)*T1 + 16*M2*S**(-1)*MS2 + 8*M2 + 16*M2**2*TG**(-1)*
     +    S**(-1)*MS2 + 8*M2**2*TG**(-1) + 8*M2**2*S**(-1) + 8*M2**3*
     +    TG**(-1)*S**(-1) + 16*TG**(-1)*MS2**2 + 8*S**(-1)*T1*MS2 + 8*
     +    S**(-1)*T1**2 - 8*T1**(-1)*MS2**2 + 4*T1 + 8*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(5,1)*CF**2 * (  - 8*S**(-1)*T1*MS2 - 8*
     +    S**(-1)*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(6,1)*N*CF * ( 36*M2*TG**(-1)*MS2 + 8*M2*
     +    S**(-1)*T1 + 16*M2*S**(-1)*MS2 - 8*M2*T1**(-1)*MS2 + 12*M2 + 
     +    16*M2**2*TG**(-1)*S**(-1)*MS2 + 20*M2**2*TG**(-1) + 16*M2**2*
     +    S**(-1) + 16*M2**3*TG**(-1)*S**(-1) + 16*TG**(-1)*MS2**2 + 8*
     +    S**(-1)*T1*MS2 - 8*T1**(-1)*MS2**2 + 12*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(6,1)*CF**2 * (  - 72*M2*TG**(-1)*MS2 - 
     +    16*M2*S**(-1)*T1 - 32*M2*S**(-1)*MS2 + 16*M2*T1**(-1)*MS2 - 
     +    24*M2 - 32*M2**2*TG**(-1)*S**(-1)*MS2 - 40*M2**2*TG**(-1) - 
     +    32*M2**2*S**(-1) - 32*M2**3*TG**(-1)*S**(-1) - 32*TG**(-1)*
     +    MS2**2 - 16*S**(-1)*T1*MS2 + 16*T1**(-1)*MS2**2 - 24*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(6,2)*N*CF * ( 16*M2*TG**(-1)*S - 8*M2*
     +    TG**(-1)*MS2 + 8*M2 + 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2 + 8
     +    *TG**(-1)*S**2 + 8*S**(-1)*T1*MS2 - 8*S**(-1)*T1**2 + 8*S - 8
     +    *T1 )
     +
      M2QBV4 = M2QBV4 + SK1C0C(6,2)*CF**2 * (  - 16*M2*TG**(-1)*S + 8*
     +    M2*TG**(-1)*MS2 - 8*M2 - 8*M2**2*TG**(-1) + 8*TG**(-1)*S*MS2
     +     - 8*TG**(-1)*S**2 - 8*S**(-1)*T1*MS2 + 8*S**(-1)*T1**2 - 8*S
     +     + 8*T1 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,1)*N*CF * ( 2*M2*TG**(-1)*S*MS2 + 2*M2*
     +    S + 4*M2*T1 + 2*M2**2*TG**(-1)*S + 4*M2**2 + 4*M2**3*TG**(-1)
     +     + 2*S*T1 + 2*S*MS2 + 4*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,1)*CF**2 * (  - 4*M2*TG**(-1)*S*MS2 - 4
     +    *M2*S - 8*M2*T1 - 4*M2**2*TG**(-1)*S - 8*M2**2 - 8*M2**3*
     +    TG**(-1) - 4*S*T1 - 4*S*MS2 - 8*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,2)*N*CF * ( 4*M2*TG**(-1)*S*MS2 + 16*M2
     +    *TG**(-1)*S**2 + 20*M2*S + 8*M2*T1 + 20*M2**2*TG**(-1)*S + 8*
     +    M2**2 + 8*M2**3*TG**(-1) + 4*TG**(-1)*S**2*MS2 + 4*TG**(-1)*
     +    S**3 + 20*S*T1 + 4*S*MS2 + 16*S**2 + 8*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(1,2)*CF**2 * (  - 4*M2*TG**(-1)*S*MS2 - 
     +    16*M2*TG**(-1)*S**2 - 20*M2*S - 8*M2*T1 - 20*M2**2*TG**(-1)*S
     +     - 8*M2**2 - 8*M2**3*TG**(-1) - 4*TG**(-1)*S**2*MS2 - 4*
     +    TG**(-1)*S**3 - 20*S*T1 - 4*S*MS2 - 16*S**2 - 8*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(4,1)*N*CF * ( 2*M2*S + 2*M2**2 + 2*S*T1
     +     + 4*S*MS2 + 2*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(4,1)*CF**2 * (  - 4*M2*S - 4*M2**2 - 4*S*
     +    T1 - 8*S*MS2 - 4*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(7,1)*N*CF * (  - 8*M2*S**(-1)*T1*MS2 + 2*
     +    M2*S + 4*M2*T1 - 4*M2*MS2 + 2*S*T1 + 4*S*MS2 - 4*T1*MS2 - 8*
     +    MS2**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(7,1)*CF**2 * ( 16*M2*S**(-1)*T1*MS2 - 4*
     +    M2*S - 8*M2*T1 + 8*M2*MS2 - 4*S*T1 - 8*S*MS2 + 8*T1*MS2 + 16*
     +    MS2**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(8,1)*N*CF * (  - 8*M2*S**(-1)*T1*MS2 + 8*
     +    M2*S**(-1)*T1**2 + 8*M2*S + 16*M2*T1 - 8*M2*MS2 + 8*S**(-1)*
     +    T1**2*MS2 + 8*S**(-1)*T1**3 + 8*S*T1 + 16*S*MS2 + 24*T1*MS2
     +     + 16*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(8,1)*CF**2 * ( 8*M2*S**(-1)*T1*MS2 - 8*M2
     +    *S**(-1)*T1**2 - 8*M2*S - 16*M2*T1 + 8*M2*MS2 - 8*S**(-1)*
     +    T1**2*MS2 - 8*S**(-1)*T1**3 - 8*S*T1 - 16*S*MS2 - 24*T1*MS2
     +     - 16*T1**2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(9,1)*N*CF * ( 2*M2*TG**(-1)*S*MS2 + 2*M2*
     +    S + 4*M2*T1 + 8*M2*MS2 + 2*M2**2*TG**(-1)*S + 8*M2**2*
     +    TG**(-1)*MS2 + 8*M2**2 + 8*M2**3*TG**(-1) + 2*S*MS2 + 4*T1*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(9,1)*CF**2 * (  - 4*M2*TG**(-1)*S*MS2 - 4
     +    *M2*S - 8*M2*T1 - 16*M2*MS2 - 4*M2**2*TG**(-1)*S - 16*M2**2*
     +    TG**(-1)*MS2 - 16*M2**2 - 16*M2**3*TG**(-1) - 4*S*MS2 - 8*T1*
     +    MS2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(9,2)*N*CF * ( 12*M2*TG**(-1)*S*MS2 + 12*
     +    M2*S + 8*M2*T1 + 12*M2**2*TG**(-1)*S + 8*M2**2 + 8*M2**3*
     +    TG**(-1) + 12*TG**(-1)*S**2*MS2 - 4*TG**(-1)*S**3 - 4*S*T1 + 
     +    20*S*MS2 - 8*S**2 + 8*T1*MS2 )
     +
      M2QBV4 = M2QBV4 + SK1D0(9,2)*CF**2 * (  - 12*M2*TG**(-1)*S*MS2 - 
     +    12*M2*S - 8*M2*T1 - 12*M2**2*TG**(-1)*S - 8*M2**2 - 8*M2**3*
     +    TG**(-1) - 12*TG**(-1)*S**2*MS2 + 4*TG**(-1)*S**3 + 4*S*T1 - 
     +    20*S*MS2 + 8*S**2 - 8*T1*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(1)*N*CF * (  - 32*M2*TG**(-1)*S + 64*M2*
     +    TG**(-1)*MS2 + 64*M2*S**(-1)*MS2 - 32*M2 + 64*M2**2*TG**(-1)*
     +    S**(-1)*MS2 - 32*M2**2*TG**(-1) - 32*TG**(-1)*S*MS2 + 64*
     +    TG**(-1)*MS2**2 + 64*S**(-1)*T1*MS2 - 32*S - 32*T1 + 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(1)*CF**2 * ( 64*M2*TG**(-1)*S - 128*M2*
     +    TG**(-1)*MS2 - 128*M2*S**(-1)*MS2 + 64*M2 - 128*M2**2*
     +    TG**(-1)*S**(-1)*MS2 + 64*M2**2*TG**(-1) + 64*TG**(-1)*S*MS2
     +     - 128*TG**(-1)*MS2**2 - 128*S**(-1)*T1*MS2 + 64*S + 64*T1 - 
     +    128*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(2)*CF**2 * (  - 64*M2*TG**(-1)*MS2 - 64*M2
     +    *S**(-1)*MS2 - 64*M2**2*TG**(-1)*S**(-1)*MS2 - 64*TG**(-1)*
     +    MS2**2 - 64*S**(-1)*T1*MS2 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(3)*CF**2 * (  - 64*M2*TG**(-1)*MS2 - 64*M2
     +    *S**(-1)*MS2 - 64*M2**2*TG**(-1)*S**(-1)*MS2 - 64*TG**(-1)*
     +    MS2**2 - 64*S**(-1)*T1*MS2 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(4)*N*CF * (  - 64*M2*TG**(-1)*S - 64*M2*
     +    TG**(-1)*MS2 - 64*M2*S**(-1)*T1 - 128*M2 - 128*M2**2*TG**(-1)
     +     - 64*M2**2*S**(-1) - 64*M2**3*TG**(-1)*S**(-1) - 64*TG**(-1)
     +    *S*MS2 - 64*S**(-1)*T1**2 - 64*S - 128*T1 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(4)*CF**2 * ( 64*M2*TG**(-1)*S + 64*M2*
     +    TG**(-1)*MS2 + 64*M2*S**(-1)*T1 + 128*M2 + 128*M2**2*TG**(-1)
     +     + 64*M2**2*S**(-1) + 64*M2**3*TG**(-1)*S**(-1) + 64*TG**(-1)
     +    *S*MS2 + 64*S**(-1)*T1**2 + 64*S + 128*T1 + 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(5)*N*CF * (  - 32*M2*TG**(-1)*MS2 - 32*M2*
     +    S**(-1)*T1 - 32*M2 - 32*M2**2*TG**(-1) - 32*M2**2*S**(-1) - 
     +    32*M2**3*TG**(-1)*S**(-1) - 32*S**(-1)*T1**2 - 32*T1 - 32*MS2
     +     )
     +
      M2QBV4 = M2QBV4 + SOF1(5)*CF**2 * ( 64*M2*TG**(-1)*MS2 + 64*M2*
     +    S**(-1)*T1 + 64*M2 + 64*M2**2*TG**(-1) + 64*M2**2*S**(-1) + 
     +    64*M2**3*TG**(-1)*S**(-1) + 64*S**(-1)*T1**2 + 64*T1 + 64*MS2
     +     )
     +
      M2QBV4 = M2QBV4 + SOF1(6)*N*CF * (  - 32*M2*TG**(-1)*MS2 - 32*M2*
     +    S**(-1)*T1 - 32*M2 - 32*M2**2*TG**(-1) - 32*M2**2*S**(-1) - 
     +    32*M2**3*TG**(-1)*S**(-1) - 32*S**(-1)*T1**2 - 32*T1 - 32*MS2
     +     )
     +
      M2QBV4 = M2QBV4 + SOF1(6)*CF**2 * ( 64*M2*TG**(-1)*MS2 + 64*M2*
     +    S**(-1)*T1 + 64*M2 + 64*M2**2*TG**(-1) + 64*M2**2*S**(-1) + 
     +    64*M2**3*TG**(-1)*S**(-1) + 64*S**(-1)*T1**2 + 64*T1 + 64*MS2
     +     )
     +
      M2QBV4 = M2QBV4 + SOF1(7)*N*CF * (  - 64*M2*TG**(-1)*S - 64*M2*
     +    TG**(-1)*MS2 - 64*M2*S**(-1)*T1 - 128*M2 - 128*M2**2*TG**(-1)
     +     - 64*M2**2*S**(-1) - 64*M2**3*TG**(-1)*S**(-1) - 64*TG**(-1)
     +    *S*MS2 - 64*S**(-1)*T1**2 - 64*S - 128*T1 - 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(7)*CF**2 * ( 64*M2*TG**(-1)*S + 64*M2*
     +    TG**(-1)*MS2 + 64*M2*S**(-1)*T1 + 128*M2 + 128*M2**2*TG**(-1)
     +     + 64*M2**2*S**(-1) + 64*M2**3*TG**(-1)*S**(-1) + 64*TG**(-1)
     +    *S*MS2 + 64*S**(-1)*T1**2 + 64*S + 128*T1 + 64*MS2 )
     +
      M2QBV4 = M2QBV4 + SOF1(8)*N*CF * (  - 32*M2*TG**(-1)*S - 32*M2 - 
     +    32*M2**2*TG**(-1) - 32*TG**(-1)*S*MS2 - 32*S - 32*T1 )
     +
      M2QBV4 = M2QBV4 + SOF1(8)*CF**2 * ( 64*M2*TG**(-1)*S + 64*M2 + 64
     +    *M2**2*TG**(-1) + 64*TG**(-1)*S*MS2 + 64*S + 64*T1 )

         END IF

      M2QBV2 = 0.D0
      M2QBV2 = M2QBV2 + N*CF * ( 16*M2*TG**(-3)*S*T**(-1)*NS*MS2**2 - 
     +    16*M2*TG**(-3)*S*T**(-1)*MS2*MT2 - 8*M2*TG**(-3)*S*NS*MS2 + 8
     +    *M2*TG**(-3)*S*MT2 + 12*M2*TG**(-2)*S*T**(-1)*NS*MS2 - 12*M2*
     +    TG**(-2)*S*T**(-1)*MT2 + 16*M2*TG**(-2)*T**(-1)*NS*MS2**2 - 
     +    16*M2*TG**(-2)*T**(-1)*MS2*MT2 - 16*M2*TG**(-2)*NS*MS2 + 16*
     +    M2*TG**(-2)*MT2 + 16*M2*TG**(-1)*T**(-1)*NS*MS2 - 16*M2*
     +    TG**(-1)*T**(-1)*MT2 + 8*M2**2*TG**(-3)*S*T**(-1)*NS*MS2 - 8*
     +    M2**2*TG**(-3)*S*T**(-1)*MT2 + 8*M2**2*TG**(-3)*T**(-1)*NS*
     +    MS2**2 - 8*M2**2*TG**(-3)*T**(-1)*MS2*MT2 - 8*M2**2*TG**(-3)*
     +    NS*MS2 + 8*M2**2*TG**(-3)*MT2 + 20*M2**2*TG**(-2)*T**(-1)*NS*
     +    MS2 - 20*M2**2*TG**(-2)*T**(-1)*MT2 + 8*M2**3*TG**(-3)*
     +    T**(-1)*NS*MS2 - 8*M2**3*TG**(-3)*T**(-1)*MT2 + 8*TG**(-3)*S*
     +    T**(-1)*NS*MS2**3 - 8*TG**(-3)*S*T**(-1)*MS2**2*MT2 - 8*
     +    TG**(-3)*S*NS*MS2**2 + 8*TG**(-3)*S*MS2*MT2 + 12*TG**(-2)*S*
     +    T**(-1)*NS*MS2**2 - 12*TG**(-2)*S*T**(-1)*MS2*MT2 - 8*
     +    TG**(-2)*S*NS*MS2 )
     +
      M2QBV2 = M2QBV2 + N*CF * ( 8*TG**(-2)*S*MT2 + 4*TG**(-1)*S*
     +    T**(-1)*NS*MS2 - 4*TG**(-1)*S*T**(-1)*MT2 + 8*TG**(-1)*
     +    T**(-1)*NS*MS2**2 - 8*TG**(-1)*T**(-1)*MS2*MT2 - 8*TG**(-1)*
     +    NS*MS2 + 8*TG**(-1)*MT2 + 4*T**(-1)*NS*MS2 - 4*T**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + N**2*CF * ( 4 - 24*M2*TG**(-3)*S*T**(-1)*MS2**2
     +     + 16*M2*TG**(-3)*S*MS2 - 24*M2*TG**(-2)*S*T**(-1)*MS2 + 12*
     +    M2*TG**(-2)*S - 16*M2*TG**(-2)*T**(-1)*MS2**2 + 16*M2*
     +    TG**(-2)*MS2 - 4*M2*TG**(-1)*S*T**(-1) - 24*M2*TG**(-1)*
     +    T**(-1)*MS2 + 16*M2*TG**(-1) - 4*M2*T**(-1) - 24*M2**2*
     +    TG**(-3)*S*T**(-1)*MS2 + 8*M2**2*TG**(-3)*S - 8*M2**2*
     +    TG**(-3)*T**(-1)*MS2**2 + 8*M2**2*TG**(-3)*MS2 - 12*M2**2*
     +    TG**(-2)*S*T**(-1) - 36*M2**2*TG**(-2)*T**(-1)*MS2 + 20*M2**2
     +    *TG**(-2) - 16*M2**2*TG**(-1)*T**(-1) - 8*M2**3*TG**(-3)*S*
     +    T**(-1) - 16*M2**3*TG**(-3)*T**(-1)*MS2 + 8*M2**3*TG**(-3) - 
     +    20*M2**3*TG**(-2)*T**(-1) - 8*M2**4*TG**(-3)*T**(-1) - 8*
     +    TG**(-3)*S*T**(-1)*MS2**3 + 8*TG**(-3)*S*MS2**2 - 12*TG**(-2)
     +    *S*T**(-1)*MS2**2 + 12*TG**(-2)*S*MS2 - 4*TG**(-1)*S*T**(-1)*
     +    MS2 + 4*TG**(-1)*S - 8*TG**(-1)*T**(-1)*MS2**2 + 8*TG**(-1)*
     +    MS2 - 4*T**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0A(1)*N**2*CF * (  - 24*M2*TG**(-3)*S*
     +    T**(-1)*MS2**2 + 16*M2*TG**(-3)*S*MS2 - 24*M2*TG**(-2)*S*
     +    T**(-1)*MS2 + 8*M2*TG**(-2)*S - 16*M2*TG**(-2)*T**(-1)*MS2**2
     +     + 16*M2*TG**(-2)*MS2 - 4*M2*TG**(-1)*S*T**(-1) - 24*M2*
     +    TG**(-1)*T**(-1)*MS2 + 8*M2*TG**(-1) - 4*M2*T**(-1) - 24*
     +    M2**2*TG**(-3)*S*T**(-1)*MS2 + 8*M2**2*TG**(-3)*S - 8*M2**2*
     +    TG**(-3)*T**(-1)*MS2**2 + 8*M2**2*TG**(-3)*MS2 - 12*M2**2*
     +    TG**(-2)*S*T**(-1) - 36*M2**2*TG**(-2)*T**(-1)*MS2 + 16*M2**2
     +    *TG**(-2) - 16*M2**2*TG**(-1)*T**(-1) - 8*M2**3*TG**(-3)*S*
     +    T**(-1) - 16*M2**3*TG**(-3)*T**(-1)*MS2 + 8*M2**3*TG**(-3) - 
     +    20*M2**3*TG**(-2)*T**(-1) - 8*M2**4*TG**(-3)*T**(-1) - 8*
     +    TG**(-3)*S*T**(-1)*MS2**3 + 8*TG**(-3)*S*MS2**2 - 12*TG**(-2)
     +    *S*T**(-1)*MS2**2 + 8*TG**(-2)*S*MS2 - 4*TG**(-1)*S*T**(-1)*
     +    MS2 - 8*TG**(-1)*T**(-1)*MS2**2 + 8*TG**(-1)*MS2 - 4*T**(-1)*
     +    MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0A(2)*N*CF * (  - 16*M2*TG**(-3)*S*T**(-1)*
     +    MS2*MT2 + 8*M2*TG**(-3)*S*MT2 - 12*M2*TG**(-2)*S*T**(-1)*MT2
     +     - 16*M2*TG**(-2)*T**(-1)*MS2*MT2 + 16*M2*TG**(-2)*MT2 - 16*
     +    M2*TG**(-1)*T**(-1)*MT2 - 8*M2**2*TG**(-3)*S*T**(-1)*MT2 - 8*
     +    M2**2*TG**(-3)*T**(-1)*MS2*MT2 + 8*M2**2*TG**(-3)*MT2 - 20*
     +    M2**2*TG**(-2)*T**(-1)*MT2 - 8*M2**3*TG**(-3)*T**(-1)*MT2 - 8
     +    *TG**(-3)*S*T**(-1)*MS2**2*MT2 + 8*TG**(-3)*S*MS2*MT2 - 12*
     +    TG**(-2)*S*T**(-1)*MS2*MT2 + 8*TG**(-2)*S*MT2 - 4*TG**(-1)*S*
     +    T**(-1)*MT2 - 8*TG**(-1)*T**(-1)*MS2*MT2 + 8*TG**(-1)*MT2 - 4
     +    *T**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + SK1B0A(3)*N*CF**2 * ( 4 + 4*M2*TG**(-2)*S + 8*
     +    M2*TG**(-1) + 4*M2**2*TG**(-2) + 4*TG**(-2)*S*MS2 + 4*
     +    TG**(-1)*S )
     +
      M2QBV2 = M2QBV2 + SK1B0C(1)*N*CF**2 * (  - 16 - 16*M2*TG**(-2)*S
     +     - 16*M2*TG**(-2)*MS2 - 32*M2*TG**(-1) - 16*M2**2*TG**(-2) - 
     +    16*TG**(-2)*S*T1**(-1)*MS2**2 - 32*TG**(-2)*S*MS2 - 16*
     +    TG**(-1)*S - 16*TG**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0C(2)*N*CF**2 * ( 8 - 8*M2*TG**(-2)*S - 16*
     +    M2*TG**(-2)*MS2 - 8*M2**2*TG**(-2) - 16*TG**(-2)*S*T1**(-1)*
     +    MS2**2 - 24*TG**(-2)*S*MS2 + 16*TG**(-1)*S*T1**(-1)*MS2 + 8*
     +    TG**(-1)*S - 16*TG**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N*CF * (  - 4 - 16*M2*TG**(-3)*S*
     +    T**(-1)*NS*MS2**2 + 16*M2*TG**(-3)*S*T**(-1)*MS2**2 + 16*M2*
     +    TG**(-3)*S*NS*MS2 - 16*M2*TG**(-3)*S*MS2 - 12*M2*TG**(-2)*S*
     +    T**(-1)*NS*MS2 + 12*M2*TG**(-2)*S*T**(-1)*MS2 + 12*M2*
     +    TG**(-2)*S*NS - 12*M2*TG**(-2)*S - 16*M2*TG**(-2)*T**(-1)*NS*
     +    MS2**2 + 16*M2*TG**(-2)*T**(-1)*MS2**2 + 16*M2*TG**(-2)*NS*
     +    MS2 - 16*M2*TG**(-2)*MS2 - 16*M2*TG**(-1)*T**(-1)*NS*MS2 + 16
     +    *M2*TG**(-1)*T**(-1)*MS2 + 16*M2*TG**(-1)*NS - 16*M2*TG**(-1)
     +     - 8*M2**2*TG**(-3)*S*T**(-1)*NS*MS2 + 8*M2**2*TG**(-3)*S*
     +    T**(-1)*MS2 + 8*M2**2*TG**(-3)*S*NS - 8*M2**2*TG**(-3)*S - 8*
     +    M2**2*TG**(-3)*T**(-1)*NS*MS2**2 + 8*M2**2*TG**(-3)*T**(-1)*
     +    MS2**2 + 8*M2**2*TG**(-3)*NS*MS2 - 8*M2**2*TG**(-3)*MS2 - 20*
     +    M2**2*TG**(-2)*T**(-1)*NS*MS2 + 20*M2**2*TG**(-2)*T**(-1)*MS2
     +     + 20*M2**2*TG**(-2)*NS - 20*M2**2*TG**(-2) - 8*M2**3*
     +    TG**(-3)*T**(-1)*NS*MS2 + 8*M2**3*TG**(-3)*T**(-1)*MS2 + 8*
     +    M2**3*TG**(-3)*NS )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N*CF * (  - 8*M2**3*TG**(-3) - 8*
     +    TG**(-3)*S*T**(-1)*NS*MS2**3 + 8*TG**(-3)*S*T**(-1)*MS2**3 + 
     +    8*TG**(-3)*S*NS*MS2**2 - 8*TG**(-3)*S*MS2**2 - 12*TG**(-2)*S*
     +    T**(-1)*NS*MS2**2 + 12*TG**(-2)*S*T**(-1)*MS2**2 + 12*
     +    TG**(-2)*S*NS*MS2 - 12*TG**(-2)*S*MS2 - 4*TG**(-1)*S*T**(-1)*
     +    NS*MS2 + 4*TG**(-1)*S*T**(-1)*MS2 + 4*TG**(-1)*S*NS - 4*
     +    TG**(-1)*S - 8*TG**(-1)*T**(-1)*NS*MS2**2 + 8*TG**(-1)*
     +    T**(-1)*MS2**2 + 8*TG**(-1)*NS*MS2 - 8*TG**(-1)*MS2 - 4*
     +    T**(-1)*NS*MS2 + 4*T**(-1)*MS2 + 4*NS )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N*CF**2 * ( 16 + 32*M2*TG**(-2)*S
     +     + 32*M2*TG**(-2)*MS2 + 48*M2*TG**(-1) + 32*M2**2*TG**(-2) + 
     +    32*TG**(-2)*S*T1**(-1)*MS2**2 + 64*TG**(-2)*S*MS2 - 16*
     +    TG**(-1)*S*T1**(-1)*MS2 + 16*TG**(-1)*S + 32*TG**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(1,1)*N**2*CF * (  - 8 - 16*M2*TG**(-2)*S
     +     - 16*M2*TG**(-2)*MS2 - 24*M2*TG**(-1) - 16*M2**2*TG**(-2) - 
     +    16*TG**(-2)*S*T1**(-1)*MS2**2 - 32*TG**(-2)*S*MS2 + 8*
     +    TG**(-1)*S*T1**(-1)*MS2 - 8*TG**(-1)*S - 16*TG**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(2,1)*N**2*CF * (  - 4 + 24*M2*TG**(-3)*S
     +    *T**(-1)*MS2**2 - 48*M2*TG**(-3)*S*MS2 + 24*M2*TG**(-2)*S*
     +    T**(-1)*MS2 - 20*M2*TG**(-2)*S + 16*M2*TG**(-2)*T**(-1)*
     +    MS2**2 - 32*M2*TG**(-2)*MS2 + 4*M2*TG**(-1)*S*T**(-1) + 24*M2
     +    *TG**(-1)*T**(-1)*MS2 - 24*M2*TG**(-1) + 4*M2*T**(-1) + 24*
     +    M2**2*TG**(-3)*S*T**(-1)*MS2 - 24*M2**2*TG**(-3)*S + 8*M2**2*
     +    TG**(-3)*T**(-1)*MS2**2 - 24*M2**2*TG**(-3)*MS2 + 12*M2**2*
     +    TG**(-2)*S*T**(-1) + 36*M2**2*TG**(-2)*T**(-1)*MS2 - 44*M2**2
     +    *TG**(-2) + 16*M2**2*TG**(-1)*T**(-1) + 8*M2**3*TG**(-3)*S*
     +    T**(-1) + 16*M2**3*TG**(-3)*T**(-1)*MS2 - 24*M2**3*TG**(-3)
     +     + 20*M2**3*TG**(-2)*T**(-1) + 8*M2**4*TG**(-3)*T**(-1) + 8*
     +    TG**(-3)*S*T**(-1)*MS2**3 - 24*TG**(-3)*S*MS2**2 + 12*
     +    TG**(-2)*S*T**(-1)*MS2**2 + 16*TG**(-2)*S*T1**(-1)*MS2**2 - 4
     +    *TG**(-2)*S*MS2 + 4*TG**(-1)*S*T**(-1)*MS2 - 8*TG**(-1)*S*
     +    T1**(-1)*MS2 - 4*TG**(-1)*S + 8*TG**(-1)*T**(-1)*MS2**2 - 8*
     +    TG**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(2,1)*N**2*CF * ( 4*T**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(3,1)*N*CF * ( 4 + 16*M2*TG**(-3)*S*
     +    T**(-1)*MS2*MT2 - 16*M2*TG**(-3)*S*T**(-1)*MS2**2 + 16*M2*
     +    TG**(-3)*S*MS2 - 12*M2*TG**(-2)*S*T**(-1)*MS2 + 12*M2*
     +    TG**(-2)*S*T**(-1)*MT2 + 12*M2*TG**(-2)*S + 16*M2*TG**(-2)*
     +    T**(-1)*MS2*MT2 - 16*M2*TG**(-2)*T**(-1)*MS2**2 + 16*M2*
     +    TG**(-2)*MS2 - 16*M2*TG**(-1)*T**(-1)*MS2 + 16*M2*TG**(-1)*
     +    T**(-1)*MT2 + 16*M2*TG**(-1) - 8*M2**2*TG**(-3)*S*T**(-1)*MS2
     +     + 8*M2**2*TG**(-3)*S*T**(-1)*MT2 + 8*M2**2*TG**(-3)*S + 8*
     +    M2**2*TG**(-3)*T**(-1)*MS2*MT2 - 8*M2**2*TG**(-3)*T**(-1)*
     +    MS2**2 + 8*M2**2*TG**(-3)*MS2 - 20*M2**2*TG**(-2)*T**(-1)*MS2
     +     + 20*M2**2*TG**(-2)*T**(-1)*MT2 + 20*M2**2*TG**(-2) - 8*
     +    M2**3*TG**(-3)*T**(-1)*MS2 + 8*M2**3*TG**(-3)*T**(-1)*MT2 + 8
     +    *M2**3*TG**(-3) + 8*TG**(-3)*S*T**(-1)*MS2**2*MT2 - 8*
     +    TG**(-3)*S*T**(-1)*MS2**3 + 8*TG**(-3)*S*MS2**2 + 12*TG**(-2)
     +    *S*T**(-1)*MS2*MT2 - 12*TG**(-2)*S*T**(-1)*MS2**2 + 12*
     +    TG**(-2)*S*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0D(3,1)*N*CF * (  - 4*TG**(-1)*S*T**(-1)*
     +    MS2 + 4*TG**(-1)*S*T**(-1)*MT2 + 4*TG**(-1)*S + 8*TG**(-1)*
     +    T**(-1)*MS2*MT2 - 8*TG**(-1)*T**(-1)*MS2**2 + 8*TG**(-1)*MS2
     +     - 4*T**(-1)*MS2 + 4*T**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + SK1B0E(1)*N**2*CF * ( 32*M2*TG**(-3)*S*MS2 + 16
     +    *M2*TG**(-2)*S + 32*M2*TG**(-2)*MS2 + 16*M2*TG**(-1) + 16*
     +    M2**2*TG**(-3)*S + 16*M2**2*TG**(-3)*MS2 + 32*M2**2*TG**(-2)
     +     + 16*M2**3*TG**(-3) + 16*TG**(-3)*S*MS2**2 + 16*TG**(-2)*S*
     +    MS2 + 16*TG**(-1)*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1B0E(2)*N*CF * (  - 8*M2*TG**(-3)*S*NS*MS2 + 
     +    8*M2*TG**(-3)*S*MS2 - 8*M2*TG**(-2)*S*NS + 8*M2*TG**(-2)*S - 
     +    8*M2*TG**(-1)*NS + 8*M2*TG**(-1) - 8*M2**2*TG**(-3)*S*NS + 8*
     +    M2**2*TG**(-3)*S - 16*M2**2*TG**(-2)*NS + 16*M2**2*TG**(-2)
     +     - 8*M2**3*TG**(-3)*NS + 8*M2**3*TG**(-3) )
     +
      M2QBV2 = M2QBV2 + SK1B0E(3)*N*CF * (  - 8*M2*TG**(-3)*S*MS2 - 8*
     +    M2*TG**(-3)*S*MT2 - 8*M2*TG**(-2)*S - 16*M2*TG**(-2)*MT2 - 8*
     +    M2*TG**(-1) - 8*M2**2*TG**(-3)*S - 8*M2**2*TG**(-3)*MT2 - 16*
     +    M2**2*TG**(-2) - 8*M2**3*TG**(-3) - 8*TG**(-3)*S*MS2*MT2 - 8*
     +    TG**(-2)*S*MT2 - 8*TG**(-1)*MT2 )
     +
      M2QBV2 = M2QBV2 + SK1BP(1)*N*CF**2 * (  - 16*M2*TG**(-2)*S*MS2 - 
     +    32*M2*TG**(-1)*MS2 - 16*M2**2*TG**(-2)*MS2 - 16*TG**(-2)*S*
     +    MS2**2 - 16*TG**(-1)*S*MS2 - 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1BP(2)*N*CF**2 * (  - 8*M2*TG**(-2)*S*MS2 - 8
     +    *M2*TG**(-1)*S - 8*M2 - 8*M2**2*TG**(-2)*S - 16*M2**2*
     +    TG**(-1) - 8*M2**3*TG**(-2) )
     +
      M2QBV2 = M2QBV2 + SK1BP(3)*N*CF**2 * ( 4*M2*TG**(-2)*S*MS2 + 4*M2
     +    *TG**(-1)*S + 4*M2 + 4*M2**2*TG**(-2)*S + 8*M2**2*TG**(-1) + 
     +    4*M2**3*TG**(-2) )
     +
      M2QBV2 = M2QBV2 + SK1C0A(1)*N*CF**2 * ( 16*M2*TG**(-1)*S + 8*
     +    TG**(-1)*S**2 + 8*S )
     +
      M2QBV2 = M2QBV2 + SK1C0A(1)*N**2*CF * (  - 4*M2*TG**(-1)*S - 2*
     +    TG**(-1)*S**2 - 2*S )
     +
      M2QBV2 = M2QBV2 + SK1C0A(5)*N*CF**2 * ( 16*TG**(-1)*S*MS2 - 8*
     +    TG**(-1)*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1C0A(5)*N**2*CF * (  - 4*TG**(-1)*S*MS2 + 2*
     +    TG**(-1)*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1C0B(1)*N*CF**2 * ( 16*M2*TG**(-1)*S - 32*M2*
     +    TG**(-1)*MS2 - 16*TG**(-1)*S*MS2 + 8*TG**(-1)*S**2 + 16*S - 
     +    32*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0B(1)*N**2*CF * (  - 4*M2*TG**(-1)*S + 8*M2
     +    *TG**(-1)*MS2 + 4*TG**(-1)*S*MS2 - 2*TG**(-1)*S**2 - 4*S + 8*
     +    MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0B(4)*N*CF**2 * (  - 16*M2*TG**(-1)*S - 16*
     +    M2 - 16*M2**2*TG**(-1) + 16*TG**(-1)*S*MS2 - 8*TG**(-1)*S**2
     +     - 8*S )
     +
      M2QBV2 = M2QBV2 + SK1C0B(4)*N**2*CF * ( 4*M2*TG**(-1)*S + 4*M2 + 
     +    4*M2**2*TG**(-1) - 4*TG**(-1)*S*MS2 + 2*TG**(-1)*S**2 + 2*S )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,1)*N*CF**2 * ( 16*M2*TG**(-2)*S*MS2 + 
     +    32*M2*TG**(-1)*S + 32*M2 + 16*M2**2*TG**(-2)*S + 48*M2**2*
     +    TG**(-1) + 16*M2**3*TG**(-2) + 16*TG**(-1)*S*MS2 + 16*S + 16*
     +    T1 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,1)*N**2*CF * (  - 8*M2*TG**(-2)*S*MS2
     +     - 16*M2*TG**(-1)*S - 16*M2 - 8*M2**2*TG**(-2)*S - 24*M2**2*
     +    TG**(-1) - 8*M2**3*TG**(-2) - 8*TG**(-1)*S*MS2 - 8*S - 8*T1 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,2)*N*CF**2 * ( 32*M2*TG**(-1)*S - 16*
     +    M2*TG**(-1)*MS2 + 16*M2 + 16*M2**2*TG**(-1) - 16*TG**(-1)*S*
     +    MS2 + 16*TG**(-1)*S**2 + 32*S + 16*T1 - 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(3,2)*N**2*CF * (  - 16*M2*TG**(-1)*S + 8
     +    *M2*TG**(-1)*MS2 - 8*M2 - 8*M2**2*TG**(-1) + 8*TG**(-1)*S*MS2
     +     - 8*TG**(-1)*S**2 - 16*S - 8*T1 + 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(4,1)*N*CF**2 * ( 16*M2*TG**(-1)*MS2 + 16
     +    *M2 + 16*M2**2*TG**(-1) + 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(4,1)*N**2*CF * (  - 8*M2*TG**(-2)*S*MS2
     +     - 12*M2*TG**(-1)*S - 8*M2*TG**(-1)*MS2 - 16*M2 - 8*M2**2*
     +    TG**(-2)*S - 24*M2**2*TG**(-1) - 8*M2**3*TG**(-2) - 8*
     +    TG**(-1)*S*MS2 - 4*S - 4*T1 - 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(5,1)*N*CF**2 * ( 16*M2*TG**(-1)*MS2 + 16
     +    *M2 + 16*M2**2*TG**(-1) + 16*T1 + 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(5,1)*N**2*CF * (  - 24*M2*TG**(-2)*S*MS2
     +     - 12*M2*TG**(-1)*S - 24*M2*TG**(-1)*MS2 - 16*M2 - 8*M2**2*
     +    TG**(-2)*S - 16*M2**2*TG**(-2)*MS2 - 24*M2**2*TG**(-1) - 8*
     +    M2**3*TG**(-2) - 16*TG**(-2)*S*MS2**2 + 8*TG**(-1)*S*T1**(-1)
     +    *MS2**2 - 8*TG**(-1)*S*MS2 - 4*S - 8*T1 - 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(6,1)*N*CF**2 * ( 64*M2*TG**(-2)*S*MS2 + 
     +    16*M2*TG**(-1)*S + 48*M2*TG**(-1)*MS2 + 16*M2 + 32*M2**2*
     +    TG**(-2)*S + 32*M2**2*TG**(-2)*MS2 + 48*M2**2*TG**(-1) + 32*
     +    M2**3*TG**(-2) + 32*TG**(-2)*S*MS2**2 - 16*TG**(-1)*S*
     +    T1**(-1)*MS2**2 + 16*S*T1**(-1)*MS2 + 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(6,1)*N**2*CF * (  - 32*M2*TG**(-2)*S*MS2
     +     - 8*M2*TG**(-1)*S - 24*M2*TG**(-1)*MS2 - 8*M2 - 16*M2**2*
     +    TG**(-2)*S - 16*M2**2*TG**(-2)*MS2 - 24*M2**2*TG**(-1) - 16*
     +    M2**3*TG**(-2) - 16*TG**(-2)*S*MS2**2 + 8*TG**(-1)*S*T1**(-1)
     +    *MS2**2 - 8*S*T1**(-1)*MS2 - 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(6,2)*N*CF**2 * (  - 32*M2*TG**(-1)*S + 
     +    16*M2*TG**(-1)*MS2 - 16*M2 - 16*M2**2*TG**(-1) + 16*TG**(-1)*
     +    S*MS2 - 16*TG**(-1)*S**2 - 32*S - 16*T1 + 16*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1C0C(6,2)*N**2*CF * ( 16*M2*TG**(-1)*S - 8*M2
     +    *TG**(-1)*MS2 + 8*M2 + 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2 + 
     +    8*TG**(-1)*S**2 + 16*S + 8*T1 - 8*MS2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(4,1)*N*CF**2 * ( 16*M2*TG**(-1)*S**2 + 8*
     +    M2*S + 16*M2**2*TG**(-1)*S + 16*TG**(-1)*S**2*MS2 + 8*S*T1 + 
     +    8*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(4,1)*N**2*CF * (  - 4*M2*TG**(-1)*S**2 - 
     +    2*M2*S - 4*M2**2*TG**(-1)*S - 4*TG**(-1)*S**2*MS2 - 2*S*T1 - 
     +    2*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(7,1)*N*CF**2 * (  - 32*M2*TG**(-1)*S*MS2
     +     + 16*M2*TG**(-1)*S**2 + 16*M2*S - 32*M2*MS2 + 16*M2**2*
     +    TG**(-1)*S - 32*M2**2*TG**(-1)*MS2 - 32*TG**(-1)*S*MS2**2 + 
     +    16*TG**(-1)*S**2*MS2 - 16*S*MS2 + 8*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(7,1)*N**2*CF * ( 8*M2*TG**(-1)*S*MS2 - 4*
     +    M2*TG**(-1)*S**2 - 4*M2*S + 8*M2*MS2 - 4*M2**2*TG**(-1)*S + 8
     +    *M2**2*TG**(-1)*MS2 + 8*TG**(-1)*S*MS2**2 - 4*TG**(-1)*S**2*
     +    MS2 + 4*S*MS2 - 2*S**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(8,1)*N*CF**2 * ( 32*M2*TG**(-1)*S*MS2 + 
     +    32*M2*TG**(-1)*S**2 + 64*M2*S + 32*M2*T1 + 64*M2**2*TG**(-1)*
     +    S + 32*M2**2 + 32*M2**3*TG**(-1) + 32*TG**(-1)*S**2*MS2 + 32*
     +    S*T1 + 48*S*MS2 + 16*S**2 + 16*T1*MS2 + 16*T1**2 )
     +
      M2QBV2 = M2QBV2 + SK1D0(8,1)*N**2*CF * (  - 16*M2*TG**(-1)*S*MS2
     +     - 16*M2*TG**(-1)*S**2 - 32*M2*S - 16*M2*T1 - 32*M2**2*
     +    TG**(-1)*S - 16*M2**2 - 16*M2**3*TG**(-1) - 16*TG**(-1)*S**2*
     +    MS2 - 16*S*T1 - 24*S*MS2 - 8*S**2 - 8*T1*MS2 - 8*T1**2 )
     +
      M2QBV2 = M2QBV2 + SOF1(1)*N*CF**2 * ( 128*M2*TG**(-2)*S*MS2 - 64*
     +    M2*TG**(-2)*S**2 - 128*M2*TG**(-1)*S + 256*M2*TG**(-1)*MS2 - 
     +    64*M2**2*TG**(-2)*S + 128*M2**2*TG**(-2)*MS2 + 128*TG**(-2)*S
     +    *MS2**2 - 64*TG**(-2)*S**2*MS2 + 128*TG**(-1)*S*MS2 - 64*
     +    TG**(-1)*S**2 - 64*S + 128*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(1)*N**2*CF * (  - 32*M2*TG**(-2)*S*MS2 + 
     +    16*M2*TG**(-2)*S**2 + 32*M2*TG**(-1)*S - 64*M2*TG**(-1)*MS2
     +     + 16*M2**2*TG**(-2)*S - 32*M2**2*TG**(-2)*MS2 - 32*TG**(-2)*
     +    S*MS2**2 + 16*TG**(-2)*S**2*MS2 - 32*TG**(-1)*S*MS2 + 16*
     +    TG**(-1)*S**2 + 16*S - 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(2)*N*CF**2 * ( 32*M2*TG**(-2)*S*MS2 + 64*
     +    M2*TG**(-1)*MS2 + 32*M2**2*TG**(-2)*MS2 + 32*TG**(-2)*S*
     +    MS2**2 + 32*TG**(-1)*S*MS2 + 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(3)*N*CF**2 * ( 32*M2*TG**(-2)*S*MS2 + 64*
     +    M2*TG**(-1)*MS2 + 32*M2**2*TG**(-2)*MS2 + 32*TG**(-2)*S*
     +    MS2**2 + 32*TG**(-1)*S*MS2 + 32*MS2 )
     +
      M2QBV2 = M2QBV2 + SOF1(4)*N*CF**2 * (  - 64*M2*TG**(-2)*S*MS2 - 
     +    64*M2*TG**(-2)*S**2 - 256*M2*TG**(-1)*S - 128*M2 - 128*M2**2*
     +    TG**(-2)*S - 192*M2**2*TG**(-1) - 64*M2**3*TG**(-2) - 64*
     +    TG**(-2)*S**2*MS2 - 64*TG**(-1)*S*MS2 - 64*TG**(-1)*S**2 - 
     +    128*S - 64*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(4)*N**2*CF * ( 32*M2*TG**(-2)*S*MS2 + 32*
     +    M2*TG**(-2)*S**2 + 128*M2*TG**(-1)*S + 64*M2 + 64*M2**2*
     +    TG**(-2)*S + 96*M2**2*TG**(-1) + 32*M2**3*TG**(-2) + 32*
     +    TG**(-2)*S**2*MS2 + 32*TG**(-1)*S*MS2 + 32*TG**(-1)*S**2 + 64
     +    *S + 32*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(5)*N*CF**2 * (  - 32*M2*TG**(-2)*S*MS2 - 
     +    64*M2*TG**(-1)*S - 64*M2 - 32*M2**2*TG**(-2)*S - 96*M2**2*
     +    TG**(-1) - 32*M2**3*TG**(-2) - 32*TG**(-1)*S*MS2 - 32*S - 32*
     +    T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(5)*N**2*CF * ( 16*M2*TG**(-2)*S*MS2 + 32*
     +    M2*TG**(-1)*S + 32*M2 + 16*M2**2*TG**(-2)*S + 48*M2**2*
     +    TG**(-1) + 16*M2**3*TG**(-2) + 16*TG**(-1)*S*MS2 + 16*S + 16*
     +    T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(6)*N*CF**2 * (  - 32*M2*TG**(-2)*S*MS2 - 
     +    64*M2*TG**(-1)*S - 64*M2 - 32*M2**2*TG**(-2)*S - 96*M2**2*
     +    TG**(-1) - 32*M2**3*TG**(-2) - 32*TG**(-1)*S*MS2 - 32*S - 32*
     +    T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(6)*N**2*CF * ( 16*M2*TG**(-2)*S*MS2 + 32*
     +    M2*TG**(-1)*S + 32*M2 + 16*M2**2*TG**(-2)*S + 48*M2**2*
     +    TG**(-1) + 16*M2**3*TG**(-2) + 16*TG**(-1)*S*MS2 + 16*S + 16*
     +    T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(7)*N*CF**2 * (  - 64*M2*TG**(-2)*S*MS2 - 
     +    64*M2*TG**(-2)*S**2 - 256*M2*TG**(-1)*S - 128*M2 - 128*M2**2*
     +    TG**(-2)*S - 192*M2**2*TG**(-1) - 64*M2**3*TG**(-2) - 64*
     +    TG**(-2)*S**2*MS2 - 64*TG**(-1)*S*MS2 - 64*TG**(-1)*S**2 - 
     +    128*S - 64*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(7)*N**2*CF * ( 32*M2*TG**(-2)*S*MS2 + 32*
     +    M2*TG**(-2)*S**2 + 128*M2*TG**(-1)*S + 64*M2 + 64*M2**2*
     +    TG**(-2)*S + 96*M2**2*TG**(-1) + 32*M2**3*TG**(-2) + 32*
     +    TG**(-2)*S**2*MS2 + 32*TG**(-1)*S*MS2 + 32*TG**(-1)*S**2 + 64
     +    *S + 32*T1 )
     +
      M2QBV2 = M2QBV2 + SOF1(8)*N*CF**2 * (  - 64*M2*TG**(-2)*S**2 - 
     +    128*M2*TG**(-1)*S - 64*M2**2*TG**(-2)*S - 64*TG**(-2)*S**2*
     +    MS2 - 64*TG**(-1)*S**2 - 64*S )
     +
      M2QBV2 = M2QBV2 + SOF1(8)*N**2*CF * ( 16*M2*TG**(-2)*S**2 + 32*M2
     +    *TG**(-1)*S + 16*M2**2*TG**(-2)*S + 16*TG**(-2)*S**2*MS2 + 16
     +    *TG**(-1)*S**2 + 16*S )


      M2QBV3 = 0.D0
      M2QBV3 = M2QBV3 + N*CF * (  - 16*M2*TG**(-3)*S*T**(-1)*NS*MS2**2
     +     + 16*M2*TG**(-3)*S*T**(-1)*MS2*MT2 + 8*M2*TG**(-3)*S*NS*MS2
     +     - 8*M2*TG**(-3)*S*MT2 - 8*M2*TG**(-2)*S*T**(-1)*NS*MS2 + 8*
     +    M2*TG**(-2)*S*T**(-1)*MT2 - 8*M2**2*TG**(-3)*S*T**(-1)*NS*MS2
     +     + 8*M2**2*TG**(-3)*S*T**(-1)*MT2 - 8*TG**(-3)*S*T**(-1)*NS*
     +    MS2**3 + 8*TG**(-3)*S*T**(-1)*MS2**2*MT2 + 8*TG**(-3)*S*NS*
     +    MS2**2 - 8*TG**(-3)*S*MS2*MT2 - 8*TG**(-2)*S*T**(-1)*NS*
     +    MS2**2 + 8*TG**(-2)*S*T**(-1)*MS2*MT2 + 4*TG**(-2)*S*NS*MS2
     +     - 4*TG**(-2)*S*MT2 )
     +
      M2QBV3 = M2QBV3 + N*CF**2 * (  - 4*M2*TG**(-2)*S )
     +
      M2QBV3 = M2QBV3 + N**2*CF * ( 24*M2*TG**(-3)*S*T**(-1)*MS2**2 - 
     +    16*M2*TG**(-3)*S*MS2 + 16*M2*TG**(-2)*S*T**(-1)*MS2 - 8*M2*
     +    TG**(-2)*S + 24*M2**2*TG**(-3)*S*T**(-1)*MS2 - 8*M2**2*
     +    TG**(-3)*S + 8*M2**2*TG**(-2)*S*T**(-1) + 8*M2**3*TG**(-3)*S*
     +    T**(-1) + 8*TG**(-3)*S*T**(-1)*MS2**3 - 8*TG**(-3)*S*MS2**2
     +     + 8*TG**(-2)*S*T**(-1)*MS2**2 - 8*TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0A(1)*N*CF**2 * (  - 4*M2*TG**(-2)*S - 4*
     +    TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0A(1)*N**2*CF * ( 24*M2*TG**(-3)*S*T**(-1)*
     +    MS2**2 - 16*M2*TG**(-3)*S*MS2 + 16*M2*TG**(-2)*S*T**(-1)*MS2
     +     - 4*M2*TG**(-2)*S + 24*M2**2*TG**(-3)*S*T**(-1)*MS2 - 8*
     +    M2**2*TG**(-3)*S + 8*M2**2*TG**(-2)*S*T**(-1) + 8*M2**3*
     +    TG**(-3)*S*T**(-1) + 8*TG**(-3)*S*T**(-1)*MS2**3 - 8*TG**(-3)
     +    *S*MS2**2 + 8*TG**(-2)*S*T**(-1)*MS2**2 - 4*TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0A(2)*N*CF * ( 16*M2*TG**(-3)*S*T**(-1)*MS2
     +    *MT2 - 8*M2*TG**(-3)*S*MT2 + 8*M2*TG**(-2)*S*T**(-1)*MT2 + 8*
     +    M2**2*TG**(-3)*S*T**(-1)*MT2 + 8*TG**(-3)*S*T**(-1)*MS2**2*
     +    MT2 - 8*TG**(-3)*S*MS2*MT2 + 8*TG**(-2)*S*T**(-1)*MS2*MT2 - 4
     +    *TG**(-2)*S*MT2 )
     +
      M2QBV3 = M2QBV3 + SK1B0A(3)*N*CF**2 * (  - 4*TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0C(1)*N*CF**2 * ( 16*M2*TG**(-2)*S + 16*
     +    TG**(-2)*S*T1**(-1)*MS2**2 + 32*TG**(-2)*S*MS2 - 16*TG**(-1)*
     +    S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0C(2)*N*CF**2 * ( 8*M2*TG**(-2)*S + 16*
     +    TG**(-2)*S*T1**(-1)*MS2**2 + 24*TG**(-2)*S*MS2 - 16*TG**(-1)*
     +    S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(1,1)*N*CF * ( 16*M2*TG**(-3)*S*T**(-1)*
     +    NS*MS2**2 - 16*M2*TG**(-3)*S*T**(-1)*MS2**2 - 16*M2*TG**(-3)*
     +    S*NS*MS2 + 16*M2*TG**(-3)*S*MS2 + 8*M2*TG**(-2)*S*T**(-1)*NS*
     +    MS2 - 8*M2*TG**(-2)*S*T**(-1)*MS2 - 8*M2*TG**(-2)*S*NS + 8*M2
     +    *TG**(-2)*S + 8*M2**2*TG**(-3)*S*T**(-1)*NS*MS2 - 8*M2**2*
     +    TG**(-3)*S*T**(-1)*MS2 - 8*M2**2*TG**(-3)*S*NS + 8*M2**2*
     +    TG**(-3)*S + 8*TG**(-3)*S*T**(-1)*NS*MS2**3 - 8*TG**(-3)*S*
     +    T**(-1)*MS2**3 - 8*TG**(-3)*S*NS*MS2**2 + 8*TG**(-3)*S*MS2**2
     +     + 8*TG**(-2)*S*T**(-1)*NS*MS2**2 - 8*TG**(-2)*S*T**(-1)*
     +    MS2**2 - 8*TG**(-2)*S*NS*MS2 + 8*TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(1,1)*N*CF**2 * (  - 32*M2*TG**(-2)*S - 
     +    32*TG**(-2)*S*T1**(-1)*MS2**2 - 64*TG**(-2)*S*MS2 + 32*
     +    TG**(-1)*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(1,1)*N**2*CF * ( 16*M2*TG**(-2)*S + 16*
     +    TG**(-2)*S*T1**(-1)*MS2**2 + 32*TG**(-2)*S*MS2 - 16*TG**(-1)*
     +    S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(2,1)*N**2*CF * (  - 24*M2*TG**(-3)*S*
     +    T**(-1)*MS2**2 + 48*M2*TG**(-3)*S*MS2 - 16*M2*TG**(-2)*S*
     +    T**(-1)*MS2 + 8*M2*TG**(-2)*S - 24*M2**2*TG**(-3)*S*T**(-1)*
     +    MS2 + 24*M2**2*TG**(-3)*S - 8*M2**2*TG**(-2)*S*T**(-1) - 8*
     +    M2**3*TG**(-3)*S*T**(-1) - 8*TG**(-3)*S*T**(-1)*MS2**3 + 24*
     +    TG**(-3)*S*MS2**2 - 8*TG**(-2)*S*T**(-1)*MS2**2 - 16*TG**(-2)
     +    *S*T1**(-1)*MS2**2 - 8*TG**(-2)*S*MS2 + 16*TG**(-1)*S*
     +    T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0D(3,1)*N*CF * (  - 16*M2*TG**(-3)*S*
     +    T**(-1)*MS2*MT2 + 16*M2*TG**(-3)*S*T**(-1)*MS2**2 - 16*M2*
     +    TG**(-3)*S*MS2 + 8*M2*TG**(-2)*S*T**(-1)*MS2 - 8*M2*TG**(-2)*
     +    S*T**(-1)*MT2 - 8*M2*TG**(-2)*S + 8*M2**2*TG**(-3)*S*T**(-1)*
     +    MS2 - 8*M2**2*TG**(-3)*S*T**(-1)*MT2 - 8*M2**2*TG**(-3)*S - 8
     +    *TG**(-3)*S*T**(-1)*MS2**2*MT2 + 8*TG**(-3)*S*T**(-1)*MS2**3
     +     - 8*TG**(-3)*S*MS2**2 - 8*TG**(-2)*S*T**(-1)*MS2*MT2 + 8*
     +    TG**(-2)*S*T**(-1)*MS2**2 - 8*TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0E(1)*N**2*CF * (  - 32*M2*TG**(-3)*S*MS2
     +     - 8*M2*TG**(-2)*S - 16*M2**2*TG**(-3)*S - 16*TG**(-3)*S*
     +    MS2**2 - 8*TG**(-2)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1B0E(2)*N*CF * ( 8*M2*TG**(-3)*S*NS*MS2 - 8*
     +    M2*TG**(-3)*S*MS2 + 4*M2*TG**(-2)*S*NS - 4*M2*TG**(-2)*S + 8*
     +    M2**2*TG**(-3)*S*NS - 8*M2**2*TG**(-3)*S )
     +
      M2QBV3 = M2QBV3 + SK1B0E(3)*N*CF * ( 8*M2*TG**(-3)*S*MS2 + 8*M2*
     +    TG**(-3)*S*MT2 + 4*M2*TG**(-2)*S + 8*M2**2*TG**(-3)*S + 8*
     +    TG**(-3)*S*MS2*MT2 + 4*TG**(-2)*S*MT2 )
     +
      M2QBV3 = M2QBV3 + SK1BP(1)*N*CF**2 * ( 16*M2*TG**(-2)*S*MS2 + 16*
     +    TG**(-2)*S*MS2**2 )
     +
      M2QBV3 = M2QBV3 + SK1BP(2)*N*CF**2 * ( 8*M2*TG**(-2)*S*MS2 + 8*
     +    M2**2*TG**(-2)*S )
     +
      M2QBV3 = M2QBV3 + SK1BP(3)*N*CF**2 * (  - 4*M2*TG**(-2)*S*MS2 - 4
     +    *M2**2*TG**(-2)*S )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,1)*N*CF**2 * (  - 16*M2*TG**(-2)*S*MS2
     +     - 16*M2*TG**(-1)*S - 16*M2**2*TG**(-2)*S - 16*TG**(-1)*S*MS2
     +     )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,1)*N**2*CF * ( 8*M2*TG**(-2)*S*MS2 + 8
     +    *M2*TG**(-1)*S + 8*M2**2*TG**(-2)*S + 8*TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,2)*N*CF**2 * ( 16*M2*TG**(-1)*S + 16*
     +    M2*TG**(-1)*MS2 + 16*M2 + 16*M2**2*TG**(-1) + 16*TG**(-1)*S*
     +    MS2 + 16*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(3,2)*N**2*CF * (  - 8*M2*TG**(-1)*S - 8*
     +    M2*TG**(-1)*MS2 - 8*M2 - 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2
     +     - 8*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(4,1)*N*CF**2 * (  - 16*M2*TG**(-1)*MS2
     +     - 16*M2 - 16*M2**2*TG**(-1) - 16*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(4,1)*N**2*CF * ( 8*M2*TG**(-2)*S*MS2 + 8
     +    *M2*TG**(-1)*S + 8*M2*TG**(-1)*MS2 + 8*M2 + 8*M2**2*TG**(-2)*
     +    S + 8*M2**2*TG**(-1) + 8*TG**(-1)*S*MS2 + 8*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(5,1)*N*CF**2 * (  - 16*M2*TG**(-1)*MS2
     +     - 16*M2 - 16*M2**2*TG**(-1) - 16*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(5,1)*N**2*CF * ( 24*M2*TG**(-2)*S*MS2 + 
     +    8*M2*TG**(-1)*S + 8*M2*TG**(-1)*MS2 + 8*M2 + 8*M2**2*TG**(-2)
     +    *S + 8*M2**2*TG**(-1) + 16*TG**(-2)*S*MS2**2 - 8*TG**(-1)*S*
     +    T1**(-1)*MS2**2 + 8*S*T1**(-1)*MS2 + 8*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(6,1)*N*CF**2 * (  - 64*M2*TG**(-2)*S*MS2
     +     - 16*M2*TG**(-1)*S - 32*M2**2*TG**(-2)*S - 32*TG**(-2)*S*
     +    MS2**2 + 16*TG**(-1)*S*T1**(-1)*MS2**2 - 16*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(6,1)*N**2*CF * ( 32*M2*TG**(-2)*S*MS2 + 
     +    8*M2*TG**(-1)*S + 16*M2**2*TG**(-2)*S + 16*TG**(-2)*S*MS2**2
     +     - 8*TG**(-1)*S*T1**(-1)*MS2**2 + 8*S*T1**(-1)*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(6,2)*N*CF**2 * ( 16*M2*TG**(-1)*S + 16*
     +    M2*TG**(-1)*MS2 + 16*M2 + 16*M2**2*TG**(-1) + 16*TG**(-1)*S*
     +    MS2 + 16*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1C0C(6,2)*N**2*CF * (  - 8*M2*TG**(-1)*S - 8*
     +    M2*TG**(-1)*MS2 - 8*M2 - 8*M2**2*TG**(-1) - 8*TG**(-1)*S*MS2
     +     - 8*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(4,1)*N*CF**2 * ( 16*M2*TG**(-1)*S*U1 + 16
     +    *M2*TG**(-1)*S*MS2 + 16*M2*S + 16*M2**2*TG**(-1)*S + 16*
     +    TG**(-1)*S*U1*MS2 + 16*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(4,1)*N**2*CF * (  - 4*M2*TG**(-1)*S*U1 - 
     +    4*M2*TG**(-1)*S*MS2 - 4*M2*S - 4*M2**2*TG**(-1)*S - 4*
     +    TG**(-1)*S*U1*MS2 - 4*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(7,1)*N*CF**2 * ( 16*M2*TG**(-1)*S*U1 + 48
     +    *M2*TG**(-1)*S*MS2 + 16*M2*S + 16*M2**2*TG**(-1)*S + 16*
     +    TG**(-1)*S*U1*MS2 + 32*TG**(-1)*S*MS2**2 + 16*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(7,1)*N**2*CF * (  - 4*M2*TG**(-1)*S*U1 - 
     +    12*M2*TG**(-1)*S*MS2 - 4*M2*S - 4*M2**2*TG**(-1)*S - 4*
     +    TG**(-1)*S*U1*MS2 - 8*TG**(-1)*S*MS2**2 - 4*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(8,1)*N*CF**2 * ( 32*M2*TG**(-1)*S*U1 - 16
     +    *M2*S - 16*M2*T1 + 32*TG**(-1)*S*U1*MS2 - 16*S*MS2 - 16*T1*
     +    MS2 )
     +
      M2QBV3 = M2QBV3 + SK1D0(8,1)*N**2*CF * (  - 16*M2*TG**(-1)*S*U1
     +     + 8*M2*S + 8*M2*T1 - 16*TG**(-1)*S*U1*MS2 + 8*S*MS2 + 8*T1*
     +    MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(1)*N*CF**2 * (  - 64*M2*TG**(-2)*S*U1 - 
     +    192*M2*TG**(-2)*S*MS2 - 64*M2*TG**(-1)*S - 64*M2**2*TG**(-2)*
     +    S - 64*TG**(-2)*S*U1*MS2 - 128*TG**(-2)*S*MS2**2 - 64*
     +    TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(1)*N**2*CF * ( 16*M2*TG**(-2)*S*U1 + 48*M2
     +    *TG**(-2)*S*MS2 + 16*M2*TG**(-1)*S + 16*M2**2*TG**(-2)*S + 16
     +    *TG**(-2)*S*U1*MS2 + 32*TG**(-2)*S*MS2**2 + 16*TG**(-1)*S*MS2
     +     )
     +
      M2QBV3 = M2QBV3 + SOF1(2)*N*CF**2 * (  - 32*M2*TG**(-2)*S*MS2 - 
     +    32*TG**(-2)*S*MS2**2 )
     +
      M2QBV3 = M2QBV3 + SOF1(3)*N*CF**2 * (  - 32*M2*TG**(-2)*S*MS2 - 
     +    32*TG**(-2)*S*MS2**2 )
     +
      M2QBV3 = M2QBV3 + SOF1(4)*N*CF**2 * (  - 64*M2*TG**(-2)*S*U1 - 64
     +    *TG**(-2)*S*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(4)*N**2*CF * ( 32*M2*TG**(-2)*S*U1 + 32*
     +    TG**(-2)*S*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(5)*N*CF**2 * ( 32*M2*TG**(-2)*S*MS2 + 32*
     +    M2*TG**(-1)*S + 32*M2**2*TG**(-2)*S + 32*TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(5)*N**2*CF * (  - 16*M2*TG**(-2)*S*MS2 - 
     +    16*M2*TG**(-1)*S - 16*M2**2*TG**(-2)*S - 16*TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(6)*N*CF**2 * ( 32*M2*TG**(-2)*S*MS2 + 32*
     +    M2*TG**(-1)*S + 32*M2**2*TG**(-2)*S + 32*TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(6)*N**2*CF * (  - 16*M2*TG**(-2)*S*MS2 - 
     +    16*M2*TG**(-1)*S - 16*M2**2*TG**(-2)*S - 16*TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(7)*N*CF**2 * (  - 64*M2*TG**(-2)*S*U1 - 64
     +    *TG**(-2)*S*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(7)*N**2*CF * ( 32*M2*TG**(-2)*S*U1 + 32*
     +    TG**(-2)*S*U1*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(8)*N*CF**2 * (  - 64*M2*TG**(-2)*S*U1 - 64
     +    *M2*TG**(-2)*S*MS2 - 64*M2*TG**(-1)*S - 64*M2**2*TG**(-2)*S
     +     - 64*TG**(-2)*S*U1*MS2 - 64*TG**(-1)*S*MS2 )
     +
      M2QBV3 = M2QBV3 + SOF1(8)*N**2*CF * ( 16*M2*TG**(-2)*S*U1 + 16*M2
     +    *TG**(-2)*S*MS2 + 16*M2*TG**(-1)*S + 16*M2**2*TG**(-2)*S + 16
     +    *TG**(-2)*S*U1*MS2 + 16*TG**(-1)*S*MS2 )

      END IF

      IF (IFL.EQ.0) M2QBV = 2.D0*M2QBV2 +2.D0*M2QBV3 
      IF (IFL.EQ.1) M2QBV = 
     +     2.D0*(NS-1.D0)*M2QBV1 +2.D0*M2QBV2 +2.D0*M2QBV3 +2.D0*M2QBV4 

            
      DSBQBV = 
     +     ALPHAS**3 * AVG * M2QBV /4.D0 /S**2 *CONV
     +    + ALPHAS/PI*DSBQBB(ALPHAS,S,T1,MS,MG,IFL) 
     +    * (LOG(MG2/MS2)*(-1.D0) + LOG(MT2/MS2)*(-1.D0/3.D0))
     +    +  DSBQBR(ALPHAS,S,T1,MS,MG,IFL)
C***  CHANGES TO SUBTRACTED MSBAR AND TO DRBAR FOR THE YUKAWA-COUPLING 
      RETURN
      END



      REAL*8 FUNCTION DSBQBH(ALPHAS,S,T1,S4,MS,MG,IFL,EPS)
C***  CROSS SECTIONS FOR Q + QB -> SQ + SQB +G
C***  IFL = 0 <--> Q NOT QB
C***  IFL = 1 <--> Q  =  QB
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,TG,U1,MS,MG,MS2,MG2,M2
      REAL*8 NS,CONV,N,CF,AVG, SYMBX,EPS
      REAL*8 ANGDEF(1:11), ANA(1:3,1:9), ANB(1:3,1:9), ANC(1:3,1:9)
      REAL*8 AHP1P1, ABP1P1, ABP1M1, ABP1P2, A4P2P2, A4P1P2, A4P1P1
      REAL*8 A4M1P2, A4M1P1, A4P0P2, A4P1P0,A4P2M2, A4M2P1
      REAL*8 M2QBH1,M2QBH2,M2QBH3,M2QBH4,M2QBH
      REAL*8 ANG4(1:84), COLO1(1:9)
      INTEGER IFL
           
      CONV = 389379660.D0
      NS = 6.D0

      N = 3.D0
      CF = (N**2 -1.D0)/N/2.D0

      AVG = (1.D0/2.D0)**2 /N**2

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 - MS2
      U1 = S4 -S -T1
      TG = T1 -M2
      SYMBX = 1.D0/((U1+M2)**2 + EPS*MG**4)

      ANGDEF(1) = 2.D0*SQRT(S4 +MS2)
      ANGDEF(2) = (S +U1)/ANGDEF(1)
      ANGDEF(3) = (S +T1)/ANGDEF(1)
      ANGDEF(4) = S4/ANGDEF(1)
      ANGDEF(5) = (S4 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(6) = -(T1 +U1 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(7) = SQRT((T1 +U1)**2 -4.D0*MS2*S)/ANGDEF(1)
      ANGDEF(8) = 
     +  (T1*S4 -S*(U1+2.D0*MS2))/(S+T1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(9) = SQRT(1 -ANGDEF(8)**2)
      ANGDEF(10) =
     +  (U1*S4 -S*(T1+2.D0*MS2))/(S+U1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(11) = SQRT(1 -ANGDEF(10)**2)

      ANA(1,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)
      ANC(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(1,2) = -ANB(1,1)
      ANC(1,2) = -ANC(1,1)
      ANA(1,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(1,3) = -ANA(1,3)
      ANC(1,3) =  0.D0
      ANA(1,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(1,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)-2.D0*ANGDEF(3)*ANGDEF(4)
      ANC(1,4) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(1,5) = -ANB(1,3)
      ANC(1,5) = -ANC(1,3)
      ANA(1,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(1,6) = -ANB(1,4)
      ANC(1,6) = -ANC(1,4)
      ANA(1,7) = +ANA(1,1) -M2
      ANB(1,7) = +ANB(1,1)
      ANC(1,7) = +ANC(1,1)
      ANA(1,8) = +ANA(1,5) -M2
      ANB(1,8) = +ANB(1,5)
      ANC(1,8) = +ANC(1,5)
      ANA(1,9) = +ANA(1,6) -M2
      ANB(1,9) = +ANB(1,6)
      ANC(1,9) = +ANC(1,6)


      ANA(2,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(2,1) = -2.D0*ANGDEF(4)*ANGDEF(7)
      ANC(2,1) =  0.D0
      ANA(2,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(2,2) = -ANB(2,1)
      ANC(2,2) = -ANC(2,1)
      ANA(2,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(2,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7) -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,4) = -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(2,5) = -ANB(2,3)
      ANC(2,5) = -ANC(2,3)
      ANA(2,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(2,6) = -ANB(2,4)
      ANC(2,6) = -ANC(2,4)
      ANA(2,7) = +ANA(2,1) -M2
      ANB(2,7) = +ANB(2,1)
      ANC(2,7) = +ANC(2,1)
      ANA(2,8) = +ANA(2,5) -M2
      ANB(2,8) = +ANB(2,5)
      ANC(2,8) = +ANC(2,5)
      ANA(2,9) = +ANA(2,6) -M2
      ANB(2,9) = +ANB(2,6)
      ANC(2,9) = +ANC(2,6)


      ANA(3,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10)
      ANC(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(3,2) = -ANB(3,1)
      ANC(3,2) = -ANC(3,1)
      ANA(3,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(3,3) = 
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10) -2.D0*ANGDEF(2)*ANGDEF(4)
      ANC(3,3) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(3,4) = -ANA(3,4)
      ANC(3,4) =  0.D0
      ANA(3,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(3,5) = -ANB(3,3)
      ANC(3,5) = -ANC(3,3)
      ANA(3,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(3,6) = -ANB(3,4)
      ANC(3,6) = -ANC(3,4)
      ANA(3,7) = +ANA(3,1) -M2
      ANB(3,7) = +ANB(3,1)
      ANC(3,7) = +ANC(3,1)
      ANA(3,8) = +ANA(3,5) -M2
      ANB(3,8) = +ANB(3,5)
      ANC(3,8) = +ANC(3,5)
      ANA(3,9) = +ANA(3,6) -M2
      ANB(3,9) = +ANB(3,6)
      ANC(3,9) = +ANC(3,6)

C$$$      ANG4(1) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(2) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(3) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(4) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(5) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(6) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(7) =A4P1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(8) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(9) =A4P2M2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(10) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))

      ANG4(11) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(12) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(13) =A4P2M2(ANA(2,2),ANB(2,2),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(14) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(15) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(16) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(17) =A4P1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(18) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(19) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(20) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,5),ANB(3,5),ANC(3,5))

C$$$      ANG4(21) =A4M1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(22) =A4M1P2(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(23) =A4M2P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(24) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(25) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(26) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,7),ANB(1,7),ANC(1,7))
C$$$      ANG4(27) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(28) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(29) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(30) =A4P1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))

C$$$      ANG4(31) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(32) =A4M1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(33) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(34) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(35) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
      ANG4(36) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(37) =A4P1P0(ANA(2,1),ANB(2,1),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(38) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(39) =A4P1P0(ANA(2,2),ANB(2,2),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(40) =A4P0P2(ANA(1,1),ANB(1,1),ANA(2,5),ANB(2,5),ANC(2,5))

C$$$      ANG4(41) =A4P1P0(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$
C$$$      ANG4(43) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(44) =A4P1P0(ANA(3,6),ANB(3,6),ANA(1,1),ANB(1,1),ANC(1,1))

      ANG4(48) =AHP1P1(ANA(1,3),ANB(1,3),ANA(1,4),ANB(1,4),ANC(1,4))
      ANG4(49) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(50) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,1),ANB(3,1),ANC(3,1))

C$$$      ANG4(51) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(52) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(53) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(54) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(55) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(56) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(57) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(58) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(59) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(60) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))

C$$$      ANG4(61) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(62) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(63) =ABP1M1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(64) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(65) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(66) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(67) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
      ANG4(68) =ABP1M1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 
C$$$      ANG4(69) =ABP1M2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(70) =ABP2P0(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))

C$$$      ANG4(71) =ABP2P0(ANA(3,4),ANB(3,4),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(72) = A4P2P2(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
      ANG4(73) = A4P1P2(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
      ANG4(74) = A4P1P2(ANA(3,9),ANB(3,9),ANA(3,1),ANB(3,1),ANC(3,1))
      ANG4(75) = A4P1P1(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
      ANG4(76) = A4P1P1(ANA(3,9),ANB(3,9),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(77) = A4M1P2(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
      ANG4(78) = A4M1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
      ANG4(79) = A4P0P2(ANA(1,1),ANB(1,1),ANA(1,9),ANB(1,9),ANC(1,9))
      ANG4(80) = A4P1P0(ANA(3,9),ANB(3,9),ANA(1,1),ANB(1,1),ANC(1,1))

      ANG4(81) = ABP1P2(ANA(1,3),ANB(1,3),ANA(1,9),ANB(1,9),ANC(1,9)) 
      ANG4(82) = ABP1P1(ANA(1,3),ANB(1,3),ANA(1,9),ANB(1,9),ANC(1,9)) 
C$$$      ANG4(83) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,7),ANB(1,7),ANC(1,7))
C$$$      ANG4(84) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,7),ANB(3,7),ANC(3,7))


      COLO1(9) = LOG(S4**2/MS2/(S4+MS2))

      IF (IFL.EQ.1) THEN

      M2QBH1 = 0.D0
      M2QBH1 = M2QBH1 + N*CF**2*(S4+MS2) * ( 16*S**(-2)*T1**(-1)*U1**2*
     +    S4**(-1) - 64*S**(-2)*T1*U1*S4**(-2) + 48*S**(-2)*T1*S4**(-1)
     +     + 16*S**(-2)*T1**2*U1**(-1)*S4**(-1) - 32*S**(-2)*T1**2*
     +    S4**(-2) + 48*S**(-2)*U1*S4**(-1) - 32*S**(-2)*U1**2*S4**(-2)
     +     - 16*S**(-1)*T1**(-2)*U1*S4**(-1)*MS2 + 16*S**(-1)*T1**(-1)*
     +    U1*S4**(-1) - 16*S**(-1)*T1**(-1)*S4**(-1)*MS2 - 16*S**(-1)*
     +    T1*U1**(-2)*S4**(-1)*MS2 + 16*S**(-1)*T1*U1**(-1)*S4**(-1) - 
     +    64*S**(-1)*T1*S4**(-2) - 16*S**(-1)*U1**(-1)*S4**(-1)*MS2 - 
     +    64*S**(-1)*U1*S4**(-2) + 32*S**(-1)*S4**(-1) - 16*T1**(-2)*
     +    S4**(-1)*MS2 - 16*U1**(-2)*S4**(-1)*MS2 - 32*S4**(-2) )
     +
      M2QBH1 = M2QBH1 + N*CF**2 * ( 32*S**(-2)*U1**2*S4**(-2)*MS2 + 32*
     +    S**(-1)*U1*S4**(-2)*MS2 - 16*S**(-1)*U1*S4**(-1) + 32*S**(-1)
     +    *S4**(-2)*MS2**2 + 32*S**(-1)*S4**(-1)*MS2 + 16*S**(-1) - 16*
     +    S4**(-1) )
     +
      M2QBH1 = M2QBH1 + N**2*CF*(S4+MS2) * ( 32*S**(-2)*T1*U1*S4**(-2)
     +     - 16*S**(-2)*T1*S4**(-1) + 16*S**(-2)*T1**2*S4**(-2) - 16*
     +    S**(-2)*U1*S4**(-1) + 16*S**(-2)*U1**2*S4**(-2) + 32*S**(-1)*
     +    T1*S4**(-2) + 32*S**(-1)*U1*S4**(-2) - 16*S**(-1)*S4**(-1) + 
     +    16*S4**(-2) )
     +
      M2QBH1 = M2QBH1 + COLO1(9)*N*CF**2*(S4+MS2) * ( 16*S**(-2)*
     +    T1**(-1)*U1**3*S4**(-2) + 32*S**(-2)*T1*U1*S4**(-2) + 16*
     +    S**(-2)*T1**3*U1**(-1)*S4**(-2) - 16*S**(-1)*T1**(-2)*U1**2*
     +    S4**(-2)*MS2 + 32*S**(-1)*T1**(-1)*U1**2*S4**(-2) - 16*
     +    S**(-1)*T1**2*U1**(-2)*S4**(-2)*MS2 + 32*S**(-1)*T1**2*
     +    U1**(-1)*S4**(-2) - 32*S**(-1)*S4**(-2)*MS2 - 16*S*T1**(-2)*
     +    S4**(-2)*MS2 - 16*S*U1**(-2)*S4**(-2)*MS2 - 32*T1**(-2)*U1*
     +    S4**(-2)*MS2 + 16*T1**(-1)*U1*S4**(-2) - 32*T1*U1**(-2)*
     +    S4**(-2)*MS2 + 16*T1*U1**(-1)*S4**(-2) )
     +
      M2QBH1 = M2QBH1 + ANG4(9)*N*CF**2 * ( 16*S**(-2)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(10)*N*CF**2 * ( 16*S**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(11)*N**2*CF * (  - 16*S**(-2)*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(12)*N*CF**2 * (  - 32*S**(-2)*U1*S4**(-1)*
     +    MS2 + 16*S**(-1)*U1*S4**(-1) - 16*S**(-1)*S4**(-1)*MS2 - 64*
     +    S**(-1)*(T1+U1)**(-1)*MS2 - 8*S**(-1) + 8*S4**(-1) )
     +
      M2QBH1 = M2QBH1 + ANG4(12)*N**2*CF * ( 16*S**(-2)*U1*S4**(-1)*MS2
     +     - 16*S**(-2)*U1*(T1+U1)**(-1)*MS2 - 8*S**(-1)*U1*S4**(-1) + 
     +    8*S**(-1)*S4**(-1)*MS2 + 8*S**(-1)*(T1+U1)**(-1)*MS2 + 4*
     +    S**(-1) - 4*S4**(-1) )
     +
      M2QBH1 = M2QBH1 + ANG4(13)*N**2*CF * ( 16*S**(-2)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(14)*N**2*CF * ( 32*S**(-2)*U1*MS2 + 16*
     +    S**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(15)*N**2*CF * (  - 16*S**(-2)*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(16)*N*CF**2 * (  - 64*S**(-1)*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(16)*N**2*CF * ( 16*S**(-2)*U1*S4**(-1)*MS2
     +     - 16*S**(-2)*U1*(T1+U1)**(-1)*MS2 - 8*S**(-1)*U1*S4**(-1) + 
     +    8*S**(-1)*S4**(-1)*MS2 + 8*S**(-1)*(T1+U1)**(-1)*MS2 + 4*
     +    S**(-1) - 4*S4**(-1) )
     +
      M2QBH1 = M2QBH1 + ANG4(36)*N*CF**2 * ( 16*S**(-1)*MS2**2 )
     +
      M2QBH1 = M2QBH1 + ANG4(37)*N*CF**2 * ( 16*S**(-1)*T1 - 16*S**(-1)
     +    *U1*S4**(-1)*MS2 - 16*S**(-1)*U1 + 32*S**(-1)*S4**(-1)*MS2**2
     +     + 16*S**(-1)*MS2 - 8*T1*S4**(-1) - 32*S4**(-1)*MS2 - 32*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(37)*N**2*CF * (  - 4*S**(-1)*T1 + 8*
     +    S**(-1)*U1*S4**(-1)*MS2 - 8*S**(-1)*U1*(T1+U1)**(-1)*MS2 + 8*
     +    S**(-1)*U1**2*(T1+U1)**(-1) - 16*S**(-1)*S4**(-1)*MS2**2 - 4*
     +    S**(-1)*MS2 + 4*T1*S4**(-1) + 16*S4**(-1)*MS2 + 8*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(38)*N*CF**2 * ( 16*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(38)*N**2*CF * ( 16*S**(-2)*U1**2*MS2 + 16*
     +    S**(-1)*U1*MS2 + 8*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(39)*N*CF**2 * (  - 64*S**(-1)*U1*S4**(-1)*
     +    MS2 - 16*T1*S4**(-1) + 16*U1*S4**(-1) - 32*S4**(-1)*MS2 - 32*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(39)*N**2*CF * ( 16*S**(-2)*U1**2*S4**(-1)*
     +    MS2 + 8*S**(-1)*T1 + 40*S**(-1)*U1*S4**(-1)*MS2 - 8*S**(-1)*
     +    U1*(T1+U1)**(-1)*MS2 - 12*S**(-1)*U1 + 8*S**(-1)*U1**2*
     +    S4**(-1) + 8*S**(-1)*U1**2*(T1+U1)**(-1) + 8*S**(-1)*MS2 + 4*
     +    T1*S4**(-1) + 20*S4**(-1)*MS2 + 8*(T1+U1)**(-1)*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(48)*N*CF**2 * (  - 8*S**(-1)*T1**2 - 8*
     +    S**(-1)*U1**2 - 8*T1 - 8*U1 - 16*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(48)*N**2*CF * ( 4*S**(-1)*T1**2 + 4*
     +    S**(-1)*U1**2 + 4*T1 + 4*U1 + 8*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(49)*N*CF**2 * (  - 32*S**(-1)*T1*U1 - 16*
     +    S**(-1)*T1*MS2 + 16*S**(-1)*T1**2 + 32*S**(-1)*U1**2 - 32*
     +    S**(-1)*U1**3*(T1+U1)**(-1) + 16*T1 - 16*U1*(T1+U1)**(-1)*MS2
     +     - 16*U1 + 16*U1**2*(T1+U1)**(-1) + 16*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(49)*N**2*CF * ( 8*S**(-1)*T1*U1 + 4*
     +    S**(-1)*T1*MS2 - 4*S**(-1)*T1**2 - 8*S**(-1)*U1**2 + 8*
     +    S**(-1)*U1**3*(T1+U1)**(-1) - 4*T1 + 4*U1*(T1+U1)**(-1)*MS2
     +     + 4*U1 - 4*U1**2*(T1+U1)**(-1) - 4*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(50)*N*CF**2 * ( 16*S**(-1)*U1*MS2 + 16*
     +    S**(-1)*U1**2 - 32*S**(-1)*U1**3*(T1+U1)**(-1) - 16*U1*
     +    (T1+U1)**(-1)*MS2 - 16*U1**2*(T1+U1)**(-1) )
     +
      M2QBH1 = M2QBH1 + ANG4(50)*N**2*CF * (  - 8*S**(-1)*U1*MS2 - 8*
     +    S**(-1)*U1**2 + 16*S**(-1)*U1**3*(T1+U1)**(-1) + 8*U1*
     +    (T1+U1)**(-1)*MS2 + 8*U1**2*(T1+U1)**(-1) )
     +
      M2QBH1 = M2QBH1 + ANG4(52)*N*CF**2 * ( 8*S*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(54)*N*CF**2 * ( 8*S**(-1)*T1**2 + 8*
     +    S**(-1)*U1**2 + 32*S**(-1)*U1**3*S4**(-1) - 32*S**(-1)*U1**3*
     +    (T1+U1)**(-1) + 8*T1 + 16*U1*S4**(-1)*MS2 - 16*U1*
     +    (T1+U1)**(-1)*MS2 - 16*U1 + 16*U1**2*S4**(-1) + 16*U1**2*
     +    (T1+U1)**(-1) + 8*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(54)*N**2*CF * (  - 4*S**(-1)*T1*U1 + 4*
     +    S**(-1)*U1**2 - 16*S**(-1)*U1**3*S4**(-1) + 8*S**(-1)*U1**3*
     +    (T1+U1)**(-1) - 8*U1*S4**(-1)*MS2 + 4*U1*(T1+U1)**(-1)*MS2 + 
     +    4*U1 - 8*U1**2*S4**(-1) - 4*U1**2*(T1+U1)**(-1) )
     +
      M2QBH1 = M2QBH1 + ANG4(56)*N*CF**2 * ( 8*S*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(58)*N*CF**2 * ( 8*S**(-1)*T1**2 + 8*
     +    S**(-1)*U1**2 + 32*S**(-1)*U1**3*S4**(-1) - 32*S**(-1)*U1**3*
     +    (T1+U1)**(-1) - 32*T1*U1*S4**(-1) - 16*T1*S4**(-1)*MS2 + 16*
     +    T1**2*S4**(-1) - 16*U1*(T1+U1)**(-1)*MS2 + 24*U1 + 32*U1**2*
     +    S4**(-1) - 16*U1**2*(T1+U1)**(-1) + 24*MS2 )
     +
      M2QBH1 = M2QBH1 + ANG4(58)*N**2*CF * ( 4*S**(-1)*T1*U1 - 4*
     +    S**(-1)*T1**2 - 8*S**(-1)*U1**2 - 8*S**(-1)*U1**3*S4**(-1) + 
     +    16*S**(-1)*U1**3*(T1+U1)**(-1) + 8*T1*U1*S4**(-1) + 4*T1*
     +    S4**(-1)*MS2 - 4*T1**2*S4**(-1) + 8*U1*(T1+U1)**(-1)*MS2 - 8*
     +    U1 - 8*U1**2*S4**(-1) + 8*U1**2*(T1+U1)**(-1) - 8*MS2 )


      M2QBH4 = 0.D0
      M2QBH4 = M2QBH4 + N*CF*(S4+MS2) * (  - 16*M2*TG**(-1)*S**(-1)*U1*
     +    S4**(-2) + 8*M2*TG**(-1)*S**(-1)*S4**(-1) - 16*M2*TG**(-1)*
     +    S4**(-2) + 8*M2*TG**(-1)*S4**(-1)*(S+T1)**(-1) - 16*M2*
     +    S**(-1)*T1*S4**(-2)*(S+U1+M2)**(-1) + 8*M2*S**(-1)*T1*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*M2*S**(-1)*S4**(-2) + 4
     +    *M2*S**(-1)*S4**(-1)*(S+U1+M2)**(-1) - 8*M2*T1*S4**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 4*M2*S4**(-1)*
     +    (S+U1)**(-1)*(S+U1+M2)**(-1) + 4*M2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*M2**2*TG**(-1)*S**(-1)*S4**(-2)
     +     + 8*M2**2*S**(-1)*S4**(-2)*(S+U1+M2)**(-1) + 4*M2**2*
     +    S4**(-1)*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*TG**(-1)
     +    *S**(-1)*U1*S4**(-1) - 8*TG**(-1)*S**(-1)*U1**2*S4**(-2) - 8*
     +    TG**(-1)*S*S4**(-2) + 8*TG**(-1)*S*S4**(-1)*(S+T1)**(-1) - 16
     +    *TG**(-1)*U1*S4**(-2) + 4*S**(-1)*T1*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1*S4**(-2) - 4*S**(-1)*
     +    T1*S4**(-1)*(S+U1+M2)**(-1) )
     +
      M2QBH4 = M2QBH4 + N*CF*(S4+MS2) * ( 8*S**(-1)*T1**2*S4**(-2)*
     +    (S+U1+M2)**(-1) - 4*S**(-1)*T1**2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*U1*S4**(-2) - 4*S**(-1)*
     +    S4**(-1) + 4*T1**2*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S4**(-2) + 8*S4**(-1)*
     +    (S+T1)**(-1) + 4*S4**(-1)*(S+U1)**(-1) - 4*S4**(-1)*
     +    (S+U1+M2)**(-1) )
     +
      M2QBH4 = M2QBH4 + CF**2*(S4+MS2) * ( 8*M2*TG**(-1)*S**(-1)*
     +    T1**(-1)*U1*S4**(-1) + 32*M2*TG**(-1)*S**(-1)*U1*S4**(-2) - 8
     +    *M2*TG**(-1)*S**(-1)*S4**(-1) + 32*M2*TG**(-1)*S4**(-2) - 8*
     +    M2*TG**(-1)*S4**(-1)*(S+T1)**(-1) - 16*M2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2*S**(-1)*T1**(-1)*S4**(-1) + 8*M2*S**(-1)*
     +    T1*U1**(-1)*S4**(-1)*(U1+M2)*SYMBX + 32*M2*S**(-1)*T1*
     +    S4**(-2)*(S+U1+M2)**(-1) - 8*M2*S**(-1)*T1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*M2*S**(-1)*U1**(-1)*S4**(-1) + 32
     +    *M2*S**(-1)*S4**(-2) - 8*M2*S**(-1)*S4**(-1)*(S+U1+M2)**(-1)
     +     + 8*M2*S*S4**(-1)*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1)*SYMBX + 8*
     +    M2*T1*S4**(-1)*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1)*SYMBX + 16*M2*
     +    T1*S4**(-1)*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*M2*
     +    U1**(-1)*S4**(-1)*(U1+M2)*SYMBX + 8*M2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*M2**2*TG**(-1)*S**(-1)*T1**(-1)*
     +    S4**(-1) + 16*M2**2*TG**(-1)*S**(-1)*S4**(-2) - 8*M2**2*
     +    S**(-1)*U1**(-1)*S4**(-1)*(U1+M2)*SYMBX )
     +
      M2QBH4 = M2QBH4 + CF**2*(S4+MS2) * (  - 16*M2**2*S**(-1)*S4**(-2)
     +    *(S+U1+M2)**(-1) - 8*M2**2*S4**(-1)*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1)*SYMBX - 8*M2**2*S4**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*TG**(-1)*S**(-1)*
     +    U1*S4**(-1) + 16*TG**(-1)*S**(-1)*U1**2*S4**(-2) + 16*
     +    TG**(-1)*S*S4**(-2) - 8*TG**(-1)*S*S4**(-1)*(S+T1)**(-1) - 16
     +    *TG**(-1)*T1**(-1)*S4**(-1)*MS2 + 32*TG**(-1)*U1*S4**(-2) - 
     +    16*TG**(-1)*S4**(-1)*(S+U1)**(-1)*MS2 - 8*TG**(-1)*S4**(-1)
     +     + 8*S**(-1)*T1**(-1)*U1*S4**(-1) - 8*S**(-1)*T1*U1**(-1)*
     +    S4**(-1) - 8*S**(-1)*T1*U1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     - 16*S**(-1)*T1*S4**(-2) + 8*S**(-1)*T1*S4**(-1)*
     +    (S+U1+M2)**(-1) - 16*S**(-1)*T1**2*S4**(-2)*(S+U1+M2)**(-1)
     +     + 16*S**(-1)*U1*S4**(-2) + 8*S**(-1)*S4**(-1) + 16*S*
     +    S4**(-1)*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1)*MS2*SYMBX - 8*T1*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*T1**2*S4**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QBH4 = M2QBH4 + CF**2*(S4+MS2) * ( 16*U1**(-1)*S4**(-1)*(U1+M2)
     +    *MS2*SYMBX - 8*U1**(-1)*S4**(-1) + 16*S4**(-2) - 8*S4**(-1)*
     +    (S+T1)**(-1) - 16*S4**(-1)*(S+U1)**(-1) + 16*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + CF**2 * ( 8*M2*TG**(-1)*S**(-1) - 8*M2*TG**(-1)
     +    *S4**(-1) + 24*TG**(-1)*S**(-1)*U1*S4**(-1)*MS2 - 32*TG**(-1)
     +    *S**(-1)*U1**2*S4**(-2)*MS2 - 8*TG**(-1)*S**(-1)*U1**2*
     +    S4**(-1) + 8*TG**(-1)*S**(-1)*MS2 - 32*TG**(-1)*U1*S4**(-2)*
     +    MS2 - 8*TG**(-1)*U1*S4**(-1) - 32*TG**(-1)*S4**(-2)*MS2**2 - 
     +    16*TG**(-1)*S4**(-1)*MS2 + 8*S**(-1) - 8*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + COLO1(9)*CF**2*(S4+MS2) * (  - 16*M2*TG**(-1)*
     +    S4**(-2)*(S+U1)**(-1)*MS2 + 16*M2*S**(-1)*S4**(-2) - 16*M2*
     +    S4**(-2)*(S+U1)**(-1) + 16*M2**2*TG**(-1)*S**(-1)*S4**(-2) - 
     +    16*M2**2*TG**(-1)*S4**(-2)*(S+U1)**(-1) + 16*TG**(-1)*S**(-1)
     +    *U1**2*S4**(-2) - 16*TG**(-1)*S*T1**(-1)*S4**(-2)*MS2 - 16*
     +    TG**(-1)*T1**(-1)*U1*S4**(-2)*MS2 + 16*TG**(-1)*U1*S4**(-2)
     +     - 16*S**(-1)*T1*U1**2*S4**(-2)*(M2*(S+T1)+T1*U1)**(-1) + 16*
     +    S**(-1)*T1*S4**(-2) - 16*S**(-1)*T1**3*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*S*T1*U1**(-1)*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-1)*MS2 - 16*S*T1*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S**2*U1**(-1)*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-1)*MS2 - 16*T1*S4**(-2)*(S+U1)**(-1) + 
     +    16*T1**2*U1**(-1)*S4**(-2)*(M2*(S+T1)+T1*U1)**(-1)*MS2 - 32*
     +    T1**2*S4**(-2)*(M2*(S+T1)+T1*U1)**(-1) + 16*U1*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-1)*MS2 - 16*S4**(-2)*(S+U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(10)*CF**2 * (  - 16*S**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(12)*N*CF * ( 2*M2*TG**(-1)*S**(-1) - 2*M2*
     +    TG**(-1)*S4**(-1) - 8*TG**(-1)*S**(-1)*U1*S4**(-1)*MS2 - 2*
     +    TG**(-1)*S**(-1)*U1 + 8*TG**(-1)*S**(-1)*MS2 + 2*TG**(-1)*U1*
     +    S4**(-1) - 4*TG**(-1)*S4**(-1)*MS2 + 8*S**(-1)*(T1+U1)**(-1)*
     +    MS2 + 2*S**(-1) - 2*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(12)*CF**2 * (  - 4*M2*TG**(-1)*S**(-1) + 4
     +    *M2*TG**(-1)*S4**(-1) + 16*TG**(-1)*S**(-1)*U1*S4**(-1)*MS2
     +     + 4*TG**(-1)*S**(-1)*U1 - 16*TG**(-1)*S**(-1)*MS2 - 4*
     +    TG**(-1)*U1*S4**(-1) + 8*TG**(-1)*S4**(-1)*MS2 - 4*S**(-1) + 
     +    4*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(16)*N*CF * ( 2*M2*TG**(-1)*S**(-1) - 2*M2*
     +    TG**(-1)*S4**(-1) - 8*TG**(-1)*S**(-1)*U1*S4**(-1)*MS2 - 2*
     +    TG**(-1)*S**(-1)*U1 + 8*TG**(-1)*S**(-1)*MS2 + 2*TG**(-1)*U1*
     +    S4**(-1) - 4*TG**(-1)*S4**(-1)*MS2 + 8*S**(-1)*(T1+U1)**(-1)*
     +    MS2 + 2*S**(-1) - 2*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(36)*CF**2 * (  - 16*M2*S**(-1)*MS2 - 16*
     +    MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(37)*N*CF * (  - 2*M2*TG**(-1)*S**(-1)*U1
     +     - 10*M2*TG**(-1)*S**(-1)*MS2 + 2*M2*TG**(-1)*U1*S4**(-1) + 8
     +    *M2*TG**(-1)*S4**(-1)*MS2 + 2*M2*TG**(-1)*(T1+U1)**(-1)*MS2
     +     + 8*M2*S**(-1)*(T1+U1)**(-1)*MS2 - 2*M2*S**(-1) + 2*M2*
     +    S4**(-1) - 2*M2**2*TG**(-1)*S**(-1) + 2*M2**2*TG**(-1)*
     +    S4**(-1) - 2*TG**(-1)*S**(-1)*U1*MS2 + 4*TG**(-1)*S**(-1)*
     +    U1**2 + 4*TG**(-1)*U1*S4**(-1)*MS2 + 2*TG**(-1)*U1*
     +    (T1+U1)**(-1)*MS2 + 8*TG**(-1)*S4**(-1)*MS2**2 - 6*TG**(-1)*
     +    MS2 - 2*S**(-1)*T1 - 8*S**(-1)*U1*S4**(-1)*MS2 + 8*S**(-1)*U1
     +    *(T1+U1)**(-1)*MS2 - 2*S**(-1)*U1 - 2*S**(-1)*MS2 + 4*U1*
     +    S4**(-1) + 4*S4**(-1)*MS2 - 2*(T1+U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(37)*CF**2 * ( 4*M2*TG**(-1)*S**(-1)*U1 + 4
     +    *M2*TG**(-1)*S**(-1)*MS2 - 4*M2*TG**(-1)*U1*S4**(-1) - 16*M2*
     +    TG**(-1)*S4**(-1)*MS2 - 4*M2*TG**(-1)*(T1+U1)**(-1)*MS2 - 4*
     +    M2*S4**(-1) - 4*M2**2*TG**(-1)*S4**(-1) + 4*TG**(-1)*S**(-1)*
     +    U1*MS2 - 4*TG**(-1)*S**(-1)*U1**2 - 8*TG**(-1)*U1*S4**(-1)*
     +    MS2 - 4*TG**(-1)*U1*(T1+U1)**(-1)*MS2 - 16*TG**(-1)*S4**(-1)*
     +    MS2**2 + 4*TG**(-1)*MS2 + 4*S**(-1)*T1 + 16*S**(-1)*U1*
     +    S4**(-1)*MS2 + 4*S**(-1)*MS2 - 8*U1*S4**(-1) - 8*S4**(-1)*MS2
     +     + 12*(T1+U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(39)*N*CF * ( 2*M2*TG**(-1)*S**(-1)*U1 + 2*
     +    M2*TG**(-1)*U1*S4**(-1) + 10*M2*TG**(-1)*S4**(-1)*MS2 + 2*M2*
     +    TG**(-1)*(T1+U1)**(-1)*MS2 + 8*M2*S**(-1)*(T1+U1)**(-1)*MS2
     +     + 2*M2*S**(-1) + 2*M2*S4**(-1) + 2*M2**2*TG**(-1)*S**(-1) + 
     +    2*M2**2*TG**(-1)*S4**(-1) + 8*TG**(-1)*S**(-1)*U1*MS2 - 8*
     +    TG**(-1)*S**(-1)*U1**2*S4**(-1)*MS2 - 10*TG**(-1)*U1*S4**(-1)
     +    *MS2 + 2*TG**(-1)*U1*(T1+U1)**(-1)*MS2 - 4*TG**(-1)*U1**2*
     +    S4**(-1) + 2*TG**(-1)*MS2 + 4*S**(-1)*T1 + 8*S**(-1)*U1*
     +    (T1+U1)**(-1)*MS2 + 2*T1*S4**(-1) + 2*U1*S4**(-1) + 10*
     +    S4**(-1)*MS2 - 2*(T1+U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(39)*CF**2 * (  - 8*M2*TG**(-1)*S4**(-1)*
     +    MS2 - 4*M2*TG**(-1)*(T1+U1)**(-1)*MS2 - 4*M2*S4**(-1) - 4*
     +    M2**2*TG**(-1)*S4**(-1) + 8*TG**(-1)*U1*S4**(-1)*MS2 - 4*
     +    TG**(-1)*U1*(T1+U1)**(-1)*MS2 + 4*TG**(-1)*U1**2*S4**(-1) - 4
     +    *TG**(-1)*MS2 - 4*T1*S4**(-1) - 8*S4**(-1)*MS2 + 12*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(48)*N*CF * (  - 2*M2*TG**(-1)*S + 2*M2*S*
     +    (S+U1+M2)**(-1) - 4*M2 - 2*M2**2*TG**(-1) + 2*M2**2*
     +    (S+U1+M2)**(-1) - 2*TG**(-1)*S*U1 - 4*TG**(-1)*S*MS2 - 2*
     +    TG**(-1)*U1**2 + 2*S*T1*(S+U1+M2)**(-1) + 4*S*(S+U1+M2)**(-1)
     +    *MS2 - 2*S - 2*T1 + 2*T1**2*(S+U1+M2)**(-1) + 2*U1 )
     +
      M2QBH4 = M2QBH4 + ANG4(48)*CF**2 * ( 4*M2*TG**(-1)*S - 4*M2*S*
     +    (S+U1+M2)**(-1) + 8*M2 + 4*M2**2*TG**(-1) - 4*M2**2*
     +    (S+U1+M2)**(-1) + 4*TG**(-1)*S*U1 + 8*TG**(-1)*S*MS2 + 4*
     +    TG**(-1)*U1**2 - 4*S*T1*(S+U1+M2)**(-1) - 8*S*(S+U1+M2)**(-1)
     +    *MS2 + 4*S + 4*T1 - 4*T1**2*(S+U1+M2)**(-1) - 4*U1 )
     +
      M2QBH4 = M2QBH4 + ANG4(49)*N*CF * ( 2*M2*TG**(-1)*S**(-1)*U1*MS2
     +     + 2*M2*TG**(-1)*S - 4*M2*TG**(-1)*U1 - 4*M2*TG**(-1)*MS2 - 2
     +    *M2*S**(-1)*T1 + 2*M2*S**(-1)*U1 + 2*M2*S**(-1)*MS2 + 2*M2**2
     +    *TG**(-1)*S**(-1)*U1 + 2*M2**2*TG**(-1)*S**(-1)*MS2 - 2*M2**2
     +    *S**(-1) - 2*M2**3*TG**(-1)*S**(-1) - 2*TG**(-1)*S*U1*
     +    (T1+U1)**(-1)*MS2 - 2*TG**(-1)*S*U1 + 2*TG**(-1)*S*U1**2*
     +    (T1+U1)**(-1) + 2*TG**(-1)*S*MS2 + 4*TG**(-1)*U1**2 - 4*
     +    TG**(-1)*U1**3*(T1+U1)**(-1) + 2*S**(-1)*T1*U1 + 2*S**(-1)*T1
     +    *MS2 - 2*S**(-1)*T1**2 + 2*S**(-1)*U1*MS2 + 2*S - 4*U1 - 4*
     +    MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(49)*CF**2 * (  - 4*M2*TG**(-1)*S**(-1)*U1*
     +    MS2 - 4*M2*TG**(-1)*S + 8*M2*TG**(-1)*U1 + 8*M2*TG**(-1)*MS2
     +     + 4*M2*S**(-1)*T1 - 4*M2*S**(-1)*U1 - 4*M2*S**(-1)*MS2 - 4*
     +    M2**2*TG**(-1)*S**(-1)*U1 - 4*M2**2*TG**(-1)*S**(-1)*MS2 + 4*
     +    M2**2*S**(-1) + 4*M2**3*TG**(-1)*S**(-1) + 4*TG**(-1)*S*U1*
     +    (T1+U1)**(-1)*MS2 + 4*TG**(-1)*S*U1 - 4*TG**(-1)*S*U1**2*
     +    (T1+U1)**(-1) - 4*TG**(-1)*S*MS2 - 8*TG**(-1)*U1**2 + 8*
     +    TG**(-1)*U1**3*(T1+U1)**(-1) - 4*S**(-1)*T1*U1 - 4*S**(-1)*T1
     +    *MS2 + 4*S**(-1)*T1**2 - 4*S**(-1)*U1*MS2 - 4*S + 8*U1 + 8*
     +    MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(50)*N*CF * (  - 4*M2*TG**(-1)*S**(-1)*U1*
     +    MS2 - 4*M2*TG**(-1)*S**(-1)*U1**2 - 4*M2*S*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1)*MS2 + 12*M2*S*(T1+U1)**(-1) - 8*M2*S*
     +    (S+U1+M2)**(-1) - 16*M2*S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1) - 
     +    8*M2*U1*(T1+U1)**(-1) + 4*M2*(S+U1+M2)**(-1)*MS2 + 4*M2 - 20*
     +    M2**2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 8*M2**2*(T1+U1)**(-1)
     +     - 4*M2**2*(S+U1+M2)**(-1) - 8*M2**3*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) - 4*TG**(-1)*S**(-1)*U1**2*MS2 + 4*TG**(-1)*
     +    S**(-1)*U1**3 + 4*TG**(-1)*U1*MS2 + 4*TG**(-1)*U1**2 - 4*
     +    S**(-1)*U1*MS2 - 4*S**(-1)*U1**2 - 4*S*U1*(T1+U1)**(-1) + 4*S
     +    *(T1+U1)**(-1)*MS2 + 4*S*(S+U1+M2)**(-1)*MS2 + 4*S - 4*S**2*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 + 4*S**2*(T1+U1)**(-1) - 4*
     +    S**2*(S+U1+M2)**(-1) - 4*S**3*(T1+U1)**(-1)*(S+U1+M2)**(-1)
     +     - 4*U1 + 8*U1**2*(T1+U1)**(-1) - 4*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(50)*CF**2 * ( 4*M2*TG**(-1)*S**(-1)*U1*MS2
     +     + 4*M2*TG**(-1)*S**(-1)*U1**2 + 4*M2*S*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1)*MS2 - 12*M2*S*(T1+U1)**(-1) + 8*M2*S*
     +    (S+U1+M2)**(-1) + 16*M2*S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 
     +    8*M2*U1*(T1+U1)**(-1) - 4*M2*(S+U1+M2)**(-1)*MS2 - 4*M2 + 20*
     +    M2**2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1) - 8*M2**2*(T1+U1)**(-1)
     +     + 4*M2**2*(S+U1+M2)**(-1) + 8*M2**3*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) + 4*TG**(-1)*S**(-1)*U1**2*MS2 - 4*TG**(-1)*
     +    S**(-1)*U1**3 - 4*TG**(-1)*U1*MS2 - 4*TG**(-1)*U1**2 + 4*
     +    S**(-1)*U1*MS2 + 4*S**(-1)*U1**2 + 4*S*U1*(T1+U1)**(-1) - 4*S
     +    *(T1+U1)**(-1)*MS2 - 4*S*(S+U1+M2)**(-1)*MS2 - 4*S + 4*S**2*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 - 4*S**2*(T1+U1)**(-1) + 4*
     +    S**2*(S+U1+M2)**(-1) + 4*S**3*(T1+U1)**(-1)*(S+U1+M2)**(-1)
     +     + 4*U1 - 8*U1**2*(T1+U1)**(-1) + 4*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(54)*N*CF * (  - 4*M2*TG**(-1)*U1*S4**(-1)*
     +    MS2 + 2*M2*TG**(-1)*U1 - 4*M2*TG**(-1)*U1**2*S4**(-1) - 2*
     +    TG**(-1)*S*U1*(T1+U1)**(-1)*MS2 - 2*TG**(-1)*S*U1 + 2*
     +    TG**(-1)*S*U1**2*(T1+U1)**(-1) + 4*TG**(-1)*U1*MS2 - 4*
     +    TG**(-1)*U1**2*S4**(-1)*MS2 + 2*TG**(-1)*U1**2 + 4*TG**(-1)*
     +    U1**3*S4**(-1) - 4*TG**(-1)*U1**3*(T1+U1)**(-1) - 4*U1*
     +    S4**(-1)*MS2 + 2*U1 - 4*U1**2*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(54)*CF**2 * ( 4*M2*TG**(-1)*S*(U1+M2)*MS2*
     +    SYMBX - 4*M2*TG**(-1)*S + 4*M2*TG**(-1)*U1*S4**(-1)*MS2 + 4*
     +    M2*TG**(-1)*U1 + 4*M2*TG**(-1)*U1**2*S4**(-1) + 4*M2*S*
     +    (U1+M2)*SYMBX + 4*M2*T1*(U1+M2)*SYMBX - 4*M2 + 4*M2**2*
     +    TG**(-1)*S*(U1+M2)*SYMBX - 8*M2**2*TG**(-1) + 4*M2**2*(U1+M2)
     +    *SYMBX + 8*M2**3*TG**(-1)*(U1+M2)*SYMBX + 4*TG**(-1)*S*U1*
     +    (T1+U1)**(-1)*MS2 + 4*TG**(-1)*S*U1 - 4*TG**(-1)*S*U1**2*
     +    (T1+U1)**(-1) - 4*TG**(-1)*S*MS2 - 4*TG**(-1)*U1*MS2 + 4*
     +    TG**(-1)*U1**2*S4**(-1)*MS2 - 8*TG**(-1)*U1**2 - 4*TG**(-1)*
     +    U1**3*S4**(-1) + 8*TG**(-1)*U1**3*(T1+U1)**(-1) + 8*S*(U1+M2)
     +    *MS2*SYMBX - 4*S - 4*T1 + 4*U1*S4**(-1)*MS2 + 4*U1**2*
     +    S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(58)*N*CF * ( 2*M2*TG**(-1)*S*
     +    (S+U1+M2)**(-1)*MS2 + 2*M2*TG**(-1)*U1*S4**(-1)*MS2 - 2*M2*
     +    TG**(-1)*MS2 - 4*M2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 + 12*
     +    M2*S*(T1+U1)**(-1) - 10*M2*S*(S+U1+M2)**(-1) - 16*M2*S**2*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) - 2*M2*T1*S4**(-1) - 2*M2*T1*
     +    (S+U1+M2)**(-1) + 2*M2*U1*S4**(-1) - 8*M2*U1*(T1+U1)**(-1) + 
     +    2*M2*S4**(-1)*MS2 + 6*M2 + 2*M2**2*TG**(-1)*S*(S+U1+M2)**(-1)
     +     + 2*M2**2*TG**(-1)*U1*S4**(-1) + 2*M2**2*TG**(-1)*S4**(-1)*
     +    MS2 - 2*M2**2*TG**(-1) - 20*M2**2*S*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) - 2*M2**2*S4**(-1) + 8*M2**2*(T1+U1)**(-1) - 
     +    4*M2**2*(S+U1+M2)**(-1) - 2*M2**3*TG**(-1)*S4**(-1) + 4*M2**3
     +    *TG**(-1)*(S+U1+M2)**(-1) - 8*M2**3*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) - 4*S*T1*(S+U1+M2)**(-1) - 4*S*U1*
     +    (T1+U1)**(-1) + 4*S*(T1+U1)**(-1)*MS2 - 2*S*(S+U1+M2)**(-1)*
     +    MS2 + 4*S - 4*S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 + 4*S**2
     +    *(T1+U1)**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(58)*N*CF * (  - 4*S**2*(S+U1+M2)**(-1) - 4
     +    *S**3*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 2*T1*U1*S4**(-1) + 2*T1
     +    *S4**(-1)*MS2 + 4*T1 - 2*T1**2*S4**(-1) - 2*T1**2*
     +    (S+U1+M2)**(-1) + 2*U1*S4**(-1)*MS2 - 8*U1 + 8*U1**2*
     +    (T1+U1)**(-1) - 2*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(58)*CF**2 * (  - 4*M2*TG**(-1)*U1*S4**(-1)
     +    *MS2 + 4*M2*TG**(-1)*U1 + 4*M2*TG**(-1)*MS2 + 4*M2*S*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 - 12*M2*S*(T1+U1)**(-1) + 
     +    12*M2*S*(S+U1+M2)**(-1) + 16*M2*S**2*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) + 4*M2*T1*S4**(-1) + 4*M2*T1*(S+U1+M2)**(-1)
     +     - 4*M2*U1*S4**(-1) + 8*M2*U1*(T1+U1)**(-1) - 4*M2*S4**(-1)*
     +    MS2 - 8*M2 - 4*M2**2*TG**(-1)*U1*S4**(-1) - 4*M2**2*TG**(-1)*
     +    S4**(-1)*MS2 + 20*M2**2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 4*
     +    M2**2*S4**(-1) - 8*M2**2*(T1+U1)**(-1) + 8*M2**2*
     +    (S+U1+M2)**(-1) + 4*M2**3*TG**(-1)*S4**(-1) + 8*M2**3*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) - 8*TG**(-1)*S*MS2 + 4*S*T1*
     +    (S+U1+M2)**(-1) + 4*S*U1*(T1+U1)**(-1) - 4*S*(T1+U1)**(-1)*
     +    MS2 + 4*S*(S+U1+M2)**(-1)*MS2 - 4*S + 4*S**2*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1)*MS2 - 4*S**2*(T1+U1)**(-1) + 4*S**2*
     +    (S+U1+M2)**(-1) + 4*S**3*(T1+U1)**(-1)*(S+U1+M2)**(-1) - 4*T1
     +    *U1*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(58)*CF**2 * (  - 4*T1*S4**(-1)*MS2 - 4*T1
     +     + 4*T1**2*S4**(-1) + 4*T1**2*(S+U1+M2)**(-1) - 4*U1*S4**(-1)
     +    *MS2 + 12*U1 - 8*U1**2*(T1+U1)**(-1) + 4*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(68)*N*CF * (  - 2*M2*TG**(-1)*S4**(-1) - 2
     +    *TG**(-1)*S**(-1)*U1*S4**(-1)*MS2 - 2*TG**(-1)*S**(-1)*U1 + 2
     +    *TG**(-1)*S**(-1)*U1**2*S4**(-1) + 2*TG**(-1)*U1*S4**(-1) - 2
     +    *TG**(-1)*S4**(-1)*MS2 - 2*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(68)*CF**2 * ( 4*M2*TG**(-1)*S4**(-1) + 4*
     +    TG**(-1)*S**(-1)*U1*S4**(-1)*MS2 + 4*TG**(-1)*S**(-1)*U1 - 4*
     +    TG**(-1)*S**(-1)*U1**2*S4**(-1) - 4*TG**(-1)*U1*S4**(-1) + 4*
     +    TG**(-1)*S4**(-1)*MS2 + 4*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(74)*CF**2 * (  - 16*M2*MS2 - 16*M2**2*
     +    S**(-1)*MS2 - 16*MS2**2 )
     +
      M2QBH4 = M2QBH4 + ANG4(75)*N*CF * ( 4*M2*TG**(-1)*S*(T1+U1)**(-1)
     +    *MS2 - 2*M2*TG**(-1)*S + 2*M2*TG**(-1)*U1*(T1+U1)**(-1)*MS2
     +     + 4*M2*TG**(-1)*U1 - 4*M2*TG**(-1)*U1**2*(T1+U1)**(-1) - 8*
     +    M2*TG**(-1)*MS2 - 8*M2*S**(-1)*U1*S4**(-1)*MS2 + 8*M2*S**(-1)
     +    *U1*(T1+U1)**(-1)*MS2 - 4*M2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1)*
     +    MS2 + 12*M2*S*(T1+U1)**(-1) - 8*M2*S*(S+U1+M2)**(-1) - 16*M2*
     +    S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1) - 2*M2*T1*S4**(-1) + 2*M2*
     +    U1*S4**(-1) - 8*M2*U1*(T1+U1)**(-1) - 4*M2*S4**(-1)*MS2 - 2*
     +    M2*(T1+U1)**(-1)*MS2 + 4*M2*(S+U1+M2)**(-1)*MS2 + 2*M2 - 8*
     +    M2**2*TG**(-1)*S**(-1)*MS2 + 2*M2**2*TG**(-1)*(T1+U1)**(-1)*
     +    MS2 - 2*M2**2*TG**(-1) + 8*M2**2*S**(-1)*(T1+U1)**(-1)*MS2 - 
     +    20*M2**2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 8*M2**2*
     +    (T1+U1)**(-1) - 4*M2**2*(S+U1+M2)**(-1) - 8*M2**3*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) + 6*TG**(-1)*S*U1*(T1+U1)**(-1)
     +    *MS2 + 2*TG**(-1)*S*U1 - 2*TG**(-1)*S*U1**2*(T1+U1)**(-1) - 6
     +    *TG**(-1)*S*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(75)*N*CF * (  - 8*TG**(-1)*MS2**2 - 4*S*U1
     +    *(T1+U1)**(-1) + 4*S*(T1+U1)**(-1)*MS2 + 4*S*(S+U1+M2)**(-1)*
     +    MS2 + 2*S - 4*S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 + 4*S**2
     +    *(T1+U1)**(-1) - 4*S**2*(S+U1+M2)**(-1) - 4*S**3*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) + 2*T1*U1*S4**(-1) + 8*T1*
     +    S4**(-1)*MS2 - 2*T1 + 2*T1**2*S4**(-1) + 4*U1*S4**(-1)*MS2 + 
     +    4*U1*(T1+U1)**(-1)*MS2 + 8*S4**(-1)*MS2**2 - 8*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(75)*CF**2 * ( 4*M2*TG**(-1)*S**(-1)*U1*MS2
     +     - 8*M2*TG**(-1)*S*(T1+U1)**(-1)*MS2 + 4*M2*TG**(-1)*S - 4*M2
     +    *TG**(-1)*U1*(T1+U1)**(-1)*MS2 - 8*M2*TG**(-1)*U1 + 8*M2*
     +    TG**(-1)*U1**2*(T1+U1)**(-1) - 4*M2*TG**(-1)*MS2 + 16*M2*
     +    S**(-1)*U1*S4**(-1)*MS2 + 4*M2*S**(-1)*MS2 + 4*M2*S*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 - 12*M2*S*(T1+U1)**(-1) + 8
     +    *M2*S*(S+U1+M2)**(-1) + 16*M2*S**2*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) + 4*M2*T1*S4**(-1) - 4*M2*U1*S4**(-1) + 8*M2*
     +    U1*(T1+U1)**(-1) + 8*M2*S4**(-1)*MS2 + 12*M2*(T1+U1)**(-1)*
     +    MS2 - 4*M2*(S+U1+M2)**(-1)*MS2 - 4*M2 + 4*M2**2*TG**(-1)*
     +    S**(-1)*U1 + 4*M2**2*TG**(-1)*S**(-1)*MS2 - 4*M2**2*TG**(-1)*
     +    (T1+U1)**(-1)*MS2 - 4*M2**2*S**(-1) + 20*M2**2*S*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) - 8*M2**2*(T1+U1)**(-1) + 4*
     +    M2**2*(S+U1+M2)**(-1) - 4*M2**3*TG**(-1)*S**(-1) + 8*M2**3*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) - 12*TG**(-1)*S*U1*
     +    (T1+U1)**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(75)*CF**2 * (  - 4*TG**(-1)*S*U1 + 4*
     +    TG**(-1)*S*U1**2*(T1+U1)**(-1) + 12*TG**(-1)*S*MS2 + 4*S*U1*
     +    (T1+U1)**(-1) - 4*S*(T1+U1)**(-1)*MS2 - 4*S*(S+U1+M2)**(-1)*
     +    MS2 + 4*S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 - 4*S**2*
     +    (T1+U1)**(-1) + 4*S**2*(S+U1+M2)**(-1) + 4*S**3*(T1+U1)**(-1)
     +    *(S+U1+M2)**(-1) - 4*T1*U1*S4**(-1) - 16*T1*S4**(-1)*MS2 + 4*
     +    T1 - 4*T1**2*S4**(-1) - 8*U1*S4**(-1)*MS2 - 16*S4**(-1)*
     +    MS2**2 + 4*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(76)*N*CF * ( 4*M2*TG**(-1)*S*(T1+U1)**(-1)
     +    *MS2 + 2*M2*TG**(-1)*S*(S+U1+M2)**(-1)*MS2 + 2*M2*TG**(-1)*U1
     +    *(T1+U1)**(-1)*MS2 + 4*M2*TG**(-1)*U1 - 4*M2*TG**(-1)*U1**2*
     +    (T1+U1)**(-1) + 6*M2*TG**(-1)*MS2 + 2*M2*S**(-1)*T1 + 8*M2*
     +    S**(-1)*U1*(T1+U1)**(-1)*MS2 - 2*M2*S**(-1)*U1 - 4*M2*S*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 + 12*M2*S*(T1+U1)**(-1) - 
     +    10*M2*S*(S+U1+M2)**(-1) - 16*M2*S**2*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) - 2*M2*T1*(S+U1+M2)**(-1) - 8*M2*U1*
     +    (T1+U1)**(-1) - 2*M2*(T1+U1)**(-1)*MS2 + 8*M2 + 2*M2**2*
     +    TG**(-1)*S*(S+U1+M2)**(-1) + 2*M2**2*TG**(-1)*(T1+U1)**(-1)*
     +    MS2 + 8*M2**2*S**(-1)*(T1+U1)**(-1)*MS2 - 20*M2**2*S*
     +    (T1+U1)**(-1)*(S+U1+M2)**(-1) + 8*M2**2*(T1+U1)**(-1) - 4*
     +    M2**2*(S+U1+M2)**(-1) + 4*M2**3*TG**(-1)*(S+U1+M2)**(-1) - 8*
     +    M2**3*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 6*TG**(-1)*S*U1*
     +    (T1+U1)**(-1)*MS2 + 2*TG**(-1)*S*U1 - 2*TG**(-1)*S*U1**2*
     +    (T1+U1)**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(76)*N*CF * (  - 4*TG**(-1)*S*MS2 + 2*
     +    S**(-1)*T1*U1 + 2*S**(-1)*T1**2 - 4*S*T1*(S+U1+M2)**(-1) - 4*
     +    S*U1*(T1+U1)**(-1) + 4*S*(T1+U1)**(-1)*MS2 - 2*S*
     +    (S+U1+M2)**(-1)*MS2 + 4*S - 4*S**2*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1)*MS2 + 4*S**2*(T1+U1)**(-1) - 4*S**2*
     +    (S+U1+M2)**(-1) - 4*S**3*(T1+U1)**(-1)*(S+U1+M2)**(-1) + 4*T1
     +     - 2*T1**2*(S+U1+M2)**(-1) + 4*U1*(T1+U1)**(-1)*MS2 + 2*U1 - 
     +    2*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(76)*CF**2 * (  - 8*M2*TG**(-1)*S*
     +    (T1+U1)**(-1)*MS2 - 4*M2*TG**(-1)*S*(U1+M2)*MS2*SYMBX + 4*M2*
     +    TG**(-1)*S - 4*M2*TG**(-1)*U1*(T1+U1)**(-1)*MS2 - 8*M2*
     +    TG**(-1)*U1 + 8*M2*TG**(-1)*U1**2*(T1+U1)**(-1) + 4*M2*
     +    TG**(-1)*MS2 + 4*M2*S*(T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 - 12*
     +    M2*S*(T1+U1)**(-1) - 4*M2*S*(U1+M2)*SYMBX + 12*M2*S*
     +    (S+U1+M2)**(-1) + 16*M2*S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1) - 
     +    4*M2*T1*(U1+M2)*SYMBX + 4*M2*T1*(S+U1+M2)**(-1) + 8*M2*U1*
     +    (T1+U1)**(-1) + 12*M2*(T1+U1)**(-1)*MS2 - 4*M2 - 4*M2**2*
     +    TG**(-1)*S*(U1+M2)*SYMBX - 4*M2**2*TG**(-1)*(T1+U1)**(-1)*MS2
     +     + 8*M2**2*TG**(-1) + 20*M2**2*S*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) - 8*M2**2*(T1+U1)**(-1) - 4*M2**2*(U1+M2)*
     +    SYMBX + 8*M2**2*(S+U1+M2)**(-1) - 8*M2**3*TG**(-1)*(U1+M2)*
     +    SYMBX + 8*M2**3*(T1+U1)**(-1)*(S+U1+M2)**(-1) - 12*TG**(-1)*S
     +    *U1*(T1+U1)**(-1)*MS2 - 4*TG**(-1)*S*U1 + 4*TG**(-1)*S*U1**2*
     +    (T1+U1)**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(76)*CF**2 * ( 12*TG**(-1)*S*MS2 + 4*S*T1*
     +    (S+U1+M2)**(-1) + 4*S*U1*(T1+U1)**(-1) - 4*S*(T1+U1)**(-1)*
     +    MS2 - 8*S*(U1+M2)*MS2*SYMBX + 4*S*(S+U1+M2)**(-1)*MS2 + 4*
     +    S**2*(T1+U1)**(-1)*(S+U1+M2)**(-1)*MS2 - 4*S**2*(T1+U1)**(-1)
     +     + 4*S**2*(S+U1+M2)**(-1) + 4*S**3*(T1+U1)**(-1)*
     +    (S+U1+M2)**(-1) + 4*T1 + 4*T1**2*(S+U1+M2)**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(78)*N*CF * (  - 2*M2*TG**(-1)*S4**(-1) - 2
     +    *TG**(-1)*S**(-1)*U1*S4**(-1)*MS2 - 2*TG**(-1)*S**(-1)*U1 + 2
     +    *TG**(-1)*S**(-1)*U1**2*S4**(-1) + 2*TG**(-1)*U1*S4**(-1) - 2
     +    *TG**(-1)*S4**(-1)*MS2 - 2*S4**(-1) )
     +
      M2QBH4 = M2QBH4 + ANG4(80)*N*CF * ( 2 + 2*M2*TG**(-1)*S**(-1)*U1*
     +    S4**(-1)*MS2 - 2*M2*TG**(-1)*S**(-1)*U1**2*S4**(-1) + 2*M2*
     +    TG**(-1)*S**(-1)*MS2 - 2*M2*TG**(-1)*U1*S4**(-1) - 4*M2*
     +    TG**(-1)*S4**(-1)*MS2 + 2*M2*TG**(-1)*(S+U1+M2)**(-1)*MS2 + 2
     +    *M2*TG**(-1) + 2*M2*S**(-1) + 2*M2*(S+U1+M2)**(-1) + 2*M2**2*
     +    TG**(-1)*S**(-1) + 2*M2**2*TG**(-1)*(S+U1+M2)**(-1) + 2*
     +    TG**(-1)*S**(-1)*U1*MS2 - 4*TG**(-1)*S**(-1)*U1**2*S4**(-1)*
     +    MS2 - 4*TG**(-1)*U1*S4**(-1)*MS2 - 8*TG**(-1)*S4**(-1)*MS2**2
     +     + 2*TG**(-1)*MS2 + 4*S**(-1)*U1 - 4*S**(-1)*U1**2*S4**(-1)
     +     + 2*T1*S4**(-1) - 4*U1*S4**(-1) - 4*S4**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(80)*CF**2 * (  - 4*S**(-1)*U1*S4**(-1)*MS2
     +     - 4*S**(-1)*U1 + 4*S**(-1)*U1**2*S4**(-1) - 4*T1*S4**(-1) + 
     +    4*U1*S4**(-1) - 4*S4**(-1)*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(82)*N*CF * (  - 2*M2*TG**(-1)*S**(-1)*U1*
     +    MS2 + 2*M2*TG**(-1)*MS2 + 2*M2*S**(-1)*T1 - 4*M2*S**(-1)*U1*
     +    S4**(-1)*MS2 + 2*M2*S**(-1)*U1 - 4*M2*S**(-1)*U1**2*S4**(-1)
     +     + 2*M2*S**(-1)*MS2 + 2*M2*S*(S+U1+M2)**(-1) - 2*M2**2*
     +    TG**(-1)*S**(-1)*U1 - 2*M2**2*TG**(-1)*S**(-1)*MS2 + 2*M2**2*
     +    TG**(-1) + 2*M2**2*S**(-1) + 2*M2**2*(S+U1+M2)**(-1) + 2*
     +    M2**3*TG**(-1)*S**(-1) + 4*S**(-1)*T1*U1 - 2*S**(-1)*T1**2 + 
     +    2*S**(-1)*U1*MS2 - 4*S**(-1)*U1**2*S4**(-1)*MS2 - 4*S**(-1)*
     +    U1**2 + 4*S**(-1)*U1**3*S4**(-1) + 2*S*T1*(S+U1+M2)**(-1) + 4
     +    *S*(S+U1+M2)**(-1)*MS2 - 2*T1 + 2*T1**2*(S+U1+M2)**(-1) + 4*
     +    U1*S4**(-1)*MS2 + 4*U1**2*S4**(-1) - 4*MS2 )
     +
      M2QBH4 = M2QBH4 + ANG4(82)*CF**2 * ( 4*M2*TG**(-1)*S**(-1)*U1*MS2
     +     + 4*M2*TG**(-1)*S*(U1+M2)*MS2*SYMBX - 8*M2*TG**(-1)*MS2 - 4*
     +    M2*S**(-1)*T1 + 4*M2*S**(-1)*U1*S4**(-1)*MS2 + 4*M2*S**(-1)*
     +    U1**2*S4**(-1) + 4*M2*S*(U1+M2)*SYMBX - 4*M2*S*
     +    (S+U1+M2)**(-1) + 4*M2*T1*(U1+M2)*SYMBX + 4*M2**2*TG**(-1)*
     +    S**(-1)*U1 + 4*M2**2*TG**(-1)*S**(-1)*MS2 + 4*M2**2*TG**(-1)*
     +    S*(U1+M2)*SYMBX - 8*M2**2*TG**(-1) - 4*M2**2*S**(-1) + 4*
     +    M2**2*(U1+M2)*SYMBX - 4*M2**2*(S+U1+M2)**(-1) - 4*M2**3*
     +    TG**(-1)*S**(-1) + 8*M2**3*TG**(-1)*(U1+M2)*SYMBX - 4*S**(-1)
     +    *T1*U1 + 4*S**(-1)*U1**2*S4**(-1)*MS2 + 4*S**(-1)*U1**2 - 4*
     +    S**(-1)*U1**3*S4**(-1) - 4*S*T1*(S+U1+M2)**(-1) + 8*S*(U1+M2)
     +    *MS2*SYMBX - 8*S*(S+U1+M2)**(-1)*MS2 - 4*T1**2*
     +    (S+U1+M2)**(-1) - 4*U1*S4**(-1)*MS2 - 4*U1**2*S4**(-1) + 4*
     +    MS2 )

      END IF

      M2QBH2 = 0.D0
      M2QBH2 = M2QBH2 + N*CF**2*(S4+MS2) * (  - 8*M2*TG**(-2)*S*
     +    S4**(-1)*(S+U1)**(-2)*MS2 - 8*M2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2*TG**(-2)*S4**(-1) - 16*M2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2) + 32*M2*TG**(-1)*S4**(-2) - 16*M2*
     +    TG**(-1)*S4**(-1)*(S+T1)**(-1) + 16*M2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 16*M2*S*T1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-2) - 
     +    8*M2*S*U1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-2) - 16*M2*S*S4**(-1)
     +    *(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*M2*S*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*M2*S**2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*M2*T1*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*M2*T1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*M2*T1**2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2) - 32*M2*S4**(-2)*(S+U1+M2)**(-1) + 16
     +    *M2*S4**(-1)*(S+U1)**(-1)*(S+U1+M2)**(-1) - 8*M2**2*TG**(-2)*
     +    S*S4**(-1)*(S+U1)**(-2) + 8*M2**2*TG**(-2)*S4**(-1)*
     +    (S+U1)**(-1) )
     +
      M2QBH2 = M2QBH2 + N*CF**2*(S4+MS2) * (  - 8*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-1)*MS2 + 32*TG**(-1)*S*S4**(-2) - 16*TG**(-1)*S*
     +    S4**(-1)*(S+T1)**(-1) - 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-2)*
     +    MS2 - 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-1) + 32*TG**(-1)*U1*
     +    S4**(-2) - 8*TG**(-1)*S4**(-1) + 16*S*T1*S4**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S*T1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*S*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*S*S4**(-1)*(S+U1)**(-2) + 8*S
     +    *S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S**2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*T1*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 + 32*T1*S4**(-2)*(S+U1+M2)**(-1)
     +     - 8*T1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 64*S4**(-2) - 16*S4**(-1)*
     +    (S+T1)**(-1) - 8*S4**(-1)*(S+U1)**(-1) + 8*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1)*MS2 )
     +
      M2QBH2 = M2QBH2 + N*CF**2 * (  - 16*M2*TG**(-2)*U1*S4**(-2)*MS2
     +     - 8*M2*TG**(-2)*U1*S4**(-1) - 16*M2*TG**(-2)*S4**(-2)*MS2**2
     +     - 16*TG**(-2)*U1*S4**(-2)*MS2**2 - 8*TG**(-2)*U1*S4**(-1)*
     +    MS2 + 16*TG**(-2)*S4**(-1)*MS2**2 + 8*TG**(-2)*MS2 - 16*
     +    TG**(-1)*U1*S4**(-2)*MS2 - 8*TG**(-1)*U1*S4**(-1) - 16*
     +    TG**(-1)*S4**(-2)*MS2**2 )
     +
      M2QBH2 = M2QBH2 + N**2*CF*(S4+MS2) * ( 4*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-2) - 4*M2*TG**(-1)*S*S4**(-1)*(S+U1)**(-1)*
     +    (S+U1+M2)**(-1) - 4*M2*TG**(-1)*S*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*M2*TG**(-1)*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*M2*TG**(-1)*S4**(-2) + 8*M2*
     +    TG**(-1)*S4**(-1)*(S+T1)**(-1) - 4*M2*S*S4**(-1)*(S+U1)**(-2)
     +    *(S+U1+M2)**(-1) + 4*M2*S*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*M2*S4**(-2)*(S+U1+M2)**(-1) - 4*
     +    M2*S4**(-1)*(S+U1)**(-1)*(S+U1+M2)**(-1) - 4*M2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*M2**2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-2)*(S+U1+M2)**(-1) - 4*M2**2*TG**(-1)*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*TG**(-1)*S*S4**(-2) + 8*TG**(-1)*
     +    S*S4**(-1)*(S+T1)**(-1) - 8*TG**(-1)*U1*S4**(-2) + 4*TG**(-1)
     +    *S4**(-1) - 4*S*T1*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S*S4**(-1)*(S+U1)**(-2) - 4*S*
     +    S4**(-1)*(S+U1)**(-1)*(S+U1+M2)**(-1) )
     +
      M2QBH2 = M2QBH2 + N**2*CF*(S4+MS2) * (  - 8*T1*S4**(-2)*
     +    (S+U1+M2)**(-1) + 4*T1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 4*
     +    U1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-2) + 8*
     +    S4**(-1)*(S+T1)**(-1) + 4*S4**(-1)*(S+U1)**(-1) )
     +
      M2QBH2 = M2QBH2 + COLO1(9)*N*CF**2*(S4+MS2) * ( 8*M2*TG**(-2)*U1*
     +    S4**(-2) - 16*M2*TG**(-1)*S*S4**(-2)*(S+U1)**(-2)*MS2 - 16*M2
     +    *S*S4**(-2)*(S+U1)**(-2) + 16*M2*S4**(-2)*(S+U1)**(-1) - 8*
     +    M2**2*TG**(-2)*S*S4**(-2)*(S+U1)**(-2)*MS2 - 24*M2**2*
     +    TG**(-1)*S*S4**(-2)*(S+U1)**(-2) + 24*M2**2*TG**(-1)*S4**(-2)
     +    *(S+U1)**(-1) - 8*M2**3*TG**(-2)*S*S4**(-2)*(S+U1)**(-2) + 8*
     +    M2**3*TG**(-2)*S4**(-2)*(S+U1)**(-1) - 8*TG**(-2)*S*S4**(-2)*
     +    MS2 + 8*TG**(-1)*U1*S4**(-2) - 8*S*T1*S4**(-2)*(S+U1)**(-2)
     +     + 16*S*T1**2*U1*S4**(-2)*(M2*(S+T1)+T1*U1)**(-2) - 8*S*T1**2
     +    *S4**(-2)*(M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*S*U1**2*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*S*S4**(-2)*(S+U1)**(-2)*MS2
     +     + 8*S**2*T1*U1*S4**(-2)*(M2*(S+T1)+T1*U1)**(-2) - 16*S**2*T1
     +    *S4**(-2)*(M2*(S+T1)+T1*U1)**(-2)*MS2 - 8*S**3*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MS2 + 8*T1*U1**3*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*T1*S4**(-2)*(S+U1)**(-1) + 8*
     +    T1**3*U1*S4**(-2)*(M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QBH2 = M2QBH2 + ANG4(36)*N*CF**2 * ( 8*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(37)*N*CF**2 * ( 16*M2*TG**(-1)*S4**(-1)*
     +    MS2 + 8*M2*S4**(-1) + 8*M2**2*TG**(-1)*S4**(-1) - 16*TG**(-1)
     +    *U1*S4**(-1)*MS2 - 8*TG**(-1)*U1**2*S4**(-1) + 16*TG**(-1)*
     +    MS2 + 8*T1*S4**(-1) + 16*S4**(-1)*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(37)*N**2*CF * (  - 4*M2*TG**(-1)*S4**(-1)*
     +    MS2 - 2*M2*S4**(-1) - 2*M2**2*TG**(-1)*S4**(-1) + 4*TG**(-1)*
     +    U1*S4**(-1)*MS2 + 2*TG**(-1)*U1**2*S4**(-1) - 4*TG**(-1)*MS2
     +     - 2*T1*S4**(-1) - 4*S4**(-1)*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(48)*N*CF**2 * (  - 8*M2*TG**(-1)*S + 16*M2
     +    *TG**(-1)*S**2*(S+U1+M2)**(-1) + 8*M2*S*(S+U1+M2)**(-1) + 16*
     +    M2**2*TG**(-1)*S*(S+U1+M2)**(-1) + 8*TG**(-1)*S*U1 + 16*
     +    TG**(-1)*S**2*(S+U1+M2)**(-1)*MS2 + 8*S*T1*(S+U1+M2)**(-1) + 
     +    8*S**2*(S+U1+M2)**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(48)*N**2*CF * ( 2*M2*TG**(-1)*S - 4*M2*
     +    TG**(-1)*S**2*(S+U1+M2)**(-1) - 2*M2*S*(S+U1+M2)**(-1) - 4*
     +    M2**2*TG**(-1)*S*(S+U1+M2)**(-1) - 2*TG**(-1)*S*U1 - 4*
     +    TG**(-1)*S**2*(S+U1+M2)**(-1)*MS2 - 2*S*T1*(S+U1+M2)**(-1) - 
     +    2*S**2*(S+U1+M2)**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(49)*N*CF**2 * (  - 4*M2*TG**(-2)*S*MS2 + 4
     +    *M2*TG**(-2)*U1*MS2 - 8*M2*TG**(-1)*S + 8*M2*TG**(-1)*U1 + 8*
     +    M2*TG**(-1)*MS2 - 8*M2 - 4*M2**2*TG**(-2)*S + 4*M2**2*
     +    TG**(-2)*U1 + 4*M2**2*TG**(-2)*MS2 - 12*M2**2*TG**(-1) - 4*
     +    M2**3*TG**(-2) - 4*TG**(-1)*S*MS2 + 4*TG**(-1)*U1*MS2 - 4*S
     +     - 4*T1 + 4*U1 + 4*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(49)*N**2*CF * ( 2*M2*TG**(-2)*S*MS2 - 2*M2
     +    *TG**(-2)*U1*MS2 + 4*M2*TG**(-1)*S - 4*M2*TG**(-1)*U1 - 4*M2*
     +    TG**(-1)*MS2 + 4*M2 + 2*M2**2*TG**(-2)*S - 2*M2**2*TG**(-2)*
     +    U1 - 2*M2**2*TG**(-2)*MS2 + 6*M2**2*TG**(-1) + 2*M2**3*
     +    TG**(-2) + 2*TG**(-1)*S*MS2 - 2*TG**(-1)*U1*MS2 + 2*S + 2*T1
     +     - 2*U1 - 2*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(50)*N*CF**2 * ( 16*M2*TG**(-1)*S*
     +    (S+U1+M2)**(-1)*MS2 - 16*M2*TG**(-1)*S + 16*M2*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1) + 16*M2*TG**(-1)*U1 + 16*M2*S*(S+U1+M2)**(-1)
     +     - 8*M2*(S+U1+M2)**(-1)*MS2 - 8*M2 + 32*M2**2*TG**(-1)*S*
     +    (S+U1+M2)**(-1) - 16*M2**2*TG**(-1) + 8*M2**2*(S+U1+M2)**(-1)
     +     + 16*M2**3*TG**(-1)*(S+U1+M2)**(-1) - 16*TG**(-1)*S*MS2 + 16
     +    *TG**(-1)*S**2*(S+U1+M2)**(-1)*MS2 + 8*TG**(-1)*U1*MS2 - 8*
     +    TG**(-1)*U1**2 - 8*S*(S+U1+M2)**(-1)*MS2 - 8*S + 8*S**2*
     +    (S+U1+M2)**(-1) + 8*U1 + 8*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(50)*N**2*CF * (  - 8*M2*TG**(-1)*S*
     +    (S+U1+M2)**(-1)*MS2 + 8*M2*TG**(-1)*S - 8*M2*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1) - 8*M2*TG**(-1)*U1 - 8*M2*S*(S+U1+M2)**(-1)
     +     + 4*M2*(S+U1+M2)**(-1)*MS2 + 4*M2 - 16*M2**2*TG**(-1)*S*
     +    (S+U1+M2)**(-1) + 8*M2**2*TG**(-1) - 4*M2**2*(S+U1+M2)**(-1)
     +     - 8*M2**3*TG**(-1)*(S+U1+M2)**(-1) + 8*TG**(-1)*S*MS2 - 8*
     +    TG**(-1)*S**2*(S+U1+M2)**(-1)*MS2 - 4*TG**(-1)*U1*MS2 + 4*
     +    TG**(-1)*U1**2 + 4*S*(S+U1+M2)**(-1)*MS2 + 4*S - 4*S**2*
     +    (S+U1+M2)**(-1) - 4*U1 - 4*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(68)*N*CF**2 * ( 4*M2*TG**(-2)*S4**(-1)*MS2
     +     + 8*M2*TG**(-1)*S4**(-1) + 4*M2**2*TG**(-2)*S4**(-1) + 4*
     +    TG**(-1)*S4**(-1)*MS2 + 4*S4**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(68)*N**2*CF * (  - 2*M2*TG**(-2)*S4**(-1)*
     +    MS2 + 2*M2*TG**(-2)*(S+U1+M2)**(-1)*MS2 - 4*M2*TG**(-1)*
     +    S4**(-1) + 2*M2*TG**(-1)*(S+U1+M2)**(-1) - 2*M2**2*TG**(-2)*
     +    S4**(-1) + 2*M2**2*TG**(-2)*(S+U1+M2)**(-1) - 2*TG**(-1)*
     +    S4**(-1)*MS2 - 2*S4**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(72)*N*CF**2 * ( 8*M2*S*MS2 + 8*M2**2*MS2
     +     + 8*S*MS2**2 )
     +
      M2QBH2 = M2QBH2 + ANG4(73)*N*CF**2 * ( 4*M2*TG**(-1)*S*MS2 - 4*M2
     +    *TG**(-1)*U1*MS2 + 4*M2*S - 4*M2*MS2 + 4*M2**2*TG**(-1)*S - 4
     +    *M2**2*TG**(-1)*U1 - 4*M2**2*TG**(-1)*MS2 + 4*M2**2 + 4*M2**3
     +    *TG**(-1) + 4*S*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(73)*N**2*CF * ( 8*M2*TG**(-1)*S*MS2 + 8*
     +    M2**2*TG**(-1)*MS2 + 8*TG**(-1)*S*MS2**2 )
     +
      M2QBH2 = M2QBH2 + ANG4(74)*N*CF**2 * ( 16*M2*MS2 + 8*S*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(75)*N*CF**2 * ( 4*M2*TG**(-2)*S*MS2 - 4*M2
     +    *TG**(-2)*U1*MS2 + 16*M2*TG**(-1)*S*(S+U1+M2)**(-1)*MS2 - 8*
     +    M2*TG**(-1)*S + 16*M2*TG**(-1)*S**2*(S+U1+M2)**(-1) - 64*M2*
     +    TG**(-1)*U1*S4**(-1)*MS2 + 8*M2*TG**(-1)*U1 - 16*M2*TG**(-1)*
     +    U1**2*S4**(-1) - 32*M2*TG**(-1)*S4**(-1)*MS2**2 + 32*M2*
     +    TG**(-1)*MS2 + 16*M2*S*(S+U1+M2)**(-1) - 16*M2*U1*S4**(-1) - 
     +    16*M2*S4**(-1)*MS2 - 8*M2*(S+U1+M2)**(-1)*MS2 + 8*M2 + 4*
     +    M2**2*TG**(-2)*S - 4*M2**2*TG**(-2)*U1 - 4*M2**2*TG**(-2)*MS2
     +     + 32*M2**2*TG**(-1)*S*(S+U1+M2)**(-1) - 16*M2**2*TG**(-1)*U1
     +    *S4**(-1) - 16*M2**2*TG**(-1)*S4**(-1)*MS2 + 4*M2**2*TG**(-1)
     +     + 8*M2**2*(S+U1+M2)**(-1) + 4*M2**3*TG**(-2) + 16*M2**3*
     +    TG**(-1)*(S+U1+M2)**(-1) - 12*TG**(-1)*S*MS2 + 16*TG**(-1)*
     +    S**2*(S+U1+M2)**(-1)*MS2 - 32*TG**(-1)*U1*S4**(-1)*MS2**2 + 
     +    12*TG**(-1)*U1*MS2 - 16*TG**(-1)*U1**2*S4**(-1)*MS2 + 32*
     +    TG**(-1)*MS2**2 - 8*S*(S+U1+M2)**(-1)*MS2 - 4*S + 8*S**2*
     +    (S+U1+M2)**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(75)*N*CF**2 * (  - 16*T1*U1*S4**(-1) - 32*
     +    T1*S4**(-1)*MS2 + 8*T1 - 8*T1**2*S4**(-1) - 48*U1*S4**(-1)*
     +    MS2 + 8*U1 - 8*U1**2*S4**(-1) - 32*S4**(-1)*MS2**2 + 28*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(75)*N**2*CF * (  - 2*M2*TG**(-2)*S*MS2 + 2
     +    *M2*TG**(-2)*U1*MS2 - 8*M2*TG**(-1)*S*(S+U1+M2)**(-1)*MS2 + 4
     +    *M2*TG**(-1)*S - 8*M2*TG**(-1)*S**2*(S+U1+M2)**(-1) + 16*M2*
     +    TG**(-1)*U1*S4**(-1)*MS2 - 4*M2*TG**(-1)*U1 + 4*M2*TG**(-1)*
     +    U1**2*S4**(-1) + 8*M2*TG**(-1)*S4**(-1)*MS2**2 + 2*M2*
     +    TG**(-1)*MS2 - 8*M2*S*(S+U1+M2)**(-1) + 4*M2*U1*S4**(-1) + 4*
     +    M2*S4**(-1)*MS2 + 4*M2*(S+U1+M2)**(-1)*MS2 - 2*M2**2*TG**(-2)
     +    *S + 2*M2**2*TG**(-2)*U1 + 2*M2**2*TG**(-2)*MS2 - 16*M2**2*
     +    TG**(-1)*S*(S+U1+M2)**(-1) + 4*M2**2*TG**(-1)*U1*S4**(-1) + 4
     +    *M2**2*TG**(-1)*S4**(-1)*MS2 + 2*M2**2*TG**(-1) - 4*M2**2*
     +    (S+U1+M2)**(-1) - 2*M2**3*TG**(-2) - 8*M2**3*TG**(-1)*
     +    (S+U1+M2)**(-1) + 10*TG**(-1)*S*MS2 - 8*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1)*MS2 + 8*TG**(-1)*U1*S4**(-1)*MS2**2 - 4*
     +    TG**(-1)*U1*MS2 + 4*TG**(-1)*U1**2*S4**(-1)*MS2 - 8*TG**(-1)*
     +    MS2**2 + 4*S*(S+U1+M2)**(-1)*MS2 + 2*S - 4*S**2*
     +    (S+U1+M2)**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(75)*N**2*CF * ( 4*T1*U1*S4**(-1) + 8*T1*
     +    S4**(-1)*MS2 - 2*T1 + 2*T1**2*S4**(-1) + 12*U1*S4**(-1)*MS2
     +     - 2*U1 + 2*U1**2*S4**(-1) + 8*S4**(-1)*MS2**2 - 8*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(77)*N**2*CF * (  - 4*M2*TG**(-2)*MS2 - 4*
     +    M2*TG**(-1) - 4*M2**2*TG**(-2) )
     +
      M2QBH2 = M2QBH2 + ANG4(78)*N**2*CF * (  - 2*M2*TG**(-2)*S4**(-1)*
     +    MS2 + 2*M2*TG**(-2)*(S+U1+M2)**(-1)*MS2 - 4*M2*TG**(-1)*
     +    S4**(-1) + 2*M2*TG**(-1)*(S+U1+M2)**(-1) - 2*M2**2*TG**(-2)*
     +    S4**(-1) + 2*M2**2*TG**(-2)*(S+U1+M2)**(-1) - 2*TG**(-1)*
     +    S4**(-1)*MS2 - 2*S4**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(79)*N**2*CF * ( 8*M2*TG**(-2)*S*MS2 - 4*M2
     +    *TG**(-2)*U1*MS2 - 4*M2*TG**(-1)*MS2 - 4*M2**2*TG**(-2)*U1 + 
     +    8*TG**(-2)*S*MS2**2 + 4*TG**(-1)*S*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(80)*N*CF**2 * (  - 8*M2*TG**(-1)*S4**(-1)*
     +    MS2 - 8*M2*TG**(-1)*(S+U1+M2)**(-1)*MS2 - 8*M2*S4**(-1) - 8*
     +    M2*(S+U1+M2)**(-1) - 8*M2**2*TG**(-1)*S4**(-1) - 8*M2**2*
     +    TG**(-1)*(S+U1+M2)**(-1) - 8*T1*S4**(-1) - 8*S4**(-1)*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(80)*N**2*CF * (  - 2*M2*TG**(-2)*S*
     +    (S+U1+M2)**(-1)*MS2 - 10*M2*TG**(-2)*U1*S4**(-1)*MS2 - 4*M2*
     +    TG**(-2)*U1 - 8*M2*TG**(-2)*S4**(-1)*MS2**2 + 6*M2*TG**(-2)*
     +    MS2 + 2*M2*TG**(-1)*S*(S+U1+M2)**(-1) - 4*M2*TG**(-1)*U1*
     +    S4**(-1) - 8*M2*TG**(-1)*S4**(-1)*MS2 + 6*M2*TG**(-1)*
     +    (S+U1+M2)**(-1)*MS2 + 2*M2*S4**(-1) + 4*M2*(S+U1+M2)**(-1) - 
     +    2*M2**2*TG**(-2)*S*(S+U1+M2)**(-1) - 2*M2**2*TG**(-2)*U1*
     +    S4**(-1) - 4*M2**2*TG**(-2)*S4**(-1)*MS2 + 2*M2**2*TG**(-2)*
     +    (S+U1+M2)**(-1)*MS2 + 2*M2**2*TG**(-2) + 2*M2**2*TG**(-1)*
     +    S4**(-1) + 4*M2**2*TG**(-1)*(S+U1+M2)**(-1) - 2*M2**3*
     +    TG**(-2)*(S+U1+M2)**(-1) + 4*TG**(-2)*S*MS2 - 8*TG**(-2)*U1*
     +    S4**(-1)*MS2**2 + 8*TG**(-2)*MS2**2 + 2*TG**(-1)*S*
     +    (S+U1+M2)**(-1)*MS2 - 8*TG**(-1)*U1*S4**(-1)*MS2 - 8*TG**(-1)
     +    *S4**(-1)*MS2**2 + 4*TG**(-1)*MS2 + 2*S*(S+U1+M2)**(-1) + 2*
     +    T1*S4**(-1) - 2*U1*S4**(-1) - 4*S4**(-1)*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(81)*N*CF**2 * ( 4*M2*TG**(-1)*S*MS2 - 4*M2
     +    *TG**(-1)*U1*MS2 + 4*M2*S + 4*M2*T1 + 4*M2**2*TG**(-1)*S - 4*
     +    M2**2*TG**(-1)*U1 - 4*M2**2*TG**(-1)*MS2 + 4*M2**2 + 4*M2**3*
     +    TG**(-1) + 4*S*MS2 )
     +
      M2QBH2 = M2QBH2 + ANG4(82)*N*CF**2 * ( 4*M2*TG**(-2)*S*MS2 - 4*M2
     +    *TG**(-2)*U1*MS2 - 8*M2*TG**(-1)*S + 16*M2*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1) - 16*M2*TG**(-1)*U1*S4**(-1)*MS2 + 8*M2*
     +    TG**(-1)*U1 - 16*M2*TG**(-1)*U1**2*S4**(-1) + 8*M2*S*
     +    (S+U1+M2)**(-1) + 4*M2**2*TG**(-2)*S - 4*M2**2*TG**(-2)*U1 - 
     +    4*M2**2*TG**(-2)*MS2 + 16*M2**2*TG**(-1)*S*(S+U1+M2)**(-1) + 
     +    4*M2**2*TG**(-1) + 4*M2**3*TG**(-2) - 12*TG**(-1)*S*MS2 + 16*
     +    TG**(-1)*S**2*(S+U1+M2)**(-1)*MS2 + 12*TG**(-1)*U1*MS2 - 16*
     +    TG**(-1)*U1**2*S4**(-1)*MS2 + 8*S*T1*(S+U1+M2)**(-1) - 4*S + 
     +    8*S**2*(S+U1+M2)**(-1) - 4*T1 - 8*U1*S4**(-1)*MS2 + 8*U1 - 8*
     +    U1**2*S4**(-1) )
     +
      M2QBH2 = M2QBH2 + ANG4(82)*N**2*CF * (  - 2*M2*TG**(-2)*S*MS2 + 2
     +    *M2*TG**(-2)*U1*MS2 + 2*M2*TG**(-1)*S - 4*M2*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1) + 8*M2*TG**(-1)*U1*S4**(-1)*MS2 - 6*M2*
     +    TG**(-1)*U1 + 8*M2*TG**(-1)*U1**2*S4**(-1) - 2*M2*TG**(-1)*
     +    MS2 - 2*M2*S*(S+U1+M2)**(-1) - 2*M2**2*TG**(-2)*S + 2*M2**2*
     +    TG**(-2)*U1 + 2*M2**2*TG**(-2)*MS2 - 4*M2**2*TG**(-1)*S*
     +    (S+U1+M2)**(-1) - 2*M2**2*TG**(-1) - 2*M2**3*TG**(-2) + 4*
     +    TG**(-1)*S*MS2 - 4*TG**(-1)*S**2*(S+U1+M2)**(-1)*MS2 - 6*
     +    TG**(-1)*U1*MS2 + 8*TG**(-1)*U1**2*S4**(-1)*MS2 - 2*S*T1*
     +    (S+U1+M2)**(-1) + 2*S - 2*S**2*(S+U1+M2)**(-1) + 2*T1 + 4*U1*
     +    S4**(-1)*MS2 - 4*U1 + 4*U1**2*S4**(-1) )

      M2QBH3 = 0.D0
      M2QBH3 = M2QBH3 + N*CF**2*(S4+MS2) * ( 8*M2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-2)*MG2 + 8*M2*S*S4**(-1)*(M2*(S+T1)+T1*U1)**(-2)*
     +    MG2 + 8*M2*T1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-2)*MG2 + 8*
     +    TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 + 8*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-2)*MG2 + 8*S*T1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-2)*
     +    MG2 + 8*S*U1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-2)*MG2 + 8*S**2*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-2)*MG2 + 8*T1*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MG2 - 8*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + N*CF**2 * ( 16*M2*TG**(-2)*S4**(-2)*MS2*MG2 + 
     +    16*M2*TG**(-2)*S4**(-1)*MG2 + 16*TG**(-2)*U1*S4**(-2)*MS2*MG2
     +     + 8*TG**(-2)*U1*S4**(-1)*MG2 - 16*TG**(-2)*S4**(-1)*MS2*MG2
     +     - 8*TG**(-2)*MG2 + 16*TG**(-1)*S4**(-2)*MS2*MG2 + 16*
     +    TG**(-1)*S4**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + COLO1(9)*N*CF**2*(S4+MS2) * ( 16*M2*TG**(-1)*S*
     +    S4**(-2)*(S+U1)**(-2)*MG2 + 8*M2**2*TG**(-2)*S*S4**(-2)*
     +    (S+U1)**(-2)*MG2 + 8*TG**(-2)*S*S4**(-2)*MG2 + 8*S*T1**2*
     +    S4**(-2)*(M2*(S+T1)+T1*U1)**(-2)*MG2 + 8*S*U1**2*S4**(-2)*
     +    (M2*(S+T1)+T1*U1)**(-2)*MG2 + 8*S*S4**(-2)*(S+U1)**(-2)*MG2
     +     + 16*S**2*T1*S4**(-2)*(M2*(S+T1)+T1*U1)**(-2)*MG2 + 8*S**3*
     +    S4**(-2)*(M2*(S+T1)+T1*U1)**(-2)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(48)*N*CF**2 * (  - 16*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(48)*N**2*CF * ( 4*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(49)*N*CF**2 * ( 4*M2*TG**(-2)*S*MG2 - 4*M2
     +    *TG**(-2)*U1*MG2 - 8*M2*TG**(-1)*MG2 - 4*M2**2*TG**(-2)*MG2
     +     + 4*TG**(-1)*S*MG2 - 4*TG**(-1)*U1*MG2 - 4*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(49)*N**2*CF * (  - 2*M2*TG**(-2)*S*MG2 + 2
     +    *M2*TG**(-2)*U1*MG2 + 4*M2*TG**(-1)*MG2 + 2*M2**2*TG**(-2)*
     +    MG2 - 2*TG**(-1)*S*MG2 + 2*TG**(-1)*U1*MG2 + 2*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(50)*N*CF**2 * (  - 16*M2*TG**(-1)*S*
     +    (S+U1+M2)**(-1)*MG2 + 8*M2*(S+U1+M2)**(-1)*MG2 + 16*TG**(-1)*
     +    S*MG2 - 16*TG**(-1)*S**2*(S+U1+M2)**(-1)*MG2 - 8*TG**(-1)*U1*
     +    MG2 + 8*S*(S+U1+M2)**(-1)*MG2 - 8*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(50)*N**2*CF * ( 8*M2*TG**(-1)*S*
     +    (S+U1+M2)**(-1)*MG2 - 4*M2*(S+U1+M2)**(-1)*MG2 - 8*TG**(-1)*S
     +    *MG2 + 8*TG**(-1)*S**2*(S+U1+M2)**(-1)*MG2 + 4*TG**(-1)*U1*
     +    MG2 - 4*S*(S+U1+M2)**(-1)*MG2 + 4*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(68)*N*CF**2 * (  - 4*M2*TG**(-2)*S4**(-1)*
     +    MG2 - 4*TG**(-1)*S4**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(68)*N**2*CF * ( 2*M2*TG**(-2)*S4**(-1)*MG2
     +     - 2*M2*TG**(-2)*(S+U1+M2)**(-1)*MG2 + 2*TG**(-1)*S4**(-1)*
     +    MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(72)*N*CF**2 * (  - 8*S*MS2*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(73)*N*CF**2 * (  - 4*M2*TG**(-1)*S*MG2 + 4
     +    *M2*TG**(-1)*U1*MG2 + 4*M2*MG2 + 4*M2**2*TG**(-1)*MG2 - 4*S*
     +    MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(73)*N**2*CF * (  - 8*TG**(-1)*S*MS2*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(75)*N*CF**2 * (  - 4*M2*TG**(-2)*S*MG2 + 4
     +    *M2*TG**(-2)*U1*MG2 - 16*M2*TG**(-1)*S*(S+U1+M2)**(-1)*MG2 + 
     +    32*M2*TG**(-1)*U1*S4**(-1)*MG2 + 32*M2*TG**(-1)*S4**(-1)*MS2*
     +    MG2 + 16*M2*S4**(-1)*MG2 + 8*M2*(S+U1+M2)**(-1)*MG2 + 4*M2**2
     +    *TG**(-2)*MG2 + 16*M2**2*TG**(-1)*S4**(-1)*MG2 + 12*TG**(-1)*
     +    S*MG2 - 16*TG**(-1)*S**2*(S+U1+M2)**(-1)*MG2 + 32*TG**(-1)*U1
     +    *S4**(-1)*MS2*MG2 - 12*TG**(-1)*U1*MG2 + 16*TG**(-1)*U1**2*
     +    S4**(-1)*MG2 - 32*TG**(-1)*MS2*MG2 + 8*S*(S+U1+M2)**(-1)*MG2
     +     + 16*T1*S4**(-1)*MG2 + 32*U1*S4**(-1)*MG2 + 32*S4**(-1)*MS2*
     +    MG2 - 12*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(75)*N**2*CF * ( 2*M2*TG**(-2)*S*MG2 - 2*M2
     +    *TG**(-2)*U1*MG2 + 8*M2*TG**(-1)*S*(S+U1+M2)**(-1)*MG2 - 8*M2
     +    *TG**(-1)*U1*S4**(-1)*MG2 - 8*M2*TG**(-1)*S4**(-1)*MS2*MG2 - 
     +    2*M2*TG**(-1)*MG2 - 4*M2*S4**(-1)*MG2 - 4*M2*(S+U1+M2)**(-1)*
     +    MG2 - 2*M2**2*TG**(-2)*MG2 - 4*M2**2*TG**(-1)*S4**(-1)*MG2 - 
     +    6*TG**(-1)*S*MG2 + 8*TG**(-1)*S**2*(S+U1+M2)**(-1)*MG2 - 8*
     +    TG**(-1)*U1*S4**(-1)*MS2*MG2 + 4*TG**(-1)*U1*MG2 - 4*TG**(-1)
     +    *U1**2*S4**(-1)*MG2 + 8*TG**(-1)*MS2*MG2 - 4*S*
     +    (S+U1+M2)**(-1)*MG2 - 4*T1*S4**(-1)*MG2 - 8*U1*S4**(-1)*MG2
     +     - 8*S4**(-1)*MS2*MG2 + 4*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(77)*N**2*CF * ( 4*M2*TG**(-2)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(78)*N**2*CF * ( 2*M2*TG**(-2)*S4**(-1)*MG2
     +     - 2*M2*TG**(-2)*(S+U1+M2)**(-1)*MG2 + 2*TG**(-1)*S4**(-1)*
     +    MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(79)*N**2*CF * ( 4*M2*TG**(-2)*U1*MG2 + 4*
     +    M2*TG**(-1)*MG2 + 8*M2**2*TG**(-2)*MG2 - 8*TG**(-2)*S*MS2*MG2
     +     - 4*TG**(-1)*S*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(80)*N*CF**2 * ( 8*M2*TG**(-1)*S4**(-1)*MG2
     +     + 8*M2*TG**(-1)*(S+U1+M2)**(-1)*MG2 + 8*S4**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(80)*N**2*CF * ( 2*M2*TG**(-2)*S*
     +    (S+U1+M2)**(-1)*MG2 + 2*M2*TG**(-2)*U1*S4**(-1)*MG2 + 8*M2*
     +    TG**(-2)*S4**(-1)*MS2*MG2 + 2*M2*TG**(-2)*MG2 + 4*M2*TG**(-1)
     +    *S4**(-1)*MG2 - 6*M2*TG**(-1)*(S+U1+M2)**(-1)*MG2 + 4*M2**2*
     +    TG**(-2)*S4**(-1)*MG2 - 2*M2**2*TG**(-2)*(S+U1+M2)**(-1)*MG2
     +     - 4*TG**(-2)*S*MG2 + 8*TG**(-2)*U1*S4**(-1)*MS2*MG2 - 8*
     +    TG**(-2)*MS2*MG2 - 2*TG**(-1)*S*(S+U1+M2)**(-1)*MG2 + 4*
     +    TG**(-1)*U1*S4**(-1)*MG2 + 8*TG**(-1)*S4**(-1)*MS2*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(81)*N*CF**2 * (  - 4*M2*TG**(-1)*S*MG2 + 4
     +    *M2*TG**(-1)*U1*MG2 + 4*M2**2*TG**(-1)*MG2 - 4*S*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(82)*N*CF**2 * (  - 4*M2*TG**(-2)*S*MG2 + 4
     +    *M2*TG**(-2)*U1*MG2 + 16*M2*TG**(-1)*U1*S4**(-1)*MG2 + 4*
     +    M2**2*TG**(-2)*MG2 + 12*TG**(-1)*S*MG2 - 16*TG**(-1)*S**2*
     +    (S+U1+M2)**(-1)*MG2 - 12*TG**(-1)*U1*MG2 + 16*TG**(-1)*U1**2*
     +    S4**(-1)*MG2 + 8*U1*S4**(-1)*MG2 )
     +
      M2QBH3 = M2QBH3 + ANG4(82)*N**2*CF * ( 2*M2*TG**(-2)*S*MG2 - 2*M2
     +    *TG**(-2)*U1*MG2 - 8*M2*TG**(-1)*U1*S4**(-1)*MG2 + 2*M2*
     +    TG**(-1)*MG2 - 2*M2**2*TG**(-2)*MG2 - 4*TG**(-1)*S*MG2 + 4*
     +    TG**(-1)*S**2*(S+U1+M2)**(-1)*MG2 + 6*TG**(-1)*U1*MG2 - 8*
     +    TG**(-1)*U1**2*S4**(-1)*MG2 - 4*U1*S4**(-1)*MG2 )


      IF (IFL.EQ.0) M2QBH = 2.D0*M2QBH2 +2.D0*M2QBH3 
      IF (IFL.EQ.1) M2QBH = 
     +     2.D0*(NS-1.D0)*M2QBH1 +2.D0*M2QBH2 +2.D0*M2QBH3 +2.D0*M2QBH4 
           
      DSBQBH = S4/(S4+MS2)/2.D0 *
     +     ALPHAS**3 * AVG * M2QBH /4.D0 /S**2 *CONV

      RETURN
      END


      REAL*8 FUNCTION DSBQBD(ALPHAS,S,T1,S4,MS,MG,DEL,S4MAX,IFL)
C***  LOG(DEL) PART OF CROSS SECTIONS FOR Q +QB -> SQ + SQB +G 
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,TG,SB,MS,MG,MS2,MG2,M2,CF
      REAL*8 M2QBD1,M2QBD2,M2QBD3,M2QBD4,M2QBD
      REAL*8 NS,CONV,N,AVG,BETA,XS,DEL,S4MAX,DLDEL1,DLDEL2
      REAL*8 COLO1(1:9)
      INTEGER IFL

      NS = 6.D0
      CONV = 389379660.D0
      N = 3.D0
      CF = (N**2-1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N**2

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 -MS2
      TG = T1 -M2
      SB = S*SQRT(1 -4.D0*MS2/S)
      BETA = SQRT(1 -4*MS2/S)
      XS = (1.D0 -BETA)/(1.D0 +BETA)

      DLDEL1 = LOG(S4MAX/MS2) -(S4MAX-DEL)/S4
      DLDEL2 = LOG(S4MAX/MS2)**2 -2.D0*(S4MAX -DEL)/S4*LOG(S4/MS2)

      COLO1(1) = LOG(XS)
      COLO1(3) = LOG(S/MS2)
      COLO1(4) = LOG(-T1/MS2)
      COLO1(8) = LOG((S+T1)/MS2)

      IF (IFL.EQ.1) THEN

      M2QBD1 = 0.D0
      M2QBD1 = M2QBD1 + N*CF**2*DLDEL1 * ( 32*S**(-2)*T1**2 + 32*
     +    S**(-1)*T1 + 32*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + N*CF**2*DLDEL2 * (  - 32*S**(-2)*T1**2 - 32*
     +    S**(-1)*T1 - 32*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(1)*N*CF**2*DLDEL1 * (  - 64*S**(-2)*T1**2
     +    *SB**(-1)*MS2 - 64*S**(-1)*T1*SB**(-1)*MS2 + 32*S**(-1)*T1**2
     +    *SB**(-1) - 64*S**(-1)*SB**(-1)*MS2**2 + 32*T1*SB**(-1) + 32*
     +    SB**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(1)*N**2*CF*DLDEL1 * ( 32*S**(-2)*T1**2*
     +    SB**(-1)*MS2 + 32*S**(-1)*T1*SB**(-1)*MS2 - 16*S**(-1)*T1**2*
     +    SB**(-1) + 32*S**(-1)*SB**(-1)*MS2**2 - 16*T1*SB**(-1) - 16*
     +    SB**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(3)*N*CF**2*DLDEL1 * (  - 32*S**(-2)*T1**2
     +     - 32*S**(-1)*T1 - 32*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(3)*N**2*CF*DLDEL1 * ( 16*S**(-2)*T1**2 + 
     +    16*S**(-1)*T1 + 16*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(4)*N*CF**2*DLDEL1 * (  - 96*S**(-2)*T1**2
     +     - 96*S**(-1)*T1 - 96*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(4)*N**2*CF*DLDEL1 * ( 32*S**(-2)*T1**2 + 
     +    32*S**(-1)*T1 + 32*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(8)*N*CF**2*DLDEL1 * ( 160*S**(-2)*T1**2
     +     + 160*S**(-1)*T1 + 160*S**(-1)*MS2 )
     +
      M2QBD1 = M2QBD1 + COLO1(8)*N**2*CF*DLDEL1 * (  - 64*S**(-2)*T1**2
     +     - 64*S**(-1)*T1 - 64*S**(-1)*MS2 )

      M2QBD4 = 0.D0
      M2QBD4 = M2QBD4 + CF**2*DLDEL1 * (  - 32 - 32*M2*TG**(-1) - 32*M2
     +    *S**(-1) - 32*M2**2*TG**(-1)*S**(-1) - 32*TG**(-1)*MS2 - 32*
     +    S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + CF**2*DLDEL2 * ( 32 + 32*M2*TG**(-1) + 32*M2*
     +    S**(-1) + 32*M2**2*TG**(-1)*S**(-1) + 32*TG**(-1)*MS2 + 32*
     +    S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + COLO1(1)*N*CF*DLDEL1 * ( 16*M2*TG**(-1)*S*
     +    SB**(-1) - 32*M2*TG**(-1)*SB**(-1)*MS2 - 32*M2*S**(-1)*
     +    SB**(-1)*MS2 + 16*M2*SB**(-1) - 32*M2**2*TG**(-1)*S**(-1)*
     +    SB**(-1)*MS2 + 16*M2**2*TG**(-1)*SB**(-1) + 16*TG**(-1)*S*
     +    SB**(-1)*MS2 - 32*TG**(-1)*SB**(-1)*MS2**2 - 32*S**(-1)*T1*
     +    SB**(-1)*MS2 + 16*S*SB**(-1) + 16*T1*SB**(-1) - 32*SB**(-1)*
     +    MS2 )
     +
      M2QBD4 = M2QBD4 + COLO1(1)*CF**2*DLDEL1 * (  - 32*M2*TG**(-1)*S*
     +    SB**(-1) + 64*M2*TG**(-1)*SB**(-1)*MS2 + 64*M2*S**(-1)*
     +    SB**(-1)*MS2 - 32*M2*SB**(-1) + 64*M2**2*TG**(-1)*S**(-1)*
     +    SB**(-1)*MS2 - 32*M2**2*TG**(-1)*SB**(-1) - 32*TG**(-1)*S*
     +    SB**(-1)*MS2 + 64*TG**(-1)*SB**(-1)*MS2**2 + 64*S**(-1)*T1*
     +    SB**(-1)*MS2 - 32*S*SB**(-1) - 32*T1*SB**(-1) + 64*SB**(-1)*
     +    MS2 )
     +
      M2QBD4 = M2QBD4 + COLO1(3)*N*CF*DLDEL1 * (  - 16 - 16*M2*TG**(-1)
     +     - 16*M2*S**(-1) - 16*M2**2*TG**(-1)*S**(-1) - 16*TG**(-1)*
     +    MS2 - 16*S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + COLO1(3)*CF**2*DLDEL1 * ( 32 + 32*M2*TG**(-1)
     +     + 32*M2*S**(-1) + 32*M2**2*TG**(-1)*S**(-1) + 32*TG**(-1)*
     +    MS2 + 32*S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + COLO1(4)*N*CF*DLDEL1 * (  - 32 - 32*M2*TG**(-1)
     +     - 32*M2*S**(-1) - 32*M2**2*TG**(-1)*S**(-1) - 32*TG**(-1)*
     +    MS2 - 32*S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + COLO1(4)*CF**2*DLDEL1 * ( 32 + 32*M2*TG**(-1)
     +     + 32*M2*S**(-1) + 32*M2**2*TG**(-1)*S**(-1) + 32*TG**(-1)*
     +    MS2 + 32*S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + COLO1(8)*N*CF*DLDEL1 * ( 64 + 64*M2*TG**(-1) + 
     +    64*M2*S**(-1) + 64*M2**2*TG**(-1)*S**(-1) + 64*TG**(-1)*MS2
     +     + 64*S**(-1)*T1 )
     +
      M2QBD4 = M2QBD4 + COLO1(8)*CF**2*DLDEL1 * (  - 96 - 96*M2*
     +    TG**(-1) - 96*M2*S**(-1) - 96*M2**2*TG**(-1)*S**(-1) - 96*
     +    TG**(-1)*MS2 - 96*S**(-1)*T1 )

      END IF

      M2QBD2 = 0.D0
      M2QBD2 = M2QBD2 + N*CF**2*DLDEL1 * ( 16 + 16*M2*TG**(-2)*S + 32*
     +    M2*TG**(-1) + 16*M2**2*TG**(-2) + 16*TG**(-2)*S*MS2 + 16*
     +    TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + N*CF**2*DLDEL2 * (  - 16 - 16*M2*TG**(-2)*S - 
     +    32*M2*TG**(-1) - 16*M2**2*TG**(-2) - 16*TG**(-2)*S*MS2 - 16*
     +    TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + COLO1(1)*N*CF**2*DLDEL1 * (  - 64*M2*TG**(-2)*S
     +    *SB**(-1)*MS2 + 32*M2*TG**(-2)*S**2*SB**(-1) + 64*M2*TG**(-1)
     +    *S*SB**(-1) - 128*M2*TG**(-1)*SB**(-1)*MS2 + 32*M2**2*
     +    TG**(-2)*S*SB**(-1) - 64*M2**2*TG**(-2)*SB**(-1)*MS2 - 64*
     +    TG**(-2)*S*SB**(-1)*MS2**2 + 32*TG**(-2)*S**2*SB**(-1)*MS2 - 
     +    64*TG**(-1)*S*SB**(-1)*MS2 + 32*TG**(-1)*S**2*SB**(-1) + 32*S
     +    *SB**(-1) - 64*SB**(-1)*MS2 )
     +
      M2QBD2 = M2QBD2 + COLO1(1)*N**2*CF*DLDEL1 * ( 16*M2*TG**(-2)*S*
     +    SB**(-1)*MS2 - 8*M2*TG**(-2)*S**2*SB**(-1) - 16*M2*TG**(-1)*S
     +    *SB**(-1) + 32*M2*TG**(-1)*SB**(-1)*MS2 - 8*M2**2*TG**(-2)*S*
     +    SB**(-1) + 16*M2**2*TG**(-2)*SB**(-1)*MS2 + 16*TG**(-2)*S*
     +    SB**(-1)*MS2**2 - 8*TG**(-2)*S**2*SB**(-1)*MS2 + 16*TG**(-1)*
     +    S*SB**(-1)*MS2 - 8*TG**(-1)*S**2*SB**(-1) - 8*S*SB**(-1) + 16
     +    *SB**(-1)*MS2 )
     +
      M2QBD2 = M2QBD2 + COLO1(3)*N*CF**2*DLDEL1 * (  - 32 - 32*M2*
     +    TG**(-2)*S - 64*M2*TG**(-1) - 32*M2**2*TG**(-2) - 32*TG**(-2)
     +    *S*MS2 - 32*TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + COLO1(3)*N**2*CF*DLDEL1 * ( 8 + 8*M2*TG**(-2)*S
     +     + 16*M2*TG**(-1) + 8*M2**2*TG**(-2) + 8*TG**(-2)*S*MS2 + 8*
     +    TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + COLO1(4)*N*CF**2*DLDEL1 * (  - 16 - 16*M2*
     +    TG**(-2)*S - 32*M2*TG**(-1) - 16*M2**2*TG**(-2) - 16*TG**(-2)
     +    *S*MS2 - 16*TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + COLO1(4)*N**2*CF*DLDEL1 * ( 16 + 16*M2*TG**(-2)
     +    *S + 32*M2*TG**(-1) + 16*M2**2*TG**(-2) + 16*TG**(-2)*S*MS2
     +     + 16*TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + COLO1(8)*N*CF**2*DLDEL1 * ( 80 + 80*M2*TG**(-2)
     +    *S + 160*M2*TG**(-1) + 80*M2**2*TG**(-2) + 80*TG**(-2)*S*MS2
     +     + 80*TG**(-1)*S )
     +
      M2QBD2 = M2QBD2 + COLO1(8)*N**2*CF*DLDEL1 * (  - 32 - 32*M2*
     +    TG**(-2)*S - 64*M2*TG**(-1) - 32*M2**2*TG**(-2) - 32*TG**(-2)
     +    *S*MS2 - 32*TG**(-1)*S )


      M2QBD3 = 0.D0
      M2QBD3 = M2QBD3 + N*CF**2*DLDEL1 * (  - 16*M2*TG**(-2)*S - 16*
     +    TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + N*CF**2*DLDEL2 * ( 16*M2*TG**(-2)*S + 16*
     +    TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(1)*N*CF**2*DLDEL1 * (  - 32*M2*TG**(-2)*S
     +    *T1*SB**(-1) + 96*M2*TG**(-2)*S*SB**(-1)*MS2 - 32*M2*TG**(-2)
     +    *S**2*SB**(-1) + 32*M2*TG**(-1)*S*SB**(-1) + 32*M2**2*
     +    TG**(-2)*S*SB**(-1) - 32*TG**(-2)*S*T1*SB**(-1)*MS2 + 64*
     +    TG**(-2)*S*SB**(-1)*MS2**2 - 32*TG**(-2)*S**2*SB**(-1)*MS2 + 
     +    32*TG**(-1)*S*SB**(-1)*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(1)*N**2*CF*DLDEL1 * ( 8*M2*TG**(-2)*S*T1*
     +    SB**(-1) - 24*M2*TG**(-2)*S*SB**(-1)*MS2 + 8*M2*TG**(-2)*S**2
     +    *SB**(-1) - 8*M2*TG**(-1)*S*SB**(-1) - 8*M2**2*TG**(-2)*S*
     +    SB**(-1) + 8*TG**(-2)*S*T1*SB**(-1)*MS2 - 16*TG**(-2)*S*
     +    SB**(-1)*MS2**2 + 8*TG**(-2)*S**2*SB**(-1)*MS2 - 8*TG**(-1)*S
     +    *SB**(-1)*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(3)*N*CF**2*DLDEL1 * ( 32*M2*TG**(-2)*S + 
     +    32*TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(3)*N**2*CF*DLDEL1 * (  - 8*M2*TG**(-2)*S
     +     - 8*TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(4)*N*CF**2*DLDEL1 * ( 16*M2*TG**(-2)*S + 
     +    16*TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(4)*N**2*CF*DLDEL1 * (  - 16*M2*TG**(-2)*S
     +     - 16*TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(8)*N*CF**2*DLDEL1 * (  - 80*M2*TG**(-2)*S
     +     - 80*TG**(-2)*S*MS2 )
     +
      M2QBD3 = M2QBD3 + COLO1(8)*N**2*CF*DLDEL1 * ( 32*M2*TG**(-2)*S + 
     +    32*TG**(-2)*S*MS2 )

      
      IF (IFL.EQ.0) M2QBD = 2.D0*M2QBD2 +2.D0*M2QBD3 
      IF (IFL.EQ.1) M2QBD = 
     +     2.D0*(NS-1.D0)*M2QBD1 +2.D0*M2QBD2 +2.D0*M2QBD3 +2.D0*M2QBD4 
           
      DSBQBD = ALPHAS**3 * AVG * M2QBD /4.D0 /S**2 *CONV

      RETURN
      END




      REAL*8 FUNCTION DSBGB3(ALPHAS,S,T1,S4,MS,MG,SCA)
C***  GIVES THE SCALE DEPENDENCE OF HARD
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,TG,U1,S4,MS,MG,MS2,MG2,M2,SCA,N,CF,NS,CONV,AVG
      REAL*8 M2GB31, M2GB32, M2GB33, M2GB34, M2GB3

      N = 3.D0
      CF = (N**2 -1.D0)/2.D0/N
      NS = 6.D0
      AVG = (1.D0/2.D0)**2 /N/(N**2 -1.D0)
      CONV = 389379660.D0

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 -MS2
      TG = T1 -M2
      U1 = S4 -S -T1

      M2GB31 = 0.D0
      M2GB31 = M2GB31 + N*CF**2 * ( 16*S**(-2)*T1**(-1)*U1**2 + 32*
     +    S**(-2)*T1 + 32*S**(-2)*U1 - 16*S**(-1)*T1**(-2)*U1*MS2 + 16*
     +    S**(-1)*T1**(-1)*U1 - 32*S**(-1)*T1**(-1)*MS2 - 32*S**(-1)*T1
     +    *(S+U1)**(-1) - 32*S**(-1)*(S+U1)**(-1)*MS2 - 64*S*T1**(-1)*
     +    U1**(-2)*MS2 - 32*S*T1**(-1)*U1**(-1)*(S+T1)**(-1)*MS2 - 64*S
     +    *U1**(-3)*MS2 + 32*S*U1**(-2) + 64*S**2*T1**(-2)*U1**(-3)*
     +    MS2**2 + 32*S**2*T1**(-2)*U1**(-2)*(S+T1)**(-1)*MS2**2 + 64*
     +    S**2*T1**(-1)*U1**(-4)*MS2**2 - 64*S**2*T1**(-1)*U1**(-3)*MS2
     +     + 64*S**3*T1**(-2)*U1**(-4)*MS2**2 - 16*T1**(-2)*MS2 + 32*T1
     +    *U1**(-2) + 32*U1**(-1) + 16*(S+T1)**(-1) )
     +
      M2GB31 = M2GB31 + N**2*CF * ( 16*S**(-2)*T1 + 32*S**(-2)*T1**2*
     +    U1**(-1) + 32*S**(-2)*T1**3*U1**(-2) - 64*S**(-1)*T1*U1**(-2)
     +    *MS2 + 32*S**(-1)*T1*U1**(-1) - 64*S**(-1)*T1**2*U1**(-3)*MS2
     +     + 64*S**(-1)*T1**2*U1**(-2) - 32*S**(-1)*U1**(-1)*MS2 + 64*S
     +    *T1**(-1)*U1**(-3)*MS2**2 + 128*S*U1**(-4)*MS2**2 - 64*S*
     +    U1**(-3)*MS2 + 64*S**2*T1**(-1)*U1**(-4)*MS2**2 + 32*T1**(-1)
     +    *U1**(-2)*MS2**2 + 64*T1*U1**(-4)*MS2**2 - 128*T1*U1**(-3)*
     +    MS2 + 32*T1*U1**(-2) + 64*U1**(-3)*MS2**2 - 64*U1**(-2)*MS2 )

      M2GB32 = 0.D0
      M2GB32 = M2GB32 + N*CF**2 * (  - 16*M2*TG**(-2)*S*(S+U1)**(-2)*
     +    MS2 - 8*M2*TG**(-2)*S*(S+U1)**(-1) + 8*M2*TG**(-2) - 32*M2*
     +    TG**(-1)*S*(S+U1)**(-3)*MS2 - 32*M2*TG**(-1)*S*(S+U1)**(-2)
     +     + 32*M2*TG**(-1)*(S+U1)**(-1) - 32*M2*S*(S+U1)**(-3) + 32*M2
     +    *(S+U1)**(-2) - 16*M2**2*TG**(-2)*S*(S+U1)**(-3)*MS2 - 16*
     +    M2**2*TG**(-2)*S*(S+U1)**(-2) + 16*M2**2*TG**(-2)*
     +    (S+U1)**(-1) - 48*M2**2*TG**(-1)*S*(S+U1)**(-3) + 48*M2**2*
     +    TG**(-1)*(S+U1)**(-2) - 16*M2**3*TG**(-2)*S*(S+U1)**(-3) + 16
     +    *M2**3*TG**(-2)*(S+U1)**(-2) - 8*TG**(-2)*S*(S+U1)**(-1)*MS2
     +     - 16*TG**(-1)*S*(S+U1)**(-2)*MS2 - 8*TG**(-1)*S*(S+U1)**(-1)
     +     + 8*TG**(-1) - 16*S*T1*(S+U1)**(-3) - 16*S*(S+U1)**(-3)*MS2
     +     - 16*S*(S+U1)**(-2) + 16*T1*(S+U1)**(-2) + 16*(S+U1)**(-1) )

      M2GB33 = 0.D0
      M2GB33 = M2GB33 + N*CF**2 * ( 16*M2*TG**(-2)*S*(S+U1)**(-2)*MG2
     +     + 32*M2*TG**(-1)*S*(S+U1)**(-3)*MG2 + 16*M2**2*TG**(-2)*S*
     +    (S+U1)**(-3)*MG2 + 8*TG**(-2)*S*(S+U1)**(-1)*MG2 + 16*
     +    TG**(-1)*S*(S+U1)**(-2)*MG2 + 16*S*(S+U1)**(-3)*MG2 )

      M2GB34 =0.D0
      M2GB34 = M2GB34 + CF**2 * ( 32*M2*TG**(-1)*S**(-1) - 32*M2*
     +    TG**(-1)*(S+U1)**(-2)*MS2 - 32*M2*TG**(-1)*(S+U1)**(-1) + 32*
     +    M2*S**(-1)*(S+U1)**(-1) - 32*M2*(S+U1)**(-2) + 32*M2**2*
     +    TG**(-1)*S**(-1)*(S+U1)**(-1) - 32*M2**2*TG**(-1)*
     +    (S+U1)**(-2) + 16*TG**(-1)*S**(-1)*U1 - 16*TG**(-1)*T1**(-1)*
     +    MS2 - 32*TG**(-1)*(S+U1)**(-1)*MS2 + 32*S**(-1)*T1*
     +    (S+U1)**(-1) + 32*S**(-1) - 32*T1*(S+U1)**(-2) - 32*
     +    (S+U1)**(-2)*MS2 - 32*(S+U1)**(-1) )

      M2GB3 = 2.D0*((NS-1.D0)*(M2GB31 +M2GB32 +M2GB33) +M2GB34)
      DSBGB3 = ALPHAS**3 * AVG *M2GB3 *LOG(1.D0/SCA)/8.D0/S**2  *CONV
      RETURN
      END


      REAL*8 FUNCTION DSBGBH(ALPHAS,S,T1,S4,MS,MG,EPS)
C***  CROSS SECTIONS FOR G + QB -> SQ + SQB +QB
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,TG,U1,MS,MG,MS2,MG2,M2,EPS
      REAL*8 NS,CONV,N,CF,AVG,S4G, S4G2
      REAL*8 ANGDEF(1:11), ANA(1:3,1:9), ANB(1:3,1:9), ANC(1:3,1:9)
      REAL*8 AHP1P1, ABP1P1, ABP1M1, ABP1P2, A4P1P1
      REAL*8 A4M1P2, A4M1P1, A4P0P2, A4P1P0, A4M2P1,ABP2P2
      REAL*8 ABP2P1, ABP2P0, ABP1M2
      REAL*8 M2GBH1,M2GBH2,M2GBH3,M2GBH4,M2GBH
      REAL*8 ANG4(1:84), COLO1(1:9)
           
      CONV = 389379660.D0
      NS = 6.D0

      N = 3.D0
      CF = (N**2 -1.D0)/N/2.D0

      AVG = (1.D0/2.D0)**2 /N /(N**2 -1.D0)

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 - MS2
      U1 = S4 -S -T1
      TG = T1 -M2
      S4G = S4 -M2
      S4G2 = S4G**2 + EPS*MG**4

      ANGDEF(1) = 2.D0*SQRT(S4 +MS2)
      ANGDEF(2) = (S +U1)/ANGDEF(1)
      ANGDEF(3) = (S +T1)/ANGDEF(1)
      ANGDEF(4) = S4/ANGDEF(1)
      ANGDEF(5) = (S4 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(6) = -(T1 +U1 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(7) = SQRT((T1 +U1)**2 -4.D0*MS2*S)/ANGDEF(1)
      ANGDEF(8) = 
     +  (T1*S4 -S*(U1+2.D0*MS2))/(S+T1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(9) = SQRT(1 -ANGDEF(8)**2)
      ANGDEF(10) =
     +  (U1*S4 -S*(T1+2.D0*MS2))/(S+U1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(11) = SQRT(1 -ANGDEF(10)**2)

      ANA(1,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)
      ANC(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(1,2) = -ANB(1,1)
      ANC(1,2) = -ANC(1,1)
      ANA(1,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(1,3) = -ANA(1,3)
      ANC(1,3) =  0.D0
      ANA(1,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(1,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)-2.D0*ANGDEF(3)*ANGDEF(4)
      ANC(1,4) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(1,5) = -ANB(1,3)
      ANC(1,5) = -ANC(1,3)
      ANA(1,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(1,6) = -ANB(1,4)
      ANC(1,6) = -ANC(1,4)
      ANA(1,7) = +ANA(1,1) -M2
      ANB(1,7) = +ANB(1,1)
      ANC(1,7) = +ANC(1,1)
      ANA(1,8) = +ANA(1,5) -M2
      ANB(1,8) = +ANB(1,5)
      ANC(1,8) = +ANC(1,5)
      ANA(1,9) = +ANA(1,6) -M2
      ANB(1,9) = +ANB(1,6)
      ANC(1,9) = +ANC(1,6)


      ANA(2,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(2,1) = -2.D0*ANGDEF(4)*ANGDEF(7)
      ANC(2,1) =  0.D0
      ANA(2,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(2,2) = -ANB(2,1)
      ANC(2,2) = -ANC(2,1)
      ANA(2,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(2,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7) -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,4) = -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(2,5) = -ANB(2,3)
      ANC(2,5) = -ANC(2,3)
      ANA(2,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(2,6) = -ANB(2,4)
      ANC(2,6) = -ANC(2,4)
      ANA(2,7) = +ANA(2,1) -M2
      ANB(2,7) = +ANB(2,1)
      ANC(2,7) = +ANC(2,1)
      ANA(2,8) = +ANA(2,5) -M2
      ANB(2,8) = +ANB(2,5)
      ANC(2,8) = +ANC(2,5)
      ANA(2,9) = +ANA(2,6) -M2
      ANB(2,9) = +ANB(2,6)
      ANC(2,9) = +ANC(2,6)


      ANA(3,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10)
      ANC(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(3,2) = -ANB(3,1)
      ANC(3,2) = -ANC(3,1)
      ANA(3,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(3,3) = 
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10) -2.D0*ANGDEF(2)*ANGDEF(4)
      ANC(3,3) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(3,4) = -ANA(3,4)
      ANC(3,4) =  0.D0
      ANA(3,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(3,5) = -ANB(3,3)
      ANC(3,5) = -ANC(3,3)
      ANA(3,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(3,6) = -ANB(3,4)
      ANC(3,6) = -ANC(3,4)
      ANA(3,7) = +ANA(3,1) -M2
      ANB(3,7) = +ANB(3,1)
      ANC(3,7) = +ANC(3,1)
      ANA(3,8) = +ANA(3,5) -M2
      ANB(3,8) = +ANB(3,5)
      ANC(3,8) = +ANC(3,5)
      ANA(3,9) = +ANA(3,6) -M2
      ANB(3,9) = +ANB(3,6)
      ANC(3,9) = +ANC(3,6)


C$$$      ANG4(1) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(2) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(3) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(4) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(5) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(6) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(7) =A4P1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(8) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(9) =A4P2M2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(10) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))

C$$$      ANG4(11) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(12) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(13) =A4P2M2(ANA(2,2),ANB(2,2),ANA(2,6),ANB(2,6),ANC(2,6))
      ANG4(14) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(15) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(16) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(17) =A4P1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(18) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(19) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(20) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,5),ANB(3,5),ANC(3,5))

      ANG4(21) =A4M1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
      ANG4(22) =A4M1P2(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
      ANG4(23) =A4M2P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(24) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(25) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(26) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,7),ANB(1,7),ANC(1,7))
C$$$      ANG4(27) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(28) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(29) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(30) =A4P1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))

C$$$      ANG4(31) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(32) =A4M1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(33) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(34) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(35) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
C$$$      ANG4(36) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(37) =A4P1P0(ANA(2,1),ANB(2,1),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(38) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(39) =A4P1P0(ANA(2,2),ANB(2,2),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(40) =A4P0P2(ANA(1,1),ANB(1,1),ANA(2,5),ANB(2,5),ANC(2,5))

C$$$      ANG4(41) =A4P1P0(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))

      ANG4(43) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,6),ANB(1,6),ANC(1,6))
      ANG4(44) =A4P1P0(ANA(3,6),ANB(3,6),ANA(1,1),ANB(1,1),ANC(1,1))

      ANG4(48) =AHP1P1(ANA(1,3),ANB(1,3),ANA(1,4),ANB(1,4),ANC(1,4))
C$$$      ANG4(49) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(50) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,1),ANB(3,1),ANC(3,1))

      ANG4(51) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(52) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(53) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
      ANG4(54) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(55) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(56) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(57) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(58) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
      ANG4(59) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(60) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 

      ANG4(61) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(62) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
      ANG4(63) =ABP1M1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6)) 
C$$$      ANG4(64) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(65) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(66) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(67) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
      ANG4(68) =ABP1M1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5)) 
      ANG4(69) =ABP1M2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
      ANG4(70) =ABP2P0(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))

C$$$      ANG4(71) =ABP2P0(ANA(3,4),ANB(3,4),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(72) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(73) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(74) =A4P1P2(ANA(3,9),ANB(3,9),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(75) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(76) =A4P1P1(ANA(3,9),ANB(3,9),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(77) =A4M1P2(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
C$$$      ANG4(78) =A4M1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
C$$$      ANG4(79) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,9),ANB(1,9),ANC(1,9))
C$$$      ANG4(80) =A4P1P0(ANA(3,9),ANB(3,9),ANA(1,1),ANB(1,1),ANC(1,1))

C$$$      ANG4(81) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,9),ANB(1,9),ANC(1,9)) 
C$$$      ANG4(82) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,9),ANB(1,9),ANC(1,9)) 
C$$$      ANG4(83) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,7),ANB(1,7),ANC(1,7))
C$$$      ANG4(84) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,7),ANB(3,7),ANC(3,7))

      COLO1(9) = LOG(S4**2/MS2/(S4+MS2))

      M2GBH1 = 0.D0
      M2GBH1 = M2GBH1 + N*(S4+MS2)*CF**2 * ( 128*M2*TG*S**(-1)*U1**(-2)
     +    *S4**(-1) - 576*M2*TG*S**(-1)*U1**(-1)*S4**(-2) + 128*M2*TG*
     +    S**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 64*M2*TG*S**(-1)*
     +    S4**(-2)*(S+U1)**(-1) - 128*M2*TG*U1**(-2)*S4**(-1)*
     +    (S+U1)**(-1) - 384*M2*TG**2*S**(-1)*U1**(-1)*S4**(-2)*
     +    (S+U1)**(-1) - 32*M2*S**(-1)*T1**(-1)*S4**(-1) - 64*M2*
     +    S**(-1)*U1**(-1)*S4**(-2)*MG2 + 128*M2*S**(-1)*U1**(-1)*
     +    S4**(-1) + 64*M2*S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 - 128*M2*
     +    S**(-1)*S4**(-2) + 64*M2*S*T1**(-2)*U1**(-2)*S4**(-1)*MG2 - 
     +    128*M2*S*T1**(-1)*U1**(-3) + 192*M2*S*T1**(-1)*U1**(-2)*
     +    S4**(-1) + 64*M2*S*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 
     +    192*M2*S*U1**(-3)*S4**(-1) + 384*M2*S**2*T1**(-2)*U1**(-4)*
     +    MG2 - 512*M2*S**2*T1**(-2)*U1**(-3)*S4**(-1)*MG2 - 128*M2*
     +    S**2*T1**(-2)*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MG2 - 512*M2*
     +    S**2*T1**(-1)*U1**(-4)*S4**(-1)*MG2 + 192*M2*S**2*T1**(-1)*
     +    U1**(-3)*S4**(-1) )
     +
      M2GBH1 = M2GBH1 + N*(S4+MS2)*CF**2 * (  - 512*M2*S**3*T1**(-2)*
     +    U1**(-4)*S4**(-1)*MG2 - 64*M2*T1**(-1)*U1**(-2)*S4**(-1)*MG2
     +     - 32*M2*T1**(-1)*U1**(-1)*S4**(-1) + 64*M2*U1**(-2)*S4**(-1)
     +    *(S+T1)**(-1)*MG2 - 192*M2*U1**(-1)*S4**(-2) + 32*M2*U1**(-1)
     +    *S4**(-1)*(S+T1)**(-1) - 384*M2**2*TG*S**(-1)*U1**(-1)*
     +    S4**(-2)*(S+U1)**(-1) + 64*M2**2*S**(-1)*U1**(-2)*S4**(-1) - 
     +    256*M2**2*S**(-1)*U1**(-1)*S4**(-2) + 64*M2**2*S**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) - 32*M2**2*S*T1**(-2)*U1**(-2)
     +    *S4**(-1) - 192*M2**2*S**2*T1**(-2)*U1**(-4) + 256*M2**2*S**2
     +    *T1**(-2)*U1**(-3)*S4**(-1) + 64*M2**2*S**2*T1**(-2)*U1**(-2)
     +    *S4**(-1)*(S+T1)**(-1) + 256*M2**2*S**2*T1**(-1)*U1**(-4)*
     +    S4**(-1) + 256*M2**2*S**3*T1**(-2)*U1**(-4)*S4**(-1) + 32*
     +    M2**2*T1**(-1)*U1**(-2)*S4**(-1) - 32*M2**2*U1**(-2)*S4**(-1)
     +    *(S+T1)**(-1) - 64*M2**2*U1**(-2)*S4**(-1)*(S+U1)**(-1) - 128
     +    *M2**3*S**(-1)*U1**(-1)*S4**(-2)*(S+U1)**(-1) - 64*TG*S**(-1)
     +    *U1**(-1)*S4**(-2)*MG2 )
     +
      M2GBH1 = M2GBH1 + N*(S4+MS2)*CF**2 * ( 96*TG*S**(-1)*U1**(-1)*
     +    S4**(-1) + 64*TG*S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 - 128*TG*
     +    S**(-1)*S4**(-2) + 64*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) - 256*
     +    TG*U1**(-1)*S4**(-2) + 96*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1)
     +     + 64*TG**2*S**(-1)*U1**(-2)*S4**(-1) - 320*TG**2*S**(-1)*
     +    U1**(-1)*S4**(-2) + 64*TG**2*S**(-1)*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 64*TG**2*S**(-1)*S4**(-2)*(S+U1)**(-1) - 64*
     +    TG**2*U1**(-2)*S4**(-1)*(S+U1)**(-1) - 128*TG**3*S**(-1)*
     +    U1**(-1)*S4**(-2)*(S+U1)**(-1) - 32*S**(-2) + 32*S**(-1)*
     +    T1**(-1)*S4**(-1)*MG2 - 32*S**(-1)*U1**(-1)*S4**(-1)*MG2 + 64
     +    *S**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 32*S**(-1)*S4**(-1) - 32
     +    *S*T1**(-2)*U1**(-2)*S4**(-1)*MG2**2 + 128*S*T1**(-1)*
     +    U1**(-3)*MG2 - 192*S*T1**(-1)*U1**(-2)*S4**(-1)*MG2 - 64*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 192*S*U1**(-3)*
     +    S4**(-1)*MG2 - 64*S*U1**(-1)*S4**(-2) + 32*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + N*(S4+MS2)*CF**2 * (  - 192*S**2*T1**(-2)*
     +    U1**(-4)*MG2**2 + 256*S**2*T1**(-2)*U1**(-3)*S4**(-1)*MG2**2
     +     + 64*S**2*T1**(-2)*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MG2**2 + 
     +    256*S**2*T1**(-1)*U1**(-4)*S4**(-1)*MG2**2 - 192*S**2*
     +    T1**(-1)*U1**(-3)*S4**(-1)*MG2 + 256*S**3*T1**(-2)*U1**(-4)*
     +    S4**(-1)*MG2**2 + 32*T1**(-1)*U1**(-2)*S4**(-1)*MG2**2 + 32*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2 - 32*U1**(-2)*S4**(-1)*
     +    (S+T1)**(-1)*MG2**2 - 64*U1**(-1)*S4**(-2)*MG2 - 32*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 + 96*U1**(-1)*S4**(-1)*(S+U1)**(-1)
     +    *MG2 + 32*U1**(-1)*S4**(-1) - 64*S4**(-2) + 16*S4**(-1)*
     +    (S+T1)**(-1) + 32*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + N**2*(S4+MS2)*CF * (  - 64*M2*TG*S**(-2)*
     +    U1**(-1)*S4**(-1) + 64*M2*TG*S**(-2)*S4**(-2) - 384*M2*TG*
     +    S**(-1)*U1**(-3)*S4**(-1)*MG2 - 128*M2*TG*S**(-1)*U1**(-3) + 
     +    160*M2*TG*S**(-1)*U1**(-2)*S4**(-1) + 288*M2*TG*S**(-1)*
     +    U1**(-1)*S4**(-2) - 32*M2*TG*S**(-1)*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 16*M2*TG*S**(-1)*S4**(-2)*(S+U1)**(-1) - 256*
     +    M2*TG*U1**(-4)*S4**(-1)*MG2 + 256*M2*TG*U1**(-3)*S4**(-1) + 
     +    32*M2*TG*U1**(-2)*S4**(-1)*(S+U1)**(-1) + 96*M2*TG**2*S**(-2)
     +    *U1**(-1)*S4**(-2) + 192*M2*TG**2*S**(-1)*U1**(-3)*S4**(-1)
     +     + 96*M2*TG**2*S**(-1)*U1**(-1)*S4**(-2)*(S+U1)**(-1) + 16*M2
     +    *S**(-2)*S4**(-1) + 128*M2*S**(-1)*U1**(-3)*MG2 - 192*M2*
     +    S**(-1)*U1**(-2)*S4**(-1)*MG2 + 32*M2*S**(-1)*U1**(-1)*
     +    S4**(-2)*MG2 - 32*M2*S**(-1)*U1**(-1)*S4**(-1) - 16*M2*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 + 64*M2*S**(-1)*S4**(-2) - 
     +    128*M2*S*T1**(-1)*U1**(-3)*S4**(-1)*MG2 - 384*M2*S*U1**(-4)*
     +    S4**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + N**2*(S4+MS2)*CF * ( 64*M2*S*U1**(-3)*S4**(-1)
     +     - 128*M2*S**2*T1**(-1)*U1**(-4)*S4**(-1)*MG2 - 64*M2*
     +    T1**(-1)*U1**(-2)*S4**(-1)*MG2 + 128*M2*U1**(-4)*S4**(-1)*
     +    MG2**2 + 128*M2*U1**(-4)*MG2 - 512*M2*U1**(-3)*S4**(-1)*MG2
     +     + 64*M2*U1**(-2)*S4**(-1) + 96*M2*U1**(-1)*S4**(-2) + 96*
     +    M2**2*TG*S**(-2)*U1**(-1)*S4**(-2) + 384*M2**2*TG*S**(-1)*
     +    U1**(-3)*S4**(-1) + 96*M2**2*TG*S**(-1)*U1**(-1)*S4**(-2)*
     +    (S+U1)**(-1) + 128*M2**2*TG*U1**(-4)*S4**(-1) - 32*M2**2*
     +    S**(-2)*U1**(-1)*S4**(-1) + 32*M2**2*S**(-2)*S4**(-2) - 192*
     +    M2**2*S**(-1)*U1**(-3)*S4**(-1)*MG2 - 128*M2**2*S**(-1)*
     +    U1**(-3) + 176*M2**2*S**(-1)*U1**(-2)*S4**(-1) + 128*M2**2*
     +    S**(-1)*U1**(-1)*S4**(-2) - 16*M2**2*S**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 64*M2**2*S*T1**(-1)*U1**(-3)*S4**(-1)
     +     + 192*M2**2*S*U1**(-4)*S4**(-1) + 64*M2**2*S**2*T1**(-1)*
     +    U1**(-4)*S4**(-1) + 32*M2**2*T1**(-1)*U1**(-2)*S4**(-1) - 256
     +    *M2**2*U1**(-4)*S4**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + N**2*(S4+MS2)*CF * (  - 64*M2**2*U1**(-4) + 384
     +    *M2**2*U1**(-3)*S4**(-1) + 16*M2**2*U1**(-2)*S4**(-1)*
     +    (S+U1)**(-1) + 32*M2**3*S**(-2)*U1**(-1)*S4**(-2) + 192*M2**3
     +    *S**(-1)*U1**(-3)*S4**(-1) + 32*M2**3*S**(-1)*U1**(-1)*
     +    S4**(-2)*(S+U1)**(-1) + 128*M2**3*U1**(-4)*S4**(-1) + 16*TG*
     +    S**(-2)*S4**(-1) + 128*TG*S**(-1)*U1**(-3)*MG2 - 192*TG*
     +    S**(-1)*U1**(-2)*S4**(-1)*MG2 + 32*TG*S**(-1)*U1**(-1)*
     +    S4**(-2)*MG2 - 56*TG*S**(-1)*U1**(-1)*S4**(-1) - 16*TG*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 + 80*TG*S**(-1)*S4**(-2) - 
     +    24*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) + 128*TG*U1**(-4)*
     +    S4**(-1)*MG2**2 - 256*TG*U1**(-3)*S4**(-1)*MG2 + 128*TG*
     +    U1**(-1)*S4**(-2) - 40*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 32
     +    *TG**2*S**(-2)*U1**(-1)*S4**(-1) + 32*TG**2*S**(-2)*S4**(-2)
     +     - 192*TG**2*S**(-1)*U1**(-3)*S4**(-1)*MG2 - 16*TG**2*S**(-1)
     +    *U1**(-2)*S4**(-1) + 160*TG**2*S**(-1)*U1**(-1)*S4**(-2) - 16
     +    *TG**2*S**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + N**2*(S4+MS2)*CF * (  - 16*TG**2*S**(-1)*
     +    S4**(-2)*(S+U1)**(-1) + 16*TG**2*U1**(-2)*S4**(-1)*
     +    (S+U1)**(-1) + 32*TG**3*S**(-2)*U1**(-1)*S4**(-2) + 32*TG**3*
     +    S**(-1)*U1**(-1)*S4**(-2)*(S+U1)**(-1) - 24*S**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 + 16*S**(-1)*S4**(-2)*MG2 - 24*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 64*S*T1**(-1)*U1**(-3)*S4**(-1)*MG2**2 + 
     +    192*S*U1**(-4)*S4**(-1)*MG2**2 - 64*S*U1**(-3)*S4**(-1)*MG2
     +     + 32*S*U1**(-1)*S4**(-2) - 16*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 64*S**2*T1**(-1)*U1**(-4)*S4**(-1)*MG2**2 + 32
     +    *T1**(-1)*U1**(-2)*S4**(-1)*MG2**2 - 64*U1**(-4)*MG2**2 + 128
     +    *U1**(-3)*S4**(-1)*MG2**2 - 64*U1**(-2)*S4**(-1)*MG2 + 32*
     +    U1**(-1)*S4**(-2)*MG2 - 40*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2
     +     - 16*U1**(-1)*S4**(-1) + 32*S4**(-2) - 16*S4**(-1)*
     +    (S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + ANG4(14)*N*CF**2 * ( 8*M2*S**(-1) - 8*S**(-1)*
     +    MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(17)*N*CF**2 * ( 64*M2*TG*S**(-1) + 64*M2*
     +    TG*U1**(-1) - 16*M2*TG*(S+U1)**(-1) + 96*M2*TG**2*S**(-1)*
     +    U1**(-1) - 96*M2*TG**2*U1**(-1)*(S+U1)**(-1) + 16*M2*S**(-1)*
     +    S4 - 64*M2*U1**(-1)*MG2 - 16*M2*(S+U1)**(-1)*MG2 + 32*M2 + 96
     +    *M2**2*TG*S**(-1)*U1**(-1) - 96*M2**2*TG*U1**(-1)*
     +    (S+U1)**(-1) + 32*M2**2*S**(-1) + 64*M2**2*U1**(-1) + 32*
     +    M2**3*S**(-1)*U1**(-1) - 32*M2**3*U1**(-1)*(S+U1)**(-1) + 32*
     +    TG*S**(-1)*S4 - 64*TG*U1**(-1)*MG2 - 16*TG*(S+U1)**(-1)*MG2
     +     - 16*TG + 32*TG**2*S**(-1) - 16*TG**2*(S+U1)**(-1) + 32*
     +    TG**3*S**(-1)*U1**(-1) - 32*TG**3*U1**(-1)*(S+U1)**(-1) + 16*
     +    S**(-1)*S4*MG2 + 16*S**(-1)*S4**2 - 16*S4 - 48*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(17)*N**2*CF * (  - 32*M2*TG*S**(-1) - 16*
     +    M2*TG*S*U1**(-2) - 64*M2*TG*U1**(-1) + 4*M2*TG*(S+U1)**(-1)
     +     - 48*M2*TG**2*S**(-1)*U1**(-1) + 24*M2*TG**2*U1**(-1)*
     +    (S+U1)**(-1) - 8*M2*S**(-1)*S4 + 16*M2*S*U1**(-2)*MG2 - 16*M2
     +    *S*U1**(-1) + 32*M2*U1**(-1)*MG2 + 4*M2*(S+U1)**(-1)*MG2 - 24
     +    *M2 - 48*M2**2*TG*S**(-1)*U1**(-1) + 24*M2**2*TG*U1**(-1)*
     +    (S+U1)**(-1) - 16*M2**2*S**(-1) - 16*M2**2*S*U1**(-2) - 48*
     +    M2**2*U1**(-1) - 16*M2**3*S**(-1)*U1**(-1) + 8*M2**3*U1**(-1)
     +    *(S+U1)**(-1) - 16*TG*S**(-1)*S4 + 16*TG*S*U1**(-2)*MG2 - 8*
     +    TG*S*U1**(-1) + 32*TG*U1**(-1)*MG2 + 4*TG*(S+U1)**(-1)*MG2 - 
     +    4*TG - 16*TG**2*S**(-1) - 16*TG**2*U1**(-1) + 4*TG**2*
     +    (S+U1)**(-1) - 16*TG**3*S**(-1)*U1**(-1) + 8*TG**3*U1**(-1)*
     +    (S+U1)**(-1) - 8*S**(-1)*S4*MG2 - 8*S**(-1)*S4**2 + 8*S*
     +    U1**(-1)*MG2 + 4*S4 + 20*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(38)*N*CF**2 * (  - 8*M2*TG*S**(-1) + 8*M2*
     +    S**(-1)*S4 + 8*M2*S**(-1)*MG2 - 8*M2**2*S**(-1) + 8*TG*
     +    S**(-1)*MG2 - 8*S**(-1)*S4*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(38)*N**2*CF * ( 8*M2 - 8*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(39)*N*CF**2 * ( 16*M2*TG*S**(-1)*U1**(-1)
     +     + 16*M2*S**(-1)*U1**(-1)*MG2 + 8*M2*S**(-1) + 32*M2*U1**(-1)
     +     + 16*TG*S**(-1)*U1**(-1)*MG2 + 8*TG*S**(-1) + 16*TG**2*
     +    S**(-1)*U1**(-1) - 32*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(39)*N**2*CF * (  - 4*M2*TG*S**(-1)*
     +    U1**(-1) - 4*M2*S**(-1)*U1**(-1)*MG2 - 8*M2*U1**(-1) - 4*TG*
     +    S**(-1)*U1**(-1)*MG2 - 4*TG*S**(-1) - 4*TG**2*S**(-1)*
     +    U1**(-1) - 4*S**(-1)*MG2 + 8*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(44)*N*CF**2 * ( 8 - 16*M2*S**(-1) + 16*TG*
     +    S**(-1) + 16*TG*U1**(-1) + 16*S**(-1)*S4 + 32*S**(-1)*MG2 + 8
     +    *S*U1**(-1) + 16*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(44)*N**2*CF * (  - 4 + 8*M2*S**(-1) - 8*TG
     +    *S**(-1) - 8*TG*U1**(-1) - 8*S**(-1)*S4 - 16*S**(-1)*MG2 - 4*
     +    S*U1**(-1) - 8*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(48)*N*CF**2 * ( 144*M2*TG*U1**(-1) - 16*M2
     +    *TG*(S+U1)**(-1) + 96*M2*TG**2*U1**(-1)*(S+U1)**(-1) + 48*M2*
     +    S*U1**(-1) + 16*M2*U1**(-1)*MG2 - 16*M2*(S+U1)**(-1)*MG2 + 32
     +    *M2 + 96*M2**2*TG*U1**(-1)*(S+U1)**(-1) + 64*M2**2*U1**(-1)
     +     + 32*M2**3*U1**(-1)*(S+U1)**(-1) + 64*TG*S*U1**(-1) + 16*TG*
     +    U1**(-1)*MG2 - 16*TG*(S+U1)**(-1)*MG2 + 32*TG + 80*TG**2*
     +    U1**(-1) - 16*TG**2*(S+U1)**(-1) + 32*TG**3*U1**(-1)*
     +    (S+U1)**(-1) + 16*S*U1**(-1)*MG2 + 16*S + 16*S**2*U1**(-1) + 
     +    16*S4 )
     +
      M2GBH1 = M2GBH1 + ANG4(48)*N**2*CF * (  - 16*M2*TG*S**(-1) - 72*
     +    M2*TG*U1**(-1) + 4*M2*TG*(S+U1)**(-1) - 24*M2*TG**2*S**(-1)*
     +    U1**(-1) - 24*M2*TG**2*U1**(-1)*(S+U1)**(-1) - 4*M2*S**(-1)*
     +    S4 - 24*M2*S*U1**(-1) - 8*M2*U1**(-1)*MG2 + 4*M2*(S+U1)**(-1)
     +    *MG2 - 16*M2 - 24*M2**2*TG*S**(-1)*U1**(-1) - 24*M2**2*TG*
     +    U1**(-1)*(S+U1)**(-1) - 8*M2**2*S**(-1) - 32*M2**2*U1**(-1)
     +     - 8*M2**3*S**(-1)*U1**(-1) - 8*M2**3*U1**(-1)*(S+U1)**(-1)
     +     - 4*TG*S**(-1)*S4 - 32*TG*S*U1**(-1) - 8*TG*U1**(-1)*MG2 + 4
     +    *TG*(S+U1)**(-1)*MG2 - 20*TG - 8*TG**2*S**(-1) - 40*TG**2*
     +    U1**(-1) + 4*TG**2*(S+U1)**(-1) - 8*TG**3*S**(-1)*U1**(-1) - 
     +    8*TG**3*U1**(-1)*(S+U1)**(-1) - 4*S**(-1)*S4**2 - 8*S*
     +    U1**(-1)*MG2 - 8*S - 8*S**2*U1**(-1) - 8*S4 - 4*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(51)*N**2*CF * ( 16*M2*S**2 - 16*S**2*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(52)*N**2*CF * ( 16*M2*S - 16*S*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(53)*N**2*CF * ( 32*M2*TG*S*U1**(-1) - 32*
     +    M2*S*U1**(-1)*MG2 + 16*M2*S**2*U1**(-1) + 32*M2**2*S*U1**(-1)
     +     - 32*TG*S*U1**(-1)*MG2 - 16*S**2*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(54)*N*CF**2 * ( 64*M2*S*U1**(-1) - 64*S*
     +    U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(54)*N**2*CF * ( 24*M2*TG*S**(-1) + 16*M2*
     +    TG*S*U1**(-2) + 64*M2*TG*U1**(-1) + 48*M2*TG**2*S**(-1)*
     +    U1**(-1) + 8*M2*S**(-1)*S4 - 16*M2*S*U1**(-2)*MG2 + 8*M2*S*
     +    U1**(-1) - 16*M2*U1**(-1)*MG2 + 16*M2 + 48*M2**2*TG*S**(-1)*
     +    U1**(-1) + 12*M2**2*S**(-1) + 16*M2**2*S*U1**(-2) + 40*M2**2*
     +    U1**(-1) + 16*M2**3*S**(-1)*U1**(-1) + 8*TG*S**(-1)*S4 - 16*
     +    TG*S*U1**(-2)*MG2 + 8*TG*S*U1**(-1) - 16*TG*U1**(-1)*MG2 + 8*
     +    TG + 12*TG**2*S**(-1) + 24*TG**2*U1**(-1) + 16*TG**3*S**(-1)*
     +    U1**(-1) + 4*S**(-1)*S4**2 - 8*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(56)*N*CF**2 * (  - 8*M2*S + 8*S*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(58)*N*CF**2 * ( 16*M2*TG*S**(-1) - 48*M2*
     +    TG*U1**(-1) - 16*M2*TG*(S+U1)**(-1) - 96*M2*TG**2*U1**(-1)*
     +    (S+U1)**(-1) + 16*M2*U1**(-1)*MG2 - 16*M2*(S+U1)**(-1)*MG2 - 
     +    96*M2**2*TG*U1**(-1)*(S+U1)**(-1) + 8*M2**2*S**(-1) - 32*
     +    M2**2*U1**(-1) - 32*M2**3*U1**(-1)*(S+U1)**(-1) + 16*TG*
     +    U1**(-1)*MG2 - 16*TG*(S+U1)**(-1)*MG2 - 16*TG + 8*TG**2*
     +    S**(-1) - 16*TG**2*U1**(-1) - 16*TG**2*(S+U1)**(-1) - 32*
     +    TG**3*U1**(-1)*(S+U1)**(-1) + 8*S**(-1)*S4**2 - 8*S4 - 16*MG2
     +     )
     +
      M2GBH1 = M2GBH1 + ANG4(58)*N**2*CF * ( 8*M2*TG*S**(-1) + 24*M2*TG
     +    *U1**(-1) + 4*M2*TG*(S+U1)**(-1) + 24*M2*TG**2*S**(-1)*
     +    U1**(-1) + 24*M2*TG**2*U1**(-1)*(S+U1)**(-1) + 4*M2*S**(-1)*
     +    S4 - 8*M2*U1**(-1)*MG2 + 4*M2*(S+U1)**(-1)*MG2 + 4*M2 + 24*
     +    M2**2*TG*S**(-1)*U1**(-1) + 24*M2**2*TG*U1**(-1)*(S+U1)**(-1)
     +     + 4*M2**2*S**(-1) + 16*M2**2*U1**(-1) + 8*M2**3*S**(-1)*
     +    U1**(-1) + 8*M2**3*U1**(-1)*(S+U1)**(-1) + 4*TG*S**(-1)*S4 - 
     +    8*TG*U1**(-1)*MG2 + 4*TG*(S+U1)**(-1)*MG2 + 4*TG + 4*TG**2*
     +    S**(-1) + 8*TG**2*U1**(-1) + 4*TG**2*(S+U1)**(-1) + 8*TG**3*
     +    S**(-1)*U1**(-1) + 8*TG**3*U1**(-1)*(S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + ANG4(59)*N*CF**2 * (  - 32*M2*TG*MG2 + 16*M2*
     +    TG**2 + 32*M2**2*TG - 16*M2**2*MG2 + 16*M2**3 - 16*TG**2*MG2
     +     )
     +
      M2GBH1 = M2GBH1 + ANG4(60)*N*CF**2 * ( 16*M2*TG + 16*M2*MG2 - 16*
     +    TG*MG2 - 16*MG2**2 )
     +
      M2GBH1 = M2GBH1 + ANG4(61)*N*CF**2 * ( 32*M2*TG*S*U1**(-1) - 64*
     +    M2*TG*U1**(-1)*MG2 + 32*M2*TG**2*U1**(-1) - 32*M2*S*U1**(-1)*
     +    MG2 + 64*M2**2*TG*U1**(-1) + 32*M2**2*S*U1**(-1) - 32*M2**2*
     +    U1**(-1)*MG2 + 32*M2**3*U1**(-1) - 32*TG*S*U1**(-1)*MG2 - 32*
     +    TG**2*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(61)*N**2*CF * ( 32*M2*TG*U1**(-1)*MG2 - 16
     +    *M2*TG**2*U1**(-1) - 32*M2**2*TG*U1**(-1) + 16*M2**2*U1**(-1)
     +    *MG2 - 16*M2**3*U1**(-1) + 16*TG**2*U1**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(62)*N*CF**2 * (  - 48*M2*TG*S**(-1) - 16*
     +    M2*TG*(S+U1)**(-1) - 96*M2*TG**2*S**(-1)*U1**(-1) + 96*M2*
     +    TG**2*U1**(-1)*(S+U1)**(-1) + 16*M2*S**(-1)*MG2 + 32*M2*S*
     +    U1**(-1) + 96*M2*U1**(-1)*MG2 - 16*M2*(S+U1)**(-1)*MG2 + 24*
     +    M2 - 96*M2**2*TG*S**(-1)*U1**(-1) + 96*M2**2*TG*U1**(-1)*
     +    (S+U1)**(-1) - 32*M2**2*S**(-1) - 48*M2**2*U1**(-1) - 32*
     +    M2**3*S**(-1)*U1**(-1) + 32*M2**3*U1**(-1)*(S+U1)**(-1) + 16*
     +    TG*S**(-1)*MG2 + 16*TG*S*U1**(-1) + 32*TG*U1**(-1)*MG2 - 16*
     +    TG*(S+U1)**(-1)*MG2 + 8*TG - 16*TG**2*S**(-1) + 16*TG**2*
     +    U1**(-1) - 16*TG**2*(S+U1)**(-1) - 32*TG**3*S**(-1)*U1**(-1)
     +     + 32*TG**3*U1**(-1)*(S+U1)**(-1) - 16*S*U1**(-1)*MG2 - 32*
     +    U1**(-1)*MG2**2 - 16*MG2 )
     +
      M2GBH1 = M2GBH1 + ANG4(62)*N**2*CF * ( 24*M2*TG*S**(-1) + 16*M2*
     +    TG*S*U1**(-2) + 32*M2*TG*U1**(-1) + 4*M2*TG*(S+U1)**(-1) + 48
     +    *M2*TG**2*S**(-1)*U1**(-1) - 24*M2*TG**2*U1**(-1)*
     +    (S+U1)**(-1) - 8*M2*S**(-1)*MG2 - 16*M2*S*U1**(-2)*MG2 - 48*
     +    M2*U1**(-1)*MG2 + 4*M2*(S+U1)**(-1)*MG2 + 48*M2**2*TG*S**(-1)
     +    *U1**(-1) - 24*M2**2*TG*U1**(-1)*(S+U1)**(-1) + 16*M2**2*
     +    S**(-1) + 16*M2**2*S*U1**(-2) + 40*M2**2*U1**(-1) + 16*M2**3*
     +    S**(-1)*U1**(-1) - 8*M2**3*U1**(-1)*(S+U1)**(-1) - 8*TG*
     +    S**(-1)*MG2 - 16*TG*S*U1**(-2)*MG2 - 16*TG*U1**(-1)*MG2 + 4*
     +    TG*(S+U1)**(-1)*MG2 + 8*TG**2*S**(-1) + 8*TG**2*U1**(-1) + 4*
     +    TG**2*(S+U1)**(-1) + 16*TG**3*S**(-1)*U1**(-1) - 8*TG**3*
     +    U1**(-1)*(S+U1)**(-1) + 16*U1**(-1)*MG2**2 )
     +
      M2GBH1 = M2GBH1 + ANG4(70)*N*CF**2 * ( 32*M2*TG*S*U1**(-2) - 32*
     +    M2*TG*U1**(-2)*MG2 + 16*M2*TG**2*U1**(-2) - 32*M2*S*U1**(-2)*
     +    MG2 + 16*M2*S**2*U1**(-2) + 32*M2**2*TG*U1**(-2) + 32*M2**2*S
     +    *U1**(-2) - 16*M2**2*U1**(-2)*MG2 + 16*M2**3*U1**(-2) - 32*TG
     +    *S*U1**(-2)*MG2 - 16*TG**2*U1**(-2)*MG2 - 16*S**2*U1**(-2)*
     +    MG2 )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 32*M2*TG*S**(-2)*
     +    T1**(-1)*S4**(-1) - 16*M2*TG*S**(-1)*T1**(-2)*S4**(-1) - 128*
     +    M2*TG*S**(-1)*U1**(-2)*S4**(-1) + 576*M2*TG*S**(-1)*U1**(-1)*
     +    S4**(-2) - 128*M2*TG*S**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)
     +     - 64*M2*TG*S**(-1)*S4**(-2)*(S+U1)**(-1) + 128*M2*TG*
     +    U1**(-2)*S4**(-1)*(S+U1)**(-1) + 384*M2*TG**2*S**(-1)*
     +    U1**(-1)*S4**(-2)*(S+U1)**(-1) - 32*M2*S**(-2)*T1**(-1) + 16*
     +    M2*S**(-1)*T1**(-2)*S4**(-1)*MG2 + 16*M2*S**(-1)*T1**(-2) + 
     +    48*M2*S**(-1)*T1**(-1)*S4**(-1) + 64*M2*S**(-1)*U1**(-1)*
     +    S4**(-2)*MG2 - 128*M2*S**(-1)*U1**(-1)*S4**(-1) - 64*M2*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 + 128*M2*S**(-1)*S4**(-2)
     +     - 64*M2*S*T1**(-2)*U1**(-2)*S4**(-1)*MG2 + 128*M2*S*T1**(-1)
     +    *U1**(-3) - 64*M2*S*T1**(-1)*U1**(-2)*S4**(-1) - 64*M2*S*
     +    U1**(-3)*S4**(-1) - 384*M2*S**2*T1**(-2)*U1**(-4)*MG2 + 256*
     +    M2*S**2*T1**(-2)*U1**(-3)*S4**(-1)*MG2 + 256*M2*S**2*T1**(-1)
     +    *U1**(-4)*S4**(-1)*MG2 )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N*(S4+MS2)*CF**2 * (  - 64*M2*S**2*
     +    T1**(-1)*U1**(-3)*S4**(-1) + 256*M2*S**3*T1**(-2)*U1**(-4)*
     +    S4**(-1)*MG2 + 64*M2*T1**(-1)*U1**(-2)*S4**(-1)*MG2 + 32*M2*
     +    T1**(-1)*U1**(-1)*S4**(-1) - 64*M2*U1**(-2)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 + 32*M2*U1**(-2)*S4**(-1) + 192*M2*U1**(-1)*
     +    S4**(-2) - 32*M2*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 384*M2**2*
     +    TG*S**(-1)*U1**(-1)*S4**(-2)*(S+U1)**(-1) + 16*M2**2*S**(-2)*
     +    T1**(-1)*S4**(-1) - 16*M2**2*S**(-1)*T1**(-2)*S4**(-1) - 64*
     +    M2**2*S**(-1)*U1**(-2)*S4**(-1) + 256*M2**2*S**(-1)*U1**(-1)*
     +    S4**(-2) - 64*M2**2*S**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 
     +    32*M2**2*S*T1**(-2)*U1**(-2)*S4**(-1) + 192*M2**2*S**2*
     +    T1**(-2)*U1**(-4) - 128*M2**2*S**2*T1**(-2)*U1**(-3)*S4**(-1)
     +     - 128*M2**2*S**2*T1**(-1)*U1**(-4)*S4**(-1) - 128*M2**2*S**3
     +    *T1**(-2)*U1**(-4)*S4**(-1) - 32*M2**2*T1**(-1)*U1**(-2)*
     +    S4**(-1) + 32*M2**2*U1**(-2)*S4**(-1)*(S+T1)**(-1) + 64*M2**2
     +    *U1**(-2)*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 128*M2**3*S**(-1)
     +    *U1**(-1)*S4**(-2)*(S+U1)**(-1) - 32*TG*S**(-2)*T1**(-1) + 16
     +    *TG*S**(-1)*T1**(-2)*S4**(-1)*MG2 + 16*TG*S**(-1)*T1**(-1)*
     +    S4**(-1) + 64*TG*S**(-1)*U1**(-1)*S4**(-2)*MG2 - 96*TG*
     +    S**(-1)*U1**(-1)*S4**(-1) - 64*TG*S**(-1)*S4**(-2)*
     +    (S+U1)**(-1)*MG2 + 128*TG*S**(-1)*S4**(-2) - 64*TG*S**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 32*TG*U1**(-2)*S4**(-1) + 256*TG*
     +    U1**(-1)*S4**(-2) - 96*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 16
     +    *TG**2*S**(-2)*T1**(-1)*S4**(-1) - 64*TG**2*S**(-1)*U1**(-2)*
     +    S4**(-1) + 320*TG**2*S**(-1)*U1**(-1)*S4**(-2) - 64*TG**2*
     +    S**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 64*TG**2*S**(-1)*
     +    S4**(-2)*(S+U1)**(-1) + 64*TG**2*U1**(-2)*S4**(-1)*
     +    (S+U1)**(-1) + 128*TG**3*S**(-1)*U1**(-1)*S4**(-2)*
     +    (S+U1)**(-1) + 16*S**(-2)*T1**(-1)*S4 + 32*S**(-2) - 16*
     +    S**(-1)*T1**(-2)*MG2 - 32*S**(-1)*T1**(-1)*S4**(-1)*MG2 - 16*
     +    S**(-1)*T1**(-1) )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 32*S**(-1)*
     +    U1**(-1)*S4**(-1)*MG2 - 64*S**(-1)*S4**(-1)*(S+U1)**(-1)*MG2
     +     - 32*S**(-1)*S4**(-1) + 32*S*T1**(-2)*U1**(-2)*S4**(-1)*
     +    MG2**2 - 128*S*T1**(-1)*U1**(-3)*MG2 + 64*S*T1**(-1)*U1**(-2)
     +    *S4**(-1)*MG2 + 64*S*U1**(-3)*S4**(-1)*MG2 + 32*S*U1**(-2)*
     +    S4**(-1) + 64*S*U1**(-1)*S4**(-2) - 32*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 192*S**2*T1**(-2)*U1**(-4)*MG2**2 - 128*S**2*
     +    T1**(-2)*U1**(-3)*S4**(-1)*MG2**2 - 128*S**2*T1**(-1)*
     +    U1**(-4)*S4**(-1)*MG2**2 + 64*S**2*T1**(-1)*U1**(-3)*S4**(-1)
     +    *MG2 - 128*S**3*T1**(-2)*U1**(-4)*S4**(-1)*MG2**2 - 32*
     +    T1**(-1)*U1**(-2)*S4**(-1)*MG2**2 - 32*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 + 32*U1**(-2)*S4**(-1)*(S+T1)**(-1)*MG2**2 + 64*
     +    U1**(-1)*S4**(-2)*MG2 + 32*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2
     +     - 96*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 64*S4**(-2) + 16*
     +    S4**(-1)*(S+T1)**(-1) - 32*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N**2*(S4+MS2)*CF * ( 128*M2*TG*S**(-2)
     +    *U1**(-1)*S4**(-1) - 64*M2*TG*S**(-2)*S4**(-2) + 128*M2*TG*
     +    S**(-1)*U1**(-3)*S4**(-1)*MG2 + 128*M2*TG*S**(-1)*U1**(-3) + 
     +    96*M2*TG*S**(-1)*U1**(-2)*S4**(-1) - 288*M2*TG*S**(-1)*
     +    U1**(-1)*S4**(-2) + 32*M2*TG*S**(-1)*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2*TG*S**(-1)*S4**(-2)*(S+U1)**(-1) - 32*M2
     +    *TG*U1**(-2)*S4**(-1)*(S+U1)**(-1) + 96*M2*TG**2*S**(-2)*
     +    U1**(-2)*S4**(-1) - 96*M2*TG**2*S**(-2)*U1**(-1)*S4**(-2) - 
     +    64*M2*TG**2*S**(-1)*U1**(-3)*S4**(-1) - 96*M2*TG**2*S**(-1)*
     +    U1**(-1)*S4**(-2)*(S+U1)**(-1) + 16*M2*S**(-2)*S4**(-1) - 128
     +    *M2*S**(-1)*U1**(-3)*MG2 + 64*M2*S**(-1)*U1**(-2)*S4**(-1)*
     +    MG2 - 32*M2*S**(-1)*U1**(-1)*S4**(-2)*MG2 + 128*M2*S**(-1)*
     +    U1**(-1)*S4**(-1) + 16*M2*S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2
     +     - 64*M2*S**(-1)*S4**(-2) - 128*M2*S*T1**(-1)*U1**(-3)*
     +    S4**(-1)*MG2 - 128*M2*S*U1**(-4)*S4**(-1)*MG2 + 64*M2*S*
     +    U1**(-3)*S4**(-1) )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 128*M2*S**2*
     +    T1**(-1)*U1**(-4)*S4**(-1)*MG2 - 64*M2*T1**(-1)*U1**(-2)*
     +    S4**(-1)*MG2 - 128*M2*U1**(-4)*MG2 + 96*M2*U1**(-2)*S4**(-1)
     +     - 96*M2*U1**(-1)*S4**(-2) + 96*M2**2*TG*S**(-2)*U1**(-2)*
     +    S4**(-1) - 96*M2**2*TG*S**(-2)*U1**(-1)*S4**(-2) - 128*M2**2*
     +    TG*S**(-1)*U1**(-3)*S4**(-1) - 96*M2**2*TG*S**(-1)*U1**(-1)*
     +    S4**(-2)*(S+U1)**(-1) + 64*M2**2*S**(-2)*U1**(-1)*S4**(-1) - 
     +    32*M2**2*S**(-2)*S4**(-2) + 64*M2**2*S**(-1)*U1**(-3)*
     +    S4**(-1)*MG2 + 128*M2**2*S**(-1)*U1**(-3) + 16*M2**2*S**(-1)*
     +    U1**(-2)*S4**(-1) - 128*M2**2*S**(-1)*U1**(-1)*S4**(-2) + 16*
     +    M2**2*S**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 64*M2**2*S*
     +    T1**(-1)*U1**(-3)*S4**(-1) + 64*M2**2*S*U1**(-4)*S4**(-1) + 
     +    64*M2**2*S**2*T1**(-1)*U1**(-4)*S4**(-1) + 32*M2**2*T1**(-1)*
     +    U1**(-2)*S4**(-1) + 64*M2**2*U1**(-4) - 16*M2**2*U1**(-2)*
     +    S4**(-1)*(S+U1)**(-1) + 32*M2**3*S**(-2)*U1**(-2)*S4**(-1) - 
     +    32*M2**3*S**(-2)*U1**(-1)*S4**(-2) )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 64*M2**3*
     +    S**(-1)*U1**(-3)*S4**(-1) - 32*M2**3*S**(-1)*U1**(-1)*
     +    S4**(-2)*(S+U1)**(-1) + 16*TG*S**(-2)*S4**(-1) - 128*TG*
     +    S**(-1)*U1**(-3)*MG2 + 64*TG*S**(-1)*U1**(-2)*S4**(-1)*MG2 - 
     +    32*TG*S**(-1)*U1**(-1)*S4**(-2)*MG2 + 88*TG*S**(-1)*U1**(-1)*
     +    S4**(-1) + 16*TG*S**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 - 80*TG*
     +    S**(-1)*S4**(-2) + 24*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) + 32*
     +    TG*U1**(-2)*S4**(-1) - 128*TG*U1**(-1)*S4**(-2) + 40*TG*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) + 64*TG**2*S**(-2)*U1**(-1)*
     +    S4**(-1) - 32*TG**2*S**(-2)*S4**(-2) + 64*TG**2*S**(-1)*
     +    U1**(-3)*S4**(-1)*MG2 + 80*TG**2*S**(-1)*U1**(-2)*S4**(-1) - 
     +    160*TG**2*S**(-1)*U1**(-1)*S4**(-2) + 16*TG**2*S**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) + 16*TG**2*S**(-1)*S4**(-2)*
     +    (S+U1)**(-1) - 16*TG**2*U1**(-2)*S4**(-1)*(S+U1)**(-1) + 32*
     +    TG**3*S**(-2)*U1**(-2)*S4**(-1) - 32*TG**3*S**(-2)*U1**(-1)*
     +    S4**(-2) )
     +
      M2GBH1 = M2GBH1 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 32*TG**3*
     +    S**(-1)*U1**(-1)*S4**(-2)*(S+U1)**(-1) - 40*S**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 - 16*S**(-1)*S4**(-2)*MG2 + 24*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 64*S*T1**(-1)*U1**(-3)*S4**(-1)*MG2**2 + 
     +    64*S*U1**(-4)*S4**(-1)*MG2**2 - 64*S*U1**(-3)*S4**(-1)*MG2 - 
     +    32*S*U1**(-1)*S4**(-2) + 16*S*U1**(-1)*S4**(-1)*(S+U1)**(-1)
     +     + 64*S**2*T1**(-1)*U1**(-4)*S4**(-1)*MG2**2 + 32*T1**(-1)*
     +    U1**(-2)*S4**(-1)*MG2**2 + 64*U1**(-4)*MG2**2 - 64*U1**(-2)*
     +    S4**(-1)*MG2 - 32*U1**(-1)*S4**(-2)*MG2 + 40*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1)*MG2 + 16*U1**(-1)*S4**(-1) - 32*
     +    S4**(-2) + 16*S4**(-1)*(S+U1)**(-1) )

      M2GBH2 = 0.D0
      M2GBH2 = M2GBH2 + N*S4G*(S4+MS2)**(-1)*S4G2**(-1)*CF**2 * ( 8*M2*
     +    TG*S**(-1)*U1**(-1)*MG2 - 4*M2*TG*S**(-1) + 16*M2*TG*U1**(-2)
     +    *MG2 - 16*M2*TG*U1**(-1) - 8*M2*TG**2*S**(-1)*U1**(-1) + 16*
     +    M2*S**(-1)*U1**(-1)*MG2**2 + 4*M2*S**(-1)*MG2 + 16*M2*S*
     +    U1**(-2)*MG2 - 8*M2*S*U1**(-1) + 8*M2*U1**(-2)*MG2**2 + 8*M2*
     +    U1**(-1)*MG2 - 8*M2 - 16*M2**2*TG*S**(-1)*U1**(-1) - 8*M2**2*
     +    TG*U1**(-2) - 12*M2**2*S**(-1)*U1**(-1)*MG2 - 8*M2**2*S**(-1)
     +     - 8*M2**2*S*U1**(-2) + 8*M2**2*U1**(-2)*MG2 - 16*M2**2*
     +    U1**(-1) - 4*M2**3*S**(-1)*U1**(-1) - 8*M2**3*U1**(-2) + 8*TG
     +    *S**(-1)*U1**(-1)*MG2**2 + 4*TG*S**(-1)*MG2 - 8*TG*U1**(-2)*
     +    MG2**2 + 16*TG*U1**(-1)*MG2 + 8*TG**2*S**(-1)*U1**(-1)*MG2 + 
     +    4*S**(-1)*MG2**2 - 8*S*U1**(-2)*MG2**2 + 8*S*U1**(-1)*MG2 - 8
     +    *U1**(-2)*MG2**3 + 8*U1**(-1)*MG2**2 + 8*MG2 )
     +
      M2GBH2 = M2GBH2 + N*S4G*S4G2**(-1)*CF**2 * ( 16 - 16*M2*TG**(-1)*
     +    S**(-1)*MG2 + 16*M2*TG**(-1)*S*U1**(-1) + 16*M2*TG**(-1)*
     +    U1**(-1)*MG2 + 16*M2*TG**(-1) + 32*M2*TG*S**(-1)*U1**(-1) - 4
     +    *M2*TG*S**(-1)*MG2**(-1) + 8*M2*TG*U1**(-2) - 16*M2*S**(-1)*
     +    U1**(-1)*MG2 + 24*M2*S**(-1) + 8*M2*S*U1**(-2) - 24*M2*
     +    U1**(-2)*MG2 + 56*M2*U1**(-1) + 36*M2**2*S**(-1)*U1**(-1) - 8
     +    *M2**2*S**(-1)*MG2**(-1) + 8*M2**2*U1**(-2) - 8*M2**2*
     +    U1**(-1)*MG2**(-1) + 4*M2**3*S**(-1)*U1**(-1)*MG2**(-1) + 8*
     +    M2**3*U1**(-2)*MG2**(-1) + 16*TG**(-1)*S*U1**(-1)*MG2 + 16*
     +    TG**(-1)*S + 16*TG**(-1)*S**2*U1**(-1) + 16*TG**(-1)*MG2 - 8*
     +    TG*S**(-1)*U1**(-1)*MG2 + 8*TG*S**(-1) - 8*TG*U1**(-2)*MG2 + 
     +    24*TG*U1**(-1) + 8*TG**2*S**(-1)*U1**(-1) - 20*S**(-1)*MG2 - 
     +    8*S*U1**(-2)*MG2 + 32*S*U1**(-1) + 8*U1**(-2)*MG2**2 - 8*
     +    U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + N*(S4+MS2)**(-1)*CF**2 * ( 4*M2*TG*S**(-1)*
     +    MG2**(-1) - 12*M2*S**(-1) - 8*M2*U1**(-2)*MG2 - 16*M2*
     +    U1**(-1) + 4*M2**2*S**(-1)*U1**(-1) + 8*M2**2*S**(-1)*
     +    MG2**(-1) + 16*M2**2*U1**(-2) + 8*M2**2*U1**(-1)*MG2**(-1) - 
     +    4*M2**3*S**(-1)*U1**(-1)*MG2**(-1) - 8*M2**3*U1**(-2)*
     +    MG2**(-1) - 4*TG*S**(-1) + 4*S**(-1)*MG2 + 8*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + N*(S4+MS2)*CF**2 * ( 16*M2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-2)*MG2 + 32*M2*TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2
     +     + 16*M2*TG**(-1)*S*S4**(-1)*(S+U1)**(-2) - 32*M2*TG**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 32*M2*S*S4**(-1)*(S+U1)**(-3) - 48*M2
     +    *S4**(-1)*(S+U1)**(-2) + 16*M2**2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 - 16*M2**2*TG**(-2)*S4**(-1)*(S+U1)**(-1) + 
     +    16*M2**2*TG**(-1)*S*S4**(-1)*(S+U1)**(-3) - 48*M2**2*TG**(-1)
     +    *S4**(-1)*(S+U1)**(-2) - 16*M2**3*TG**(-2)*S4**(-1)*
     +    (S+U1)**(-2) + 16*TG**(-1)*S*S4**(-1)*(S+U1)**(-2)*MG2 + 16*
     +    TG*S*S4**(-1)*(S+U1)**(-3) - 16*TG*S4**(-1)*(S+U1)**(-2) + 16
     +    *S*S4**(-1)*(S+U1)**(-3)*MG2 + 16*S*S4**(-1)*(S+U1)**(-2) - 
     +    16*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + N*CF**2 * (  - 16*M2*TG**(-1)*S**(-1) + 16*
     +    TG**(-1) - 12*S**(-1) )
     +
      M2GBH2 = M2GBH2 + N**2*S4G*(S4+MS2)**(-1)*S4G2**(-1)*CF * ( 4*M2*
     +    TG**(-2)*S*MG2 - 8*M2*TG**(-2)*MG2**2 + 10*M2*TG**(-1)*S*
     +    U1**(-1)*MG2 + 4*M2*TG**(-1)*S + 4*M2*TG**(-1)*S**2*U1**(-1)
     +     + 2*M2*TG**(-1)*MG2 - 4*M2*TG*S**(-1)*U1**(-1)*MG2 + 2*M2*TG
     +    *S**(-1) + 12*M2*TG*U1**(-1) + 4*M2*TG**2*S**(-1)*U1**(-1) - 
     +    8*M2*S**(-1)*U1**(-1)*MG2**2 - 4*M2*S**(-1)*MG2 + 12*M2*S*
     +    U1**(-1) + 6*M2*U1**(-1)*MG2 + 6*M2 + 8*M2**2*TG**(-2)*MG2 - 
     +    2*M2**2*TG**(-1)*S**(-1)*MG2 + 2*M2**2*TG**(-1)*S*U1**(-1) + 
     +    10*M2**2*TG**(-1)*U1**(-1)*MG2 + 2*M2**2*TG**(-1) + 8*M2**2*
     +    TG*S**(-1)*U1**(-1) + 6*M2**2*S**(-1)*U1**(-1)*MG2 + 4*M2**2*
     +    S**(-1) + 10*M2**2*U1**(-1) + 2*M2**3*TG**(-1)*S**(-1) - 2*
     +    M2**3*TG**(-1)*U1**(-1) + 2*M2**3*S**(-1)*U1**(-1) - 4*
     +    TG**(-2)*S*MG2**2 - 12*TG**(-1)*S*U1**(-1)*MG2**2 - 4*
     +    TG**(-1)*S*MG2 - 4*TG**(-1)*S**2*U1**(-1)*MG2 - 8*TG**(-1)*
     +    U1**(-1)*MG2**3 - 4*TG**(-1)*MG2**2 - 4*TG*S**(-1)*U1**(-1)*
     +    MG2**2 )
     +
      M2GBH2 = M2GBH2 + N**2*S4G*(S4+MS2)**(-1)*S4G2**(-1)*CF * (  - 2*
     +    TG*S**(-1)*MG2 - 12*TG*U1**(-1)*MG2 - 4*TG**2*S**(-1)*
     +    U1**(-1)*MG2 - 12*S*U1**(-1)*MG2 - 16*U1**(-1)*MG2**2 - 6*MG2
     +     )
     +
      M2GBH2 = M2GBH2 + N**2*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 8*M2*
     +    TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 + 24*M2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 8*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-1) - 16*M2*TG**(-1)*S4**(-1) + 16*M2*TG*S*S4**(-1)*
     +    (S+U1)**(-3) - 24*M2*TG*S4**(-1)*(S+U1)**(-2) + 16*M2*S*
     +    S4**(-1)*(S+U1)**(-3)*MG2 + 24*M2*S*S4**(-1)*(S+U1)**(-2) - 
     +    40*M2*S4**(-1)*(S+U1)**(-1) + 8*M2**2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-2)*MG2 - 8*M2**2*TG**(-2)*S4**(-1) + 8*M2**2*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2 + 8*M2**2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2) - 32*M2**2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2**2*S*S4**(-1)*(S+U1)**(-3) - 24*M2**2*
     +    S4**(-1)*(S+U1)**(-2) - 8*M2**3*TG**(-2)*S4**(-1)*
     +    (S+U1)**(-1) - 8*M2**3*TG**(-1)*S4**(-1)*(S+U1)**(-2) + 8*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-1)*MG2 + 8*TG*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 + 16*TG*S*S4**(-1)*(S+U1)**(-2) - 16*TG*
     +    S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + N**2*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 8*TG**2*S*
     +    S4**(-1)*(S+U1)**(-3) - 8*TG**2*S4**(-1)*(S+U1)**(-2) + 16*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 8*S*S4**(-1)*(S+U1)**(-1) - 8*
     +    S4**(-1) )
     +
      M2GBH2 = M2GBH2 + N**2*S4G*S4G2**(-1)*CF * (  - 2 - 8*M2*TG**(-2)
     +    *S + 16*M2*TG**(-2)*MG2 - 14*M2*TG**(-1)*S*U1**(-1) - 4*M2*
     +    TG**(-1)*S*MG2**(-1) - 20*M2*TG**(-1)*U1**(-1)*MG2 - 6*M2*
     +    TG**(-1) - 16*M2*TG*S**(-1)*U1**(-1) - 8*M2*S**(-1) - 30*M2*
     +    U1**(-1) - 4*M2*MG2**(-1) - 24*M2**2*TG**(-2) + 8*M2**2*
     +    TG**(-1)*S**(-2)*MG2 + 10*M2**2*TG**(-1)*S**(-1) - 2*M2**2*
     +    TG**(-1)*U1**(-1) - 8*M2**2*TG**(-1)*MG2**(-1) + 8*M2**2*
     +    S**(-2)*U1**(-1)*MG2 - 6*M2**2*S**(-1)*U1**(-1) - 4*M2**3*
     +    TG**(-1)*S**(-2) - 8*M2**3*TG**(-1)*S**(-1)*MG2**(-1) - 4*
     +    M2**3*S**(-2)*U1**(-1) - 6*M2**3*S**(-1)*U1**(-1)*MG2**(-1)
     +     + 12*TG**(-2)*S*MG2 - 4*TG**(-1)*S**2*U1**(-1) + 8*TG**(-1)*
     +    U1**(-1)*MG2**2 + 4*TG*S**(-1)*U1**(-1)*MG2 - 2*TG*S**(-1) - 
     +    12*TG*U1**(-1) - 4*TG**2*S**(-1)*U1**(-1) + 8*S**(-1)*MG2 - 
     +    12*S*U1**(-1) + 4*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + N**2*(S4+MS2)**(-1)*CF * (  - 4 + 8*M2*TG**(-2)
     +    *S - 12*M2*TG**(-2)*MG2 - 8*M2*TG**(-1)*S**(-2)*MG2**2 - 8*M2
     +    *TG**(-1)*S**(-2)*MG2**3*(S4G-S)**(-1) + 8*M2*TG**(-1)*
     +    S**(-1)*MG2 + 2*M2*TG**(-1)*S*U1**(-1) + 4*M2*TG**(-1)*S*
     +    MG2**(-1) - 2*M2*TG**(-1)*U1**(-1)*MG2 - 8*M2*TG**(-1) - 8*M2
     +    *S**(-2)*U1**(-1)*MG2**2 - 8*M2*S**(-2)*U1**(-1)*MG2**3*
     +    (S4G-S)**(-1) + 8*M2*S**(-1)*U1**(-1)*MG2 + 2*M2*U1**(-1) + 4
     +    *M2*MG2**(-1) + 12*M2**2*TG**(-2) + 12*M2**2*TG**(-1)*S**(-2)
     +    *MG2 + 12*M2**2*TG**(-1)*S**(-2)*MG2**2*(S4G-S)**(-1) - 16*
     +    M2**2*TG**(-1)*S**(-1) + 2*M2**2*TG**(-1)*U1**(-1) + 8*M2**2*
     +    TG**(-1)*MG2**(-1) + 12*M2**2*S**(-2)*U1**(-1)*MG2 + 12*M2**2
     +    *S**(-2)*U1**(-1)*MG2**2*(S4G-S)**(-1) - 14*M2**2*S**(-1)*
     +    U1**(-1) - 4*M2**3*TG**(-1)*S**(-2)*MG2*(S4G-S)**(-1) - 4*
     +    M2**3*TG**(-1)*S**(-2) + 8*M2**3*TG**(-1)*S**(-1)*MG2**(-1)
     +     - 4*M2**3*S**(-2)*U1**(-1)*MG2*(S4G-S)**(-1) - 4*M2**3*
     +    S**(-2)*U1**(-1) )
     +
      M2GBH2 = M2GBH2 + N**2*(S4+MS2)**(-1)*CF * ( 6*M2**3*S**(-1)*
     +    U1**(-1)*MG2**(-1) - 8*TG**(-2)*S*MG2 - 2*TG**(-1)*S*U1**(-1)
     +    *MG2 - 4*TG**(-1)*S - 2*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + N**2*(S4+MS2)*CF * (  - 8*M2*TG**(-2)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 - 16*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 - 8*M2*TG**(-1)*S*S4**(-1)*(S+U1)**(-2) + 16
     +    *M2*TG**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2*S*S4**(-1)*
     +    (S+U1)**(-3) + 24*M2*S4**(-1)*(S+U1)**(-2) - 8*M2**2*TG**(-2)
     +    *S*S4**(-1)*(S+U1)**(-3)*MG2 + 8*M2**2*TG**(-2)*S4**(-1)*
     +    (S+U1)**(-1) - 8*M2**2*TG**(-1)*S*S4**(-1)*(S+U1)**(-3) + 24*
     +    M2**2*TG**(-1)*S4**(-1)*(S+U1)**(-2) + 8*M2**3*TG**(-2)*
     +    S4**(-1)*(S+U1)**(-2) - 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-2)*
     +    MG2 - 8*TG*S*S4**(-1)*(S+U1)**(-3) + 8*TG*S4**(-1)*
     +    (S+U1)**(-2) - 8*S*S4**(-1)*(S+U1)**(-3)*MG2 - 8*S*S4**(-1)*
     +    (S+U1)**(-2) + 8*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + N**2*CF * (  - 4*M2*TG**(-2) + 8*M2*TG**(-1)*
     +    S**(-2)*MG2**2*(S4G-S)**(-1) + 4*M2*TG**(-1)*S**(-1) - 2*M2*
     +    TG**(-1)*U1**(-1) + 8*M2*S**(-2)*U1**(-1)*MG2**2*
     +    (S4G-S)**(-1) - 20*M2**2*TG**(-1)*S**(-2)*MG2*(S4G-S)**(-1)
     +     - 20*M2**2*S**(-2)*U1**(-1)*MG2*(S4G-S)**(-1) + 8*M2**3*
     +    TG**(-1)*S**(-2)*(S4G-S)**(-1) + 8*M2**3*S**(-2)*U1**(-1)*
     +    (S4G-S)**(-1) - 2*TG**(-1)*S*U1**(-1) - 8*TG**(-1) + 4*
     +    S**(-1) - 2*U1**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(21)*N*S4G*S4G2**(-1)*CF**2 * ( 16 + 16*M2*
     +    TG**(-1)*S**(-1)*MG2 - 16*M2*TG**(-1)*S*U1**(-1) - 16*M2*
     +    TG**(-1) - 8*M2*S**(-1) - 32*M2*U1**(-1) - 16*M2**2*TG**(-1)*
     +    S**(-1) - 16*M2**2*TG**(-1)*U1**(-1) + 48*TG**(-1)*S*U1**(-1)
     +    *MG2 + 16*TG**(-1)*S + 16*TG**(-1)*S**2*U1**(-1) + 32*
     +    TG**(-1)*U1**(-1)*MG2**2 + 48*TG**(-1)*MG2 + 16*TG*U1**(-1)
     +     + 8*S**(-1)*MG2 + 32*S*U1**(-1) + 64*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(21)*N*CF**2 * ( 8*M2*TG**(-2)*(S+U1)**(-1)
     +    *MG2 - 8*M2*TG**(-1)*S**(-1) + 8*M2*TG**(-1)*(S+U1)**(-1) + 4
     +    *TG**(-2)*MG2 + 8*TG**(-1)*S**(-1)*MG2 + 8*TG**(-1)*
     +    (S+U1)**(-1)*MG2 + 20*TG**(-1) + 8*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(21)*N**2*S4G*S4G2**(-1)*CF * (  - 4 - 4*M2
     +    *TG**(-2)*MG2 - 8*M2*TG**(-1)*S**(-1)*MG2 + 4*M2*TG**(-1)*S*
     +    U1**(-1) + 2*M2*TG**(-1) + 4*M2*S**(-1) + 8*M2*U1**(-1) + 8*
     +    M2**2*TG**(-1)*S**(-1) + 4*M2**2*TG**(-1)*U1**(-1) + 8*
     +    TG**(-2)*MG2**2 - 12*TG**(-1)*S*U1**(-1)*MG2 - 4*TG**(-1)*S
     +     - 4*TG**(-1)*S**2*U1**(-1) - 8*TG**(-1)*U1**(-1)*MG2**2 - 10
     +    *TG**(-1)*MG2 - 4*TG*U1**(-1) - 4*S**(-1)*MG2 - 8*S*U1**(-1)
     +     - 16*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(21)*N**2*CF * (  - 4*M2*TG**(-2)*
     +    (S+U1)**(-1)*MG2 + 4*M2*TG**(-1)*S**(-1) - 4*M2*TG**(-1)*
     +    (S+U1)**(-1) - 4*TG**(-1)*S**(-1)*MG2 - 4*TG**(-1)*
     +    (S+U1)**(-1)*MG2 - 8*TG**(-1) - 4*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(22)*N*CF**2 * (  - 8*M2*TG**(-2)*MG2 - 8*
     +    M2*TG**(-1) + 8*TG**(-2)*MG2**2 + 8*TG**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(23)*N*S4G*S4G2**(-1)*CF**2 * ( 16*TG**(-1)
     +    *S**(-1)*MG2 + 8*TG**(-1)*S*U1**(-1) + 16*TG**(-1)*U1**(-1)*
     +    MG2 + 8*TG**(-1) + 8*S**(-1) + 16*U1**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(23)*N*CF**2 * ( 8*TG**(-1)*S**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(23)*N**2*S4G*S4G2**(-1)*CF * (  - 8*
     +    TG**(-1)*S**(-1)*MG2 - 2*TG**(-1)*S*U1**(-1) - 4*TG**(-1)*
     +    U1**(-1)*MG2 - 2*TG**(-1) - 4*S**(-1) - 4*U1**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(23)*N**2*CF * (  - 4*TG**(-1)*S**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(43)*N*CF**2 * (  - 8*M2*TG**(-2)*S*MG2 + 8
     +    *M2*TG**(-2)*MG2**2 - 8*M2*TG**(-1)*S - 16*M2*TG**(-1)*MG2 - 
     +    16*M2**2*TG**(-2)*MG2 + 8*M2**2*TG**(-1) + 8*M2**3*TG**(-2)
     +     + 8*TG**(-2)*S*MG2**2 + 8*TG**(-1)*S*MG2 + 8*TG**(-1)*MG2**2
     +     )
     +
      M2GBH2 = M2GBH2 + ANG4(44)*N*S4G*S4G2**(-1)*CF**2 * (  - 16*M2*
     +    TG**(-1)*S*U1**(-1)*MG2 - 16*M2*TG**(-1)*S - 16*M2*TG**(-1)*
     +    S**2*U1**(-1) + 32*M2*TG**(-1)*U1**(-1)*MG2**2 + 8*M2*
     +    TG**(-1)*MG2 - 16*M2*TG*U1**(-1) - 32*M2*S*U1**(-1) - 32*M2*
     +    U1**(-1)*MG2 - 8*M2**2*TG**(-1)*S*U1**(-1) - 48*M2**2*
     +    TG**(-1)*U1**(-1)*MG2 - 8*M2**2*TG**(-1) + 16*M2**3*TG**(-1)*
     +    U1**(-1) + 32*TG**(-1)*S*U1**(-1)*MG2**2 + 32*TG**(-1)*S*MG2
     +     + 32*TG**(-1)*S**2*U1**(-1)*MG2 + 8*TG**(-1)*S**2 + 8*
     +    TG**(-1)*S**3*U1**(-1) + 8*TG*S*U1**(-1) + 16*TG*U1**(-1)*MG2
     +     + 48*S*U1**(-1)*MG2 + 8*S + 16*S**2*U1**(-1) + 32*U1**(-1)*
     +    MG2**2 )
     +
      M2GBH2 = M2GBH2 + ANG4(44)*N*CF**2 * ( 4*M2*TG**(-2)*S*
     +    (S+U1)**(-1)*MG2 + 8*M2*TG**(-2)*MG2 + 4*M2*TG**(-1)*S*
     +    (S+U1)**(-1) + 16*M2*TG**(-1)*(S+U1)**(-1)*MG2 - 16*M2*
     +    TG**(-1) - 8*M2*(S+U1)**(-1) + 8*M2**2*TG**(-2)*(S+U1)**(-1)*
     +    MG2 - 8*M2**2*TG**(-2) - 16*M2**2*TG**(-1)*(S+U1)**(-1) - 8*
     +    M2**3*TG**(-2)*(S+U1)**(-1) + 4*TG**(-2)*S*MG2 + 4*TG**(-1)*S
     +    *(S+U1)**(-1)*MG2 + 12*TG**(-1)*S + 16*TG**(-1)*MG2 + 4*S*
     +    (S+U1)**(-1) + 8*(S+U1)**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(44)*N**2*S4G*S4G2**(-1)*CF * (  - 6*M2*
     +    TG**(-2)*S*MG2 + 8*M2*TG**(-2)*MG2**2 + 4*M2*TG**(-1)*S*
     +    U1**(-1)*MG2 + 4*M2*TG**(-1)*S**2*U1**(-1) - 8*M2*TG**(-1)*
     +    U1**(-1)*MG2**2 - 12*M2*TG**(-1)*MG2 + 4*M2*TG*U1**(-1) + 8*
     +    M2*S*U1**(-1) + 8*M2*U1**(-1)*MG2 - 4*M2 - 12*M2**2*TG**(-2)*
     +    MG2 + 2*M2**2*TG**(-1)*S*U1**(-1) + 12*M2**2*TG**(-1)*
     +    U1**(-1)*MG2 + 4*M2**2*TG**(-1) + 4*M2**3*TG**(-2) - 4*M2**3*
     +    TG**(-1)*U1**(-1) + 8*TG**(-2)*S*MG2**2 - 8*TG**(-1)*S*
     +    U1**(-1)*MG2**2 - 4*TG**(-1)*S*MG2 - 8*TG**(-1)*S**2*U1**(-1)
     +    *MG2 - 2*TG**(-1)*S**2 - 2*TG**(-1)*S**3*U1**(-1) + 8*
     +    TG**(-1)*MG2**2 - 2*TG*S*U1**(-1) - 4*TG*U1**(-1)*MG2 - 12*S*
     +    U1**(-1)*MG2 - 2*S - 4*S**2*U1**(-1) - 8*U1**(-1)*MG2**2 + 4*
     +    MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(44)*N**2*CF * (  - 2*M2*TG**(-2)*S*
     +    (S+U1)**(-1)*MG2 - 2*M2*TG**(-1)*S*(S+U1)**(-1) - 8*M2*
     +    TG**(-1)*(S+U1)**(-1)*MG2 + 4*M2*TG**(-1) + 4*M2*(S+U1)**(-1)
     +     - 4*M2**2*TG**(-2)*(S+U1)**(-1)*MG2 + 8*M2**2*TG**(-1)*
     +    (S+U1)**(-1) + 4*M2**3*TG**(-2)*(S+U1)**(-1) - 2*TG**(-1)*S*
     +    (S+U1)**(-1)*MG2 - 4*TG**(-1)*S - 4*TG**(-1)*MG2 - 2*S*
     +    (S+U1)**(-1) - 4*(S+U1)**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(68)*N*S4G*S4G2**(-1)*CF**2 * ( 32*M2*
     +    TG**(-1)*S**(-1)*MG2 - 8*M2*TG**(-1)*S*U1**(-1) + 16*M2*
     +    TG**(-1)*U1**(-1)*MG2 - 8*M2*TG**(-1) - 16*M2*U1**(-1) - 16*
     +    M2**2*TG**(-1)*S**(-1) - 16*M2**2*TG**(-1)*U1**(-1) + 8*
     +    TG**(-1)*S*U1**(-1)*MG2 + 8*TG**(-1)*MG2 + 8*TG*S**(-1) + 32*
     +    S**(-1)*MG2 + 16*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(68)*N*CF**2 * ( 8*M2*TG**(-2)*(S+U1)**(-1)
     +    *MG2 + 8*M2*TG**(-1)*S**(-1) + 8*M2*TG**(-1)*(S+U1)**(-1) + 4
     +    *TG**(-2)*MG2 - 8*TG**(-1)*S**(-1)*S4 + 8*TG**(-1)*
     +    (S+U1)**(-1)*MG2 + 4*TG**(-1) + 16*S**(-1) + 8*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(68)*N**2*S4G*S4G2**(-1)*CF * ( 2 + 4*M2*
     +    TG**(-2)*MG2 - 8*M2*TG**(-1)*S**(-1)*MG2 + 4*M2*TG**(-1)*S*
     +    U1**(-1) - 8*M2*TG**(-1)*U1**(-1)*MG2 + 6*M2*TG**(-1) + 8*M2*
     +    U1**(-1) + 4*M2**2*TG**(-1)*S**(-1) + 8*M2**2*TG**(-1)*
     +    U1**(-1) - 4*TG**(-1)*S*U1**(-1)*MG2 + 2*TG**(-1)*MG2 - 2*TG*
     +    S**(-1) - 8*S**(-1)*MG2 - 8*U1**(-1)*MG2 )
     +
      M2GBH2 = M2GBH2 + ANG4(68)*N**2*CF * (  - 4*M2*TG**(-2)*
     +    (S+U1)**(-1)*MG2 - 2*M2*TG**(-1)*S**(-1) - 4*M2*TG**(-1)*
     +    (S+U1)**(-1) + 2*TG**(-1)*S**(-1)*S4 - 4*TG**(-1)*
     +    (S+U1)**(-1)*MG2 + 2*TG**(-1) - 4*S**(-1) - 4*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(69)*N*S4G*S4G2**(-1)*CF**2 * ( 16*TG**(-1)
     +    *S**(-1)*MG2 + 8*TG**(-1)*S*U1**(-1) + 16*TG**(-1)*U1**(-1)*
     +    MG2 + 8*TG**(-1) + 8*S**(-1) + 16*U1**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(69)*N*CF**2 * ( 8*TG**(-1)*S**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(69)*N**2*S4G*S4G2**(-1)*CF * (  - 4*
     +    TG**(-1)*S**(-1)*MG2 - 4*TG**(-1)*S*U1**(-1) - 8*TG**(-1)*
     +    U1**(-1)*MG2 - 4*TG**(-1) - 2*S**(-1) - 8*U1**(-1) )
     +
      M2GBH2 = M2GBH2 + ANG4(69)*N**2*CF * (  - 2*TG**(-1)*S**(-1) )
     +
      M2GBH2 = M2GBH2 + COLO1(9)*N*(S4+MS2)*CF**2 * (  - 16*M2*TG**(-2)
     +    *S*S4**(-1)*(S+U1)**(-2)*MG2 + 8*M2*TG**(-2)*S4**(-1) - 32*M2
     +    *TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2 - 16*M2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2) + 32*M2*TG**(-1)*S4**(-1)*(S+U1)**(-1)
     +     - 32*M2*S*S4**(-1)*(S+U1)**(-3) + 48*M2*S4**(-1)*
     +    (S+U1)**(-2) - 16*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-3)*MG2
     +     + 16*M2**2*TG**(-2)*S4**(-1)*(S+U1)**(-1) - 16*M2**2*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-3) + 48*M2**2*TG**(-1)*S4**(-1)
     +    *(S+U1)**(-2) + 16*M2**3*TG**(-2)*S4**(-1)*(S+U1)**(-2) - 8*
     +    TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 - 16*TG**(-1)*S*S4**(-1)
     +    *(S+U1)**(-2)*MG2 - 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-1) + 8*
     +    TG**(-1)*S4**(-1) - 16*TG*S*S4**(-1)*(S+U1)**(-3) + 16*TG*
     +    S4**(-1)*(S+U1)**(-2) - 16*S*S4**(-1)*(S+U1)**(-3)*MG2 - 16*S
     +    *S4**(-1)*(S+U1)**(-2) + 16*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + COLO1(9)*N**2*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 
     +     - 8*M2*TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 - 24*M2*TG**(-1)
     +    *S*S4**(-1)*(S+U1)**(-2)*MG2 - 8*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2*TG**(-1)*S4**(-1) - 16*M2*TG*S*S4**(-1)*
     +    (S+U1)**(-3) + 24*M2*TG*S4**(-1)*(S+U1)**(-2) - 16*M2*S*
     +    S4**(-1)*(S+U1)**(-3)*MG2 - 24*M2*S*S4**(-1)*(S+U1)**(-2) + 
     +    40*M2*S4**(-1)*(S+U1)**(-1) - 8*M2**2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-2)*MG2 + 8*M2**2*TG**(-2)*S4**(-1) - 8*M2**2*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2 - 8*M2**2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2) + 32*M2**2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 8*M2**2*S*S4**(-1)*(S+U1)**(-3) + 24*M2**2*
     +    S4**(-1)*(S+U1)**(-2) + 8*M2**3*TG**(-2)*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2**3*TG**(-1)*S4**(-1)*(S+U1)**(-2) - 8*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-1)*MG2 - 8*TG*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 - 16*TG*S*S4**(-1)*(S+U1)**(-2) + 16*TG*
     +    S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH2 = M2GBH2 + COLO1(9)*N**2*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 
     +     - 8*TG**2*S*S4**(-1)*(S+U1)**(-3) + 8*TG**2*S4**(-1)*
     +    (S+U1)**(-2) - 16*S*S4**(-1)*(S+U1)**(-2)*MG2 - 8*S*S4**(-1)*
     +    (S+U1)**(-1) + 8*S4**(-1) )
     +
      M2GBH2 = M2GBH2 + COLO1(9)*N**2*(S4+MS2)*CF * ( 8*M2*TG**(-2)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 16*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 + 8*M2*TG**(-1)*S*S4**(-1)*(S+U1)**(-2) - 16
     +    *M2*TG**(-1)*S4**(-1)*(S+U1)**(-1) + 16*M2*S*S4**(-1)*
     +    (S+U1)**(-3) - 24*M2*S4**(-1)*(S+U1)**(-2) + 8*M2**2*TG**(-2)
     +    *S*S4**(-1)*(S+U1)**(-3)*MG2 - 8*M2**2*TG**(-2)*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2**2*TG**(-1)*S*S4**(-1)*(S+U1)**(-3) - 24*
     +    M2**2*TG**(-1)*S4**(-1)*(S+U1)**(-2) - 8*M2**3*TG**(-2)*
     +    S4**(-1)*(S+U1)**(-2) + 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-2)*
     +    MG2 + 8*TG*S*S4**(-1)*(S+U1)**(-3) - 8*TG*S4**(-1)*
     +    (S+U1)**(-2) + 8*S*S4**(-1)*(S+U1)**(-3)*MG2 + 8*S*S4**(-1)*
     +    (S+U1)**(-2) - 8*S4**(-1)*(S+U1)**(-1) )

      M2GBH3 = 0.D0
      M2GBH3 = M2GBH3 + N*S4G*(S4+MS2)**(-1)*S4G2**(-1)*CF**2 * ( 8*M2*
     +    TG*S**(-1)*U1**(-1)*MG2 - 16*M2*S**(-1)*U1**(-1)*MG2**2 + 4*
     +    M2*S**(-1)*MG2 - 16*M2*U1**(-2)*MG2**2 + 8*M2*U1**(-1)*MG2 + 
     +    16*M2**2*S**(-1)*U1**(-1)*MG2 + 8*M2**2*U1**(-2)*MG2 - 8*TG*
     +    S**(-1)*U1**(-1)*MG2**2 - 4*S**(-1)*MG2**2 + 8*U1**(-2)*
     +    MG2**3 - 8*U1**(-1)*MG2**2 )
     +
      M2GBH3 = M2GBH3 + N*S4G*S4G2**(-1)*CF**2 * ( 16*M2*TG**(-1)*
     +    S**(-1)*MG2 - 16*M2*TG**(-1)*U1**(-1)*MG2 + 4*M2*TG*S**(-1)*
     +    MG2**(-1) + 16*M2*S**(-1)*U1**(-1)*MG2 - 8*M2*S**(-1) - 16*M2
     +    *U1**(-1) + 8*M2**2*S**(-1)*U1**(-1) + 8*M2**2*S**(-1)*
     +    MG2**(-1) + 16*M2**2*U1**(-2) + 8*M2**2*U1**(-1)*MG2**(-1) - 
     +    8*M2**3*S**(-1)*U1**(-1)*MG2**(-1) - 8*M2**3*U1**(-2)*
     +    MG2**(-1) - 16*TG**(-1)*S*U1**(-1)*MG2 - 16*TG**(-1)*MG2 + 8*
     +    TG*S**(-1)*U1**(-1)*MG2 - 4*TG*S**(-1) + 20*S**(-1)*MG2 - 8*
     +    U1**(-2)*MG2**2 - 8*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + N*(S4+MS2)**(-1)*CF**2 * (  - 4*M2*TG*S**(-1)*
     +    MG2**(-1) + 8*M2*S**(-1) + 8*M2*U1**(-2)*MG2 + 16*M2*U1**(-1)
     +     - 8*M2**2*S**(-1)*U1**(-1) - 8*M2**2*S**(-1)*MG2**(-1) - 16*
     +    M2**2*U1**(-2) - 8*M2**2*U1**(-1)*MG2**(-1) + 8*M2**3*S**(-1)
     +    *U1**(-1)*MG2**(-1) + 8*M2**3*U1**(-2)*MG2**(-1) + 4*TG*
     +    S**(-1) - 8*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + N*(S4+MS2)*CF**2 * (  - 16*M2*TG**(-2)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 - 32*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 - 16*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-3)*
     +    MG2 - 16*TG**(-1)*S*S4**(-1)*(S+U1)**(-2)*MG2 - 16*S*S4**(-1)
     +    *(S+U1)**(-3)*MG2 )
     +
      M2GBH3 = M2GBH3 + N**2*S4G*(S4+MS2)**(-1)*S4G2**(-1)*CF * (  - 4*
     +    M2*TG**(-2)*S*MG2 + 8*M2*TG**(-2)*MG2**2 - 8*M2*TG**(-1)*S*
     +    U1**(-1)*MG2 - 4*M2*TG**(-1)*U1**(-1)*MG2**2 - 4*M2*TG**(-1)*
     +    MG2 - 4*M2*TG*S**(-1)*U1**(-1)*MG2 + 8*M2*S**(-1)*U1**(-1)*
     +    MG2**2 - 12*M2*U1**(-1)*MG2 - 8*M2**2*TG**(-2)*MG2 - 4*M2**2*
     +    TG**(-1)*U1**(-1)*MG2 - 8*M2**2*S**(-1)*U1**(-1)*MG2 + 4*
     +    TG**(-2)*S*MG2**2 + 8*TG**(-1)*S*U1**(-1)*MG2**2 + 8*TG**(-1)
     +    *U1**(-1)*MG2**3 + 4*TG**(-1)*MG2**2 + 4*TG*S**(-1)*U1**(-1)*
     +    MG2**2 + 12*U1**(-1)*MG2**2 )
     +
      M2GBH3 = M2GBH3 + N**2*S4G*(S4+MS2)*S4G2**(-1)*CF * (  - 8*M2*
     +    TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 - 24*M2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 - 16*M2*S*S4**(-1)*(S+U1)**(-3)*MG2
     +     - 8*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-2)*MG2 - 8*M2**2*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2 - 8*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-1)*MG2 - 8*TG*S*S4**(-1)*(S+U1)**(-3)*MG2 - 16*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 )
     +
      M2GBH3 = M2GBH3 + N**2*S4G*S4G2**(-1)*CF * (  - 4 + 8*M2*TG**(-2)
     +    *S - 16*M2*TG**(-2)*MG2 + 4*M2*TG**(-1)*S*MG2**(-1) + 8*M2*
     +    TG**(-1)*U1**(-1)*MG2 - 8*M2*TG**(-1) + 4*M2*MG2**(-1) + 8*
     +    M2**2*TG**(-2) + 8*M2**2*TG**(-1)*S**(-2)*MG2 - 16*M2**2*
     +    TG**(-1)*S**(-1) + 8*M2**2*TG**(-1)*MG2**(-1) + 8*M2**2*
     +    S**(-2)*U1**(-1)*MG2 - 16*M2**2*S**(-1)*U1**(-1) - 4*M2**3*
     +    TG**(-1)*S**(-2) + 8*M2**3*TG**(-1)*S**(-1)*MG2**(-1) - 4*
     +    M2**3*S**(-2)*U1**(-1) + 8*M2**3*S**(-1)*U1**(-1)*MG2**(-1)
     +     - 12*TG**(-2)*S*MG2 + 4*TG**(-1)*S*U1**(-1)*MG2 - 4*TG**(-1)
     +    *S - 8*TG**(-1)*U1**(-1)*MG2**2 - 4*TG*S**(-1)*U1**(-1)*MG2
     +     - 8*S**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + N**2*(S4+MS2)**(-1)*CF * ( 4 - 8*M2*TG**(-2)*S
     +     + 8*M2*TG**(-2)*MG2 + 8*M2*TG**(-1)*S**(-2)*MG2**2 + 8*M2*
     +    TG**(-1)*S**(-2)*MG2**3*(S4G-S)**(-1) - 8*M2*TG**(-1)*S**(-1)
     +    *MG2 - 4*M2*TG**(-1)*S*MG2**(-1) + 8*M2*TG**(-1) + 8*M2*
     +    S**(-2)*U1**(-1)*MG2**2 + 8*M2*S**(-2)*U1**(-1)*MG2**3*
     +    (S4G-S)**(-1) - 8*M2*S**(-1)*U1**(-1)*MG2 - 4*M2*MG2**(-1) - 
     +    8*M2**2*TG**(-2) - 12*M2**2*TG**(-1)*S**(-2)*MG2 - 12*M2**2*
     +    TG**(-1)*S**(-2)*MG2**2*(S4G-S)**(-1) + 16*M2**2*TG**(-1)*
     +    S**(-1) - 8*M2**2*TG**(-1)*MG2**(-1) - 12*M2**2*S**(-2)*
     +    U1**(-1)*MG2 - 12*M2**2*S**(-2)*U1**(-1)*MG2**2*(S4G-S)**(-1)
     +     + 16*M2**2*S**(-1)*U1**(-1) + 4*M2**3*TG**(-1)*S**(-2)*MG2*
     +    (S4G-S)**(-1) + 4*M2**3*TG**(-1)*S**(-2) - 8*M2**3*TG**(-1)*
     +    S**(-1)*MG2**(-1) + 4*M2**3*S**(-2)*U1**(-1)*MG2*
     +    (S4G-S)**(-1) + 4*M2**3*S**(-2)*U1**(-1) - 8*M2**3*S**(-1)*
     +    U1**(-1)*MG2**(-1) + 8*TG**(-2)*S*MG2 + 4*TG**(-1)*S )
     +
      M2GBH3 = M2GBH3 + N**2*(S4+MS2)*CF * ( 8*M2*TG**(-2)*S*S4**(-1)*
     +    (S+U1)**(-2)*MG2 + 16*M2*TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2
     +     + 8*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-3)*MG2 + 8*TG**(-1)*
     +    S*S4**(-1)*(S+U1)**(-2)*MG2 + 8*S*S4**(-1)*(S+U1)**(-3)*MG2 )
     +
      M2GBH3 = M2GBH3 + N**2*CF * (  - 8*M2*TG**(-1)*S**(-2)*MG2**2*
     +    (S4G-S)**(-1) - 8*M2*S**(-2)*U1**(-1)*MG2**2*(S4G-S)**(-1) + 
     +    4*M2**2*TG**(-1)*S**(-2)*MG2*(S4G-S)**(-1) + 4*M2**2*S**(-2)*
     +    U1**(-1)*MG2*(S4G-S)**(-1) )
     +
      M2GBH3 = M2GBH3 + ANG4(21)*N*S4G*S4G2**(-1)*CF**2 * (  - 16*M2*
     +    TG**(-1)*S**(-1)*MG2 - 32*TG**(-1)*S*U1**(-1)*MG2 - 32*
     +    TG**(-1)*U1**(-1)*MG2**2 - 32*TG**(-1)*MG2 - 8*S**(-1)*MG2 - 
     +    32*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(21)*N*CF**2 * (  - 8*M2*TG**(-2)*
     +    (S+U1)**(-1)*MG2 - 4*TG**(-2)*MG2 - 8*TG**(-1)*S**(-1)*MG2 - 
     +    8*TG**(-1)*(S+U1)**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(21)*N**2*S4G*S4G2**(-1)*CF * ( 4*M2*
     +    TG**(-2)*MG2 + 8*M2*TG**(-1)*S**(-1)*MG2 - 8*TG**(-2)*MG2**2
     +     + 8*TG**(-1)*S*U1**(-1)*MG2 + 8*TG**(-1)*U1**(-1)*MG2**2 + 
     +    10*TG**(-1)*MG2 + 4*S**(-1)*MG2 + 8*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(21)*N**2*CF * ( 4*M2*TG**(-2)*(S+U1)**(-1)
     +    *MG2 + 4*TG**(-1)*S**(-1)*MG2 + 4*TG**(-1)*(S+U1)**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(22)*N*CF**2 * ( 8*M2*TG**(-2)*MG2 - 8*
     +    TG**(-2)*MG2**2 )
     +
      M2GBH3 = M2GBH3 + ANG4(23)*N*S4G*S4G2**(-1)*CF**2 * (  - 16*
     +    TG**(-1)*S**(-1)*MG2 - 16*TG**(-1)*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(23)*N**2*S4G*S4G2**(-1)*CF * ( 8*TG**(-1)*
     +    S**(-1)*MG2 + 4*TG**(-1)*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(43)*N*CF**2 * ( 8*M2*TG**(-2)*S*MG2 - 8*M2
     +    *TG**(-2)*MG2**2 + 8*M2*TG**(-1)*MG2 + 8*M2**2*TG**(-2)*MG2
     +     - 8*TG**(-2)*S*MG2**2 - 8*TG**(-1)*MG2**2 )
     +
      M2GBH3 = M2GBH3 + ANG4(44)*N*S4G*S4G2**(-1)*CF**2 * (  - 32*M2*
     +    TG**(-1)*U1**(-1)*MG2**2 - 24*M2*TG**(-1)*MG2 + 16*M2**2*
     +    TG**(-1)*U1**(-1)*MG2 - 32*TG**(-1)*S*U1**(-1)*MG2**2 - 16*
     +    TG**(-1)*S*MG2 - 16*TG**(-1)*S**2*U1**(-1)*MG2 - 16*TG*
     +    U1**(-1)*MG2 - 32*S*U1**(-1)*MG2 - 32*U1**(-1)*MG2**2 - 16*
     +    MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(44)*N*CF**2 * (  - 4*M2*TG**(-2)*S*
     +    (S+U1)**(-1)*MG2 - 8*M2*TG**(-2)*MG2 - 16*M2*TG**(-1)*
     +    (S+U1)**(-1)*MG2 - 8*M2**2*TG**(-2)*(S+U1)**(-1)*MG2 - 4*
     +    TG**(-2)*S*MG2 - 4*TG**(-1)*S*(S+U1)**(-1)*MG2 - 16*TG**(-1)*
     +    MG2 - 8*(S+U1)**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(44)*N**2*S4G*S4G2**(-1)*CF * ( 6*M2*
     +    TG**(-2)*S*MG2 - 8*M2*TG**(-2)*MG2**2 + 8*M2*TG**(-1)*
     +    U1**(-1)*MG2**2 + 12*M2*TG**(-1)*MG2 + 4*M2**2*TG**(-2)*MG2
     +     - 4*M2**2*TG**(-1)*U1**(-1)*MG2 - 8*TG**(-2)*S*MG2**2 + 8*
     +    TG**(-1)*S*U1**(-1)*MG2**2 + 4*TG**(-1)*S*MG2 + 4*TG**(-1)*
     +    S**2*U1**(-1)*MG2 - 8*TG**(-1)*MG2**2 + 4*TG*U1**(-1)*MG2 + 8
     +    *S*U1**(-1)*MG2 + 8*U1**(-1)*MG2**2 + 4*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(44)*N**2*CF * ( 2*M2*TG**(-2)*S*
     +    (S+U1)**(-1)*MG2 + 8*M2*TG**(-1)*(S+U1)**(-1)*MG2 + 4*M2**2*
     +    TG**(-2)*(S+U1)**(-1)*MG2 + 2*TG**(-1)*S*(S+U1)**(-1)*MG2 + 4
     +    *TG**(-1)*MG2 + 4*(S+U1)**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(68)*N*S4G*S4G2**(-1)*CF**2 * (  - 32*M2*
     +    TG**(-1)*S**(-1)*MG2 - 16*M2*TG**(-1)*U1**(-1)*MG2 - 8*
     +    TG**(-1)*S*U1**(-1)*MG2 - 8*TG**(-1)*MG2 - 32*S**(-1)*MG2 - 
     +    16*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(68)*N*CF**2 * (  - 8*M2*TG**(-2)*
     +    (S+U1)**(-1)*MG2 - 4*TG**(-2)*MG2 - 8*TG**(-1)*(S+U1)**(-1)*
     +    MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(68)*N**2*S4G*S4G2**(-1)*CF * (  - 4*M2*
     +    TG**(-2)*MG2 + 8*M2*TG**(-1)*S**(-1)*MG2 + 8*M2*TG**(-1)*
     +    U1**(-1)*MG2 + 4*TG**(-1)*S*U1**(-1)*MG2 - 2*TG**(-1)*MG2 + 8
     +    *S**(-1)*MG2 + 8*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(68)*N**2*CF * ( 4*M2*TG**(-2)*(S+U1)**(-1)
     +    *MG2 + 4*TG**(-1)*(S+U1)**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(69)*N*S4G*S4G2**(-1)*CF**2 * (  - 16*
     +    TG**(-1)*S**(-1)*MG2 - 16*TG**(-1)*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + ANG4(69)*N**2*S4G*S4G2**(-1)*CF * ( 4*TG**(-1)*
     +    S**(-1)*MG2 + 8*TG**(-1)*U1**(-1)*MG2 )
     +
      M2GBH3 = M2GBH3 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 16*M2*TG**(-2)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 32*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 + 16*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-3)*
     +    MG2 + 8*TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 + 16*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 16*S*S4**(-1)*(S+U1)**(-3)*MG2 )
     +
      M2GBH3 = M2GBH3 + COLO1(9)*N**2*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 8*
     +    M2*TG**(-2)*S*S4**(-1)*(S+U1)**(-1)*MG2 + 24*M2*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 16*M2*S*S4**(-1)*(S+U1)**(-3)*MG2
     +     + 8*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-2)*MG2 + 8*M2**2*
     +    TG**(-1)*S*S4**(-1)*(S+U1)**(-3)*MG2 + 8*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 8*TG*S*S4**(-1)*(S+U1)**(-3)*MG2 + 16*S*
     +    S4**(-1)*(S+U1)**(-2)*MG2 )
     +
      M2GBH3 = M2GBH3 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 8*M2*TG**(-2)*
     +    S*S4**(-1)*(S+U1)**(-2)*MG2 - 16*M2*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 - 8*M2**2*TG**(-2)*S*S4**(-1)*(S+U1)**(-3)*
     +    MG2 - 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-2)*MG2 - 8*S*S4**(-1)*
     +    (S+U1)**(-3)*MG2 )

      M2GBH4 = 0.D0
      M2GBH4 = M2GBH4 + N*S4G*(S4+MS2)*S4G2**(-1)*CF * (  - 8*M2*
     +    TG**(-1)*S*T1**(-1)*U1**(-1)*S4**(-1)*MG2 + 24*M2*TG**(-1)*S*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 4*M2*TG**(-1)*S*S4**(-1)
     +    *(S+T1)**(-2)*MG2 - 8*M2*TG**(-1)*S*S4**(-1)*(S+T1)**(-1) - 8
     +    *M2*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 4*M2*
     +    TG**(-1)*S**2*S4**(-1)*(S+T1)**(-2) + 4*M2*TG**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 + 8*M2*TG**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 
     +    8*M2*TG*S**(-1)*T1**(-1)*S4**(-1) - 32*M2*TG*S**(-1)*U1**(-1)
     +    *S4**(-1) + 16*M2*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) + 32*M2*TG
     +    *U1**(-1)*S4**(-1)*(S+U1)**(-1) + 8*M2*TG*S4**(-1)*
     +    (S+U1)**(-2) - 24*M2*S**(-1)*S4**(-1) - 8*M2*S*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) + 16*M2*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 4*M2*S*S4**(-1)*(S+T1)**(-2) - 16*M2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MG2 + 16*M2*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MG2 - 24*M2*U1**(-1)*S4**(-1) + 8*M2*S4**(-1)*(S+U1)**(-2)*
     +    MG2 )
     +
      M2GBH4 = M2GBH4 + N*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 24*M2*S4**(-1)
     +    *(S+U1)**(-1) - 16*M2**2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) - 16*M2**2*TG**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*
     +    MG2 + 16*M2**2*TG**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 
     +    8*M2**2*TG**(-1)*S4**(-1)*(S+T1)**(-1) - 16*M2**2*S**(-1)*
     +    U1**(-1)*S4**(-1) + 8*M2**2*S**(-1)*S4**(-1)*(S+U1)**(-1) + 8
     +    *M2**2*T1**(-1)*U1**(-1)*S4**(-1) - 8*M2**2*U1**(-1)*S4**(-1)
     +    *(S+T1)**(-1) + 16*M2**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*
     +    M2**3*TG**(-1)*S**(-1)*T1**(-1)*S4**(-1) + 8*M2**3*TG**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1) - 8*M2**3*TG**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) + 8*TG**(-1)*S*S4**(-1)*(S+T1)**(-1)*
     +    MG2 + 8*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 4*
     +    TG**(-1)*S**2*S4**(-1)*(S+T1)**(-2)*MG2 + 4*TG**(-1)*S**2*
     +    S4**(-1)*(S+T1)**(-1) - 4*TG**(-1)*S**3*S4**(-1)*(S+T1)**(-2)
     +     + 4*TG**(-1)*S4**(-1)*MG2 - 8*TG*S**(-1)*S4**(-1) + 8*TG*S*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + N*S4G*(S4+MS2)*S4G2**(-1)*CF * (  - 4*TG*S*
     +    S4**(-1)*(S+T1)**(-2) - 16*TG*U1**(-1)*S4**(-1) + 8*TG*
     +    S4**(-1)*(S+T1)**(-1) + 8*TG*S4**(-1)*(S+U1)**(-2)*MG2 + 24*
     +    TG*S4**(-1)*(S+U1)**(-1) - 16*TG**2*S**(-1)*U1**(-1)*S4**(-1)
     +     + 8*TG**2*S**(-1)*S4**(-1)*(S+U1)**(-1) + 16*TG**2*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 8*TG**2*S4**(-1)*(S+U1)**(-2) - 8*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2 + 8*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 8*S*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 8
     +    *S*U1**(-1)*S4**(-1) - 4*S*S4**(-1)*(S+T1)**(-2)*MG2 + 12*S*
     +    S4**(-1)*(S+T1)**(-1) - 8*S**2*S4**(-1)*(S+T1)**(-2) + 8*
     +    U1**(-1)*S4**(-1)*MG2 + 4*S4**(-1)*(S+T1)**(-1)*MG2 + 8*
     +    S4**(-1)*(S+U1)**(-1)*MG2 - 8*S4**(-1) )
     +
      M2GBH4 = M2GBH4 + N*S4G*S4G2**(-1)*CF * ( 8*M2*TG**(-1)*S**(-1)*
     +    MG2 - 8*M2*TG**(-1)*S*U1**(-1) - 8*M2*TG**(-1)*U1**(-1)*MG2
     +     - 8*M2*TG**(-1) + 16*M2*TG*S**(-1)*U1**(-1) + 16*M2*S**(-1)*
     +    U1**(-1)*MG2 + 12*M2*S**(-1) + 12*M2*U1**(-1) + 12*TG**(-1)*S
     +    *U1**(-1)*MG2 + 16*TG**(-1)*U1**(-1)*MG2**2 - 4*TG**(-1)*MG2
     +     + 16*TG*S**(-1)*U1**(-1)*MG2 + 12*TG*S**(-1) + 20*TG*
     +    U1**(-1) + 16*TG**2*S**(-1)*U1**(-1) + 4*S**(-1)*MG2 + 8*S*
     +    U1**(-1) + 16*U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + N*(S4+MS2)*CF * ( 4*M2*TG**(-1)*S*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 48*M2*TG**(-1)*S*
     +    U1**(-1)*S4**(-2) + 24*M2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) + 16*M2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 4*M2*TG**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MG2
     +     - 4*M2*TG**(-1)*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 16*M2*
     +    TG**(-1)*U1**(-2)*S4**(-1)*MG2 - 16*M2*TG**(-1)*U1**(-1)*
     +    S4**(-2)*MG2 + 8*M2*TG**(-1)*U1**(-1)*S4**(-1) + 8*M2*
     +    TG**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 - 32*M2*TG**(-1)*S4**(-2)
     +     - 8*M2*TG**(-1)*S4**(-1)*(S+U1)**(-2)*MG2 + 16*M2*TG**(-1)*
     +    S4**(-1)*(S+U1)**(-1) - 96*M2*TG*S**(-1)*U1**(-1)*S4**(-2) + 
     +    48*M2*TG*S**(-1)*S4**(-2)*(S+U1)**(-1) + 16*M2*S**(-1)*
     +    U1**(-1)*S4**(-1) - 32*M2*S**(-1)*S4**(-2) + 32*M2*S**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 16*M2*T1**(-1)*U1**(-2)*S4**(-1)*MG2
     +     - 144*M2*U1**(-1)*S4**(-2) + 16*M2*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) )
     +
      M2GBH4 = M2GBH4 + N*(S4+MS2)*CF * ( 48*M2*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2*S4**(-2)*(S+U1)**(-1) - 8*M2*S4**(-1)*
     +    (S+U1)**(-2) + 8*M2**2*TG**(-1)*S**(-1)*U1**(-1)*S4**(-1) - 
     +    16*M2**2*TG**(-1)*S**(-1)*S4**(-2) + 8*M2**2*TG**(-1)*S**(-1)
     +    *S4**(-1)*(S+U1)**(-1) + 16*M2**2*TG**(-1)*T1**(-1)*U1**(-2)*
     +    S4**(-1)*MG2 + 16*M2**2*TG**(-1)*U1**(-2)*S4**(-1) - 64*M2**2
     +    *TG**(-1)*U1**(-1)*S4**(-2) + 16*M2**2*TG**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) + 16*M2**2*TG**(-1)*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 8*M2**2*S**(-1)*T1**(-1)*S4**(-1)*(S+U1)**(-1)
     +     - 96*M2**2*S**(-1)*U1**(-1)*S4**(-2) + 48*M2**2*S**(-1)*
     +    S4**(-2)*(S+U1)**(-1) - 16*M2**2*T1**(-1)*U1**(-2)*S4**(-1)
     +     + 8*M2**2*T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 8*M2**3*
     +    TG**(-1)*S**(-1)*T1**(-1)*S4**(-1)*(S+U1)**(-1) - 32*M2**3*
     +    TG**(-1)*S**(-1)*U1**(-1)*S4**(-2) + 16*M2**3*TG**(-1)*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1) - 16*M2**3*TG**(-1)*T1**(-1)*
     +    U1**(-2)*S4**(-1) )
     +
      M2GBH4 = M2GBH4 + N*(S4+MS2)*CF * ( 8*M2**3*TG**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) - 16*TG**(-1)*S*U1**(-1)*
     +    S4**(-2)*MG2 + 8*TG**(-1)*S*U1**(-1)*S4**(-1)*(S+U1)**(-1)*
     +    MG2 - 16*TG**(-1)*S*S4**(-2) + 8*TG**(-1)*S*S4**(-1)*
     +    (S+U1)**(-1) - 16*TG**(-1)*S**2*U1**(-1)*S4**(-2) + 8*
     +    TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 8*TG**(-1)*
     +    S**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*TG**(-1)*S4**(-2)*MG2
     +     + 8*TG*S**(-1)*U1**(-1)*S4**(-1) - 16*TG*S**(-1)*S4**(-2) + 
     +    16*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) - 80*TG*U1**(-1)*S4**(-2)
     +     + 24*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 8*TG*S4**(-2)*
     +    (S+U1)**(-1) - 8*TG*S4**(-1)*(S+U1)**(-2) - 32*TG**2*S**(-1)*
     +    U1**(-1)*S4**(-2) + 16*TG**2*S**(-1)*S4**(-2)*(S+U1)**(-1) + 
     +    4*S*T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 64*S*
     +    U1**(-1)*S4**(-2) + 8*S*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 28*S
     +    *U1**(-1)*S4**(-1)*(S+U1)**(-1) + 4*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + N*(S4+MS2)*CF * (  - 4*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 - 16*U1**(-1)*S4**(-2)*MG2 + 28*U1**(-1)*
     +    S4**(-1) + 8*S4**(-2)*(S+U1)**(-1)*MG2 - 40*S4**(-2) - 8*
     +    S4**(-1)*(S+U1)**(-2)*MG2 + 12*S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + N*CF * ( 4*M2*TG**(-1)*S**(-1) + 4*M2*TG**(-1)*
     +    U1**(-1) + 4*TG**(-1)*S**(-1)*MG2 + 8*TG**(-1)*S*U1**(-1) + 8
     +    *TG**(-1)*U1**(-1)*MG2 + 4*S**(-1) + 12*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + S4G*(S4+MS2)*S4G2**(-1)*CF**2 * ( 32*M2*TG*
     +    S**(-1)*U1**(-1)*S4**(-1) - 32*M2*TG*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 32*M2*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 16*
     +    M2*S*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 16*M2*U1**(-1)*S4**(-1)
     +     - 16*M2*S4**(-1)*(S+U1)**(-1) + 16*M2**2*S**(-1)*U1**(-1)*
     +    S4**(-1) - 16*M2**2*S**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2**2*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*TG*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 8*TG*U1**(-1)*S4**(-1) - 8*TG*S4**(-1)*
     +    (S+U1)**(-1) + 16*TG**2*S**(-1)*U1**(-1)*S4**(-1) - 16*TG**2*
     +    S**(-1)*S4**(-1)*(S+U1)**(-1) - 16*TG**2*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 8*S*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 8*
     +    U1**(-1)*S4**(-1)*MG2 + 8*S4**(-1)*(S+U1)**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + S4G*S4G2**(-1)*CF**2 * (  - 8 - 32*M2*TG*
     +    S**(-1)*U1**(-1) - 32*M2*TG*U1**(-2) - 32*M2*S**(-1)*U1**(-1)
     +    *MG2 - 32*M2*S**(-1) - 32*M2*S*U1**(-2) - 32*M2*U1**(-2)*MG2
     +     - 64*M2*U1**(-1) - 32*TG*S**(-1)*U1**(-1)*MG2 - 24*TG*
     +    S**(-1) + 32*TG*U1**(-2)*MG2 - 32*TG*U1**(-1) - 32*TG**2*
     +    S**(-1)*U1**(-1) + 32*S*U1**(-2)*MG2 - 8*S*U1**(-1) + 32*
     +    U1**(-2)*MG2**2 + 32*U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + (S4+MS2)*CF**2 * (  - 16*M2*TG**(-1)*S**(-1)*
     +    S4**(-1) - 32*M2*TG**(-1)*S*T1**(-1)*U1**(-2)*S4**(-1)*MG2 - 
     +    8*M2*TG**(-1)*S*T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2
     +     + 48*M2*TG**(-1)*S*U1**(-1)*S4**(-2) - 8*M2*TG**(-1)*S*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1) - 16*M2*TG**(-1)*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 8*M2*TG**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 + 8*M2*TG**(-1)*T1**(-1)*S4**(-1)*(S+U1)**(-1)*
     +    MG2 + 16*M2*TG**(-1)*U1**(-1)*S4**(-2)*MG2 - 8*M2*TG**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 16*M2*TG**(-1)*S4**(-2)*
     +    (S+U1)**(-1)*MG2 + 32*M2*TG**(-1)*S4**(-2) + 32*M2*TG**(-1)*
     +    S4**(-1)*(S+U1)**(-2)*MG2 - 16*M2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 96*M2*TG*S**(-1)*U1**(-1)*S4**(-2) - 96*M2*TG*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1) - 16*M2*S**(-1)*T1**(-1)*
     +    S4**(-1) - 64*M2*S**(-1)*S4**(-1)*(S+U1)**(-1) + 32*M2*S*
     +    T1**(-2)*U1**(-2)*S4**(-1)*MG2 - 16*M2*T1**(-1)*U1**(-1)*
     +    S4**(-1) )
     +
      M2GBH4 = M2GBH4 + (S4+MS2)*CF**2 * ( 144*M2*U1**(-1)*S4**(-2) - 
     +    64*M2*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2*S4**(-2)*
     +    (S+U1)**(-1) + 32*M2*S4**(-1)*(S+U1)**(-2) - 16*M2**2*
     +    TG**(-1)*S**(-1)*T1**(-1)*S4**(-1) - 16*M2**2*TG**(-1)*
     +    S**(-1)*S4**(-1)*(S+U1)**(-1) + 32*M2**2*TG**(-1)*S*T1**(-2)*
     +    U1**(-2)*S4**(-1)*MG2 + 32*M2**2*TG**(-1)*S*T1**(-1)*U1**(-2)
     +    *S4**(-1) - 16*M2**2*TG**(-1)*T1**(-1)*U1**(-1)*S4**(-1) + 64
     +    *M2**2*TG**(-1)*U1**(-1)*S4**(-2) - 16*M2**2*TG**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2**2*S**(-1)*T1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 96*M2**2*S**(-1)*U1**(-1)*S4**(-2) - 
     +    96*M2**2*S**(-1)*S4**(-2)*(S+U1)**(-1) - 32*M2**2*S*T1**(-2)*
     +    U1**(-2)*S4**(-1) - 16*M2**2*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 16*M2**3*TG**(-1)*S**(-1)*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 32*M2**3*TG**(-1)*S**(-1)*U1**(-1)*S4**(-2) - 
     +    32*M2**3*TG**(-1)*S**(-1)*S4**(-2)*(S+U1)**(-1) - 32*M2**3*
     +    TG**(-1)*S*T1**(-2)*U1**(-2)*S4**(-1) )
     +
      M2GBH4 = M2GBH4 + (S4+MS2)*CF**2 * (  - 16*M2**3*TG**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 16*TG**(-1)*S*
     +    U1**(-1)*S4**(-2)*MG2 - 8*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 8*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 16*TG**(-1)*S*S4**(-2) - 8*TG**(-1)*S*
     +    S4**(-1)*(S+U1)**(-1) + 16*TG**(-1)*S**2*U1**(-1)*S4**(-2) - 
     +    8*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 8*TG**(-1)*
     +    S**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 24*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 - 32*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) + 80*
     +    TG*U1**(-1)*S4**(-2) - 32*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1)
     +     - 16*TG*S4**(-2)*(S+U1)**(-1) + 32*TG*S4**(-1)*(S+U1)**(-2)
     +     + 32*TG**2*S**(-1)*U1**(-1)*S4**(-2) - 32*TG**2*S**(-1)*
     +    S4**(-2)*(S+U1)**(-1) - 32*S**(-1)*S4**(-1) - 8*S*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 64*S*U1**(-1)*S4**(-2)
     +     - 8*S*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 32*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + (S4+MS2)*CF**2 * ( 8*T1**(-1)*U1**(-1)*S4**(-1)
     +    *MG2 + 8*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 16*U1**(-1)*
     +    S4**(-2)*MG2 - 8*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 16*
     +    U1**(-1)*S4**(-1) - 16*S4**(-2)*(S+U1)**(-1)*MG2 + 32*
     +    S4**(-2) + 32*S4**(-1)*(S+U1)**(-2)*MG2 + 16*S4**(-1)*
     +    (S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + CF**2 * (  - 8*S**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(16)*N*S4G*S4G2**(-1)*CF * (  - 2 + 2*M2*
     +    TG**(-1)*S**(-1)*MG2 + 2*M2*TG*S**(-1)*U1**(-1) + 2*M2*
     +    S**(-1)*U1**(-1)*MG2 + 2*M2*S**(-1) + 4*M2*U1**(-1) - 2*
     +    TG**(-1)*MG2 + 2*TG*S**(-1)*U1**(-1)*MG2 + 2*TG*S**(-1) + 2*
     +    TG**2*S**(-1)*U1**(-1) + 2*S**(-1)*MG2 - 4*U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(16)*N*CF * ( 2*TG**(-1)*S**(-1)*MG2 + 2*
     +    S**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(16)*S4G*S4G2**(-1)*CF**2 * (  - 4*M2*TG*
     +    S**(-1)*U1**(-1) - 4*M2*S**(-1)*U1**(-1)*MG2 - 8*M2*S**(-1)
     +     - 8*M2*U1**(-1) - 4*TG*S**(-1)*U1**(-1)*MG2 - 4*TG*S**(-1)
     +     - 4*TG**2*S**(-1)*U1**(-1) + 4*S**(-1)*MG2 + 8*U1**(-1)*MG2
     +     )
     +
      M2GBH4 = M2GBH4 + ANG4(17)*N*CF * (  - 4*M2*TG**(-1)*S**(-1)*S4*
     +    MG2 - 8*M2*TG**(-1)*S*U1**(-1)*MG2 + 2*M2*TG**(-1)*S*
     +    (S+U1)**(-1)*MG2 - 4*M2*TG**(-1)*S + 10*M2*TG**(-1)*S4 - 8*M2
     +    *TG**(-1)*MG2 - 12*M2*TG*(S+U1)**(-1) + 4*M2*S**(-1)*S4 + 8*
     +    M2*S*U1**(-1) + 2*M2*S*(S+U1)**(-1) + 4*M2**2*TG**(-1)*
     +    S**(-1)*S4 + 8*M2**2*TG**(-1)*S*U1**(-1) + 4*M2**2*TG**(-1)
     +     - 12*M2**2*(S+U1)**(-1) - 4*M2**3*TG**(-1)*(S+U1)**(-1) + 4*
     +    TG**(-1)*S**(-1)*S4**2*MG2 + 4*TG**(-1)*S**(-1)*S4**3 + 2*
     +    TG**(-1)*S*S4 + 6*TG**(-1)*S*MG2 - 14*TG**(-1)*S4*MG2 - 6*
     +    TG**(-1)*S4**2 + 2*TG*S*(S+U1)**(-1) - 4*TG - 4*TG**2*
     +    (S+U1)**(-1) - 4*S**(-1)*S4*MG2 + 4*S**(-1)*S4**2 - 8*S*
     +    U1**(-1)*MG2 + 2*S*(S+U1)**(-1)*MG2 + 2*S - 4*S4 - 8*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(17)*CF**2 * ( 4*M2*TG**(-1)*S**(-1)*S4*MG2
     +     - 4*M2*TG**(-1)*S*(S+U1)**(-1)*MG2 + 8*M2*TG**(-1)*S - 8*M2*
     +    TG**(-1)*S4 + 8*M2*TG**(-1)*MG2 + 24*M2*TG*(S+U1)**(-1) - 4*
     +    M2*S**(-1)*S4 - 4*M2*S*(S+U1)**(-1) + 8*M2 - 4*M2**2*TG**(-1)
     +    *S**(-1)*S4 + 24*M2**2*(S+U1)**(-1) + 8*M2**3*TG**(-1)*
     +    (S+U1)**(-1) - 4*TG**(-1)*S**(-1)*S4**2*MG2 - 4*TG**(-1)*
     +    S**(-1)*S4**3 - 4*TG**(-1)*S*S4 - 12*TG**(-1)*S*MG2 + 16*
     +    TG**(-1)*S4*MG2 + 8*TG**(-1)*S4**2 - 4*TG*S*(S+U1)**(-1) + 8*
     +    TG + 8*TG**2*(S+U1)**(-1) + 4*S**(-1)*S4*MG2 - 4*S**(-1)*
     +    S4**2 - 4*S*(S+U1)**(-1)*MG2 - 4*S + 8*S4 + 8*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(21)*N*S4G*S4G2**(-1)*CF * (  - 2 - 4*TG*
     +    S**(-1) - 4*TG*U1**(-1) - 8*S**(-1)*MG2 - 2*S*U1**(-1) - 4*
     +    U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(21)*N*CF * ( 4*M2*TG**(-1)*S**(-1) - 4*
     +    TG**(-1)*S**(-1)*S4 - 8*TG**(-1)*S**(-1)*MG2 - 2*TG**(-1)*S*
     +    U1**(-1) - 4*TG**(-1)*U1**(-1)*MG2 - 2*TG**(-1) - 8*S**(-1)
     +     - 4*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(21)*S4G*S4G2**(-1)*CF**2 * ( 4 + 4*TG*
     +    S**(-1) + 8*TG*U1**(-1) + 8*S**(-1)*MG2 + 4*S*U1**(-1) + 8*
     +    U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(21)*CF**2 * (  - 4*M2*TG**(-1)*S**(-1) + 4
     +    *TG**(-1)*S**(-1)*S4 + 8*TG**(-1)*S**(-1)*MG2 + 4*TG**(-1)*S*
     +    U1**(-1) + 8*TG**(-1)*U1**(-1)*MG2 + 4*TG**(-1) + 8*S**(-1)
     +     + 8*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(39)*N*S4G*S4G2**(-1)*CF * (  - 12*M2*
     +    TG**(-1)*MG2 + 12*M2*TG*S**(-1) + 12*M2*TG*U1**(-1) + 12*M2*
     +    TG**2*S**(-1)*U1**(-1) + 4*M2*S*U1**(-1) - 12*M2*U1**(-1)*MG2
     +     + 2*M2 + 4*M2**2*TG**(-1) + 12*M2**2*TG*S**(-1)*U1**(-1) + 
     +    12*M2**2*S**(-1) + 12*M2**2*U1**(-1) + 4*M2**3*TG**(-1)*
     +    S**(-1) + 4*M2**3*S**(-1)*U1**(-1) + 2*TG**(-1)*S*MG2 - 12*TG
     +    *U1**(-1)*MG2 - 4*TG + 4*TG**2*S**(-1) + 4*TG**3*S**(-1)*
     +    U1**(-1) - 4*S*U1**(-1)*MG2 - 12*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(39)*N*CF * (  - 8 + 2*M2*TG**(-1)*S**(-1)*
     +    S4 + 2*M2*TG**(-1) + 8*M2*S**(-1) + 4*M2**2*TG**(-1)*S**(-1)
     +     + 6*TG**(-1)*S**(-1)*S4*MG2 + 2*TG**(-1)*S**(-1)*S4**2 + 2*
     +    TG**(-1)*S - 4*TG**(-1)*S4 - 12*TG**(-1)*MG2 + 4*TG*S**(-1)
     +     + 8*S**(-1)*S4 )
     +
      M2GBH4 = M2GBH4 + ANG4(39)*S4G*S4G2**(-1)*CF**2 * (  - 16*M2*TG*
     +    S**(-1) - 28*M2*TG*U1**(-1) - 24*M2*TG**2*S**(-1)*U1**(-1) - 
     +    4*M2*S**(-1)*MG2 - 16*M2*S*U1**(-1) + 20*M2*U1**(-1)*MG2 - 16
     +    *M2 - 24*M2**2*TG*S**(-1)*U1**(-1) - 16*M2**2*S**(-1) - 24*
     +    M2**2*U1**(-1) - 8*M2**3*S**(-1)*U1**(-1) - 8*TG*S**(-1)*MG2
     +     + 20*TG*U1**(-1)*MG2 - 8*TG**2*S**(-1) - 4*TG**2*U1**(-1) - 
     +    8*TG**3*S**(-1)*U1**(-1) + 16*S*U1**(-1)*MG2 + 20*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(39)*CF**2 * ( 8 - 16*M2*S**(-1) - 4*
     +    TG**(-1)*S**(-1)*S4*MG2 + 4*TG**(-1)*MG2 - 8*TG*S**(-1) - 8*
     +    S**(-1)*S4 + 4*S**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(43)*CF**2 * (  - 16*M2*TG**(-1)*MG2 - 16*
     +    M2 + 16*TG**(-1)*MG2**2 + 16*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(44)*N*S4G*S4G2**(-1)*CF * (  - 4*M2*
     +    TG**(-1)*MG2 + 8*M2*TG*S**(-1) + 12*M2*TG*U1**(-1) + 6*M2*S*
     +    U1**(-1) + 4*M2*U1**(-1)*MG2 + 4*M2 + 8*M2**2*S**(-1) + 4*
     +    M2**2*U1**(-1) + 8*TG**(-1)*MG2**2 + 4*TG*S**(-1)*MG2 - 2*TG*
     +    S*U1**(-1) - 12*TG*U1**(-1)*MG2 + 2*TG + 4*TG**2*S**(-1) - 8*
     +    S*U1**(-1)*MG2 - 2*S - 2*S**2*U1**(-1) - 8*U1**(-1)*MG2**2 - 
     +    2*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(44)*N*CF * (  - 8 - 4*M2*TG**(-1)*S**(-1)*
     +    MG2 + 6*M2*TG**(-1)*S*U1**(-1) + 4*M2*TG**(-1)*U1**(-1)*MG2
     +     - 8*M2*TG**(-1)*(S+U1)**(-1)*MG2 + 14*M2*TG**(-1) + 12*M2*
     +    S**(-1) + 12*M2*U1**(-1) - 8*M2*(S+U1)**(-1) + 4*M2**2*
     +    TG**(-1)*S**(-1) + 4*M2**2*TG**(-1)*U1**(-1) + 4*TG**(-1)*
     +    S**(-1)*S4*MG2 + 4*TG**(-1)*S**(-1)*S4**2 - 8*TG**(-1)*S*
     +    U1**(-1)*MG2 - 2*TG**(-1)*S - 2*TG**(-1)*S**2*U1**(-1) - 8*
     +    TG**(-1)*U1**(-1)*MG2**2 - 6*TG**(-1)*S4 - 18*TG**(-1)*MG2 + 
     +    4*TG*S**(-1) - 8*TG*(S+U1)**(-1) + 4*S**(-1)*S4 - 8*S**(-1)*
     +    MG2 - 2*S*U1**(-1) - 12*U1**(-1)*MG2 - 8*(S+U1)**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(44)*S4G*S4G2**(-1)*CF**2 * (  - 8*M2*TG*
     +    S**(-1) - 24*M2*TG*U1**(-1) - 12*M2*S*U1**(-1) - 8*M2*
     +    U1**(-1)*MG2 - 12*M2 - 8*M2**2*S**(-1) - 8*M2**2*U1**(-1) - 4
     +    *TG*S**(-1)*MG2 + 4*TG*S*U1**(-1) + 24*TG*U1**(-1)*MG2 - 4*
     +    TG**2*S**(-1) + 16*S*U1**(-1)*MG2 + 4*S + 4*S**2*U1**(-1) + 
     +    16*U1**(-1)*MG2**2 + 16*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(44)*CF**2 * ( 16 + 4*M2*TG**(-1)*S**(-1)*
     +    MG2 - 12*M2*TG**(-1)*S*U1**(-1) - 8*M2*TG**(-1)*U1**(-1)*MG2
     +     + 16*M2*TG**(-1)*(S+U1)**(-1)*MG2 - 20*M2*TG**(-1) - 12*M2*
     +    S**(-1) - 24*M2*U1**(-1) + 16*M2*(S+U1)**(-1) - 4*M2**2*
     +    TG**(-1)*S**(-1) - 8*M2**2*TG**(-1)*U1**(-1) - 4*TG**(-1)*
     +    S**(-1)*S4*MG2 - 4*TG**(-1)*S**(-1)*S4**2 + 16*TG**(-1)*S*
     +    U1**(-1)*MG2 + 4*TG**(-1)*S + 4*TG**(-1)*S**2*U1**(-1) + 16*
     +    TG**(-1)*U1**(-1)*MG2**2 + 8*TG**(-1)*S4 + 32*TG**(-1)*MG2 - 
     +    4*TG*S**(-1) + 16*TG*(S+U1)**(-1) - 4*S**(-1)*S4 + 8*S**(-1)*
     +    MG2 + 4*S*U1**(-1) + 24*U1**(-1)*MG2 + 16*(S+U1)**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(48)*N*CF * ( 4*M2*TG**(-1)*S*U1**(-1)*MG2
     +     - 2*M2*TG**(-1)*S*(S+U1)**(-1)*MG2 + 8*M2*TG**(-1)*S + 12*M2
     +    *TG**(-1)*S**2*U1**(-1) + 2*M2*TG**(-1)*S4 + 24*M2*TG*
     +    U1**(-1) - 12*M2*TG*(S+U1)**(-1) + 36*M2*S*U1**(-1) - 2*M2*S*
     +    (S+U1)**(-1) + 8*M2 + 16*M2**2*TG**(-1)*S*U1**(-1) + 4*M2**2*
     +    TG**(-1) + 24*M2**2*U1**(-1) - 12*M2**2*(S+U1)**(-1) + 8*
     +    M2**3*TG**(-1)*U1**(-1) - 4*M2**3*TG**(-1)*(S+U1)**(-1) + 4*
     +    TG**(-1)*S*S4 + 2*TG**(-1)*S*MG2 + 4*TG**(-1)*S**2*U1**(-1)*
     +    MG2 + 4*TG**(-1)*S**2 + 4*TG**(-1)*S**3*U1**(-1) + 2*TG**(-1)
     +    *S4**2 + 20*TG*S*U1**(-1) - 2*TG*S*(S+U1)**(-1) + 4*TG + 8*
     +    TG**2*U1**(-1) - 4*TG**2*(S+U1)**(-1) + 4*S*U1**(-1)*MG2 - 2*
     +    S*(S+U1)**(-1)*MG2 + 10*S + 16*S**2*U1**(-1) + 2*S4 )
     +
      M2GBH4 = M2GBH4 + ANG4(48)*CF**2 * (  - 4*M2*TG**(-1)*S*U1**(-1)*
     +    MG2 + 4*M2*TG**(-1)*S*(S+U1)**(-1)*MG2 - 8*M2*TG**(-1)*S - 12
     +    *M2*TG**(-1)*S**2*U1**(-1) - 24*M2*TG*U1**(-1) + 24*M2*TG*
     +    (S+U1)**(-1) - 36*M2*S*U1**(-1) + 4*M2*S*(S+U1)**(-1) - 16*
     +    M2**2*TG**(-1)*S*U1**(-1) - 24*M2**2*U1**(-1) + 24*M2**2*
     +    (S+U1)**(-1) - 8*M2**3*TG**(-1)*U1**(-1) + 8*M2**3*TG**(-1)*
     +    (S+U1)**(-1) - 4*TG**(-1)*S*S4 - 4*TG**(-1)*S**2*U1**(-1)*MG2
     +     - 4*TG**(-1)*S**2 - 4*TG**(-1)*S**3*U1**(-1) - 20*TG*S*
     +    U1**(-1) + 4*TG*S*(S+U1)**(-1) - 8*TG**2*U1**(-1) + 8*TG**2*
     +    (S+U1)**(-1) - 4*S*U1**(-1)*MG2 + 4*S*(S+U1)**(-1)*MG2 - 8*S
     +     - 16*S**2*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(54)*N*S4G*S4G2**(-1)*CF * (  - 8*M2*TG*S*
     +    U1**(-1) - 4*M2*TG + 8*M2*S*U1**(-1)*MG2 - 8*M2*S - 8*M2*S**2
     +    *U1**(-1) - 8*M2**2*S*U1**(-1) - 4*M2**2 + 8*TG*S*U1**(-1)*
     +    MG2 - 2*TG*S - 2*TG**2 + 8*S*MG2 + 8*S**2*U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(54)*N*CF * ( 8*M2*TG**(-1)*S*U1**(-1)*MG2
     +     - 2*M2*TG**(-1)*S - 8*M2*S*U1**(-1) - 6*M2 - 8*M2**2*
     +    TG**(-1)*S*U1**(-1) - 2*M2**2*TG**(-1) + 2*TG**(-1)*S*S4 - 2*
     +    TG**(-1)*S4**2 - 2*TG + 8*S*U1**(-1)*MG2 - 2*S4 )
     +
      M2GBH4 = M2GBH4 + ANG4(58)*N*S4G*S4G2**(-1)*CF * (  - 2*M2*
     +    TG**(-1)*S*MG2 + 12*M2*TG*S*U1**(-1) + 18*M2*TG + 24*M2*TG**2
     +    *U1**(-1) - 4*M2*S*U1**(-1)*MG2 + 24*M2**2*TG*U1**(-1) + 8*
     +    M2**2*S*U1**(-1) + 16*M2**2 + 4*M2**3*TG**(-1) + 8*M2**3*
     +    U1**(-1) - 4*TG*S*U1**(-1)*MG2 + 4*TG**2*S*U1**(-1) + 6*TG**2
     +     + 8*TG**3*U1**(-1) - 2*S*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(58)*N*CF * ( 2*M2*TG**(-1)*S*(S+U1)**(-1)*
     +    MG2 - 12*M2*TG*(S+U1)**(-1) + 2*M2*S*(S+U1)**(-1) + 2*M2 - 12
     +    *M2**2*(S+U1)**(-1) - 4*M2**3*TG**(-1)*(S+U1)**(-1) + 2*TG*S*
     +    (S+U1)**(-1) + 2*TG - 4*TG**2*(S+U1)**(-1) + 2*S*(S+U1)**(-1)
     +    *MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(58)*S4G*S4G2**(-1)*CF**2 * (  - 12*M2*TG*S
     +    *U1**(-1) - 12*M2*TG - 24*M2*TG**2*U1**(-1) + 4*M2*S*U1**(-1)
     +    *MG2 - 24*M2**2*TG*U1**(-1) - 8*M2**2*S*U1**(-1) - 8*M2**2 - 
     +    8*M2**3*U1**(-1) + 4*TG*S*U1**(-1)*MG2 - 4*TG**2*S*U1**(-1)
     +     - 4*TG**2 - 8*TG**3*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(58)*CF**2 * (  - 4*M2*TG**(-1)*S*
     +    (S+U1)**(-1)*MG2 + 4*M2*TG**(-1)*S + 4*M2*TG**(-1)*S4 + 24*M2
     +    *TG*(S+U1)**(-1) - 4*M2*S*(S+U1)**(-1) + 4*M2 + 4*M2**2*
     +    TG**(-1) + 24*M2**2*(S+U1)**(-1) + 8*M2**3*TG**(-1)*
     +    (S+U1)**(-1) - 8*TG**(-1)*S*MG2 - 4*TG*S*(S+U1)**(-1) + 8*
     +    TG**2*(S+U1)**(-1) - 4*S*(S+U1)**(-1)*MG2 - 4*S + 4*S4 )
     +
      M2GBH4 = M2GBH4 + ANG4(60)*CF**2 * (  - 16*M2*TG + 32*M2*MG2 + 16
     +    *M2**2*TG**(-1)*MG2 - 32*M2**2 - 16*M2**3*TG**(-1) + 16*TG*
     +    MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(62)*N*S4G*S4G2**(-1)*CF * ( 2*M2*TG**(-1)*
     +    S*MG2 + 4*M2*TG*S**(-1)*MG2 + 8*M2*TG*S*U1**(-1) - 16*M2*TG*
     +    U1**(-1)*MG2 + 18*M2*TG + 12*M2*TG**2*S**(-1) + 8*M2*TG**2*
     +    U1**(-1) - 8*M2*S*U1**(-1)*MG2 + 2*M2*S + 6*M2*MG2 + 8*M2**2*
     +    TG**(-1)*MG2 + 16*M2**2*TG*S**(-1) + 16*M2**2*TG*U1**(-1) + 8
     +    *M2**2*S*U1**(-1) - 8*M2**2*U1**(-1)*MG2 + 8*M2**2 - 4*M2**3*
     +    TG**(-1) + 8*M2**3*S**(-1) + 8*M2**3*U1**(-1) - 8*TG*S*
     +    U1**(-1)*MG2 + 2*TG*S - 2*TG*MG2 + 4*TG**2*S**(-1)*MG2 - 8*
     +    TG**2*U1**(-1)*MG2 + 6*TG**2 + 4*TG**3*S**(-1) + 2*S*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(62)*N*CF * (  - 2*M2*TG**(-1)*S*
     +    (S+U1)**(-1)*MG2 + 12*M2*TG*S**(-1) + 8*M2*TG*U1**(-1) - 12*
     +    M2*TG*(S+U1)**(-1) - 4*M2*S**(-1)*MG2 - 2*M2*S*(S+U1)**(-1)
     +     - 16*M2*U1**(-1)*MG2 - 8*M2**2*TG**(-1)*U1**(-1)*MG2 + 8*
     +    M2**2*S**(-1) + 16*M2**2*U1**(-1) - 12*M2**2*(S+U1)**(-1) + 8
     +    *M2**3*TG**(-1)*U1**(-1) - 4*M2**3*TG**(-1)*(S+U1)**(-1) - 4*
     +    TG*S**(-1)*MG2 - 2*TG*S*(S+U1)**(-1) - 8*TG*U1**(-1)*MG2 + 4*
     +    TG**2*S**(-1) - 4*TG**2*(S+U1)**(-1) - 2*S*(S+U1)**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(62)*S4G*S4G2**(-1)*CF**2 * (  - 4*M2*TG*
     +    S**(-1)*MG2 - 16*M2*TG*S*U1**(-1) + 32*M2*TG*U1**(-1)*MG2 - 
     +    20*M2*TG - 12*M2*TG**2*S**(-1) - 16*M2*TG**2*U1**(-1) + 16*M2
     +    *S*U1**(-1)*MG2 + 12*M2*MG2 - 16*M2**2*TG*S**(-1) - 32*M2**2*
     +    TG*U1**(-1) - 16*M2**2*S*U1**(-1) + 16*M2**2*U1**(-1)*MG2 - 
     +    16*M2**2 - 8*M2**3*S**(-1) - 16*M2**3*U1**(-1) + 16*TG*S*
     +    U1**(-1)*MG2 + 12*TG*MG2 - 4*TG**2*S**(-1)*MG2 + 16*TG**2*
     +    U1**(-1)*MG2 - 4*TG**2 - 4*TG**3*S**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(62)*CF**2 * ( 16*M2*TG**(-1)*S*U1**(-1)*
     +    MG2 + 4*M2*TG**(-1)*S*(S+U1)**(-1)*MG2 - 4*M2*TG**(-1)*MG2 - 
     +    12*M2*TG*S**(-1) - 16*M2*TG*U1**(-1) + 24*M2*TG*(S+U1)**(-1)
     +     + 4*M2*S**(-1)*MG2 - 16*M2*S*U1**(-1) + 4*M2*S*(S+U1)**(-1)
     +     + 32*M2*U1**(-1)*MG2 + 12*M2 - 16*M2**2*TG**(-1)*S*U1**(-1)
     +     + 16*M2**2*TG**(-1)*U1**(-1)*MG2 + 8*M2**2*TG**(-1) - 8*
     +    M2**2*S**(-1) - 32*M2**2*U1**(-1) + 24*M2**2*(S+U1)**(-1) - 
     +    16*M2**3*TG**(-1)*U1**(-1) + 8*M2**3*TG**(-1)*(S+U1)**(-1) + 
     +    4*TG*S**(-1)*MG2 + 4*TG*S*(S+U1)**(-1) + 16*TG*U1**(-1)*MG2
     +     + 4*TG - 4*TG**2*S**(-1) + 8*TG**2*(S+U1)**(-1) + 16*S*
     +    U1**(-1)*MG2 + 4*S*(S+U1)**(-1)*MG2 - 4*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(63)*N*S4G*S4G2**(-1)*CF * (  - 2 - 2*M2*
     +    TG**(-1)*U1**(-1)*MG2 - 2*M2*TG*S**(-1)*U1**(-1) - 2*M2*
     +    S**(-1)*U1**(-1)*MG2 - 2*M2*S**(-1) - 2*M2*U1**(-1) - 2*
     +    TG**(-1)*S*U1**(-1)*MG2 - 2*TG*S**(-1)*U1**(-1)*MG2 - 2*TG*
     +    S**(-1) - 4*TG*U1**(-1) - 2*TG**2*S**(-1)*U1**(-1) - 2*S*
     +    U1**(-1) - 4*U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(63)*N*CF * (  - 2*M2*TG**(-1)*U1**(-1) - 2
     +    *TG**(-1)*S*U1**(-1) - 2*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(63)*S4G*S4G2**(-1)*CF**2 * ( 4 + 4*M2*TG*
     +    S**(-1)*U1**(-1) + 4*M2*S**(-1)*U1**(-1)*MG2 + 4*M2*S**(-1)
     +     + 4*M2*U1**(-1) + 4*TG*S**(-1)*U1**(-1)*MG2 + 4*TG*S**(-1)
     +     + 8*TG*U1**(-1) + 4*TG**2*S**(-1)*U1**(-1) + 4*S*U1**(-1) + 
     +    4*U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(68)*N*S4G*S4G2**(-1)*CF * (  - 4 - 2*TG*
     +    S**(-1) - 8*TG*U1**(-1) - 4*S**(-1)*MG2 - 4*S*U1**(-1) - 8*
     +    U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(68)*N*CF * ( 2*M2*TG**(-1)*S**(-1) - 2*
     +    TG**(-1)*S**(-1)*S4 - 4*TG**(-1)*S**(-1)*MG2 - 4*TG**(-1)*S*
     +    U1**(-1) - 8*TG**(-1)*U1**(-1)*MG2 - 4*TG**(-1) - 4*S**(-1)
     +     - 8*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + ANG4(68)*S4G*S4G2**(-1)*CF**2 * ( 4 + 4*TG*
     +    S**(-1) + 8*TG*U1**(-1) + 8*S**(-1)*MG2 + 4*S*U1**(-1) + 8*
     +    U1**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + ANG4(68)*CF**2 * (  - 4*M2*TG**(-1)*S**(-1) + 4
     +    *TG**(-1)*S**(-1)*S4 + 8*TG**(-1)*S**(-1)*MG2 + 4*TG**(-1)*S*
     +    U1**(-1) + 8*TG**(-1)*U1**(-1)*MG2 + 4*TG**(-1) + 8*S**(-1)
     +     + 8*U1**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 8*M2*
     +    TG**(-1)*S*T1**(-1)*U1**(-1)*S4**(-1)*MG2 - 24*M2*TG**(-1)*S*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 4*M2*TG**(-1)*S*S4**(-1)
     +    *(S+T1)**(-2)*MG2 + 8*M2*TG**(-1)*S*S4**(-1)*(S+T1)**(-1) + 8
     +    *M2*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 4*M2*
     +    TG**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 8*M2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 - 8*M2*TG*S**(-1)*T1**(-1)*S4**(-1) + 32*M2*
     +    TG*S**(-1)*U1**(-1)*S4**(-1) - 16*M2*TG*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 32*M2*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*M2
     +    *TG*S4**(-1)*(S+U1)**(-2) + 24*M2*S**(-1)*S4**(-1) + 8*M2*S*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1) - 16*M2*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2*T1**(-1)*U1**(-1)*S4**(-1)*MG2 - 16*M2*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 24*M2*U1**(-1)*S4**(-1)
     +     - 8*M2*S4**(-1)*(S+U1)**(-2)*MG2 - 24*M2*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2**2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 16*
     +    M2**2*TG**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MG2 - 16*M2**2*
     +    TG**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 8*M2**2*
     +    TG**(-1)*S4**(-1)*(S+T1)**(-1) + 16*M2**2*S**(-1)*U1**(-1)*
     +    S4**(-1) - 8*M2**2*S**(-1)*S4**(-1)*(S+U1)**(-1) - 8*M2**2*
     +    T1**(-1)*U1**(-1)*S4**(-1) + 8*M2**2*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) - 16*M2**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 8*
     +    M2**3*TG**(-1)*S**(-1)*T1**(-1)*S4**(-1) - 8*M2**3*TG**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1) + 8*M2**3*TG**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) - 8*TG**(-1)*S*S4**(-1)*(S+T1)**(-1)*
     +    MG2 - 8*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 4*
     +    TG**(-1)*S**2*S4**(-1)*(S+T1)**(-2)*MG2 - 4*TG**(-1)*S4**(-1)
     +    *MG2 + 8*TG*S**(-1)*S4**(-1) - 8*TG*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 16*TG*U1**(-1)*S4**(-1) - 8*TG*S4**(-1)*
     +    (S+T1)**(-1) - 8*TG*S4**(-1)*(S+U1)**(-2)*MG2 - 24*TG*
     +    S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*S4G*(S4+MS2)*S4G2**(-1)*CF * ( 16*
     +    TG**2*S**(-1)*U1**(-1)*S4**(-1) - 8*TG**2*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 16*TG**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*
     +    TG**2*S4**(-1)*(S+U1)**(-2) + 8*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    MG2 - 8*S*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 8*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1)*MG2 + 8*S*U1**(-1)*S4**(-1) + 4*S*
     +    S4**(-1)*(S+T1)**(-2)*MG2 - 8*S*S4**(-1)*(S+T1)**(-1) - 8*
     +    U1**(-1)*S4**(-1)*MG2 - 4*S4**(-1)*(S+T1)**(-1)*MG2 - 8*
     +    S4**(-1)*(S+U1)**(-1)*MG2 + 8*S4**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 4*M2*TG**(-1)*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 48*M2*TG**(-1)*
     +    S*U1**(-1)*S4**(-2) - 24*M2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) - 16*M2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 4*M2*TG**(-1)*S*S4**(-1)*(S+T1)**(-2) - 4*M2*
     +    TG**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MG2 + 4*M2*TG**(-1)*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 16*M2*TG**(-1)*U1**(-2)*
     +    S4**(-1)*MG2 + 16*M2*TG**(-1)*U1**(-1)*S4**(-2)*MG2 - 8*M2*
     +    TG**(-1)*U1**(-1)*S4**(-1) - 8*M2*TG**(-1)*S4**(-2)*
     +    (S+U1)**(-1)*MG2 + 32*M2*TG**(-1)*S4**(-2) - 8*M2*TG**(-1)*
     +    S4**(-1)*(S+T1)**(-1) + 8*M2*TG**(-1)*S4**(-1)*(S+U1)**(-2)*
     +    MG2 - 16*M2*TG**(-1)*S4**(-1)*(S+U1)**(-1) + 96*M2*TG*S**(-1)
     +    *U1**(-1)*S4**(-2) - 48*M2*TG*S**(-1)*S4**(-2)*(S+U1)**(-1)
     +     - 16*M2*S**(-1)*U1**(-1)*S4**(-1) + 32*M2*S**(-1)*S4**(-2)
     +     - 32*M2*S**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2*T1**(-1)*
     +    U1**(-2)*S4**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*(S4+MS2)*CF * ( 144*M2*U1**(-1)*
     +    S4**(-2) - 16*M2*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 48*M2*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*M2*S4**(-2)*(S+U1)**(-1)
     +     + 8*M2*S4**(-1)*(S+U1)**(-2) - 8*M2**2*TG**(-1)*S**(-1)*
     +    U1**(-1)*S4**(-1) + 16*M2**2*TG**(-1)*S**(-1)*S4**(-2) - 8*
     +    M2**2*TG**(-1)*S**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2**2*
     +    TG**(-1)*T1**(-1)*U1**(-2)*S4**(-1)*MG2 - 16*M2**2*TG**(-1)*
     +    U1**(-2)*S4**(-1) + 64*M2**2*TG**(-1)*U1**(-1)*S4**(-2) - 16*
     +    M2**2*TG**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 16*M2**2*
     +    TG**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*M2**2*S**(-1)*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-1) + 96*M2**2*S**(-1)*U1**(-1)*
     +    S4**(-2) - 48*M2**2*S**(-1)*S4**(-2)*(S+U1)**(-1) + 16*M2**2*
     +    T1**(-1)*U1**(-2)*S4**(-1) - 8*M2**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) - 8*M2**3*TG**(-1)*S**(-1)*T1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 32*M2**3*TG**(-1)*S**(-1)*U1**(-1)*
     +    S4**(-2) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 16*M2**3*TG**(-1)
     +    *S**(-1)*S4**(-2)*(S+U1)**(-1) + 16*M2**3*TG**(-1)*T1**(-1)*
     +    U1**(-2)*S4**(-1) - 8*M2**3*TG**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 16*TG**(-1)*S*U1**(-1)*S4**(-2)*MG2
     +     - 8*TG**(-1)*S*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 16*
     +    TG**(-1)*S*S4**(-2) - 12*TG**(-1)*S*S4**(-1)*(S+T1)**(-1) - 8
     +    *TG**(-1)*S*S4**(-1)*(S+U1)**(-1) + 16*TG**(-1)*S**2*U1**(-1)
     +    *S4**(-2) - 8*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 
     +    8*TG**(-1)*S**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 4*TG**(-1)*
     +    S**2*S4**(-1)*(S+T1)**(-2) + 8*TG**(-1)*S4**(-2)*MG2 + 8*
     +    TG**(-1)*S4**(-1) - 8*TG*S**(-1)*U1**(-1)*S4**(-1) + 16*TG*
     +    S**(-1)*S4**(-2) - 16*TG*S**(-1)*S4**(-1)*(S+U1)**(-1) + 80*
     +    TG*U1**(-1)*S4**(-2) - 24*TG*U1**(-1)*S4**(-1)*(S+U1)**(-1)
     +     - 8*TG*S4**(-2)*(S+U1)**(-1) + 8*TG*S4**(-1)*(S+U1)**(-2) + 
     +    32*TG**2*S**(-1)*U1**(-1)*S4**(-2) - 16*TG**2*S**(-1)*
     +    S4**(-2)*(S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 4*S*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 64*S*U1**(-1)*S4**(-2)
     +     - 8*S*U1**(-1)*S4**(-1)*(S+T1)**(-1) - 28*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 4*S*S4**(-1)*(S+T1)**(-2) - 4*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2 + 4*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 16*U1**(-1)*S4**(-2)*MG2 - 28*U1**(-1)*
     +    S4**(-1) - 8*S4**(-2)*(S+U1)**(-1)*MG2 + 40*S4**(-2) - 8*
     +    S4**(-1)*(S+T1)**(-1) + 8*S4**(-1)*(S+U1)**(-2)*MG2 - 12*
     +    S4**(-1)*(S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*S4G*(S4+MS2)*S4G2**(-1)*CF**2 * (  - 
     +    32*M2*TG*S**(-1)*U1**(-1)*S4**(-1) + 32*M2*TG*S**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 32*M2*TG*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2*S*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 16*M2
     +    *U1**(-1)*S4**(-1) + 16*M2*S4**(-1)*(S+U1)**(-1) - 16*M2**2*
     +    S**(-1)*U1**(-1)*S4**(-1) + 16*M2**2*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2**2*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 8*TG
     +    *S*U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*TG*U1**(-1)*S4**(-1) + 
     +    8*TG*S4**(-1)*(S+U1)**(-1) - 16*TG**2*S**(-1)*U1**(-1)*
     +    S4**(-1) + 16*TG**2*S**(-1)*S4**(-1)*(S+U1)**(-1) + 16*TG**2*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) - 8*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 8*U1**(-1)*S4**(-1)*MG2 - 8*S4**(-1)*
     +    (S+U1)**(-1)*MG2 )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 8*M2*TG**(-1)*
     +    S**(-1)*T1**(-1) + 8*M2*TG**(-1)*S**(-1)*S4**(-1) + 32*M2*
     +    TG**(-1)*S*T1**(-1)*U1**(-2)*S4**(-1)*MG2 + 8*M2*TG**(-1)*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 48*M2*TG**(-1)*
     +    S*U1**(-1)*S4**(-2) + 8*M2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) + 16*M2*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 8*M2*TG**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MG2
     +     - 8*M2*TG**(-1)*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 + 8*M2*
     +    TG**(-1)*T1**(-1)*S4**(-1) - 16*M2*TG**(-1)*U1**(-1)*S4**(-2)
     +    *MG2 + 8*M2*TG**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 16*
     +    M2*TG**(-1)*S4**(-2)*(S+U1)**(-1)*MG2 - 32*M2*TG**(-1)*
     +    S4**(-2) + 8*M2*TG**(-1)*S4**(-1)*(S+T1)**(-1) - 32*M2*
     +    TG**(-1)*S4**(-1)*(S+U1)**(-2)*MG2 + 16*M2*TG**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 96*M2*TG*S**(-1)*U1**(-1)*S4**(-2) + 96*M2*TG*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1) + 64*M2*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 32*M2*S*T1**(-2)
     +    *U1**(-2)*S4**(-1)*MG2 + 16*M2*T1**(-1)*U1**(-1)*S4**(-1) - 
     +    144*M2*U1**(-1)*S4**(-2) + 64*M2*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) + 16*M2*S4**(-2)*(S+U1)**(-1) - 32*M2*S4**(-1)*
     +    (S+U1)**(-2) + 8*M2**2*TG**(-1)*S**(-1)*T1**(-1)*S4**(-1) + 
     +    16*M2**2*TG**(-1)*S**(-1)*S4**(-1)*(S+U1)**(-1) - 32*M2**2*
     +    TG**(-1)*S*T1**(-2)*U1**(-2)*S4**(-1)*MG2 - 32*M2**2*TG**(-1)
     +    *S*T1**(-1)*U1**(-2)*S4**(-1) + 16*M2**2*TG**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1) - 64*M2**2*TG**(-1)*U1**(-1)*S4**(-2) + 16*
     +    M2**2*TG**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1) + 16*M2**2*
     +    S**(-1)*T1**(-1)*S4**(-1)*(S+U1)**(-1) - 96*M2**2*S**(-1)*
     +    U1**(-1)*S4**(-2) + 96*M2**2*S**(-1)*S4**(-2)*(S+U1)**(-1) + 
     +    32*M2**2*S*T1**(-2)*U1**(-2)*S4**(-1) + 16*M2**2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1) + 16*M2**3*TG**(-1)*S**(-1)*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-1) - 32*M2**3*TG**(-1)*S**(-1)*
     +    U1**(-1)*S4**(-2) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 32*M2**3*TG**(-1)*
     +    S**(-1)*S4**(-2)*(S+U1)**(-1) + 32*M2**3*TG**(-1)*S*T1**(-2)*
     +    U1**(-2)*S4**(-1) + 16*M2**3*TG**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 8*TG**(-1)*S**(-1) - 16*TG**(-1)*S*
     +    U1**(-1)*S4**(-2)*MG2 + 8*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 + 8*TG**(-1)*S*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 - 16*TG**(-1)*S*S4**(-2) + 8*TG**(-1)*S*
     +    S4**(-1)*(S+T1)**(-1) + 8*TG**(-1)*S*S4**(-1)*(S+U1)**(-1) - 
     +    16*TG**(-1)*S**2*U1**(-1)*S4**(-2) + 8*TG**(-1)*S**2*U1**(-1)
     +    *S4**(-1)*(S+T1)**(-1) + 8*TG**(-1)*S**2*U1**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 16*TG**(-1)*T1**(-1)*S4**(-1)*MG2 - 24*
     +    TG**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 16*TG**(-1)*S4**(-1) - 8
     +    *TG*S**(-1)*T1**(-1)*S4**(-1) + 32*TG*S**(-1)*S4**(-1)*
     +    (S+U1)**(-1) - 80*TG*U1**(-1)*S4**(-2) + 32*TG*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) + 16*TG*S4**(-2)*(S+U1)**(-1) - 32*TG*
     +    S4**(-1)*(S+U1)**(-2) )
     +
      M2GBH4 = M2GBH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 32*TG**2*S**(-1)
     +    *U1**(-1)*S4**(-2) + 32*TG**2*S**(-1)*S4**(-2)*(S+U1)**(-1)
     +     + 8*S**(-1)*T1**(-1) + 24*S**(-1)*S4**(-1) + 8*S*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 64*S*U1**(-1)*S4**(-2)
     +     + 8*S*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 32*S*U1**(-1)*
     +    S4**(-1)*(S+U1)**(-1) - 8*T1**(-1)*U1**(-1)*S4**(-1)*MG2 - 8*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2 - 8*T1**(-1)*S4**(-1) - 16
     +    *U1**(-1)*S4**(-2)*MG2 + 8*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2
     +     + 16*U1**(-1)*S4**(-1) + 16*S4**(-2)*(S+U1)**(-1)*MG2 - 32*
     +    S4**(-2) + 8*S4**(-1)*(S+T1)**(-1) - 32*S4**(-1)*(S+U1)**(-2)
     +    *MG2 - 16*S4**(-1)*(S+U1)**(-1) )

      M2GBH = 2.D0*((NS-1.D0)*(M2GBH1 +M2GBH2 +M2GBH3) +M2GBH4) 

      DSBGBH = S4/(S4+MS2)/2.D0 *
     +     ALPHAS**3 * AVG * M2GBH /4.D0 /S**2 *CONV

      RETURN
      END


      REAL*8 FUNCTION DSBGBS(ALPHAS,S,T1,S4,MS,MG,EPS)
C***  DOUBLE-POLE PART OF CROSS SECTIONS FOR G +QB -> SQ + SQB +QB
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,TG,U1,MS,MG,MS2,MG2,M2,CF,EPS
      REAL*8 M2GBS2,M2GBS3,M2GBS
      REAL*8 NS,CONV,N,AVG

      NS = 6.D0
      CONV = 389379660.D0
      N = 3.D0
      CF = (N**2-1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N /(N**2 -1.D0)

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 -MS2
      TG = T1 -M2
      U1 = S4 -S -T1

      M2GBS2 = 0.D0
      M2GBS2 = M2GBS2 + N*CF**2 * (  - 4*M2*TG*S**(-1) - 8*M2**2*
     +    S**(-1) - 8*M2**2*U1**(-2)*MG2 - 8*M2**2*U1**(-1) + 8*M2**3*
     +    S**(-1)*U1**(-1) + 8*M2**3*U1**(-2) )
     +
      M2GBS2 = M2GBS2 + N**2*CF * (  - 4*M2*TG**(-1)*S - 4*M2 - 8*M2**2
     +    *TG**(-2)*MG2 + 8*M2**2*TG**(-1)*S**(-1)*MG2 - 8*M2**2*
     +    TG**(-1) + 8*M2**2*S**(-1)*U1**(-1)*MG2 - 8*M2**3*TG**(-1)*
     +    S**(-1) - 8*M2**3*S**(-1)*U1**(-1) )

      M2GBS3 = 0.D0
      M2GBS3 = M2GBS3 + N*CF**2 * (  - 4*M2*TG*S**(-1) - 8*M2**2*
     +    S**(-1) - 8*M2**2*U1**(-2)*MG2 - 8*M2**2*U1**(-1) + 8*M2**3*
     +    S**(-1)*U1**(-1) + 8*M2**3*U1**(-2) )
     +
      M2GBS3 = M2GBS3 + N**2*CF * (  - 4*M2*TG**(-1)*S - 4*M2 - 8*M2**2
     +    *TG**(-2)*MG2 + 8*M2**2*TG**(-1)*S**(-1)*MG2 - 8*M2**2*
     +    TG**(-1) + 8*M2**2*S**(-1)*U1**(-1)*MG2 - 8*M2**3*TG**(-1)*
     +    S**(-1) - 8*M2**3*S**(-1)*U1**(-1) )


      M2GBS = 2.D0*(NS-1.D0)*(M2GBS2 + M2GBS3)

      DSBGBS = ALPHAS**3 *AVG *M2GBS *S4/(S4 +MS2)/8.D0/S**2 *CONV
      RETURN
      END





      REAL*8 FUNCTION DSBQG3(ALPHAS,S,T1,S4,MS,MG,SCA)
C***  GIVES THE SCALE DEPENDENCE OF HARD
C***  SCA = Q**2/MS**2    
      IMPLICIT NONE
      REAL*8 ALPHAS,S,T1,TG,U1,S4,MS,MG,MS2,MG2,M2,SCA,N,CF,NS,CONV,AVG
      REAL*8 M2QG31, M2QG32, M2QG33, M2QG34, M2QG3

      N = 3.D0
      CF = (N**2 -1.D0)/2.D0/N
      NS = 6.D0
      AVG = (1.D0/2.D0)**2 /N/(N**2 -1.D0)
      CONV = 389379660.D0

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 -MS2
      TG = T1 -M2
      U1 = S4 -S -T1

      M2QG31 =0.D0
      M2QG31 = M2QG31 + N*CF**2 * ( 32*S**(-2)*T1 + 16*S**(-2)*T1**2*
     +    U1**(-1) + 32*S**(-2)*U1 - 16*S**(-1)*T1*U1**(-2)*MS2 + 16*
     +    S**(-1)*T1*U1**(-1) - 32*S**(-1)*U1**(-1)*MS2 - 32*S**(-1)*U1
     +    *(S+T1)**(-1) - 32*S**(-1)*(S+T1)**(-1)*MS2 - 64*S*T1**(-3)*
     +    MS2 - 64*S*T1**(-2)*U1**(-1)*MS2 + 32*S*T1**(-2) - 32*S*
     +    T1**(-1)*U1**(-1)*(S+U1)**(-1)*MS2 + 64*S**2*T1**(-4)*
     +    U1**(-1)*MS2**2 + 64*S**2*T1**(-3)*U1**(-2)*MS2**2 - 64*S**2*
     +    T1**(-3)*U1**(-1)*MS2 + 32*S**2*T1**(-2)*U1**(-2)*
     +    (S+U1)**(-1)*MS2**2 + 64*S**3*T1**(-4)*U1**(-2)*MS2**2 + 32*
     +    T1**(-2)*U1 + 32*T1**(-1) - 16*U1**(-2)*MS2 + 16*(S+U1)**(-1)
     +     )
     +
      M2QG31 = M2QG31 + N**2*CF * ( 32*S**(-2)*T1**(-2)*U1**3 + 32*
     +    S**(-2)*T1**(-1)*U1**2 + 16*S**(-2)*U1 - 64*S**(-1)*T1**(-3)*
     +    U1**2*MS2 - 64*S**(-1)*T1**(-2)*U1*MS2 + 64*S**(-1)*T1**(-2)*
     +    U1**2 + 32*S**(-1)*T1**(-1)*U1 - 32*S**(-1)*T1**(-1)*MS2 + 
     +    128*S*T1**(-4)*MS2**2 + 64*S*T1**(-3)*U1**(-1)*MS2**2 - 64*S*
     +    T1**(-3)*MS2 + 64*S**2*T1**(-4)*U1**(-1)*MS2**2 + 64*T1**(-4)
     +    *U1*MS2**2 - 128*T1**(-3)*U1*MS2 + 64*T1**(-3)*MS2**2 + 32*
     +    T1**(-2)*U1**(-1)*MS2**2 + 32*T1**(-2)*U1 - 64*T1**(-2)*MS2 )

      M2QG32 = 0.D0
      M2QG32 = M2QG32 + N*CF**2 * ( 8*S*T1*U1*(M2*(S+T1)+T1*U1)**(-2)
     +     - 8*S*T1*MS2*(M2*(S+T1)+T1*U1)**(-2) - 16*S*U1*MS2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*U1**2*(S+T1)**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*U1**3*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S**2*MS2*(M2*(S+T1)+T1*U1)**(-2)
     +     + 16*T1*U1**2*(M2*(S+T1)+T1*U1)**(-2) + 8*T1**2*U1*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*U1**3*(M2*(S+T1)+T1*U1)**(-2) )

      M2QG33 = 0.D0
      M2QG33 = M2QG33 + N*CF**2 * ( 8*S*T1*MG2*(M2*(S+T1)+T1*U1)**(-2)
     +     + 16*S*U1*MG2*(M2*(S+T1)+T1*U1)**(-2) + 16*S*U1**2*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-2) + 8*S**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) )

      M2QG34 = 0.D0
      M2QG34 = M2QG34 + CF**2 * (  - 32*S**(-1)*T1*U1*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S**(-1)*T1**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*U1**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S*U1**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1*U1**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1*(M2*(S+T1)+T1*U1)**(-1) + 32*
     +    U1*(S+T1)**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 32*U1**2*
     +    (S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 32*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) )


      M2QG3 = 2.D0*((NS-1.D0)*(M2QG31 +M2QG32 +M2QG33) +M2QG34)

      DSBQG3 = ALPHAS**3 * AVG *M2QG3 *LOG(1.D0/SCA)/8.D0/S**2  *CONV
      RETURN
      END


      REAL*8 FUNCTION DSBQGH(ALPHAS,S,T1,S4,MS,MG,EPS)
C***  CROSS SECTIONS FOR G + Q -> SQ + SQB +Q
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,TG,U1,MS,MG,MS2,MG2,M2,EPS,DEL
      REAL*8 NS,CONV,N,CF,AVG,S4G,SYMBT,SYMBU,SYMBX
      REAL*8 ANGDEF(1:11), ANA(1:3,1:9), ANB(1:3,1:9), ANC(1:3,1:9)
      REAL*8 AHP1P1, ABP1P1, ABP1P2, A4P1P2, A4P1P1
      REAL*8 A4M1P2, A4M1P1, A4P0P2, A4P1P0, ABP2P2
      REAL*8 ABP2P1, ABP2P0
      REAL*8 C4P1P2, C4P1P1, C4M1P1, C4P1P0, CBP1P1
      REAL*8 M2QGH1,M2QGH2,M2QGH3,M2QGH4,M2QGH
      REAL*8 ANG4(1:84), COLO1(1:9)

           
      CONV = 389379660.D0
      NS = 6.D0

      N = 3.D0
      CF = (N**2 -1.D0)/N/2.D0

      AVG = (1.D0/2.D0)**2 /N /(N**2 -1.D0)

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 - MS2
      U1 = S4 -S -T1
      TG = T1 -M2
      S4G = S4 -M2
      SYMBT = (M2*(S+T1)+T1*S4)/((M2*(S+T1)+T1*S4)**2 +(S+T1)*EPS*MG**4)
      SYMBU = (M2*(S+U1)+U1*S4)/((M2*(S+U1)+U1*S4)**2 +(S+U1)*EPS*MG**4)
      SYMBX = 1.D0/((U1+M2)**2 +EPS*MG**4)


      ANGDEF(1) = 2.D0*SQRT(S4 +MS2)
      ANGDEF(2) = (S +U1)/ANGDEF(1)
      ANGDEF(3) = (S +T1)/ANGDEF(1)
      ANGDEF(4) = S4/ANGDEF(1)
      ANGDEF(5) = (S4 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(6) = -(T1 +U1 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(7) = SQRT((T1 +U1)**2 -4.D0*MS2*S)/ANGDEF(1)
      ANGDEF(8) = 
     +  (T1*S4 -S*(U1+2.D0*MS2))/(S+T1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(9) = SQRT(1 -ANGDEF(8)**2)
      ANGDEF(10) =
     +  (U1*S4 -S*(T1+2.D0*MS2))/(S+U1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(11) = SQRT(1 -ANGDEF(10)**2)

      ANA(1,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)
      ANC(1,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(1,2) = -ANB(1,1)
      ANC(1,2) = -ANC(1,1)
      ANA(1,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(1,3) = -ANA(1,3)
      ANC(1,3) =  0.D0
      ANA(1,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(1,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(8)-2.D0*ANGDEF(3)*ANGDEF(4)
      ANC(1,4) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(9)
      ANA(1,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(1,5) = -ANB(1,3)
      ANC(1,5) = -ANC(1,3)
      ANA(1,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(1,6) = -ANB(1,4)
      ANC(1,6) = -ANC(1,4)
      ANA(1,7) = +ANA(1,1) -M2
      ANB(1,7) = +ANB(1,1)
      ANC(1,7) = +ANC(1,1)
      ANA(1,8) = +ANA(1,5) -M2
      ANB(1,8) = +ANB(1,5)
      ANC(1,8) = +ANC(1,5)
      ANA(1,9) = +ANA(1,6) -M2
      ANB(1,9) = +ANB(1,6)
      ANC(1,9) = +ANC(1,6)

      ANA(2,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(2,1) = -2.D0*ANGDEF(4)*ANGDEF(7)
      ANC(2,1) =  0.D0
      ANA(2,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(2,2) = -ANB(2,1)
      ANC(2,2) = -ANC(2,1)
      ANA(2,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,3) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(2,4) =
     +     2.D0*ANGDEF(4)*ANGDEF(7) -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,4) = -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(2,5) = -ANB(2,3)
      ANC(2,5) = -ANC(2,3)
      ANA(2,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(2,6) = -ANB(2,4)
      ANC(2,6) = -ANC(2,4)
      ANA(2,7) = +ANA(2,1) -M2
      ANB(2,7) = +ANB(2,1)
      ANC(2,7) = +ANC(2,1)
      ANA(2,8) = +ANA(2,5) -M2
      ANB(2,8) = +ANB(2,5)
      ANC(2,8) = +ANC(2,5)
      ANA(2,9) = +ANA(2,6) -M2
      ANB(2,9) = +ANB(2,6)
      ANC(2,9) = +ANC(2,6)


      ANA(3,1) = +2.D0*ANGDEF(4)*ANGDEF(6)
      ANB(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10)
      ANC(3,1) = -2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,2) = +2.D0*ANGDEF(5)*ANGDEF(6) +2.D0*MS2
      ANB(3,2) = -ANB(3,1)
      ANC(3,2) = -ANC(3,1)
      ANA(3,3) = -2.D0*ANGDEF(3)*ANGDEF(4)
      ANB(3,3) = 
     +     2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(10) -2.D0*ANGDEF(2)*ANGDEF(4)
      ANC(3,3) = +2.D0*ANGDEF(4)*ANGDEF(7)*ANGDEF(11)
      ANA(3,4) = -2.D0*ANGDEF(2)*ANGDEF(4)
      ANB(3,4) = -ANA(3,4)
      ANC(3,4) =  0.D0
      ANA(3,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(3,5) = -ANB(3,3)
      ANC(3,5) = -ANC(3,3)
      ANA(3,6) = -2.D0*ANGDEF(5)*ANGDEF(2)
      ANB(3,6) = -ANB(3,4)
      ANC(3,6) = -ANC(3,4)
      ANA(3,7) = +ANA(3,1) -M2
      ANB(3,7) = +ANB(3,1)
      ANC(3,7) = +ANC(3,1)
      ANA(3,8) = +ANA(3,5) -M2
      ANB(3,8) = +ANB(3,5)
      ANC(3,8) = +ANC(3,5)
      ANA(3,9) = +ANA(3,6) -M2
      ANB(3,9) = +ANB(3,6)
      ANC(3,9) = +ANC(3,6)


C$$$      ANG4(1) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(2) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(3) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(4) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(5) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(6) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(7) =A4P1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(8) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(9) =A4P2M2(ANA(2,1),ANB(2,1),ANA(2,6),ANB(2,6),ANC(2,6))
C$$$      ANG4(10) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$
C$$$      ANG4(11) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(12) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(13) =A4P2M2(ANA(2,2),ANB(2,2),ANA(2,6),ANB(2,6),ANC(2,6))
       ANG4(14) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(15) =A4M2P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
       ANG4(16) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(17) =A4P1P1(ANA(3,6),ANB(3,6),ANA(3,2),ANB(3,2),ANC(3,2))
       ANG4(18) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(19) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
       ANG4(20) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,5),ANB(3,5),ANC(3,5))

C$$$      ANG4(21) =A4M1P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(22) =A4M1P2(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(23) =A4M2P1(ANA(1,5),ANB(1,5),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(24) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
       ANG4(25) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(26) =A4P1P2(ANA(1,5),ANB(1,5),ANA(1,7),ANB(1,7),ANC(1,7))
       ANG4(27) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
C$$$      ANG4(28) =A4P2P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
       ANG4(29) =A4P1P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(30) =A4P1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))

       ANG4(31) =A4P1P1(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(32) =A4M1P2(ANA(3,9),ANB(3,9),ANA(3,7),ANB(3,7),ANC(3,7))
C$$$      ANG4(33) =A4M1P2(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
       ANG4(34) =A4M1P1(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
       ANG4(35) =A4P1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
C$$$      ANG4(36) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(37) =A4P1P0(ANA(2,1),ANB(2,1),ANA(1,1),ANB(1,1),ANC(1,1))
       ANG4(38) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,2),ANB(1,2),ANC(1,2))
       ANG4(39) =A4P1P0(ANA(2,2),ANB(2,2),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(40) =A4P0P2(ANA(1,1),ANB(1,1),ANA(2,5),ANB(2,5),ANC(2,5))

       ANG4(41) =A4P1P0(ANA(1,5),ANB(1,5),ANA(1,1),ANB(1,1),ANC(1,1))

C$$$      ANG4(43) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(44) =A4P1P0(ANA(3,6),ANB(3,6),ANA(1,1),ANB(1,1),ANC(1,1))
 
       ANG4(47) =A4P1P0(ANA(2,7),ANB(2,7),ANA(1,1),ANB(1,1),ANC(1,1))
       ANG4(48) =AHP1P1(ANA(1,3),ANB(1,3),ANA(1,4),ANB(1,4),ANC(1,4))
C$$$      ANG4(49) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(50) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,1),ANB(3,1),ANC(3,1))

C$$$      ANG4(51) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
       ANG4(52) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
C$$$      ANG4(53) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
       ANG4(54) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,2),ANB(1,2),ANC(1,2))
       ANG4(55) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
       ANG4(56) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
       ANG4(57) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
       ANG4(58) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(59) =ABP2P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(60) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$
C$$$      ANG4(61) =ABP2P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(62) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
C$$$      ANG4(63) =ABP1M1(ANA(1,3),ANB(1,3),ANA(1,6),ANB(1,6),ANC(1,6))
       ANG4(64) =ABP2P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
       ANG4(65) =ABP1P2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
       ANG4(66) =ABP2P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
       ANG4(67) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(68) =ABP1M1(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(69) =ABP1M2(ANA(3,4),ANB(3,4),ANA(3,5),ANB(3,5),ANC(3,5))
C$$$      ANG4(70) =ABP2P0(ANA(1,3),ANB(1,3),ANA(1,1),ANB(1,1),ANC(1,1))

       ANG4(71) =ABP2P0(ANA(3,4),ANB(3,4),ANA(1,1),ANB(1,1),ANC(1,1))
C$$$      ANG4(72) =A4P2P2(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(73) =A4P1P2(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
C$$$      ANG4(74) =A4P1P2(ANA(3,9),ANB(3,9),ANA(3,1),ANB(3,1),ANC(3,1))
C$$$      ANG4(75) =A4P1P1(ANA(2,1),ANB(2,1),ANA(2,9),ANB(2,9),ANC(2,9))
       ANG4(76) =A4P1P1(ANA(3,9),ANB(3,9),ANA(3,2),ANB(3,2),ANC(3,2))
C$$$      ANG4(77) =A4M1P2(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
C$$$      ANG4(78) =A4M1P1(ANA(1,5),ANB(1,5),ANA(1,9),ANB(1,9),ANC(1,9))
       ANG4(79) =A4P0P2(ANA(1,1),ANB(1,1),ANA(1,9),ANB(1,9),ANC(1,9))
       ANG4(80) =A4P1P0(ANA(3,9),ANB(3,9),ANA(1,1),ANB(1,1),ANC(1,1))

       ANG4(81) =ABP1P2(ANA(1,3),ANB(1,3),ANA(1,9),ANB(1,9),ANC(1,9)) 
       ANG4(82) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,9),ANB(1,9),ANC(1,9)) 
       ANG4(83) =ABP1P1(ANA(1,3),ANB(1,3),ANA(1,7),ANB(1,7),ANC(1,7))
       ANG4(84) =ABP1P1(ANA(3,4),ANB(3,4),ANA(3,7),ANB(3,7),ANC(3,7))

      DEL = EPS * MG**4
      IF (MG.GT.MS) THEN
         ANG4(25) =C4P1P2(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
         ANG4(27) =C4P1P1(ANA(2,7),ANB(2,7),ANA(2,5),ANB(2,5),ANC(2,5))
         ANG4(29) =C4P1P2(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
         ANG4(31) =C4P1P1(ANA(2,7),ANB(2,7),ANA(2,9),ANB(2,9),ANC(2,9))
         ANG4(34) =C4M1P1(ANA(3,6),ANB(3,6),ANA(3,7),ANB(3,7),ANC(3,7))
         ANG4(47) =C4P1P0(ANA(2,7),ANB(2,7),ANA(1,1),ANB(1,1),ANC(1,1))
      ANG4(83) =CBP1P1(ANA(1,3),ANB(1,3),ANA(1,7),ANB(1,7),ANC(1,7),DEL)
      ANG4(84) =CBP1P1(ANA(3,4),ANB(3,4),ANA(3,7),ANB(3,7),ANC(3,7),DEL)
      ENDIF


      COLO1(9) = LOG(S4**2/MS2/(S4+MS2))

      M2QGH1 = 0D0
      M2QGH1 = M2QGH1 + N*(S4+MS2)*CF**2 * (  - 32*S**(-2)*T1*S4**(-1)
     +     - 32*S**(-2)*U1*S4**(-1) - 64*S**(-1)*T1**(-2)*U1**2*
     +    S4**(-1) + 64*S**(-1)*T1**(-1)*U1*S4**(-2)*MS2 - 96*S**(-1)*
     +    T1**(-1)*U1*S4**(-1) + 320*S**(-1)*T1**(-1)*U1**2*S4**(-2) - 
     +    64*S**(-1)*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1) + 128*S**(-1)
     +    *T1**(-1)*U1**3*S4**(-2)*(S+T1)**(-1) + 32*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2 + 32*S**(-1)*U1**(-1)*S4**(-1)*MS2 - 64*S**(-1)*
     +    U1*S4**(-2)*(S+T1)**(-1)*MS2 + 128*S**(-1)*U1*S4**(-2) - 64*
     +    S**(-1)*U1**2*S4**(-2)*(S+T1)**(-1) + 128*S*T1**(-3)*U1**(-1)
     +    *MS2 - 192*S*T1**(-3)*S4**(-1)*MS2 - 32*S*T1**(-2)*U1**(-2)*
     +    S4**(-1)*MS2**2 - 192*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2 - 64*S
     +    *T1**(-1)*U1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2 + 64*S*T1**(-1)*
     +    S4**(-2) - 32*S*T1**(-1)*S4**(-1)*(S+T1)**(-1) - 192*S**2*
     +    T1**(-4)*U1**(-2)*MS2**2 + 256*S**2*T1**(-4)*U1**(-1)*
     +    S4**(-1)*MS2**2 + 256*S**2*T1**(-3)*U1**(-2)*S4**(-1)*MS2**2
     +     - 192*S**2*T1**(-3)*U1**(-1)*S4**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + N*(S4+MS2)*CF**2 * ( 64*S**2*T1**(-2)*U1**(-2)*
     +    S4**(-1)*(S+U1)**(-1)*MS2**2 + 256*S**3*T1**(-4)*U1**(-2)*
     +    S4**(-1)*MS2**2 + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 + 64*
     +    T1**(-2)*U1**2*S4**(-1)*(S+T1)**(-1) - 32*T1**(-2)*S4**(-1)*
     +    (S+U1)**(-1)*MS2**2 + 32*T1**(-1)*U1**(-1)*S4**(-1)*MS2 + 256
     +    *T1**(-1)*U1*S4**(-2) - 96*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)
     +     + 64*T1**(-1)*S4**(-2)*MS2 - 96*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 - 32*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2 - 32
     +    *T1**(-1)*S4**(-1) + 64*S4**(-2) - 32*S4**(-1)*(S+T1)**(-1)
     +     + 16*S4**(-1)*(S+U1)**(-1) )
     +
      M2QGH1 = M2QGH1 + N**2*(S4+MS2)*CF * (  - 32*S**(-2)*T1**(-1)*
     +    U1**2*S4**(-1) + 32*S**(-2)*T1**(-1)*U1**3*S4**(-2) + 32*
     +    S**(-2)*T1*U1*S4**(-2) - 16*S**(-2)*U1*S4**(-1) + 64*S**(-2)*
     +    U1**2*S4**(-2) + 128*S**(-1)*T1**(-3)*U1*MS2 - 192*S**(-1)*
     +    T1**(-3)*U1**2*S4**(-1)*MS2 - 192*S**(-1)*T1**(-2)*U1*
     +    S4**(-1)*MS2 + 32*S**(-1)*T1**(-2)*U1**2*S4**(-1) - 16*
     +    S**(-1)*T1**(-1)*U1*S4**(-2)*MS2 + 16*S**(-1)*T1**(-1)*U1*
     +    S4**(-1) - 80*S**(-1)*T1**(-1)*U1**2*S4**(-2) + 32*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1) - 64*S**(-1)*T1**(-1)*
     +    U1**3*S4**(-2)*(S+T1)**(-1) - 48*S**(-1)*T1**(-1)*S4**(-1)*
     +    MS2 + 16*S**(-1)*T1*S4**(-2) + 32*S**(-1)*U1*S4**(-2)*
     +    (S+T1)**(-1)*MS2 + 32*S**(-1)*U1*S4**(-2) + 32*S**(-1)*U1**2*
     +    S4**(-2)*(S+T1)**(-1) + 16*S**(-1)*S4**(-2)*MS2 - 16*S**(-1)*
     +    S4**(-1) + 192*S*T1**(-4)*S4**(-1)*MS2**2 + 64*S*T1**(-3)*
     +    U1**(-1)*S4**(-1)*MS2**2 - 64*S*T1**(-3)*S4**(-1)*MS2 - 16*S*
     +    T1**(-1)*S4**(-2) )
     +
      M2QGH1 = M2QGH1 + N**2*(S4+MS2)*CF * ( 8*S*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) + 64*S**2*T1**(-4)*U1**(-1)*S4**(-1)*MS2**2 + 
     +    128*T1**(-4)*U1*S4**(-1)*MS2**2 - 64*T1**(-4)*MS2**2 - 256*
     +    T1**(-3)*U1*S4**(-1)*MS2 + 128*T1**(-3)*S4**(-1)*MS2**2 + 32*
     +    T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 - 32*T1**(-2)*U1**2*
     +    S4**(-1)*(S+T1)**(-1) - 64*T1**(-2)*S4**(-1)*MS2 - 64*
     +    T1**(-1)*U1*S4**(-2) + 32*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)
     +     - 16*T1**(-1)*S4**(-2)*MS2 + 32*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 + 8*T1**(-1)*S4**(-1) + 8*S4**(-1)*
     +    (S+T1)**(-1) )
     +
      M2QGH1 = M2QGH1 + ANG4(14)*N*CF**2 * ( 8*S**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(18)*N*CF**2 * (  - 32*S**(-1)*T1**(-1)*
     +    U1**3 - 64*S**(-1)*T1*U1 - 16*S**(-1)*T1*MS2 - 16*S**(-1)*
     +    T1**2 - 16*S**(-1)*U1*MS2 - 80*S**(-1)*U1**2 + 64*T1**(-1)*U1
     +    *MS2 + 32*T1**(-1)*U1**3*(S+T1)**(-1) - 16*T1 + 16*U1*
     +    (S+T1)**(-1)*MS2 - 32*U1 + 16*U1**2*(S+T1)**(-1) + 32*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(18)*N**2*CF * ( 8*S**(-1)*T1**(-1)*U1**3
     +     + 16*S**(-1)*T1*U1 + 4*S**(-1)*T1*MS2 + 4*S**(-1)*T1**2 + 4*
     +    S**(-1)*U1*MS2 + 20*S**(-1)*U1**2 + 16*S*T1**(-2)*U1*MS2 - 8*
     +    S*T1**(-1)*U1 + 8*S*T1**(-1)*MS2 - 4*S - 16*T1**(-1)*U1*MS2
     +     - 16*T1**(-1)*U1**2 - 16*T1**(-1)*U1**3*(S+T1)**(-1) - 8*U1*
     +    (S+T1)**(-1)*MS2 - 8*U1 - 8*U1**2*(S+T1)**(-1) - 12*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(38)*N*CF**2 * ( 8*S**(-1)*U1*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(38)*N**2*CF * (  - 8*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(39)*N*CF**2 * (  - 16*S**(-1)*T1**(-1)*U1*
     +    MS2 - 16*S**(-1)*T1**(-1)*U1**2 - 24*S**(-1)*U1 - 24*S**(-1)*
     +    MS2 + 32*T1**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(39)*N**2*CF * ( 8*S**(-1)*T1**(-1)*U1*MS2
     +     + 8*S**(-1)*T1**(-1)*U1**2 + 8*S**(-1)*U1 + 8*S**(-1)*MS2 - 
     +    16*T1**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(41)*N*CF**2 * (  - 8 - 16*S**(-1)*T1 - 32*
     +    S**(-1)*U1 - 32*S**(-1)*MS2 + 8*S*T1**(-1) + 16*T1**(-1)*U1
     +     + 16*T1**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(41)*N**2*CF * ( 4*S**(-1)*T1 + 8*S**(-1)*
     +    U1 + 8*S**(-1)*MS2 - 4*S*T1**(-1) - 8*T1**(-1)*U1 - 8*
     +    T1**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(48)*N*CF**2 * (  - 64*S*T1**(-1)*U1 - 16*S
     +    *T1**(-1)*MS2 - 32*S - 16*S**2*T1**(-1) - 16*T1**(-1)*U1*MS2
     +     - 80*T1**(-1)*U1**2 - 32*T1**(-1)*U1**3*(S+T1)**(-1) - 16*T1
     +     + 16*U1*(S+T1)**(-1)*MS2 - 48*U1 + 16*U1**2*(S+T1)**(-1) )
     +
      M2QGH1 = M2QGH1 + ANG4(48)*N**2*CF * (  - 8*S**(-1)*T1**(-1)*
     +    U1**3 - 12*S**(-1)*T1*U1 - 4*S**(-1)*T1**2 - 16*S**(-1)*U1**2
     +     + 16*S*T1**(-1)*U1 + 4*S*T1**(-1)*MS2 + 4*S + 4*S**2*
     +    T1**(-1) + 4*T1**(-1)*U1*MS2 + 20*T1**(-1)*U1**2 + 16*
     +    T1**(-1)*U1**3*(S+T1)**(-1) - 4*T1 - 8*U1*(S+T1)**(-1)*MS2 - 
     +    4*U1 - 8*U1**2*(S+T1)**(-1) - 4*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(52)*N*CF**2 * ( 8*S*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(54)*N*CF**2 * ( 16*S**(-1)*T1*U1 + 8*
     +    S**(-1)*T1**2 + 16*S**(-1)*U1**2 - 16*T1**(-1)*U1*MS2 + 16*
     +    T1**(-1)*U1**2 + 32*T1**(-1)*U1**3*(S+T1)**(-1) + 8*T1 + 16*
     +    U1*(S+T1)**(-1)*MS2 + 24*U1 + 16*U1**2*(S+T1)**(-1) - 16*MS2
     +     )
     +
      M2QGH1 = M2QGH1 + ANG4(54)*N**2*CF * ( 8*S**(-1)*T1**(-1)*U1**3
     +     + 4*S**(-1)*T1*U1 + 8*S**(-1)*U1**2 + 4*T1**(-1)*U1*MS2 - 4*
     +    T1**(-1)*U1**2 - 16*T1**(-1)*U1**3*(S+T1)**(-1) - 8*U1*
     +    (S+T1)**(-1)*MS2 - 4*U1 - 8*U1**2*(S+T1)**(-1) )
     +
      M2QGH1 = M2QGH1 + ANG4(55)*N**2*CF * (  - 16*S**2*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(56)*N**2*CF * (  - 16*S*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(57)*N**2*CF * (  - 32*S*T1**(-1)*U1*MS2 - 
     +    16*S**2*T1**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(58)*N*CF**2 * ( 64*S*T1**(-1)*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(58)*N**2*CF * ( 16*S**(-1)*T1**(-1)*U1**3
     +     + 16*S**(-1)*T1*U1 + 4*S**(-1)*T1**2 + 24*S**(-1)*U1**2 - 16
     +    *S*T1**(-2)*U1*MS2 + 8*S*T1**(-1)*U1 - 48*S*T1**(-1)*MS2 + 4*
     +    S - 16*T1**(-1)*U1*MS2 + 24*T1**(-1)*U1**2 + 8*T1 + 24*U1 - 8
     +    *MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(64)*N*CF**2 * (  - 16*U1**2*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(65)*N*CF**2 * (  - 16*U1*MS2 - 16*MS2**2 )
     +
      M2QGH1 = M2QGH1 + ANG4(66)*N*CF**2 * (  - 32*S*T1**(-1)*U1*MS2 - 
     +    32*T1**(-1)*U1**2*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(66)*N**2*CF * ( 16*T1**(-1)*U1**2*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(67)*N*CF**2 * ( 32*S**(-1)*T1**(-1)*U1**3
     +     - 16*S**(-1)*U1*MS2 + 16*S**(-1)*U1**2 + 16*S*T1**(-1)*U1 - 
     +    16*S*T1**(-1)*MS2 - 96*T1**(-1)*U1*MS2 + 16*T1**(-1)*U1**2 - 
     +    32*T1**(-1)*U1**3*(S+T1)**(-1) - 32*T1**(-1)*MS2**2 + 16*U1*
     +    (S+T1)**(-1)*MS2 + 8*U1 + 16*U1**2*(S+T1)**(-1) - 16*MS2 )
     +
      M2QGH1 = M2QGH1 + ANG4(67)*N**2*CF * (  - 8*S**(-1)*T1**(-1)*
     +    U1**3 + 4*S**(-1)*U1*MS2 - 4*S**(-1)*U1**2 - 16*S*T1**(-2)*U1
     +    *MS2 + 32*T1**(-1)*U1*MS2 + 8*T1**(-1)*U1**2 + 16*T1**(-1)*
     +    U1**3*(S+T1)**(-1) + 16*T1**(-1)*MS2**2 - 8*U1*(S+T1)**(-1)*
     +    MS2 - 8*U1**2*(S+T1)**(-1) )
     +
      M2QGH1 = M2QGH1 + ANG4(71)*N*CF**2 * (  - 32*S*T1**(-2)*U1*MS2 - 
     +    16*S**2*T1**(-2)*MS2 - 16*T1**(-2)*U1**2*MS2 )
     +
      M2QGH1 = M2QGH1 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 32*S**(-2)*T1*
     +    S4**(-1) + 16*S**(-2)*T1**2*U1**(-1)*S4**(-1) + 32*S**(-2)*U1
     +    *S4**(-1) + 64*S**(-1)*T1**(-2)*U1**2*S4**(-1) - 64*S**(-1)*
     +    T1**(-1)*U1*S4**(-2)*MS2 + 96*S**(-1)*T1**(-1)*U1*S4**(-1) - 
     +    320*S**(-1)*T1**(-1)*U1**2*S4**(-2) + 64*S**(-1)*T1**(-1)*
     +    U1**2*S4**(-1)*(S+T1)**(-1) - 128*S**(-1)*T1**(-1)*U1**3*
     +    S4**(-2)*(S+T1)**(-1) - 32*S**(-1)*T1**(-1)*S4**(-1)*MS2 - 16
     +    *S**(-1)*T1*U1**(-2)*S4**(-1)*MS2 + 16*S**(-1)*T1*U1**(-1)*
     +    S4**(-1) - 64*S**(-1)*T1*S4**(-2) - 32*S**(-1)*U1**(-1)*
     +    S4**(-1)*MS2 + 64*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MS2 - 192*
     +    S**(-1)*U1*S4**(-2) + 64*S**(-1)*U1**2*S4**(-2)*(S+T1)**(-1)
     +     + 64*S**(-1)*S4**(-1) - 128*S*T1**(-3)*U1**(-1)*MS2 + 64*S*
     +    T1**(-3)*S4**(-1)*MS2 + 32*S*T1**(-2)*U1**(-2)*S4**(-1)*
     +    MS2**2 + 64*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2 + 32*S*T1**(-2)*
     +    S4**(-1) - 64*S*T1**(-1)*S4**(-2) + 32*S*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) )
     +
      M2QGH1 = M2QGH1 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 192*S**2*T1**(-4)
     +    *U1**(-2)*MS2**2 - 128*S**2*T1**(-4)*U1**(-1)*S4**(-1)*MS2**2
     +     - 128*S**2*T1**(-3)*U1**(-2)*S4**(-1)*MS2**2 + 64*S**2*
     +    T1**(-3)*U1**(-1)*S4**(-1)*MS2 - 128*S**3*T1**(-4)*U1**(-2)*
     +    S4**(-1)*MS2**2 - 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 + 32*
     +    T1**(-2)*U1*S4**(-1) - 64*T1**(-2)*U1**2*S4**(-1)*
     +    (S+T1)**(-1) + 32*T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2**2 - 32*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2 - 256*T1**(-1)*U1*S4**(-2) + 
     +    96*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) - 64*T1**(-1)*S4**(-2)*
     +    MS2 + 96*T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 + 32*T1**(-1)*
     +    S4**(-1)*(S+U1)**(-1)*MS2 + 64*T1**(-1)*S4**(-1) - 16*
     +    U1**(-2)*S4**(-1)*MS2 - 128*S4**(-2) + 32*S4**(-1)*
     +    (S+T1)**(-1) + 16*S4**(-1)*(S+U1)**(-1) )
     +
      M2QGH1 = M2QGH1 + COLO1(9)*N**2*(S4+MS2)*CF * ( 32*S**(-2)*
     +    T1**(-2)*U1**3*S4**(-1) + 64*S**(-2)*T1**(-1)*U1**2*S4**(-1)
     +     - 32*S**(-2)*T1**(-1)*U1**3*S4**(-2) - 48*S**(-2)*T1*U1*
     +    S4**(-2) + 16*S**(-2)*T1*S4**(-1) - 16*S**(-2)*T1**2*S4**(-2)
     +     + 48*S**(-2)*U1*S4**(-1) - 64*S**(-2)*U1**2*S4**(-2) - 128*
     +    S**(-1)*T1**(-3)*U1*MS2 + 64*S**(-1)*T1**(-3)*U1**2*S4**(-1)*
     +    MS2 + 64*S**(-1)*T1**(-2)*U1*S4**(-1)*MS2 + 32*S**(-1)*
     +    T1**(-2)*U1**2*S4**(-1) + 16*S**(-1)*T1**(-1)*U1*S4**(-2)*MS2
     +     + 16*S**(-1)*T1**(-1)*U1*S4**(-1) + 80*S**(-1)*T1**(-1)*
     +    U1**2*S4**(-2) - 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1) + 64*S**(-1)*T1**(-1)*U1**3*S4**(-2)*
     +    (S+T1)**(-1) - 16*S**(-1)*T1**(-1)*S4**(-1)*MS2 - 16*S**(-1)*
     +    T1*S4**(-2) - 32*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MS2 - 16*
     +    S**(-1)*U1*S4**(-2) - 32*S**(-1)*U1**2*S4**(-2)*(S+T1)**(-1)
     +     - 16*S**(-1)*S4**(-2)*MS2 + 64*S*T1**(-4)*S4**(-1)*MS2**2 + 
     +    64*S*T1**(-3)*U1**(-1)*S4**(-1)*MS2**2 )
     +
      M2QGH1 = M2QGH1 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 64*S*T1**(-3)*
     +    S4**(-1)*MS2 + 16*S*T1**(-1)*S4**(-2) - 8*S*T1**(-1)*S4**(-1)
     +    *(S+T1)**(-1) + 64*S**2*T1**(-4)*U1**(-1)*S4**(-1)*MS2**2 + 
     +    64*T1**(-4)*MS2**2 + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 + 
     +    32*T1**(-2)*U1*S4**(-1) + 32*T1**(-2)*U1**2*S4**(-1)*
     +    (S+T1)**(-1) - 64*T1**(-2)*S4**(-1)*MS2 + 64*T1**(-1)*U1*
     +    S4**(-2) - 32*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) + 16*T1**(-1)
     +    *S4**(-2)*MS2 - 32*T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 - 8*
     +    T1**(-1)*S4**(-1) + 16*S4**(-2) - 8*S4**(-1)*(S+T1)**(-1) )

      M2QGH2 = 0D0
      M2QGH2 = M2QGH2 + N*SYMBT*(S4+MS2)*CF**2 * ( 32*S**(-1)*T1**(-1)*
     +    U1*S4**(-1)*MS2**2 - 32*S**(-1)*T1**(-1)*U1*S4**(-1)*MG2**2
     +     - 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 - 32*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2*MG2**2 + 64*S**(-1)*T1**(-1)*S4**(-1)*
     +    MS2**2*MG2 - 32*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 - 32*S**(-1)
     +    *T1*U1*S4**(-1) - 32*S**(-1)*T1*S4**(-1)*MG2 - 16*S**(-1)*
     +    T1**2*S4**(-1) - 64*S**(-1)*U1*S4**(-1)*MG2 - 16*S**(-1)*
     +    U1**2*S4**(-1) - 32*S**(-1)*S4**(-1)*MS2*MG2 + 48*S**(-1)*
     +    S4**(-1)*MS2**2 - 16*S**(-1)*S4**(-1)*MG2**2 + 32*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MS2*MG2 - 32*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2 + 32*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)
     +    *MG2 + 16*T1**(-1)*S4**(-1)*MS2*MG2 - 16*T1**(-1)*S4**(-1)*
     +    MG2**2 - 16*T1*S4**(-1) + 16*U1*S4**(-1)*(S+T1)**(-1)*MS2 + 
     +    16*U1**2*S4**(-1)*(S+T1)**(-1) + 16*S4**(-1)*MS2 - 32*
     +    S4**(-1)*MG2 )
     +
      M2QGH2 = M2QGH2 + N*(S4+MS2)*CF**2 * (  - 32*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*T1**(-1)
     +    *U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 32*
     +    S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2*(M2*(S+T1)+T1*U1)**(-1)
     +     - 64*S**(-1)*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*T1**(-1)*S4**(-1)*MS2**3
     +    *(M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1) - 16*S**(-1)*U1**2*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MS2 - 32*S**(-1)*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 
     +    16*S**(-1)*S4**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*
     +    S**(-1)*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*S**(-1)*
     +    S4**(-1) + 16*S*T1**(-1)*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*T1**(-1)*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH2 = M2QGH2 + N*(S4+MS2)*CF**2 * (  - 8*S*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) - 16*S*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S*T1**(-1)*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*S*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*T1**(-1)*S4**(-1)*MS2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S*T1**(-1)*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 32*S*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*S**2*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S**2*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 32*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) - 16*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) )
     +
      M2QGH2 = M2QGH2 + N*(S4+MS2)*CF**2 * (  - 16*T1**(-1)*U1*S4**(-1)
     +    *MS2*(M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2 + 16*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*T1**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 24*T1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*T1*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*T1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*U1*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + N*(S4+MS2)*CF**2 * ( 16*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S4**(-1)*
     +    (S+T1)**(-1) - 16*S4**(-1)*MS2*MG2**2*(M2*(S+T1)+T1*U1)**(-2)
     +     - 24*S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 32*S4**(-1)*
     +    MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-2) - 16*S4**(-1)*MS2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + N**2*SYMBT*(S4+MS2)*CF * (  - 8*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2**2 + 8*S**(-1)*T1**(-1)*U1*S4**(-1)*
     +    MG2**2 + 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 + 8*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2*MG2**2 - 16*S**(-1)*T1**(-1)*S4**(-1)*
     +    MS2**2*MG2 + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 + 8*S**(-1)*
     +    T1*U1*S4**(-1) + 8*S**(-1)*T1*S4**(-1)*MG2 + 4*S**(-1)*T1**2*
     +    S4**(-1) + 16*S**(-1)*U1*S4**(-1)*MG2 + 4*S**(-1)*U1**2*
     +    S4**(-1) + 8*S**(-1)*S4**(-1)*MS2*MG2 - 12*S**(-1)*S4**(-1)*
     +    MS2**2 + 4*S**(-1)*S4**(-1)*MG2**2 + 4*S*T1**(-2)*S4**(-1)*
     +    MS2*MG2 - 4*S*T1**(-2)*S4**(-1)*MG2**2 + 4*S*T1**(-1)*
     +    S4**(-1)*MS2 - 8*S*T1**(-1)*S4**(-1)*MG2 - 4*S*S4**(-1) + 8*
     +    T1**(-2)*U1*S4**(-1)*MS2*MG2 - 8*T1**(-2)*U1*S4**(-1)*MG2**2
     +     - 8*T1**(-2)*S4**(-1)*MS2*MG2**2 + 16*T1**(-2)*S4**(-1)*
     +    MS2**2*MG2 - 8*T1**(-2)*S4**(-1)*MS2**3 - 16*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MS2*MG2 + 16*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2 )
     +
      M2QGH2 = M2QGH2 + N**2*SYMBT*(S4+MS2)*CF * ( 4*T1**(-1)*U1*
     +    S4**(-1)*MS2 - 16*T1**(-1)*U1*S4**(-1)*MG2 - 16*T1**(-1)*
     +    U1**2*S4**(-1)*(S+T1)**(-1)*MG2 - 12*T1**(-1)*S4**(-1)*MS2*
     +    MG2 + 12*T1**(-1)*S4**(-1)*MS2**2 - 8*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2 - 8*U1*S4**(-1) - 8*U1**2*S4**(-1)*
     +    (S+T1)**(-1) - 4*S4**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + N**2*(S4+MS2)*CF * ( 8*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*T1**(-1)*
     +    U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*
     +    S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2*(M2*(S+T1)+T1*U1)**(-1)
     +     + 16*S**(-1)*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**3*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*U1*S4**(-1)*(S+T1)**(-1)
     +     + 4*S**(-1)*U1**2*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*
     +    S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 + 8*S**(-1)*S4**(-1)*MS2*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*S4**(-1) - 4*S*T1**(-2)*
     +    S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 4*S*T1**(-2)*
     +    S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) )
     +
      M2QGH2 = M2QGH2 + N**2*(S4+MS2)*CF * (  - 8*T1**(-2)*U1*S4**(-1)*
     +    MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*U1*S4**(-1)*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*S4**(-1)*MS2*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-2)*S4**(-1)*MS2**2*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*S4**(-1)*MS2**3*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 8*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) + 4*T1**(-1)*U1*S4**(-1)*
     +    MS2*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2 - 4*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*T1**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*U1*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*U1**2*S4**(-1)*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + N**2*(S4+MS2)*CF * ( 4*S4**(-1)*(S+T1)**(-1) )
     +
      M2QGH2 = M2QGH2 + ANG4(25)*N*CF**2 * ( 8*U1*MS2 + 8*MS2**2 )
     +
      M2QGH2 = M2QGH2 + ANG4(27)*N*CF**2 * (  - 16*S**(-1)*T1**(-1)*U1*
     +    MS2*MG2 - 16*S**(-1)*T1**(-1)*U1*MG2**2 - 16*S**(-1)*T1**(-1)
     +    *MS2*MG2**2 + 16*S**(-1)*T1**(-1)*MS2**3 + 4*S**(-1)*T1*MS2
     +     - 8*S**(-1)*T1*MG2 + 8*S**(-1)*U1*MS2 - 16*S**(-1)*U1*MG2 - 
     +    8*S**(-1)*MS2*MG2 + 16*S**(-1)*MS2**2 - 8*S**(-1)*MG2**2 - 16
     +    *T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 + 16*T1**(-1)*U1*
     +    (S+T1)**(-1)*MS2**2 + 16*T1**(-1)*U1*MS2 - 16*T1**(-1)*U1**2*
     +    (S+T1)**(-1)*MG2 + 8*T1**(-1)*U1**2 + 8*T1**(-1)*MS2**2 - 8*
     +    T1**(-1)*MG2**2 - 8*U1*(S+T1)**(-1)*MS2 + 4*U1 - 8*U1**2*
     +    (S+T1)**(-1) + 4*MS2 - 8*MG2 )
     +
      M2QGH2 = M2QGH2 + ANG4(27)*N**2*CF * ( 2*S**(-2)*T1*MS2*MG2 - 2*
     +    S**(-2)*T1*MG2**2 - 4*S**(-2)*U1*MS2*MG2 - 4*S**(-2)*U1*
     +    MG2**2 - 4*S**(-2)*MS2*MG2**2 + 4*S**(-2)*MS2**3 + 4*S**(-1)*
     +    T1**(-1)*U1*MS2**2 + 4*S**(-1)*T1**(-1)*U1*MG2**2 - 4*S**(-1)
     +    *T1**(-1)*U1**2*MG2 + 4*S**(-1)*T1**(-1)*MS2*MG2**2 - 4*
     +    S**(-1)*T1**(-1)*MS2**3 - 6*S**(-1)*U1*MS2 - 2*S**(-1)*MS2*
     +    MG2 - 6*S**(-1)*MS2**2 + 8*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2
     +     - 8*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 - 4*T1**(-1)*U1*MS2 + 8*
     +    T1**(-1)*U1**2*(S+T1)**(-1)*MG2 - 2*T1**(-1)*U1**2 - 2*
     +    T1**(-1)*MS2**2 + 2*T1**(-1)*MG2**2 + 4*U1*(S+T1)**(-1)*MS2
     +     + 4*U1**2*(S+T1)**(-1) )
     +
      M2QGH2 = M2QGH2 + ANG4(29)*N**2*CF * ( 8*S*T1**(-1)*MS2*MG2 + 4*S
     +    *MG2 + 8*T1**(-1)*U1*MS2*MG2 + 8*T1**(-1)*MS2**2*MG2 - 8*
     +    T1**(-1)*MS2**3 - 4*T1*MS2 + 4*T1*MG2 + 4*U1*MG2 + 8*MS2*MG2
     +     - 8*MS2**2 )
     +
      M2QGH2 = M2QGH2 + ANG4(31)*N*CF**2 * (  - 32*S**(-1)*T1**(-1)*U1*
     +    MG2**2 - 32*S**(-1)*T1**(-1)*MS2*MG2**2 + 32*S**(-1)*T1**(-1)
     +    *MS2**2*MG2 + 16*S**(-1)*MS2*MG2 - 16*S**(-1)*MG2**2 - 16*
     +    T1**(-1)*MS2*MG2 - 16*T1**(-1)*MG2**2 )
     +
      M2QGH2 = M2QGH2 + ANG4(31)*N**2*CF * ( 2*S**(-2)*T1*MS2*MG2 - 2*
     +    S**(-2)*T1*MG2**2 - 4*S**(-2)*U1*MS2*MG2 - 4*S**(-2)*U1*
     +    MG2**2 - 4*S**(-2)*MS2*MG2**2 + 4*S**(-2)*MS2**3 - 8*S**(-1)*
     +    T1**(-1)*U1*MS2*MG2 + 8*S**(-1)*T1**(-1)*U1*MS2**2 + 8*
     +    S**(-1)*T1**(-1)*U1*MG2**2 - 8*S**(-1)*T1**(-1)*U1**2*MG2 + 8
     +    *S**(-1)*T1**(-1)*MS2*MG2**2 - 8*S**(-1)*T1**(-1)*MS2**2*MG2
     +     - 2*S**(-1)*T1*MG2 - 4*S**(-1)*U1*MS2 - 8*S**(-1)*U1*MG2 - 
     +    10*S**(-1)*MS2*MG2 + 2*S**(-1)*MG2**2 + 2*S*T1**(-2)*MS2*MG2
     +     - 2*S*T1**(-2)*MG2**2 + 4*S*T1**(-1)*MS2 - 2*S*T1**(-1)*MG2
     +     + 4*T1**(-2)*U1*MS2*MG2 - 4*T1**(-2)*U1*MG2**2 - 4*T1**(-2)*
     +    MS2*MG2**2 + 8*T1**(-2)*MS2**2*MG2 - 4*T1**(-2)*MS2**3 + 4*
     +    T1**(-1)*U1*MS2 - 8*T1**(-1)*U1*MG2 + 6*T1**(-1)*MS2*MG2 + 8*
     +    T1**(-1)*MS2**2 + 2*T1**(-1)*MG2**2 - 4*MG2 )
     +
      M2QGH2 = M2QGH2 + ANG4(35)*N*CF**2 * ( 16*S**(-1)*T1**(-1)*U1*MS2
     +    *MG2 + 16*S**(-1)*T1**(-1)*U1*MG2**2 + 16*S**(-1)*T1**(-1)*
     +    MS2*MG2**2 - 16*S**(-1)*T1**(-1)*MS2**3 - 8*S**(-1)*MS2*MG2
     +     + 8*S**(-1)*MG2**2 + 16*S*T1**(-1)*U1 + 16*S*T1**(-1)*MS2 + 
     +    16*S*T1**(-1)*MG2 + 8*S + 8*S**2*T1**(-1) + 16*T1**(-1)*U1*
     +    (S+T1)**(-1)*MS2*MG2 - 16*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 + 
     +    16*T1**(-1)*U1*MS2 + 32*T1**(-1)*U1*MG2 + 16*T1**(-1)*U1**2*
     +    (S+T1)**(-1)*MG2 + 8*T1**(-1)*U1**2 + 32*T1**(-1)*MS2*MG2 - 8
     +    *T1**(-1)*MS2**2 + 8*T1**(-1)*MG2**2 - 8*U1*(S+T1)**(-1)*MS2
     +     - 8*U1**2*(S+T1)**(-1) - 8*MS2 + 16*MG2 )
     +
      M2QGH2 = M2QGH2 + ANG4(35)*N**2*CF * (  - 2*S**(-2)*T1*MS2*MG2 + 
     +    2*S**(-2)*T1*MG2**2 + 4*S**(-2)*U1*MS2*MG2 + 4*S**(-2)*U1*
     +    MG2**2 + 4*S**(-2)*MS2*MG2**2 - 4*S**(-2)*MS2**3 - 4*S**(-1)*
     +    T1**(-1)*U1*MS2**2 - 4*S**(-1)*T1**(-1)*U1*MG2**2 + 4*S**(-1)
     +    *T1**(-1)*U1**2*MG2 - 4*S**(-1)*T1**(-1)*MS2*MG2**2 + 4*
     +    S**(-1)*T1**(-1)*MS2**3 - 2*S**(-1)*T1*MS2 + 4*S**(-1)*T1*MG2
     +     + 2*S**(-1)*U1*MS2 + 8*S**(-1)*U1*MG2 + 10*S**(-1)*MS2*MG2
     +     - 2*S**(-1)*MS2**2 - 4*S*T1**(-1)*U1 - 4*S*T1**(-1)*MS2 - 4*
     +    S*T1**(-1)*MG2 - 2*S**2*T1**(-1) - 8*T1**(-1)*U1*(S+T1)**(-1)
     +    *MS2*MG2 + 8*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 - 4*T1**(-1)*U1*
     +    MS2 - 8*T1**(-1)*U1*MG2 - 8*T1**(-1)*U1**2*(S+T1)**(-1)*MG2
     +     - 2*T1**(-1)*U1**2 - 8*T1**(-1)*MS2*MG2 + 2*T1**(-1)*MS2**2
     +     - 2*T1**(-1)*MG2**2 + 2*T1 + 4*U1*(S+T1)**(-1)*MS2 + 4*U1 + 
     +    4*U1**2*(S+T1)**(-1) + 6*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(41)*N*CF**2 * ( 4 - 4*S**(-1)*T1 - 8*
     +    S**(-1)*U1 - 8*S**(-1)*MS2 + 8*S*T1**(-1) + 16*T1**(-1)*U1 + 
     +    16*T1**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(41)*N**2*CF * ( 2*S**(-1)*T1 + 4*S**(-1)*
     +    U1 + 4*S**(-1)*MS2 - 2*S*T1**(-1) - 4*T1**(-1)*U1 - 4*
     +    T1**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(47)*N*CF**2 * ( 8*S**(-1)*T1**(-1)*U1*MS2
     +     + 8*S**(-1)*T1**(-1)*U1**2 + 12*S**(-1)*U1 + 12*S**(-1)*MS2
     +     - 16*T1**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(47)*N**2*CF * (  - 4*S**(-1)*T1**(-1)*U1*
     +    MS2 - 4*S**(-1)*T1**(-1)*U1**2 - 4*S**(-1)*U1 - 4*S**(-1)*MS2
     +     + 8*T1**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(79)*N*CF**2 * ( 8*S*T1**(-2)*MS2*MG2 + 4*S
     +    *T1**(-1)*MG2 + 8*T1**(-2)*U1*MS2*MG2 + 8*T1**(-2)*MS2**2*MG2
     +     - 8*T1**(-2)*MS2**3 + 4*T1**(-1)*U1*MG2 + 8*T1**(-1)*MS2*MG2
     +     - 8*T1**(-1)*MS2**2 - 4*MS2 + 4*MG2 )
     +
      M2QGH2 = M2QGH2 + ANG4(80)*N*CF**2 * ( 12 + 8*S**(-1)*T1**(-1)*U1
     +    *MS2 + 8*S**(-1)*T1**(-1)*U1**2 + 8*S**(-1)*U1 + 8*S*T1**(-2)
     +    *MS2 + 12*S*T1**(-1) + 8*T1**(-2)*U1*MS2 + 8*T1**(-2)*MS2**2
     +     + 20*T1**(-1)*U1 + 16*T1**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(80)*N**2*CF * (  - 4 - 4*S**(-1)*T1**(-1)*
     +    U1*MS2 - 4*S**(-1)*T1**(-1)*U1**2 - 4*S**(-1)*U1 - 4*S*
     +    T1**(-1) - 8*T1**(-1)*U1 - 4*T1**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(81)*N*CF**2 * ( 4*S*T1**(-1)*MS2*MG2 - 4*S
     +    *T1**(-1)*MG2**2 + 4*S*MG2 + 8*T1**(-1)*U1*MS2*MG2 - 8*
     +    T1**(-1)*U1*MG2**2 - 8*T1**(-1)*MS2*MG2**2 + 16*T1**(-1)*
     +    MS2**2*MG2 - 8*T1**(-1)*MS2**3 - 4*T1*MS2 + 4*T1*MG2 + 4*U1*
     +    MG2 + 12*MS2*MG2 - 8*MS2**2 - 4*MG2**2 )
     +
      M2QGH2 = M2QGH2 + ANG4(82)*N*CF**2 * ( 16*S**(-1)*T1**(-1)*U1*
     +    MS2**2 - 16*S**(-1)*T1**(-1)*U1*MG2**2 - 16*S**(-1)*T1**(-1)*
     +    U1**2*MG2 - 16*S**(-1)*T1**(-1)*MS2*MG2**2 + 32*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 - 16*S**(-1)*T1**(-1)*MS2**3 + 8*S**(-1)*
     +    T1*U1 - 8*S**(-1)*T1*MS2 + 8*S**(-1)*T1*MG2 + 8*S**(-1)*U1**2
     +     + 16*S**(-1)*MS2*MG2 - 8*S**(-1)*MS2**2 - 8*S**(-1)*MG2**2
     +     + 4*S*T1**(-1)*MS2 - 8*S*T1**(-1)*MG2 + 4*S + 16*T1**(-1)*U1
     +    *(S+T1)**(-1)*MS2*MG2 - 16*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 + 
     +    8*T1**(-1)*U1*MS2 - 16*T1**(-1)*U1*MG2 + 16*T1**(-1)*U1**2*
     +    (S+T1)**(-1)*MG2 - 8*T1**(-1)*MS2*MG2 + 16*T1**(-1)*MS2**2 - 
     +    8*T1**(-1)*MG2**2 + 4*T1 - 8*U1*(S+T1)**(-1)*MS2 + 4*U1 - 8*
     +    U1**2*(S+T1)**(-1) + 4*MS2 )
     +
      M2QGH2 = M2QGH2 + ANG4(82)*N**2*CF * (  - 4*S**(-1)*T1**(-1)*U1*
     +    MS2**2 + 4*S**(-1)*T1**(-1)*U1*MG2**2 + 4*S**(-1)*T1**(-1)*
     +    U1**2*MG2 + 4*S**(-1)*T1**(-1)*MS2*MG2**2 - 8*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 + 4*S**(-1)*T1**(-1)*MS2**3 - 2*S**(-1)*
     +    T1*U1 + 2*S**(-1)*T1*MS2 - 2*S**(-1)*T1*MG2 - 2*S**(-1)*U1**2
     +     - 4*S**(-1)*MS2*MG2 + 2*S**(-1)*MS2**2 + 2*S**(-1)*MG2**2 + 
     +    2*S*T1**(-2)*MS2*MG2 - 2*S*T1**(-2)*MG2**2 - 2*S*T1**(-1)*MS2
     +     + 2*S*T1**(-1)*MG2 + 4*T1**(-2)*U1*MS2*MG2 - 4*T1**(-2)*U1*
     +    MG2**2 - 4*T1**(-2)*MS2*MG2**2 + 8*T1**(-2)*MS2**2*MG2 - 4*
     +    T1**(-2)*MS2**3 - 8*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 + 8*
     +    T1**(-1)*U1*(S+T1)**(-1)*MS2**2 - 2*T1**(-1)*U1*MS2 - 8*
     +    T1**(-1)*U1**2*(S+T1)**(-1)*MG2 + 2*T1**(-1)*MS2*MG2 - 2*
     +    T1**(-1)*MS2**2 + 4*U1*(S+T1)**(-1)*MS2 + 2*U1 + 4*U1**2*
     +    (S+T1)**(-1) )
     +
      M2QGH2 = M2QGH2 + ANG4(83)*N*CF**2 * (  - 16*S**(-1)*T1**(-1)*U1*
     +    MS2**2 + 16*S**(-1)*T1**(-1)*U1*MG2**2 + 16*S**(-1)*T1**(-1)*
     +    U1**2*MG2 + 16*S**(-1)*T1**(-1)*MS2*MG2**2 - 32*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 + 16*S**(-1)*T1**(-1)*MS2**3 + 8*S**(-1)*
     +    T1*U1 + 8*S**(-1)*T1*MS2 + 8*S**(-1)*T1*MG2 + 32*S**(-1)*U1*
     +    MG2 + 8*S**(-1)*U1**2 + 16*S**(-1)*MS2*MG2 - 24*S**(-1)*
     +    MS2**2 + 8*S**(-1)*MG2**2 - 16*T1**(-1)*U1*(S+T1)**(-1)*MS2*
     +    MG2 + 16*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 - 16*T1**(-1)*U1**2*
     +    (S+T1)**(-1)*MG2 - 8*T1**(-1)*MS2*MG2 + 8*T1**(-1)*MG2**2 - 8
     +    *U1*(S+T1)**(-1)*MS2 - 8*U1**2*(S+T1)**(-1) + 8*MG2 )
     +
      M2QGH2 = M2QGH2 + ANG4(83)*N**2*CF * ( 4*S**(-1)*T1**(-1)*U1*
     +    MS2**2 - 4*S**(-1)*T1**(-1)*U1*MG2**2 - 4*S**(-1)*T1**(-1)*
     +    U1**2*MG2 - 4*S**(-1)*T1**(-1)*MS2*MG2**2 + 8*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 - 4*S**(-1)*T1**(-1)*MS2**3 - 2*S**(-1)*
     +    T1*U1 - 2*S**(-1)*T1*MS2 - 2*S**(-1)*T1*MG2 - 8*S**(-1)*U1*
     +    MG2 - 2*S**(-1)*U1**2 - 4*S**(-1)*MS2*MG2 + 6*S**(-1)*MS2**2
     +     - 2*S**(-1)*MG2**2 - 2*S*T1**(-2)*MS2*MG2 + 2*S*T1**(-2)*
     +    MG2**2 + 2*S*T1**(-1)*MG2 - 4*T1**(-2)*U1*MS2*MG2 + 4*
     +    T1**(-2)*U1*MG2**2 + 4*T1**(-2)*MS2*MG2**2 - 8*T1**(-2)*
     +    MS2**2*MG2 + 4*T1**(-2)*MS2**3 + 8*T1**(-1)*U1*(S+T1)**(-1)*
     +    MS2*MG2 - 8*T1**(-1)*U1*(S+T1)**(-1)*MS2**2 - 2*T1**(-1)*U1*
     +    MS2 + 8*T1**(-1)*U1*MG2 + 8*T1**(-1)*U1**2*(S+T1)**(-1)*MG2
     +     + 6*T1**(-1)*MS2*MG2 - 6*T1**(-1)*MS2**2 + 4*U1*(S+T1)**(-1)
     +    *MS2 + 2*U1 + 4*U1**2*(S+T1)**(-1) + 2*MS2 )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N*SYMBT*(S4+MS2)*CF**2 * (  - 32*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MS2**2 + 32*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MG2**2 + 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 + 32
     +    *S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2 - 64*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**2*MG2 + 32*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 + 
     +    16*S**(-1)*T1*U1*S4**(-1) + 16*S**(-1)*T1*S4**(-1)*MS2 + 16*
     +    S**(-1)*T1*S4**(-1)*MG2 + 64*S**(-1)*U1*S4**(-1)*MG2 + 16*
     +    S**(-1)*U1**2*S4**(-1) + 32*S**(-1)*S4**(-1)*MS2*MG2 - 48*
     +    S**(-1)*S4**(-1)*MS2**2 + 16*S**(-1)*S4**(-1)*MG2**2 - 32*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2 + 32*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MS2**2 - 32*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 16*T1**(-1)*S4**(-1)*MS2*MG2 + 16*T1**(-1)
     +    *S4**(-1)*MG2**2 - 16*U1*S4**(-1)*(S+T1)**(-1)*MS2 - 16*U1**2
     +    *S4**(-1)*(S+T1)**(-1) + 16*S4**(-1)*MG2 )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 32*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) - 32*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1)
     +     - 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 64*S**(-1)*T1**(-1)*S4**(-1)
     +    *MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**3*(M2*(S+T1)+T1*U1)**(-1) + 16*S**(-1)*T1*U1*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S**(-1)*T1*S4**(-1)*MS2
     +    *(M2*(S+T1)+T1*U1)**(-1) + 16*S**(-1)*T1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*U1*S4**(-1)*(S+T1)**(-1)
     +     + 16*S**(-1)*U1**2*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 32*
     +    S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 + 32*S**(-1)*S4**(-1)*MS2*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) - 16*S**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N*(S4+MS2)*CF**2 * (  - 16*S**(-1)*
     +    S4**(-1) - 16*S*T1**(-1)*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S*T1**(-1)*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*S*T1**(-1)*S4**(-1)*(S+T1)**(-1)
     +     + 16*S*T1**(-1)*S4**(-1)*MS2*MG2**2*(M2*(S+T1)+T1*U1)**(-2)
     +     + 8*S*T1**(-1)*S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) - 32*S*
     +    T1**(-1)*S4**(-1)*MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-2) + 16*S*
     +    T1**(-1)*S4**(-1)*MS2**3*(M2*(S+T1)+T1*U1)**(-2) - 16*S*
     +    T1**(-1)*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*S*T1*
     +    S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-2) - 16*S*T1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 32*S*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*S*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N*(S4+MS2)*CF**2 * (  - 8*S**2*
     +    T1**(-1)*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-2) + 8*S**2*
     +    T1**(-1)*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-2) - 8*S**2*
     +    S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-2) + 32*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 32*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1) + 16*T1**(-1)*U1*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*T1**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2 - 16*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N*(S4+MS2)*CF**2 * (  - 24*T1*S4**(-1)
     +    *MS2*MG2*(M2*(S+T1)+T1*U1)**(-2) + 16*T1*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*T1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*T1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1**2*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*T1**2*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*U1*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*U1**2*S4**(-1)*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S4**(-1)*(S+T1)**(-1) + 16*
     +    S4**(-1)*MS2*MG2**2*(M2*(S+T1)+T1*U1)**(-2) + 8*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S4**(-1)*MS2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N**2*SYMBT*(S4+MS2)*CF * ( 8*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2**2 - 8*S**(-1)*T1**(-1)*U1*S4**(-1)*
     +    MG2**2 - 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 - 8*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2*MG2**2 + 16*S**(-1)*T1**(-1)*S4**(-1)*
     +    MS2**2*MG2 - 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**3 - 4*S**(-1)*
     +    T1*U1*S4**(-1) - 4*S**(-1)*T1*S4**(-1)*MS2 - 4*S**(-1)*T1*
     +    S4**(-1)*MG2 - 16*S**(-1)*U1*S4**(-1)*MG2 - 4*S**(-1)*U1**2*
     +    S4**(-1) - 8*S**(-1)*S4**(-1)*MS2*MG2 + 12*S**(-1)*S4**(-1)*
     +    MS2**2 - 4*S**(-1)*S4**(-1)*MG2**2 - 4*S*T1**(-2)*S4**(-1)*
     +    MS2*MG2 + 4*S*T1**(-2)*S4**(-1)*MG2**2 + 4*S*T1**(-1)*
     +    S4**(-1)*MG2 - 8*T1**(-2)*U1*S4**(-1)*MS2*MG2 + 8*T1**(-2)*U1
     +    *S4**(-1)*MG2**2 + 8*T1**(-2)*S4**(-1)*MS2*MG2**2 - 16*
     +    T1**(-2)*S4**(-1)*MS2**2*MG2 + 8*T1**(-2)*S4**(-1)*MS2**3 + 
     +    16*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2 - 16*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MS2**2 - 4*T1**(-1)*U1*S4**(-1)*MS2 + 
     +    16*T1**(-1)*U1*S4**(-1)*MG2 )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N**2*SYMBT*(S4+MS2)*CF * ( 16*T1**(-1)
     +    *U1**2*S4**(-1)*(S+T1)**(-1)*MG2 + 12*T1**(-1)*S4**(-1)*MS2*
     +    MG2 - 12*T1**(-1)*S4**(-1)*MS2**2 + 8*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2 + 4*U1*S4**(-1) + 8*U1**2*S4**(-1)*
     +    (S+T1)**(-1) + 4*S4**(-1)*MS2 )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 8*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 8*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1)
     +     + 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 16*S**(-1)*T1**(-1)*S4**(-1)
     +    *MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**3*(M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*T1*U1*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*T1*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*T1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*U1*S4**(-1)*(S+T1)**(-1)
     +     - 4*S**(-1)*U1**2*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*
     +    S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 - 8*S**(-1)*S4**(-1)*MS2*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N**2*(S4+MS2)*CF * ( 4*S**(-1)*
     +    S4**(-1) + 4*S*T1**(-2)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S*T1**(-2)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S*T1**(-1)*S4**(-1)*(S+T1)**(-1)
     +     - 4*S*T1**(-1)*S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 4*S*
     +    T1**(-1)*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*U1
     +    *S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*U1*
     +    S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*S4**(-1)
     +    *MS2*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-2)*S4**(-1)*
     +    MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*S4**(-1)*
     +    MS2**3*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) - 8*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) - 4*T1**(-1)*U1*S4**(-1)*
     +    MS2*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH2 = M2QGH2 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 8*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2 + 4*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*T1**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*U1*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*U1**2*S4**(-1)*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S4**(-1)*(S+T1)**(-1) )

      M2QGH3 = 0D0
      M2QGH3 = M2QGH3 + N*SYMBT*(S4+MS2)*CF**2 * (  - 64*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2 + 64*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MG2**2 + 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 - 64
     +    *S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2 + 32*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**2*MG2 + 32*S**(-1)*T1**(-1)*S4**(-1)*MG2**3 + 
     +    32*S**(-1)*T1*S4**(-1)*MG2 + 64*S**(-1)*U1*S4**(-1)*MG2 - 64*
     +    S**(-1)*S4**(-1)*MS2*MG2 + 64*S**(-1)*S4**(-1)*MG2**2 + 32*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2 - 32*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MG2**2 - 32*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 16*T1**(-1)*S4**(-1)*MS2*MG2 + 16*T1**(-1)
     +    *S4**(-1)*MG2**2 - 16*U1*S4**(-1)*(S+T1)**(-1)*MG2 + 16*
     +    S4**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + N*(S4+MS2)*CF**2 * ( 64*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 64*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 32*
     +    S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1)
     +     + 64*S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*T1**(-1)*S4**(-1)*MS2**2
     +    *MG2*(M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*T1**(-1)*S4**(-1)*
     +    MG2**3*(M2*(S+T1)+T1*U1)**(-1) - 32*S**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 16*S*T1**(-1)*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S*T1**(-1)*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 32*S*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S*T1**(-1)*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S*T1**(-1)*S4**(-1)*MG2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH3 = M2QGH3 + N*(S4+MS2)*CF**2 * (  - 8*S**2*T1**(-1)*
     +    S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-2) + 8*S**2*T1**(-1)*
     +    S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-2) - 32*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 32*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 + 48*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 48*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*T1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*T1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*U1*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH3 = M2QGH3 + N*(S4+MS2)*CF**2 * ( 16*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 32*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MG2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH3 = M2QGH3 + N**2*SYMBT*(S4+MS2)*CF * ( 16*S**(-1)*T1**(-1)*
     +    U1*S4**(-1)*MS2*MG2 - 16*S**(-1)*T1**(-1)*U1*S4**(-1)*MG2**2
     +     - 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 + 16*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2*MG2**2 - 8*S**(-1)*T1**(-1)*S4**(-1)*
     +    MS2**2*MG2 - 8*S**(-1)*T1**(-1)*S4**(-1)*MG2**3 - 8*S**(-1)*
     +    T1*S4**(-1)*MG2 - 16*S**(-1)*U1*S4**(-1)*MG2 + 16*S**(-1)*
     +    S4**(-1)*MS2*MG2 - 16*S**(-1)*S4**(-1)*MG2**2 - 4*S*T1**(-2)*
     +    S4**(-1)*MS2*MG2 + 4*S*T1**(-2)*S4**(-1)*MG2**2 + 4*S*
     +    T1**(-1)*S4**(-1)*MG2 - 8*T1**(-2)*U1*S4**(-1)*MS2*MG2 + 8*
     +    T1**(-2)*U1*S4**(-1)*MG2**2 - 16*T1**(-2)*S4**(-1)*MS2*MG2**2
     +     + 8*T1**(-2)*S4**(-1)*MS2**2*MG2 + 8*T1**(-2)*S4**(-1)*
     +    MG2**3 - 16*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2 + 16*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2**2 + 12*T1**(-1)*U1*
     +    S4**(-1)*MG2 + 16*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*MG2 - 
     +    12*T1**(-1)*S4**(-1)*MS2*MG2 + 12*T1**(-1)*S4**(-1)*MG2**2 + 
     +    8*U1*S4**(-1)*(S+T1)**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + N**2*SYMBT*(S4+MS2)*CF * ( 4*S4**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + N**2*(S4+MS2)*CF * (  - 16*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 16*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 8*
     +    S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1)
     +     - 16*S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1**(-1)*S4**(-1)*
     +    MG2**3*(M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 + 4*S*T1**(-2)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S*T1**(-2)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-2)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*S4**(-1)*MG2**3*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH3 = M2QGH3 + N**2*(S4+MS2)*CF * ( 16*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 4*
     +    T1**(-1)*U1*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 16*
     +    T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MG2 - 4*T1**(-1)*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 4
     +    *T1**(-1)*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 8*U1*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH3 = M2QGH3 + ANG4(25)*N*CF**2 * (  - 8*MS2*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(27)*N*CF**2 * ( 16*S**(-1)*T1**(-1)*U1*MS2
     +    *MG2 + 16*S**(-1)*T1**(-1)*U1*MG2**2 - 16*S**(-1)*T1**(-1)*
     +    MS2**2*MG2 + 16*S**(-1)*T1**(-1)*MG2**3 + 4*S**(-1)*T1*MG2 + 
     +    8*S**(-1)*U1*MG2 - 24*S**(-1)*MS2*MG2 + 24*S**(-1)*MG2**2 - 
     +    16*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 + 16*T1**(-1)*U1*
     +    (S+T1)**(-1)*MG2**2 + 16*T1**(-1)*U1**2*(S+T1)**(-1)*MG2 + 8*
     +    U1*(S+T1)**(-1)*MG2 - 4*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(27)*N**2*CF * (  - 2*S**(-2)*T1*MS2*MG2 + 
     +    2*S**(-2)*T1*MG2**2 + 4*S**(-2)*U1*MS2*MG2 + 4*S**(-2)*U1*
     +    MG2**2 - 4*S**(-2)*MS2**2*MG2 + 4*S**(-2)*MG2**3 - 8*S**(-1)*
     +    T1**(-1)*U1*MS2*MG2 + 4*S**(-1)*T1**(-1)*U1**2*MG2 + 4*
     +    S**(-1)*T1**(-1)*MS2**2*MG2 - 4*S**(-1)*T1**(-1)*MG2**3 + 2*
     +    S**(-1)*U1*MG2 + 10*S**(-1)*MS2*MG2 - 2*S**(-1)*MG2**2 + 8*
     +    T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 - 8*T1**(-1)*U1*(S+T1)**(-1)
     +    *MG2**2 - 8*T1**(-1)*U1**2*(S+T1)**(-1)*MG2 - 4*U1*
     +    (S+T1)**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(29)*N**2*CF * (  - 8*S*T1**(-1)*MS2*MG2 - 
     +    4*S*MG2 - 8*T1**(-1)*U1*MS2*MG2 - 8*T1**(-1)*MS2*MG2**2 + 8*
     +    T1**(-1)*MS2**2*MG2 - 4*U1*MG2 + 8*MS2*MG2 - 8*MG2**2 )
     +
      M2QGH3 = M2QGH3 + ANG4(31)*N*CF**2 * ( 32*S**(-1)*T1**(-1)*U1*
     +    MG2**2 - 32*S**(-1)*T1**(-1)*MS2*MG2**2 + 32*S**(-1)*T1**(-1)
     +    *MG2**3 - 16*S**(-1)*MS2*MG2 + 16*S**(-1)*MG2**2 + 16*
     +    T1**(-1)*MS2*MG2 + 16*T1**(-1)*MG2**2 )
     +
      M2QGH3 = M2QGH3 + ANG4(31)*N**2*CF * (  - 2*S**(-2)*T1*MS2*MG2 + 
     +    2*S**(-2)*T1*MG2**2 + 4*S**(-2)*U1*MS2*MG2 + 4*S**(-2)*U1*
     +    MG2**2 - 4*S**(-2)*MS2**2*MG2 + 4*S**(-2)*MG2**3 - 8*S**(-1)*
     +    T1**(-1)*U1*MS2*MG2 + 8*S**(-1)*T1**(-1)*U1**2*MG2 + 8*
     +    S**(-1)*T1**(-1)*MS2*MG2**2 - 8*S**(-1)*T1**(-1)*MG2**3 + 2*
     +    S**(-1)*T1*MG2 + 8*S**(-1)*U1*MG2 + 6*S**(-1)*MS2*MG2 + 2*
     +    S**(-1)*MG2**2 - 2*S*T1**(-2)*MS2*MG2 + 2*S*T1**(-2)*MG2**2
     +     + 2*S*T1**(-1)*MG2 - 4*T1**(-2)*U1*MS2*MG2 + 4*T1**(-2)*U1*
     +    MG2**2 - 8*T1**(-2)*MS2*MG2**2 + 4*T1**(-2)*MS2**2*MG2 + 4*
     +    T1**(-2)*MG2**3 + 8*T1**(-1)*U1*MG2 - 18*T1**(-1)*MS2*MG2 + 2
     +    *T1**(-1)*MG2**2 )
     +
      M2QGH3 = M2QGH3 + ANG4(35)*N*CF**2 * (  - 16*S**(-1)*T1**(-1)*U1*
     +    MS2*MG2 - 16*S**(-1)*T1**(-1)*U1*MG2**2 + 16*S**(-1)*T1**(-1)
     +    *MS2**2*MG2 - 16*S**(-1)*T1**(-1)*MG2**3 + 8*S**(-1)*MS2*MG2
     +     - 8*S**(-1)*MG2**2 - 16*S*T1**(-1)*MG2 + 16*T1**(-1)*U1*
     +    (S+T1)**(-1)*MS2*MG2 - 16*T1**(-1)*U1*(S+T1)**(-1)*MG2**2 - 
     +    32*T1**(-1)*U1*MG2 - 16*T1**(-1)*U1**2*(S+T1)**(-1)*MG2 - 32*
     +    T1**(-1)*MG2**2 + 8*U1*(S+T1)**(-1)*MG2 - 8*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(35)*N**2*CF * ( 2*S**(-2)*T1*MS2*MG2 - 2*
     +    S**(-2)*T1*MG2**2 - 4*S**(-2)*U1*MS2*MG2 - 4*S**(-2)*U1*
     +    MG2**2 + 4*S**(-2)*MS2**2*MG2 - 4*S**(-2)*MG2**3 + 8*S**(-1)*
     +    T1**(-1)*U1*MS2*MG2 - 4*S**(-1)*T1**(-1)*U1**2*MG2 - 4*
     +    S**(-1)*T1**(-1)*MS2**2*MG2 + 4*S**(-1)*T1**(-1)*MG2**3 - 2*
     +    S**(-1)*T1*MG2 - 6*S**(-1)*U1*MG2 - 2*S**(-1)*MS2*MG2 - 6*
     +    S**(-1)*MG2**2 + 4*S*T1**(-1)*MG2 - 8*T1**(-1)*U1*
     +    (S+T1)**(-1)*MS2*MG2 + 8*T1**(-1)*U1*(S+T1)**(-1)*MG2**2 + 8*
     +    T1**(-1)*U1*MG2 + 8*T1**(-1)*U1**2*(S+T1)**(-1)*MG2 + 8*
     +    T1**(-1)*MG2**2 - 4*U1*(S+T1)**(-1)*MG2 - 2*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(41)*N*CF**2 * ( 8*S**(-1)*MG2 - 16*
     +    T1**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(41)*N**2*CF * (  - 4*S**(-1)*MG2 + 4*
     +    T1**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(47)*N*CF**2 * (  - 8*S**(-1)*T1**(-1)*U1*
     +    MG2 - 12*S**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(47)*N**2*CF * ( 4*S**(-1)*T1**(-1)*U1*MG2
     +     + 4*S**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(79)*N*CF**2 * (  - 8*S*T1**(-2)*MS2*MG2 - 
     +    4*S*T1**(-1)*MG2 - 8*T1**(-2)*U1*MS2*MG2 - 8*T1**(-2)*MS2*
     +    MG2**2 + 8*T1**(-2)*MS2**2*MG2 - 4*T1**(-1)*U1*MG2 + 8*
     +    T1**(-1)*MS2*MG2 - 8*T1**(-1)*MG2**2 )
     +
      M2QGH3 = M2QGH3 + ANG4(80)*N*CF**2 * (  - 8*S**(-1)*T1**(-1)*U1*
     +    MG2 - 8*T1**(-2)*MS2*MG2 - 16*T1**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(80)*N**2*CF * ( 4*S**(-1)*T1**(-1)*U1*MG2
     +     + 4*T1**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(81)*N*CF**2 * (  - 4*S*T1**(-1)*MS2*MG2 + 
     +    4*S*T1**(-1)*MG2**2 - 4*S*MG2 - 8*T1**(-1)*U1*MS2*MG2 + 8*
     +    T1**(-1)*U1*MG2**2 - 16*T1**(-1)*MS2*MG2**2 + 8*T1**(-1)*
     +    MS2**2*MG2 + 8*T1**(-1)*MG2**3 - 4*U1*MG2 + 4*MS2*MG2 - 4*
     +    MG2**2 )
     +
      M2QGH3 = M2QGH3 + ANG4(82)*N*CF**2 * (  - 32*S**(-1)*T1**(-1)*U1*
     +    MS2*MG2 + 32*S**(-1)*T1**(-1)*U1*MG2**2 + 16*S**(-1)*T1**(-1)
     +    *U1**2*MG2 - 32*S**(-1)*T1**(-1)*MS2*MG2**2 + 16*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 + 16*S**(-1)*T1**(-1)*MG2**3 + 4*S*
     +    T1**(-1)*MG2 + 16*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 - 16*
     +    T1**(-1)*U1*(S+T1)**(-1)*MG2**2 + 8*T1**(-1)*U1*MG2 - 16*
     +    T1**(-1)*U1**2*(S+T1)**(-1)*MG2 - 24*T1**(-1)*MS2*MG2 + 24*
     +    T1**(-1)*MG2**2 + 8*U1*(S+T1)**(-1)*MG2 - 4*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(82)*N**2*CF * ( 8*S**(-1)*T1**(-1)*U1*MS2*
     +    MG2 - 8*S**(-1)*T1**(-1)*U1*MG2**2 - 4*S**(-1)*T1**(-1)*U1**2
     +    *MG2 + 8*S**(-1)*T1**(-1)*MS2*MG2**2 - 4*S**(-1)*T1**(-1)*
     +    MS2**2*MG2 - 4*S**(-1)*T1**(-1)*MG2**3 - 2*S*T1**(-2)*MS2*MG2
     +     + 2*S*T1**(-2)*MG2**2 - 4*T1**(-2)*U1*MS2*MG2 + 4*T1**(-2)*
     +    U1*MG2**2 - 8*T1**(-2)*MS2*MG2**2 + 4*T1**(-2)*MS2**2*MG2 + 4
     +    *T1**(-2)*MG2**3 - 8*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 + 8*
     +    T1**(-1)*U1*(S+T1)**(-1)*MG2**2 + 2*T1**(-1)*U1*MG2 + 8*
     +    T1**(-1)*U1**2*(S+T1)**(-1)*MG2 + 2*T1**(-1)*MS2*MG2 - 2*
     +    T1**(-1)*MG2**2 - 4*U1*(S+T1)**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(83)*N*CF**2 * ( 32*S**(-1)*T1**(-1)*U1*MS2
     +    *MG2 - 32*S**(-1)*T1**(-1)*U1*MG2**2 - 16*S**(-1)*T1**(-1)*
     +    U1**2*MG2 + 32*S**(-1)*T1**(-1)*MS2*MG2**2 - 16*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 - 16*S**(-1)*T1**(-1)*MG2**3 - 16*S**(-1)
     +    *T1*MG2 - 32*S**(-1)*U1*MG2 + 32*S**(-1)*MS2*MG2 - 32*S**(-1)
     +    *MG2**2 - 16*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 + 16*T1**(-1)*
     +    U1*(S+T1)**(-1)*MG2**2 + 16*T1**(-1)*U1**2*(S+T1)**(-1)*MG2
     +     + 8*T1**(-1)*MS2*MG2 - 8*T1**(-1)*MG2**2 + 8*U1*(S+T1)**(-1)
     +    *MG2 - 8*MG2 )
     +
      M2QGH3 = M2QGH3 + ANG4(83)*N**2*CF * (  - 8*S**(-1)*T1**(-1)*U1*
     +    MS2*MG2 + 8*S**(-1)*T1**(-1)*U1*MG2**2 + 4*S**(-1)*T1**(-1)*
     +    U1**2*MG2 - 8*S**(-1)*T1**(-1)*MS2*MG2**2 + 4*S**(-1)*
     +    T1**(-1)*MS2**2*MG2 + 4*S**(-1)*T1**(-1)*MG2**3 + 4*S**(-1)*
     +    T1*MG2 + 8*S**(-1)*U1*MG2 - 8*S**(-1)*MS2*MG2 + 8*S**(-1)*
     +    MG2**2 + 2*S*T1**(-2)*MS2*MG2 - 2*S*T1**(-2)*MG2**2 - 2*S*
     +    T1**(-1)*MG2 + 4*T1**(-2)*U1*MS2*MG2 - 4*T1**(-2)*U1*MG2**2
     +     + 8*T1**(-2)*MS2*MG2**2 - 4*T1**(-2)*MS2**2*MG2 - 4*T1**(-2)
     +    *MG2**3 + 8*T1**(-1)*U1*(S+T1)**(-1)*MS2*MG2 - 8*T1**(-1)*U1*
     +    (S+T1)**(-1)*MG2**2 - 6*T1**(-1)*U1*MG2 - 8*T1**(-1)*U1**2*
     +    (S+T1)**(-1)*MG2 + 6*T1**(-1)*MS2*MG2 - 6*T1**(-1)*MG2**2 - 4
     +    *U1*(S+T1)**(-1)*MG2 - 2*MG2 )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N*SYMBT*(S4+MS2)*CF**2 * ( 64*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2 - 64*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MG2**2 - 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 + 64
     +    *S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2 - 32*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**2*MG2 - 32*S**(-1)*T1**(-1)*S4**(-1)*MG2**3 - 
     +    32*S**(-1)*T1*S4**(-1)*MG2 - 64*S**(-1)*U1*S4**(-1)*MG2 + 64*
     +    S**(-1)*S4**(-1)*MS2*MG2 - 64*S**(-1)*S4**(-1)*MG2**2 - 32*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2 + 32*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MG2**2 + 32*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2 + 16*T1**(-1)*S4**(-1)*MS2*MG2 - 16*T1**(-1)
     +    *S4**(-1)*MG2**2 + 16*U1*S4**(-1)*(S+T1)**(-1)*MG2 - 16*
     +    S4**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N*(S4+MS2)*CF**2 * (  - 64*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 64*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1)
     +     + 32*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 64*S**(-1)*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*T1**(-1)*S4**(-1)
     +    *MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*T1**(-1)*
     +    S4**(-1)*MG2**3*(M2*(S+T1)+T1*U1)**(-1) + 32*S**(-1)*S4**(-1)
     +    *(S+T1)**(-1)*MG2 + 16*S*T1**(-1)*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*T1**(-1)*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 32*S*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*S*T1**(-1)*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S*T1**(-1)*S4**(-1)*MG2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) + 8*S*T1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 8*S*U1*S4**(-1)*
     +    MG2*(M2*(S+T1)+T1*U1)**(-2) + 8*S**2*T1**(-1)*S4**(-1)*MS2*
     +    MG2*(M2*(S+T1)+T1*U1)**(-2) - 8*S**2*T1**(-1)*S4**(-1)*MG2**2
     +    *(M2*(S+T1)+T1*U1)**(-2) + 8*S**2*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 32*T1**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 32*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*
     +    T1**(-1)*U1*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 32*
     +    T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MG2 - 48*T1**(-1)*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 
     +    48*T1**(-1)*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1*U1
     +    *S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-2) - 8*T1*S4**(-1)*MS2*MG2
     +    *(M2*(S+T1)+T1*U1)**(-2) + 8*T1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 16*U1*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N*(S4+MS2)*CF**2 * ( 16*U1*S4**(-1)*
     +    MS2*MG2*(M2*(S+T1)+T1*U1)**(-2) - 16*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) + 32*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 16*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-2) - 8*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MG2**3*
     +    (M2*(S+T1)+T1*U1)**(-2) )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N**2*SYMBT*(S4+MS2)*CF * (  - 16*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MS2*MG2 + 16*S**(-1)*T1**(-1)*U1
     +    *S4**(-1)*MG2**2 + 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 - 16
     +    *S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2**2 + 8*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2**2*MG2 + 8*S**(-1)*T1**(-1)*S4**(-1)*MG2**3 + 8*
     +    S**(-1)*T1*S4**(-1)*MG2 + 16*S**(-1)*U1*S4**(-1)*MG2 - 16*
     +    S**(-1)*S4**(-1)*MS2*MG2 + 16*S**(-1)*S4**(-1)*MG2**2 + 4*S*
     +    T1**(-2)*S4**(-1)*MS2*MG2 - 4*S*T1**(-2)*S4**(-1)*MG2**2 - 4*
     +    S*T1**(-1)*S4**(-1)*MG2 + 8*T1**(-2)*U1*S4**(-1)*MS2*MG2 - 8*
     +    T1**(-2)*U1*S4**(-1)*MG2**2 + 16*T1**(-2)*S4**(-1)*MS2*MG2**2
     +     - 8*T1**(-2)*S4**(-1)*MS2**2*MG2 - 8*T1**(-2)*S4**(-1)*
     +    MG2**3 + 16*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2 - 16*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2**2 - 12*T1**(-1)*U1*
     +    S4**(-1)*MG2 - 16*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*MG2 + 
     +    12*T1**(-1)*S4**(-1)*MS2*MG2 - 12*T1**(-1)*S4**(-1)*MG2**2 - 
     +    8*U1*S4**(-1)*(S+T1)**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N**2*SYMBT*(S4+MS2)*CF * (  - 4*
     +    S4**(-1)*MG2 )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N**2*(S4+MS2)*CF * ( 16*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 16*
     +    S**(-1)*T1**(-1)*U1*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1)
     +     - 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S**(-1)*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*T1**(-1)*S4**(-1)*
     +    MS2**2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*T1**(-1)*
     +    S4**(-1)*MG2**3*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 4*S*T1**(-2)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S*T1**(-2)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-2)*U1*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*U1*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-2)*S4**(-1)*MS2*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*S4**(-1)*MS2**2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-2)*S4**(-1)*MG2**3*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH3 = M2QGH3 + COLO1(9)*N**2*(S4+MS2)*CF * (  - 16*T1**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 16*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*T1**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 + 4*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*U1*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) )

      M2QGH4 = 0D0
      M2QGH4 = M2QGH4 + N*SYMBT*(S4+MS2)*CF * ( 16*S**(-1)*T1*U1*
     +    S4**(-1)*MG2*(T1+U1+M2)**(-1) + 4*S**(-1)*T1*U1**2*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 12*S**(-1)*T1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*T1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1**2*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1**2*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*T1**3*S4**(-1)*(T1+U1+M2)**(-1)
     +     - 8*S**(-1)*U1*S4**(-1)*MS2**2*(T1+U1+M2)**(-1) + 8*S**(-1)*
     +    U1*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) + 8*S**(-1)*U1**2*
     +    S4**(-1)*MG2*(T1+U1+M2)**(-1) + 8*S**(-1)*S4**(-1)*MS2*MG2**2
     +    *(T1+U1+M2)**(-1) - 16*S**(-1)*S4**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*S4**(-1)*MS2**3*(T1+U1+M2)**(-1)
     +     + 4*S*T1**(-1)*S4**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 4*S*
     +    T1**(-1)*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) - 4*S*T1*S4**(-1)*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*SYMBT*(S4+MS2)*CF * ( 8*S*U1*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) + 8*S*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*(T1+U1+M2)**(-1) + 4*S*S4**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 8*S*S4**(-1)*MG2*(T1+U1+M2)**(-1) + 8*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 8*T1**(-1)*U1
     +    *S4**(-1)*MG2**2*(T1+U1+M2)**(-1) - 8*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(T1+U1+M2)**(-1) + 16*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) - 8*T1**(-1)*S4**(-1)*MS2**3*
     +    (T1+U1+M2)**(-1) - 8*T1*U1*S4**(-1)*(T1+U1+M2)**(-1) - 4*T1*
     +    S4**(-1)*MS2*(T1+U1+M2)**(-1) - 16*U1*S4**(-1)*(S+T1)**(-1)*
     +    MS2*MG2*(T1+U1+M2)**(-1) + 16*U1*S4**(-1)*(S+T1)**(-1)*MS2**2
     +    *(T1+U1+M2)**(-1) - 4*U1*S4**(-1)*MS2*(T1+U1+M2)**(-1) - 16*
     +    U1*S4**(-1)*MG2*(T1+U1+M2)**(-1) - 16*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 8*U1**2*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 12*S4**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 12*
     +    S4**(-1)*MS2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*SYMBU*(S4+MS2)*CF * (  - 8*S**(-1)*T1**(-1)*
     +    U1*S4**(-1)*MS2*MG2 + 8*S**(-1)*T1**(-1)*U1*S4**(-1)*MS2**2
     +     - 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 + 8*S**(-1)*T1**(-1)
     +    *U1**3*S4**(-1) + 8*S**(-1)*T1*U1*S4**(-1) - 8*S**(-1)*T1*
     +    U1**2*S4**(-1)*(T1+U1+M2)**(-1) - 4*S**(-1)*T1*S4**(-1)*MS2*
     +    MG2*(S+U1+M2)**(-1) - 4*S**(-1)*T1*S4**(-1)*MG2 + 4*S**(-1)*
     +    T1*S4**(-1)*MG2**2*(S+U1+M2)**(-1) - 4*S**(-1)*T1**2*U1*
     +    S4**(-1)*(T1+U1+M2)**(-1) - 8*S**(-1)*U1*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*U1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 12*S**(-1)*U1*S4**(-1)*MG2 + 4*S**(-1)*U1*
     +    S4**(-1)*MG2**2*(T1+U1+M2)**(-1) + 16*S**(-1)*U1**2*S4**(-1)
     +     - 4*S**(-1)*U1**3*S4**(-1)*(T1+U1+M2)**(-1) - 16*S**(-1)*
     +    S4**(-1)*MS2*MG2 + 16*S**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) + 8*S**(-1)*S4**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 32*
     +    S**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 8*S**(-1)*
     +    S4**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*SYMBU*(S4+MS2)*CF * ( 8*S**(-1)*S4**(-1)*
     +    MS2**2 + 16*S**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) - 8*S**(-1)
     +    *S4**(-1)*MS2**3*(S+U1+M2)**(-1) + 8*S**(-1)*S4**(-1)*MG2**2
     +     - 8*S**(-1)*S4**(-1)*MG2**3*(S+U1+M2)**(-1) + 4*S*T1**(-1)*
     +    U1*S4**(-1) - 16*S*T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) - 
     +    4*S*T1**(-1)*S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 4*S*T1**(-1)*
     +    S4**(-1)*MS2 + 16*S*T1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) - 
     +    4*S*T1**(-1)*S4**(-1)*MG2 + 4*S*T1**(-1)*S4**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) + 4*S*T1*S4**(-1)*(S+U1+M2)**(-1) - 2*S*T1*
     +    S4**(-1)*(T1+U1+M2)**(-1) - 6*S*U1*S4**(-1)*(T1+U1+M2)**(-1)
     +     + 16*S*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 4*S
     +    *S4**(-1)*MS2*(S+U1+M2)**(-1) - 2*S*S4**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 16*S*S4**(-1)*MS2**2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) - 8*S*S4**(-1)*MG2*(S+U1+M2)**(-1) + 2*S*
     +    S4**(-1)*MG2*(T1+U1+M2)**(-1) + 2*S*S4**(-1) + 4*T1**(-1)*U1*
     +    S4**(-1)*MS2 )
     +
      M2QGH4 = M2QGH4 + N*SYMBU*(S4+MS2)*CF * (  - 8*T1**(-1)*U1*
     +    S4**(-1)*MG2 + 12*T1**(-1)*U1**2*S4**(-1) - 24*T1**(-1)*
     +    S4**(-1)*MS2*MG2 + 16*T1**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) - 8*T1**(-1)*S4**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 32
     +    *T1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 8*T1**(-1)*
     +    S4**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 32*T1**(-1)*S4**(-1)*
     +    MS2**2 + 16*T1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) + 8*
     +    T1**(-1)*S4**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*T1**(-1)*
     +    S4**(-1)*MG2**2 + 8*T1**(-1)*S4**(-1)*MG2**3*(S+U1+M2)**(-1)
     +     - 12*T1*U1*S4**(-1)*(T1+U1+M2)**(-1) + 16*T1*S4**(-1)*MS2*
     +    MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 4*T1*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1) - 4*T1*S4**(-1)*MS2*(T1+U1+M2)**(-1) - 12*T1*
     +    S4**(-1)*MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 8*T1*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1) + 4*T1*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 4*T1*S4**(-1)*MG2**2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*SYMBU*(S4+MS2)*CF * (  - 2*T1*S4**(-1) - 2*
     +    T1**2*S4**(-1)*(T1+U1+M2)**(-1) - 4*U1*S4**(-1)*MS2*
     +    (T1+U1+M2)**(-1) + 4*U1*S4**(-1)*MG2*(T1+U1+M2)**(-1) + 10*U1
     +    *S4**(-1) - 10*U1**2*S4**(-1)*(T1+U1+M2)**(-1) - 16*S4**(-1)*
     +    MS2*MG2*SYMBX*(U1+M2) + 8*S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 
     +    4*S4**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 2*S4**(-1)*MS2 + 12*
     +    S4**(-1)*MS2**2*SYMBX*(U1+M2) + 8*S4**(-1)*MS2**2*
     +    (S+U1+M2)**(-1) - 6*S4**(-1)*MS2**2*(T1+U1+M2)**(-1) + 2*
     +    S4**(-1)*MG2 + 4*S4**(-1)*MG2**2*SYMBX*(U1+M2) - 16*S4**(-1)*
     +    MG2**2*(S+U1+M2)**(-1) + 2*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * ( 16*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*MG2 - 16*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MS2*
     +    MG2**2*SYMBX*(U1+M2) - 8*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*
     +    MS2*MG2**2*(S+U1+M2)**(-1) + 32*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 8*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 8*S**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 16*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) + 8*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*S**(-1)*T1**(-1)
     +    *U1**(-1)*S4**(-1)*MG2**2 + 8*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2**3*(S+U1+M2)**(-1) - 16*S**(-1)*T1**(-1)*U1*
     +    S4**(-2)*MS2 + 16*S**(-1)*T1**(-1)*U1*S4**(-2)*MG2 - 4*
     +    S**(-1)*T1**(-1)*U1*S4**(-1) - 16*S**(-1)*T1**(-1)*U1**2*
     +    S4**(-2) - 20*S**(-1)*T1**(-1)*U1**2*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 32*S**(-1)*T1**(-1)*S4**(-2)*MS2*MG2 - 48*
     +    S**(-1)*T1**(-1)*S4**(-2)*MS2*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * ( 48*S**(-1)*T1**(-1)*S4**(-2)*
     +    MS2**2*MG2*(S+U1+M2)**(-1) - 16*S**(-1)*T1**(-1)*S4**(-2)*
     +    MS2**2 - 16*S**(-1)*T1**(-1)*S4**(-2)*MS2**3*(S+U1+M2)**(-1)
     +     - 16*S**(-1)*T1**(-1)*S4**(-2)*MG2**2 + 16*S**(-1)*T1**(-1)*
     +    S4**(-2)*MG2**3*(S+U1+M2)**(-1) - 8*S**(-1)*T1**(-1)*S4**(-1)
     +    *MS2*MG2*SYMBX*(U1+M2) + 16*S**(-1)*T1**(-1)*S4**(-1)*MS2*MG2
     +    *(S+U1+M2)**(-1) - 4*S**(-1)*T1**(-1)*S4**(-1)*MS2 + 4*
     +    S**(-1)*T1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) - 8*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2**2*(S+U1+M2)**(-1) + 12*S**(-1)*
     +    T1**(-1)*S4**(-1)*MG2 + 4*S**(-1)*T1**(-1)*S4**(-1)*MG2**2*
     +    SYMBX*(U1+M2) - 8*S**(-1)*T1**(-1)*S4**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) - 4*S**(-1)*T1*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S**(-1)*T1*S4**(-2)*MS2*
     +    (S+U1+M2)**(-1) - 16*S**(-1)*T1*S4**(-2)*MG2*(S+U1+M2)**(-1)
     +     + 16*S**(-1)*T1*S4**(-2) - 4*S**(-1)*T1*S4**(-1)*MS2*SYMBX*
     +    (U1+M2)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 4*S**(-1)*T1*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*T1*S4**(-1)*MG2*SYMBX*
     +    (U1+M2)*(T1+U1+M2)**(-1) + 4*S**(-1)*T1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 12*S**(-1)*T1*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*U1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 4*S**(-1)*U1**(-1)*S4**(-1)*MG2 - 4*S**(-1)
     +    *U1**(-1)*S4**(-1)*MG2**2*(S+U1+M2)**(-1) + 32*S**(-1)*U1*
     +    S4**(-2)*(S+T1)**(-1)*MS2 - 32*S**(-1)*U1*S4**(-2)*
     +    (S+T1)**(-1)*MG2 + 32*S**(-1)*U1*S4**(-2) - 8*S**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 8*S**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1) + 8*S**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 24*S**(-1)*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 32*S**(-1)*U1**2*S4**(-2)*(S+T1)**(-1) + 8
     +    *S**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*(T1+U1+M2)**(-1) - 64*
     +    S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2*MG2 + 96*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 96*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 32*S**(-1)*S4**(-2)
     +    *(S+T1)**(-1)*MS2**2 + 32*S**(-1)*S4**(-2)*(S+T1)**(-1)*
     +    MS2**3*(S+U1+M2)**(-1) + 32*S**(-1)*S4**(-2)*(S+T1)**(-1)*
     +    MG2**2 - 32*S**(-1)*S4**(-2)*(S+T1)**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) - 64*S**(-1)*S4**(-2)*MS2*MG2*(S+U1+M2)**(-1)
     +     + 32*S**(-1)*S4**(-2)*MS2 + 32*S**(-1)*S4**(-2)*MS2**2*
     +    (S+U1+M2)**(-1) - 32*S**(-1)*S4**(-2)*MG2 + 32*S**(-1)*
     +    S4**(-2)*MG2**2*(S+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 + 8*S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*S4**(-1)*MS2*SYMBX*
     +    (U1+M2) - 8*S**(-1)*S4**(-1)*MS2*(S+U1+M2)**(-1) - 4*S**(-1)*
     +    S4**(-1)*MS2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 4*S**(-1)*S4**(-1)*MS2**2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*S4**(-1)*MG2*SYMBX*
     +    (U1+M2) + 8*S**(-1)*S4**(-1)*MG2*(S+U1+M2)**(-1) - 4*S**(-1)*
     +    S4**(-1)*MG2*(T1+U1+M2)**(-1) - 4*S**(-1)*S4**(-1)*MG2**2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 8*S**(-1)*S4**(-1) - 16*S*
     +    T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 16*S*
     +    T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*(S+U1+M2)**(-1) + 72*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 16*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 - 32*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) + 24*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 - 40*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) - 16*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*(S+U1+M2)**(-1) + 24*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 8*S*T1**(-1)*U1**(-1)*
     +    S4**(-1) + 4*S*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) - 8*S*T1**(-1)*S4**(-2) - 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) + 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) - 4*S*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 4*S*T1**(-1)*
     +    S4**(-1)*MG2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 4*S
     +    *T1*S4**(-1)*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 4*S*
     +    U1**(-1)*S4**(-1)*(S+U1+M2)**(-1) - 8*S*U1*S4**(-1)*
     +    (S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 48*S*S4**(-2)*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) - 64*S*S4**(-2)*(S+T1)**(-1)
     +    *MG2*(S+U1+M2)**(-1) + 16*S*S4**(-2)*(S+T1)**(-1) + 16*S*
     +    S4**(-2)*(S+U1+M2)**(-1) + 72*S*S4**(-1)*(S+T1)**(-1)*MS2*MG2
     +    *(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 32*S*S4**(-1)*(S+T1)**(-1)
     +    *MS2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 24*S*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 40*S*
     +    S4**(-1)*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 2*S*S4**(-1)*(S+U1)**(-1)*
     +    (T1+U1+M2)**(-1) - 20*S*S4**(-1)*MS2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 24*S*S4**(-1)*MG2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 24*S**2*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) - 32*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) + 8*S**2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1) + 8*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1+M2)**(-1) - 16*S**2*S4**(-2)*(S+T1)**(-1)*
     +    (S+U1+M2)**(-1) + 24*S**2*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 32*S**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * ( 8*S**2*S4**(-1)*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**2*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**3*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*(S+U1+M2)**(-1) - 8*S**3*S4**(-1)*(S+T1)**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-2)*
     +    U1**(-1)*S4**(-1)*MS2*MG2 - 16*T1**(-2)*U1**(-1)*S4**(-1)*MS2
     +    *MG2**2*SYMBX*(U1+M2) - 16*T1**(-2)*U1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1) + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2
     +    *MG2*SYMBX*(U1+M2) + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*MG2
     +    *(S+U1+M2)**(-1) - 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 - 16*
     +    T1**(-2)*U1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) - 16*T1**(-2)
     +    *U1**(-1)*S4**(-1)*MS2**3*(S+U1+M2)**(-1) - 16*T1**(-2)*
     +    S4**(-1)*MS2*MG2*SYMBX*(U1+M2) - 16*T1**(-2)*S4**(-1)*MS2*MG2
     +    *(S+U1+M2)**(-1) + 16*T1**(-2)*S4**(-1)*MS2**2*SYMBX*(U1+M2)
     +     + 16*T1**(-2)*S4**(-1)*MS2**2*(S+U1+M2)**(-1) - 32*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2 )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * ( 48*T1**(-1)*U1**(-1)*S4**(-1)
     +    *(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 48*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 
     +    16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2 + 16*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) + 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MG2**2 - 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) - 24*T1**(-1)*U1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 8*T1**(-1)*U1**(-1)*S4**(-1)*MS2 - 16*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2 + 24*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2**2*(S+U1+M2)**(-1) - 24*T1**(-1)*U1*S4**(-2) + 8
     +    *T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 24*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) - 12*T1**(-1)*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 8*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) - 8*T1**(-1)*S4**(-2)*MS2*MG2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 16*T1**(-1)*S4**(-2)*MS2
     +     + 8*T1**(-1)*S4**(-2)*MG2 + 8*T1**(-1)*S4**(-2)*MG2**2*
     +    (S+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 24*T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 - 8*
     +    T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*(T1+U1+M2)**(-1) - 16*
     +    T1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 + 16*T1**(-1)*S4**(-1)*MS2
     +    *MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*MS2
     +    *MG2*(M2*(S+T1)+T1*U1)**(-1) - 24*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-1)*
     +    S4**(-1)*MS2*SYMBX*(U1+M2) + 8*T1**(-1)*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1) - 4*T1**(-1)*S4**(-1)*MS2*(T1+U1+M2)**(-1) + 
     +    24*T1**(-1)*S4**(-1)*MS2**2*MG2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*S4**(-1)*MS2**2*SYMBX*
     +    (U1+M2)*(T1+U1+M2)**(-1) - 8*T1**(-1)*S4**(-1)*MS2**3*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-1)*S4**(-1)
     +    *MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 8*T1**(-1)*S4**(-1)*MG2**2
     +    *(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-1)*S4**(-1)*MG2**3*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*
     +    S4**(-1) - 8*T1*S4**(-2)*(S+U1+M2)**(-1) + 2*T1*S4**(-1)*
     +    (S+U1)**(-1)*(T1+U1+M2)**(-1) + 8*T1*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*T1*S4**(-1)*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 4*T1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*U1**(-1)*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1) - 8*U1**(-1)*S4**(-1)*MG2*(S+U1+M2)**(-1) + 4
     +    *U1**(-1)*S4**(-1) - 16*U1*S4**(-2)*(S+T1)**(-1) - 16*U1*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 12*U1*
     +    S4**(-1)*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 8*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 144*S4**(-2)*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 32*S4**(-2)*(S+T1)**(-1)*MS2 - 64*S4**(-2)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) + 48*S4**(-2)*
     +    (S+T1)**(-1)*MG2 )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 80*S4**(-2)*(S+T1)**(-1)*
     +    MG2**2*(S+U1+M2)**(-1) - 40*S4**(-2)*MS2*(S+U1+M2)**(-1) + 48
     +    *S4**(-2)*MG2*(S+U1+M2)**(-1) - 16*S4**(-2) - 32*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 48*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 48*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*
     +    (S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     + 4*S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 16*S4**(-1)
     +    *(S+T1)**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     + 4*S4**(-1)*(S+T1)**(-1) - 2*S4**(-1)*(S+U1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) + 2*S4**(-1)*(S+U1)**(-1)*MG2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*(S4+MS2)*CF * (  - 2*S4**(-1)*(S+U1)**(-1) - 
     +    32*S4**(-1)*MS2*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     - 8*S4**(-1)*MS2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 12*
     +    S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MS2**2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 4*S4**(-1)*(S+U1+M2)**(-1) + 2*
     +    S4**(-1)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + N*CF * (  - 2*S**(-1)*T1*(T1+U1+M2)**(-1) - 2*
     +    S**(-1)*U1*(T1+U1+M2)**(-1) + 2*S**(-1)*MS2*(T1+U1+M2)**(-1)
     +     - 2*S**(-1)*MG2*(T1+U1+M2)**(-1) + 2*S**(-1) )
     +
      M2QGH4 = M2QGH4 + SYMBT*(S4+MS2)*CF**2 * (  - 32*S**(-1)*T1*U1*
     +    S4**(-1)*MG2*(T1+U1+M2)**(-1) - 8*S**(-1)*T1*U1**2*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 16*S**(-1)*T1*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 24*S**(-1)*T1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*T1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) - 16*S**(-1)*T1**2*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 16*S**(-1)*T1**2*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*T1**3*S4**(-1)*(T1+U1+M2)**(-1)
     +     + 16*S**(-1)*U1*S4**(-1)*MS2**2*(T1+U1+M2)**(-1) - 16*
     +    S**(-1)*U1*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) - 16*S**(-1)*
     +    U1**2*S4**(-1)*MG2*(T1+U1+M2)**(-1) - 16*S**(-1)*S4**(-1)*MS2
     +    *MG2**2*(T1+U1+M2)**(-1) + 32*S**(-1)*S4**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) - 16*S**(-1)*S4**(-1)*MS2**3*
     +    (T1+U1+M2)**(-1) - 8*S*U1*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 8*S*U1**2*S4**(-1)*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + SYMBT*(S4+MS2)*CF**2 * ( 8*T1*S4**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 16*T1*S4**(-1)*MG2*(T1+U1+M2)**(-1) - 8*
     +    T1**2*S4**(-1)*(T1+U1+M2)**(-1) + 16*U1*S4**(-1)*(S+T1)**(-1)
     +    *MS2*MG2*(T1+U1+M2)**(-1) - 16*U1*S4**(-1)*(S+T1)**(-1)*
     +    MS2**2*(T1+U1+M2)**(-1) + 8*U1*S4**(-1)*MS2*(T1+U1+M2)**(-1)
     +     + 16*U1**2*S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 8*
     +    U1**2*S4**(-1)*(T1+U1+M2)**(-1) + 8*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 8*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + SYMBU*(S4+MS2)*CF**2 * ( 8*S**(-1)*T1**(-1)*U1*
     +    S4**(-1)*MS2*MG2 - 8*S**(-1)*T1**(-1)*U1*S4**(-1)*MS2**2 + 8*
     +    S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 - 8*S**(-1)*T1**(-1)*
     +    U1**3*S4**(-1) - 8*S**(-1)*T1*U1*S4**(-1) + 8*S**(-1)*T1*
     +    S4**(-1)*MS2*MG2*SYMBX*(U1+M2) + 8*S**(-1)*T1*S4**(-1)*MG2 - 
     +    8*S**(-1)*T1*S4**(-1)*MG2**2*SYMBX*(U1+M2) + 16*S**(-1)*U1*
     +    S4**(-1)*MG2 - 16*S**(-1)*U1**2*S4**(-1) + 32*S**(-1)*
     +    S4**(-1)*MS2*MG2 - 48*S**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) + 48*S**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 16*
     +    S**(-1)*S4**(-1)*MS2**2 - 16*S**(-1)*S4**(-1)*MS2**3*SYMBX*
     +    (U1+M2) - 16*S**(-1)*S4**(-1)*MG2**2 + 16*S**(-1)*S4**(-1)*
     +    MG2**3*SYMBX*(U1+M2) + 32*S*T1**(-1)*U1**(-1)*S4**(-1)*MS2*
     +    MG2 - 32*S*T1**(-1)*U1**(-1)*S4**(-1)*MS2**2 + 96*S*T1**(-1)*
     +    U1*S4**(-1)*MS2*SYMBX*(U1+M2) - 8*S*T1**(-1)*U1*S4**(-1) + 32
     +    *S*T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) - 72*S*T1**(-1)*
     +    S4**(-1)*MS2 )
     +
      M2QGH4 = M2QGH4 + SYMBU*(S4+MS2)*CF**2 * (  - 32*S*T1**(-1)*
     +    S4**(-1)*MS2**2*SYMBX*(U1+M2) + 8*S*T1**(-1)*S4**(-1)*MG2 - 
     +    72*T1**(-1)*U1*S4**(-1)*MS2 + 16*T1**(-1)*U1*S4**(-1)*MG2 - 
     +    64*T1**(-1)*U1*MS2*SYMBX*(U1+M2) + 96*T1**(-1)*U1**2*S4**(-1)
     +    *MS2*SYMBX*(U1+M2) - 16*T1**(-1)*U1**2*S4**(-1) + 72*T1**(-1)
     +    *S4**(-1)*MS2*MG2 - 32*T1**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) + 64*T1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 72*
     +    T1**(-1)*S4**(-1)*MS2**2 - 32*T1**(-1)*S4**(-1)*MS2**3*SYMBX*
     +    (U1+M2) + 32*T1**(-1)*MS2 + 64*U1*S4**(-1)*MS2*SYMBX*(U1+M2)
     +     - 16*U1*S4**(-1) + 64*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) - 40*
     +    S4**(-1)*MS2 - 48*S4**(-1)*MS2**2*SYMBX*(U1+M2) + 16*S4**(-1)
     +    *MG2 - 16*S4**(-1)*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 64*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2*MG2 + 96*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*MG2**2*SYMBX*(U1+M2) - 96*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 32*S**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2**2 + 32*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) + 32*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MG2**2 - 32*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2**3*SYMBX*(U1+M2) + 32*S**(-1)*T1**(-1)*U1*
     +    S4**(-2)*MS2 - 32*S**(-1)*T1**(-1)*U1*S4**(-2)*MG2 + 24*
     +    S**(-1)*T1**(-1)*U1*S4**(-1) + 32*S**(-1)*T1**(-1)*U1**2*
     +    S4**(-2) - 64*S**(-1)*T1**(-1)*S4**(-2)*MS2*MG2 + 96*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2*MG2**2*(S+U1+M2)**(-1) - 96*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2**2*MG2*(S+U1+M2)**(-1) + 32*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2**2 + 32*S**(-1)*T1**(-1)*S4**(-2)*
     +    MS2**3*(S+U1+M2)**(-1) + 32*S**(-1)*T1**(-1)*S4**(-2)*MG2**2
     +     - 32*S**(-1)*T1**(-1)*S4**(-2)*MG2**3*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * ( 24*S**(-1)*T1**(-1)*S4**(-1)
     +    *MS2 - 32*S**(-1)*T1**(-1)*S4**(-1)*MG2 + 8*S**(-1)*T1*U1*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*T1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*T1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 40*S**(-1)*U1**(-1)*S4**(-1)*MS2*MG2*SYMBX
     +    *(U1+M2) + 16*S**(-1)*U1**(-1)*S4**(-1)*MS2 + 16*S**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) - 24*S**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 + 24*S**(-1)*U1**(-1)*S4**(-1)*MG2**2*SYMBX*
     +    (U1+M2) - 32*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MS2 + 32*
     +    S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MG2 + 16*S**(-1)*U1*S4**(-1)
     +    *(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 16*S**(-1)*U1*S4**(-1)*
     +    (S+T1)**(-1) - 16*S**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 32*S**(-1)*U1**2*S4**(-2)*(S+T1)**(-1) + 
     +    64*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2*MG2 )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 96*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 96*S**(-1)*S4**(-2)
     +    *(S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 32*S**(-1)*
     +    S4**(-2)*(S+T1)**(-1)*MS2**2 - 32*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) - 32*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MG2**2 + 32*S**(-1)*S4**(-2)*(S+T1)**(-1)*MG2**3
     +    *(S+U1+M2)**(-1) + 16*S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 16*S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 - 16*
     +    S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*(T1+U1+M2)**(-1) - 16*
     +    S**(-1)*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*
     +    S4**(-1)*MS2*(T1+U1+M2)**(-1) + 16*S**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**(-1)*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 32*S**(-1)*S4**(-1) + 32*S*T1**(-2)*
     +    U1**(-2)*S4**(-1)*MS2*MG2 - 32*S*T1**(-2)*U1**(-2)*S4**(-1)*
     +    MS2*MG2**2*SYMBX*(U1+M2) + 64*S*T1**(-2)*U1**(-2)*S4**(-1)*
     +    MS2**2*MG2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 32*S*T1**(-2)*U1**(-2)*
     +    S4**(-1)*MS2**2 - 32*S*T1**(-2)*U1**(-2)*S4**(-1)*MS2**3*
     +    SYMBX*(U1+M2) - 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2*SYMBX
     +    *(U1+M2) + 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*
     +    SYMBX*(U1+M2) - 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*
     +    (S+U1+M2)**(-1) - 72*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 16*S*T1**(-1)*U1**(-1)
     +    *S4**(-1)*(S+T1)**(-1)*MS2 + 32*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 24*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 + 40*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) + 8*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*(S+U1+M2)**(-1) - 16*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1) + 8*S*T1**(-1)*U1**(-1)*S4**(-1)
     +     + 16*S*T1**(-1)*S4**(-2) - 8*S*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 8*S*T1**(-1)*S4**(-1)*MS2
     +    *MG2*SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) + 8*S*T1**(-1)*
     +    S4**(-1)*MS2*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*
     +    S*T1**(-1)*S4**(-1)*MG2**2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S*T1**(-1)*S4**(-1)*MG2**2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S*U1*S4**(-1)*
     +    (S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 48*S*S4**(-2)*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 64*S*S4**(-2)*(S+T1)**(-1)
     +    *MG2*(S+U1+M2)**(-1) - 16*S*S4**(-2)*(S+T1)**(-1) - 16*S*
     +    S4**(-2)*(S+U1+M2)**(-1) - 72*S*S4**(-1)*(S+T1)**(-1)*MS2*MG2
     +    *(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 16*S*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 32*S*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     - 24*S*S4**(-1)*(S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 
     +    40*S*S4**(-1)*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * ( 16*S*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S*S4**(-1)*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 24*S**2*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 32*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) - 8*S**2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1) - 8*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1+M2)**(-1) + 16*S**2*S4**(-2)*(S+T1)**(-1)*
     +    (S+U1+M2)**(-1) - 24*S**2*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 32*S**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*
     +    S**2*S4**(-1)*(S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S**2*
     +    S4**(-1)*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S**3*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*(S+U1+M2)**(-1) + 8*
     +    S**3*S4**(-1)*(S+T1)**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 32*T1**(-2)*U1**(-1)*
     +    S4**(-1)*MS2*MG2 + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) - 64*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 + 32*
     +    T1**(-2)*U1**(-1)*S4**(-1)*MS2**3*(S+U1+M2)**(-1) + 32*
     +    T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2*MG2 - 32*T1**(-2)*S4**(-1)
     +    *(S+U1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 64*T1**(-2)*
     +    S4**(-1)*(S+U1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 32*
     +    T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2**2 - 32*T1**(-2)*S4**(-1)*
     +    (S+U1)**(-1)*MS2**3*(S+U1+M2)**(-1) + 32*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2*MG2 - 48*T1**(-1)*U1**(-1)*S4**(-1)
     +    *(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 48*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 
     +    16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2 - 16*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 16*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2**2 + 16*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) - 72*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*MG2*SYMBX*(U1+M2) + 8*T1**(-1)*U1**(-1)*S4**(-1)
     +    *MS2*MG2*(S+U1+M2)**(-1) + 16*T1**(-1)*U1**(-1)*S4**(-1)*MS2
     +     + 48*T1**(-1)*U1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) - 16*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2 + 24*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2**2*SYMBX*(U1+M2) - 8*T1**(-1)*U1**(-1)*S4**(-1)*
     +    MG2**2*(S+U1+M2)**(-1) + 48*T1**(-1)*U1*S4**(-2) - 48*
     +    T1**(-1)*U1*S4**(-1)*(S+T1)**(-1) + 16*T1**(-1)*S4**(-2)*MS2*
     +    MG2*(S+U1+M2)**(-1) + 32*T1**(-1)*S4**(-2)*MS2 - 16*T1**(-1)*
     +    S4**(-2)*MG2 - 16*T1**(-1)*S4**(-2)*MG2**2*(S+U1+M2)**(-1) - 
     +    48*T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 + 16*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 + 24*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 16*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2 - 16*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * ( 8*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 - 8*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) - 32*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 48*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) + 48*T1**(-1)*S4**(-1)*
     +    MS2*MG2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 48*
     +    T1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 48*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*
     +    S4**(-1)*MS2**3*SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) + 16*
     +    T1**(-1)*S4**(-1)*MS2**3*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 32*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*S4**(-1)*MG2**3*SYMBX*
     +    (U1+M2)*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*S4**(-1)*MG2**3
     +    *(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 16*U1*S4**(-2)*
     +    (S+T1)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * ( 16*U1*S4**(-1)*(S+T1)**(-1)*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*U1*S4**(-1)*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) - 8*U1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 
     +    144*S4**(-2)*(S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 32*
     +    S4**(-2)*(S+T1)**(-1)*MS2 + 64*S4**(-2)*(S+T1)**(-1)*MS2**2*
     +    (S+U1+M2)**(-1) - 48*S4**(-2)*(S+T1)**(-1)*MG2 + 80*S4**(-2)*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) + 32*S4**(-2)*MS2*
     +    (S+U1+M2)**(-1) - 32*S4**(-2)*MG2*(S+U1+M2)**(-1) + 32*
     +    S4**(-2) + 32*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 48*S4**(-1)*(S+T1)**(-1)*MS2*MG2**2
     +    *(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) + 48*S4**(-1)*(S+T1)**(-1)*
     +    MS2**2*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*
     +    S4**(-1)*(S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) - 16*
     +    S4**(-1)*(S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + (S4+MS2)*CF**2 * (  - 16*S4**(-1)*(S+T1)**(-1)*
     +    MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*(S+T1)**(-1)*
     +    MG2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*
     +    (S+T1)**(-1) - 32*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MS2**2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MG2**2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(16)*CF**2 * ( 8*S**(-1)*MS2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(18)*N*CF * ( 2*S**(-1)*T1*U1*MS2*
     +    (T1+U1+M2)**(-1) + 10*S**(-1)*T1*U1**2*(T1+U1+M2)**(-1) + 8*
     +    S**(-1)*T1**2*U1*(T1+U1+M2)**(-1) + 2*S**(-1)*T1**2*MS2*
     +    (T1+U1+M2)**(-1) + 2*S**(-1)*T1**3*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*U1**3*(T1+U1+M2)**(-1) - 8*S*T1**(-1)*MS2*MG2*SYMBX*
     +    (U1+M2) + 8*S*T1**(-1)*MS2 + 8*S*T1**(-1)*MS2**2*SYMBX*
     +    (U1+M2) - 2*S*T1*MS2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 2*S*T1*
     +    MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - S*T1*SYMBX*(U1+M2) + S*
     +    T1*(T1+U1+M2)**(-1) + S*T1**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1)
     +     + 4*S*U1*(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) - 2*S*U1*
     +    (T1+U1+M2)**(-1) + 4*S*U1**2*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 
     +    8*S*MS2*MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 2*S*MS2*SYMBX*
     +    (U1+M2) - 8*S*MS2*(T1+U1+M2)**(-1) - 8*S*MS2**2*SYMBX*(U1+M2)
     +    *(T1+U1+M2)**(-1) + 2*S*MG2*SYMBX*(U1+M2) - 2*S + T1*U1*
     +    (T1+U1+M2)**(-1) - 3*T1*MS2*SYMBX*(U1+M2) - 8*T1*MS2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(18)*N*CF * (  - 2*T1*MS2**2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + 3*T1*MG2*SYMBX*(U1+M2) - 2*T1*MG2*
     +    (T1+U1+M2)**(-1) + 2*T1*MG2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1)
     +     - 3*T1 + 3*T1**2*MS2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 3*
     +    T1**2*MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - T1**2*SYMBX*
     +    (U1+M2) + 4*T1**2*(T1+U1+M2)**(-1) + T1**3*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) - 16*U1*MS2*(T1+U1+M2)**(-1) - 2*U1 - 10*
     +    U1**2*(T1+U1+M2)**(-1) - 8*U1**3*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) + 2*MS2 + 2*MS2**2*SYMBX*(U1+M2) + 2*MG2 - 2
     +    *MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(18)*CF**2 * (  - 4*S**(-1)*T1*U1*MS2*
     +    (T1+U1+M2)**(-1) - 20*S**(-1)*T1*U1**2*(T1+U1+M2)**(-1) - 16*
     +    S**(-1)*T1**2*U1*(T1+U1+M2)**(-1) - 4*S**(-1)*T1**2*MS2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1**3*(T1+U1+M2)**(-1) - 8*
     +    S**(-1)*U1**3*(T1+U1+M2)**(-1) - 4*S*U1*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 4*S*U1**2*(S+T1)**(-1)*(T1+U1+M2)**(-1) - 
     +    8*T1*U1*(T1+U1+M2)**(-1) + 8*T1*MS2*(T1+U1+M2)**(-1) - 4*
     +    T1**2*(T1+U1+M2)**(-1) + 20*U1*MS2*(T1+U1+M2)**(-1) + 4*U1**2
     +    *(T1+U1+M2)**(-1) + 8*U1**3*(S+T1)**(-1)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(20)*N*CF * (  - T1*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + SYMBX*(U1+M2) - (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(25)*CF**2 * ( 16*MS2*MG2 - 16*MS2*MG2**2*
     +    SYMBX*(U1+M2) + 32*MS2**2*MG2*SYMBX*(U1+M2) - 16*MS2**3*SYMBX
     +    *(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(27)*N*CF * (  - 2*S**(-1)*T1*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 2*S**(-1)*T1*MG2**2*(T1+U1+M2)**(-1) - 4*
     +    S**(-1)*U1*MS2*MG2*(T1+U1+M2)**(-1) + 4*S**(-1)*U1*MG2**2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*MS2*MG2 + 8*S**(-1)*MS2*MG2**2*
     +    SYMBX*(U1+M2) + 4*S**(-1)*MS2*MG2**2*(T1+U1+M2)**(-1) - 16*
     +    S**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 8*S**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*MS2**3*SYMBX*(U1+M2) + 4*S**(-1)
     +    *MS2**3*(T1+U1+M2)**(-1) + 2*S*T1**(-1)*U1 - 8*S*T1**(-1)*MS2
     +    *MG2*SYMBX*(U1+M2) + 6*S*T1**(-1)*MS2 + 8*S*T1**(-1)*MS2**2*
     +    SYMBX*(U1+M2) - 2*S*T1**(-1)*MG2 - 4*S*U1*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 2*S*U1*(T1+U1+M2)**(-1) - 4*S*U1**2*
     +    (S+T1)**(-1)*(T1+U1+M2)**(-1) + 8*S*MS2*MG2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) - 6*S*MS2*(T1+U1+M2)**(-1) - 8*S*MS2**2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 2*S*MG2*(T1+U1+M2)**(-1) - 4
     +    *T1**(-1)*U1*MG2 - 12*T1**(-1)*MS2*MG2 + 8*T1**(-1)*MS2*
     +    MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(27)*N*CF * (  - 16*T1**(-1)*MS2**2*MG2*
     +    SYMBX*(U1+M2) + 4*T1**(-1)*MS2**2 + 8*T1**(-1)*MS2**3*SYMBX*
     +    (U1+M2) + 2*T1*U1*(T1+U1+M2)**(-1) + 8*T1*MS2*MG2*SYMBX*
     +    (U1+M2)*(T1+U1+M2)**(-1) + 2*T1*MS2*(T1+U1+M2)**(-1) - 6*T1*
     +    MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 2*T1*MG2**2*SYMBX*
     +    (U1+M2)*(T1+U1+M2)**(-1) + 8*U1*(S+T1)**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 8*U1*(S+T1)**(-1)*MS2**2*(T1+U1+M2)**(-1)
     +     - 2*U1*MS2*(T1+U1+M2)**(-1) + 6*U1*MG2*(T1+U1+M2)**(-1) - 2*
     +    U1 + 8*U1**2*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 2*U1**2*
     +    (T1+U1+M2)**(-1) - 8*MS2*MG2*SYMBX*(U1+M2) + 16*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 2*MS2 + 6*MS2**2*SYMBX*(U1+M2) - 14*MS2**2
     +    *(T1+U1+M2)**(-1) + 2*MG2**2*SYMBX*(U1+M2) - 2*MG2**2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(27)*CF**2 * ( 4*S**(-1)*T1*MS2*MG2*SYMBX*
     +    (U1+M2) + 4*S**(-1)*T1*MS2*MG2*(T1+U1+M2)**(-1) - 4*S**(-1)*
     +    T1*MG2**2*SYMBX*(U1+M2) - 4*S**(-1)*T1*MG2**2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*U1*MS2*MG2*(T1+U1+M2)**(-1) - 8*
     +    S**(-1)*U1*MG2**2*(T1+U1+M2)**(-1) + 8*S**(-1)*MS2*MG2 - 24*
     +    S**(-1)*MS2*MG2**2*SYMBX*(U1+M2) - 8*S**(-1)*MS2*MG2**2*
     +    (T1+U1+M2)**(-1) + 24*S**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 16*
     +    S**(-1)*MS2**2*MG2*(T1+U1+M2)**(-1) - 8*S**(-1)*MS2**3*SYMBX*
     +    (U1+M2) - 8*S**(-1)*MS2**3*(T1+U1+M2)**(-1) - 8*S**(-1)*
     +    MG2**2 + 8*S**(-1)*MG2**3*SYMBX*(U1+M2) - 4*S*T1**(-1)*U1 + 
     +    16*S*T1**(-1)*MS2*MG2*SYMBX*(U1+M2) - 12*S*T1**(-1)*MS2 - 16*
     +    S*T1**(-1)*MS2**2*SYMBX*(U1+M2) + 4*S*T1**(-1)*MG2 + 4*S*U1*
     +    (S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) + 4*S*U1**2*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) + 8*T1**(-1)*U1*MG2 + 24*T1**(-1)*MS2*MG2 - 
     +    16*T1**(-1)*MS2*MG2**2*SYMBX*(U1+M2) + 32*T1**(-1)*MS2**2*MG2
     +    *SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(27)*CF**2 * (  - 8*T1**(-1)*MS2**2 - 16*
     +    T1**(-1)*MS2**3*SYMBX*(U1+M2) - 4*T1*U1*(T1+U1+M2)**(-1) - 4*
     +    T1*MS2*(T1+U1+M2)**(-1) + 4*T1*MG2*(T1+U1+M2)**(-1) - 8*U1*
     +    (S+T1)**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 8*U1*(S+T1)**(-1)*
     +    MS2**2*(T1+U1+M2)**(-1) + 4*U1*MS2*(T1+U1+M2)**(-1) + 4*U1 - 
     +    8*U1**2*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 4*U1**2*
     +    (T1+U1+M2)**(-1) + 32*MS2*MG2*SYMBX*(U1+M2) - 8*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 16*MS2*MG2**2*SYMBX + 32*MS2**2*MG2*SYMBX
     +     - 24*MS2**2*SYMBX*(U1+M2) + 8*MS2**2*(T1+U1+M2)**(-1) - 16*
     +    MS2**3*SYMBX + 4*MG2 - 8*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(31)*N*CF * ( 2*S**(-1)*T1*MS2*MG2*
     +    (S+U1+M2)**(-1) - 2*S**(-1)*T1*MS2*MG2*(T1+U1+M2)**(-1) - 2*
     +    S**(-1)*T1*MG2**2*(S+U1+M2)**(-1) + 2*S**(-1)*T1*MG2**2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*U1*MS2*MG2*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*U1*MG2**2*(T1+U1+M2)**(-1) - 4*S**(-1)*MS2*MG2 - 4*
     +    S**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 4*S**(-1)*MS2*MG2**2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 8*
     +    S**(-1)*MS2**2*MG2*(T1+U1+M2)**(-1) + 4*S**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) + 4*S**(-1)*MS2**3*(T1+U1+M2)**(-1) - 4*
     +    S**(-1)*MG2**2 + 4*S**(-1)*MG2**3*(S+U1+M2)**(-1) + 2*S*
     +    T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 2*S*T1**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 2*S*T1**(-1)*MG2**2*(S+U1+M2)**(-1) - 2*S*
     +    T1**(-1)*MG2**2*(T1+U1+M2)**(-1) + 2*S*MG2*(S+U1+M2)**(-1) - 
     +    2*S*MG2*(T1+U1+M2)**(-1) + 4*T1**(-1)*U1*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 4*T1**(-1)*U1*MG2**2*(T1+U1+M2)**(-1) + 4*
     +    T1**(-1)*MS2*MG2 )
     +
      M2QGH4 = M2QGH4 + ANG4(31)*N*CF * ( 4*T1**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) - 4*T1**(-1)*MS2*MG2**2*(T1+U1+M2)**(-1) + 4*
     +    T1**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 8*T1**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) - 4*T1**(-1)*MS2**3*(S+U1+M2)**(-1) - 4*
     +    T1**(-1)*MS2**3*(T1+U1+M2)**(-1) + 4*T1**(-1)*MG2**2 - 4*
     +    T1**(-1)*MG2**3*(S+U1+M2)**(-1) - 2*T1*MG2*(S+U1+M2)**(-1) - 
     +    2*T1*MG2*(T1+U1+M2)**(-1) - 4*U1*MG2*(T1+U1+M2)**(-1) - 4*MS2
     +    *MG2*(S+U1+M2)**(-1) + 4*MS2*MG2*(T1+U1+M2)**(-1) - 4*MS2**2*
     +    (S+U1+M2)**(-1) - 4*MS2**2*(T1+U1+M2)**(-1) - 4*MG2 + 8*
     +    MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(34)*CF**2 * ( 8*S**(-1)*MS2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(35)*N*CF * (  - 2*S**(-1)*T1*MS2*MG2*
     +    (S+U1+M2)**(-1) + 2*S**(-1)*T1*MG2**2*(S+U1+M2)**(-1) + 4*
     +    S**(-1)*MS2*MG2 + 4*S**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 4*
     +    S**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 4*S**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) + 4*S**(-1)*MG2**2 - 4*S**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) - 2*S*T1**(-1)*U1 + 8*S*T1**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 6*S*T1**(-1)*MS2 - 8*S*T1**(-1)*MS2**2*
     +    (S+U1+M2)**(-1) - 2*S*T1**(-1)*MG2 + 2*S*T1*(S+U1+M2)**(-1)
     +     + 4*S*U1*(S+T1)**(-1) - 36*S*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 8*S*(S+T1)**(-1)*MS2 + 16*S*(S+T1)**(-1)*
     +    MS2**2*(S+U1+M2)**(-1) - 12*S*(S+T1)**(-1)*MG2 + 20*S*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) + 8*S*MS2*(S+U1+M2)**(-1)
     +     - 12*S*MG2*(S+U1+M2)**(-1) + 4*S - 2*S**2*T1**(-1) - 12*S**2
     +    *(S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 16*S**2*(S+T1)**(-1)*MG2*
     +    (S+U1+M2)**(-1) - 4*S**2*(S+T1)**(-1) - 4*S**2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(35)*N*CF * ( 4*S**3*(S+T1)**(-1)*
     +    (S+U1+M2)**(-1) - 4*T1**(-1)*U1*MG2 - 12*T1**(-1)*MS2*MG2 + 8
     +    *T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 16*T1**(-1)*MS2**2*MG2
     +    *(S+U1+M2)**(-1) + 4*T1**(-1)*MS2**2 + 8*T1**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) - 2*T1*MS2*(S+U1+M2)**(-1) + 4*T1*MG2*
     +    (S+U1+M2)**(-1) + 8*U1*(S+T1)**(-1)*MG2 + 16*(S+T1)**(-1)*MS2
     +    *MG2 - 24*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 24*
     +    (S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 8*(S+T1)**(-1)*
     +    MS2**2 - 8*(S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*
     +    (S+T1)**(-1)*MG2**2 + 8*(S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1)
     +     + 12*MS2*MG2*(S+U1+M2)**(-1) + 2*MS2 + 8*MG2 - 12*MG2**2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(35)*CF**2 * ( 4*S*T1**(-1)*U1 - 16*S*
     +    T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 12*S*T1**(-1)*MS2 + 16*S*
     +    T1**(-1)*MS2**2*(S+U1+M2)**(-1) + 4*S*T1**(-1)*MG2 - 4*S*U1*
     +    (S+T1)**(-1) + 36*S*(S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 8*
     +    S*(S+T1)**(-1)*MS2 - 16*S*(S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1)
     +     + 12*S*(S+T1)**(-1)*MG2 - 20*S*(S+T1)**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) - 4*S*MS2*(S+U1+M2)**(-1) + 8*S*MG2*
     +    (S+U1+M2)**(-1) + 4*S**2*T1**(-1) + 12*S**2*(S+T1)**(-1)*MS2*
     +    (S+U1+M2)**(-1) - 16*S**2*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) + 
     +    4*S**2*(S+T1)**(-1) + 4*S**2*(S+U1+M2)**(-1) - 4*S**3*
     +    (S+T1)**(-1)*(S+U1+M2)**(-1) + 8*T1**(-1)*U1*MG2 + 24*
     +    T1**(-1)*MS2*MG2 - 16*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 
     +    32*T1**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 8*T1**(-1)*MS2**2 - 
     +    16*T1**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*U1*(S+T1)**(-1)*MG2 - 
     +    16*(S+T1)**(-1)*MS2*MG2 + 24*(S+T1)**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(35)*CF**2 * (  - 24*(S+T1)**(-1)*MS2**2*
     +    MG2*(S+U1+M2)**(-1) + 8*(S+T1)**(-1)*MS2**2 + 8*(S+T1)**(-1)*
     +    MS2**3*(S+U1+M2)**(-1) + 8*(S+T1)**(-1)*MG2**2 - 8*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) - 4*MS2*MG2*
     +    (S+U1+M2)**(-1) + 4*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(39)*N*CF * (  - 1 + 4*S**(-1)*T1**(-1)*U1*
     +    MS2 + 4*S**(-1)*T1**(-1)*U1**2 + 4*S**(-1)*U1 + 4*S**(-1)*MS2
     +     - 8*T1**(-1)*MS2 + T1*MS2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 
     +    T1*MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - T1*SYMBX*(U1+M2) + 2*
     +    T1*(T1+U1+M2)**(-1) + T1**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 
     +    U1*(T1+U1+M2)**(-1) - MS2*SYMBX*(U1+M2) + MG2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(39)*CF**2 * (  - 4*S**(-1)*T1**(-1)*U1*MS2
     +     - 4*S**(-1)*T1**(-1)*U1**2 - 4*S**(-1)*T1*U1*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1*MS2*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*U1*MS2*(T1+U1+M2)**(-1) - 4*S**(-1)*U1 - 4*S**(-1)*
     +    U1**2*(T1+U1+M2)**(-1) - 4*S**(-1)*MS2 + 8*T1**(-1)*MS2 )
     +
      M2QGH4 = M2QGH4 + ANG4(41)*N*CF * (  - 1 + 6*S**(-1)*T1*U1*
     +    (T1+U1+M2)**(-1) + 2*S**(-1)*T1*MS2*(T1+U1+M2)**(-1) + 2*
     +    S**(-1)*T1*MG2*(T1+U1+M2)**(-1) + 2*S**(-1)*T1 + 2*S**(-1)*
     +    T1**2*(T1+U1+M2)**(-1) + 4*S**(-1)*U1*MG2*(T1+U1+M2)**(-1) + 
     +    4*S**(-1)*U1 + 4*S**(-1)*U1**2*(T1+U1+M2)**(-1) + 4*S**(-1)*
     +    MS2*MG2*(T1+U1+M2)**(-1) + 4*S**(-1)*MS2 - 4*S**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 4*S*T1**(-1) - 8*T1**(-1)*U1 - 8*T1**(-1)*
     +    MS2 + 3*T1*MS2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 3*T1*MG2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - T1*SYMBX*(U1+M2) + 2*T1*
     +    (T1+U1+M2)**(-1) + T1**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + U1*
     +    (T1+U1+M2)**(-1) - 3*MS2*SYMBX*(U1+M2) + 2*MS2*
     +    (T1+U1+M2)**(-1) + 3*MG2*SYMBX*(U1+M2) - 2*MG2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(41)*CF**2 * (  - 12*S**(-1)*T1*U1*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1*MS2*(T1+U1+M2)**(-1) - 4*
     +    S**(-1)*T1*MG2*(T1+U1+M2)**(-1) - 4*S**(-1)*T1 - 4*S**(-1)*
     +    T1**2*(T1+U1+M2)**(-1) - 8*S**(-1)*U1*MG2*(T1+U1+M2)**(-1) - 
     +    8*S**(-1)*U1 - 8*S**(-1)*U1**2*(T1+U1+M2)**(-1) - 8*S**(-1)*
     +    MS2*MG2*(T1+U1+M2)**(-1) - 8*S**(-1)*MS2 + 8*S**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) + 8*S*T1**(-1) + 16*T1**(-1)*U1 + 16*
     +    T1**(-1)*MS2 )
     +
      M2QGH4 = M2QGH4 + ANG4(47)*N*CF * (  - 1 - 4*S**(-1)*T1**(-1)*U1*
     +    MS2 - 4*S**(-1)*T1**(-1)*U1**2 + S**(-1)*T1*MS2*
     +    (T1+U1+M2)**(-1) - S**(-1)*T1*MG2*(T1+U1+M2)**(-1) + S**(-1)*
     +    U1*MS2*(T1+U1+M2)**(-1) - S**(-1)*U1*MG2*(T1+U1+M2)**(-1) - 4
     +    *S**(-1)*U1 + 2*S**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 5*S**(-1)*
     +    MS2 - S**(-1)*MS2**2*(T1+U1+M2)**(-1) + S**(-1)*MG2 - S**(-1)
     +    *MG2**2*(T1+U1+M2)**(-1) + 8*T1**(-1)*MS2 + T1*
     +    (T1+U1+M2)**(-1) + U1*(T1+U1+M2)**(-1) - MS2*(T1+U1+M2)**(-1)
     +     + MG2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(47)*CF**2 * ( 4*S**(-1)*T1**(-1)*U1*MS2 + 
     +    4*S**(-1)*T1**(-1)*U1**2 + 4*S**(-1)*T1*U1*(T1+U1+M2)**(-1)
     +     + 4*S**(-1)*T1*MS2*(T1+U1+M2)**(-1) + 4*S**(-1)*U1*MS2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*U1*MG2*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*U1 + 4*S**(-1)*U1**2*(T1+U1+M2)**(-1) + 8*S**(-1)*MS2
     +    *MG2*(T1+U1+M2)**(-1) + 4*S**(-1)*MS2 - 8*S**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 8*T1**(-1)*MS2 )
     +
      M2QGH4 = M2QGH4 + ANG4(48)*N*CF * ( 6*S*T1**(-1)*U1 + 2*S*
     +    T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 4*S*T1**(-1)*MS2 - 2*S*
     +    T1**(-1)*MG2 - 2*S*T1**(-1)*MG2**2*(S+U1+M2)**(-1) + 4*S*T1*
     +    (S+U1+M2)**(-1) + 4*S*U1*(S+T1)**(-1) - 36*S*(S+T1)**(-1)*MS2
     +    *MG2*(S+U1+M2)**(-1) + 8*S*(S+T1)**(-1)*MS2 + 16*S*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 12*S*(S+T1)**(-1)*MG2
     +     + 20*S*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) + 12*S*MS2*
     +    (S+U1+M2)**(-1) - 14*S*MG2*(S+U1+M2)**(-1) + 6*S + 2*S**2*
     +    T1**(-1) - 12*S**2*(S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 16*S**2
     +    *(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) - 4*S**2*(S+T1)**(-1) - 4*
     +    S**2*(S+U1+M2)**(-1) + 4*S**3*(S+T1)**(-1)*(S+U1+M2)**(-1) + 
     +    4*T1**(-1)*U1*MS2 - 4*T1**(-1)*U1*MG2 + 4*T1**(-1)*U1**2 - 8*
     +    T1**(-1)*MS2*MG2 + 12*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 
     +    12*T1**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 4*T1**(-1)*MS2**2 + 
     +    4*T1**(-1)*MS2**3*(S+U1+M2)**(-1) + 4*T1**(-1)*MG2**2 - 4*
     +    T1**(-1)*MG2**3*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(48)*N*CF * (  - 6*T1*MS2*(S+U1+M2)**(-1)
     +     + 6*T1*MG2*(S+U1+M2)**(-1) - 6*T1 - 2*T1**2*(S+U1+M2)**(-1)
     +     - 8*U1*(S+T1)**(-1)*MS2 + 8*U1*(S+T1)**(-1)*MG2 - 8*U1 - 8*
     +    U1**2*(S+T1)**(-1) + 16*(S+T1)**(-1)*MS2*MG2 - 24*
     +    (S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 24*(S+T1)**(-1)*
     +    MS2**2*MG2*(S+U1+M2)**(-1) - 8*(S+T1)**(-1)*MS2**2 - 8*
     +    (S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*(S+T1)**(-1)*MG2**2
     +     + 8*(S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) + 16*MS2*MG2*
     +    (S+U1+M2)**(-1) - 8*MS2 - 8*MS2**2*(S+U1+M2)**(-1) + 8*MG2 - 
     +    8*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(48)*CF**2 * (  - 12*S*T1**(-1)*U1 - 4*S*
     +    T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 8*S*T1**(-1)*MS2 + 4*S*
     +    T1**(-1)*MG2 + 4*S*T1**(-1)*MG2**2*(S+U1+M2)**(-1) - 4*S*T1*
     +    (S+U1+M2)**(-1) - 4*S*U1*(S+T1)**(-1) + 36*S*(S+T1)**(-1)*MS2
     +    *MG2*(S+U1+M2)**(-1) - 8*S*(S+T1)**(-1)*MS2 - 16*S*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) + 12*S*(S+T1)**(-1)*MG2
     +     - 20*S*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) - 12*S*MS2*
     +    (S+U1+M2)**(-1) + 12*S*MG2*(S+U1+M2)**(-1) - 12*S - 4*S**2*
     +    T1**(-1) + 12*S**2*(S+T1)**(-1)*MS2*(S+U1+M2)**(-1) - 16*S**2
     +    *(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) + 4*S**2*(S+T1)**(-1) + 4*
     +    S**2*(S+U1+M2)**(-1) - 4*S**3*(S+T1)**(-1)*(S+U1+M2)**(-1) - 
     +    8*T1**(-1)*U1*MS2 + 8*T1**(-1)*U1*MG2 - 8*T1**(-1)*U1**2 + 16
     +    *T1**(-1)*MS2*MG2 - 24*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 
     +    24*T1**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 8*T1**(-1)*MS2**2 - 
     +    8*T1**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*T1**(-1)*MG2**2 + 8*
     +    T1**(-1)*MG2**3*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(48)*CF**2 * ( 8*U1*(S+T1)**(-1)*MS2 - 8*U1
     +    *(S+T1)**(-1)*MG2 + 8*U1**2*(S+T1)**(-1) - 16*(S+T1)**(-1)*
     +    MS2*MG2 + 24*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 24*
     +    (S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 8*(S+T1)**(-1)*
     +    MS2**2 + 8*(S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) + 8*
     +    (S+T1)**(-1)*MG2**2 - 8*(S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(54)*N*CF * (  - 2*S*T1**(-1)*U1*MS2*
     +    (T1+U1+M2)**(-1) + 2*S*T1**(-1)*U1**2*(T1+U1+M2)**(-1) + 4*S*
     +    U1*(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) + 2*S*U1*
     +    (T1+U1+M2)**(-1) + 4*S*U1**2*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 
     +    4*T1**(-1)*U1**3*(T1+U1+M2)**(-1) - 2*T1*U1*(T1+U1+M2)**(-1)
     +     - 4*U1**2*(T1+U1+M2)**(-1) - 8*U1**3*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(54)*CF**2 * (  - 4*S*T1**(-1)*U1 + 4*S*
     +    T1**(-1)*MS2*MG2*SYMBX*(U1+M2) + 4*S*T1**(-1)*MG2 - 4*S*
     +    T1**(-1)*MG2**2*SYMBX*(U1+M2) - 4*S*U1*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 4*S*U1**2*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 
     +    4*S*MS2*SYMBX*(U1+M2) + 4*S*MG2*SYMBX*(U1+M2) - 4*S - 8*
     +    T1**(-1)*U1*MS2 + 8*T1**(-1)*U1*MG2 - 8*T1**(-1)*U1**2 + 16*
     +    T1**(-1)*MS2*MG2 - 24*T1**(-1)*MS2*MG2**2*SYMBX*(U1+M2) + 24*
     +    T1**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 8*T1**(-1)*MS2**2 - 8*
     +    T1**(-1)*MS2**3*SYMBX*(U1+M2) - 8*T1**(-1)*MG2**2 + 8*
     +    T1**(-1)*MG2**3*SYMBX*(U1+M2) + 4*T1*U1*(T1+U1+M2)**(-1) - 4*
     +    T1*MS2*SYMBX*(U1+M2) + 4*T1*MG2*SYMBX*(U1+M2) - 4*T1 - 8*U1
     +     + 8*U1**2*(T1+U1+M2)**(-1) + 8*U1**3*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) + 16*MS2*MG2*SYMBX*(U1+M2) - 8*MS2 - 8*
     +    MS2**2*SYMBX*(U1+M2) + 8*MG2 - 8*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(58)*N*CF * ( 8*S*T1**(-1)*MS2*MG2*SYMBX*
     +    (U1+M2) + 8*S*T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 16*S*
     +    T1**(-1)*MS2 - 8*S*T1**(-1)*MS2**2*SYMBX*(U1+M2) - 8*S*
     +    T1**(-1)*MS2**2*(S+U1+M2)**(-1) + 4*S*T1*MS2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) - 2*S*T1*(S+U1+M2)**(-1) + 2*S*T1*
     +    (T1+U1+M2)**(-1) + 4*S*U1*(T1+U1+M2)**(-1) - 8*S*MS2*MG2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 4*S*MS2*SYMBX*(U1+M2) - 4*S*
     +    MS2*(S+U1+M2)**(-1) + 4*S*MS2*(T1+U1+M2)**(-1) + 8*S*MS2**2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 4*S*MG2*(S+U1+M2)**(-1) + 8*
     +    T1*U1*(T1+U1+M2)**(-1) - 4*T1*MS2*MG2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) - 2*T1*MS2*SYMBX*(U1+M2) + 4*T1*MS2*
     +    (S+U1+M2)**(-1) + 2*T1*MS2*(T1+U1+M2)**(-1) + 2*T1*MS2**2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 2*T1*MG2*SYMBX*(U1+M2) - 4*
     +    T1*MG2*(S+U1+M2)**(-1) - 2*T1*MG2*(T1+U1+M2)**(-1) + 2*T1*
     +    MG2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 2*T1 + 2*T1**2*MS2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(58)*N*CF * (  - 2*T1**2*MG2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + 2*T1**2*(S+U1+M2)**(-1) + 4*T1**2*
     +    (T1+U1+M2)**(-1) + 2*U1 + 6*U1**2*(T1+U1+M2)**(-1) + 4*MS2*
     +    MG2*SYMBX*(U1+M2) - 8*MS2*MG2*(S+U1+M2)**(-1) + 2*MS2 - 2*
     +    MS2**2*SYMBX*(U1+M2) + 4*MS2**2*(S+U1+M2)**(-1) - 2*MG2 - 2*
     +    MG2**2*SYMBX*(U1+M2) + 4*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(65)*CF**2 * (  - 16*U1*MS2 + 16*MS2*MG2 - 
     +    16*MS2*MG2**2*SYMBX*(U1+M2) + 32*MS2**2*MG2*SYMBX*(U1+M2) - 
     +    16*MS2**2 - 16*MS2**3*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(67)*N*CF * ( 2*S**(-1)*T1*U1 - 2*S**(-1)*
     +    T1*MS2*MG2*(S+U1+M2)**(-1) - 2*S**(-1)*T1*MG2 + 2*S**(-1)*T1*
     +    MG2**2*(S+U1+M2)**(-1) + 4*S**(-1)*U1*MS2 - 4*S**(-1)*U1*MG2
     +     + 4*S**(-1)*U1**2 - 8*S**(-1)*MS2*MG2 + 8*S**(-1)*MS2*MG2**2
     +    *SYMBX*(U1+M2) + 4*S**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 16*
     +    S**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 4*S**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) + 4*S**(-1)*MS2**2 + 8*S**(-1)*MS2**3*SYMBX*
     +    (U1+M2) - 4*S**(-1)*MS2**3*(S+U1+M2)**(-1) + 4*S**(-1)*MG2**2
     +     - 4*S**(-1)*MG2**3*(S+U1+M2)**(-1) + 8*S*T1**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 8*S*T1**(-1)*MS2**2*(S+U1+M2)**(-1) + 2*S*
     +    T1*(S+U1+M2)**(-1) + 4*S*U1*(S+T1)**(-1) - 36*S*(S+T1)**(-1)*
     +    MS2*MG2*(S+U1+M2)**(-1) + 8*S*(S+T1)**(-1)*MS2 + 16*S*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 12*S*(S+T1)**(-1)*MG2
     +     + 20*S*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) + 8*S*MS2*
     +    (S+U1+M2)**(-1) - 12*S*MG2*(S+U1+M2)**(-1) + 4*S - 12*S**2*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(67)*N*CF * ( 16*S**2*(S+T1)**(-1)*MG2*
     +    (S+U1+M2)**(-1) - 4*S**2*(S+T1)**(-1) - 4*S**2*
     +    (S+U1+M2)**(-1) + 4*S**3*(S+T1)**(-1)*(S+U1+M2)**(-1) + 16*
     +    T1**(-1)*U1*MS2 - 16*T1**(-1)*MS2*MG2 + 8*T1**(-1)*MS2*MG2**2
     +    *SYMBX*(U1+M2) + 8*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 16*
     +    T1**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 16*T1**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) + 16*T1**(-1)*MS2**2 + 8*T1**(-1)*MS2**3*
     +    SYMBX*(U1+M2) + 8*T1**(-1)*MS2**3*(S+U1+M2)**(-1) - 2*T1*MS2*
     +    (S+U1+M2)**(-1) + 4*T1*MG2*(S+U1+M2)**(-1) - 2*T1 - 8*U1*
     +    (S+T1)**(-1)*MS2 + 8*U1*(S+T1)**(-1)*MG2 - 4*U1 - 8*U1**2*
     +    (S+T1)**(-1) + 16*(S+T1)**(-1)*MS2*MG2 - 24*(S+T1)**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1) + 24*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) - 8*(S+T1)**(-1)*MS2**2 - 8*(S+T1)**(-1)*
     +    MS2**3*(S+U1+M2)**(-1) - 8*(S+T1)**(-1)*MG2**2 + 8*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) + 12*MS2*MG2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(67)*N*CF * (  - 4*MS2 + 8*MG2 - 12*MG2**2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(67)*CF**2 * (  - 4*S**(-1)*T1*U1 + 4*
     +    S**(-1)*T1*MS2*MG2*SYMBX*(U1+M2) + 4*S**(-1)*T1*MG2 - 4*
     +    S**(-1)*T1*MG2**2*SYMBX*(U1+M2) - 8*S**(-1)*U1*MS2 + 8*
     +    S**(-1)*U1*MG2 - 8*S**(-1)*U1**2 + 16*S**(-1)*MS2*MG2 - 24*
     +    S**(-1)*MS2*MG2**2*SYMBX*(U1+M2) + 24*S**(-1)*MS2**2*MG2*
     +    SYMBX*(U1+M2) - 8*S**(-1)*MS2**2 - 8*S**(-1)*MS2**3*SYMBX*
     +    (U1+M2) - 8*S**(-1)*MG2**2 + 8*S**(-1)*MG2**3*SYMBX*(U1+M2)
     +     + 16*S*T1**(-1)*MS2*MG2*SYMBX*(U1+M2) - 16*S*T1**(-1)*MS2*
     +    MG2*(S+U1+M2)**(-1) - 16*S*T1**(-1)*MS2 - 16*S*T1**(-1)*
     +    MS2**2*SYMBX*(U1+M2) + 16*S*T1**(-1)*MS2**2*(S+U1+M2)**(-1)
     +     - 4*S*U1*(S+T1)**(-1) + 36*S*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 8*S*(S+T1)**(-1)*MS2 - 16*S*(S+T1)**(-1)*
     +    MS2**2*(S+U1+M2)**(-1) + 12*S*(S+T1)**(-1)*MG2 - 20*S*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) - 4*S*MS2*(S+U1+M2)**(-1)
     +     + 8*S*MG2*(S+U1+M2)**(-1) - 4*S + 12*S**2*(S+T1)**(-1)*MS2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(67)*CF**2 * (  - 16*S**2*(S+T1)**(-1)*MG2*
     +    (S+U1+M2)**(-1) + 4*S**2*(S+T1)**(-1) + 4*S**2*
     +    (S+U1+M2)**(-1) - 4*S**3*(S+T1)**(-1)*(S+U1+M2)**(-1) - 32*
     +    T1**(-1)*U1*MS2 + 32*T1**(-1)*MS2*MG2 - 16*T1**(-1)*MS2*
     +    MG2**2*SYMBX*(U1+M2) - 16*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1)
     +     + 32*T1**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 32*T1**(-1)*MS2**2*
     +    MG2*(S+U1+M2)**(-1) - 32*T1**(-1)*MS2**2 - 16*T1**(-1)*MS2**3
     +    *SYMBX*(U1+M2) - 16*T1**(-1)*MS2**3*(S+U1+M2)**(-1) + 8*U1*
     +    (S+T1)**(-1)*MS2 - 8*U1*(S+T1)**(-1)*MG2 - 4*U1 + 8*U1**2*
     +    (S+T1)**(-1) - 16*(S+T1)**(-1)*MS2*MG2 + 24*(S+T1)**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1) - 24*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) + 8*(S+T1)**(-1)*MS2**2 + 8*(S+T1)**(-1)*
     +    MS2**3*(S+U1+M2)**(-1) + 8*(S+T1)**(-1)*MG2**2 - 8*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) + 32*MS2*MG2*SYMBX*
     +    (U1+M2) - 4*MS2*MG2*(S+U1+M2)**(-1) - 16*MS2*MG2**2*SYMBX - 8
     +    *MS2 )
     +
      M2QGH4 = M2QGH4 + ANG4(67)*CF**2 * ( 32*MS2**2*MG2*SYMBX - 24*
     +    MS2**2*SYMBX*(U1+M2) - 16*MS2**3*SYMBX + 4*MG2 - 8*MG2**2*
     +    SYMBX*(U1+M2) + 4*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(76)*N*CF * ( 4*S**(-1)*T1**(-1)*U1*MS2*MG2
     +     - 4*S**(-1)*T1**(-1)*U1*MS2**2 + 4*S**(-1)*T1**(-1)*U1**2*
     +    MG2 - 4*S**(-1)*T1**(-1)*U1**3 + 2*S**(-1)*T1*U1*MS2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1*U1*MG2*(T1+U1+M2)**(-1) - 10*
     +    S**(-1)*T1*U1 + 4*S**(-1)*T1*U1**2*(T1+U1+M2)**(-1) - 2*
     +    S**(-1)*T1*MS2*MG2*(T1+U1+M2)**(-1) + 2*S**(-1)*T1*MS2**2*
     +    (T1+U1+M2)**(-1) + 2*S**(-1)*T1**2*U1*(T1+U1+M2)**(-1) + 2*
     +    S**(-1)*T1**2*MS2*(T1+U1+M2)**(-1) - 2*S**(-1)*T1**2*MG2*
     +    (T1+U1+M2)**(-1) - 2*S**(-1)*T1**2 - 2*S**(-1)*U1*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 2*S**(-1)*U1*MS2**2*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*U1*MG2 - 2*S**(-1)*U1**2*MG2*(T1+U1+M2)**(-1) - 12*
     +    S**(-1)*U1**2 + 2*S**(-1)*U1**3*(T1+U1+M2)**(-1) + 4*S**(-1)*
     +    MS2*MG2 - 4*S**(-1)*MS2**2 + 2*S*T1**(-1)*U1*MS2*
     +    (T1+U1+M2)**(-1) - 2*S*T1**(-1)*U1**2*(T1+U1+M2)**(-1) + 8*S*
     +    T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 4*S*T1**(-1)*MS2 - 8*S*
     +    T1**(-1)*MS2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(76)*N*CF * (  - 2*S*T1*(S+U1+M2)**(-1) - 2
     +    *S*U1*(T1+U1+M2)**(-1) - 4*S*MS2*(S+U1+M2)**(-1) + 4*S*MG2*
     +    (S+U1+M2)**(-1) + 4*T1**(-1)*U1*MS2*MG2*(T1+U1+M2)**(-1) + 10
     +    *T1**(-1)*U1*MS2 - 4*T1**(-1)*U1*MS2**2*(T1+U1+M2)**(-1) + 4*
     +    T1**(-1)*U1**2*MS2*(T1+U1+M2)**(-1) - 2*T1**(-1)*U1**2 - 4*
     +    T1**(-1)*U1**3*(T1+U1+M2)**(-1) - 8*T1**(-1)*MS2*MG2 + 8*
     +    T1**(-1)*MS2**2 + 6*T1*U1*(T1+U1+M2)**(-1) + 4*T1*MS2*
     +    (S+U1+M2)**(-1) - 6*T1*MS2*(T1+U1+M2)**(-1) - 4*T1*MG2*
     +    (S+U1+M2)**(-1) + 2*T1*MG2*(T1+U1+M2)**(-1) - 2*T1 + 2*T1**2*
     +    (S+U1+M2)**(-1) + 4*T1**2*(T1+U1+M2)**(-1) - 4*U1*MS2*
     +    (T1+U1+M2)**(-1) + 2*U1*MG2*(T1+U1+M2)**(-1) - 8*U1 - 2*U1**2
     +    *(T1+U1+M2)**(-1) - 8*MS2*MG2*(S+U1+M2)**(-1) + 6*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 12*MS2 + 4*MS2**2*(S+U1+M2)**(-1) - 6*
     +    MS2**2*(T1+U1+M2)**(-1) - 4*MG2 + 4*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(76)*CF**2 * (  - 4*S**(-1)*T1**(-1)*U1*MS2
     +    *MG2 + 4*S**(-1)*T1**(-1)*U1*MS2**2 - 4*S**(-1)*T1**(-1)*
     +    U1**2*MG2 + 4*S**(-1)*T1**(-1)*U1**3 + 12*S**(-1)*T1*U1 + 4*
     +    S**(-1)*T1**2 - 4*S**(-1)*U1*MG2 + 12*S**(-1)*U1**2 - 4*
     +    S**(-1)*MS2*MG2 + 4*S**(-1)*MS2**2 + 4*S*T1**(-1)*U1 - 4*S*
     +    T1**(-1)*MS2*MG2*SYMBX*(U1+M2) - 8*S*T1**(-1)*MS2 - 4*S*
     +    T1**(-1)*MG2 + 4*S*T1**(-1)*MG2**2*SYMBX*(U1+M2) - 4*S*MS2*
     +    SYMBX*(U1+M2) - 4*S*MG2*SYMBX*(U1+M2) + 4*S - 8*T1**(-1)*U1*
     +    MS2 - 8*T1**(-1)*U1*MG2 + 8*T1**(-1)*U1**2 - 8*T1**(-1)*MS2*
     +    MG2 + 24*T1**(-1)*MS2*MG2**2*SYMBX*(U1+M2) - 24*T1**(-1)*
     +    MS2**2*MG2*SYMBX*(U1+M2) + 8*T1**(-1)*MS2**3*SYMBX*(U1+M2) + 
     +    8*T1**(-1)*MG2**2 - 8*T1**(-1)*MG2**3*SYMBX*(U1+M2) + 4*T1*
     +    MS2*SYMBX*(U1+M2) - 4*T1*MG2*SYMBX*(U1+M2) + 8*T1 + 16*U1 - 
     +    16*MS2*MG2*SYMBX*(U1+M2) - 4*MS2 + 8*MS2**2*SYMBX*(U1+M2) - 8
     +    *MG2 + 8*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(80)*N*CF * (  - 6 - 4*S**(-1)*T1**(-1)*U1*
     +    MS2 - 4*S**(-1)*T1**(-1)*U1**2 + 2*S**(-1)*T1*U1*
     +    (T1+U1+M2)**(-1) - 2*S**(-1)*U1*MS2*(T1+U1+M2)**(-1) + 2*
     +    S**(-1)*U1*MG2*(T1+U1+M2)**(-1) - 6*S**(-1)*U1 + 2*S**(-1)*
     +    U1**2*(T1+U1+M2)**(-1) - 2*S*T1**(-1)*U1*(T1+U1+M2)**(-1) + 2
     +    *S*T1**(-1)*MS2*(T1+U1+M2)**(-1) - 2*S*T1**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 2*S*T1**(-1) - 2*S*(T1+U1+M2)**(-1) + 4*
     +    T1**(-1)*U1*MS2*(T1+U1+M2)**(-1) - 4*T1**(-1)*U1*MG2*
     +    (T1+U1+M2)**(-1) - 4*T1**(-1)*U1 - 4*T1**(-1)*U1**2*
     +    (T1+U1+M2)**(-1) - 4*T1**(-1)*MS2 + 2*T1*(T1+U1+M2)**(-1) - 2
     +    *U1*(T1+U1+M2)**(-1) - 2*MS2*(T1+U1+M2)**(-1) + 2*MG2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(80)*CF**2 * ( 8 + 4*S**(-1)*T1**(-1)*U1*
     +    MS2 + 4*S**(-1)*T1**(-1)*U1**2 + 4*S**(-1)*U1 + 8*S*T1**(-1)
     +     + 16*T1**(-2)*MS2*MG2 - 16*T1**(-2)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) + 32*T1**(-2)*MS2**2*MG2*(S+U1+M2)**(-1) - 16
     +    *T1**(-2)*MS2**3*(S+U1+M2)**(-1) + 12*T1**(-1)*U1 + 12*
     +    T1**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 12*T1**(-1)*MS2 - 8*
     +    T1**(-1)*MS2**2*(S+U1+M2)**(-1) + 4*T1**(-1)*MG2 - 4*T1**(-1)
     +    *MG2**2*(S+U1+M2)**(-1) - 4*MS2*(S+U1+M2)**(-1) + 4*MG2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(82)*N*CF * ( 2*S**(-1)*T1*U1 + 2*S**(-1)*
     +    T1*MS2 - 2*S**(-1)*T1*MG2 + 2*S**(-1)*T1**2 - 4*S**(-1)*U1*
     +    MG2 - 4*S**(-1)*MS2*MG2 + 4*S**(-1)*MS2**2 + 2*S*T1**(-1)*MS2
     +    *MG2*(S+U1+M2)**(-1) - 2*S*T1**(-1)*MG2**2*(S+U1+M2)**(-1) + 
     +    4*S*T1*(S+U1+M2)**(-1) + 4*S*U1*(S+T1)**(-1) - 36*S*
     +    (S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 8*S*(S+T1)**(-1)*MS2
     +     + 16*S*(S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 12*S*
     +    (S+T1)**(-1)*MG2 + 20*S*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1)
     +     + 12*S*MS2*(S+U1+M2)**(-1) - 14*S*MG2*(S+U1+M2)**(-1) + 4*S
     +     - 12*S**2*(S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 16*S**2*
     +    (S+T1)**(-1)*MG2*(S+U1+M2)**(-1) - 4*S**2*(S+T1)**(-1) - 4*
     +    S**2*(S+U1+M2)**(-1) + 4*S**3*(S+T1)**(-1)*(S+U1+M2)**(-1) - 
     +    4*T1**(-1)*MS2*MG2 + 12*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1)
     +     - 12*T1**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 4*T1**(-1)*MS2**3
     +    *(S+U1+M2)**(-1) + 4*T1**(-1)*MG2**2 - 4*T1**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(82)*N*CF * (  - 6*T1*MS2*(S+U1+M2)**(-1)
     +     + 6*T1*MG2*(S+U1+M2)**(-1) - 4*T1 - 2*T1**2*(S+U1+M2)**(-1)
     +     + 8*U1*(S+T1)**(-1)*MG2 - 4*U1 + 16*(S+T1)**(-1)*MS2*MG2 - 
     +    24*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 24*(S+T1)**(-1)*
     +    MS2**2*MG2*(S+U1+M2)**(-1) - 8*(S+T1)**(-1)*MS2**2 - 8*
     +    (S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) - 8*(S+T1)**(-1)*MG2**2
     +     + 8*(S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) + 16*MS2*MG2*
     +    (S+U1+M2)**(-1) - 6*MS2 - 8*MS2**2*(S+U1+M2)**(-1) + 8*MG2 - 
     +    8*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(82)*CF**2 * (  - 4*S**(-1)*T1*U1 - 4*
     +    S**(-1)*T1*MS2 + 4*S**(-1)*T1*MG2 - 4*S**(-1)*T1**2 + 8*
     +    S**(-1)*U1*MG2 + 8*S**(-1)*MS2*MG2 - 8*S**(-1)*MS2**2 + 4*S*
     +    T1**(-1)*MS2*MG2*SYMBX*(U1+M2) - 4*S*T1**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 4*S*T1**(-1)*MG2**2*SYMBX*(U1+M2) + 4*S*
     +    T1**(-1)*MG2**2*(S+U1+M2)**(-1) - 4*S*T1*(S+U1+M2)**(-1) - 4*
     +    S*U1*(S+T1)**(-1) + 36*S*(S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1)
     +     - 8*S*(S+T1)**(-1)*MS2 - 16*S*(S+T1)**(-1)*MS2**2*
     +    (S+U1+M2)**(-1) + 12*S*(S+T1)**(-1)*MG2 - 20*S*(S+T1)**(-1)*
     +    MG2**2*(S+U1+M2)**(-1) + 4*S*MS2*SYMBX*(U1+M2) - 12*S*MS2*
     +    (S+U1+M2)**(-1) + 4*S*MG2*SYMBX*(U1+M2) + 12*S*MG2*
     +    (S+U1+M2)**(-1) - 4*S + 12*S**2*(S+T1)**(-1)*MS2*
     +    (S+U1+M2)**(-1) - 16*S**2*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) + 
     +    4*S**2*(S+T1)**(-1) + 4*S**2*(S+U1+M2)**(-1) - 4*S**3*
     +    (S+T1)**(-1)*(S+U1+M2)**(-1) + 16*T1**(-1)*MS2*MG2 - 24*
     +    T1**(-1)*MS2*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(82)*CF**2 * (  - 24*T1**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) + 24*T1**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 24*
     +    T1**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 8*T1**(-1)*MS2**3*SYMBX
     +    *(U1+M2) - 8*T1**(-1)*MS2**3*(S+U1+M2)**(-1) - 16*T1**(-1)*
     +    MG2**2 + 8*T1**(-1)*MG2**3*SYMBX*(U1+M2) + 8*T1**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) - 4*T1*MS2*SYMBX*(U1+M2) + 4*T1*MG2*SYMBX*
     +    (U1+M2) - 8*U1*(S+T1)**(-1)*MG2 + 4*U1 - 16*(S+T1)**(-1)*MS2*
     +    MG2 + 24*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 24*
     +    (S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 8*(S+T1)**(-1)*
     +    MS2**2 + 8*(S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) + 8*
     +    (S+T1)**(-1)*MG2**2 - 8*(S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1)
     +     + 16*MS2*MG2*SYMBX*(U1+M2) + 8*MS2 - 8*MS2**2*SYMBX*(U1+M2)
     +     + 4*MG2 - 8*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + ANG4(83)*N*CF * (  - 8*S**(-1)*T1*U1*MG2*
     +    (T1+U1+M2)**(-1) - 2*S**(-1)*T1*U1**2*(T1+U1+M2)**(-1) - 4*
     +    S**(-1)*T1*MS2*MG2*(T1+U1+M2)**(-1) + 6*S**(-1)*T1*MS2**2*
     +    (T1+U1+M2)**(-1) - 2*S**(-1)*T1*MG2**2*(T1+U1+M2)**(-1) - 2*
     +    S**(-1)*T1**2*U1*(T1+U1+M2)**(-1) - 2*S**(-1)*T1**2*MS2*
     +    (T1+U1+M2)**(-1) - 2*S**(-1)*T1**2*MG2*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*U1*MS2**2*(T1+U1+M2)**(-1) - 4*S**(-1)*U1*MG2**2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*U1**2*MG2*(T1+U1+M2)**(-1) - 4*
     +    S**(-1)*MS2*MG2**2*(T1+U1+M2)**(-1) + 8*S**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*MS2**3*(T1+U1+M2)**(-1) - 2*S*
     +    T1**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 2*S*T1**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) - 4*S*U1*(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1)
     +     - 4*S*U1**2*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 2*S*MG2*
     +    (T1+U1+M2)**(-1) - 4*T1**(-1)*U1*MS2*MG2*(T1+U1+M2)**(-1) + 4
     +    *T1**(-1)*U1*MG2**2*(T1+U1+M2)**(-1) + 4*T1**(-1)*MS2*MG2**2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(83)*N*CF * (  - 8*T1**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) + 4*T1**(-1)*MS2**3*(T1+U1+M2)**(-1) + 2*T1*
     +    U1*(T1+U1+M2)**(-1) + 2*T1*MS2*(T1+U1+M2)**(-1) + 8*U1*
     +    (S+T1)**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 8*U1*(S+T1)**(-1)*
     +    MS2**2*(T1+U1+M2)**(-1) + 2*U1*MS2*(T1+U1+M2)**(-1) + 8*U1*
     +    MG2*(T1+U1+M2)**(-1) + 8*U1**2*(S+T1)**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 4*U1**2*(T1+U1+M2)**(-1) + 6*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 6*MS2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(83)*CF**2 * ( 16*S**(-1)*T1*U1*MG2*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*T1*U1**2*(T1+U1+M2)**(-1) + 8*
     +    S**(-1)*T1*MS2*MG2*(T1+U1+M2)**(-1) - 12*S**(-1)*T1*MS2**2*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*T1*MG2**2*(T1+U1+M2)**(-1) + 4*
     +    S**(-1)*T1**2*U1*(T1+U1+M2)**(-1) + 4*S**(-1)*T1**2*MS2*
     +    (T1+U1+M2)**(-1) + 4*S**(-1)*T1**2*MG2*(T1+U1+M2)**(-1) - 8*
     +    S**(-1)*U1*MS2**2*(T1+U1+M2)**(-1) + 8*S**(-1)*U1*MG2**2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*U1**2*MG2*(T1+U1+M2)**(-1) + 8*
     +    S**(-1)*MS2*MG2**2*(T1+U1+M2)**(-1) - 16*S**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*MS2**3*(T1+U1+M2)**(-1) + 4*S*U1
     +    *(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) + 4*S*U1**2*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) + 4*T1*MG2*(T1+U1+M2)**(-1) - 8*U1*
     +    (S+T1)**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 8*U1*(S+T1)**(-1)*
     +    MS2**2*(T1+U1+M2)**(-1) - 4*U1*MS2*(T1+U1+M2)**(-1) - 8*U1**2
     +    *(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 4*U1**2*(T1+U1+M2)**(-1)
     +     - 4*MS2*MG2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(83)*CF**2 * ( 4*MG2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(84)*N*CF * ( 4*S**(-1)*T1**(-1)*U1*MS2*MG2
     +     - 4*S**(-1)*T1**(-1)*U1*MS2**2 + 4*S**(-1)*T1**(-1)*U1**2*
     +    MG2 - 4*S**(-1)*T1**(-1)*U1**3 - S**(-1)*T1*U1 + 4*S**(-1)*T1
     +    *U1**2*(T1+U1+M2)**(-1) + 2*S**(-1)*T1*MS2*MG2*
     +    (S+U1+M2)**(-1) + 2*S**(-1)*T1*MG2 - 2*S**(-1)*T1*MG2**2*
     +    (S+U1+M2)**(-1) + S**(-1)*T1**2*U1*(T1+U1+M2)**(-1) + 2*
     +    S**(-1)*U1*MS2*MG2*(T1+U1+M2)**(-1) - 3*S**(-1)*U1*MS2 - 
     +    S**(-1)*U1*MS2**2*(T1+U1+M2)**(-1) + 9*S**(-1)*U1*MG2 - 
     +    S**(-1)*U1*MG2**2*(T1+U1+M2)**(-1) - 2*S**(-1)*U1**2*MS2*
     +    (T1+U1+M2)**(-1) + 2*S**(-1)*U1**2*MG2*(T1+U1+M2)**(-1) - 5*
     +    S**(-1)*U1**2 + 3*S**(-1)*U1**3*(T1+U1+M2)**(-1) + 8*S**(-1)*
     +    MS2*MG2 - 8*S**(-1)*MS2*MG2**2*SYMBX*(U1+M2) - 4*S**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1) + 16*S**(-1)*MS2**2*MG2*SYMBX*(U1+M2)
     +     - 4*S**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 4*S**(-1)*MS2**2 - 
     +    8*S**(-1)*MS2**3*SYMBX*(U1+M2) + 4*S**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(84)*N*CF * (  - 4*S**(-1)*MG2**2 + 4*
     +    S**(-1)*MG2**3*(S+U1+M2)**(-1) - 2*S*T1**(-1)*U1 + 8*S*
     +    T1**(-1)*MS2*MG2*SYMBX*(U1+M2) + 2*S*T1**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 2*S*T1**(-1)*MS2 - 8*S*T1**(-1)*MS2**2*
     +    SYMBX*(U1+M2) + 2*S*T1**(-1)*MG2 - 2*S*T1**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) + S*T1*(T1+U1+M2)**(-1) + 3*S*U1*
     +    (T1+U1+M2)**(-1) - 8*S*MS2*MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1)
     +     + S*MS2*(T1+U1+M2)**(-1) + 8*S*MS2**2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + 2*S*MG2*(S+U1+M2)**(-1) - S*MG2*
     +    (T1+U1+M2)**(-1) - S - 2*T1**(-1)*U1*MS2 + 4*T1**(-1)*U1*MG2
     +     - 6*T1**(-1)*U1**2 + 12*T1**(-1)*MS2*MG2 - 8*T1**(-1)*MS2*
     +    MG2**2*SYMBX*(U1+M2) + 4*T1**(-1)*MS2*MG2**2*(S+U1+M2)**(-1)
     +     + 16*T1**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 4*T1**(-1)*MS2**2*
     +    MG2*(S+U1+M2)**(-1) - 16*T1**(-1)*MS2**2 - 8*T1**(-1)*MS2**3*
     +    SYMBX*(U1+M2) - 4*T1**(-1)*MS2**3*(S+U1+M2)**(-1) + 4*
     +    T1**(-1)*MG2**2 )
     +
      M2QGH4 = M2QGH4 + ANG4(84)*N*CF * (  - 4*T1**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) + 5*T1*U1*(T1+U1+M2)**(-1) - 8*T1*MS2*MG2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 3*T1*MS2*(T1+U1+M2)**(-1) + 
     +    6*T1*MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 2*T1*MG2*
     +    (S+U1+M2)**(-1) - 3*T1*MG2*(T1+U1+M2)**(-1) + 2*T1*MG2**2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - T1 + T1**2*(T1+U1+M2)**(-1)
     +     - 4*U1 + 6*U1**2*(T1+U1+M2)**(-1) + 8*MS2*MG2*SYMBX*(U1+M2)
     +     - 4*MS2*MG2*(S+U1+M2)**(-1) - 4*MS2*MG2*(T1+U1+M2)**(-1) - 4
     +    *MS2 - 6*MS2**2*SYMBX*(U1+M2) - 4*MS2**2*(S+U1+M2)**(-1) + 4*
     +    MS2**2*(T1+U1+M2)**(-1) + 2*MG2 - 2*MG2**2*SYMBX*(U1+M2) + 8*
     +    MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + ANG4(84)*CF**2 * (  - 4*S**(-1)*T1**(-1)*U1*MS2
     +    *MG2 + 4*S**(-1)*T1**(-1)*U1*MS2**2 - 4*S**(-1)*T1**(-1)*
     +    U1**2*MG2 + 4*S**(-1)*T1**(-1)*U1**3 - 4*S**(-1)*T1*MS2*MG2*
     +    SYMBX*(U1+M2) - 4*S**(-1)*T1*MG2 + 4*S**(-1)*T1*MG2**2*SYMBX*
     +    (U1+M2) + 4*S**(-1)*U1*MS2 - 12*S**(-1)*U1*MG2 + 4*S**(-1)*
     +    U1**2 - 16*S**(-1)*MS2*MG2 + 24*S**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) - 24*S**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 8*S**(-1)*
     +    MS2**2 + 8*S**(-1)*MS2**3*SYMBX*(U1+M2) + 8*S**(-1)*MG2**2 - 
     +    8*S**(-1)*MG2**3*SYMBX*(U1+M2) + 4*S*T1**(-1)*U1 - 16*S*
     +    T1**(-1)*MS2*MG2*SYMBX*(U1+M2) + 4*S*T1**(-1)*MS2 + 16*S*
     +    T1**(-1)*MS2**2*SYMBX*(U1+M2) - 4*S*T1**(-1)*MG2 + 4*T1**(-1)
     +    *U1*MS2 - 8*T1**(-1)*U1*MG2 + 8*T1**(-1)*U1**2 - 20*T1**(-1)*
     +    MS2*MG2 + 16*T1**(-1)*MS2*MG2**2*SYMBX*(U1+M2) - 32*T1**(-1)*
     +    MS2**2*MG2*SYMBX*(U1+M2) + 20*T1**(-1)*MS2**2 + 16*T1**(-1)*
     +    MS2**3*SYMBX*(U1+M2) + 4*U1 - 32*MS2*MG2*SYMBX*(U1+M2) + 16*
     +    MS2*MG2**2*SYMBX )
     +
      M2QGH4 = M2QGH4 + ANG4(84)*CF**2 * ( 8*MS2 - 32*MS2**2*MG2*SYMBX
     +     + 24*MS2**2*SYMBX*(U1+M2) + 16*MS2**3*SYMBX - 12*MG2 + 8*
     +    MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*SYMBT*(S4+MS2)*CF * (  - 16*S**(-1)*
     +    T1*U1*S4**(-1)*MG2*(T1+U1+M2)**(-1) - 4*S**(-1)*T1*U1**2*
     +    S4**(-1)*(T1+U1+M2)**(-1) - 8*S**(-1)*T1*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 12*S**(-1)*T1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1**2*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1**2*S4**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 4*S**(-1)*T1**2*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*U1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*U1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*U1**2*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*MS2*MG2**2*
     +    (T1+U1+M2)**(-1) + 16*S**(-1)*S4**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*MS2**3*(T1+U1+M2)**(-1)
     +     - 4*S*T1**(-1)*S4**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 4*S*
     +    T1**(-1)*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*SYMBT*(S4+MS2)*CF * (  - 8*S*U1*
     +    S4**(-1)*(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) - 8*S*U1**2*
     +    S4**(-1)*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 4*S*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 8*T1**(-1)*U1*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 8*T1**(-1)*U1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (T1+U1+M2)**(-1) - 16*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*MS2**3*
     +    (T1+U1+M2)**(-1) + 4*T1*U1*S4**(-1)*(T1+U1+M2)**(-1) + 4*T1*
     +    S4**(-1)*MS2*(T1+U1+M2)**(-1) + 16*U1*S4**(-1)*(S+T1)**(-1)*
     +    MS2*MG2*(T1+U1+M2)**(-1) - 16*U1*S4**(-1)*(S+T1)**(-1)*MS2**2
     +    *(T1+U1+M2)**(-1) + 4*U1*S4**(-1)*MS2*(T1+U1+M2)**(-1) + 16*
     +    U1*S4**(-1)*MG2*(T1+U1+M2)**(-1) + 16*U1**2*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 8*U1**2*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 12*S4**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 12*
     +    S4**(-1)*MS2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*SYMBU*(S4+MS2)*CF * ( 8*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2 - 8*S**(-1)*T1**(-1)*U1*S4**(-1)
     +    *MS2**2 + 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 - 8*S**(-1)*
     +    T1**(-1)*U1**3*S4**(-1) - 2*S**(-1)*T1*U1*S4**(-1) + 8*
     +    S**(-1)*T1*U1**2*S4**(-1)*(T1+U1+M2)**(-1) + 4*S**(-1)*T1*
     +    S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 4*S**(-1)*T1*S4**(-1)*MG2
     +     - 4*S**(-1)*T1*S4**(-1)*MG2**2*(S+U1+M2)**(-1) + 2*S**(-1)*
     +    T1**2*U1*S4**(-1)*(T1+U1+M2)**(-1) + 4*S**(-1)*U1*S4**(-1)*
     +    MS2*MG2*(T1+U1+M2)**(-1) - 6*S**(-1)*U1*S4**(-1)*MS2 - 2*
     +    S**(-1)*U1*S4**(-1)*MS2**2*(T1+U1+M2)**(-1) + 18*S**(-1)*U1*
     +    S4**(-1)*MG2 - 2*S**(-1)*U1*S4**(-1)*MG2**2*(T1+U1+M2)**(-1)
     +     - 4*S**(-1)*U1**2*S4**(-1)*MS2*(T1+U1+M2)**(-1) + 4*S**(-1)*
     +    U1**2*S4**(-1)*MG2*(T1+U1+M2)**(-1) - 10*S**(-1)*U1**2*
     +    S4**(-1) + 6*S**(-1)*U1**3*S4**(-1)*(T1+U1+M2)**(-1) + 16*
     +    S**(-1)*S4**(-1)*MS2*MG2 - 16*S**(-1)*S4**(-1)*MS2*MG2**2*
     +    SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*SYMBU*(S4+MS2)*CF * (  - 8*S**(-1)*
     +    S4**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 32*S**(-1)*S4**(-1)*
     +    MS2**2*MG2*SYMBX*(U1+M2) - 8*S**(-1)*S4**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*MS2**2 - 16*S**(-1)*
     +    S4**(-1)*MS2**3*SYMBX*(U1+M2) + 8*S**(-1)*S4**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*MG2**2 + 8*S**(-1)*
     +    S4**(-1)*MG2**3*(S+U1+M2)**(-1) - 4*S*T1**(-1)*U1*S4**(-1) + 
     +    16*S*T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) + 4*S*T1**(-1)*
     +    S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 4*S*T1**(-1)*S4**(-1)*MS2
     +     - 16*S*T1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) + 4*S*T1**(-1)
     +    *S4**(-1)*MG2 - 4*S*T1**(-1)*S4**(-1)*MG2**2*(S+U1+M2)**(-1)
     +     + 2*S*T1*S4**(-1)*(T1+U1+M2)**(-1) + 6*S*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 16*S*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + 2*S*S4**(-1)*MS2*(T1+U1+M2)**(-1) + 16*S*
     +    S4**(-1)*MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 4*S*S4**(-1)
     +    *MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*SYMBU*(S4+MS2)*CF * (  - 2*S*
     +    S4**(-1)*MG2*(T1+U1+M2)**(-1) - 2*S*S4**(-1) - 4*T1**(-1)*U1*
     +    S4**(-1)*MS2 + 8*T1**(-1)*U1*S4**(-1)*MG2 - 12*T1**(-1)*U1**2
     +    *S4**(-1) + 24*T1**(-1)*S4**(-1)*MS2*MG2 - 16*T1**(-1)*
     +    S4**(-1)*MS2*MG2**2*SYMBX*(U1+M2) + 8*T1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1) + 32*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    SYMBX*(U1+M2) + 8*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) - 32*T1**(-1)*S4**(-1)*MS2**2 - 16*T1**(-1)*
     +    S4**(-1)*MS2**3*SYMBX*(U1+M2) - 8*T1**(-1)*S4**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*MG2**2 - 8*T1**(-1)*
     +    S4**(-1)*MG2**3*(S+U1+M2)**(-1) + 10*T1*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 16*T1*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)*
     +    (T1+U1+M2)**(-1) + 6*T1*S4**(-1)*MS2*(T1+U1+M2)**(-1) + 12*T1
     +    *S4**(-1)*MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 4*T1*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1) - 6*T1*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*SYMBU*(S4+MS2)*CF * ( 4*T1*S4**(-1)*
     +    MG2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 2*T1*S4**(-1) + 2*
     +    T1**2*S4**(-1)*(T1+U1+M2)**(-1) - 8*U1*S4**(-1) + 12*U1**2*
     +    S4**(-1)*(T1+U1+M2)**(-1) + 16*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)
     +     - 8*S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 8*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 8*S4**(-1)*MS2 - 12*S4**(-1)*MS2**2*SYMBX*
     +    (U1+M2) - 8*S4**(-1)*MS2**2*(S+U1+M2)**(-1) + 8*S4**(-1)*
     +    MS2**2*(T1+U1+M2)**(-1) + 4*S4**(-1)*MG2 - 4*S4**(-1)*MG2**2*
     +    SYMBX*(U1+M2) + 16*S4**(-1)*MG2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 16*S**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2*MG2 + 16*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*(U1+M2) + 8*S**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 32*
     +    S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2)
     +     + 8*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) + 8*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MS2**2
     +     + 16*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2)
     +     - 8*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) + 8*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MG2**2
     +     - 8*S**(-1)*T1**(-1)*U1**(-1)*S4**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) + 16*S**(-1)*T1**(-1)*U1*S4**(-2)*MS2 - 16*
     +    S**(-1)*T1**(-1)*U1*S4**(-2)*MG2 + 4*S**(-1)*T1**(-1)*U1*
     +    S4**(-1) + 16*S**(-1)*T1**(-1)*U1**2*S4**(-2) + 20*S**(-1)*
     +    T1**(-1)*U1**2*S4**(-1)*(T1+U1+M2)**(-1) - 32*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2*MG2 )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 48*S**(-1)*T1**(-1)*
     +    S4**(-2)*MS2*MG2**2*(S+U1+M2)**(-1) - 48*S**(-1)*T1**(-1)*
     +    S4**(-2)*MS2**2*MG2*(S+U1+M2)**(-1) + 16*S**(-1)*T1**(-1)*
     +    S4**(-2)*MS2**2 + 16*S**(-1)*T1**(-1)*S4**(-2)*MS2**3*
     +    (S+U1+M2)**(-1) + 16*S**(-1)*T1**(-1)*S4**(-2)*MG2**2 - 16*
     +    S**(-1)*T1**(-1)*S4**(-2)*MG2**3*(S+U1+M2)**(-1) + 8*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) - 16*S**(-1)*T1**(-1)
     +    *S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 4*S**(-1)*T1**(-1)*
     +    S4**(-1)*MS2 - 4*S**(-1)*T1**(-1)*S4**(-1)*MS2**2*SYMBX*
     +    (U1+M2) + 8*S**(-1)*T1**(-1)*S4**(-1)*MS2**2*(S+U1+M2)**(-1)
     +     - 12*S**(-1)*T1**(-1)*S4**(-1)*MG2 - 4*S**(-1)*T1**(-1)*
     +    S4**(-1)*MG2**2*SYMBX*(U1+M2) + 8*S**(-1)*T1**(-1)*S4**(-1)*
     +    MG2**2*(S+U1+M2)**(-1) + 4*S**(-1)*T1*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 24*S**(-1)*T1*S4**(-2)*MS2*
     +    (S+U1+M2)**(-1) + 24*S**(-1)*T1*S4**(-2)*MG2*(S+U1+M2)**(-1)
     +     - 24*S**(-1)*T1*S4**(-2) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 4*S**(-1)*T1*
     +    S4**(-1)*MS2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 4*S**(-1)*T1*
     +    S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*T1*S4**(-1)*
     +    MG2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 4*S**(-1)*T1*S4**(-1)*
     +    MG2*(M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*T1*S4**(-1)*
     +    (S+U1+M2)**(-1) + 10*S**(-1)*T1*S4**(-1)*(T1+U1+M2)**(-1) - 8
     +    *S**(-1)*T1**2*S4**(-2)*(S+U1+M2)**(-1) + 4*S**(-1)*T1**2*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 4*S**(-1)*U1**(-1)*
     +    S4**(-1)*MG2 + 4*S**(-1)*U1**(-1)*S4**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) - 32*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MS2 + 
     +    32*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MG2 - 32*S**(-1)*U1*
     +    S4**(-2) + 8*S**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*U1*S4**(-1)*(S+T1)**(-1) - 8*
     +    S**(-1)*U1*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 22*S**(-1)*
     +    U1*S4**(-1)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 32*S**(-1)*U1**2*
     +    S4**(-2)*(S+T1)**(-1) - 8*S**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)
     +    *(T1+U1+M2)**(-1) + 64*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2*MG2
     +     - 96*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) + 96*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2**2*MG2
     +    *(S+U1+M2)**(-1) - 32*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2**2 - 
     +    32*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) - 32*
     +    S**(-1)*S4**(-2)*(S+T1)**(-1)*MG2**2 + 32*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1) + 64*S**(-1)*S4**(-2)*MS2
     +    *MG2*(S+U1+M2)**(-1) - 32*S**(-1)*S4**(-2)*MS2 - 32*S**(-1)*
     +    S4**(-2)*MS2**2*(S+U1+M2)**(-1) + 32*S**(-1)*S4**(-2)*MG2 - 
     +    32*S**(-1)*S4**(-2)*MG2**2*(S+U1+M2)**(-1) + 8*S**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2*MG2*(T1+U1+M2)**(-1) + 8*S**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2 - 8*S**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MS2**2*(T1+U1+M2)**(-1) - 8*S**(-1)*S4**(-1)*MS2*MG2*SYMBX*
     +    (U1+M2)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 8*S**(-1)*
     +    S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 4*S**(-1)*S4**(-1)
     +    *MS2*SYMBX*(U1+M2) + 8*S**(-1)*S4**(-1)*MS2*(S+U1+M2)**(-1)
     +     + 6*S**(-1)*S4**(-1)*MS2*(T1+U1+M2)**(-1) + 4*S**(-1)*
     +    S4**(-1)*MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 8*S**(-1)*
     +    S4**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 4*S**(-1)*S4**(-1)*
     +    MG2*SYMBX*(U1+M2) - 8*S**(-1)*S4**(-1)*MG2*(S+U1+M2)**(-1) + 
     +    2*S**(-1)*S4**(-1)*MG2*(T1+U1+M2)**(-1) + 4*S**(-1)*S4**(-1)*
     +    MG2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 2*S**(-1)*S4**(-1) + 
     +    16*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 16*
     +    S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*(S+U1+M2)**(-1) - 72*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 16*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 + 32*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 24*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 40*S*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) + 16*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2*(S+U1+M2)**(-1) - 24*S*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2*(S+U1+M2)**(-1) + 8*S*T1**(-1)
     +    *U1**(-1)*S4**(-1) - 4*S*T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) + 8*S*T1**(-1)*S4**(-2) + 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2*(T1+U1+M2)**(-1) - 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) + 4*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) + 4*S*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 4*S*T1**(-1)*
     +    S4**(-1)*MG2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S
     +    *T1*S4**(-1)*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 4*S*
     +    U1**(-1)*S4**(-1)*(S+U1+M2)**(-1) + 8*S*U1*S4**(-1)*
     +    (S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 48*S*S4**(-2)*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 64*S*S4**(-2)*(S+T1)**(-1)
     +    *MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 16*S*S4**(-2)*
     +    (S+T1)**(-1) - 16*S*S4**(-2)*(S+U1+M2)**(-1) - 72*S*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     + 16*S*S4**(-1)*(S+T1)**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 
     +    32*S*S4**(-1)*(S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 24*S*S4**(-1)*(S+T1)**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 40*S*S4**(-1)*(S+T1)**(-1)*MG2**2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 2*S*S4**(-1)*
     +    (S+U1)**(-1)*(T1+U1+M2)**(-1) + 24*S*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 28*S*S4**(-1)*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 24*S**2*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(S+U1+M2)**(-1) + 32*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) - 8*S**2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1) - 8*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 16*S**2*S4**(-2)*
     +    (S+T1)**(-1)*(S+U1+M2)**(-1) - 24*S**2*S4**(-1)*(S+T1)**(-1)*
     +    MS2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 32*S**2*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**2*S4**(-1)*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**2*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S**3*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*(S+U1+M2)**(-1) + 8*S**3*S4**(-1)*(S+T1)**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 32*T1**(-2)*
     +    U1**(-1)*S4**(-1)*MS2*MG2 + 16*T1**(-2)*U1**(-1)*S4**(-1)*MS2
     +    *MG2**2*SYMBX*(U1+M2) + 16*T1**(-2)*U1**(-1)*S4**(-1)*MS2*
     +    MG2**2*(S+U1+M2)**(-1) - 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2
     +    *MG2*SYMBX*(U1+M2) - 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*MG2
     +    *(S+U1+M2)**(-1) + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2 + 16*
     +    T1**(-2)*U1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) + 16*T1**(-2)
     +    *U1**(-1)*S4**(-1)*MS2**3*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 16*T1**(-2)*S4**(-1)
     +    *MS2*MG2*SYMBX*(U1+M2) + 16*T1**(-2)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 16*T1**(-2)*S4**(-1)*MS2**2*SYMBX*(U1+M2)
     +     - 16*T1**(-2)*S4**(-1)*MS2**2*(S+U1+M2)**(-1) + 32*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2 - 48*T1**(-1)*U1**(-1)
     +    *S4**(-1)*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 48*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) - 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MS2**2 - 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) - 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MG2**2 + 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) + 24*T1**(-1)*U1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 8*T1**(-1)*U1**(-1)*S4**(-1)*MS2 + 16*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MG2 - 24*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2**2*(S+U1+M2)**(-1) + 24*T1**(-1)*U1*S4**(-2) - 8
     +    *T1**(-1)*U1*S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 24*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1) + 12*T1**(-1)*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) - 8*T1**(-1)*U1**2*S4**(-1)*(S+T1)**(-1)*
     +    (T1+U1+M2)**(-1) + 8*T1**(-1)*S4**(-2)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 16*T1**(-1)*S4**(-2)*MS2 - 8*T1**(-1)*
     +    S4**(-2)*MG2 - 8*T1**(-1)*S4**(-2)*MG2**2*(S+U1+M2)**(-1) - 8
     +    *T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2*(T1+U1+M2)**(-1) - 24
     +    *T1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 + 8*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(T1+U1+M2)**(-1) + 16*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2 - 16*T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)
     +    *(T1+U1+M2)**(-1) - 8*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) + 24*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-1)*S4**(-1)
     +    *MS2*SYMBX*(U1+M2) - 8*T1**(-1)*S4**(-1)*MS2*(S+U1+M2)**(-1)
     +     + 4*T1**(-1)*S4**(-1)*MS2*(T1+U1+M2)**(-1) - 24*T1**(-1)*
     +    S4**(-1)*MS2**2*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 16*T1**(-1)*S4**(-1)
     +    *MS2**2*SYMBX*(U1+M2)*(T1+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*
     +    MS2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*T1**(-1)*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1) + 8*T1**(-1)*S4**(-1)*MG2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*T1**(-1)*S4**(-1)*MG2**3*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*
     +    S4**(-1) + 16*T1*S4**(-2)*(S+U1+M2)**(-1) - 2*T1*S4**(-1)*
     +    (S+U1)**(-1)*(T1+U1+M2)**(-1) - 12*T1*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 12*T1*S4**(-1)*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*T1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*T1**2*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*U1**(-1)*S4**(-1)*MS2*
     +    (S+U1+M2)**(-1) + 8*U1**(-1)*S4**(-1)*MG2*(S+U1+M2)**(-1) - 4
     +    *U1**(-1)*S4**(-1) + 16*U1*S4**(-2)*(S+T1)**(-1) + 16*U1*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 12*U1*
     +    S4**(-1)*(S+T1)**(-1)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * (  - 8*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 144*S4**(-2)*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) + 32*S4**(-2)*(S+T1)**(-1)*MS2 + 64*S4**(-2)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 48*S4**(-2)*
     +    (S+T1)**(-1)*MG2 + 80*S4**(-2)*(S+T1)**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) + 48*S4**(-2)*MS2*(S+U1+M2)**(-1) - 56*
     +    S4**(-2)*MG2*(S+U1+M2)**(-1) + 24*S4**(-2) + 32*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 48*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) + 48*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*
     +    (S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     - 4*S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 16*S4**(-1)
     +    *(S+T1)**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*N*(S4+MS2)*CF * ( 16*S4**(-1)*
     +    (S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     - 4*S4**(-1)*(S+T1)**(-1) + 2*S4**(-1)*(S+U1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 2*S4**(-1)*(S+U1)**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 2*S4**(-1)*(S+U1)**(-1) + 32*S4**(-1)*MS2*
     +    MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S4**(-1)*MS2*
     +    SYMBX*(U1+M2)*(T1+U1+M2)**(-1) - 12*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MS2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 4*S4**(-1)*(S+U1+M2)**(-1) + 2*
     +    S4**(-1)*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*SYMBT*(S4+MS2)*CF**2 * ( 32*S**(-1)*T1
     +    *U1*S4**(-1)*MG2*(T1+U1+M2)**(-1) + 8*S**(-1)*T1*U1**2*
     +    S4**(-1)*(T1+U1+M2)**(-1) + 16*S**(-1)*T1*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 24*S**(-1)*T1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1**2*U1*S4**(-1)*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1**2*S4**(-1)*MS2*
     +    (T1+U1+M2)**(-1) + 8*S**(-1)*T1**2*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 16*S**(-1)*U1*S4**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) + 16*S**(-1)*U1*S4**(-1)*MG2**2*
     +    (T1+U1+M2)**(-1) + 16*S**(-1)*U1**2*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) + 16*S**(-1)*S4**(-1)*MS2*MG2**2*
     +    (T1+U1+M2)**(-1) - 32*S**(-1)*S4**(-1)*MS2**2*MG2*
     +    (T1+U1+M2)**(-1) + 16*S**(-1)*S4**(-1)*MS2**3*
     +    (T1+U1+M2)**(-1) + 8*S*U1*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*SYMBT*(S4+MS2)*CF**2 * ( 8*S*U1**2*
     +    S4**(-1)*(S+T1)**(-1)*(T1+U1+M2)**(-1) + 8*T1*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 16*U1*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 16*U1*S4**(-1)*(S+T1)**(-1)*MS2**2*
     +    (T1+U1+M2)**(-1) - 8*U1*S4**(-1)*MS2*(T1+U1+M2)**(-1) - 16*
     +    U1**2*S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 8*U1**2*
     +    S4**(-1)*(T1+U1+M2)**(-1) - 8*S4**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) + 8*S4**(-1)*MG2**2*(T1+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*SYMBU*(S4+MS2)*CF**2 * (  - 8*S**(-1)*
     +    T1**(-1)*U1*S4**(-1)*MS2*MG2 + 8*S**(-1)*T1**(-1)*U1*S4**(-1)
     +    *MS2**2 - 8*S**(-1)*T1**(-1)*U1**2*S4**(-1)*MG2 + 8*S**(-1)*
     +    T1**(-1)*U1**3*S4**(-1) - 8*S**(-1)*T1*S4**(-1)*MS2*MG2*SYMBX
     +    *(U1+M2) - 8*S**(-1)*T1*S4**(-1)*MG2 + 8*S**(-1)*T1*S4**(-1)*
     +    MG2**2*SYMBX*(U1+M2) + 8*S**(-1)*U1*S4**(-1)*MS2 - 24*S**(-1)
     +    *U1*S4**(-1)*MG2 + 8*S**(-1)*U1**2*S4**(-1) - 32*S**(-1)*
     +    S4**(-1)*MS2*MG2 + 48*S**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) - 48*S**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 16*
     +    S**(-1)*S4**(-1)*MS2**2 + 16*S**(-1)*S4**(-1)*MS2**3*SYMBX*
     +    (U1+M2) + 16*S**(-1)*S4**(-1)*MG2**2 - 16*S**(-1)*S4**(-1)*
     +    MG2**3*SYMBX*(U1+M2) - 32*S*T1**(-1)*U1**(-1)*S4**(-1)*MS2*
     +    MG2 + 32*S*T1**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 96*S*T1**(-1)*
     +    U1*S4**(-1)*MS2*SYMBX*(U1+M2) + 8*S*T1**(-1)*U1*S4**(-1) - 32
     +    *S*T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) + 72*S*T1**(-1)*
     +    S4**(-1)*MS2 )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*SYMBU*(S4+MS2)*CF**2 * ( 32*S*T1**(-1)
     +    *S4**(-1)*MS2**2*SYMBX*(U1+M2) - 8*S*T1**(-1)*S4**(-1)*MG2 + 
     +    72*T1**(-1)*U1*S4**(-1)*MS2 - 16*T1**(-1)*U1*S4**(-1)*MG2 + 
     +    64*T1**(-1)*U1*MS2*SYMBX*(U1+M2) - 96*T1**(-1)*U1**2*S4**(-1)
     +    *MS2*SYMBX*(U1+M2) + 16*T1**(-1)*U1**2*S4**(-1) - 72*T1**(-1)
     +    *S4**(-1)*MS2*MG2 + 32*T1**(-1)*S4**(-1)*MS2*MG2**2*SYMBX*
     +    (U1+M2) - 64*T1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 72*
     +    T1**(-1)*S4**(-1)*MS2**2 + 32*T1**(-1)*S4**(-1)*MS2**3*SYMBX*
     +    (U1+M2) - 32*T1**(-1)*MS2 - 64*U1*S4**(-1)*MS2*SYMBX*(U1+M2)
     +     + 8*U1*S4**(-1) - 64*S4**(-1)*MS2*MG2*SYMBX*(U1+M2) + 48*
     +    S4**(-1)*MS2 + 48*S4**(-1)*MS2**2*SYMBX*(U1+M2) - 24*S4**(-1)
     +    *MG2 + 16*S4**(-1)*MG2**2*SYMBX*(U1+M2) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 64*S**(-1)*T1**(-1)
     +    *U1**(-1)*S4**(-1)*MS2*MG2 - 96*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*MG2**2*SYMBX*(U1+M2) + 96*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) - 32*S**(-1)*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2**2 - 32*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MS2**3*SYMBX*(U1+M2) - 32*S**(-1)*T1**(-1)*
     +    U1**(-1)*S4**(-1)*MG2**2 + 32*S**(-1)*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2**3*SYMBX*(U1+M2) - 32*S**(-1)*T1**(-1)*U1*
     +    S4**(-2)*MS2 + 32*S**(-1)*T1**(-1)*U1*S4**(-2)*MG2 - 24*
     +    S**(-1)*T1**(-1)*U1*S4**(-1) - 32*S**(-1)*T1**(-1)*U1**2*
     +    S4**(-2) + 64*S**(-1)*T1**(-1)*S4**(-2)*MS2*MG2 - 96*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2*MG2**2*(S+U1+M2)**(-1) + 96*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2**2*MG2*(S+U1+M2)**(-1) - 32*S**(-1)*
     +    T1**(-1)*S4**(-2)*MS2**2 - 32*S**(-1)*T1**(-1)*S4**(-2)*
     +    MS2**3*(S+U1+M2)**(-1) - 32*S**(-1)*T1**(-1)*S4**(-2)*MG2**2
     +     + 32*S**(-1)*T1**(-1)*S4**(-2)*MG2**3*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 24*S**(-1)*
     +    T1**(-1)*S4**(-1)*MS2 + 32*S**(-1)*T1**(-1)*S4**(-1)*MG2 - 8*
     +    S**(-1)*T1*U1**(-1)*S4**(-1)*MS2*SYMBX*(U1+M2) + 8*S**(-1)*T1
     +    *U1**(-1)*S4**(-1)*MG2*SYMBX*(U1+M2) - 8*S**(-1)*T1*U1**(-1)*
     +    S4**(-1) - 8*S**(-1)*T1*U1*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     - 8*S**(-1)*T1*S4**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) + 8*
     +    S**(-1)*T1*S4**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*
     +    T1**2*S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 40*S**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*MG2*SYMBX*(U1+M2) - 16*S**(-1)*U1**(-1)*S4**(-1)
     +    *MS2 - 16*S**(-1)*U1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) + 24
     +    *S**(-1)*U1**(-1)*S4**(-1)*MG2 - 24*S**(-1)*U1**(-1)*S4**(-1)
     +    *MG2**2*SYMBX*(U1+M2) + 32*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*
     +    MS2 - 32*S**(-1)*U1*S4**(-2)*(S+T1)**(-1)*MG2 - 16*S**(-1)*U1
     +    *S4**(-1)*(S+T1)**(-1)*MG2*(T1+U1+M2)**(-1) - 16*S**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1) + 16*S**(-1)*U1*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 8*S**(-1)*U1*
     +    S4**(-1)*(T1+U1+M2)**(-1) + 32*S**(-1)*U1**2*S4**(-2)*
     +    (S+T1)**(-1) - 64*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2*MG2 + 96*
     +    S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 96
     +    *S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) + 
     +    32*S**(-1)*S4**(-2)*(S+T1)**(-1)*MS2**2 + 32*S**(-1)*S4**(-2)
     +    *(S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1) + 32*S**(-1)*S4**(-2)*
     +    (S+T1)**(-1)*MG2**2 - 32*S**(-1)*S4**(-2)*(S+T1)**(-1)*MG2**3
     +    *(S+U1+M2)**(-1) - 16*S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (T1+U1+M2)**(-1) - 16*S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2 + 16*
     +    S**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*(T1+U1+M2)**(-1) + 16*
     +    S**(-1)*S4**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*
     +    S4**(-1)*MS2*(T1+U1+M2)**(-1) - 16*S**(-1)*S4**(-1)*MS2**2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S**(-1)*S4**(-1)*MG2*
     +    (T1+U1+M2)**(-1) - 24*S**(-1)*S4**(-1) - 32*S*T1**(-2)*
     +    U1**(-2)*S4**(-1)*MS2*MG2 )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 32*S*T1**(-2)*
     +    U1**(-2)*S4**(-1)*MS2*MG2**2*SYMBX*(U1+M2) - 64*S*T1**(-2)*
     +    U1**(-2)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2) + 32*S*T1**(-2)*
     +    U1**(-2)*S4**(-1)*MS2**2 + 32*S*T1**(-2)*U1**(-2)*S4**(-1)*
     +    MS2**3*SYMBX*(U1+M2) + 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2*
     +    MG2*SYMBX*(U1+M2) - 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*
     +    SYMBX*(U1+M2) + 32*S*T1**(-2)*U1**(-1)*S4**(-1)*MS2**2*
     +    (S+U1+M2)**(-1) + 72*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(S+U1+M2)**(-1) - 16*S*T1**(-1)*U1**(-1)
     +    *S4**(-1)*(S+T1)**(-1)*MS2 - 32*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) + 24*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MG2 - 40*S*T1**(-1)*U1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1) - 8*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MS2*(S+U1+M2)**(-1) + 16*S*T1**(-1)*U1**(-1)*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 8*S*T1**(-1)*
     +    U1**(-1)*S4**(-1) - 16*S*T1**(-1)*S4**(-2) + 8*S*T1**(-1)*
     +    S4**(-1)*(S+T1)**(-1) + 8*S*T1**(-1)*S4**(-1)*MS2*MG2*SYMBX*
     +    (U1+M2)*(M2*(S+T1)+T1*U1)**(-1) - 8*S*T1**(-1)*S4**(-1)*MS2*
     +    MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S*T1**(-1)*
     +    S4**(-1)*MG2**2*SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) + 8*S*
     +    T1**(-1)*S4**(-1)*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S*T1*S4**(-1)*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) - 8*S*U1*S4**(-1)*(S+T1)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 48*S*S4**(-2)*(S+T1)**(-1)*MS2*
     +    (S+U1+M2)**(-1) - 64*S*S4**(-2)*(S+T1)**(-1)*MG2*
     +    (S+U1+M2)**(-1) + 16*S*S4**(-2)*(S+T1)**(-1) + 16*S*S4**(-2)*
     +    (S+U1+M2)**(-1) + 72*S*S4**(-1)*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*S*S4**(-1)*
     +    (S+T1)**(-1)*MS2*(M2*(S+T1)+T1*U1)**(-1) - 32*S*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 24*S*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) - 40*S*S4**(-1)*
     +    (S+T1)**(-1)*MG2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     + 8*S*S4**(-1)*MS2*SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) - 
     +    24*S*S4**(-1)*MS2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8
     +    *S*S4**(-1)*MG2*SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) + 24*S*
     +    S4**(-1)*MG2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S*
     +    S4**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 24*S**2*T1**(-1)*U1**(-1)*
     +    S4**(-1)*(S+T1)**(-1)*MS2*(S+U1+M2)**(-1) - 32*S**2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1) + 8*S**2*
     +    T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1) + 8*S**2*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+U1+M2)**(-1) - 16*S**2*S4**(-2)*
     +    (S+T1)**(-1)*(S+U1+M2)**(-1) + 24*S**2*S4**(-1)*(S+T1)**(-1)*
     +    MS2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 32*S**2*
     +    S4**(-1)*(S+T1)**(-1)*MG2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 8*S**2*S4**(-1)*
     +    (S+T1)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 8*S**2*S4**(-1)*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 8*S**3*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*(S+U1+M2)**(-1) - 8*S**3*
     +    S4**(-1)*(S+T1)**(-1)*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     + 32*T1**(-2)*U1**(-1)*S4**(-1)*MS2*MG2 - 32*T1**(-2)*
     +    U1**(-1)*S4**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) + 64*T1**(-2)*
     +    U1**(-1)*S4**(-1)*MS2**2*MG2*(S+U1+M2)**(-1) - 32*T1**(-2)*
     +    U1**(-1)*S4**(-1)*MS2**2 - 32*T1**(-2)*U1**(-1)*S4**(-1)*
     +    MS2**3*(S+U1+M2)**(-1) - 32*T1**(-2)*S4**(-1)*(S+U1)**(-1)*
     +    MS2*MG2 + 32*T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2*MG2**2*
     +    (S+U1+M2)**(-1) - 64*T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2**2*
     +    MG2*(S+U1+M2)**(-1) + 32*T1**(-2)*S4**(-1)*(S+U1)**(-1)*
     +    MS2**2 + 32*T1**(-2)*S4**(-1)*(S+U1)**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) - 32*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MS2*MG2 )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 48*T1**(-1)*
     +    U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1) - 
     +    48*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1) + 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MS2**2 + 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MS2**3*
     +    (S+U1+M2)**(-1) + 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*
     +    MG2**2 - 16*T1**(-1)*U1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2**3*
     +    (S+U1+M2)**(-1) + 72*T1**(-1)*U1**(-1)*S4**(-1)*MS2*MG2*SYMBX
     +    *(U1+M2) - 8*T1**(-1)*U1**(-1)*S4**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 16*T1**(-1)*U1**(-1)*S4**(-1)*MS2 - 48*
     +    T1**(-1)*U1**(-1)*S4**(-1)*MS2**2*SYMBX*(U1+M2) + 16*T1**(-1)
     +    *U1**(-1)*S4**(-1)*MG2 - 24*T1**(-1)*U1**(-1)*S4**(-1)*MG2**2
     +    *SYMBX*(U1+M2) + 8*T1**(-1)*U1**(-1)*S4**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) - 48*T1**(-1)*U1*S4**(-2) + 48*T1**(-1)*U1*
     +    S4**(-1)*(S+T1)**(-1) - 16*T1**(-1)*S4**(-2)*MS2*MG2*
     +    (S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 32*T1**(-1)*
     +    S4**(-2)*MS2 + 16*T1**(-1)*S4**(-2)*MG2 + 16*T1**(-1)*
     +    S4**(-2)*MG2**2*(S+U1+M2)**(-1) + 48*T1**(-1)*S4**(-1)*
     +    (S+T1)**(-1)*MS2 - 16*T1**(-1)*S4**(-1)*(S+T1)**(-1)*MG2 - 24
     +    *T1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2*MG2*(S+U1+M2)**(-1) + 16*
     +    T1**(-1)*S4**(-1)*(S+U1)**(-1)*MS2 + 16*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MS2**2*(S+U1+M2)**(-1) - 8*T1**(-1)*S4**(-1)*
     +    (S+U1)**(-1)*MG2 + 8*T1**(-1)*S4**(-1)*(S+U1)**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) + 32*T1**(-1)*S4**(-1)*MS2*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 48*T1**(-1)*S4**(-1)*MS2*MG2**2*
     +    SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) - 48*T1**(-1)*S4**(-1)*
     +    MS2*MG2**2*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 48*
     +    T1**(-1)*S4**(-1)*MS2**2*MG2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 48*T1**(-1)*S4**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*T1**(-1)*
     +    S4**(-1)*MS2**3*SYMBX*(U1+M2)*(M2*(S+T1)+T1*U1)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * (  - 16*T1**(-1)*
     +    S4**(-1)*MS2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 32*
     +    T1**(-1)*S4**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*
     +    T1**(-1)*S4**(-1)*MG2**3*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*T1**(-1)*S4**(-1)*MG2**3*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) - 16*T1*S4**(-2)*
     +    (S+U1+M2)**(-1) - 8*T1*S4**(-1)*MS2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*T1*S4**(-1)*MG2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*U1**(-1)*S4**(-1)*MS2*SYMBX*
     +    (U1+M2) + 8*U1**(-1)*S4**(-1)*MG2*SYMBX*(U1+M2) - 8*U1**(-1)*
     +    S4**(-1) - 16*U1*S4**(-2)*(S+T1)**(-1) - 16*U1*S4**(-1)*
     +    (S+T1)**(-1)*MG2*(M2*(S+T1)+T1*U1)**(-1) + 8*U1*S4**(-1)*
     +    (S+T1)**(-1)*(T1+U1+M2)**(-1) + 8*U1*S4**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 144*S4**(-2)*(S+T1)**(-1)*MS2*MG2*
     +    (S+U1+M2)**(-1) - 32*S4**(-2)*(S+T1)**(-1)*MS2 - 64*S4**(-2)*
     +    (S+T1)**(-1)*MS2**2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 48*S4**(-2)*
     +    (S+T1)**(-1)*MG2 - 80*S4**(-2)*(S+T1)**(-1)*MG2**2*
     +    (S+U1+M2)**(-1) - 48*S4**(-2)*MS2*(S+U1+M2)**(-1) + 48*
     +    S4**(-2)*MG2*(S+U1+M2)**(-1) - 48*S4**(-2) - 32*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2*(M2*(S+T1)+T1*U1)**(-1) + 48*S4**(-1)*
     +    (S+T1)**(-1)*MS2*MG2**2*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S4**(-1)*(S+T1)**(-1)*MS2*
     +    (T1+U1+M2)**(-1) - 48*S4**(-1)*(S+T1)**(-1)*MS2**2*MG2*
     +    (S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*
     +    (S+T1)**(-1)*MS2**2*(M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*
     +    (S+T1)**(-1)*MS2**3*(S+U1+M2)**(-1)*(M2*(S+T1)+T1*U1)**(-1)
     +     + 16*S4**(-1)*(S+T1)**(-1)*MG2**2*(M2*(S+T1)+T1*U1)**(-1) - 
     +    16*S4**(-1)*(S+T1)**(-1)*MG2**3*(S+U1+M2)**(-1)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*(S+T1)**(-1) + 8*
     +    S4**(-1)*(S+U1)**(-1)*MS2*(S+U1+M2)**(-1) - 8*S4**(-1)*
     +    (S+U1)**(-1)*MG2*(S+U1+M2)**(-1) )
     +
      M2QGH4 = M2QGH4 + COLO1(9)*(S4+MS2)*CF**2 * ( 8*S4**(-1)*
     +    (S+U1)**(-1) + 32*S4**(-1)*MS2*MG2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 16*S4**(-1)*MS2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MS2**2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) + 8*S4**(-1)*MG2*
     +    (M2*(S+T1)+T1*U1)**(-1) - 16*S4**(-1)*MG2**2*SYMBX*(U1+M2)*
     +    (M2*(S+T1)+T1*U1)**(-1) )


      M2QGH = 2.D0*((NS-1.D0)*(M2QGH1 +M2QGH2 +M2QGH3) +M2QGH4) 

      DSBQGH = S4/(S4+MS2)/2.D0 *
     +     ALPHAS**3 * AVG * M2QGH /4.D0 /S**2 *CONV

      RETURN
      END


      REAL*8 FUNCTION DSBQGT(ALPHAS,S,T1,S4,S3G,MS,MG,EPS)
C***  DOUBLE-POLE PART OF CROSS SECTIONS FOR G +Q -> SQ + SQB +Q
C***  SUMMED OVER LL +RR +LR +RL
      IMPLICIT NONE
      REAL*8 ALPHAS,S,S4,T1,TG,U1,S3G,MS,MG,MS2,MG2,M2,CF,EPS
      REAL*8 M2QGT2,M2QGT3,M2QGT
      REAL*8 NS,CONV,N,AVG
      REAL*8 ANGDEF(1:11), ANA(1:3,1:9), ANB(1:3,1:9), ANC(1:3,1:9)
      REAL*8 ANGS3(1:5),XX(5:9),YY2(5:9),XPHI

      NS = 6.D0
      CONV = 389379660.D0
      N = 3.D0
      CF = (N**2-1.D0)/N/2.D0
      AVG = (1.D0/2.D0)**2 /N /(N**2 -1.D0)

      MS2 = MS**2
      MG2 = MG**2
      M2 = MG2 -MS2
      TG = T1 -M2
      U1 = S4 -S -T1


      ANGDEF(1) = 2.D0*SQRT(S4 +MS2)
      ANGDEF(2) = (S +U1)/ANGDEF(1)
      ANGDEF(3) = (S +T1)/ANGDEF(1)
      ANGDEF(4) = S4/ANGDEF(1)
      ANGDEF(5) = (S4 +2.D0*MS2)/ANGDEF(1)
      ANGDEF(6) = -(S4 -S +2.D0*MS2)/ANGDEF(1)
      ANGDEF(7) = SQRT((S -S4)**2 -4.D0*MS2*S)/ANGDEF(1)
      ANGDEF(8) = 
     +  (T1*S4 -S*(U1+2.D0*MS2))/(S+T1)/SQRT((S -S4)**2-4.D0*MS2*S)
      ANGDEF(9) = SQRT(1 -ANGDEF(8)**2)
      ANGDEF(10) =
     +  (U1*S4 -S*(T1+2.D0*MS2))/(S+U1)/SQRT((T1+U1)**2-4.D0*MS2*S)
      ANGDEF(11) = SQRT(1 -ANGDEF(10)**2)


      ANA(2,5) = -2.D0*ANGDEF(5)*ANGDEF(3)
      ANB(2,5) = -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,5) = -2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)
      ANA(2,7) = +2.D0*ANGDEF(4)*ANGDEF(6) -M2
      ANB(2,7) = -2.D0*ANGDEF(4)*ANGDEF(7)
      ANC(2,7) =  0.D0
      ANA(2,9) = -2.D0*ANGDEF(5)*ANGDEF(2) -M2
      ANB(2,9) = 
     +     -2.D0*ANGDEF(4)*ANGDEF(7) +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(8)
      ANC(2,9) = +2.D0*ANGDEF(3)*ANGDEF(4)*ANGDEF(9)

      XPHI = (S3G -ANA(2,7))/ANB(2,7)

      XX(5) = (ANA(2,5) + ANB(2,5)*XPHI)
      YY2(5)= ANC(2,5)**2 * (1.D0-XPHI**2)
      XX(9) = (ANA(2,9) + ANB(2,9)*XPHI)
      YY2(9)= ANC(2,9)**2 * (1.D0-XPHI**2)

      ANGS3(1) = -XX(5)/(XX(5)**2 - YY2(5))**(1.5D0)
      ANGS3(2) = -1.D0/SQRT(XX(5)**2 - YY2(5))
      ANGS3(3) = -XX(9)/(XX(9)**2 - YY2(9))**(1.5D0)
      ANGS3(4) = -1.D0/SQRT(XX(9)**2 - YY2(9))
      ANGS3(5) = XX(9)

      M2QGT2 = 0.D0
      M2QGT2 = M2QGT2 + N*CF**2 * ( 4*S**(-1)*U1*MG2 + 8*S**(-1)*MS2*
     +    MG2 - 8*S**(-1)*MS2**2 )
     +
      M2QGT2 = M2QGT2 + N**2*CF * ( 4*MS2 )
     +
      M2QGT2 = M2QGT2 + ANGS3(1)*N*CF**2 * ( 8*U1*MS2*MG2 + 8*MS2**2*
     +    MG2 - 8*MS2**3 )
     +
      M2QGT2 = M2QGT2 + ANGS3(2)*N*CF**2 * ( 4*S**(-1)*T1*MS2*MG2 - 4*
     +    S**(-1)*T1*MG2**2 + 8*S**(-1)*U1*MS2*MG2 - 8*S**(-1)*U1*
     +    MG2**2 - 8*S**(-1)*MS2*MG2**2 + 16*S**(-1)*MS2**2*MG2 - 8*
     +    S**(-1)*MS2**3 + 4*U1*MG2 + 4*MS2*MG2 - 8*MS2**2 - 4*MG2**2 )
     +
      M2QGT2 = M2QGT2 + ANGS3(2)*N**2*CF * (  - 8*S**(-1)*U1*MS2*MG2 - 
     +    8*S**(-1)*MS2**2*MG2 + 8*S**(-1)*MS2**3 )
     +
      M2QGT2 = M2QGT2 + ANGS3(3)*N**2*CF * ( 4*S*MS2*MG2 + 4*S*MG2**2
     +     - 4*T1*MS2*MG2 + 4*T1*MG2**2 + 8*U1*MG2**2 + 8*MS2*MG2**2 - 
     +    8*MS2**2*MG2 )
     +
      M2QGT2 = M2QGT2 + ANGS3(4)*N**2*CF * (  - 8*S**(-1)*U1*MS2*MG2 - 
     +    8*S**(-1)*MS2**2*MG2 + 8*S**(-1)*MS2**3 + 4*S*MS2 + 4*U1*MG2
     +     + 8*MS2*MG2 - 8*MS2**2 )
     +
      M2QGT2 = M2QGT2 + ANGS3(5)*N*CF**2 * ( 4*S**(-1)*MS2 )

      M2QGT3 = 0.D0
      M2QGT3 = M2QGT3 + N*CF**2 * (  - 4*S**(-1)*U1*MG2 + 8*S**(-1)*MS2
     +    *MG2 - 8*S**(-1)*MG2**2 )
     +
      M2QGT3 = M2QGT3 + N**2*CF * (  - 4*MG2 )
     +
      M2QGT3 = M2QGT3 + ANGS3(1)*N*CF**2 * (  - 8*U1*MS2*MG2 - 8*MS2*
     +    MG2**2 + 8*MS2**2*MG2 )
     +
      M2QGT3 = M2QGT3 + ANGS3(2)*N*CF**2 * (  - 4*S**(-1)*T1*MS2*MG2 + 
     +    4*S**(-1)*T1*MG2**2 - 8*S**(-1)*U1*MS2*MG2 + 8*S**(-1)*U1*
     +    MG2**2 - 16*S**(-1)*MS2*MG2**2 + 8*S**(-1)*MS2**2*MG2 + 8*
     +    S**(-1)*MG2**3 - 4*U1*MG2 + 12*MS2*MG2 - 4*MG2**2 )
     +
      M2QGT3 = M2QGT3 + ANGS3(2)*N**2*CF * ( 8*S**(-1)*U1*MS2*MG2 + 8*
     +    S**(-1)*MS2*MG2**2 - 8*S**(-1)*MS2**2*MG2 )
     +
      M2QGT3 = M2QGT3 + ANGS3(3)*N**2*CF * (  - 4*S*MS2*MG2 - 4*S*
     +    MG2**2 + 4*T1*MS2*MG2 - 4*T1*MG2**2 - 8*U1*MG2**2 + 8*MS2*
     +    MG2**2 - 8*MG2**3 )
     +
      M2QGT3 = M2QGT3 + ANGS3(4)*N**2*CF * ( 8*S**(-1)*U1*MS2*MG2 + 8*
     +    S**(-1)*MS2*MG2**2 - 8*S**(-1)*MS2**2*MG2 - 4*S*MG2 - 4*U1*
     +    MG2 + 8*MS2*MG2 - 8*MG2**2 )
     +
      M2QGT3 = M2QGT3 + ANGS3(5)*N*CF**2 * (  - 4*S**(-1)*MG2 )


      M2QGT = 2.D0*(NS-1.D0)*(M2QGT2 + M2QGT3)

      DSBQGT = ALPHAS**3 *AVG *M2QGT *S4/(S4 +MS2)/8.D0/S**2 *CONV
      RETURN
      END









